package com.ecell.internationalize.app.aspect;
import com.ecell.internationalize.app.annotation.AppUnbindAnnotation;
import com.ecell.internationalize.app.entity.AppUnbindLog;
import com.ecell.internationalize.app.entity.dto.DeviceUnbindDto;
import com.ecell.internationalize.app.service.impl.AppUnbindLogServiceImpl;
import com.ecell.internationalize.app.service.impl.BindingDeviceImpl;
import com.ecell.internationalize.common.core.context.SecurityContextHolder;
import com.ecell.internationalize.common.core.utils.StringUtils;
import com.ecell.internationalize.common.core.utils.uuid.UUID;
import com.ecell.internationalize.common.system.entity.UserDeviceBindApp;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.Aspect;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;

/**
 * @author borui
 */
@Component
@Aspect
public class AppUnbindLogInfo {
    private static final Logger logg =  LoggerFactory.getLogger(AppUnbindLogInfo.class);
        @Autowired
        private AppUnbindLogServiceImpl appUnbindLogServiceImpl;
        @Autowired
        private BindingDeviceImpl bindingDeviceImpl;
        /**
         * 处理完请求后执行
         *
         * @param joinPoint 切点
         */
        @AfterReturning(pointcut = "@annotation(factoryLog)", returning = "jsonResult")
        public void doAfterReturning(JoinPoint joinPoint, AppUnbindAnnotation factoryLog, Object jsonResult)
        {
            handleLog(joinPoint, factoryLog, null, jsonResult);
        }

        protected void handleLog(final JoinPoint joinPoint, AppUnbindAnnotation controllerLog, final Exception e, Object jsonResult){
            try {
                // *========数据库日志=========*//
                AppUnbindLog info =new AppUnbindLog();
                info.setId(UUID.fastUUID().toString(true));
                info.setCreateTime(new Date());

                // 处理设置注解上的参数
                AppUnbindLog log= getControllerMethodDescription(joinPoint, controllerLog, info, jsonResult);
                // 保存数据库
               if (StringUtils.isNotNull(log)){
                   appUnbindLogServiceImpl.saveUnbindInfo(info);
               }

            }catch (Exception exp){
                // 记录本地异常日志
                logg.error("==通知异常==");
                logg.error("异常信息:{}", exp.getMessage());
                exp.printStackTrace();


            }

        }

    /**
     * 获取参数
     * @param joinPoint
     * @param controllerLog
     * @param info
     * @param jsonResult
     */
        private AppUnbindLog getControllerMethodDescription(JoinPoint joinPoint, AppUnbindAnnotation controllerLog, AppUnbindLog info, Object jsonResult) {
            Object[] args = joinPoint.getArgs();
            if (StringUtils.isNotNull(args) &&args.length>0){
                try {
                    DeviceUnbindDto dto = (DeviceUnbindDto) args[0];
                    info.setImei(dto.getImei());
                    String userId = SecurityContextHolder.getStringUserId();
                    logg.info("本地线程获取到的id是：{}",userId);
                    String id = dto.getUserId() == null ? SecurityContextHolder.getStringUserId() : dto.getUserId();
                    info.setUserId(id);
                    UserDeviceBindApp bingDeviceInfo = bindingDeviceImpl.getBingDeviceInfo(id);
                    if (bingDeviceInfo!=null){
                        info.setAccount(bingDeviceInfo.getUserEmail());
                        info.setRelation(bingDeviceInfo.getRelation());
                    }


                }catch (Exception e){
                    logg.info("解绑日志参数异常:{}", e.getMessage());
                    e.printStackTrace();
                    return null;
                }
            }

            return  info;
        }
}

