package com.ecell.internationalize.app.config;
import com.ecell.internationalize.app.util.I18nUtil;
import com.ecell.internationalize.common.core.entity.Rest;
import com.ecell.internationalize.common.core.exception.ServiceException;
import com.ecell.internationalize.common.core.web.domain.AjaxResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;

import javax.servlet.http.HttpServletRequest;

import static com.ecell.internationalize.common.system.constant.ServiceExceptionResultConstant.RESULT_MSG_MAP;

/**
 * 统一异常处理
 * @Title: ExceptionAdviceConfig
 * @Author: liy
 * @Date: 2022/7/8 13:47
 * @Version:1.0
 */
@RestControllerAdvice
public class ExceptionAdviceConfig {

    //设置默认值
    private static final String msg="messagesParameterError";
    private static final Logger logger = LoggerFactory.getLogger(ExceptionAdviceConfig.class);
    /**
     * 全局异常处理,配置国际化
     * @Author: liy
     * @Date: 2022/7/8 13:53
     */
    @ExceptionHandler(Exception.class)
    public Rest exceptionHandler(Exception e) {
        e.printStackTrace();
        return Rest.fail("服务器异常");
    }

    @ExceptionHandler(ServiceException.class)
    public AjaxResult serviceException(HttpServletRequest request, ServiceException e) {
      //  ApplicationUtil.
       // String stringUserId = SecurityContextHolder.getStringUserId();
    //    String cacheMapValue = redisService.getCacheMapValue(stringUserId, CommonConstant.IOS_LANGUAGE);
        String language = request.getHeader("Accept-Language");
        logger.info("获取到的异常语言是:{}",language);
        logger.error("异常业务错误信息:{}",e.getMessage());
        String get = RESULT_MSG_MAP.get(e.getMessage());
        logger.error("获取到的值:{}",get);
        String message = I18nUtil.getMessage((RESULT_MSG_MAP.containsKey(e.getMessage()) ? RESULT_MSG_MAP.get(e.getMessage()) : msg), language);
        logger.error("获取到国际化语言是:{}",message);
        return AjaxResult.error(message);
    }
}

