package com.ecell.internationalize.app.config;
import com.ecell.internationalize.common.core.utils.StringUtils;
import com.ecell.internationalize.common.system.constant.FieldConstant;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.servlet.LocaleResolver;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Locale;

/**
 * 国际化语言配置
 * @Title: MyLocaleResolverConfig
 * @Author: liy
 * @Date: 2022/7/7 10:27
 * @Version:1.0
 */
@Configuration
public class MyLocaleResolverConfig implements LocaleResolver {
    @Autowired
    private HttpServletRequest request;

    public Locale getLocal() {
        return resolveLocale(request);
    }
    /**
     * 从HttpServletRequest中获取Locale
     * @param  request
     * @return  语言Local
     */
    @Override
    public Locale resolveLocale(HttpServletRequest request) {
        //获取请求中的语言参数
        String language = request.getParameter(FieldConstant.LANG);
        //如果没有就使用默认的（根据主机的语言环境生成一个 Locale
        Locale locale = Locale.getDefault();
        //如果请求的链接中携带了 国际化的参数
        if (!StringUtils.isEmpty(language)){
            //zh_CN,通过_分割
            String[] s = language.split(FieldConstant.CONN_CHAR);
            //国家，地区
            locale = new Locale(s[0], s[1]);
        }
        return locale;
    }

    /**
     * 用于实现Locale的切换。比如SessionLocaleResolver获取Locale的方式是从session中读取，但如果
     * 用户想要切换其展示的样式(由英文切换为中文)，那么这里的setLocale()方法就提供了这样一种可能
     *
     * @param request  HttpServletRequest
     * @param response HttpServletResponse
     * @param locale   locale
     */
    @Override
    public void setLocale(HttpServletRequest request, HttpServletResponse response, Locale locale) {

    }
}
