package com.ecell.internationalize.app.config;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.web.client.RestTemplateBuilder;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.io.ClassPathResource;
import org.springframework.data.redis.core.script.DefaultRedisScript;
import org.springframework.scripting.support.ResourceScriptSource;
import org.springframework.web.client.RestTemplate;

import java.time.Duration;
import java.util.Collections;

/**
 * @author borui
 */
@Configuration
public class RestTemplateConfig {
  private static final Logger log = LoggerFactory.getLogger(RestTemplateConfig.class);
  @Autowired
  private RestTemplateInterceptor restTemplateInterceptor;
    @Bean
    public RestTemplate restTemplateOne(RestTemplateBuilder restTemplateBuilder) {
        return restTemplateBuilder.setConnectTimeout(Duration.ofSeconds(3)).setReadTimeout(Duration.ofSeconds(5)).build();
    }

    @Bean
    public RestTemplate restTemplateTwo(RestTemplateBuilder restTemplateBuilder) {
        RestTemplate build = restTemplateBuilder.setConnectTimeout(Duration.ofSeconds(3)).setReadTimeout(Duration.ofSeconds(5)).build();
        build.setInterceptors(Collections.singletonList(restTemplateInterceptor));
        return build;
    }

    @Bean
    public DefaultRedisScript<Long> localLimitScript(){
        DefaultRedisScript<Long> script = new DefaultRedisScript<>();
        script.setResultType(Long.class);
        script.setScriptSource(new ResourceScriptSource(new ClassPathResource("lua\\localLimit.lua")));
        return script;
    }
}
