package com.ecell.internationalize.app.controller;


import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.ecell.internationalize.app.entity.AddressBookApp;
import com.ecell.internationalize.app.entity.NotifyApp;
import com.ecell.internationalize.app.entity.dto.*;
import com.ecell.internationalize.app.service.*;
import com.ecell.internationalize.app.service.impl.BindingDeviceImpl;
import com.ecell.internationalize.app.util.I18nUtil;
import com.ecell.internationalize.common.core.context.SecurityContextHolder;
import com.ecell.internationalize.common.core.entity.Rest;
import com.ecell.internationalize.common.core.utils.uuid.UUID;
import com.ecell.internationalize.common.issue.constant.CommonConstant;
import com.ecell.internationalize.common.issue.mqtt.dto.AddressBookMessage;
import com.ecell.internationalize.common.issue.mqtt.dto.AddressBookMessageChild;
import com.ecell.internationalize.common.issue.mqtt.push.MqttPush;
import com.ecell.internationalize.common.redis.service.RedisService;
import com.ecell.internationalize.common.system.constant.FieldConstant;
import com.ecell.internationalize.common.system.entity.DeviceOwnerInfoApp;
import com.ecell.internationalize.common.system.entity.UserDeviceBindApp;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.List;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author ${author}
 * @since 2022-08-18
 */
@RestController
@Api(value="通讯录接口",tags={"通讯录接口"})
@RequestMapping("/address_book_app")
public class AddressBookAppController {
    @Autowired
    private AddressBookAppService addressBookAppService;
    @Autowired
    private BindingDevice bindingDevice;
    @Autowired
    private MqttPush mqttPush;
    @Autowired
    private DeviceOwnerInfoAppService deviceOwnerInfoAppService;
    @Autowired
    private NotifyAppService notifyAppService;
    @Autowired
    private RedisService redisService;
    @Autowired
    private BindingDeviceImpl bindingDeviceImpl;
    @Autowired
    private UserDeviceBindService userDeviceBindService;
    @ApiOperation(value ="根据imei查询通讯录成员")
    @GetMapping("/query/queryAddress")
    public Rest<List<AddressBookApp>> queryAddress(@ApiParam(value = "设备imei号") String imei) {
        return Rest.ok(addressBookAppService.queryAddress(imei));
    }

    @ApiOperation(value ="新增通讯录成员(注：不管是通讯录导入方式还是手动添加，都需要传对象集合给后端)")
    @PostMapping("/insert/addressImport")
    public Rest addressImport(HttpServletRequest request,@RequestBody List<SaveAddressBookAppDto> addressBookAppList) {
        int listSize=addressBookAppList.size();
        LambdaQueryWrapper<AddressBookApp> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(AddressBookApp::getImei,addressBookAppList.get(0).getImei());
        int count = addressBookAppService.count(lambdaQueryWrapper);
        int sum=listSize+count;
        if (sum>50){
            return  Rest.fail(I18nUtil.getMessage( CommonConstant.ADDRESS_BOOK_MAX,request.getHeader("Accept-Language")));
        }
        addressBookAppService.insertData(addressBookAppList);
       //4.开始指令下发
        addressBookAppService.sendToKafka(addressBookAppList.get(0).getImei());
        return Rest.ok(I18nUtil.getMessage(FieldConstant.MESSAGES_SUCCESS,request.getHeader("Accept-Language")));
    }
    @ApiOperation(value ="修改通讯录成员信息")
    @PostMapping("/update/updateAddress")
    public Rest updateAddress(HttpServletRequest request,@RequestBody UpdateAddressBookAppDto addressBookApp) {
        String userId= SecurityContextHolder.getStringUserId();
        LambdaQueryWrapper<UserDeviceBindApp> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(UserDeviceBindApp::getPhone,addressBookApp.getPhone());
        List<UserDeviceBindApp> userDeviceBindApp = userDeviceBindService.getBaseMapper().selectList(lambdaQueryWrapper);
        if (!CollectionUtils.isEmpty(userDeviceBindApp) && !userId.equals(userDeviceBindApp.get(0).getUserId())){
            return  Rest.fail(addressBookApp.getPhone()+""+I18nUtil.getMessage(CommonConstant.PHONE_HAS_EXIT,request.getHeader("Accept-Language")));
        }


        addressBookAppService.updateData(addressBookApp);
        //开始指令下发
        addressBookAppService.sendToKafka(addressBookApp.getImei());
        return Rest.ok(I18nUtil.getMessage(FieldConstant.MESSAGES_SUCCESS,request.getHeader("Accept-Language")));
    }
    @ApiOperation(value ="删除单个通讯录成员")
    @PostMapping("/del/delAddress")
    public Rest delAddress(HttpServletRequest request,@RequestBody DelAddressBookAppDto delAddressBookAppDto) {
        //普通导入，直接删除
        String lang=request.getHeader("Accept-Language");
        if ("0".equals(delAddressBookAppDto.getIdentity())) {
            addressBookAppService.getBaseMapper().deleteById(delAddressBookAppDto.getId());

        }else {
            LambdaQueryWrapper<DeviceOwnerInfoApp> ownerInfoAppLambdaQueryWrapper=new LambdaQueryWrapper<>();
            ownerInfoAppLambdaQueryWrapper.eq(DeviceOwnerInfoApp::getImei,delAddressBookAppDto.getImei());
            DeviceOwnerInfoApp deviceOwnerInfoApp = deviceOwnerInfoAppService.getBaseMapper().selectOne(ownerInfoAppLambdaQueryWrapper);
            String name=null!=deviceOwnerInfoApp?deviceOwnerInfoApp.getName():"宝贝";

            System.out.println("圈子里面成员,进行解绑操作");
            //根据id查找要删除成员的userId
            AddressBookApp addressBookApp = addressBookAppService.getBaseMapper().selectById(delAddressBookAppDto.getId());
            DeviceUnbindDto deviceUnbindDto = new DeviceUnbindDto();
            deviceUnbindDto.setImei(delAddressBookAppDto.getImei());
            deviceUnbindDto.setUserId(addressBookApp.getUserId());
            boolean flag = bindingDevice.DeviceUnbind(deviceUnbindDto);
            if (flag) {
                //发送到mqtt
                AddressBookMessage addressBookMessage=new AddressBookMessage();
                addressBookMessage.setType("accountMessage");
                AddressBookMessageChild addressBookMessageChild=new AddressBookMessageChild();
                addressBookMessageChild.setTimestamp(System.currentTimeMillis());
                addressBookMessageChild.setType("unbind");
                addressBookMessageChild.setImei(delAddressBookAppDto.getImei());
                addressBookMessageChild.setName(addressBookApp.getName());
                addressBookMessage.setMessage(addressBookMessageChild);
                String pushMsgData = JSONObject.toJSONString(addressBookMessage);
                //5.发送到mqtt
                String userId=addressBookApp.getUserId();;
                String agent=redisService.getCacheMapValue(userId,CommonConstant.USER_AGENT);
                String alert=name+"("+delAddressBookAppDto.getImei()+")"+I18nUtil.getMessage(CommonConstant.CALL_FOLLOW,lang);
                //判断是发IOS还是安卓
                if (CommonConstant.ANDROID.equals(agent)){
                    mqttPush.sendMqtt("3", userId, "1", "accountMessage", "unbind", pushMsgData);
                }
                else {
                    bindingDeviceImpl.sendIOS(userId, addressBookMessage, "accountMessage", "unbind", alert);
                }
            }
            //删除入通知表
            NotifyApp notifyApp=new NotifyApp();
            notifyApp.setId(UUID.fastUUID().toString(true));
            notifyApp.setImei(delAddressBookAppDto.getImei());
            notifyApp.setCreateTime(new Date());
            notifyApp.setMessageRead("0");
            notifyApp.setName(name);
            notifyApp.setPhone(addressBookApp.getPhone());
            notifyApp.setType("unbind");
            notifyApp.setUserId(addressBookApp.getUserId());
            notifyAppService.getBaseMapper().insert(notifyApp);
        }

        //开始指令下发
        addressBookAppService.sendToKafka(delAddressBookAppDto.getImei());
        return Rest.ok(I18nUtil.getMessage(FieldConstant.MESSAGES_SUCCESS,lang));
    }
    @ApiOperation(value ="批量删除通讯录成员")
    @PostMapping("/del/delBatchAddress")
    public Rest delBatchAddress(HttpServletRequest request,@RequestBody  List<String> ids,@RequestParam("imei") @ApiParam("imei号") String imei) {
        addressBookAppService.getBaseMapper().deleteBatchIds(ids);
        //开始指令下发
        addressBookAppService.sendToKafka(imei);
        return Rest.ok(I18nUtil.getMessage(FieldConstant.MESSAGES_SUCCESS,request.getHeader("Accept-Language")));
    }
    @ApiOperation(value ="转让管理员(只有管理员才能进行此操作)")
    @PostMapping("/update/setAdmin")
    public Rest setAdmin(HttpServletRequest request,@RequestBody UpdateSetAdminDto addressBookApp) {
        String lang=request.getHeader("Accept-Language");
        addressBookAppService.updateSetAdmin(addressBookApp,lang);
        return Rest.ok(I18nUtil.getMessage(FieldConstant.MESSAGES_SUCCESS,request.getHeader("Accept-Language")));
    }
}

