package com.ecell.internationalize.app.controller;
import com.ecell.internationalize.app.annotation.AppUnbindAnnotation;
import com.ecell.internationalize.app.entity.dto.ApplyFollowDto;
import com.ecell.internationalize.app.entity.dto.ApprovalDto;
import com.ecell.internationalize.app.entity.dto.BindingDeviceDto;
import com.ecell.internationalize.app.entity.dto.DeviceUnbindDto;
import com.ecell.internationalize.app.service.BindingDevice;
import com.ecell.internationalize.common.core.utils.StringUtils;
import com.ecell.internationalize.common.core.web.domain.AjaxResult;
import com.ecell.internationalize.common.issue.constant.CommonConstant;
import com.ecell.internationalize.common.redis.service.RedisService;
import com.ecell.internationalize.common.system.utlis.UploadUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.Map;

/**
 * 绑定设备
 * @author borui
 */
@Api(value="设备绑定",tags="设备绑定")
@RestController
@RequestMapping("/app/device")
public class BindingDeviceController {
    @Autowired
    private RedisService redisService;
    @Autowired
    private BindingDevice bindingDevice;
    @ApiOperation(value ="判断该设备是否已经有管理员")
    @GetMapping("/binding/scanDevice/{imei}")
    public AjaxResult ScanBindingDevice(
            @ApiParam(name ="imei",value = "设备imei",required = true) @PathVariable(value = "imei")  String imei){
       return AjaxResult.success(bindingDevice.queryDeviceByImei(imei));
    }


//    @ApiOperation(value ="判断该用户是否输入了手机号码接口",notes ="user对象为null或者user对象中phone属性 为空：表示该用户没有输入手机号")
//    @GetMapping("/isInput/phone/{email}")
//    public AjaxResult isInputPhoneByEmail(@ApiParam(name ="登录邮箱",required = true)@PathVariable("email") String email){
//
//        return AjaxResult.success(bindingDevice.isInputPhoneByEmail(email));
//    }

    @ApiOperation(value ="设备绑定添加接口")
    @PostMapping("/binding/add/device")
    public AjaxResult BindingDevice(MultipartFile file, BindingDeviceDto deviceDto) throws IOException {
        String path="";
        if (StringUtils.isNotNull(file)){
           path = UploadUtil.upload(file);
        }
        Map<String, String> map = bindingDevice.addDevice(deviceDto,path);
        if (StringUtils.isNotEmpty(map)) {
            if (CommonConstant.MESSAGE_INFO_TWO.equals(map.get(CommonConstant.MESSAGE_CODE))) {
                return AjaxResult.success();
            } else {
                return AjaxResult.error("绑定新增设备失败该设备已经绑定", map);
            }

        } else {
            return AjaxResult.error("绑定新增设备失败该设备已经绑定",map);
        }

    }


    @ApiOperation(value ="设备绑定成功之后申请关注推送消息给管理员")
    @PostMapping("/binding/send/account")
    public AjaxResult SendPushToAccount( @RequestBody ApplyFollowDto applyFollowDto){
      return   bindingDevice.SendPushToAccount(applyFollowDto) ? AjaxResult.success() :AjaxResult.error();

    }

//    @Deprecated
//    @ApiOperation(value ="此接口不用管理员审批,自动绑定")
//    @PostMapping("/automatic/binding")
//    public AjaxResult AutomaticBinding( @RequestBody ApplyFollowDto applyFollowDto){
//        return   bindingDevice.AutomaticBinding(applyFollowDto) ? AjaxResult.success() :AjaxResult.error();
//
//    }


    @ApiOperation(value ="管理员审批申请关注接口")
    @PostMapping("/binding/approval/follow")
    public AjaxResult approvalFollowStatus( @RequestBody ApprovalDto dto){
        return   bindingDevice.approvalFollowStatus(dto) ? AjaxResult.success() :AjaxResult.error();

    }



//    @ApiOperation(value ="解除与手表的绑定")
//    @GetMapping("/device/unbind")
//    public AjaxResult DeviceUnbind( @ApiParam(name ="imei",value = "设备imei",required = true) @PathVariable(value = "imei")  String imei){
//        return   bindingDevice.DeviceUnbind(imei) ? AjaxResult.success() :AjaxResult.error();
//    }

    @ApiOperation(value ="解除与手表的绑定")
    @PostMapping("/device/unbind")
    @AppUnbindAnnotation
    public AjaxResult DeviceUnbind(@RequestBody DeviceUnbindDto dto){
        return   bindingDevice.DeviceUnbind(dto) ? AjaxResult.success() :AjaxResult.error();
    }


//    @DeleteMapping("/del")
//    public void del(){
////        List<String> list = Arrays.asList("messageVoice", "messageVoiceSync", "contactsSync","contacts");
////        Map<String, Object> cacheMap = redisService.getCacheMap("867188040000098");
//////        if (StringUtils.isNotNull(cacheMap)){
//////            if (cacheMap.containsKey("contacts")){
//////                cacheMap.remove("contacts",cacheMap.get("contacts"));
//////            }
//////
//////        }
////        boolean b = redisService.deleteObject("867188040000098");
////        if (b){
////            Iterator<Map.Entry<String, Object>> iterator = cacheMap.entrySet().iterator();
////            while (iterator.hasNext()){
////                Map.Entry<String, Object> next = iterator.next();
////                if (list.stream().anyMatch(a->a.equals(next.getKey()))){
////                    iterator.remove();
////
////                }else {
////                    redisService.setCacheMapValue("867188040000098",next.getKey(),next.getValue());
////                }
////
////
////            }
////
////
////        }
////        List<ChatMessage> messages =new ArrayList<>();
////        ChatMessage chatMessage =new ChatMessage();
////        chatMessage.setId("1111");
////        chatMessage.setOpenid("22222");
////        chatMessage.setContent("你好");
////        messages.add(chatMessage);
////        redisService.setCacheMapValue("867188040000098","messageText",messages);
//        Object messageVoice = redisService.getCacheMapValue("867188040000098", "messageText");
//        String s = JSON.toJSONString(messageVoice);
//        System.out.println(s);
//        List<ChatMessage> chatMessages = JSON.parseArray(s, ChatMessage.class);
//        chatMessages.forEach(System.out::println);
//
//
//        //System.out.println(cacheMap.entrySet().size());
//
//        System.out.println("删除成功");
//
//    }

}
