package com.ecell.internationalize.app.controller;


import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ecell.internationalize.app.entity.ChantGroupApp;
import com.ecell.internationalize.app.entity.ChatGroupMemberApp;
import com.ecell.internationalize.app.entity.ChatGroupMessageApp;
import com.ecell.internationalize.app.entity.dto.ChatGroupMessageDto;
import com.ecell.internationalize.app.service.*;
import com.ecell.internationalize.app.service.impl.BindingDeviceImpl;
import com.ecell.internationalize.app.util.I18nUtil;
import com.ecell.internationalize.common.core.context.SecurityContextHolder;
import com.ecell.internationalize.common.core.entity.Rest;
import com.ecell.internationalize.common.core.utils.uuid.UUID;
import com.ecell.internationalize.common.issue.constant.CommonConstant;
import com.ecell.internationalize.common.issue.mqtt.dto.ChatMessage;
import com.ecell.internationalize.common.issue.mqtt.dto.ChatMessageChild;
import com.ecell.internationalize.common.issue.mqtt.push.MqttPush;
import com.ecell.internationalize.common.redis.service.RedisService;
import com.ecell.internationalize.common.system.constant.FieldConstant;
import com.ecell.internationalize.common.system.entity.ChatOssDto;
import com.ecell.internationalize.common.system.entity.DeviceOwnerInfoApp;
import com.ecell.internationalize.common.system.entity.User;
import com.ecell.internationalize.common.system.utlis.UploadUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.*;

/**
 * <p>
 * 群聊消息表 前端控制器
 * </p>
 *
 * @author ${author}
 * @since 2022-09-26
 */
@RestController
@RequestMapping("/chat_group_message_app")
@Api(value="微聊接口",tags={"微聊接口"})
public class ChatGroupMessageAppController {
    @Autowired
    private ChatGroupMessageAppService chatGroupMessageAppService;
    @Autowired
    private ChatGroupMemberAppService chatGroupMemberAppService;
    @Autowired
    private UserService userService;
    @Autowired
    private DeviceOwnerInfoAppService deviceOwnerInfoAppService;
    @Autowired
    private MqttPush mqttPush;
    @Autowired
    private RedisService redisService;
    @Autowired
    private ChantGroupAppService chantGroupAppService;
    @Autowired
    private BindingDeviceImpl bindingDeviceImpl;

    @ApiOperation(value ="查询微聊数据")
    @GetMapping("/message/queryChat")
    public Rest<IPage<ChatGroupMessageApp>> queryAlarm(@ApiParam(value = "群组id",required = true) String groupId, @ApiParam(value = "当前页",required = true) Integer current,
                                                       @ApiParam(value = "每页显示多少条",required = true) Integer pageSize) {
        Page<ChatGroupMessageApp> page=new Page<>(current,pageSize);
        LambdaQueryWrapper<ChatGroupMessageApp> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(ChatGroupMessageApp::getUserId, SecurityContextHolder.getStringUserId());
        lambdaQueryWrapper.eq(ChatGroupMessageApp::getGroupId,groupId);
        lambdaQueryWrapper.orderByDesc(ChatGroupMessageApp::getCreateTime);
        IPage<ChatGroupMessageApp> chatGroupMessageAppPage = chatGroupMessageAppService.getBaseMapper().selectPage(page, lambdaQueryWrapper);
        Map<String, Object> queryImageUser = queryImageUser();
        Map<String, Object> imageImei = queryImageImei();
        for (ChatGroupMessageApp chatGroupMessageApp :chatGroupMessageAppPage.getRecords()){
            //发送者类型为用户
            if ("1".equals(chatGroupMessageApp.getSendType())){
                User user = (User) queryImageUser.get(chatGroupMessageApp.getSendUser());
                String userImage=null==user?"":user.getImage();
                String userName=null==user?"":user.getName();
                chatGroupMessageApp.setImage(userImage);
                chatGroupMessageApp.setNickName(userName);
            }
            //发送者类型为设备
            else {
                DeviceOwnerInfoApp deviceOwnerInfoApp =(DeviceOwnerInfoApp)imageImei.get(chatGroupMessageApp.getSendUser());
                String imeiImage=null==deviceOwnerInfoApp?"":deviceOwnerInfoApp.getImage();
                String imeiName=null==deviceOwnerInfoApp?"":deviceOwnerInfoApp.getName();
                chatGroupMessageApp.setImage(imeiImage);
                chatGroupMessageApp.setNickName(imeiName);
            }
        }
        return Rest.ok(chatGroupMessageAppPage);
    }



    @ApiOperation(value ="新增微聊数据")
    @PostMapping("/message/addChat")
    public Rest<List<ChatGroupMessageApp>> addressImport(HttpServletRequest request, @ApiParam("语音文件上传") MultipartFile file, ChatGroupMessageDto chatGroupMessageDto) throws IOException {
        System.out.println("file:"+file);
        System.out.println("chatGroupMessageDto:"+chatGroupMessageDto);
        if (null!=file){
           ChatOssDto chatOssDto = UploadUtil.uploadChat(file);
            chatGroupMessageDto.setContent(chatOssDto.getUrl());
            chatGroupMessageDto.setSize((int) file.getSize());
            chatGroupMessageDto.setFileName(chatOssDto.getFileName());
            System.out.println("file文件大小:"+file.getSize());
        }
        List<ChatGroupMessageApp> list=new ArrayList<>();
        //imei设备入库
        LambdaQueryWrapper<ChantGroupApp> chantGroupAppLambdaQueryWrapper=new LambdaQueryWrapper<>();
        chantGroupAppLambdaQueryWrapper.eq(ChantGroupApp::getId,chatGroupMessageDto.getGroupId());
        ChantGroupApp chantGroupApp = chantGroupAppService.getBaseMapper().selectOne(chantGroupAppLambdaQueryWrapper);
        String imei="";
        if (null!=chantGroupApp){
            //封装数据
            ChatGroupMessageApp chatGroupMessageApp1 = new ChatGroupMessageApp();
            chatGroupMessageApp1.setUserId(chantGroupApp.getImei());
            imei=chantGroupApp.getImei();
            chatGroupMessageApp1.setId(com.ecell.internationalize.common.core.utils.uuid.UUID.fastUUID().toString(true));
            //用户类型：1.用户 2.设备
            chatGroupMessageApp1.setSendType("2");
            //消息类型：1:文字消息, 2:图片消息, 3:语音消息',
            chatGroupMessageApp1.setMyType("3");
            chatGroupMessageApp1.setCreateTime(new Date());
            //群组id
            chatGroupMessageApp1.setGroupId(chantGroupApp.getId());
            //语音内容
            chatGroupMessageApp1.setContent(chatGroupMessageDto.getContent());
            chatGroupMessageApp1.setSendUser(SecurityContextHolder.getStringUserId());
            chatGroupMessageApp1.setReadStatus("0");
            if ("3".equals(chatGroupMessageDto.getMyType())){
                chatGroupMessageApp1.setSize(chatGroupMessageDto.getSize());
                chatGroupMessageApp1.setDuration(chatGroupMessageDto.getDuration());
            }
            list.add(chatGroupMessageApp1);
        }

        //1.根据groupId查询群组下面所有的群成员
        LambdaQueryWrapper<ChatGroupMemberApp> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(ChatGroupMemberApp::getGroupId,chatGroupMessageDto.getGroupId());
        List<ChatGroupMemberApp> chatGroupMessageAppList=chatGroupMemberAppService.getBaseMapper().selectList(lambdaQueryWrapper);
        //2.userId和groupId一起批量入库
        ChatMessage chatMessage;
        List<ChatGroupMessageApp> returnList=new ArrayList<>();
        if (!CollectionUtils.isEmpty(chatGroupMessageAppList)) {
            for (ChatGroupMemberApp chatGroupMemberApp : chatGroupMessageAppList) {
                //封装数据
                String id= UUID.fastUUID().toString(true);
                ChatGroupMessageApp chatGroupMessageApp = new ChatGroupMessageApp();
                chatGroupMessageApp.setId(id);
                chatGroupMessageApp.setUserId(chatGroupMemberApp.getUserId());
                chatGroupMessageApp.setContent(chatGroupMessageDto.getContent());
                chatGroupMessageApp.setCreateTime(new Date());
                chatGroupMessageApp.setGroupId(chatGroupMemberApp.getGroupId());
                chatGroupMessageApp.setMyType(chatGroupMessageDto.getMyType());
                chatGroupMessageApp.setSendType("1");
                chatGroupMessageApp.setReadStatus("0");
                chatGroupMessageApp.setSendUser(SecurityContextHolder.getStringUserId());
                if ("3".equals(chatGroupMessageDto.getMyType())){
                    chatGroupMessageApp.setSize(chatGroupMessageDto.getSize());
                    chatGroupMessageApp.setDuration(chatGroupMessageDto.getDuration());
                }
                if (SecurityContextHolder.getStringUserId().equals(chatGroupMemberApp.getUserId())){
                    returnList.add(chatGroupMessageApp);
                    chatGroupMessageApp.setReadStatus("1");
                }
                list.add(chatGroupMessageApp);
            }
            //4.开始指令下发
            chatGroupMessageAppService.sendToKafka(chatGroupMessageDto);
            //数据入库
            chatGroupMessageAppService.saveBatch(list);
            //发送mqtt数据封装
            for (ChatGroupMessageApp chatGroupMessageApp:list){
                if (!SecurityContextHolder.getStringUserId().equals(chatGroupMessageApp.getUserId())) {
                    if (!imei.equals(chatGroupMessageApp.getUserId())) {
                        chatMessage = new ChatMessage();
                        chatMessage.setType("chatGroupMessage");
                        chatMessage.setBadge(1);
                        chatMessage.setGroupid(chatGroupMessageApp.getGroupId());
                        ChatMessageChild chatMessageChild = new ChatMessageChild();
                        chatMessageChild.setId(chatGroupMessageApp.getId());
                        chatMessageChild.setContent(chatGroupMessageApp.getContent());
                        chatMessageChild.setDuration(chatGroupMessageApp.getDuration());
                        chatMessageChild.setSendUser(chatGroupMessageApp.getSendUser());
                        chatMessageChild.setSendType(chatGroupMessageApp.getSendType());

                        chatMessageChild.setSenderType(chatGroupMessageApp.getSendType());
                        chatMessageChild.setType(chatGroupMessageApp.getMyType());

                        chatMessageChild.setTimestamp(System.currentTimeMillis());
                        chatMessageChild.setMyType(chatGroupMessageApp.getMyType());
                        chatMessageChild.setSenderID(chatGroupMessageApp.getSendUser());
                        chatMessage.setMessage(chatMessageChild);
                        String sendPushToChatGroup = JSONObject.toJSONString(chatMessage);
                        //5.发送到mqtt
                        String userId = chatGroupMessageApp.getUserId();
                        System.out.println("userId:" + userId);
                        String lang = request.getHeader("Accept-Language");
                        String agent = redisService.getCacheMapValue(userId, CommonConstant.USER_AGENT);
                        System.out.println("agent:" + agent);
                        String alert = I18nUtil.getMessage(CommonConstant.ACCEPT_CHANT_VIDEO, lang);
                        //判断是发IOS还是安卓
                        if (CommonConstant.ANDROID.equals(agent)) {
                            mqttPush.sendMqtt("1", userId, "1", "chatGroupMessage", "", sendPushToChatGroup);
                        } else {
                            System.out.println("语音走IOS推送==========");
                            bindingDeviceImpl.sendIOS(userId, chatMessage, "chatGroupMessage", "", alert);
                        }
                    }
                }
            }
        }
        return Rest.ok(returnList);
    }

    @ApiOperation(value ="删除微聊数据")
    @GetMapping("/message/deleteChat")
    public Rest deleteChat(HttpServletRequest request,@ApiParam(value = "群组id",required = true) String groupId) {
        LambdaQueryWrapper<ChatGroupMessageApp> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(ChatGroupMessageApp::getUserId,SecurityContextHolder.getStringUserId());
        lambdaQueryWrapper.eq(ChatGroupMessageApp::getGroupId,groupId);
        chatGroupMessageAppService.getBaseMapper().delete(lambdaQueryWrapper);
        return Rest.ok(I18nUtil.getMessage(FieldConstant.MESSAGES_SUCCESS,request.getHeader("Accept-Language")));
    }

    @ApiOperation(value ="根据消息id删除微聊数据")
    @PostMapping("/message/deleteChatById")
    public Rest deleteChatById(HttpServletRequest request,@ApiParam(value = "消息id",required = true)@RequestBody List<String> id) {
        chatGroupMessageAppService.getBaseMapper().deleteBatchIds(id);
        return Rest.ok(I18nUtil.getMessage(FieldConstant.MESSAGES_SUCCESS,request.getHeader("Accept-Language")));
    }

    @ApiOperation(value ="查询当前设备用户未读消息数")
    @GetMapping("/message/unreadTotal")
    public Rest unreadTotal(HttpServletRequest request,@ApiParam(value = "群组id",required = true) String groupId) {
        LambdaQueryWrapper<ChatGroupMessageApp> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(ChatGroupMessageApp::getGroupId,groupId);
        lambdaQueryWrapper.eq(ChatGroupMessageApp::getUserId,SecurityContextHolder.getStringUserId());
        lambdaQueryWrapper.eq(ChatGroupMessageApp::getReadStatus,"0");
        return Rest.ok(chatGroupMessageAppService.count(lambdaQueryWrapper));
    }

    @ApiOperation(value ="根据消息id更新当前消息为已读状态")
    @GetMapping("/message/updateById")
    public Rest updateById(HttpServletRequest request,@ApiParam(value = "消息id",required = true) String id) {
        ChatGroupMessageApp chatGroupMessageApp=new ChatGroupMessageApp();
        chatGroupMessageApp.setId(id);
        chatGroupMessageApp.setReadStatus("1");
        chatGroupMessageAppService.updateById(chatGroupMessageApp);
        return Rest.ok(I18nUtil.getMessage(FieldConstant.MESSAGES_SUCCESS,request.getHeader("Accept-Language")));
    }

    public Map<String,Object> queryImageUser(){
        Map<String,Object> map=new HashMap<>(16);
        List<User> userList = userService.getBaseMapper().selectList(null);
        if (!CollectionUtils.isEmpty(userList)) {
            for (User user : userList) {
                map.put(user.getUserId(),user);
            }
        }
        return map;
    }

    public Map<String,Object> queryImageImei(){
        Map<String,Object> map=new HashMap<>(16);
        List<DeviceOwnerInfoApp> deviceOwnerInfoAppList = deviceOwnerInfoAppService.getBaseMapper().selectList(null);
        if (!CollectionUtils.isEmpty(deviceOwnerInfoAppList)) {
            for (DeviceOwnerInfoApp deviceOwnerInfoApp : deviceOwnerInfoAppList) {
                map.put(deviceOwnerInfoApp.getImei(), deviceOwnerInfoApp);
            }
        }
        return map;
    }
}

