package com.ecell.internationalize.app.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.ecell.internationalize.app.entity.DeviceAlarmApp;
import com.ecell.internationalize.app.entity.dto.DelDeviceAlarmDto;
import com.ecell.internationalize.app.entity.dto.SaveDeviceAlarmDto;
import com.ecell.internationalize.app.entity.dto.StatusDeviceAlarmDto;
import com.ecell.internationalize.app.service.DeviceAlarmAppService;
import com.ecell.internationalize.app.util.I18nUtil;
import com.ecell.internationalize.common.core.entity.Rest;
import com.ecell.internationalize.common.core.utils.uuid.UUID;
import com.ecell.internationalize.common.system.constant.FieldConstant;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 闹钟设置表 前端控制器
 * </p>
 *
 * @author liy
 * @since 2022-09-07
 */
@RestController
@RequestMapping("/device_alarm_app")
@Api(value="设置闹钟接口",tags={"设置闹钟接口"})
public class DeviceAlarmAppController {
    @Autowired
    private DeviceAlarmAppService deviceAlarmAppService;

    @ApiOperation(value ="查询设置的闹钟")
    @GetMapping("/query/queryAlarm")
    public Rest<List<DeviceAlarmApp>> queryAlarm(@ApiParam(value = "设备imei号") String imei) {
      LambdaQueryWrapper<DeviceAlarmApp> lambdaQueryWrapper=new LambdaQueryWrapper<>();
      lambdaQueryWrapper.eq(DeviceAlarmApp::getImei,imei);
       return Rest.ok(deviceAlarmAppService.getBaseMapper().selectList(lambdaQueryWrapper));
    }
    @ApiOperation(value ="新增闹钟")
    @PostMapping("/insert/addAlarm")
    public Rest addAlarm(HttpServletRequest request,@RequestBody SaveDeviceAlarmDto saveDeviceAlarmDto) {
        QueryWrapper<DeviceAlarmApp> queryWrapper=new QueryWrapper<>();
        queryWrapper.eq("imei",saveDeviceAlarmDto.getImei());
        //1.查询设置的闹钟个数
        int count = deviceAlarmAppService.getBaseMapper().selectCount(queryWrapper);
        if (count>=3){
            return Rest.fail(I18nUtil.getMessage(FieldConstant.ALARM_CLOCK_NUMBER,request.getHeader("Accept-Language")));
        }
        String id = UUID.randomUUID().toString(true);
        DeviceAlarmApp deviceAlarmApp=new DeviceAlarmApp();
        deviceAlarmApp.setId(id);
        deviceAlarmApp.setImei(saveDeviceAlarmDto.getImei());
        deviceAlarmApp.setAlarmTime(saveDeviceAlarmDto.getAlarmTime());
        deviceAlarmApp.setTag(saveDeviceAlarmDto.getTag());
        deviceAlarmApp.setAlarmWeek(saveDeviceAlarmDto.getAlarmWeek());
        deviceAlarmApp.setCreateTime(new Date());
        //2.新增闹钟
        deviceAlarmAppService.getBaseMapper().insert(deviceAlarmApp);
        //3.开始指令下发
        deviceAlarmAppService.sendToKafka(deviceAlarmApp);
        return Rest.ok(I18nUtil.getMessage(FieldConstant.MESSAGES_SUCCESS,request.getHeader("Accept-Language")));
    }
    @ApiOperation(value ="修改闹钟")
    @PostMapping("/update/updateAlarm")
    public Rest updateAlarm(HttpServletRequest request,@RequestBody SaveDeviceAlarmDto saveDeviceAlarmDto) {
        DeviceAlarmApp deviceAlarmApp=new DeviceAlarmApp();
        deviceAlarmApp.setId(saveDeviceAlarmDto.getId());
        deviceAlarmApp.setImei(saveDeviceAlarmDto.getImei());
        deviceAlarmApp.setAlarmTime(saveDeviceAlarmDto.getAlarmTime());
        deviceAlarmApp.setTag(saveDeviceAlarmDto.getTag());
        deviceAlarmApp.setAlarmWeek(saveDeviceAlarmDto.getAlarmWeek());
        deviceAlarmApp.setUpdateTime(new Date());
        //1.修改闹钟信息
        deviceAlarmAppService.updateById(deviceAlarmApp);
        //2.开始指令下发
        deviceAlarmAppService.sendToKafka(deviceAlarmApp);
        return Rest.ok(I18nUtil.getMessage(FieldConstant.MESSAGES_SUCCESS,request.getHeader("Accept-Language")));
    }
    @ApiOperation(value ="删除闹钟")
    @PostMapping("/delete/deleteAlarm")
    public Rest deleteAlarm(HttpServletRequest request,@RequestBody DelDeviceAlarmDto delDeviceAlarmDto) {
        DeviceAlarmApp deviceAlarmApp=new DeviceAlarmApp();
        deviceAlarmApp.setId(delDeviceAlarmDto.getId());
        deviceAlarmApp.setImei(delDeviceAlarmDto.getImei());
        //1.删除闹钟信息
        deviceAlarmAppService.getBaseMapper().deleteById(deviceAlarmApp);
        //2.开始指令下发
        deviceAlarmAppService.sendToKafka(deviceAlarmApp);
        return Rest.ok(I18nUtil.getMessage(FieldConstant.MESSAGES_SUCCESS,request.getHeader("Accept-Language")));
    }
    @ApiOperation(value ="修改闹钟开关")
    @PostMapping("/update/updateAlarmStatus")
    public Rest updateAlarmStatus(HttpServletRequest request,@RequestBody StatusDeviceAlarmDto statusDeviceAlarmDto) {
        DeviceAlarmApp deviceAlarmApp=new DeviceAlarmApp();
        deviceAlarmApp.setId(statusDeviceAlarmDto.getId());
        deviceAlarmApp.setAlarmStatus(statusDeviceAlarmDto.getAlarmStatus());
        deviceAlarmApp.setImei(statusDeviceAlarmDto.getImei());
        //1.修改闹钟信息
        deviceAlarmAppService.updateById(deviceAlarmApp);
        //2.开始指令下发
        deviceAlarmAppService.sendToKafka(deviceAlarmApp);
        return Rest.ok(I18nUtil.getMessage(FieldConstant.MESSAGES_SUCCESS,request.getHeader("Accept-Language")));
    }
}

