package com.ecell.internationalize.app.controller;


import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.ecell.internationalize.app.entity.DeviceDisableApp;
import com.ecell.internationalize.app.entity.dto.DelDeviceDisable;
import com.ecell.internationalize.app.entity.dto.SaveDeviceDisable;
import com.ecell.internationalize.app.entity.dto.StatusDeviceDisable;
import com.ecell.internationalize.app.entity.dto.UpdateDeviceDisable;
import com.ecell.internationalize.app.service.DeviceDisableAppService;
import com.ecell.internationalize.app.util.I18nUtil;
import com.ecell.internationalize.common.core.entity.Rest;
import com.ecell.internationalize.common.core.utils.uuid.UUID;
import com.ecell.internationalize.common.issue.constant.CommonConstant;
import com.ecell.internationalize.common.redis.service.RedisService;
import com.ecell.internationalize.common.system.constant.FieldConstant;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 上课禁用(免打扰设置)表 前端控制器
 * </p>
 *
 * @author liy
 * @since 2022-09-13
 */
@RestController
@RequestMapping("/device_disable_app")
@Api(value="上课禁用接口",tags={"上课禁用接口"})
public class DeviceDisableAppController {
    @Autowired
    private DeviceDisableAppService deviceDisableAppService;
    @Autowired
    private RedisService redisService;

    @ApiOperation(value ="查询上课禁用的闹钟")
    @GetMapping("/query/disabled")
    public Rest<List<DeviceDisableApp>> queryDisabled(@ApiParam(value = "设备imei号") String imei) {
        LambdaQueryWrapper<DeviceDisableApp> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(DeviceDisableApp::getImei,imei);
        return Rest.ok(deviceDisableAppService.getBaseMapper().selectList(lambdaQueryWrapper));
    }
    @ApiOperation(value ="新增上课禁用闹钟")
    @PostMapping("/insert/disabled")
    public Rest addDisabled(HttpServletRequest request, @RequestBody SaveDeviceDisable saveDeviceDisable) {
        QueryWrapper<DeviceDisableApp> queryWrapper=new QueryWrapper<>();
        queryWrapper.eq("imei",saveDeviceDisable.getImei());
        //1.查询设置的闹钟个数
        int count = deviceDisableAppService.getBaseMapper().selectCount(queryWrapper);
        if (count>=3){
            return Rest.fail(I18nUtil.getMessage(FieldConstant.DISABLE_CLASS_NUMBER,request.getHeader("Accept-Language")));
        }
        DeviceDisableApp deviceDisableApp=new DeviceDisableApp();
        String id = UUID.randomUUID().toString(true);
        deviceDisableApp.setId(id);
        deviceDisableApp.setCreateTime(new Date());
        deviceDisableApp.setAlarmStartTime(saveDeviceDisable.getAlarmStartTime());
        deviceDisableApp.setAlarmEndTime(saveDeviceDisable.getAlarmEndTime());
        deviceDisableApp.setAlarmWeek(saveDeviceDisable.getAlarmWeek());
        deviceDisableApp.setImei(saveDeviceDisable.getImei());
        //2.新增闹钟
        deviceDisableAppService.getBaseMapper().insert(deviceDisableApp);
        //4.开始指令下发
         deviceDisableAppService.sendToKafka(deviceDisableApp);
        return Rest.ok(I18nUtil.getMessage(FieldConstant.MESSAGES_SUCCESS,request.getHeader("Accept-Language")));
    }
    @ApiOperation(value ="修改上课禁用闹钟")
    @PostMapping("/update/disabled")
    public Rest updateDisabled(HttpServletRequest request,@RequestBody UpdateDeviceDisable updateDeviceDisable) {
        DeviceDisableApp deviceDisableApp=new DeviceDisableApp();
        deviceDisableApp.setId(updateDeviceDisable.getId());
        deviceDisableApp.setUpdateTime(new Date());
        deviceDisableApp.setAlarmStartTime(updateDeviceDisable.getAlarmStartTime());
        deviceDisableApp.setAlarmEndTime(updateDeviceDisable.getAlarmEndTime());
        deviceDisableApp.setAlarmWeek(updateDeviceDisable.getAlarmWeek());
        deviceDisableApp.setImei(updateDeviceDisable.getImei());
        deviceDisableAppService.updateById(deviceDisableApp);
        //2.开始指令下发
        deviceDisableAppService.sendToKafka(deviceDisableApp);
        return Rest.ok(I18nUtil.getMessage(FieldConstant.MESSAGES_SUCCESS,request.getHeader("Accept-Language")));
    }
    @ApiOperation(value ="删除上课禁用闹钟")
    @PostMapping("/delete/disabled")
    public Rest deleteDisabled(HttpServletRequest request,@RequestBody DelDeviceDisable delDeviceDisable) {
        DeviceDisableApp deviceDisableApp=new DeviceDisableApp();
        deviceDisableApp.setImei(delDeviceDisable.getImei());
        deviceDisableApp.setId(delDeviceDisable.getId());
        deviceDisableAppService.getBaseMapper().deleteById(deviceDisableApp);
        //2.开始指令下发
        deviceDisableAppService.sendToKafka(deviceDisableApp);
        return Rest.ok(I18nUtil.getMessage(FieldConstant.MESSAGES_SUCCESS,request.getHeader("Accept-Language")));
    }

    @ApiOperation(value ="修改禁用闹钟开关")
    @PostMapping("/update/statusDisabled")
    public Rest statusDisabled(HttpServletRequest request,@RequestBody StatusDeviceDisable statusDeviceDisable) {
        DeviceDisableApp deviceDisableApp=new DeviceDisableApp();
        deviceDisableApp.setId(statusDeviceDisable.getId());
        deviceDisableApp.setImei(statusDeviceDisable.getImei());
        deviceDisableApp.setAlarmStatus(statusDeviceDisable.getAlarmStatus());
        deviceDisableAppService.updateById(deviceDisableApp);
        //2.开始指令下发
        deviceDisableAppService.sendToKafka(deviceDisableApp);
        return Rest.ok(I18nUtil.getMessage(FieldConstant.MESSAGES_SUCCESS,request.getHeader("Accept-Language")));
    }

    @ApiOperation(value ="查找手表")
    @GetMapping("/find/queryDevice")
    public Rest queryDevice(HttpServletRequest request,@ApiParam(value = "设备imei号") String imei) {
        String onlineStatus = redisService.getCacheMapValue(imei, "onlineStatus");
        if(CommonConstant.AUDIT_STATUS_ONE.equals(onlineStatus)){
            //1.开始下发查找手表指令
            deviceDisableAppService.findToKafka(imei);
            return Rest.ok(I18nUtil.getMessage(FieldConstant.MESSAGES_SUCCESS,request.getHeader("Accept-Language")));
        }else {

            return Rest.fail( I18nUtil.getMessage(FieldConstant.QUERY_DEVICE_ERROR,request.getHeader("Accept-Language")));
        }

    }
}

