package com.ecell.internationalize.app.controller;


import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.ecell.internationalize.app.entity.ChantGroupApp;
import com.ecell.internationalize.app.entity.DeviceInfo;
import com.ecell.internationalize.app.entity.DeviceVideoApp;
import com.ecell.internationalize.app.entity.dto.*;
import com.ecell.internationalize.app.service.*;
import com.ecell.internationalize.app.util.I18nUtil;
import com.ecell.internationalize.common.core.context.SecurityContextHolder;
import com.ecell.internationalize.common.core.entity.Rest;
import com.ecell.internationalize.common.issue.constant.CommonConstant;
import com.ecell.internationalize.common.issue.entity.issuedto.DialSwitchDto;
import com.ecell.internationalize.common.system.constant.FieldConstant;
import com.ecell.internationalize.common.system.entity.DeviceOwnerInfoApp;
import com.ecell.internationalize.common.system.entity.UserDeviceBindApp;
import com.ecell.internationalize.common.system.utlis.UploadUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 设备拥有者信息 前端控制器
 * </p>
 *
 * @author ${author}
 * @since 2022-08-18
 */
@Api(value="设备拥有者信息类",tags="设备拥有者信息处理接口")
@RestController
@RequestMapping("/device-owner-info-app")
public class DeviceOwnerInfoAppController {
    @Autowired
    private DeviceOwnerInfoAppService deviceOwnerInfoAppService;
    @Autowired
    private UserDeviceBindService userDeviceBindService;
    @Autowired
    private DeviceInfoService deviceInfoService;
    @Autowired
    private DeviceModelInfoService deviceModelInfoService;
    @Autowired
    private ChantGroupAppService chantGroupAppService;
    @Autowired
    private DeviceVideoAppService deviceVideoAppService;
    @ApiOperation(value ="设备扫码绑定接口")
    @GetMapping("/query/ownerInfo/{imei}")
    public Rest<DeviceOwnerInfoApp> queryOwnerInfoByImei(@ApiParam(name ="imei",value = "设备imei",required = true) @PathVariable(value = "imei")  String imei){
        return Rest.ok(deviceOwnerInfoAppService.queryOwnerInfoByimei(imei));
    }
    @ApiOperation(value ="右上角+号显示用户已绑定设备")
    @GetMapping("/show/bindDevice")
    public Rest<List<DeviceModelDto>> bindDevice(HttpServletRequest request)  {
        String userId= SecurityContextHolder.getStringUserId();
        List<DeviceModelDto> list=new ArrayList<>();
        //1.根据userId查询用户与设备绑定信息+设备用户信息
        List<UserDeviceBindDto> userDeviceBindApp = userDeviceBindService.queryByUserId(userId);
        //为空直接返回
        if (CollectionUtils.isEmpty(userDeviceBindApp)){
            return Rest.ok(I18nUtil.getMessage(FieldConstant.NOT_BIND_DEVICE,request.getHeader("Accept-Language")));
        }
        for (UserDeviceBindDto userDeviceBindDto:userDeviceBindApp){

            //2.根据imei和userId查询当前视频通话类型
            LambdaQueryWrapper<DeviceVideoApp> videoAppLambdaQueryWrapper = new LambdaQueryWrapper<>();
            videoAppLambdaQueryWrapper.eq(DeviceVideoApp::getUserId, userId);
            videoAppLambdaQueryWrapper.eq(DeviceVideoApp::getImei, userDeviceBindDto.getImei());
            DeviceVideoApp deviceVideoApp = deviceVideoAppService.getBaseMapper().selectOne(videoAppLambdaQueryWrapper);

            //1.根据iMei查询设备信息表获取设备型号id
            LambdaQueryWrapper<DeviceInfo> infoLambdaQueryWrapper=new LambdaQueryWrapper<>();
            infoLambdaQueryWrapper.eq(DeviceInfo::getImei,userDeviceBindDto.getImei());
            infoLambdaQueryWrapper.eq(DeviceInfo::getDelFlag,"1");
            infoLambdaQueryWrapper.eq(DeviceInfo::getAuditStatus,"2");

            DeviceInfo deviceInfo=deviceInfoService.getBaseMapper().selectOne(infoLambdaQueryWrapper);
            String id=null==deviceInfo?"":deviceInfo.getDeviceModelId();
            //4.根据设备型号获取对应的功能项
            DeviceModelDto deviceModelInfo = deviceModelInfoService.queryById(id);
            //5.根据imei+账号获取groupId
            LambdaQueryWrapper<ChantGroupApp> lambdaQueryWrapper=new LambdaQueryWrapper<>();
            lambdaQueryWrapper.eq(ChantGroupApp::getImei,userDeviceBindDto.getImei());
            ChantGroupApp chantGroupApp = chantGroupAppService.getBaseMapper().selectOne(lambdaQueryWrapper);
            //群组id
            String groupId=null==chantGroupApp?"":chantGroupApp.getId();
            deviceModelInfo.setImei(userDeviceBindDto.getImei());
            deviceModelInfo.setImage(userDeviceBindDto.getImage());
            deviceModelInfo.setName(userDeviceBindDto.getName());
            deviceModelInfo.setBirthday(userDeviceBindDto.getBirthday());
            deviceModelInfo.setHeight(userDeviceBindDto.getHeight());
            deviceModelInfo.setWeight(userDeviceBindDto.getWeight());
            deviceModelInfo.setSex(userDeviceBindDto.getSex());
            //设备手机号
            deviceModelInfo.setPhone(userDeviceBindDto.getPhone());
            deviceModelInfo.setGroupid(groupId);
            deviceModelInfo.setIdentity(userDeviceBindDto.getIdentity());
            deviceModelInfo.setRejectStrange(userDeviceBindDto.getRejectStrange());
            deviceModelInfo.setDialSwitch(userDeviceBindDto.getDialSwitch());
            deviceModelInfo.setPositioningModelId(userDeviceBindDto.getPositioningModelId());
            deviceModelInfo.setPositioningInterval(userDeviceBindDto.getPositioningInterval());

            deviceModelInfo.setAppKey(CommonConstant.VIDEO_APP_KEY_MAP.get(deviceModelInfo.getVideoModelName()));
            if (CommonConstant.AUDIT_STATUS_THREE.equals(deviceModelInfo.getVideoModelName())){
                deviceModelInfo.setBaiRuiId(null!=deviceVideoApp?deviceVideoApp.getVideoId():-1);
            }
            list.add(deviceModelInfo);
        }
        return Rest.ok(list);
    }
    @ApiOperation(value ="动态显示设备功能列")
    @GetMapping("/show/info")
    public Rest<DeviceModelDto> showInfo(HttpServletRequest request,@ApiParam(value = "设备imei号") String imei) throws IllegalAccessException {
        //1.根据iMei查询设备信息表获取设备型号id
        LambdaQueryWrapper<DeviceInfo> infoLambdaQueryWrapper=new LambdaQueryWrapper<>();
        infoLambdaQueryWrapper.eq(DeviceInfo::getImei,imei);
        infoLambdaQueryWrapper.eq(DeviceInfo::getDelFlag,"1");
         infoLambdaQueryWrapper.eq(DeviceInfo::getAuditStatus,"2");
        DeviceInfo deviceInfo=deviceInfoService.getBaseMapper().selectOne(infoLambdaQueryWrapper);
        //查询到的设备型号为空
        if (null==deviceInfo){
            return Rest.ok(I18nUtil.getMessage(FieldConstant.DEVICE_IS_NULL,request.getHeader("Accept-Language")));
        }
        //4.根据设备型号获取对应的功能项
        DeviceModelDto deviceModelInfo = deviceModelInfoService.queryById(deviceInfo.getDeviceModelId());

        return Rest.ok(objectToMap(deviceModelInfo));
    }

    @ApiOperation(value ="查询手表资料")
    @GetMapping("/query/deviceInfo")
    public Rest<QueryOwnerInfoDto> queryDeviceInfo(@ApiParam(value = "设备imei号") String imei) {
        return Rest.ok(deviceOwnerInfoAppService.queryDeviceInfo(imei));
    }

    @ApiOperation(value ="查询手表绑定二维码")
    @GetMapping("/query/bindInfo")
    public Rest<SecondOwnerInfoDto> queryBindInfo(HttpServletRequest request, @ApiParam(value = "设备imei号") String imei) {

        //2.根据iMei获取设备拥有者信息
        LambdaQueryWrapper<DeviceOwnerInfoApp> ownerInfoAppLambdaQueryWrapper=new LambdaQueryWrapper<>();
        ownerInfoAppLambdaQueryWrapper.eq(DeviceOwnerInfoApp::getImei,imei);
        DeviceOwnerInfoApp deviceOwnerInfoApp = deviceOwnerInfoAppService.getBaseMapper().selectOne(ownerInfoAppLambdaQueryWrapper);

        //封装数据返回
        SecondOwnerInfoDto queryOwnerInfoDto=new SecondOwnerInfoDto();
        queryOwnerInfoDto.setImage(deviceOwnerInfoApp.getImage());
        queryOwnerInfoDto.setName(deviceOwnerInfoApp.getName());
        queryOwnerInfoDto.setImei(imei);
        return Rest.ok(queryOwnerInfoDto);
    }

    @ApiOperation(value ="修改手表资料")
    @PostMapping("/update/deviceInfo")
    public Rest<QueryOwnerInfoDto> updateDeviceInfo(MultipartFile file, UpdateOwnerInfoDto deviceOwnerInfoApp) throws IOException {
        System.out.println("deviceOwnerInfoApp===="+deviceOwnerInfoApp);
        if (null!=file){
            System.out.println("file不为空："+file.getSize());
            String path = UploadUtil.upload(file);
            deviceOwnerInfoApp.setImage(path);
        }
        deviceOwnerInfoAppService.updateData(deviceOwnerInfoApp);
        return Rest.ok(deviceOwnerInfoAppService.queryDeviceInfo(deviceOwnerInfoApp.getImei()));
    }
    /**
     * 对象转化为map
     * @param deviceModelInfo 对象
     * @return 转化成功的map
     */
    private static DeviceModelDto objectToMap(DeviceModelDto deviceModelInfo) throws IllegalAccessException {

        Map<String,Object> map = new HashMap<>(16);
        Field[] declaredFields = deviceModelInfo.getClass().getDeclaredFields();
        for (Field field :declaredFields) {
            // （此处如果不设置 无法获取对象的私有属性）
            field.setAccessible(true);
            map.put(field.getName(),field.get(deviceModelInfo));
        }
        //过滤value为0的字段
        map.entrySet().removeIf(m -> "0".equals(m.getValue()));
        return JSON.parseObject(JSON.toJSONString(map), DeviceModelDto.class);
    }
    @ApiOperation(value ="拒绝陌生来电")
    @PostMapping("/reject/strangeCalls")
    public Rest strangeCalls(HttpServletRequest request, @RequestBody RejectDto rejectDto) {
        //1.根据imei更新设备用户是否接收陌生来电状态
        DeviceOwnerInfoApp deviceOwnerInfoApp=new DeviceOwnerInfoApp();
        deviceOwnerInfoApp.setImei(rejectDto.getImei());
        deviceOwnerInfoApp.setRejectStrange(rejectDto.getRejectStrange());
        LambdaQueryWrapper<DeviceOwnerInfoApp> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(DeviceOwnerInfoApp::getImei,deviceOwnerInfoApp.getImei());
        deviceOwnerInfoAppService.getBaseMapper().update(deviceOwnerInfoApp,lambdaQueryWrapper);
        //2.下发指令
        deviceOwnerInfoAppService.sendToKafka(deviceOwnerInfoApp);
        return Rest.ok(I18nUtil.getMessage(FieldConstant.MESSAGES_SUCCESS,request.getHeader("Accept-Language")));
    }
    @ApiOperation(value ="打电话给手表")
    @GetMapping("/device/call")
    public Rest<DeviceOwnerVo> deviceCall(HttpServletRequest request,@ApiParam(value = "设备imei号") String imei){
        return Rest.ok(deviceOwnerInfoAppService.queryOwnerInfo(imei));
    }
    @ApiOperation(value ="设备远程关机")
    @GetMapping("/device/shutDown")
    public Rest deviceShutdown(HttpServletRequest request,@ApiParam(value = "设备imei号") String imei){
        //1.根据账号和imei号查询当前用户是否是管理员，非管理员无法操作
        LambdaQueryWrapper<UserDeviceBindApp> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(UserDeviceBindApp::getUserId,SecurityContextHolder.getStringUserId());
        lambdaQueryWrapper.eq(UserDeviceBindApp::getImei,imei);
        UserDeviceBindApp userDeviceBindApp = userDeviceBindService.getBaseMapper().selectOne(lambdaQueryWrapper);
        if (null==userDeviceBindApp){
            return Rest.fail(I18nUtil.getMessage(FieldConstant.DEVICE_BIND_USER,request.getHeader("Accept-Language")));
        }
        if (!"2".equals(userDeviceBindApp.getIdentity())){
            return Rest.fail(I18nUtil.getMessage(FieldConstant.ADMIN_OPERATION,request.getHeader("Accept-Language")));
        }
        //2.为管理员，进行远程关机指令下发
        deviceOwnerInfoAppService.shutDownToKafka(imei);
        return Rest.ok(I18nUtil.getMessage(FieldConstant.MESSAGES_SUCCESS,request.getHeader("Accept-Language")));
    }
    @ApiOperation(value ="拨号盘开关")
    @PostMapping("/device/dialSwitch")
    public Rest dialSwitch(HttpServletRequest request, @RequestBody DialSwitchDto dialSwitchDto) {
        //1.根据imei更新拨号盘状态
        DeviceOwnerInfoApp deviceOwnerInfoApp=new DeviceOwnerInfoApp();
        deviceOwnerInfoApp.setDialSwitch(dialSwitchDto.getDialSwitch());
        LambdaQueryWrapper<DeviceOwnerInfoApp> deviceOwnerInfoAppLambdaQueryWrapper=new LambdaQueryWrapper<>();
        deviceOwnerInfoAppLambdaQueryWrapper.eq(DeviceOwnerInfoApp::getImei,dialSwitchDto.getImei());
        deviceOwnerInfoAppService.update(deviceOwnerInfoApp,deviceOwnerInfoAppLambdaQueryWrapper);
        //2.下发指令
        deviceOwnerInfoAppService.sendDialSwitchToKafka(dialSwitchDto);
        return Rest.ok(I18nUtil.getMessage(FieldConstant.MESSAGES_SUCCESS,request.getHeader("Accept-Language")));
    }
}

