package com.ecell.internationalize.app.controller;


import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.ecell.internationalize.app.entity.dto.DeviceVideoConfigDto;
import com.ecell.internationalize.app.entity.dto.DeviceVideoDto;
import com.ecell.internationalize.app.service.DeviceOwnerInfoAppService;
import com.ecell.internationalize.app.service.DeviceVideoAppService;
import com.ecell.internationalize.app.util.I18nUtil;
import com.ecell.internationalize.common.core.context.SecurityContextHolder;
import com.ecell.internationalize.common.core.entity.Rest;
import com.ecell.internationalize.common.issue.constant.CommonConstant;
import com.ecell.internationalize.common.system.entity.DeviceOwnerInfoApp;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * <p>
 * 视频通话表 前端控制器
 * </p>
 *
 * @author ${author}
 * @since 2022-10-17
 */
@RestController
@Slf4j
@RequestMapping("/device_video_app")
@Api(value="视频通话",tags={"视频通话接口"})
public class DeviceVideoAppController {
    @Autowired
    private DeviceVideoAppService deviceVideoAppService;
    @Autowired
    private DeviceOwnerInfoAppService deviceOwnerInfoAppService;
    @ApiOperation(value ="根据imei查询可视频通话成员")
    @GetMapping("/query/queryVideo")
    public Rest<List<DeviceVideoDto>> queryAddress(@ApiParam(value = "设备imei号") String imei) {
        return Rest.ok(deviceVideoAppService.findVideoList(imei));
    }
    @ApiOperation(value ="返回视频id，参数配置(分辨率、通话时长限制等)信息")
    @GetMapping("/query/userQueryVideo")
    public Rest<DeviceVideoConfigDto> userQueryVideo(@ApiParam(value = "设备imei号") String imei, @ApiParam(value = "视频通话类型(0:视频,1:语音)") String callType, @ApiParam(value = "手表发起传:device,APP发起传：APP") String type) {
        DeviceVideoConfigDto deviceVideoConfigDto = deviceVideoAppService.userQueryVideo(imei, callType,type);
        if (null == deviceVideoConfigDto){
            return Rest.ok("");
        }
        return Rest.ok(deviceVideoConfigDto);
    }

    @ApiOperation(value ="与设备进行视频通话")
    @GetMapping("/video/videoCall")
    public Rest videoCall(HttpServletRequest request, @ApiParam(value = "设备imei号") String imei, @ApiParam(value = "视频通话类型(0:视频,1:语音)") String callType) {
        int i = deviceVideoAppService.videoCall(imei, callType, CommonConstant.DOWN_VIDEO_CALL_WITH_WATCH, SecurityContextHolder.getStringUserId(),null,null,null);
        if (CommonConstant.VIDEO_RESULT_PRE_ONE == i){
            return Rest.fail(I18nUtil.getMessage(CommonConstant.EXIT_LIST_VIDEO_CALL,request.getHeader("Accept-Language")));
        }
        if (CommonConstant.VIDEO_RESULT_PRE_TWO == i){
            return Rest.fail(I18nUtil.getMessage(CommonConstant.CAN_NOT_VIDEO_CALL,request.getHeader("Accept-Language")));
        }
        if (CommonConstant.VIDEO_RESULT_PRE_THREE == i){
            return Rest.fail(I18nUtil.getMessage(CommonConstant.DISABLED_VIDEO_CALL,request.getHeader("Accept-Language")));
        }
        if (CommonConstant.NUMBER_ZERO == i){
            return Rest.fail(I18nUtil.getMessage(CommonConstant.NOT_INTERNET_VIDEO_CALL,request.getHeader("Accept-Language")));
        }
        if (CommonConstant.PRE_NUMBER_FOUR == i){
            log.info("拨打视频电话前限制 .............新JF 553 机制限制提示 ............");
            return Rest.fail(I18nUtil.getMessage(CommonConstant.LONG_TIME_VIDEO_CALL,request.getHeader("Accept-Language")));
        }
        if (CommonConstant.PRE_NUMBER_FIVE == i){
            return Rest.fail(I18nUtil.getMessage(CommonConstant.PROCESS_VIDEO_CALL,request.getHeader("Accept-Language")));
        }
        LambdaQueryWrapper<DeviceOwnerInfoApp> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(DeviceOwnerInfoApp::getImei,imei);
        DeviceOwnerInfoApp deviceOwnerInfoApp = deviceOwnerInfoAppService.getBaseMapper().selectOne(lambdaQueryWrapper);
        String videoId=deviceOwnerInfoApp.getNewVideoId();
        return Rest.ok("操作成功",videoId);
    }
    @ApiOperation(value ="APP发起视频/语音通话后主动挂断(在手表还未接到响铃的情况下)")
    @GetMapping("/video/deleteCall")
    public Rest deleteCall(@ApiParam(value = "设备imei号") String imei,@ApiParam(value = "挂断的视频的id") String videoId) {
        deviceVideoAppService.deleteCall(imei,videoId);
        return Rest.ok("");
    }


    @ApiOperation(value ="声网====》APP发起视频/语音通话后主动挂断(在手表还未接到响铃的情况下)")
    @GetMapping("/video/hangUpCall")
    public Rest hangUpCall(@ApiParam(value = "设备imei号") String imei,@ApiParam(value = "挂断的视频的id") String videoId) {
        deviceVideoAppService.hangUpCall(imei,videoId);
        return Rest.ok("挂断成功");
    }
}

