package com.ecell.internationalize.app.controller;
import com.ecell.internationalize.app.entity.dto.HeartTemperatureConfigDto;
import com.ecell.internationalize.app.service.HeartTemperatureConfigService;
import com.ecell.internationalize.common.core.entity.Rest;
import com.ecell.internationalize.common.issue.constant.CommonConstant;
import com.ecell.internationalize.common.redis.service.RedisService;
import com.ecell.internationalize.common.system.entity.HeartTemperatureRecord;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.text.ParseException;
import java.util.List;

/**
 * @Title: HeartAndTemperatureController
 * @Author: liy
 * @Date: 2022/10/28 11:40
 * @Description:
 * @Version:1.0
 */
@RestController
@Api(value="心率、体温接口",tags={"心率、体温接口"})
@RequestMapping("/heart_temperature")
public class HeartAndTemperatureController {
    @Autowired
    private HeartTemperatureConfigService heartTemperatureConfigService;
    @Autowired
    private RedisService redisService;

    @ApiOperation(value ="查询心率/体温记录和配置信息")
    @GetMapping("/query/queryHeartOrTemperature")
    public Rest<HeartTemperatureConfigDto> queryHeartOrTemperature(@ApiParam(value = "设备imei号") String imei, @ApiParam("type为：1 表示查询心率配置,type为 2 表示查询体温配置,3 表示血氧配置")String type, @ApiParam(value = "日期") String date) throws ParseException {
        return Rest.ok(heartTemperatureConfigService.queryHeartOrTemperature(imei,type,date));
    }

    @ApiOperation(value ="开始测量心率/体温")
    @GetMapping("/query/checkHeartOrTemperature")
    public Rest checkHeartOrTemperature(@ApiParam(value = "设备imei号") String imei, @ApiParam("type为：1 表示查询心率配置,type为 2 表示查询体温配置")String type, @ApiParam(value = "cmd为1:表示开启开关,cmd为2:上传频率") String cmd,@ApiParam(value = "上传开关状态/上传频率") String obj){
        String onlineStatus = null != redisService.getCacheMapValue(imei, "onlineStatus")?redisService.getCacheMapValue(imei, "onlineStatus").toString():"0";
        if (CommonConstant.AUDIT_STATUS_ZERO.equals(onlineStatus)){
            return Rest.fail("设备未联网或者不在线");
        }
        heartTemperatureConfigService.checkHeartOrTemperature(imei,type,cmd,obj);
        return Rest.ok("");
    }
    //新增血氧查询接口 20240603
    @ApiOperation(value ="查询血氧数据")
    @Deprecated()
    @GetMapping("/query/queryHeartOrBodyOxygen")
    public Rest<List<HeartTemperatureRecord>> queryHeartOrBodyOxygen(@ApiParam(value = "设备imei号") String imei) throws ParseException {
        return Rest.ok(heartTemperatureConfigService.queryHeartOrBodyOxygen(imei));
    }


}
