package com.ecell.internationalize.app.controller;


import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.ecell.internationalize.app.entity.NotifyApp;
import com.ecell.internationalize.app.entity.ReminderMessageApp;
import com.ecell.internationalize.app.entity.dto.UnReadTotal;
import com.ecell.internationalize.app.service.DeviceAttentionAppService;
import com.ecell.internationalize.app.service.NotifyAppService;
import com.ecell.internationalize.app.service.ReminderMessageAppService;
import com.ecell.internationalize.common.core.context.SecurityContextHolder;
import com.ecell.internationalize.common.core.entity.Rest;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 提醒消息表 前端控制器
 * </p>
 *
 * @author ${author}
 * @since 2022-09-26
 */
@RestController
@RequestMapping("/reminder_message_app")
@Api(value="查看手表的告警信息（低电提醒、超出围栏等）",tags={"查看手表的告警信息（低电提醒、超出围栏等）"})
public class ReminderMessageAppController {
    @Autowired
    private ReminderMessageAppService reminderMessageAppService;
    @Autowired
    private DeviceAttentionAppService deviceAttentionAppService;
    @Autowired
    private NotifyAppService notifyAppService;
    @ApiOperation(value ="查询手表的告警信息")
    @GetMapping("/query/queryRemind")
    public Rest<IPage<ReminderMessageApp>> queryRemind(@ApiParam(value = "当前页",required = true) Integer current, @ApiParam(value = "每页显示多少条",required = true) Integer pageSize) {
        return Rest.ok(reminderMessageAppService.queryRemind(current,pageSize));
    }
    @ApiOperation(value ="修改已读/未读状态")
    @PostMapping("/update/updateRemind")
    public Rest updateRemind(HttpServletRequest request, @RequestBody ReminderMessageApp reminderMessageApp) {
        reminderMessageAppService.updateById(reminderMessageApp);
        return Rest.ok("");
    }
    @ApiOperation(value ="删除提醒消息")
    @PostMapping("/delete/deleteRemind")
    public Rest deleteRemind() {
        String userId=SecurityContextHolder.getStringUserId();
        LambdaQueryWrapper<ReminderMessageApp> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(ReminderMessageApp::getUserId,userId);
        reminderMessageAppService.getBaseMapper().delete(lambdaQueryWrapper);
        return Rest.ok("");
    }

    @ApiOperation(value ="未读消息数")
    @GetMapping("/query/unReadTotal")
    public Rest<List<UnReadTotal>> unReadTotal() {
        String userId= SecurityContextHolder.getStringUserId();
        if (null == userId){
            return Rest.fail("不存在该用户");
        }
        List<UnReadTotal> list=new ArrayList<>();
        //1.根据userId和未读数查询消息提醒表
        LambdaQueryWrapper<ReminderMessageApp> reminderMessageAppLambdaQueryWrapper=new LambdaQueryWrapper<>();
        reminderMessageAppLambdaQueryWrapper.eq(ReminderMessageApp::getUserId,userId);
        reminderMessageAppLambdaQueryWrapper.eq(ReminderMessageApp::getMessageRead,"0");
        int reminderTotal = reminderMessageAppService.count(reminderMessageAppLambdaQueryWrapper);
        UnReadTotal unReadTotal=new UnReadTotal();
        unReadTotal.setType("1");
        unReadTotal.setTotal(reminderTotal);
        list.add(unReadTotal);
        //2.查询关注记录表
        int attentionTotal = deviceAttentionAppService.queryUnreadCount();
        UnReadTotal unReadTotal1=new UnReadTotal();
        unReadTotal1.setType("2");
        unReadTotal1.setTotal(attentionTotal);
        list.add(unReadTotal1);
        //3.查询通知表
        LambdaQueryWrapper<NotifyApp> notifyAppLambdaQueryWrapper=new LambdaQueryWrapper<>();
        notifyAppLambdaQueryWrapper.eq(NotifyApp::getUserId,userId);
        notifyAppLambdaQueryWrapper.eq(NotifyApp::getMessageRead,"0");
        int count = notifyAppService.count(notifyAppLambdaQueryWrapper);
        UnReadTotal unReadTotal2=new UnReadTotal();
        unReadTotal2.setType("3");
        unReadTotal2.setTotal(count);
        list.add(unReadTotal2);
        return Rest.ok(list);
    }
}

