package com.ecell.internationalize.app.controller;


import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.ecell.internationalize.app.entity.SafetyRailApp;
import com.ecell.internationalize.app.entity.dto.SaveSafetyRailApp;
import com.ecell.internationalize.app.entity.dto.UpdateSafetyRailApp;
import com.ecell.internationalize.app.service.SafetyRailAppService;
import com.ecell.internationalize.app.util.I18nUtil;
import com.ecell.internationalize.common.core.context.SecurityContextHolder;
import com.ecell.internationalize.common.core.entity.Rest;
import com.ecell.internationalize.common.core.utils.uuid.UUID;
import com.ecell.internationalize.common.system.constant.FieldConstant;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author liy
 * @since 2022-10-11
 */
@RestController
@RequestMapping("/safety_rail_app")
@Api(value="安全围栏",tags={"安全围栏接口"})
public class SafetyRailAppController {
    @Autowired
    private SafetyRailAppService safetyRailAppService;
    @ApiOperation(value ="根据imei号查询安全围栏信息")
    @GetMapping("/query/queryRails")
    public Rest<List<SafetyRailApp>> queryRails(@RequestParam("imei") @ApiParam(value = "设备imei号") String imei) {
        LambdaQueryWrapper<SafetyRailApp> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(SafetyRailApp::getImei,imei);
        return Rest.ok(safetyRailAppService.getBaseMapper().selectList(lambdaQueryWrapper));
    }
    @ApiOperation(value ="新增安全围栏信息")
    @PostMapping("/insert/addRails")
    public Rest<SafetyRailApp> addRails(HttpServletRequest request, @RequestBody SaveSafetyRailApp saveSafetyRailApp) {
        //根据imei号查询是否超过4个
        LambdaQueryWrapper<SafetyRailApp> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(SafetyRailApp::getImei,saveSafetyRailApp.getImei());
        int count = safetyRailAppService.count(lambdaQueryWrapper);
        if (count>=4){
            return Rest.fail(I18nUtil.getMessage(FieldConstant.RAILS_COUNT,request.getHeader("Accept-Language")));
        }
        SafetyRailApp safetyRailApp=new SafetyRailApp();
        //id
        safetyRailApp.setId(UUID.fastUUID().toString(true));
        //创建人
        safetyRailApp.setCreateUser(SecurityContextHolder.getStringUserId());
        //安全区域名字
        safetyRailApp.setName(saveSafetyRailApp.getName());
        //经度
        safetyRailApp.setLon(saveSafetyRailApp.getLon());
        //纬度
        safetyRailApp.setLat(saveSafetyRailApp.getLat());
        //安全区域半径
        safetyRailApp.setRadius(saveSafetyRailApp.getRadius());
        //安全区域地址
        safetyRailApp.setAddress(saveSafetyRailApp.getAddress());
        //重复星期数
        safetyRailApp.setWeek(saveSafetyRailApp.getWeek());
        //开始时间, 24小时制, eg: '08:00'
        safetyRailApp.setStartTime(saveSafetyRailApp.getStartTime());
        //结束时间, 24小时制, eg: '08:00'
        safetyRailApp.setEndTime(saveSafetyRailApp.getEndTime());
        //是否开启(0:关闭 1：开启)
        safetyRailApp.setStatus("1");
        //设备imei
        safetyRailApp.setImei(saveSafetyRailApp.getImei());
        //最后一次进入或离开该安全区域的动作('enter','leave')
        safetyRailAppService.save(safetyRailApp);
        return Rest.ok(safetyRailApp);
    }
    @ApiOperation(value ="修改安全围栏信息")
    @PostMapping("/update/updateRails")
    public Rest<SafetyRailApp> updateRails(HttpServletRequest request, @RequestBody UpdateSafetyRailApp updateSafetyRailApp) {
        SafetyRailApp safetyRailApp=new SafetyRailApp();
        //id
        safetyRailApp.setId(updateSafetyRailApp.getId());
        //安全区域名字
        safetyRailApp.setName(updateSafetyRailApp.getName());
        //经度
        safetyRailApp.setLon(updateSafetyRailApp.getLon());
        //纬度
        safetyRailApp.setLat(updateSafetyRailApp.getLat());
        //安全区域半径
        safetyRailApp.setRadius(updateSafetyRailApp.getRadius());
        //安全区域地址
        safetyRailApp.setAddress(updateSafetyRailApp.getAddress());
        //重复星期数
        safetyRailApp.setWeek(updateSafetyRailApp.getWeek());
        //开始时间, 24小时制, eg: '08:00'
        safetyRailApp.setStartTime(updateSafetyRailApp.getStartTime());
        //结束时间, 24小时制, eg: '08:00'
        safetyRailApp.setEndTime(updateSafetyRailApp.getEndTime());
        safetyRailApp.setStatus(updateSafetyRailApp.getStatus());
        //最后一次进入或离开该安全区域的动作('enter','leave')
        safetyRailAppService.getBaseMapper().updateById(safetyRailApp);
        return Rest.ok(safetyRailAppService.getBaseMapper().selectById(updateSafetyRailApp.getId()));
    }

    @ApiOperation(value ="修改启用状态")
    @PostMapping("/update/updateRailStatus")
    public Rest updateRailStatus(HttpServletRequest request, @RequestParam("status") @ApiParam("启用状态：0：停用，1：启用") String status,@RequestParam("id") @ApiParam("主键id") String id) {
        SafetyRailApp safetyRailApp=new SafetyRailApp();
        //id
        safetyRailApp.setId(id);
        safetyRailApp.setStatus(status);
        safetyRailAppService.getBaseMapper().updateById(safetyRailApp);
        return Rest.ok(I18nUtil.getMessage(FieldConstant.MESSAGES_SUCCESS,request.getHeader("Accept-Language")));
    }
    @ApiOperation(value ="删除安全围栏信息")
    @PostMapping("/delete/delRails")
    public Rest delRails(HttpServletRequest request,@RequestParam("id") @ApiParam("主键id") String id) {
        safetyRailAppService.getBaseMapper().deleteById(id);
        return Rest.ok(I18nUtil.getMessage(FieldConstant.MESSAGES_SUCCESS,request.getHeader("Accept-Language")));
    }
}

