package com.ecell.internationalize.app.controller;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.ecell.internationalize.app.entity.UserVo;
import com.ecell.internationalize.app.entity.dto.*;
import com.ecell.internationalize.app.service.BindingDevice;
import com.ecell.internationalize.app.service.UserDeviceBindService;
import com.ecell.internationalize.app.service.UserService;
import com.ecell.internationalize.app.service.impl.BindingDeviceImpl;
import com.ecell.internationalize.app.util.I18nUtil;
import com.ecell.internationalize.app.util.UserAgentUtil;
import com.ecell.internationalize.common.core.context.SecurityContextHolder;
import com.ecell.internationalize.common.core.entity.Rest;
import com.ecell.internationalize.common.core.utils.StringUtils;
import com.ecell.internationalize.common.issue.constant.CommonConstant;
import com.ecell.internationalize.common.issue.mqtt.dto.AccountMessage;
import com.ecell.internationalize.common.issue.mqtt.dto.AccountMessageChild;
import com.ecell.internationalize.common.issue.mqtt.push.MqttPush;
import com.ecell.internationalize.common.issue.util.ApplicationUtil;
import com.ecell.internationalize.common.issue.util.RandomUtil;
import com.ecell.internationalize.common.redis.service.RedisService;
import com.ecell.internationalize.common.security.utils.SecurityUtils;
import com.ecell.internationalize.common.system.constant.FieldConstant;
import com.ecell.internationalize.common.system.entity.User;
import com.ecell.internationalize.common.system.entity.UserDeviceBindApp;
import com.ecell.internationalize.common.system.utlis.UploadUtil;
import cz.mallat.uasparser.UserAgentInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.mail.SimpleMailMessage;
import org.springframework.mail.javamail.JavaMailSender;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.Date;
import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * @Title: UserController
 * @Author: liy
 * @Date: 2022/8/16 15:58
 * @Description:
 * @Version:1.0
 */
@RequestMapping("/user")
@RestController
@Api(value="用户注册登录",tags={"用户注册登录接口"})
public class UserController {
    private static final Logger logger = LoggerFactory.getLogger(UserController.class);
    @Autowired
    private JavaMailSender mailSender;
    @Value("${spring.mail.username}")
    private String from;
    @Autowired
    private RedisService redisService;
    @Autowired
    private UserService userService;
    @Autowired
    private MqttPush mqttPush;
    @Autowired
    private UserDeviceBindService userDeviceBindService;
    @Autowired
    private BindingDevice bindingDevice;
    /**
     * 请求邮箱验证码
     * @param email 邮箱
     */
    @ApiOperation("请求邮箱验证码,为防止重复请求,建议前端做计数器,一分钟请求一次该接口")
    @GetMapping("send/email")
    public Rest sendMail(@ApiParam(name ="email",value = "邮箱账号",required = true) @RequestParam("email") String email, HttpServletRequest request, @ApiParam(value = "请求类型:(1:注册,2:重置密码)") String type) {
        if (FieldConstant.SECOND_FLAG.equals(type)){
            LambdaQueryWrapper<User> lambdaQueryWrapper=new LambdaQueryWrapper<>();
            lambdaQueryWrapper.eq(User::getUserEmail,email);
            User user = userService.getBaseMapper().selectOne(lambdaQueryWrapper);
            if (null == user){
                return Rest.fail(I18nUtil.getMessage(FieldConstant.EMAIL_LOGIN_EXIT,request.getHeader("Accept-Language")));
            }
        }
        SimpleMailMessage message = new SimpleMailMessage();
        //获取验证码
        String code = RandomUtil.generateVerCode();
        //放入redis,时间为5分钟
        redisService.setCacheObject(email, code, (long) 300, TimeUnit.SECONDS);
        message.setFrom(from);
        message.setTo(email);
        message.setSubject(I18nUtil.getMessage(FieldConstant.EMAIL_SUBJECT,request.getHeader("Accept-Language")));
        message.setSentDate(new Date());
        message.setText(I18nUtil.getMessage(FieldConstant.EMAIL_TEXT_PRE,request.getHeader("Accept-Language"))+ code + I18nUtil.getMessage(FieldConstant.EMAIL_TEXT_TAIL,request.getHeader("Accept-Language")));
        mailSender.send(message);
        return Rest.ok(I18nUtil.getMessage(FieldConstant.MESSAGES_SUCCESS,request.getHeader("Accept-Language")));
    }
    /**
     * App注册接口
     * @param userDto 接收实体
     */
    @ApiOperation("App注册接口")
    @PostMapping("send/register")
    public Rest register(@RequestBody UserDto userDto, HttpServletRequest request){
        LambdaQueryWrapper<User> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(User::getUserEmail,userDto.getEmail());
        User user = userService.getBaseMapper().selectOne(lambdaQueryWrapper);
        //用户已注册
        if (null!=user){
            return Rest.fail(I18nUtil.getMessage(FieldConstant.EMAIL_REGISTER_EXIT,request.getHeader("Accept-Language")));
        }
        //验证码过期
        if (!redisService.hasKey(userDto.getEmail())){
            return Rest.fail(I18nUtil.getMessage(FieldConstant.EMAIL_CODE_EXPIRE,request.getHeader("Accept-Language")));
        }

        logger.info("redis获取到的验证码：{},APP传过来的验证码:{}",redisService.getCacheObject(userDto.getEmail()).toString(),userDto.getCode());
        //验证码错误
        if (!redisService.getCacheObject(userDto.getEmail()).toString().equals(userDto.getCode())){

            return Rest.fail(I18nUtil.getMessage(FieldConstant.EMAIL_CODE_FAIL,request.getHeader("Accept-Language")));
        }
        //获取手机相关信息
        String userAgent = request.getHeader("User-Agent");
        userDto.setUserAgent(userAgent);
        logger.info("获取到的userAgent信息：{}",userAgent);
        //调用新增接口
        userService.insertUser(userDto);
        return Rest.ok(I18nUtil.getMessage(FieldConstant.MESSAGES_SUCCESS,request.getHeader("Accept-Language")));
    }
    /**
     * App登陆接口
     * @param userDto 邮箱
     */
    @ApiOperation("App登陆接口")
    @PostMapping("send/login")
    public Rest login(@RequestBody UserLoginDto userDto, HttpServletRequest request) throws IOException {
        //1.根据email查询当前登陆用户
        LambdaQueryWrapper<User> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(User::getUserEmail,userDto.getEmail());
        User user = userService.getBaseMapper().selectOne(lambdaQueryWrapper);
        //2.判断当前用户是否存在
        if (null == user){
            return Rest.fail(I18nUtil.getMessage(FieldConstant.EMAIL_LOGIN_EXIT,request.getHeader("Accept-Language")));
        }
        //3.判断当前用户是否被禁用
        if (FieldConstant.MATH_ONE.equals(user.getStatus())){
            return Rest.fail(I18nUtil.getMessage(FieldConstant.EMAIL_STOP,request.getHeader("Accept-Language")));
        }
        //4.判断密码是否一致
        if (!SecurityUtils.matchesPassword(userDto.getPassword(), user.getPassword())){
            return Rest.fail(I18nUtil.getMessage(FieldConstant.EMAIL_PWD_ERROR,request.getHeader("Accept-Language")));
        }
        //5.获取手机相关信息
        String userAgent = request.getHeader("User-Agent");
        UserAgentInfo userAgentInfo = UserAgentUtil.getUasParser().parse(userAgent);
        logger.info("操作系统："+userAgentInfo.getOsName());
        redisService.setCacheMapValue(user.getUserId(), CommonConstant.USER_AGENT,userAgentInfo.getOsName());
        String language=request.getHeader("Accept-Language");
        redisService.setCacheMapValue(user.getUserId(), CommonConstant.IOS_LANGUAGE,language);


//        user.setUserAgent(userAgentInfo.getOsName());
        user.setCompleteUserAgent(userAgent);
        user.setLastLoginTime(new Date());
        //7.登陆成功
//        userService.updateById(user);
        //终端只允许一台设备登录
        String redisUserKey = "user_keys:"+ userDto.getEmail();
        System.out.println("redisUserKey:"+redisUserKey);

        if (redisService.hasKey(redisUserKey)) {
            String redisTokenKey = "app_tokens:"+ redisService.getCacheObject(redisUserKey);
            System.out.println("获取到的redisTokenKey："+redisTokenKey);
            redisService.deleteObject(redisTokenKey);
            System.out.println("删除后的app_tokens:"+redisService.getCacheObject(redisTokenKey));
            //redis存在,当另外一台设备同时登录时的处理
            AccountMessage accountMessage=new AccountMessage();
            accountMessage.setType("accountMessage");
            AccountMessageChild accountMessageChild=new AccountMessageChild();
            accountMessageChild.setType("LoginOffLine");
            accountMessageChild.setTimestamp(System.currentTimeMillis());
            accountMessageChild.setContent(I18nUtil.getMessage(FieldConstant.USER_HAS_LOGIN,request.getHeader("Accept-Language")));
            accountMessageChild.setAccesstoken(redisService.getCacheObject(redisUserKey));
            accountMessage.setMessage(accountMessageChild);
            String pushMsgData= JSONObject.toJSONString(accountMessage);
            //判断是发IOS还是安卓
            System.out.println("数据库用户机型:"+user.getUserAgent());
            if (CommonConstant.ANDROID.equals(user.getUserAgent())){
                mqttPush.sendMqtt("3",user.getUserId(),"1","accountMessage","LoginOffLine",pushMsgData);
                logger.info("安卓系统用户：{}, 异地登录, 推送成功,pushMsgData:{}",userDto.getEmail(),pushMsgData);
            }

            if (CommonConstant.IOS.equals(user.getUserAgent())) {
                String deviceToken = redisService.getCacheMapValue(user.getUserId(), CommonConstant.DEVICE_TOKEN);
                System.out.println("deviceToken:"+deviceToken);
                String alert=I18nUtil.getMessage(FieldConstant.USER_HAS_LOGIN,request.getHeader("Accept-Language"));
                ApplicationUtil.getApplicationContext().getBean(BindingDeviceImpl.class).sendIOSLogin(user.getUserId(), accountMessage, "accountMessage", "LoginOffLine", alert,deviceToken);
                logger.info("苹果系统用户：{}, 异地登录, 推送成功,pushMsgData:{}",userDto.getEmail(),accountMessage);
            }
        }
        //8.生成token返回
        return Rest.ok(userService.createToken(user,userAgentInfo.getOsName(),request.getHeader("deviceToken"),request.getHeader("package"),request.getHeader("version")));
    }
    /**
     * 忘记密码功能
     * @param userDto 邮箱
     */
    @ApiOperation("忘记密码功能")
    @PostMapping("send/updatePwd")
    public Rest updatePwd(@RequestBody UserDto userDto, HttpServletRequest request){
        //1.根据email查询当前登陆用户
        LambdaQueryWrapper<User> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(User::getUserEmail,userDto.getEmail());
        User user = userService.getBaseMapper().selectOne(lambdaQueryWrapper);
        //2.判断当前用户是否存在
        if (null == user){
            return Rest.fail(I18nUtil.getMessage(FieldConstant.EMAIL_LOGIN_EXIT,request.getHeader("Accept-Language")));
        }
        //验证码过期
        if (!redisService.hasKey(userDto.getEmail())){
            return Rest.fail(I18nUtil.getMessage(FieldConstant.EMAIL_CODE_EXPIRE,request.getHeader("Accept-Language")));
        }
        //验证码错误
        if (!redisService.getCacheObject(userDto.getEmail()).toString().equals(userDto.getCode())){
            return Rest.fail(I18nUtil.getMessage(FieldConstant.EMAIL_CODE_FAIL,request.getHeader("Accept-Language")));
        }
        //对密码进行加密
        String encryptPassword = SecurityUtils.encryptPassword(userDto.getPassword());
        user.setPassword(encryptPassword);
        //调用新增接口
        userService.updateById(user);
        return Rest.ok(I18nUtil.getMessage(FieldConstant.MESSAGES_SUCCESS,request.getHeader("Accept-Language")));
    }

    /**
     * 忘记密码功能
     * @param userDto 邮箱
     */
    @ApiOperation("修改密码功能")
    @PostMapping("send/editPwd")
    public Rest ChangePassword(@RequestBody ChangePasswordDto userDto, HttpServletRequest request){

        if (StringUtils.isEmpty(userDto.getEmail())){
            userDto.setEmail(SecurityContextHolder.getUserName());
        }
        //1.根据email查询当前登陆用户
        LambdaQueryWrapper<User> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(User::getUserEmail,userDto.getEmail());

        User user = userService.getBaseMapper().selectOne(lambdaQueryWrapper);

        //2.判断当前用户是否存在
        if (null == user){
            return Rest.fail(I18nUtil.getMessage(FieldConstant.EMAIL_LOGIN_EXIT,request.getHeader("Accept-Language")));
        }

        //2.判断旧密码是否匹配
        if (!SecurityUtils.matchesPassword(userDto.getOldPassword(),user.getPassword())){
            return Rest.fail(I18nUtil.getMessage(FieldConstant.OLD_PASSWORD_ERROR,request.getHeader("Accept-Language")));
        }

        //对新密码进行加密
        String encryptPassword = SecurityUtils.encryptPassword(userDto.getPassword());
        user.setPassword(encryptPassword);
        //调用新增接口
        userService.updateById(user);
        return Rest.ok(I18nUtil.getMessage(FieldConstant.MESSAGES_SUCCESS,request.getHeader("Accept-Language")));
    }




//    /**
//     * 查看个人资料 20241227注释掉
//     */
//    @ApiOperation("查看个人资料")
//    @GetMapping("send/query")
//    public Rest<UserVo> queryUsers(HttpServletRequest request, @ApiParam(name ="imei",value = "设备imei") String imei) {
//        return Rest.ok(userService.queryUser(SecurityContextHolder.getStringUserId(),imei));
//    }

    /**
     * 查看个人资料
     */
    @ApiOperation("查看个人资料")
    @GetMapping("send/query")
    public Rest<UserVo> queryUsers(HttpServletRequest request, @ApiParam(name ="imei",value = "设备imei") String imei) {
        return Rest.ok(userService.queryUser(SecurityContextHolder.getStringUserId(),imei));
    }




    /**
     * 修改账号信息 20241227注释
     */
//    @ApiOperation("修改账号信息")
//    @PostMapping("send/updateInfo")
//    public Rest<UserVo> updateInfo(MultipartFile file, UpdateUserDto user, HttpServletRequest request) throws IOException {
//        String userId= SecurityContextHolder.getStringUserId();
//        LambdaQueryWrapper<UserDeviceBindApp> lambdaQueryWrapper=new LambdaQueryWrapper<>();
//        lambdaQueryWrapper.eq(UserDeviceBindApp::getPhone,user.getPhone());
//        List<UserDeviceBindApp> userDeviceBindApp = userDeviceBindService.getBaseMapper().selectList(lambdaQueryWrapper);
//        if (!CollectionUtils.isEmpty(userDeviceBindApp) && !userId.equals(userDeviceBindApp.get(0).getUserId())){
//            return  Rest.fail(user.getPhone()+""+I18nUtil.getMessage(CommonConstant.PHONE_HAS_EXIT,request.getHeader("Accept-Language")));
//        }
//
//        if (null!=file){
//            System.out.println("头像文件不为空");
//            System.out.println("文件file:"+file.getOriginalFilename());
//            String path = UploadUtil.upload(file);
//            user.setImage(path);
//        }
//        userService.updateData(user);
//        return Rest.ok(userService.queryUser(SecurityContextHolder.getStringUserId(),user.getImei()));
//    }



    /**
     * 修改账号信息 20241227修改
     */
    @ApiOperation("修改账号信息")
    @PostMapping("send/updateInfo")
    public Rest<User> updateInfo(MultipartFile file, UpdateUserDto user, HttpServletRequest request) throws IOException {

        if (null!=file){
            System.out.println("头像文件不为空");
            System.out.println("文件file:"+file.getOriginalFilename());
            String path = UploadUtil.upload(file);
            user.setImage(path);
        }


        return    Rest.ok( userService.updateData(user));
    }








    /**
     * 退出登录
     */
    @ApiOperation("退出登录")
    @GetMapping("send/logout")
    public Rest logout(HttpServletRequest request) {
        //清除token返回
        String token = request.getHeader("Authorization");
        System.out.println("退出登录获取的token:"+token);
        String redisTokenKey = "app_tokens:"+ token;
        String redisUserKey = "user_keys:"+ SecurityContextHolder.getUserName();
        System.out.println("退出登录获取的user_key:"+redisUserKey);
        String redisGetUserIdKey = "getUserId:"+ SecurityContextHolder.getUserName();
        redisService.deleteObject(redisUserKey);
        redisService.deleteObject(redisTokenKey);
        System.out.println("删除后的获取的redisUserKey:"+redisService.getCacheObject(redisUserKey));
        System.out.println("退出登录获取的redisTokenKey:"+redisService.getCacheObject(redisTokenKey));
        redisService.deleteObject(redisGetUserIdKey);
        return Rest.ok(I18nUtil.getMessage(FieldConstant.MESSAGES_SUCCESS,request.getHeader("Accept-Language")));
    }
    @ApiOperation("注销用户")
    @GetMapping("user/cancel")
    public Rest cancel(HttpServletRequest request) {
        String userId=SecurityContextHolder.getStringUserId();
        //清除token返回
        String token = request.getHeader("Authorization");
        String redisTokenKey = "app_tokens:"+ token;
        String redisUserKey = "user_keys:"+ SecurityContextHolder.getUserName();
        String redisGetUserIdKey = "getUserId:"+ SecurityContextHolder.getUserName();
        //根据userId查询该用户绑定的所有设备
        LambdaQueryWrapper<UserDeviceBindApp> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(UserDeviceBindApp::getUserId,userId);
        List<UserDeviceBindApp> userDeviceBindAppList = userDeviceBindService.getBaseMapper().selectList(lambdaQueryWrapper);
        //没有绑定设备
        if (CollectionUtils.isEmpty(userDeviceBindAppList)){
            userService.getBaseMapper().deleteById(userId);
            redisService.deleteObject(redisUserKey);
            redisService.deleteObject(redisTokenKey);
            redisService.deleteObject(redisGetUserIdKey);
            return Rest.ok("");
        }
        for (UserDeviceBindApp userDeviceBindApp:userDeviceBindAppList){
            DeviceUnbindDto deviceUnbindDto = new DeviceUnbindDto();
            deviceUnbindDto.setImei(userDeviceBindApp.getImei());
            deviceUnbindDto.setUserId(userDeviceBindApp.getUserId());
            bindingDevice.DeviceUnbind(deviceUnbindDto);
        }
        userService.getBaseMapper().deleteById(userId);
        redisService.deleteObject(redisUserKey);
        redisService.deleteObject(redisTokenKey);
        redisService.deleteObject(redisGetUserIdKey);

        return Rest.ok("");
    }

    /**
     * 校验用户是否删除或者禁用
     */
    @ApiOperation("校验用户是否删除或者禁用")
    @GetMapping("send/checkFlag")
    public Rest checkFlag() {
        //1.根据email查询当前登陆用户
        LambdaQueryWrapper<User> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(User::getUserId,SecurityContextHolder.getStringUserId());
        User user = userService.getBaseMapper().selectOne(lambdaQueryWrapper);
        //2.判断当前用户是否存在
        if (null == user){
            return Rest.ok(true);
        }
        //3.判断当前用户是否被禁用
        if (FieldConstant.MATH_ONE.equals(user.getStatus())){
            return Rest.ok(true);
        }
        return Rest.ok(false);
    }
    /**
     * Android存入token
     */
    @ApiOperation("Android获取系统推送的token")
    @GetMapping("send/getSystemToken")
    public Rest getSystemToken(@ApiParam("手机厂商（HUAWEI、HONOR等）")String type,@ApiParam("存入的token") String token) {
        userService.getSystemToken(type,token);
        return Rest.ok("");
    }
}
