package com.ecell.internationalize.app.entity.dto;

import com.baomidou.mybatisplus.annotation.FieldFill;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.NotEmpty;
import java.io.Serializable;
import java.util.Date;

@Data
@EqualsAndHashCode(callSuper = false)
@Accessors(chain = true)
@ApiModel(value="DeviceInfo对象", description="设备信息表")
public class DeviceInfoDTO implements Serializable {
    private static final long serialVersionUID=1L;
    @TableId(value = "device_id",type = IdType.INPUT)
    @ApiModelProperty(value = "设备ID")
    private String deviceId;

    @ApiModelProperty(value = "IMEI")
    private String imei;

    @NotEmpty(message = "厂商id不能为空")
    @ApiModelProperty(value = "厂商id")
    private String firmId;

    @NotEmpty(message = "厂商名称不能为空")
    @ApiModelProperty(value = "厂商名称")
    private String firmName;

    @NotEmpty(message = "设备型号id不能为空")
    @ApiModelProperty(value = "设备型号id")
    private String deviceModelId;

    @NotEmpty(message = "设备型号名称不能为空")
    @ApiModelProperty(value = "设备型号名称")
    private String deviceModelName;

    @ApiModelProperty(value = "导入数量")
    private Integer totalImport;

    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @ApiModelProperty(value = "审核时间")
    private Date auditTime;

    @ApiModelProperty(value = "审核者")
    private String reviewer;

    @ApiModelProperty(value = "审核设备（1.单个录入这里存的是设备的IMEI，如果是批量录入这里存的是文件的唯一标识）")
    private String auditDevice;

    @ApiModelProperty(value = "录入标志（0.是单个录入，1.是批量录入")
    private String inputFlag;

    @ApiModelProperty(value = "审核状态（0.是拒绝，1.是待审核,2.是通过，默认为1）")
    private String auditStatus;

    @ApiModelProperty(value = "是否删除（0.是，1.否。默认为1）")
    private String delFlag;

    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @ApiModelProperty(value = "授权时间")
    private Date authorizaTime;

    @ApiModelProperty(value = "代理商id")
    private String agentId;

    @ApiModelProperty(value = "代理商名称")
    private String agentName;

    @ApiModelProperty(value = "授权标志（0.未授权，1.是已授权。默认为0）")
    private String authorizaFlag;

    @ApiModelProperty(value = "创建人")
    @TableField(value = "create_user",fill = FieldFill.INSERT)
    private String createUser;

    @ApiModelProperty(value = "修改人")
    @TableField(value = "update_user",fill = FieldFill.UPDATE)
    private String updateUser;

    @TableField(value = "create_time",fill = FieldFill.INSERT)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @ApiModelProperty(value = "创建时间")
    private Date createTime;

    @TableField(value = "update_time",fill = FieldFill.UPDATE)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @ApiModelProperty(value = "修改时间")
    private Date updateTime;

    @ApiModelProperty(value = "录入批次号")
    private  String inputBatch;

    @ApiModelProperty(value = "是否激活（0.未激活，1.是已激活。默认为0 这个字段没用）")
    private  String whetherActivation;

    @ApiModelProperty(value = "是否绑定（0.未绑定，1.是已绑定。默认为0 这个字段没用）")
    private  String whetherBinding;

    @ApiModelProperty(value = "设备首次登录时间")
    private Date firstLoginTime;

    @ApiModelProperty(value = "设备最后登录时间")
    private Date lastLoginTime;

}
