package com.ecell.internationalize.app.interceptor;

import com.alibaba.fastjson2.JSON;
import com.ecell.internationalize.common.core.constant.SecurityConstants;
import com.ecell.internationalize.common.core.context.SecurityContextHolder;
import com.ecell.internationalize.common.core.utils.JwtUtils;
import com.ecell.internationalize.common.core.utils.StringUtils;
import com.ecell.internationalize.common.core.web.domain.AjaxResult;
import com.ecell.internationalize.common.issue.constant.CommonConstant;
import com.ecell.internationalize.common.redis.service.RedisService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.servlet.AsyncHandlerInterceptor;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

@Component
public class    AppTokenCheck implements AsyncHandlerInterceptor {
    private static final Logger log = LoggerFactory.getLogger(AppTokenCheck.class);
    private final String TOKE_KEY="Authorization";
    private final String ACCEPT_LANGUAGE="Accept-Language";
    private final String LANGUAGE_ZH_CN="zh_CN";
    private final String TOKE_ISNULL_ZH="token不能为空";
    private final String TOKE_ISNULL_US="Token cannot be empty";
    private final String TOKE_IS_EXPIRED_ZH="token已过期";
    private final String TOKE_IS_EXPIRED_US="Token has expired";
    private final String IOS_DEVICE_TOKEN="deviceToken";
    private final String OS_TYPE="osType";
    private final String IOS="IOS";

    @Autowired
    private RedisService redisService;
    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        log.info("获取到的路径:{}",request.getRequestURI());
            String token = request.getHeader(TOKE_KEY);
            if (!StringUtils.isNotEmpty(token)){
               String message=LANGUAGE_ZH_CN.equalsIgnoreCase(request.getHeader(ACCEPT_LANGUAGE)) ? TOKE_ISNULL_ZH:TOKE_ISNULL_US;
                this.responseMessage(response,message,HttpServletResponse.SC_UNAUTHORIZED);
                return false;
            }else {
                String redis_token_key= "app_tokens:"+ token;
               if (!redisService.hasKey(redis_token_key)){
                   String message=LANGUAGE_ZH_CN.equalsIgnoreCase(request.getHeader(ACCEPT_LANGUAGE)) ? TOKE_IS_EXPIRED_ZH:TOKE_IS_EXPIRED_US;
                   this.responseMessage(response,message,HttpServletResponse.SC_UNAUTHORIZED);
                   return false;
               }

            }
            //解析token
            String userId = JwtUtils.parseToken(token).get(SecurityConstants.DETAILS_USER_ID, String.class);
            String userEmail = JwtUtils.parseToken(token).get(SecurityConstants.DETAILS_USERNAME, String.class);
            //放入本地线程中
            SecurityContextHolder.setUserName(userEmail);
            SecurityContextHolder.setUserId(userId);
            String osType = request.getHeader(OS_TYPE);
            if (StringUtils.isNotEmpty(osType) && IOS.equalsIgnoreCase(osType) ){
                String deviceToken = request.getHeader(IOS_DEVICE_TOKEN);
                log.info("获取到的类型:{},设备token:{}",osType,deviceToken);
                redisService.setCacheMapValue(userId, CommonConstant.DEVICE_TOKEN,deviceToken);
            }





        // TODO 把手机系统信息放入本地线程中(暂时不做)

            log.info("放入本地缓存的email:{},userId为:{}",SecurityContextHolder.getUserName(),SecurityContextHolder.getStringUserId());
            return true;


    }

    @Override
    public void afterCompletion(HttpServletRequest request, HttpServletResponse response, Object handler, Exception ex) throws Exception {
        SecurityContextHolder.remove();
    }



    private void  responseMessage(HttpServletResponse response,String message,int status){
        AjaxResult error = AjaxResult.error(status, message);
        String s = JSON.toJSONString(error);
        //设置请求头
        response.setContentType("application/json;charset=UTF-8");
        try {
            response.getWriter().print(s);
        } catch (IOException e) {
            log.info("写回信息错误:{}",e.getMessage());
            e.printStackTrace();
        }

    }
}
