package com.ecell.internationalize.app.kafka;

import lombok.extern.slf4j.Slf4j;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.clients.consumer.ConsumerRecords;
import org.apache.kafka.clients.consumer.KafkaConsumer;

import java.time.Duration;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;


/**
 * @Title: MyKafkaConsumer
 * @Author: liy
 * @Date: 2022/9/2 10:31
 * @Description:
 * @Version:1.0
 */
@Slf4j
public class MyKafkaConsumer{
    private static KafkaConsumer<String,String> consumer;
    /** 构造器注入值*/
    public MyKafkaConsumer() {
        consumer = MyKafkaConfig.inConsumerConfig();
    }
    /** 消费消息*/
    public static void runConsumer() {
        try {
            //获取配置文件
            consumer= MyKafkaConfig.inConsumerConfig();
            //使用线程池开启多线程
            ExecutorService executorService = new ThreadPoolExecutor(Runtime.getRuntime().availableProcessors(), Runtime.getRuntime().availableProcessors(), 0L, TimeUnit.MILLISECONDS,
                    new ArrayBlockingQueue<>(1000), new ThreadPoolExecutor.CallerRunsPolicy());

            while (true) {
                ConsumerRecords<String, String> records = consumer.poll(Duration.ofMillis(1000));
                if(!records.isEmpty()){
                    for(ConsumerRecord<String, String> record : records){
                        //提交线程任务
                        executorService.submit(new ConsumerTask(record));
                    }
                }
            }
        }catch (Exception e){
            log.error("kafka 消费者订阅 失败, 5秒后重新订阅", e);
            try {
                Thread.sleep(5000);
                runConsumer();
            } catch (Throwable ignore) {
                e.printStackTrace();
                log.error("kafka 消费者订阅 失败, 无法延时5秒", e);
            }
        }finally {
            consumer.close();
        }
    }
}
