package com.ecell.internationalize.app.service.impl;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ecell.internationalize.app.entity.AddressBookApp;
import com.ecell.internationalize.app.entity.NotifyApp;
import com.ecell.internationalize.app.entity.dto.BindingDeviceDto;
import com.ecell.internationalize.app.entity.dto.SaveAddressBookAppDto;
import com.ecell.internationalize.app.entity.dto.UpdateAddressBookAppDto;
import com.ecell.internationalize.app.entity.dto.UpdateSetAdminDto;
import com.ecell.internationalize.app.mapper.AddressBookAppMapper;
import com.ecell.internationalize.app.mapper.DeviceOwnerInfoAppMapper;
import com.ecell.internationalize.app.mapper.NotifyAppMapper;
import com.ecell.internationalize.app.mapper.UserDeviceBindAppMapper;
import com.ecell.internationalize.app.service.AddressBookAppService;
import com.ecell.internationalize.app.util.I18nUtil;
import com.ecell.internationalize.common.core.context.SecurityContextHolder;
import com.ecell.internationalize.common.core.utils.StringUtils;
import com.ecell.internationalize.common.core.utils.uuid.UUID;
import com.ecell.internationalize.common.issue.component.IssueInstructions;
import com.ecell.internationalize.common.issue.constant.CommonConstant;
import com.ecell.internationalize.common.issue.entity.issuedto.Contact;
import com.ecell.internationalize.common.issue.entity.issuedto.ContactInfo;
import com.ecell.internationalize.common.issue.mqtt.dto.IdentityTransfer;
import com.ecell.internationalize.common.issue.mqtt.dto.IdentityTransferChild;
import com.ecell.internationalize.common.issue.mqtt.push.MqttPush;
import com.ecell.internationalize.common.redis.service.RedisService;
import com.ecell.internationalize.common.system.entity.DeviceOwnerInfoApp;
import com.ecell.internationalize.common.system.entity.UserDeviceBindApp;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.*;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author ${author}
 * @since 2022-08-18
 */
@Service
@Slf4j
public class AddressBookAppServiceImpl extends ServiceImpl<AddressBookAppMapper, AddressBookApp> implements AddressBookAppService {
    private static final Logger logger = LoggerFactory.getLogger(AddressBookAppServiceImpl.class);
    @Autowired
    private AddressBookAppMapper addressBookAppMapper;
    @Autowired
    private UserDeviceBindAppMapper userDeviceBindAppMapper;
    @Autowired
    private IssueInstructions issueInstructions;
    @Value("${my.topic}")
    private String topic;
    @Autowired
    @Lazy
    private BindingDeviceImpl bindingDevice;
    @Autowired
    private MqttPush mqttPush;
    @Autowired
    private NotifyAppMapper notifyAppMapper;
    @Autowired
    private DeviceOwnerInfoAppMapper deviceOwnerInfoAppMapper;
    @Autowired
    private RedisService redisService;
    @Override
    public List<AddressBookApp> queryAddress(String imei) {
        //1.查询所有通讯录成员
        LambdaQueryWrapper<AddressBookApp> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(AddressBookApp::getImei,imei);
        List<AddressBookApp> addressBookAppList = addressBookAppMapper.selectList(lambdaQueryWrapper);
        if (!CollectionUtils.isEmpty(addressBookAppList)){
            for (AddressBookApp addressBookApp:addressBookAppList){
                if ("1".equals(addressBookApp.getSetBind())){
                    addressBookApp.setOpenId(addressBookApp.getUserId());
                }
            }
        }
        return addressBookAppList;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean insertData(List<SaveAddressBookAppDto> addressBookAppList) {
        List<AddressBookApp> addressBookAppList1=new ArrayList<>();
        //1.获取imei号
        String imei = addressBookAppList.get(0).getImei();
        //2.根据imei查询对应的通讯录好友
        LambdaQueryWrapper<AddressBookApp> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(AddressBookApp::getImei,imei);
        List<AddressBookApp> addressBookApps = addressBookAppMapper.selectList(lambdaQueryWrapper);
        //3.判断查询到的通讯录好友是否已存在手机号
        Map<String, AddressBookApp> addressBookAppMap = addressBookToMap(addressBookApps);
        for (SaveAddressBookAppDto addressBookApp:addressBookAppList){
            AddressBookApp bookApp=addressBookAppMap.get(addressBookApp.getPhone() + "_" + addressBookApp.getImei());
            //4.不为空则已经存在相同号码，进行覆盖
            if (null!=bookApp){
                addressBookAppMapper.deleteById(bookApp.getId());


            }


            //5.否则进行新增操作
            String id = UUID.randomUUID().toString(true);
            AddressBookApp app=new AddressBookApp();
            if (StringUtils.isNotNull(bookApp)){
                if (StringUtils.isNotEmpty(bookApp.getSetBind())){
                    app.setSetBind(bookApp.getSetBind());
                    if (CommonConstant.AUDIT_STATUS_ONE.equals(bookApp.getSetBind())){
                        app.setUserId(bookApp.getUserId());
                    }
                }else {
                    app.setSetBind("0");
                }

                app.setAddMethod(addressBookApp.getAddMethod());
                if (StringUtils.isNotEmpty(bookApp.getIdentity())){
                    app.setIdentity(bookApp.getIdentity());
                }else {
                    app.setIdentity("0");
                }

                if (StringUtils.isNotEmpty(bookApp.getCanVideo())){
                    app.setCanVideo(bookApp.getCanVideo());
                }else {
                    app.setCanVideo("0");
                }
            }else {
                app.setIdentity("0");
                app.setCanVideo("0");
                app.setSetBind("0");
            }
            app.setPhone(addressBookApp.getPhone());
            app.setImei(addressBookApp.getImei());
            app.setName(addressBookApp.getName());
            app.setRelation(addressBookApp.getRelation());


            app.setAddMethod(addressBookApp.getAddMethod());
            app.setImage(addressBookApp.getImage());
            app.setId(id);
            addressBookAppList1.add(app);
        }
        return this.saveBatch(addressBookAppList1);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public int updateData(UpdateAddressBookAppDto addressBookApp) {
        //1.根据imei号和userId修改用户设备绑定信息与孩子关系字段
        //根据Id查询用户账号
        AddressBookApp addressBookApp1=addressBookAppMapper.selectById(addressBookApp.getId());
        UserDeviceBindApp userDeviceBindApp=new UserDeviceBindApp();
        userDeviceBindApp.setRelation(addressBookApp.getRelation());
        userDeviceBindApp.setRelationImageId(addressBookApp.getImage());
        userDeviceBindApp.setPhone(addressBookApp.getPhone());
        LambdaQueryWrapper<UserDeviceBindApp> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(UserDeviceBindApp::getImei,addressBookApp.getImei());
        lambdaQueryWrapper.eq(UserDeviceBindApp::getUserId,null == addressBookApp1?null:addressBookApp1.getUserId());
        userDeviceBindAppMapper.update(userDeviceBindApp,lambdaQueryWrapper);

        AddressBookApp updateAddressBook=new AddressBookApp();
        updateAddressBook.setId(addressBookApp.getId());
        updateAddressBook.setRelation(addressBookApp.getRelation());
        updateAddressBook.setImage(addressBookApp.getImage());
        updateAddressBook.setName(addressBookApp.getName());
        updateAddressBook.setPhone(addressBookApp.getPhone());
        //2.修改通讯录信息
        addressBookAppMapper.updateById(updateAddressBook);
        //若为管理员，中心号码下发
        UserDeviceBindApp deviceBindApp = userDeviceBindAppMapper.selectOne(lambdaQueryWrapper);
        if (null!=deviceBindApp && "2".equals(deviceBindApp.getIdentity())){
            BindingDeviceDto bindingDeviceDto=new BindingDeviceDto();
            bindingDeviceDto.setMy_phone(deviceBindApp.getPhone());
            bindingDeviceDto.setImei(deviceBindApp.getImei());
            bindingDevice.assembCenterPhone(bindingDeviceDto,deviceBindApp.getUserId());
        }
       return 1;
    }

    @Override
    public void sendToKafka(String imei) {
        //1.根据imei查询所有通讯录成员
        LambdaQueryWrapper<AddressBookApp> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(AddressBookApp::getImei,imei);
        List<AddressBookApp> addressBookApps = addressBookAppMapper.selectList(lambdaQueryWrapper);
        //2.开始封装数据
        List<ContactInfo> list = new ArrayList<>();
        ContactInfo contactInfo;
        if (!CollectionUtils.isEmpty(addressBookApps)) {
            for (AddressBookApp addressBookApp : addressBookApps) {
                contactInfo = new ContactInfo();
                contactInfo.setId(addressBookApp.getId());
                contactInfo.setName(addressBookApp.getName());
                contactInfo.setPhone(addressBookApp.getPhone());
                contactInfo.setTime(System.currentTimeMillis());
                contactInfo.setUrl(addressBookApp.getImage());
                list.add(contactInfo);
            }
        }
        Contact contact = new Contact();
        contact.setContact(list);
        contact.setVender("20000");
        contact.setIdent("66666");
        contact.setImei(imei);
        contact.setOpenid(SecurityContextHolder.getStringUserId());
        contact.setType("contact");
        contact.setTime(System.currentTimeMillis());
        //3.下发指令之前先存入redis,开始下发指令
        issueInstructions.emitDeviceDemand("contact",imei,contact);
    }
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int updateSetAdmin(UpdateSetAdminDto addressBookApp, String lang) {
        LambdaQueryWrapper<DeviceOwnerInfoApp> ownerInfoAppLambdaQueryWrapper=new LambdaQueryWrapper<>();
        ownerInfoAppLambdaQueryWrapper.eq(DeviceOwnerInfoApp::getImei,addressBookApp.getImei());
        DeviceOwnerInfoApp deviceOwnerInfoApp = deviceOwnerInfoAppMapper.selectOne(ownerInfoAppLambdaQueryWrapper);
        String name=null!=deviceOwnerInfoApp?deviceOwnerInfoApp.getName():"宝贝";
        //根据Id查询用户账号
        AddressBookApp addressBookApp2=addressBookAppMapper.selectById(addressBookApp.getId());
        //1.用户设备绑定表和通讯录表管理员用户身份改为普通用户（1）,转让的用户改为管理员身份
        UserDeviceBindApp userDeviceBindApp=new UserDeviceBindApp();
        userDeviceBindApp.setIdentity(addressBookApp.getIdentity());
        //2.用户设备绑定表转让的用户改为管理员身份
        LambdaQueryWrapper<UserDeviceBindApp> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(UserDeviceBindApp::getImei,addressBookApp.getImei());
        lambdaQueryWrapper.eq(UserDeviceBindApp::getUserId,addressBookApp2.getUserId());
        userDeviceBindAppMapper.update(userDeviceBindApp,lambdaQueryWrapper);
        //3.用户设备绑定表管理员身份改为普通用户
        UserDeviceBindApp userDeviceBindApp1=new UserDeviceBindApp();
        userDeviceBindApp1.setIdentity("1");
        LambdaQueryWrapper<UserDeviceBindApp> lambdaQueryWrapper1 = new LambdaQueryWrapper<>();
        lambdaQueryWrapper1.eq(UserDeviceBindApp::getImei,addressBookApp.getImei());
        lambdaQueryWrapper1.eq(UserDeviceBindApp::getUserId,SecurityContextHolder.getStringUserId());
        userDeviceBindAppMapper.update(userDeviceBindApp1,lambdaQueryWrapper1);
        //4.管理员修改通讯录信息身份为普通用户
        AddressBookApp addressBookApp1=new AddressBookApp();
        addressBookApp1.setIdentity("1");
        LambdaQueryWrapper<AddressBookApp> lambdaQueryWrapper2 = new LambdaQueryWrapper<>();
        lambdaQueryWrapper2.eq(AddressBookApp::getImei,addressBookApp.getImei());
        lambdaQueryWrapper2.eq(AddressBookApp::getUserId,SecurityContextHolder.getStringUserId());
        addressBookAppMapper.update(addressBookApp1,lambdaQueryWrapper2);
        //5.普通用户修改为管理员身份
        AddressBookApp addressBookApp3=new AddressBookApp();
        addressBookApp3.setId(addressBookApp.getId());
        addressBookApp3.setIdentity(addressBookApp.getIdentity());
        addressBookAppMapper.updateById(addressBookApp3);
        //若为管理员，中心号码下发
        UserDeviceBindApp deviceBindApp = userDeviceBindAppMapper.selectOne(lambdaQueryWrapper);
        if (null!=deviceBindApp && "2".equals(deviceBindApp.getIdentity())){
            BindingDeviceDto bindingDeviceDto=new BindingDeviceDto();
            bindingDeviceDto.setMy_phone(deviceBindApp.getPhone());
            bindingDeviceDto.setImei(deviceBindApp.getImei());
            bindingDevice.assembCenterPhone(bindingDeviceDto,deviceBindApp.getUserId());
        }
        //主动转让管理员发送到mqtt
        AddressBookApp adminAddress = addressBookAppMapper.selectOne(lambdaQueryWrapper2);
        IdentityTransfer identityTransfer=new IdentityTransfer();
        identityTransfer.setType("accountMessage");
        IdentityTransferChild identityTransferChild=new IdentityTransferChild();
        identityTransferChild.setImei(addressBookApp.getImei());
        identityTransferChild.setType("identityTransfer");
        identityTransferChild.setName(adminAddress.getName());
        identityTransferChild.setPhone(addressBookApp2.getPhone());
        identityTransferChild.setTimestamp(System.currentTimeMillis());
        identityTransfer.setMessage(identityTransferChild);
        String pushMsgData= JSONObject.toJSONString(identityTransfer);
        //String lang = redisService.getCacheMapValue(adminAddress.getUserId(), CommonConstant.IOS_LANGUAGE);
        String agent=redisService.getCacheMapValue(adminAddress.getUserId(), CommonConstant.USER_AGENT);
        String alert= I18nUtil.getMessage(CommonConstant.PRE_IDENTITY_TRANSFER,lang)+"("+addressBookApp.getImei()+")"+I18nUtil.getMessage(CommonConstant.AFTER_IDENTITY_TRANSFER,lang)+addressBookApp2.getName()+"("+addressBookApp2.getPhone()+")";
        //判断是发IOS还是安卓
        identityTransferChild.setBody(alert);
        if (CommonConstant.ANDROID.equals(agent)){
            mqttPush.sendMqtt("3",adminAddress.getUserId(),"1","accountMessage","identityTransfer",pushMsgData);
        }
        if (CommonConstant.IOS.equals(agent)) {
            String deviceToken = redisService.getCacheMapValue(adminAddress.getUserId(), CommonConstant.DEVICE_TOKEN);
            logger.info("主动转让管理员deviceToken:"+deviceToken);
            bindingDevice.sendIOSLogin(adminAddress.getUserId(), identityTransfer, "accountMessage", "identityTransfer", alert,deviceToken);
            logger.info("苹果系统用户：{}, 转让管理员, 推送成功,pushMsgData:{}",adminAddress.getUserId(),identityTransfer);
        }
//        else {
//            bindingDevice.sendIOS(adminAddress.getUserId(), identityTransfer, "accountMessage", "identityTransfered", alert);
//        }
        logger.info("转让管理员,主动转让用户信息推送成功,pushMsgData:{}",pushMsgData);
        //管理员主动转让入通知表
        NotifyApp notifyApp=new NotifyApp();
        notifyApp.setId(com.ecell.internationalize.common.core.utils.uuid.UUID.fastUUID().toString(true));
        notifyApp.setImei(addressBookApp.getImei());
        notifyApp.setCreateTime(new Date());
        notifyApp.setMessageRead("0");
        notifyApp.setName(name);
        notifyApp.setPhone(addressBookApp2.getPhone());
        notifyApp.setType("identityTransfer");
        notifyApp.setUserId(adminAddress.getUserId());
        notifyAppMapper.insert(notifyApp);


        //被动接收管理员发送到mqtt
        IdentityTransfer commonIdentityTransfer=new IdentityTransfer();
        commonIdentityTransfer.setType("accountMessage");
        IdentityTransferChild commonIdentityTransferChild=new IdentityTransferChild();
        commonIdentityTransferChild.setImei(addressBookApp.getImei());
        commonIdentityTransferChild.setType("identityTransfered");
        commonIdentityTransferChild.setName(addressBookApp2.getName());
        commonIdentityTransferChild.setPhone(adminAddress.getPhone());
        commonIdentityTransferChild.setTimestamp(System.currentTimeMillis());
        commonIdentityTransfer.setMessage(commonIdentityTransferChild);
        String pushMsgData2= JSONObject.toJSONString(commonIdentityTransfer);

        //String lang2 = redisService.getCacheMapValue(addressBookApp2.getUserId(), CommonConstant.IOS_LANGUAGE);
        String agent2=redisService.getCacheMapValue(addressBookApp2.getUserId(), CommonConstant.USER_AGENT);
        String alert2=name+"("+addressBookApp.getImei()+")"+I18nUtil.getMessage(CommonConstant.ACCEPT_IDENTITY_TRANSFER,lang);
        commonIdentityTransferChild.setBody(alert2);
        //判断是发IOS还是安卓
        if (CommonConstant.ANDROID.equals(agent2)){
            mqttPush.sendMqtt("3",addressBookApp2.getUserId(),"1","accountMessage","identityTransfered",pushMsgData2);
        }
        if (CommonConstant.IOS.equals(agent2)) {
            String deviceToken = redisService.getCacheMapValue(addressBookApp2.getUserId(), CommonConstant.DEVICE_TOKEN);
            logger.info("被动接收成为管理员deviceToken:"+deviceToken);
            bindingDevice.sendIOSLogin(addressBookApp2.getUserId(), commonIdentityTransfer, "accountMessage", "identityTransfered", alert2,deviceToken);
            logger.info("苹果系统用户：{}, 转让管理员, 推送成功,pushMsgData:{}",addressBookApp2.getUserId(),identityTransfer);
        }
//        else {
//            bindingDevice.sendIOS(addressBookApp2.getUserId(), commonIdentityTransfer, "accountMessage", "identityTransfer", alert2);
//        }

        logger.info("转让管理员,被动接收管理员信息推送成功,pushMsgData:{}",pushMsgData2);

        //成员被动接收管理员入通知表
        NotifyApp notifyApp2=new NotifyApp();
        notifyApp2.setId(com.ecell.internationalize.common.core.utils.uuid.UUID.fastUUID().toString(true));
        notifyApp2.setImei(addressBookApp.getImei());
        notifyApp2.setCreateTime(new Date());
        notifyApp2.setMessageRead("0");
        notifyApp2.setName(name);
        notifyApp2.setPhone(adminAddress.getPhone());
        notifyApp2.setType("identityTransfered");
        notifyApp2.setUserId(addressBookApp2.getUserId());
        notifyAppMapper.insert(notifyApp2);
        return 1;
    }

    public Map<String,AddressBookApp> addressBookToMap(List<AddressBookApp> addressBookAppList){
        Map<String,AddressBookApp> map=new HashMap<>(16);
        if (!CollectionUtils.isEmpty(addressBookAppList)) {
            for (AddressBookApp addressBookApp : addressBookAppList) {
                map.put(addressBookApp.getPhone() + "_" + addressBookApp.getImei(), addressBookApp);
            }
        }
        return map;
    }

    /**
     * 根据imei,canVideo,查询可视频列表
     * @return
     */
    public List<AddressBookApp> getListByImei(String imei,String canVideo){
        QueryWrapper wrapper =new QueryWrapper();
        wrapper.eq("imei",imei);
        wrapper.eq("canVideo",canVideo);
       return baseMapper.selectList(wrapper);

    }

}
