package com.ecell.internationalize.app.service.impl;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.ecell.internationalize.app.entity.*;
import com.ecell.internationalize.app.entity.dto.ApplyFollowDto;
import com.ecell.internationalize.app.entity.dto.ApprovalDto;
import com.ecell.internationalize.app.entity.dto.BindingDeviceDto;
import com.ecell.internationalize.app.entity.dto.DeviceUnbindDto;
import com.ecell.internationalize.app.entity.po.VideoModelInfoPO;
import com.ecell.internationalize.app.feign.DeviceInfoFeign;
import com.ecell.internationalize.app.mapper.ChatGroupMessageAppMapper;
import com.ecell.internationalize.app.mapper.UserMapper;
import com.ecell.internationalize.app.service.*;
import com.ecell.internationalize.app.util.I18nUtil;
import com.ecell.internationalize.app.util.LocationUtil;
import com.ecell.internationalize.common.core.context.SecurityContextHolder;
import com.ecell.internationalize.common.core.exception.ServiceException;
import com.ecell.internationalize.common.core.utils.SpringUtils;
import com.ecell.internationalize.common.core.utils.StringUtils;
import com.ecell.internationalize.common.core.utils.bean.BeanUtils;
import com.ecell.internationalize.common.core.utils.uuid.UUID;
import com.ecell.internationalize.common.issue.component.IssueInstructions;
import com.ecell.internationalize.common.issue.constant.CommonConstant;
import com.ecell.internationalize.common.issue.entity.issuedto.CenterPhoneSetting;
import com.ecell.internationalize.common.issue.entity.issuedto.ChatGroupDisbandMessage;
import com.ecell.internationalize.common.issue.entity.issuedto.Contact;
import com.ecell.internationalize.common.issue.entity.issuedto.ContactInfo;
import com.ecell.internationalize.common.issue.entity.tcp.DistributeParent;
import com.ecell.internationalize.common.issue.mqtt.dto.*;
import com.ecell.internationalize.common.issue.mqtt.push.IOSPush;
import com.ecell.internationalize.common.issue.mqtt.push.MqttPush;
import com.ecell.internationalize.common.issue.util.RandomUtil;
import com.ecell.internationalize.common.redis.service.RedisService;
import com.ecell.internationalize.common.system.entity.DeviceOwnerInfoApp;
import com.ecell.internationalize.common.system.entity.PositioningModelInfo;
import com.ecell.internationalize.common.system.entity.User;
import com.ecell.internationalize.common.system.entity.UserDeviceBindApp;
import org.eclipse.paho.client.mqttv3.MqttClient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.security.SecureRandom;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class BindingDeviceImpl implements BindingDevice {
    private static final Logger log = LoggerFactory.getLogger(BindingDeviceImpl.class);
    @Autowired
    private UserMapper userMapper;
    @Autowired
    @Lazy
    private BindingDeviceImpl bindingDevice;
    @Autowired
    private DeviceInfoFeign deviceInfoFeign;
    @Autowired
    private DeviceOwnerInfoAppService deviceOwnerInfoAppService;
    @Autowired
    private UserDeviceBindService userDeviceBindService;
    @Autowired
    private AddressBookAppService addressBookAppService;
    @Autowired
    private ChantGroupAppService chantGroupAppService;
    @Autowired
    private ChatGroupMemberAppService chatGroupMemberAppService;
    @Autowired
    private RedisService redisService;
    @Autowired
    private IssueInstructions issueInstructions;
    @Autowired
    private VideoCallAppServiceImpl videoCallAppService;
    @Autowired
    private DeviceVideoAppService deviceVideoAppService;
    @Autowired
    private DeviceAttentionAppService deviceAttentionAppService;
    @Autowired
    private MqttPush mqttPush;
    @Autowired
    private ChantGroupAppServiceImpl chantGroupAppServiceImlp;
    @Autowired
    private ChatGroupMessageAppServiceImpl chatGroupMessageAppService;
    @Autowired
    private LatestLocationAppService latestLocationAppService;
    @Autowired
    private DeviceLocationAppService deviceLocationAppService;
    @Autowired
    private DeviceAlarmAppService deviceAlarmAppService;
    @Autowired
    private DeviceDisableAppService deviceDisableAppService;
    @Autowired
    private DeviceTimeSwitchAppService deviceTimeSwitchAppService;
    @Autowired
    private SafetyRailAppService safetyRailAppService;
    @Autowired
    private ImeiSosInfoService imeiSosInfoService;
    @Autowired
    private ReminderMessageAppService reminderMessageAppService;
    @Autowired
    private TrackInfoAppService trackInfoAppService;
    @Autowired
    private DeviceVideoRecordAppService deviceVideoRecordAppService;
    @Autowired
    private IOSPush iosPush;
    @Autowired
    private DeviceInfoService deviceInfoService;

    @Autowired
    private DeviceModelInfoService deviceModelInfoService;

    /**
     * 根据设备imei查询该imei是否已入库
     */
    @Override
    public Map<String, Object> queryDeviceByImei(String imei) {
        //获取email
        String email = SecurityContextHolder.getUserName();
        String userId = SecurityContextHolder.getStringUserId();
        Map<String, Object> map = new HashMap<>(8);
//        User user = this.isInputPhoneByEmail(email);
//        if (!StringUtils.isNotEmpty(user.getPhone())){
//            throw new ServiceException("请先绑定手机号");
//        }

        QueryWrapper<DeviceInfo> wrapper = new QueryWrapper<>();
        wrapper.eq("imei", imei);
        wrapper.eq("audit_status", CommonConstant.AUDIT_STATUS_TWO);
        wrapper.eq("del_flag", CommonConstant.AUDIT_STATUS_ONE);
        DeviceInfo deviceInfoPO = deviceInfoService.getBaseMapper().selectOne(wrapper);




        //根据imei查询后台录入设备的信息
//        DeviceInfoPO deviceInfoPO = deviceInfoFeign.queryDeviceInfoByImei(imei);
        log.info("查询到的设备信息:{}", deviceInfoPO);
        if (StringUtils.isNull(deviceInfoPO)) {
            //  throw new ServiceException("此imei在后台系统中不存在请联系管理员");
            throw new ServiceException("messagesImeiIsNull");
        }
        //自动审批需要
        DeviceModelInfo deviceModelInfo = deviceModelInfoService.getBaseMapper().selectById(deviceInfoPO.getDeviceModelId());

        String deviceType = videoCallAppService.queryDeviceModelById(deviceInfoPO.getDeviceModelId());
        if (StringUtils.isNotNull(deviceInfoPO)) {
            //判断该imei是否审批过
            if (!CommonConstant.AUDIT_STATUS_TWO.equals(deviceInfoPO.getAuditStatus())) {
                // throw new ServiceException("此imei在后台系统中未通过请联系管理员");
                throw new ServiceException("messagesImeiNotAudit");
            }
        }
        //判断该用户绑定的数量是否超过限制（最大5个）
        if (this.queryBindingDeviceNumber(email) > CommonConstant.NUMBER_FIVE) {
            //throw new ServiceException("您已经添加申请了6个设备不能在添加其它设备");
            throw new ServiceException("messagesMaximumLimitExceeded");
        }
        //根据用户id查询+imei 查询绑定表
        UserDeviceBindApp bingInfoByUserId = getBingInfoByUserId(userId, imei);
        if (StringUtils.isNotNull(bingInfoByUserId)) {
            map.put(CommonConstant.BINDING_STATUS, CommonConstant.AUDIT_STATUS_TWO); //说明已经绑定过了
            return map;
        }

        //根据imei号查询设备与用户绑定表管理员信息
        UserDeviceBindApp userDeviceBindApp = this.queryAdminByImei(imei);
        //给APP做隐藏使用
        map.put("deviceModelName", deviceInfoPO.getDeviceModelName());
        if (StringUtils.isNotNull(userDeviceBindApp)) {//此设备已经绑定过，就把管理员的手机号,跟邮箱，还有绑定状态返回前端  -->这里如果设备有管理员就会直接去申请关注
            map.put(CommonConstant.BINDING_STATUS, CommonConstant.AUDIT_STATUS_ONE);
            //如果是自动审批就把状态改为3
            if (StringUtils.isNotNull(deviceModelInfo) && CommonConstant.AUDIT_STATUS_ONE.equals(deviceModelInfo.getWhetherApply())){
                map.put(CommonConstant.BINDING_STATUS, CommonConstant.AUDIT_STATUS_THREE);
            }
            map.put(CommonConstant.DEVICE_TYPE, deviceType);
            map.put(CommonConstant.ADMIN_EMAIL, userDeviceBindApp.getUserEmail());
            map.put("adminUserId", userDeviceBindApp.getUserId());
            //根据email查询用户信息
            //User userTwo = this.queryUserByEmail(userDeviceBindApp.getUserEmail());
            map.put(CommonConstant.ADMIN_PHONE, userDeviceBindApp.getPhone());
            return map;
        } else {

            map.put(CommonConstant.BINDING_STATUS, CommonConstant.AUDIT_STATUS_ZERO);
            map.put(CommonConstant.DEVICE_TYPE, deviceType);

            return map;
        }

    }

    @Override
    public User isInputPhoneByEmail(String email) {
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("user_email", email);
        return userMapper.selectOne(wrapper);

    }


    /**
     * 绑定新增设备
     *
     * @param dto
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    //这里只有管理员才能进这个方法
    public Map<String, String> addDevice(BindingDeviceDto dto, String path) {
        Map<String, String> map = new HashMap<>(5);
        //获取用户ID
        String userId = SecurityContextHolder.getStringUserId();
        //获取邮箱
        String email = SecurityContextHolder.getUserName();
        if (!StringUtils.isNotEmpty(dto.getPhone())) {
            // throw new ServiceException("设备手机号不能为空");
            throw new ServiceException("messagesDevicePhone");
        }
        log.info("当前登录人手机号码:{}", dto.getMy_phone());
        UserDeviceBindApp userDeviceBindApp1 = queryByPhone(userId, dto.getMy_phone());

        if (StringUtils.isNotNull(userDeviceBindApp1)) {
            throw new ServiceException("messagesIsPhoneBound");
        }

        //判断该用户绑定的数量是否超过限制（最大6个）
        if (this.queryBindingDeviceNumber(email) > CommonConstant.NUMBER_FIVE) {
            //throw new ServiceException("您已经添加申请了6个设备不能在添加其它设备");
            throw new ServiceException("messagesMaximumLimitExceeded");
        }
        //根据imei+email 查询用户绑定表看这个用户是否绑定过
        UserDeviceBindApp bindApp1 = this.queryIsBindingDevice(email, dto.getImei());
        //说明已经绑定过
        if (StringUtils.isNotNull(bindApp1)) {
            if (CommonConstant.AUDIT_STATUS_ZERO.equals(bindApp1.getApproveStatus())) {
                // throw new ServiceException("您已经绑定过该设备管理员正在审批中请联系管理员审批");
                throw new ServiceException("messagesBindingDeviceError");
            } else {
                //  throw new ServiceException("您已经绑定过该设备");
                throw new ServiceException("messagesBindingDeviceIsNotNull");

            }

        }
        //根据imei查询定位模式信息
        PositioningModelInfo positioningModelInfo = queryPositioningModelInfo(dto.getImei());


        //根据imei号查询设备与用户绑定表管理员信息
        UserDeviceBindApp userDeviceBindApp = this.queryAdminByImei(dto.getImei());
        //        if (StringUtils.isNotNull(userDeviceBindApp) && CommonConstant.AUDIT_STATUS_TWO.equals(dto.getIdentity())){//此设备已经绑定过，并且APP端传过来的身份也是管理员,(说明这里有可能会出现问题，所以这里做了判断)就把管理员的手机号,跟邮箱，还有绑定状态返回前端
//            map.put(CommonConstant.BINDING_STATUS,CommonConstant.AUDIT_STATUS_ONE);
//            map.put(CommonConstant.ADMIN_EMAIL,userDeviceBindApp.getUserEmail());
//            //根据email查询用户信息
//           // User userTwo = this.queryUserByEmail(userDeviceBindApp.getUserEmail());
//            map.put(CommonConstant.ADMIN_PHONE,userDeviceBindApp.getPhone());
//            return map;
        // }else {
        if (StringUtils.isNull(userDeviceBindApp)) {
            String groupId = com.ecell.internationalize.common.core.utils.uuid.UUID.fastUUID().toString(true);
            //申请关注信息表
            // MessageRecordApp recordApp =new MessageRecordApp();
            //先查询聊天群组表有没有记录
            ChantGroupApp groupApp = chantGroupAppService.getBaseMapper().selectOne(new QueryWrapper<ChantGroupApp>().eq("imei", dto.getImei()));
            if (StringUtils.isNotNull(groupApp)) {
                groupId = groupApp.getId();
            }
            //添加设备信息
            DeviceOwnerInfoApp app = new DeviceOwnerInfoApp();
            BeanUtils.copyBeanProp(app, dto);
            app.setImage(path);
            app.setId(com.ecell.internationalize.common.core.utils.uuid.UUID.fastUUID().toString(true));
            app.setCreateUser(email);
            app.setPositioningModelId(positioningModelInfo.getPositioningModeId());
            //默认是第一个定位模型
            int pInfo = positioningModelInfo.getPositioningOneInterva() == 0 ? 10 : positioningModelInfo.getPositioningOneInterva();
            app.setPositioningInterval(pInfo);
            deviceOwnerInfoAppService.save(app);
            //============================================================
            //初始化聊天群组表 只有管理员才添加到聊天群组表中 申请关注的人要去审核通过才能添加聊天群组表
            ChantGroupApp chatApp = new ChantGroupApp();
            chatApp.setId(groupId);
            chatApp.setImei(dto.getImei());
            // chatApp.setEmail(email);
            chatApp.setUserId(userId);
            chantGroupAppService.save(chatApp);

            //通讯录表初始化 只有管理员才添加到通讯录中 申请关注的人要去审核通过才能添加通讯录中
            AddressBookApp bookApp = new AddressBookApp();
            bookApp.setId(com.ecell.internationalize.common.core.utils.uuid.UUID.fastUUID().toString(true));
            bookApp.setImei(dto.getImei());
            // bookApp.setEmail(email);
            bookApp.setUserId(userId);
            bookApp.setImage(dto.getRelationshipId());
            bookApp.setName(dto.getRelationship());
            bookApp.setPhone(dto.getMy_phone());
            bookApp.setCanVideo(CommonConstant.AUDIT_STATUS_ONE);
            //已绑定通讯录那里要用
            bookApp.setSetBind(CommonConstant.AUDIT_STATUS_ONE);
            //与孩子的关系通讯录那里要用
            bookApp.setRelation(dto.getRelationship());
            bookApp.setIdentity(CommonConstant.AUDIT_STATUS_TWO);
            //根据imei和绑定手机号码查询数据库，有就先删除在新增
            bindingDevice.queryBookByImeiAndPhone(bookApp);
            //初始化聊天群组成员表
            ChatGroupMemberApp memberApp = new ChatGroupMemberApp();
            memberApp.setId(com.ecell.internationalize.common.core.utils.uuid.UUID.fastUUID().toString(true));
            // memberApp.setEmail(email);
            memberApp.setUserId(userId);
            memberApp.setGroupId(groupId);
            chatGroupMemberAppService.save(memberApp);
            //初始化可视频列表 //只有绑定了手表的人才能在可视频列表中
//            VideoCallApp videoCallApp = new VideoCallApp();
//            videoCallApp.setUserId(userId);
//            videoCallApp.setImei(dto.getImei());
//            //根据imei查询视频型号跟视频类型
//            VideoModelInfoPO videoByImei = getVideoByImei(dto.getImei());
//            if (StringUtils.isNotNull(videoByImei)) {
//                //视频类型（1.佰锐视频，2.菊风视频，3.鹈鹕视频）
//                videoCallApp.setVideoType(videoByImei.getVideoModelType());
//                videoCallApp.setVideoTypeId(videoByImei.getVideoModelId());
//            }else {
//                videoCallApp.setVideoType(CommonConstant.AUDIT_STATUS_ZERO);
//            }
            DeviceVideoApp videoApp = getVideoApp(dto.getImei(), userId);
            //一个用户多个多个设备相同的视频类型只能存相同的视频id
            LambdaQueryWrapper<DeviceVideoApp> lambdaQueryWrapper = new LambdaQueryWrapper<>();
            lambdaQueryWrapper.eq(DeviceVideoApp::getUserId, userId);
            lambdaQueryWrapper.eq(DeviceVideoApp::getVideoType, videoApp.getVideoType());
            List<DeviceVideoApp> deviceVideoApp = deviceVideoAppService.getBaseMapper().selectList(lambdaQueryWrapper);
            videoApp.setId(com.ecell.internationalize.common.core.utils.uuid.UUID.fastUUID().toString(true));
            String time = String.valueOf(System.currentTimeMillis());
            String s = time.substring(8, 13);
            SecureRandom secureRandom = new SecureRandom();
            int i = secureRandom.nextInt(10000);
            String total = i + s;
            videoApp.setVideoId(Integer.parseInt(total));
            //已经存在视频类型相同的多个设备
            if (!CollectionUtils.isEmpty(deviceVideoApp)) {
                videoApp.setVideoId(deviceVideoApp.get(0).getVideoId());
            }
            deviceVideoAppService.save(videoApp);

            //初始化用户设备绑定表
            UserDeviceBindApp bindApp = new UserDeviceBindApp();
            bindApp.setId(com.ecell.internationalize.common.core.utils.uuid.UUID.fastUUID().toString(true));
            //设置邮箱
            bindApp.setUserEmail(email);
            //设置用户id
            bindApp.setUserId(userId);
            //设置设备imei
            bindApp.setImei(dto.getImei());
            //与设备拥有者的昵称
            bindApp.setRelation(dto.getRelationship());
            //设置头像
            bindApp.setRelationImageId(dto.getRelationshipId());
            //设置手机号码
            bindApp.setPhone(dto.getMy_phone());
            //设置成已绑定
            bindApp.setBindStatus(CommonConstant.AUDIT_STATUS_ONE);
            //============================================================
            //通讯录表初始化
//            AddressBookApp bookApp =new AddressBookApp();
//            bookApp.setId(UUID.fastUUID().toString(true));
//            bookApp.setImei(dto.getImei());
//            bookApp.setEmail(dto.getEmail());
//            bookApp.setImage(dto.getUserImage());
//            bookApp.setName(dto.getNickname());
//            bookApp.setPhone(dto.getUserPhone());
//            bookApp.setCanVideo(CommonConstant.AUDIT_STATUS_ONE);
//            bookApp.setSetBind(CommonConstant.AUDIT_STATUS_ONE); //已绑定通讯录那里要用
//            bindApp.setRelation(dto.getNickname()); //与孩子的关系通讯录那里要用

            //============================================================
//            //初始化聊天群组表
//            ChantGroupApp chatApp =new ChantGroupApp();
//            String groupId= UUID.fastUUID().toString(true);
//            chatApp.setId(groupId);
//            chatApp.setImei(dto.getImei());
            //============================================================
            //初始化聊天群组成员表
//            ChatGroupMemberApp memberApp =new ChatGroupMemberApp();
//            memberApp.setId(UUID.fastUUID().toString(true));
//            memberApp.setEmail(dto.getEmail());
//            memberApp.setGroupId(groupId);
            //============================================================

             //更新用户信息表，把手机号码也更新 20241227新增
            UserService userService = SpringUtils.getBean(UserService.class);
            User user = userService.getBaseMapper().selectById(userId);
            if (StringUtils.isNotNull(user)){
                if (StringUtils.isNotNull(user.getPhone())){
                    if (!user.getPhone().equals(dto.getMy_phone())){
                        user.setPhone(dto.getMy_phone());

                    }
                }else {
                    user.setPhone(dto.getMy_phone());
                }
                userService.getBaseMapper().updateById(user);

            }

            //============================================================

            //设置成已审核
            bindApp.setApproveStatus(CommonConstant.AUDIT_STATUS_ONE);
            //设置成管理员
            bindApp.setIdentity(CommonConstant.AUDIT_STATUS_TWO);

            userDeviceBindService.save(bindApp);
            // TODO 这里添加到redis中不知道为什么 根据python代码来
            redisService.setCacheMapValue(dto.getImei(), "related_status", "1");
//            //把设备名称放入缓存
//            redisService.setCacheMapValue(dto.getImei(),"deviceName",dto.getName());
//            log.info("添加关联设备成功");

            // # 获取一下redis 中是否有这个设备标识头, 如果有的话证明是三基协议的
            //  log.info("协议:{}",redisService.getCacheMapValue(dto.getImei(), "deviceHead").toString());
            //  if (StringUtils.isEmpty(redisService.getCacheMapValue(dto.getImei(), "deviceHead").toString())) {
            //如果是管理员就下发中心号码设置
            assembCenterPhone(dto, userId);
            //管理员通讯下发
            assembleContactParam(dto, userId);
            //可视频通话用户列表下发
            //  assembDownGetVideoUserList(dto,userId);
            Object onlineStatus = redisService.getCacheMapValue(dto.getImei(), "onlineStatus");
            log.info("第一次绑定时设备imei:{},如果在线onlineStatus:{} 就主动定位一下", dto.getImei(), onlineStatus);
            //如果设备在线就主动定位一下
            if (CommonConstant.AUDIT_STATUS_ONE.equals(onlineStatus)) {
                LocationUtil.sendLocation(dto.getImei(), userId);
                // issueInstructions.emitDeviceDemand("requestLocation",dto.getImei(),null);
//                        String requestLocation =String.format("requestLocation:%s:%s",dto.getImei(),userId);
//                        //设置缓存即可
//                        redisService.setCacheObject(requestLocation,userId, 60L, TimeUnit.SECONDS);
            }

            //   }

            map.put(CommonConstant.MESSAGE_CODE, "200");
            map.put("msg", "添加设备成功");
            return map;
            // }
        } else {
            //  throw new ServiceException("此设备已经绑定过请联系管理员");
            throw new ServiceException("messagesBindingDeviceIsNotNull");
        }

    }


    /**
     * 查询用户是否使用其余用户的手机号码进行绑定设备
     *
     * @param
     * @return
     */
    public UserDeviceBindApp queryByPhone(String userId, String phone) {
        QueryWrapper<UserDeviceBindApp> wrapper = new QueryWrapper();
        wrapper.eq("phone", phone);
        wrapper.ne("user_id", userId);
        wrapper.last("LIMIT 1");
        return userDeviceBindService.getBaseMapper().selectOne(wrapper);

    }




//    /**
//     * 查询同一imei 用户手机号码是否被其他用户绑定
//     *
//     * @param
//     * @return
//     */
//    private UserDeviceBindApp queryByPhoneAndImei(String imei, String phone) {
//        QueryWrapper<UserDeviceBindApp> wrapper = new QueryWrapper();
//        wrapper.eq("phone", phone);
//        wrapper.ne("imei", imei);
//        wrapper.last("LIMIT 1");
//        return userDeviceBindService.getBaseMapper().selectOne(wrapper);
//
//    }


    /**
     * 设备绑定成功之后推送消息给管理员(申请关注)
     *
     * @param applyFollowDto
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean SendPushToAccount(ApplyFollowDto applyFollowDto) {
        //获取用户ID
        String userId = SecurityContextHolder.getStringUserId();
        if (!StringUtils.isNotEmpty(applyFollowDto.getPhone())) {
            // throw new ServiceException("请先绑定手机号");
            throw new ServiceException("messagesIsPhoneNull");
        }
        // 查询用户是否使用其余绑定该设备用户的手机号码进行绑定设备
        UserDeviceBindApp userDeviceBindApp = queryByPhone(userId, applyFollowDto.getPhone());
        if (StringUtils.isNotNull(userDeviceBindApp)) {
            throw new ServiceException("messagesIsPhoneBound");
        }


        //判断该用户绑定的数量是否超过限制（最大6个）
        if (this.queryBindingDeviceNumber(applyFollowDto.getImei()) > CommonConstant.NUMBER_FIVE) {
            // throw new ServiceException("您已经添加申请了6个设备不能在添加其它设备");
            throw new ServiceException("messagesMaximumLimitExceeded");
        }
        boolean flag = true;
        String UserId = SecurityContextHolder.getStringUserId();
        if (UserId.equals(applyFollowDto.getAdminUserId())) {
            // throw new ServiceException("该用户已是管理员不能申请关注"); messages.apply.follow.error
            throw new ServiceException("messagesApplyFollowError");
        }
        DeviceModelInfo deviceModelInfo = getDeviceModelInfo(applyFollowDto.getImei());
        if (StringUtils.isNull(deviceModelInfo)) {
            throw new ServiceException("messagesApplyDeviceModelError");
        }
        //根据Imei查询设备表
        DeviceOwnerInfoApp deviceOwnerInfoApp = getDeviceOwnerInfoApp(applyFollowDto.getImei());
        //  UserDeviceBindApp bingDeviceInfo = getBingDeviceInfo(applyFollowDto.getAdminUserId());
        if (StringUtils.isNotNull(deviceOwnerInfoApp)) {
            //自定关注成功
            if (CommonConstant.AUDIT_STATUS_ONE.equals(deviceModelInfo.getWhetherApply())) {
                UserDeviceBindApp userDeviceBindApp1 = getUserDeviceBindApp(applyFollowDto.getImei(), UserId);
                if (StringUtils.isNotNull(userDeviceBindApp1)) {
                    throw new ServiceException("messagesApplyBindError");
                }

                //自动同意审批
                List<DeviceAttentionApp> list = new ArrayList<>();
                //审批通过 绑定表，通讯录表，聊天群组表， 新增一条数据,修改记录表的状态
                //初始化用户设备绑定表
                UserDeviceBindApp bindApp = new UserDeviceBindApp();
                bindApp.setId(com.ecell.internationalize.common.core.utils.uuid.UUID.fastUUID().toString(true));
                //设置邮箱
                //  bindApp.setUserEmail(applyFollowDto.getEmail());
                //设置用户id
                bindApp.setUserId(userId);
                //设置设备imei
                bindApp.setImei(applyFollowDto.getImei());
                //与设备拥有者的昵称
                bindApp.setRelation(applyFollowDto.getRelationship());
                //设置头像
                bindApp.setRelationImageId(applyFollowDto.getRelationshipImageId());
                //设置手机号码
                bindApp.setPhone(applyFollowDto.getPhone());
                //设置成已绑定
                bindApp.setBindStatus(CommonConstant.AUDIT_STATUS_ONE);
                //设置成已审核
                bindApp.setApproveStatus(CommonConstant.AUDIT_STATUS_ONE);
                //设置成申请关注的人
                bindApp.setIdentity(CommonConstant.AUDIT_STATUS_ONE);
                //查询用户表获取改申请人的email
                User user = userMapper.selectById(userId);
                if (!StringUtils.isNotNull(user)) {
                    throw new ServiceException("messagesUserDoesNotExist");
                }
                bindApp.setUserEmail(user.getUserEmail());
                //======================================================//
                //通讯录表初始化 只有管理员才添加到通讯录中 申请关注的人要去审核通过才能添加通讯录中
                AddressBookApp bookApp = new AddressBookApp();
                bookApp.setId(com.ecell.internationalize.common.core.utils.uuid.UUID.fastUUID().toString(true));
                bookApp.setImei(applyFollowDto.getImei());
                // bookApp.setEmail(user.getUserEmail());
                bookApp.setUserId(userId);
                bookApp.setImage(applyFollowDto.getRelationshipImageId());
                bookApp.setName(applyFollowDto.getRelationship());
                bookApp.setPhone(applyFollowDto.getPhone());
                bookApp.setCanVideo(CommonConstant.AUDIT_STATUS_ONE);
                //已绑定通讯录那里要用
                bookApp.setSetBind(CommonConstant.AUDIT_STATUS_ONE);
                //与孩子的关系通讯录那里要用
                bookApp.setRelation(applyFollowDto.getRelationship());
                bookApp.setIdentity(CommonConstant.AUDIT_STATUS_ONE);
                //根据imei和绑定手机号码查询数据库，有就先删除在新增
                bindingDevice.queryBookByImeiAndPhone(bookApp);
                //======================================================//
                //初始化聊天群组成员表
                ChatGroupMemberApp memberApp = new ChatGroupMemberApp();
                //根据查询聊天群组表
                ChantGroupApp chatGroupMember = getChatGroupMember(applyFollowDto.getImei());

                if (!StringUtils.isNotNull(chatGroupMember)) {
                    //暂无群组信息
                    throw new ServiceException("messagesGroupInfo");
                }
                memberApp.setId(com.ecell.internationalize.common.core.utils.uuid.UUID.fastUUID().toString(true));
                //  memberApp.setEmail(user.getUserEmail());
                memberApp.setUserId(userId);
                memberApp.setGroupId(chatGroupMember.getId());
                //======================================================//
                //新增可视视频列表
                DeviceVideoApp videoApp = getVideoApp(applyFollowDto.getImei(), userId);
                //一个用户多个多个设备相同的视频类型只能存相同的视频id
                LambdaQueryWrapper<DeviceVideoApp> lambdaQueryWrapper = new LambdaQueryWrapper<>();
                lambdaQueryWrapper.eq(DeviceVideoApp::getUserId, userId);
                lambdaQueryWrapper.eq(DeviceVideoApp::getVideoType, videoApp.getVideoType());
                List<DeviceVideoApp> deviceVideoApp = deviceVideoAppService.getBaseMapper().selectList(lambdaQueryWrapper);
                videoApp.setId(com.ecell.internationalize.common.core.utils.uuid.UUID.fastUUID().toString(true));
                String time = String.valueOf(System.currentTimeMillis());
                String s = time.substring(8, 13);
                SecureRandom secureRandom = new SecureRandom();
                int i = secureRandom.nextInt(10000);
                String total = i + s;
                videoApp.setVideoId(Integer.parseInt(total));
                //已经存在视频类型相同的多个设备
                if (!CollectionUtils.isEmpty(deviceVideoApp)) {
                    videoApp.setVideoId(deviceVideoApp.get(0).getVideoId());
                }
                deviceVideoAppService.save(videoApp);

                flag = userDeviceBindService.save(bindApp);
                flag = chatGroupMemberAppService.save(memberApp);
                if (flag) {
                    //下发通讯
                    addressBookAppService.sendToKafka(applyFollowDto.getImei());
                }
//                //下发管理员及其他成员加入群组
//              bindingDevice.sendAutomaticApprovalToApp(applyFollowDto,userId);


            } else {

                //先查询关注记录表
                DeviceAttentionApp attentionApp = getAttentionApp(applyFollowDto.getAdminUserId(), UserId, applyFollowDto.getImei());
                if (StringUtils.isNotNull(attentionApp)) {
                    //  throw  new ServiceException("你已经申请绑定了请耐心等待手表管理员审核");
                    throw new ServiceException("messagesApplyError");
                }
                List<DeviceAttentionApp> apps = new ArrayList<>(2);
                //这里要初始化两条申请关注记录一条是申请人的，一条是管理员的
                for (int i = 0; i < 2; i++) {
                    DeviceAttentionApp deviceAttentionApp = new DeviceAttentionApp();
                    deviceAttentionApp.setImei(applyFollowDto.getImei());
                    deviceAttentionApp.setId(com.ecell.internationalize.common.core.utils.uuid.UUID.fastUUID().toString(true));
                    deviceAttentionApp.setApplyId(UserId);
                    deviceAttentionApp.setAdminId(applyFollowDto.getAdminUserId());
                    deviceAttentionApp.setPhone(applyFollowDto.getPhone());
                    deviceAttentionApp.setName(deviceOwnerInfoApp.getName());
                    deviceAttentionApp.setRelationship(applyFollowDto.getRelationship());
                    deviceAttentionApp.setRelationshipImageId(applyFollowDto.getRelationshipImageId());
                    //审批中
                    deviceAttentionApp.setApprovalStatus(CommonConstant.AUDIT_STATUS_ZERO);
                    deviceAttentionApp.setApplyPhone(applyFollowDto.getPhone());
                    deviceAttentionApp.setMessagesType("applicationRrecord");
                    if (i == 0) {
                        //这里改成已读
                        deviceAttentionApp.setApplyRead(CommonConstant.AUDIT_STATUS_ONE);
                        deviceAttentionApp.setUserId(UserId);
                    } else {
                        //这里改成已读
                        deviceAttentionApp.setApplyRead(CommonConstant.AUDIT_STATUS_ZERO);
                        deviceAttentionApp.setUserId(applyFollowDto.getAdminUserId());
                    }
                    apps.add(deviceAttentionApp);

                }

                flag = deviceAttentionAppService.saveBatch(apps);

                //发送MQTT到管理员
                sendAttentionApp(applyFollowDto, deviceOwnerInfoApp.getName());

            }

        } else {
            //  throw new ServiceException("此设备还没有绑定过请先绑定在申请关注");
            throw new ServiceException("messagesBindingNotError");
        }

        return flag;

    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean AutomaticBinding(ApplyFollowDto applyFollowDto) {
        //获取用户ID
        String userId = SecurityContextHolder.getStringUserId();
        if (!StringUtils.isNotEmpty(applyFollowDto.getPhone())) {
            // throw new ServiceException("请先绑定手机号");
            throw new ServiceException("messagesIsPhoneNull");
        }
        // 查询用户是否使用其余绑定该设备用户的手机号码进行绑定设备
        UserDeviceBindApp userDeviceBindApp = queryByPhone(userId, applyFollowDto.getPhone());
        if (StringUtils.isNotNull(userDeviceBindApp)) {
            throw new ServiceException("messagesIsPhoneBound");
        }
        //判断该用户绑定的数量是否超过限制（最大6个）
        if (this.queryBindingDeviceNumber(applyFollowDto.getImei()) > CommonConstant.NUMBER_FIVE) {
            // throw new ServiceException("您已经添加申请了6个设备不能在添加其它设备");
            throw new ServiceException("messagesMaximumLimitExceeded");
        }
        boolean flag = true;
        String UserId = SecurityContextHolder.getStringUserId();
        if (UserId.equals(applyFollowDto.getAdminUserId())) {
            // throw new ServiceException("该用户已是管理员不能申请关注"); messages.apply.follow.error
            throw new ServiceException("messagesApplyFollowError");
        }


        //自动同意审批
        List<DeviceAttentionApp> list = new ArrayList<>();
        //审批通过 绑定表，通讯录表，聊天群组表， 新增一条数据,修改记录表的状态
        //初始化用户设备绑定表
        UserDeviceBindApp bindApp = new UserDeviceBindApp();
        bindApp.setId(com.ecell.internationalize.common.core.utils.uuid.UUID.fastUUID().toString(true));
        //设置邮箱
        //  bindApp.setUserEmail(applyFollowDto.getEmail());
        //设置用户id
        bindApp.setUserId(userId);
        //设置设备imei
        bindApp.setImei(applyFollowDto.getImei());
        //与设备拥有者的昵称
        bindApp.setRelation(applyFollowDto.getRelationship());
        //设置头像
        bindApp.setRelationImageId(applyFollowDto.getRelationshipImageId());
        //设置手机号码
        bindApp.setPhone(applyFollowDto.getPhone());
        //设置成已绑定
        bindApp.setBindStatus(CommonConstant.AUDIT_STATUS_ONE);
        //设置成已审核
        bindApp.setApproveStatus(CommonConstant.AUDIT_STATUS_ONE);
        //设置成申请关注的人
        bindApp.setIdentity(CommonConstant.AUDIT_STATUS_ONE);
        //查询用户表获取改申请人的email
        User user = userMapper.selectById(userId);
        if (!StringUtils.isNotNull(user)) {
            throw new ServiceException("messagesUserDoesNotExist");
        }
        bindApp.setUserEmail(user.getUserEmail());
        //======================================================//
        //通讯录表初始化 只有管理员才添加到通讯录中 申请关注的人要去审核通过才能添加通讯录中
        AddressBookApp bookApp = new AddressBookApp();
        bookApp.setId(UUID.fastUUID().toString(true));
        bookApp.setImei(applyFollowDto.getImei());
        // bookApp.setEmail(user.getUserEmail());
        bookApp.setUserId(userId);
        bookApp.setImage(applyFollowDto.getRelationshipImageId());
        bookApp.setName(applyFollowDto.getRelationship());
        bookApp.setPhone(applyFollowDto.getPhone());
        bookApp.setCanVideo(CommonConstant.AUDIT_STATUS_ONE);
        //已绑定通讯录那里要用
        bookApp.setSetBind(CommonConstant.AUDIT_STATUS_ONE);
        //与孩子的关系通讯录那里要用
        bookApp.setRelation(applyFollowDto.getRelationship());
        bookApp.setIdentity(CommonConstant.AUDIT_STATUS_ONE);
        //根据imei和绑定手机号码查询数据库，有就先删除在新增
        bindingDevice.queryBookByImeiAndPhone(bookApp);
        //======================================================//
        //初始化聊天群组成员表
        ChatGroupMemberApp memberApp = new ChatGroupMemberApp();
        //根据查询聊天群组表
        ChantGroupApp chatGroupMember = getChatGroupMember(applyFollowDto.getImei());

        if (!StringUtils.isNotNull(chatGroupMember)) {
            //暂无群组信息
            throw new ServiceException("messagesGroupInfo");
        }
        memberApp.setId(com.ecell.internationalize.common.core.utils.uuid.UUID.fastUUID().toString(true));
        //  memberApp.setEmail(user.getUserEmail());
        memberApp.setUserId(userId);
        memberApp.setGroupId(chatGroupMember.getId());
        //======================================================//
        //新增可视视频列表
        DeviceVideoApp videoApp = getVideoApp(applyFollowDto.getImei(), userId);
        //一个用户多个多个设备相同的视频类型只能存相同的视频id
        LambdaQueryWrapper<DeviceVideoApp> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(DeviceVideoApp::getUserId, userId);
        lambdaQueryWrapper.eq(DeviceVideoApp::getVideoType, videoApp.getVideoType());
        List<DeviceVideoApp> deviceVideoApp = deviceVideoAppService.getBaseMapper().selectList(lambdaQueryWrapper);
        videoApp.setId(com.ecell.internationalize.common.core.utils.uuid.UUID.fastUUID().toString(true));
        String time = String.valueOf(System.currentTimeMillis());
        String s = time.substring(8, 13);
        SecureRandom secureRandom = new SecureRandom();
        int i = secureRandom.nextInt(10000);
        String total = i + s;
        videoApp.setVideoId(Integer.parseInt(total));
        //已经存在视频类型相同的多个设备
        if (!CollectionUtils.isEmpty(deviceVideoApp)) {
            videoApp.setVideoId(deviceVideoApp.get(0).getVideoId());
        }
        deviceVideoAppService.save(videoApp);

        flag = userDeviceBindService.save(bindApp);
        flag = chatGroupMemberAppService.save(memberApp);
        if (flag) {
            //下发通讯
            addressBookAppService.sendToKafka(applyFollowDto.getImei());
        }

        return flag;
    }

    /**
     * 管理员审批通过申请关注
     *
     * @param dto
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean approvalFollowStatus(ApprovalDto dto) {
        if (!StringUtils.isNotEmpty(dto.getAdminId())) {
            throw new ServiceException("messagesAdminIdNotNull");
        }
        if (!StringUtils.isNotEmpty(dto.getUserId())) {
            throw new ServiceException("messagesUserIdNotNull");
        }
        boolean flag = true;

        //根据申请记录ID查询申请信息
        DeviceAttentionApp deviceAttention = getDeviceAttention(dto.getId(), dto.getUserId(), dto.getImei(), dto.getAdminId());
        log.info("获取到管理员的申请记录是:{}", deviceAttention);
        //根据userId查询申请人记录
        DeviceAttentionApp deviceAttention1 = getDeviceAttention(null, dto.getUserId(), dto.getImei(), dto.getAdminId());
        log.info("获取到申请人申请记录是:{}", deviceAttention1);
        if (!(StringUtils.isNotNull(deviceAttention) && StringUtils.isNotNull(deviceAttention1))) {
            //"申请关注记录不存在"
            throw new ServiceException("messagesAttentionNotExist");
        }
        //审批通过
        if (CommonConstant.AUDIT_STATUS_ONE.equals(dto.getStatus())) {
            List<DeviceAttentionApp> list = new ArrayList<>();
            //审批通过 绑定表，通讯录表，聊天群组表， 新增一条数据,修改记录表的状态
            //初始化用户设备绑定表
            UserDeviceBindApp bindApp = new UserDeviceBindApp();
            bindApp.setId(com.ecell.internationalize.common.core.utils.uuid.UUID.fastUUID().toString(true));
            //设置邮箱
            bindApp.setUserEmail(dto.getEmail());
            //设置用户id
            bindApp.setUserId(dto.getUserId());
            //设置设备imei
            bindApp.setImei(dto.getImei());
            //与设备拥有者的昵称
            bindApp.setRelation(dto.getRelationship());
            //设置头像
            bindApp.setRelationImageId(dto.getRelationshipImageId());
            //设置手机号码
            bindApp.setPhone(dto.getPhone());
            //设置成已绑定
            bindApp.setBindStatus(CommonConstant.AUDIT_STATUS_ONE);
            //设置成已审核
            bindApp.setApproveStatus(CommonConstant.AUDIT_STATUS_ONE);
            //设置成申请关注的人
            bindApp.setIdentity(CommonConstant.AUDIT_STATUS_ONE);
            //查询用户表获取改申请人的email
            User user = userMapper.selectById(dto.getUserId());
            if (!StringUtils.isNotNull(user)) {
                throw new ServiceException("messagesUserDoesNotExist");
            }
            /******************20241227新增***********************/
            if (StringUtils.isNotNull(user.getPhone())){
                if (!user.getPhone().equals(dto.getPhone())){
                    //更新用户表的手机号码
                    user.setPhone(dto.getPhone());
                }else {
                    user.setPhone(dto.getPhone());
                }
                userMapper.updateById(user);
            }
            /******************20241227新增***********************/
            bindApp.setUserEmail(user.getUserEmail());
            //======================================================//
            //通讯录表初始化 只有管理员才添加到通讯录中 申请关注的人要去审核通过才能添加通讯录中
            AddressBookApp bookApp = new AddressBookApp();
            bookApp.setId(com.ecell.internationalize.common.core.utils.uuid.UUID.fastUUID().toString(true));
            bookApp.setImei(dto.getImei());
            // bookApp.setEmail(user.getUserEmail());
            bookApp.setUserId(dto.getUserId());
            bookApp.setImage(dto.getRelationshipImageId());
            bookApp.setName(dto.getRelationship());
            bookApp.setPhone(dto.getPhone());
            bookApp.setCanVideo(CommonConstant.AUDIT_STATUS_ONE);
            //已绑定通讯录那里要用
            bookApp.setSetBind(CommonConstant.AUDIT_STATUS_ONE);
            //与孩子的关系通讯录那里要用
            bookApp.setRelation(dto.getRelationship());
            bookApp.setIdentity(CommonConstant.AUDIT_STATUS_ONE);
            //根据imei和绑定手机号码查询数据库，有就先删除在新增
            bindingDevice.queryBookByImeiAndPhone(bookApp);
            //======================================================//
            //初始化聊天群组成员表
            ChatGroupMemberApp memberApp = new ChatGroupMemberApp();
            //根据查询聊天群组表
            ChantGroupApp chatGroupMember = getChatGroupMember(dto.getImei());

            if (!StringUtils.isNotNull(chatGroupMember)) {
                //暂无群组信息
                throw new ServiceException("messagesGroupInfo");
            }
            memberApp.setId(com.ecell.internationalize.common.core.utils.uuid.UUID.fastUUID().toString(true));
            //  memberApp.setEmail(user.getUserEmail());
            memberApp.setUserId(dto.getUserId());
            memberApp.setGroupId(chatGroupMember.getId());
            //======================================================//
            //修改申请记录的信息
            if (StringUtils.isNotNull(deviceAttention) && StringUtils.isNotNull(deviceAttention1)) {
                deviceAttention.setApprovalStatus(CommonConstant.AUDIT_STATUS_ONE);
                deviceAttention.setApplyRead(CommonConstant.AUDIT_STATUS_ONE);

                deviceAttention1.setApprovalStatus(CommonConstant.AUDIT_STATUS_ONE);
                deviceAttention1.setApplyRead(CommonConstant.AUDIT_STATUS_ZERO);
                list.add(deviceAttention);
                list.add(deviceAttention1);
                deviceAttentionAppService.updateBatchById(list);
            }
            //新增可视视频列表
            DeviceVideoApp videoApp = getVideoApp(dto.getImei(), dto.getUserId());
            //一个用户多个多个设备相同的视频类型只能存相同的视频id
            LambdaQueryWrapper<DeviceVideoApp> lambdaQueryWrapper = new LambdaQueryWrapper<>();
            lambdaQueryWrapper.eq(DeviceVideoApp::getUserId, dto.getUserId());
            lambdaQueryWrapper.eq(DeviceVideoApp::getVideoType, videoApp.getVideoType());
            List<DeviceVideoApp> deviceVideoApp = deviceVideoAppService.getBaseMapper().selectList(lambdaQueryWrapper);
            videoApp.setId(com.ecell.internationalize.common.core.utils.uuid.UUID.fastUUID().toString(true));
            String time = String.valueOf(System.currentTimeMillis());
            String s = time.substring(8, 13);
            SecureRandom secureRandom = new SecureRandom();
            int i = secureRandom.nextInt(10000);
            String total = i + s;
            videoApp.setVideoId(Integer.parseInt(total));
            //已经存在视频类型相同的多个设备
            if (!CollectionUtils.isEmpty(deviceVideoApp)) {
                videoApp.setVideoId(deviceVideoApp.get(0).getVideoId());
            }
            deviceVideoAppService.save(videoApp);

            flag = userDeviceBindService.save(bindApp);
            flag = chatGroupMemberAppService.save(memberApp);





            if (flag) {
                //下发通讯
                addressBookAppService.sendToKafka(dto.getImei());
            }

        } else { //拒绝
            List<DeviceAttentionApp> list = new ArrayList<>();
            //修改申请记录的信息
            if (StringUtils.isNotNull(deviceAttention) && StringUtils.isNotNull(deviceAttention1)) {
                deviceAttention.setApprovalStatus(CommonConstant.AUDIT_STATUS_TWO);
                deviceAttention.setApplyRead(CommonConstant.AUDIT_STATUS_ONE);

                deviceAttention1.setApprovalStatus(CommonConstant.AUDIT_STATUS_TWO);
                deviceAttention1.setApplyRead(CommonConstant.AUDIT_STATUS_ZERO);

                list.add(deviceAttention);
                list.add(deviceAttention1);

                deviceAttentionAppService.updateBatchById(list);
            }
        }

        //通过，或者失败都推送给申请人
        sendAttentionToApp(dto);
        return flag;
    }


    /**
     * 解除设备绑定
     *
     * @param dto
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean DeviceUnbind(DeviceUnbindDto dto) {
        String userId = SecurityContextHolder.getStringUserId();
        String imei = dto.getImei();

        if (StringUtils.isNotEmpty(dto.getUserId())) {
            userId = dto.getUserId();
        }
        //根据imei和用户id查询绑定表是否是管理员
        UserDeviceBindApp bingInfoByUser = getBingInfoByUserId(userId, imei);
        if (!StringUtils.isNotNull(bingInfoByUser)) {
            throw new ServiceException("messagesUnbindingError");
        }
        //如果是管理员解绑
        if (CommonConstant.AUDIT_STATUS_TWO.equals(bingInfoByUser.getIdentity())) {
            //删除设备拥有者信息表
            DeviceOwnerInfoApp deviceOwnerInfoApp = bindingDevice.deleteDeviceOwnerInfo(imei);
            //=========================//
            //删除绑定表数据
            bindingDevice.deleteUserBindInfo(imei, null);
            //==============================//
            //删除群组成员表
            ChantGroupApp chatGroupMember = getChatGroupMember(imei);
            //删除群组表
            bindingDevice.deleteChatGroup(imei);
            String groupId = null;
            if (StringUtils.isNotNull(chatGroupMember)) {
                groupId = chatGroupMember.getId();

            }

            List<ChatGroupMemberApp> list = bindingDevice.deleteGroupMember(null, groupId);
            //==============//
            //删除聊天信息表
            bindingDevice.deleteMessage(null, groupId);


            //=================//
            //删除通讯录
            bindingDevice.deleteAddressBook(null, imei);
            //删除申请关注记录表
            bindingDevice.deleteDeviceAttention(imei, null);
            //删除最新位置信息表
            bindingDevice.deleteLatestLocation(imei);
            //删除解析后的位置信息表
            bindingDevice.deleteDeviceLocation(imei);
            //删除闹钟信息表
            bindingDevice.deleteAlarm(imei);
            //删除上课免打扰信息表device_disable_app
            bindingDevice.deleteDeviceDisable(imei);
            //删除定时开关机信息表
            bindingDevice.deleteDeviceTimeSwitch(imei);
            //删除安全围栏信息表
            bindingDevice.deleteSafetyRail(imei);
            //删除SOS信息表
            bindingDevice.deleteSOSInfo(imei);
            //删除提醒信息表记录
            bindingDevice.deleteReminderMessage(imei, null);
            //删除可视视频信息
            bindingDevice.deleteVideoInfo(imei, null);
            //删除轨迹信息
            bindingDevice.deleteTrackInfo(imei);
            //删除通话记录
            deleteVideoRecordInfo(imei, null);
            log.info("管理员解绑删除结束======");
            //恢复出厂设置指令下发
            bindingDevice.sendDeviceInfo(imei, userId);
            //解绑发送信息给APP
            bindingDevice.sendUnbindInfoToApp(userId, list, deviceOwnerInfoApp.getName(), imei);
        } else {//非管理员解绑
            //删除绑定表的数据
            List<UserDeviceBindApp> userDeviceBindApps = bindingDevice.deleteUserBindInfo(imei, userId);
            //  userDeviceBindService.getBaseMapper().deleteById(bingInfoByUser.getId());
            //删除通讯录数据
            //  bindingDevice.deleteBook(userId,imei);
            //根据设备imei查询群组ID
            ChantGroupApp chatGroupMember = getChatGroupMember(imei);
            String groupId = chatGroupMember.getId();
            //删除群员群组表
            bindingDevice.deleteGroupMember(userId, groupId);
            //删除聊天信息记录
            bindingDevice.deleteMessage(userId, groupId);
            //删除通讯录表
            bindingDevice.deleteAddressBook(userId, imei);
            //删除申请关注记录表
            bindingDevice.deleteDeviceAttention(imei, userId);
            //删除提醒信息表记录
            bindingDevice.deleteReminderMessage(imei, userId);
            //删除可视视频信息
            bindingDevice.deleteVideoInfo(imei, userId);
            //删除通话记录
            deleteVideoRecordInfo(imei, userId);
            //下发通讯录
            bindingDevice.sendToDevice(imei, userId);
            //这里是申请关注的人主动解绑才通过mqtt发送信息到管理员,如果是删除通讯录就不会下发这个
            if (!StringUtils.isNotEmpty(dto.getUserId())) {
                bindingDevice.sendUnbindInfoToAdminApp(userId, imei, userDeviceBindApps);
            }

        }
        log.info("解绑成功:userID{},imei{},identity{}", userId, imei, bingInfoByUser.getIdentity());

        return true;
    }

    /**
     * 恢复出厂设置指令下发
     *
     * @param imei
     */
    public void sendDeviceInfo(String imei, String userId) {
        //组装参数
        DistributeParent distributeParent = new DistributeParent();
        distributeParent.setTime(System.currentTimeMillis());
        distributeParent.setOpenid(userId);
        distributeParent.setType("reset");
        distributeParent.setVender("20000");
        distributeParent.setIdent(RandomUtil.generateVerCode(6));
        distributeParent.setImei(imei);
        issueInstructions.emitDeviceDemand("reset", imei, distributeParent);
    }

    /**
     * 申请关注的人解绑发送信息到管理员（APP）
     */
    public void sendUnbindInfoToAdminApp(String userId, String imei, List<UserDeviceBindApp> userDeviceBindApps) {
        //根据imei号查询设备拥有者信息
        DeviceOwnerInfoApp deviceOwnerInfoApp = deviceOwnerInfoAppService.queryOwnerInfoByimei(imei);
        String adminId = "";
        String name = "";
        String phone = "";
        String deviceName = "";
        if (StringUtils.isNotNull(deviceOwnerInfoApp)) {
            deviceName = deviceOwnerInfoApp.getName();
        }
//         List<UserDeviceBindApp> userIdList = getUserIdList(imei);
        //查询管理员信息
        UserDeviceBindApp adminUser = getAdminUser(imei);
        if (StringUtils.isNotNull(adminUser)) {
            userDeviceBindApps.add(adminUser);
        }
        if (StringUtils.isNotEmpty(userDeviceBindApps)) {
            for (UserDeviceBindApp bindApp : userDeviceBindApps) {
                //管理员
                if (CommonConstant.AUDIT_STATUS_TWO.equals(bindApp.getIdentity())) {
                    adminId = bindApp.getUserId();
                }
                if (userId.equals(bindApp.getUserId())) {
                    name = bindApp.getRelation();
                    phone = bindApp.getPhone();
                }

            }

        }
        ChatGroupRemoveMessage message = new ChatGroupRemoveMessage();
        message.setDevice_name(deviceName);
        message.setAccount_name(name);
        message.setCount(CommonConstant.NUMBER_ONE);
        message.setRemoveTime(System.currentTimeMillis());
        message.setName(name);
        message.setImei(imei);
        message.setType("chatGroupRemove");
        message.setPhone(phone);
        String s = JSON.toJSONString(message);
        //根据管理员ID查询手机型号
        String phoneType = redisService.getCacheMapValue(adminId, CommonConstant.USER_AGENT);
        if (CommonConstant.ANDROID.equals(phoneType)) {
            //发送消息
            mqttPush.sendMqtt(CommonConstant.AUDIT_STATUS_THREE, adminId, CommonConstant.AUDIT_STATUS_ONE, "chatGroupRemove", "chatGroupRemove", s);

        } else if (CommonConstant.IOS.equals(phoneType)) {
//            //根据管理员Id获取语言信息
//            String language = redisService.getCacheMapValue(userId, CommonConstant.IOS_LANGUAGE);
//            String alert;
//            //中文
//        if (CommonConstant.CN.contains(language)){
//
//             alert = String.format("【%s】的%s取消关注手表", deviceName, name);
//            //英文
//        }else {
//            alert = String.format("%s的%sCancelWatching", deviceName, name);
//        }
            String lang = redisService.getCacheMapValue(adminId, CommonConstant.IOS_LANGUAGE);
            String alert = String.format(I18nUtil.getMessage("messages.ios.chatGroupRemove", lang), deviceName, name);
            sendIOS(adminId, message, "chatGroupRemove", "chatGroupRemove", alert);

        }

    }

    /**
     * 删除设备拥有者信息表(管理员删除)
     *
     * @param imei
     */
    public DeviceOwnerInfoApp deleteDeviceOwnerInfo(String imei) {
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("imei", imei);
        BaseMapper<DeviceOwnerInfoApp> baseMapper = deviceOwnerInfoAppService.getBaseMapper();
        DeviceOwnerInfoApp app = baseMapper.selectOne(wrapper);
        if (!StringUtils.isNotNull(app)) {
            //暂无设备拥有者信息
            throw new ServiceException("messagesDeviceOwnerInfo");
        }
        baseMapper.deleteById(app.getId());
        return app;
    }

    /**
     * 删除绑定表数据(管理员删除)
     *
     * @param imei
     */
    public List<UserDeviceBindApp> deleteUserBindInfo(String imei, String userId) {
        QueryWrapper<UserDeviceBindApp> wrapper = new QueryWrapper();
        //单个删除
        if (StringUtils.isNotEmpty(userId)) {
            wrapper.eq("user_id", userId);
        }
        wrapper.eq("imei", imei);
        BaseMapper<UserDeviceBindApp> baseMapper = userDeviceBindService.getBaseMapper();
        //先查询
        List<UserDeviceBindApp> list = baseMapper.selectList(wrapper);
        if (StringUtils.isNotEmpty(list) && list.size() > CommonConstant.NUMBER_ZERO) {
            List<String> collect = list.parallelStream().map(UserDeviceBindApp::getId).collect(Collectors.toList());
            //删除
            baseMapper.deleteBatchIds(collect);
        }
        return list;
    }

    /**
     * 删除绑定表数据(管理员删除)
     *
     * @param imei
     */
    public void deleteChatGroup(String imei) {
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("imei", imei);
        chantGroupAppService.getBaseMapper().delete(wrapper);
    }

    /**
     * 删除群组成员表
     *
     * @param userId
     * @param groupId
     */
    public List<ChatGroupMemberApp> deleteGroupMember(String userId, String groupId) {
        QueryWrapper<ChatGroupMemberApp> wrapper = new QueryWrapper();
        if (StringUtils.isNotEmpty(userId)) {
            wrapper.eq("user_id", userId);
        }
        wrapper.eq("group_id", groupId);
        //查询
        BaseMapper<ChatGroupMemberApp> baseMapper = chatGroupMemberAppService.getBaseMapper();
        List<ChatGroupMemberApp> list = baseMapper.selectList(wrapper);
        if (StringUtils.isNotEmpty(list) && list.size() > CommonConstant.NUMBER_ZERO) {
            List<String> collect = list.parallelStream().map(ChatGroupMemberApp::getId).collect(Collectors.toList());
            baseMapper.deleteBatchIds(collect);
        }
        return list;
    }

    /**
     * 根据email 删除该用户的聊天信息
     *
     * @param userId
     */
    public void deleteMessage(String userId, String groupId) {
        //先查询，在删除
        QueryWrapper<ChatGroupMessageApp> wrapper = new QueryWrapper();
        if (StringUtils.isNotEmpty(userId)) {
            wrapper.eq("user_id", userId);
        }
        wrapper.eq("group_id", groupId);
        ChatGroupMessageAppMapper baseMapper = chatGroupMessageAppService.getBaseMapper();
        List<ChatGroupMessageApp> list = baseMapper.selectList(wrapper);
        if (StringUtils.isNotNull(list) && list.size() > CommonConstant.NUMBER_ZERO) {
            List<String> collect = list.parallelStream().map(ChatGroupMessageApp::getId).collect(Collectors.toList());
            if (StringUtils.isNotEmpty(collect) && collect.size() > 0) {
                //删除
                baseMapper.deleteBatchIds(collect);
            }


        }
        // chatGroupMemberAppService.getBaseMapper().delete(wrapper);
    }

    /**
     * 删除通讯录表
     *
     * @param userId
     * @param imei
     */
    public void deleteAddressBook(String userId, String imei) {
        //先查询，在删除
        QueryWrapper<AddressBookApp> wrapper = new QueryWrapper();
        if (StringUtils.isNotEmpty(userId)) {
            wrapper.eq("user_id", userId);
        }
        wrapper.eq("imei", imei);
        //先查询
        BaseMapper<AddressBookApp> baseMapper = addressBookAppService.getBaseMapper();
        List<AddressBookApp> addressBookApps = baseMapper.selectList(wrapper);
        if (StringUtils.isNotEmpty(addressBookApps) && addressBookApps.size() > CommonConstant.NUMBER_ZERO) {
            List<String> collect = addressBookApps.parallelStream().map(AddressBookApp::getId).collect(Collectors.toList());
            baseMapper.deleteBatchIds(collect);
        }


    }

    /**
     * 通讯录下发
     *
     * @param imei
     */
    @Async
    public void sendToDevice(String imei, String userId) {
        log.info("下发通讯录:{}", Thread.currentThread().getName());
        //1.根据imei查询所有通讯录成员
        LambdaQueryWrapper<AddressBookApp> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(AddressBookApp::getImei, imei);
        List<AddressBookApp> addressBookApps = addressBookAppService.getBaseMapper().selectList(lambdaQueryWrapper);
        //2.开始封装数据
        List<ContactInfo> list = new ArrayList<>();
        ContactInfo contactInfo;
        if (StringUtils.isNotEmpty(addressBookApps)) {
            //排除自己的userId
            List<AddressBookApp> collect = addressBookApps.stream().filter(s -> !s.getUserId().equals(userId)).collect(Collectors.toList());
            for (AddressBookApp addressBookApp : collect) {
                contactInfo = new ContactInfo();
                contactInfo.setId(addressBookApp.getId());
                contactInfo.setName(addressBookApp.getName());
                contactInfo.setPhone(addressBookApp.getPhone());
                contactInfo.setTime(System.currentTimeMillis());
                contactInfo.setUrl(addressBookApp.getImage());
                list.add(contactInfo);
            }
        }
        Contact contact = new Contact();
        contact.setContact(list);
        contact.setVender("20000");
        contact.setIdent("66666");
        contact.setImei(imei);
        contact.setOpenid(SecurityContextHolder.getStringUserId());
        contact.setType("contact");
        contact.setTime(System.currentTimeMillis());
        //3.下发指令之前先存入redis,开始下发指令
        issueInstructions.emitDeviceDemand("contact", imei, contact);
    }

    /**
     * 根据imei 和用户ID 删除申请关注记录表
     *
     * @param imei
     * @param userId
     */
    public void deleteDeviceAttention(String imei, String userId) {
        QueryWrapper<DeviceAttentionApp> wrapper = new QueryWrapper();
        wrapper.eq("imei", imei);
        if (StringUtils.isNotEmpty(userId)) {
            wrapper.eq("user_id", userId);
        }
        BaseMapper<DeviceAttentionApp> baseMapper = deviceAttentionAppService.getBaseMapper();
        //先查询在删除
        List<DeviceAttentionApp> deviceAttentionApps = baseMapper.selectList(wrapper);
        List<String> collect = null;
        if (StringUtils.isNotEmpty(deviceAttentionApps) && deviceAttentionApps.size() > 0) {

            if (StringUtils.isNotEmpty(userId)) {
                collect = deviceAttentionApps.stream().map(DeviceAttentionApp::getId).collect(Collectors.toList());
            } else {
                collect = deviceAttentionApps.parallelStream().map(DeviceAttentionApp::getId).collect(Collectors.toList());

            }
            if (StringUtils.isNotEmpty(collect) && collect.size() > 0) {
                baseMapper.deleteBatchIds(collect);
            }


        }

    }

    /**
     * 根据imei 和用户ID 删除提醒消息表reminder_message_app
     *
     * @param imei
     * @param userId
     */
    public void deleteReminderMessage(String imei, String userId) {
        QueryWrapper<ReminderMessageApp> wrapper = new QueryWrapper();
        wrapper.eq("imei", imei);
        if (StringUtils.isNotEmpty(userId)) {
            wrapper.eq("userId", userId);
        }
        BaseMapper<ReminderMessageApp> baseMapper = reminderMessageAppService.getBaseMapper();
        List<ReminderMessageApp> list = baseMapper.selectList(wrapper);
        if (StringUtils.isNotEmpty(list) && list.size() > 0) {
            List<String> collect = list.parallelStream().map(ReminderMessageApp::getId).collect(Collectors.toList());
            //删除
            baseMapper.deleteBatchIds(collect);
        }


    }

    //删除可视视频信息
    public void deleteVideoInfo(String imei, String userId) {
        QueryWrapper<DeviceVideoApp> wrapper = new QueryWrapper();
        wrapper.eq("imei", imei);
        if (StringUtils.isNotEmpty(userId)) {
            wrapper.eq("user_id", userId);
        }
        BaseMapper<DeviceVideoApp> baseMapper = deviceVideoAppService.getBaseMapper();
        List<DeviceVideoApp> list = baseMapper.selectList(wrapper);
        if (StringUtils.isNotEmpty(list) && list.size() > 0) {
            List<String> collect = list.parallelStream().map(DeviceVideoApp::getId).collect(Collectors.toList());
            //删除
            baseMapper.deleteBatchIds(collect);
        }

    }

    /**
     * 删除最新位置信息表
     *
     * @param imei
     */
    public void deleteLatestLocation(String imei) {
        //先查询，在删除
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("imei", imei);
        latestLocationAppService.getBaseMapper().delete(wrapper);

    }

    /**
     * 删除解析后的位置信息表device_location_app
     *
     * @param imei
     */
    public void deleteDeviceLocation(String imei) {
        //先查询，在删除
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("imei", imei);
        deviceLocationAppService.getBaseMapper().delete(wrapper);

    }

    /**
     * 删除原始的位置信息表original_location_info_app（暂时不删。删的话还要加字段）
     * @param imei
     */
//    public void deleteOriginalLocation(String imei){
//        //先查询，在删除
//        QueryWrapper wrapper =new QueryWrapper();
//        wrapper.eq("imei",imei);
//        deviceLocationAppService.getBaseMapper().delete(wrapper);
//
//    }

    /**
     * 删除闹钟信息表device_alarm_app
     *
     * @param imei
     */
    public void deleteAlarm(String imei) {

        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("imei", imei);
        deviceAlarmAppService.getBaseMapper().delete(wrapper);

    }

    /**
     * 删除上课免打扰信息表device_disable_app
     *
     * @param imei
     */
    public void deleteDeviceDisable(String imei) {

        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("imei", imei);
        deviceDisableAppService.getBaseMapper().delete(wrapper);

    }


    /**
     * 删除定然开关机信息表device_time_switch_app
     *
     * @param imei
     */
    public void deleteDeviceTimeSwitch(String imei) {
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("imei", imei);
        deviceTimeSwitchAppService.getBaseMapper().delete(wrapper);

    }

    /**
     * 删除安全围栏信息表safety_rail
     *
     * @param imei
     */
    public void deleteSafetyRail(String imei) {
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("imei", imei);
        safetyRailAppService.getBaseMapper().delete(wrapper);

    }

    /**
     * 删除SOS信息表
     *
     * @param imei
     */
    public void deleteSOSInfo(String imei) {
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("imei", imei);
        imeiSosInfoService.getBaseMapper().delete(wrapper);
    }

    /**
     * 管理员解绑删除轨迹信息
     *
     * @param imei
     */
    public void deleteTrackInfo(String imei) {
        BaseMapper<TrackInfoApp> baseMapper = trackInfoAppService.getBaseMapper();
        QueryWrapper<TrackInfoApp> wrapper = new QueryWrapper();
        wrapper.eq("imei", imei);
        List<TrackInfoApp> trackInfoApps = baseMapper.selectList(wrapper);
        if (StringUtils.isNotEmpty(trackInfoApps) && trackInfoApps.size() > 0) {
            List<String> collect = trackInfoApps.parallelStream().map(TrackInfoApp::getId).collect(Collectors.toList());
            baseMapper.deleteBatchIds(collect);
        }

    }

    /**
     * 管理员解绑根据imei查询绑定表，准备推送到APP
     *
     * @param adminId
     * @param list
     */
    @Async
    public void sendUnbindInfoToApp(String adminId, List<ChatGroupMemberApp> list, String name, String imei) {
        log.info("异步发送信息,异步线程名称:{}", Thread.currentThread().getName());
        log.info("管理员解绑用户:{}", list);
        if (StringUtils.isNotEmpty(list) && list.size() > CommonConstant.NUMBER_ZERO) {
            //过滤掉管理员自己
            List<ChatGroupMemberApp> collect = list.stream().filter(s -> !s.getUserId().equals(adminId)).collect(Collectors.toList());
            for (ChatGroupMemberApp memberApp : collect) {
                ChatGroupDisbandMessage message = new ChatGroupDisbandMessage();
                message.setDisbandTime(new Date());
                message.setOperator(adminId);
                message.setGroupId(memberApp.getGroupId());
                message.setName(name);
                message.setType("chatGroupDisband");
                message.setImei(imei);
                String s = JSON.toJSONString(message);
                //根据管理员ID查询手机型号
                String phoneType = redisService.getCacheMapValue(memberApp.getUserId(), CommonConstant.USER_AGENT);
                if (CommonConstant.ANDROID.equals(phoneType)) {
                    //发送
                    mqttPush.sendMqtt(CommonConstant.AUDIT_STATUS_THREE, memberApp.getUserId(), CommonConstant.AUDIT_STATUS_ONE, "chatGroupDisband", "chatGroupDisband", s);
                } else if (CommonConstant.IOS.equals(phoneType)) {
//                        //根据管理员Id获取语言信息
//                        String language = redisService.getCacheMapValue(memberApp.getUserId(), CommonConstant.IOS_LANGUAGE);
//                        String alert;
//                        //中文
//                        if (CommonConstant.CN.contains(language)) {
//
//                            alert = String.format("手表【%s】的管理员将手表恢复出厂设置",name);
//                            //英文
//                        } else {
//                            alert = String.format("Wrist watch【%s】s administrator restores the watch to factory settings", name);
//                        }
                    //String alert= I18nUtil.getMessage("messages.ios.chatGroupDisband");
                    String lang = redisService.getCacheMapValue(adminId, CommonConstant.IOS_LANGUAGE);
                    String alert = String.format(I18nUtil.getMessage("messages.ios.chatGroupDisband", lang), name);
                    sendIOS(adminId, message, "chatGroupDisband", "chatGroupDisband", alert);
                }


            }
        }

    }


    /**
     * 解绑视频通话记录
     *
     * @param imei
     */
    public void deleteVideoRecordInfo(String imei, String userId) {
        LambdaQueryWrapper<DeviceVideoRecordApp> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(DeviceVideoRecordApp::getImei, imei);
        if (null != userId) {
            lambdaQueryWrapper.eq(DeviceVideoRecordApp::getUserId, userId);
        }
        deviceVideoRecordAppService.getBaseMapper().delete(lambdaQueryWrapper);
    }


    /**
     * 根据imei查询群组表，获取群组表ID
     *
     * @param
     * @return
     */
    private ChantGroupApp getChatGroupMember(String imei) {
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("imei", imei);
        return chantGroupAppServiceImlp.getBaseMapper().selectOne(wrapper);

    }

    /**
     * 根据消息ID查询消息记录和申请人userId查询消息记录
     *
     * @param id
     * @return
     */
    private DeviceAttentionApp getDeviceAttention(String id, String userId, String imei, String adminId) {
        DeviceAttentionApp deviceVideoApp = null;
        if (StringUtils.isNotEmpty(id)) {
            deviceVideoApp = deviceAttentionAppService.getBaseMapper().selectById(id);
        } else {
            //查询出申请人的那条记录信息
            QueryWrapper wrapper = new QueryWrapper();
            wrapper.eq("user_id", userId);
            wrapper.eq("approval_status", CommonConstant.AUDIT_STATUS_ZERO);
            wrapper.eq("imei", imei);
            wrapper.eq("admin_id", adminId);
            wrapper.orderByDesc("create_time");
            wrapper.last("LIMIT 1");

            deviceVideoApp = deviceAttentionAppService.getBaseMapper().selectOne(wrapper);
        }

        return deviceVideoApp;
    }


    /**
     * 根据管理员id，申请关注人ID，imei查询申请关注记录表
     *
     * @param adminUserId
     * @param userId
     * @param imei
     * @return
     */
    private DeviceAttentionApp getAttentionApp(String adminUserId, String userId, String imei) {
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("admin_id", adminUserId);
//        wrapper.eq("apply_id",userId);
        wrapper.eq("user_id", userId);
        wrapper.eq("imei", imei);
        wrapper.eq("approval_status", CommonConstant.AUDIT_STATUS_ZERO);
        wrapper.last("LIMIT 1");
        return deviceAttentionAppService.getBaseMapper().selectOne(wrapper);

    }

    /**
     * 申请关注成功失败推送到申请人。
     */
    private void sendAttentionToApp(ApprovalDto dto) {
        AttentionMessage message = new AttentionMessage();
        message.setType("accountMessage");
        AttentionMessageChild child = new AttentionMessageChild();
        child.setPhone(dto.getPhone());
        child.setImei(dto.getImei());
        child.setTimestamp(System.currentTimeMillis());
        child.setType("checkedResults");
        if (CommonConstant.AUDIT_STATUS_ONE.equals(dto.getStatus())) {
            //同意
            child.setAction("agree");
        } else {
            //拒绝
            child.setAction("reject");
        }
        //根据imei获取设备拥有者信息
        DeviceOwnerInfoApp deviceOwnerInfoApp = getDeviceOwnerInfoApp(dto.getImei());
        if (!StringUtils.isNotNull(deviceOwnerInfoApp)) {
            throw new ServiceException("此设备暂无绑定信息请联系管理员");
        }
        child.setName(Optional.ofNullable(deviceOwnerInfoApp.getName()).orElse("宝贝"));
        message.setMessage(child);
        String s = JSON.toJSONString(message);
        //根据用户id查询手机型号
        String phoneType = redisService.getCacheMapValue(dto.getUserId(), CommonConstant.USER_AGENT);
        if (CommonConstant.ANDROID.equals(phoneType)) {
            //发送
            mqttPush.sendMqtt(CommonConstant.AUDIT_STATUS_THREE, dto.getUserId(), CommonConstant.AUDIT_STATUS_ONE, "accountMessage", "checkedResults", s);
        } else if (CommonConstant.IOS.equals(phoneType)) {
//            //根据管理员Id获取语言信息
//            String language = redisService.getCacheMapValue(dto.getUserId(), CommonConstant.IOS_LANGUAGE);
//            String alert;
//            //中文
//            if (CommonConstant.CN.contains(language)) {
//
//                alert = String.format("手表（%s）的管理员已同意您 关注手表",dto.getImei());
//                //英文
//            } else {
//                alert = String.format("Wrist watch（%s）s administrator restores the watch to factory settings", dto.getImei());
//            }
            String lang = redisService.getCacheMapValue(dto.getUserId(), CommonConstant.IOS_LANGUAGE);
            String alert = String.format(I18nUtil.getMessage("messages.ios.checkedResults", lang), dto.getImei());
            sendIOS(dto.getUserId(), message, "accountMessage", "checkedResults", alert);
        }
    }

    /**
     * 自动审批发送给管理员跟其余绑定成员
     * @param dto
     */
//    @Async
//    public void sendAutomaticApprovalToApp(ApplyFollowDto dto,String uId) {
//        AttentionMessage message = new AttentionMessage();
//        message.setType("accountMessage");
//        AttentionMessageChild child = new AttentionMessageChild();
//        child.setPhone(dto.getPhone());
//        child.setImei(dto.getImei());
//        child.setTimestamp(System.currentTimeMillis());
//        child.setType("automaticApproval");
//        //同意
////        child.setAction("agree");
//        //根据imei获取设备拥有者信息
////        DeviceOwnerInfoApp deviceOwnerInfoApp = getDeviceOwnerInfoApp(dto.getImei());
////        if (!StringUtils.isNotNull(deviceOwnerInfoApp)) {
////            throw new ServiceException("此设备暂无绑定信息请联系管理员");
////        }
////        child.setName(Optional.ofNullable(deviceOwnerInfoApp.getName()).orElse("宝贝"));
//        message.setMessage(child);
//        String s = JSON.toJSONString(message);
//
//        QueryWrapper<UserDeviceBindApp> wrapper =new QueryWrapper<>();
//        wrapper.eq("imei",dto.getImei());
//        wrapper.eq("bind_status",CommonConstant.AUDIT_STATUS_ONE);
//        List<UserDeviceBindApp> userDeviceBindApps = userDeviceBindService.getBaseMapper().selectList(wrapper);
//        if (StringUtils.isNotEmpty(userDeviceBindApps)){
//            userDeviceBindApps.stream().map(UserDeviceBindApp::getUserId).filter(sid->!sid.equals(uId)).collect(Collectors.toList()).forEach(userId -> {
//
//                //根据用户id查询手机型号
//                String phoneType = redisService.getCacheMapValue(userId, CommonConstant.USER_AGENT);
//                if (CommonConstant.ANDROID.equals(phoneType)) {
//                    //发送
//                    mqttPush.sendMqtt(CommonConstant.AUDIT_STATUS_THREE, userId, CommonConstant.AUDIT_STATUS_ONE, "accountMessage", "automaticApproval", s);
//                } else if (CommonConstant.IOS.equals(phoneType)) {
//                    String lang = redisService.getCacheMapValue(userId, CommonConstant.IOS_LANGUAGE);
//                    String alert = String.format(I18nUtil.getMessage("messages.ios.automaticApproval", lang), dto.getPhone());
//                    sendIOS(userId, message, "accountMessage", "automaticApproval", alert);
//                }
//
//            });
//
//        }
//
//
//
//    }


    /**
     * 发送关注信息到管理员
     */
    private void sendAttentionApp(ApplyFollowDto applyFollowDto, String name) {
        ApprovalMessages approvalMessages = new ApprovalMessages();
        BaseMqttMessages message = new BaseMqttMessages();
        approvalMessages.setType("accountMessage");
        message.setType("applicationRrecord");
        message.setAdminId(applyFollowDto.getAdminUserId());
        message.setName(name);
        message.setPhone(applyFollowDto.getPhone());
        message.setTimestamp(System.currentTimeMillis());
        message.setUserId(SecurityContextHolder.getStringUserId());
        message.setEmail(SecurityContextHolder.getUserName());
        message.setImei(applyFollowDto.getImei());
        approvalMessages.setMessage(message);
        String s = JSON.toJSONString(approvalMessages);
        //根据用户id查询手机型号
        String phoneType = redisService.getCacheMapValue(applyFollowDto.getAdminUserId(), CommonConstant.USER_AGENT);
        if (CommonConstant.ANDROID.equals(phoneType)) {
            //发送
            mqttPush.sendMqtt(CommonConstant.AUDIT_STATUS_THREE, applyFollowDto.getAdminUserId(), CommonConstant.AUDIT_STATUS_ONE, "accountMessage", "applicationRrecord", s);
        } else if (CommonConstant.IOS.equals(phoneType)) {
//            //根据管理员Id获取语言信息
//            String language = redisService.getCacheMapValue(applyFollowDto.getAdminUserId(), CommonConstant.IOS_LANGUAGE);
//            String alert;
//            //中文
//            if (CommonConstant.CN.contains(language)) {
//
//                alert = String.format("用户（%s）申请关注[%s] 手表",applyFollowDto.getPhone(),name);
//                //英文
//            } else {
//                alert = String.format("User %s applied for following %s watch", applyFollowDto.getPhone(),name);
//            }
            String lang = redisService.getCacheMapValue(applyFollowDto.getAdminUserId(), CommonConstant.IOS_LANGUAGE);
            String alert = String.format(I18nUtil.getMessage("messages.ios.applicationRrecord", lang), applyFollowDto.getPhone(), name);
            sendIOS(applyFollowDto.getAdminUserId(), approvalMessages, "accountMessage", "applicationRrecord", alert);
        }
        //推送传管理员ID
        // mqttPush.sendMqtt("3",applyFollowDto.getAdminUserId(),"1","accountMessage","applicationRrecord",s);
    }

    /**
     * 下发通讯录 组装参数
     *
     * @param dto
     * @param userId
     */
    public void assembleContactParam(BindingDeviceDto dto, String userId) {
        //下发通讯录 组装参数
        ContactInfo info = new ContactInfo();
        info.setId(userId);
        info.setName(dto.getRelationship());
        info.setPhone(dto.getMy_phone());
        info.setTime(System.currentTimeMillis()); //接口文档是float现在暂时用LONG来代替
        info.setUrl(dto.getRelationshipId());
        Contact contact = new Contact();
        contact.setImei(dto.getImei());
        contact.setTime(System.currentTimeMillis());
        contact.setOpenid(userId);
        contact.setIdent(RandomUtil.generateVerCode(6));
        contact.setType("contact");
        contact.setVender(redisService.getCacheMapValue(dto.getImei(), "agent"));
        contact.setContact(Arrays.asList(info));
        issueInstructions.emitDeviceDemand("contact", dto.getImei(), contact);
    }

    /**
     * 中心号码设置下发
     *
     * @param dto
     * @param userId
     */
    public void assembCenterPhone(BindingDeviceDto dto, String userId) {
        CenterPhoneSetting centerPhoneSetting = new CenterPhoneSetting();
        centerPhoneSetting.setPhone(dto.getMy_phone());
        centerPhoneSetting.setOpenid(userId);
        centerPhoneSetting.setIdent(RandomUtil.generateVerCode(6));
        centerPhoneSetting.setTime(System.currentTimeMillis());
        centerPhoneSetting.setType("centerPhone");
        centerPhoneSetting.setVender(redisService.getCacheMapValue(dto.getImei(), "agent"));
        centerPhoneSetting.setImei(dto.getImei());
        issueInstructions.emitDeviceDemand("centerPhone", dto.getImei(), centerPhoneSetting);
    }

//    /**
//     * 可视频通话用户列表下发
//     * @param dto
//     * @param userId
//     */
//    private void assembDownGetVideoUserList(BindingDeviceDto dto, String userId){
//       // emitDeviceDemand(imei, {'type': 'downGetVideoUserList', 'user_list': checked_video_list})
//        List<VideoUser> list =new ArrayList<>();
//        DownGetVideoUserList downGetVideoUserList =new DownGetVideoUserList();
//        downGetVideoUserList.setImei(dto.getImei());
//        downGetVideoUserList.setIdent(RandomUtil.generateVerCode(6));
//        downGetVideoUserList.setType("downGetVideoUserList");
//        downGetVideoUserList.setVender(redisService.getCacheMapValue(dto.getImei(),"agent"));
//        downGetVideoUserList.setTime(System.currentTimeMillis());
//        VideoUser user =new VideoUser();
//        user.setUser_openid(userId);
//        user.setRelationship(dto.getNickname());
//        user.setPhone(dto.getUserPhone());
//        user.setCanVideo(CommonConstant.NUMBER_ONE);
//        // TODO 这里还要修改
//        user.setRelationship_image_id(CommonConstant.NUMBER_ONE);
//        list.add(user);
//        downGetVideoUserList.setUser_list(list);
//        issueInstructions.emitDeviceDemand("downGetVideoUserList",dto.getEmail(),downGetVideoUserList);
//    }

    /**
     * 根据imei查询视频类型，及视频id
     *
     * @param imei
     * @return
     */
    public VideoModelInfoPO getVideoByImei(String imei) {
        return videoCallAppService.getBaseMapper().getVideoByImei(imei);
    }


//    @Override
//    public boolean SendPushToAccount(BindingDeviceDto deviceDto) {
//        //查询该设备的管理email
//        UserDeviceBindApp bindApp = this.queryAdminByImei(deviceDto.getImei());
//        if (StringUtils.isNotNull(bindApp)){
//            MessageRecordApp recordApp = new MessageRecordApp();
//            recordApp.setId(UUID.fastUUID().toString(true));
//            //审批状态(0.未审批1.审批通过2.拒绝)
//            recordApp.setApprovalStatus(CommonConstant.AUDIT_STATUS_ZERO);
//            recordApp.setImei(deviceDto.getImei());
//            recordApp.setEmail(deviceDto.getEmail());
//            recordApp.setPhone(deviceDto.getUserPhone());
//            recordApp.setName(deviceDto.getNickname());
//            boolean save = messageRecordAppService.save(recordApp);
//            String topic ="iot08/user/"+bindApp.getUserId();
//            log.info("发送的信息是:{}",deviceDto);
//            boolean flag= this.pubMessage("测试",0,topic);
//            log.info("是否发送成功:{}",flag);
//            //这里就不管消息是否成功，只要保证数据插入成功即可
//            return save;
//        }else {
//            return false;
//        }
//
//
//    }

//    /**
//     * 发送审批信息给管理员
//     * @param deviceDto
//     */
//    private void   SendApprovalAccountInfo(BindingDeviceDto deviceDto,String type) {//type，区分推送给安卓，还是IOS
//        ApprovalMessages approvalMessages =new ApprovalMessages();
//        BaseMqttMessages baseMqttMessages =new BaseMqttMessages();
//        baseMqttMessages.setImei(deviceDto.getImei());
//        baseMqttMessages.setName(deviceDto.getNickname());
//        baseMqttMessages.setType("checkedResults");
//       // baseMqttMessages.setAction();
//        //查询该设备的管理email
//        UserDeviceBindApp bindApp = this.queryAdminByImei(deviceDto.getImei());
//        if (StringUtils.isNotNull(bindApp)) {
//            MessageRecordApp recordApp = new MessageRecordApp();
//            recordApp.setId(UUID.fastUUID().toString(true));
//            //审批状态(0.未审批1.审批通过2.拒绝)
//            recordApp.setApprovalStatus(CommonConstant.AUDIT_STATUS_ZERO);
//            recordApp.setImei(deviceDto.getImei());
//            recordApp.setEmail(deviceDto.getEmail());
//            recordApp.setPhone(deviceDto.getUserPhone());
//            recordApp.setName(deviceDto.getNickname());
//            boolean save = messageRecordAppService.save(recordApp);
//            String topic = "iot08/user/" + bindApp.getUserId();
//            log.info("发送的信息是:{}", deviceDto);
//            boolean flag = this.pubMessage("测试", 0, topic);
//            log.info("是否发送成功:{}", flag);
//            //这里就不管消息是否成功，只要保证数据插入成功即可
//
//        }
//    }


    /**
     * 根据imei和绑定手机号码查询通讯录数据库，有就先删除在新增
     */
    public void queryBookByImeiAndPhone(AddressBookApp app) {
        QueryWrapper<AddressBookApp> queryWrapper = new QueryWrapper();
        queryWrapper.eq("imei", app.getImei());
        queryWrapper.eq("phone", app.getPhone());
        BaseMapper<AddressBookApp> baseMapper = addressBookAppService.getBaseMapper();
        AddressBookApp bookApp = baseMapper.selectOne(queryWrapper);
        if (StringUtils.isNotNull(bookApp)) { //先删除
            baseMapper.deleteById(bookApp.getId());
        }
        addressBookAppService.save(app);


    }

    /**
     * 根据imei和绑定手机号码查询通讯录数据库，有就先删除在新增
     */
    public void deleteBook(String userId, String imei) {
        QueryWrapper<AddressBookApp> queryWrapper = new QueryWrapper();
        queryWrapper.eq("imei", imei);
        queryWrapper.eq("user_id", userId);
        addressBookAppService.getBaseMapper().delete(queryWrapper);
    }


    /**
     * 查询用户绑定设备数量
     *
     * @param email
     * @return
     */
    private int queryBindingDeviceNumber(String email) {
        QueryWrapper<UserDeviceBindApp> wrapper = new QueryWrapper();
//        wrapper.eq("imei",imei);
        wrapper.eq("user_email", email);
        // TODO 这里查询的是待审批，跟审批通过的
        wrapper.lambda().and(s -> s.eq(UserDeviceBindApp::getApproveStatus, CommonConstant.AUDIT_STATUS_ONE).or().eq(UserDeviceBindApp::getApproveStatus, CommonConstant.AUDIT_STATUS_ZERO));
        return userDeviceBindService.getBaseMapper().selectCount(wrapper);

    }

    /**
     * 查询用户是否申请，绑定过改设备
     *
     * @param email
     * @return
     */
    private UserDeviceBindApp queryIsBindingDevice(String email, String imei) {
        QueryWrapper<UserDeviceBindApp> wrapper = new QueryWrapper();
        wrapper.eq("imei", imei);
        wrapper.eq("user_email", email);
        // TODO 这里查询的是待审批，跟审批通过的
        wrapper.lambda().and(s -> s.eq(UserDeviceBindApp::getApproveStatus, CommonConstant.AUDIT_STATUS_ONE).or().eq(UserDeviceBindApp::getApproveStatus, CommonConstant.AUDIT_STATUS_ZERO));
        return userDeviceBindService.getBaseMapper().selectOne(wrapper);

    }


    /**
     * 根据查询用户绑定设备数量
     *
     * @param imei
     * @return
     */
    private UserDeviceBindApp queryAdminByImei(String imei) {
        QueryWrapper<UserDeviceBindApp> wrapper = new QueryWrapper();
        wrapper.eq("imei", imei);
        //已审批通过
        wrapper.eq("approve_status", CommonConstant.AUDIT_STATUS_ONE);
        //管理员
        wrapper.eq("identity", CommonConstant.AUDIT_STATUS_TWO);
        return userDeviceBindService.getBaseMapper().selectOne(wrapper);

    }

    /**
     * 根据imei 查询设备拥有者信息
     *
     * @param imei
     * @return
     */
    public DeviceOwnerInfoApp getDeviceOwnerInfoApp(String imei) {
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("imei", imei);
        return deviceOwnerInfoAppService.getBaseMapper().selectOne(wrapper);

    }

    /**
     * 根据 管理员id查询用户绑定表
     *
     * @param
     * @return
     */
    public UserDeviceBindApp getBingDeviceInfo(String adminUserId) {
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("user_id", adminUserId);
        wrapper.last("LIMIT 1");
        return userDeviceBindService.getBaseMapper().selectOne(wrapper);

    }

    /**
     * 根据imei，查询用户绑定表的数据
     *
     * @param imei
     * @return
     */
    public List<UserDeviceBindApp> getUserIdList(String imei) {
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("imei", imei);
        return userDeviceBindService.getBaseMapper().selectList(wrapper);

    }

    /**
     * 根据imei，查询用户绑定表管理员的数据
     *
     * @param imei
     * @return
     */
    public UserDeviceBindApp getAdminUser(String imei) {
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("imei", imei);
        wrapper.eq("identity", CommonConstant.AUDIT_STATUS_TWO);
        return userDeviceBindService.getBaseMapper().selectOne(wrapper);

    }

    /**
     * 根据邮箱查询用户信息
     *
     * @param email
     * @return
     */
    private User queryUserByEmail(String email) {
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("user_email", email);

        return userMapper.selectOne(wrapper);
    }

    /**
     * 根据用户id,查询绑定信息
     *
     * @param userId
     * @return
     */
    public UserDeviceBindApp getBingInfoByUserId(String userId, String imei) {
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("user_id", userId);
        wrapper.eq("imei", imei);
        wrapper.eq("bind_status", CommonConstant.AUDIT_STATUS_ONE);
        wrapper.last("LIMIT 1");
        return userDeviceBindService.getBaseMapper().selectOne(wrapper);
    }


//    /**
//     * 发送消息
//     */
//    private boolean pubMessage(String message, int qos, String topic) {
//        MqttClient mqttClient = AppMqttClient.customStartTwo("tcp://120.77.209.176:18083", "yisai@@@2222", "admin", "public", 20, 60);
//        if (StringUtils.isNotNull(mqttClient)) {
//            AppMqttClient.publishCustomMessage(mqttClient, qos, message, topic);
//            return true;
//        } else {
//            return false;
//        }
//
//    }

    /**
     * 根据imei查询定位模式信息
     *
     * @param imei
     * @return
     */
    public PositioningModelInfo queryPositioningModelInfo(String imei) {
        PositioningModelInfo positioningModelInfo = videoCallAppService.getBaseMapper().queryPositioningModelInfo(imei);
        if (StringUtils.isNull(positioningModelInfo)) {
            //查询默认的
            positioningModelInfo = videoCallAppService.getBaseMapper().queryPositioningModelInfoById(imei);
            //该设备没有设置定位模式请联系管理员进行设置
            throw new ServiceException("messagesPositioningModelError");

        }
        return positioningModelInfo;

    }

    /**
     * 获取视频类型
     *
     * @param imei
     */
    public DeviceVideoApp getVideoApp(String imei, String userId) {
        //初始化可视频列表 //只有绑定了手表的人才能在可视频列表中
        DeviceVideoApp videoCallApp = new DeviceVideoApp();
        videoCallApp.setUserId(userId);
        videoCallApp.setImei(imei);
        //根据imei查询视频型号跟视频类型
        VideoModelInfoPO videoByImei = getVideoByImei(imei);
        if (StringUtils.isNotNull(videoByImei)) {
            //视频类型(1.菊风，2.声网，3.佰锐，4.鹈鹕)
            videoCallApp.setVideoType(videoByImei.getVideoModelType());
            //videoCallApp.setVideoTypeId(videoByImei.getVideoModelId());
        } else {
            videoCallApp.setVideoType(CommonConstant.AUDIT_STATUS_ZERO);
        }

        return videoCallApp;

    }

    /**
     * 发送IOS公共参数封装
     *
     * @param userId    用户id
     * @param s         发送的内容
     * @param type      发送的类型
     * @param childType 发送的子类型
     * @param alert     IOS系统推送的提示信息
     */
    public void sendIOS(String userId, Object s, String type, String childType, String alert) {
        SendIOSParamete paramete = new SendIOSParamete();
        IOSMessage message = new IOSMessage();
        paramete.setType(type);
        paramete.setChildType(childType);
        paramete.setOpenid(userId);
        message.setSound(CommonConstant.SOUND);
        message.setContent_available(CommonConstant.NUMBER_ONE);
        message.setData(s);
        message.setType(childType);
        message.setAlert(alert);
        paramete.setApns_param(message);
        iosPush.sendIOS(paramete);

    }

    public void sendIOSLogin(String userId, Object s, String type, String childType, String alert, String iOSToken) {
        SendIOSParamete paramete = new SendIOSParamete();
        IOSMessage message = new IOSMessage();
        paramete.setType(type);
        paramete.setIOSToken(iOSToken);
        paramete.setChildType(childType);
        paramete.setOpenid(userId);
        message.setSound(CommonConstant.SOUND);
        message.setContent_available(CommonConstant.NUMBER_ONE);
        message.setData(s);
        message.setType(childType);
        message.setAlert(alert);
        paramete.setApns_param(message);
        iosPush.sendIOS(paramete);

    }

    /**
     * 根据imei查询设备型号
     *
     * @param imei
     * @return
     */
    public DeviceModelInfo getDeviceModelInfo(String imei) {
        QueryWrapper<DeviceInfo> wrapper = new QueryWrapper<>();
        wrapper.eq("imei", imei);
        wrapper.eq("audit_status", CommonConstant.AUDIT_STATUS_TWO);
        wrapper.eq("del_flag", CommonConstant.AUDIT_STATUS_ONE);
        DeviceInfo deviceInfo = deviceInfoService.getBaseMapper().selectOne(wrapper);


        return deviceModelInfoService.getBaseMapper().selectById(deviceInfo.getDeviceModelId());
    }

    public UserDeviceBindApp getUserDeviceBindApp(String imei, String userId) {
        QueryWrapper<UserDeviceBindApp> wrapper = new QueryWrapper<>();
        wrapper.eq("imei", imei);
        wrapper.eq("user_id", userId);
        wrapper.eq("approve_status", CommonConstant.AUDIT_STATUS_ONE);
        return userDeviceBindService.getBaseMapper().selectOne(wrapper);
    }
}
