package com.ecell.internationalize.app.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ecell.internationalize.app.entity.DeviceAttentionApp;
import com.ecell.internationalize.app.entity.po.DeviceAttentionPO;
import com.ecell.internationalize.app.mapper.DeviceAttentionAppMapper;
import com.ecell.internationalize.app.service.DeviceAttentionAppService;
import com.ecell.internationalize.common.core.context.SecurityContextHolder;
import com.ecell.internationalize.common.core.exception.ServiceException;
import com.ecell.internationalize.common.core.utils.StringUtils;
import com.ecell.internationalize.common.issue.constant.CommonConstant;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author ${author}
 * @since 2022-09-22
 */
@Service
public class DeviceAttentionAppServiceImpl extends ServiceImpl<DeviceAttentionAppMapper, DeviceAttentionApp> implements DeviceAttentionAppService {
    private static final Logger log = LoggerFactory.getLogger(DeviceAttentionAppServiceImpl.class);
    @Override
    public IPage queryFollowInfo(String userId, int pageSize, int pageNumber) {
         userId= SecurityContextHolder.getStringUserId();

        if (pageSize <= 0) {
            pageSize = 5;
        }
        if (pageNumber <= 0) {
            pageNumber = 1;
        }
        IPage page = new Page(pageNumber, pageSize);
        List<DeviceAttentionPO> list = getList(userId);

        return page.setRecords(list);
    }

    @Override
    public int queryUnreadCount() {
        //获取用户id
        String userId = SecurityContextHolder.getStringUserId();
        return getUnreadCount(userId);
    }

    /**
     * 更改已读状态
     *
     * @param id
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean updateStatus(String id, String status) {
        boolean flag = true;
        //获取用户id
        String userId = SecurityContextHolder.getStringUserId();
        //已读状态
        if (CommonConstant.AUDIT_STATUS_ONE.equals(status)) {
            DeviceAttentionApp app = baseMapper.selectById(id);
            if (!StringUtils.isNotNull(app)) {
                throw new ServiceException("messagesDeviceAttentionError");
            }
            if (StringUtils.isNotNull(app)) {
                app.setApplyRead(status);
                flag = baseMapper.updateById(app) > 0 ? true : false;
            }
            //删除
        } else if (CommonConstant.AUDIT_STATUS_TWO.equals(status)) {
            flag = deleteAttention(userId);
        }
        return flag;
    }

    /**
     * 查询消息记录
     *
     * @param userId
     * @return
     */
    private List<DeviceAttentionPO> getList(String userId) {
        QueryWrapper<DeviceAttentionApp> wrapper = new QueryWrapper();
        wrapper.eq("user_id", userId);
        wrapper.ne("apply_read", CommonConstant.AUDIT_STATUS_TWO);
        wrapper.orderByDesc("create_time");
        List<DeviceAttentionApp> deviceAttentionApps = baseMapper.selectList(wrapper);
        List<DeviceAttentionPO> DeviceAttentionPOs = null;
        if (StringUtils.isNotEmpty(deviceAttentionApps)) {
            DeviceAttentionPOs = deviceAttentionApps.stream().map(s -> {
                DeviceAttentionPO po = new DeviceAttentionPO();
                po.setId(s.getId());
                po.setAdminId(s.getAdminId());
                po.setImei(s.getImei());
                po.setName(s.getName());
                po.setPhone(s.getPhone());
                po.setRead(s.getApplyRead());
                po.setType(s.getMessagesType());
                po.setAction(s.getApprovalStatus());
                po.setCreateTime(s.getCreateTime());
                po.setRelationship(s.getRelationship());
                po.setRelationshipImageId(s.getRelationshipImageId());
                po.setUserId(s.getApplyId());
                return po;
            }).collect(Collectors.toList());
        }

        return DeviceAttentionPOs;
    }

    /**
     * 查询消息记录
     *
     * @param userId
     * @return
     */
    private int getUnreadCount(String userId) {
        QueryWrapper<DeviceAttentionApp> wrapper = new QueryWrapper();
        wrapper.eq("user_id", userId);
        wrapper.eq("apply_read", CommonConstant.AUDIT_STATUS_ZERO);
        Integer count = baseMapper.selectCount(wrapper);
        if (StringUtils.isNotNull(count)) {
            return count;
        } else {
            return 0;
        }

    }


    /**
     * 查询消息记录
     *
     * @param userId
     * @return
     */
    public boolean deleteAttention(String userId) {
        //这里查询
        QueryWrapper<DeviceAttentionApp> wrapper = new QueryWrapper();
        wrapper.eq("user_id", userId);
        return baseMapper.delete(wrapper) > 0 ? true : false;

    }
}
