package com.ecell.internationalize.app.service.impl;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ecell.internationalize.app.entity.DeviceLocationApp;
import com.ecell.internationalize.app.entity.po.PositionModelInfoPO;
import com.ecell.internationalize.app.mapper.DeviceLocationAppMapper;
import com.ecell.internationalize.app.service.DeviceLocationAppService;
import com.ecell.internationalize.app.service.DeviceOwnerInfoAppService;
import com.ecell.internationalize.app.util.LocationUtil;
import com.ecell.internationalize.common.core.context.SecurityContextHolder;
import com.ecell.internationalize.common.core.utils.StringUtils;
import com.ecell.internationalize.common.core.utils.bean.BeanUtils;
import com.ecell.internationalize.common.issue.component.IssueInstructions;
import com.ecell.internationalize.common.issue.constant.CommonConstant;
import com.ecell.internationalize.common.issue.entity.issuedto.LocationUploadInterval;
import com.ecell.internationalize.common.issue.mqtt.dto.LocationMessages;
import com.ecell.internationalize.common.issue.mqtt.push.MqttPush;
import com.ecell.internationalize.common.issue.util.RandomUtil;
import com.ecell.internationalize.common.system.entity.DeviceOwnerInfoApp;
import com.ecell.internationalize.common.system.entity.PositioningModelInfo;
import com.ecell.internationalize.common.system.entity.UserDeviceBindApp;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author ${author}
 * @since 2022-09-05
 */
@Service
public class DeviceLocationAppServiceImpl extends ServiceImpl<DeviceLocationAppMapper, DeviceLocationApp> implements DeviceLocationAppService {
    private static final Logger log = LoggerFactory.getLogger(DeviceLocationAppServiceImpl.class);
    @Autowired
    private IssueInstructions issueInstructions;
    @Autowired
    @Lazy
    private BindingDeviceImpl bindingDevice;
    @Autowired
    private MqttPush mqttPush;
    @Autowired
    private DeviceOwnerInfoAppService deviceOwnerInfoAppService;

    /**
     * 根据imei查询定位模式信息
     * @param imei
     * @return
     */
    @Override
    public PositionModelInfoPO getDevicePosition(String imei) {
        PositionModelInfoPO info =new PositionModelInfoPO();
        PositioningModelInfo positioningModelInfo = bindingDevice.queryPositioningModelInfo(imei);
        BeanUtils.copyBeanProp(info,positioningModelInfo);
        return info;
    }

    /**
     * 更新定位模型信息
     * @param number
     * @param imei
     * @return
     */
    @Override
    public boolean updatePosition(String number, String imei) {
        //根据imei查询设备拥有者信息表
        QueryWrapper<DeviceOwnerInfoApp> wrapper =new QueryWrapper<>();
        wrapper.eq("imei",imei);
        wrapper.last("LIMIT 1");
        BaseMapper<DeviceOwnerInfoApp> baseOwnerMapper = deviceOwnerInfoAppService.getBaseMapper();
        DeviceOwnerInfoApp deviceOwnerInfoApp = baseOwnerMapper.selectOne(wrapper);
        int i=10;
        if (StringUtils.isNotNull(deviceOwnerInfoApp)){
            if (StringUtils.isNotEmpty(number)){
                i=Integer.parseInt(number);
            }
            deviceOwnerInfoApp.setPositioningInterval(i);
            baseOwnerMapper.updateById(deviceOwnerInfoApp);
            //根据imei查询定位模型信息
        //    PositioningModelInfo info = bindingDevice.queryPositioningModelInfo(imei);
//            //下发的是秒数，我们后台存的是分钟
//            int minute =30;
//            if (CommonConstant.AUDIT_STATUS_ONE.equals(number)){
//                minute =info.getPositioningOneInterva();
//            }else if (CommonConstant.AUDIT_STATUS_TWO.equals(number)){
//                minute=info.getPositioningTwoInterva();
//            }else if (CommonConstant.AUDIT_STATUS_THREE.equals(number)){
//                minute=info.getPositioningThreeInterva();
//            }
            //下发指令
            distributionIssued(i,imei);
            return true;
        }else {
            return false;
        }

    }

    /**
     * 定位模式下发
     * @param minute
     * @param imei
     */
    public void distributionIssued(int minute,String imei) {
        LocationUploadInterval interval =new LocationUploadInterval();
        interval.setTime(System.currentTimeMillis());
        interval.setSecond(minute);
        interval.setImei(imei);
        interval.setOpenid(SecurityContextHolder.getStringUserId());
        interval.setVender(LocationUtil.getVender(imei,"agent"));
        interval.setIdent(RandomUtil.generateVerCode(6));
        interval.setType("locationUploadInterval");
        issueInstructions.emitDeviceDemand("locationUploadInterval",imei,interval);
    }

    /**
     * 主动定位发送APP
     * @param imei
     */
    public void sendLocationApp(String imei, DeviceLocationApp app){
        //根据imei 查询用户绑定表
        List<UserDeviceBindApp> userIdList = bindingDevice.getUserIdList(imei);
        if (StringUtils.isNotEmpty(userIdList)){
            List<String> userIds = userIdList.stream().map(UserDeviceBindApp::getUserId).collect(Collectors.toList());
            for (String userId : userIds) {
                //发送位置信息
                LocationMessages messages =new LocationMessages();
                messages.setType("deviceNowLocation");
                messages.setLoc_type(app.getLocationType());
                messages.setAddress("");
                messages.setLat(app.getLatitude());
                messages.setLon(app.getLongitude());
                messages.setRadius(app.getRadius());
                messages.setImei(imei);
                messages.setTimestamp(System.currentTimeMillis());
                String s = JSON.toJSONString(messages);
                log.info("主动请求下发位置:{}",messages);
                mqttPush.sendMqtt(CommonConstant.AUDIT_STATUS_THREE,userId, CommonConstant.AUDIT_STATUS_ONE,"deviceNowLocation","deviceNowLocation",s);

            }

        }

    }

//    /**
//     * 下发请求位置的指令
//     * @param imei
//     */
//    private void sendLocation(String imei) {
//      //  DeviceLocationApp deviceLocationApp =null;
//        //下发指令到KAFKA
//        DistributeParent distributeParent =new DistributeParent();
//        distributeParent.setImei(imei);
//        distributeParent.setOpenid(SecurityContextHolder.getStringUserId()); //用户ID
//        distributeParent.setType("requestLocation"); //请求位置
//        Object agent = redisService.getCacheMapValue(imei, "agent");
//        if (StringUtils.isNotNull(agent)){
//            distributeParent.setVender(agent.toString()); //厂商编号
//        }else {
//            distributeParent.setVender(redisService.getCacheMapValue(imei, "V")); //厂商编号
//        }
//        distributeParent.setTime(System.currentTimeMillis());
//        distributeParent.setIdent(RandomUtil.generateVerCode(6)); //6位随机总数
//            kafkaTemplate.send(CommonConstant.WATCHPUSH_KAFKA, JSON.toJSONString(distributeParent)).addCallback(success->{
//            log.info("发送请求位置信息成功topic{},partition 分区 {},offset {}", success.getRecordMetadata().topic(),success.getRecordMetadata().partition(),success.getRecordMetadata().offset());
//
//            },failure -> {
//            //失败
//             log.info("下发请求位置信息失败：{}",failure.getMessage());
//             log.info("请求下发位置失败的内容{}",JSON.toJSONString(distributeParent));
//
//        });
//
//    }
}
