package com.ecell.internationalize.app.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ecell.internationalize.app.entity.AddressBookApp;
import com.ecell.internationalize.app.entity.dto.DeviceOwnerVo;
import com.ecell.internationalize.app.entity.dto.QueryOwnerInfoDto;
import com.ecell.internationalize.app.entity.dto.UpdateOwnerInfoDto;
import com.ecell.internationalize.app.mapper.AddressBookAppMapper;
import com.ecell.internationalize.app.mapper.DeviceOwnerInfoAppMapper;
import com.ecell.internationalize.app.mapper.UserDeviceBindAppMapper;
import com.ecell.internationalize.app.service.AddressBookAppService;
import com.ecell.internationalize.app.service.DeviceOwnerInfoAppService;
import com.ecell.internationalize.common.core.context.SecurityContextHolder;
import com.ecell.internationalize.common.issue.component.IssueInstructions;
import com.ecell.internationalize.common.issue.entity.issuedto.DialSwitch;
import com.ecell.internationalize.common.issue.entity.issuedto.DialSwitchDto;
import com.ecell.internationalize.common.issue.entity.issuedto.RejectStrangeCallStatus;
import com.ecell.internationalize.common.issue.entity.tcp.DistributeParent;
import com.ecell.internationalize.common.system.entity.DeviceOwnerInfoApp;
import com.ecell.internationalize.common.system.entity.UserDeviceBindApp;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * <p>
 * 设备拥有者信息 服务实现类
 * </p>
 *
 * @author ${author}
 * @since 2022-08-18
 */
@Service
@Slf4j
public class DeviceOwnerInfoAppServiceImpl extends ServiceImpl<DeviceOwnerInfoAppMapper, DeviceOwnerInfoApp> implements DeviceOwnerInfoAppService {
    @Autowired
    private DeviceOwnerInfoAppMapper deviceOwnerInfoAppMapper;
    @Autowired
    private UserDeviceBindAppMapper userDeviceBindAppMapper;
    @Value("${my.topic}")
    private String topic;
    @Autowired
    private IssueInstructions issueInstructions;
    @Autowired
    private AddressBookAppMapper addressBookAppMapper;
    @Autowired
    private AddressBookAppService addressBookAppService;
    /**
     * 根据imei查询设备拥有者信息
     * @param imei
     * @return
     */
    @Override
    public DeviceOwnerInfoApp queryOwnerInfoByimei(String imei) {
        QueryWrapper wrapper =new QueryWrapper();
        wrapper.eq("imei",imei);

        return deviceOwnerInfoAppMapper.selectOne(wrapper);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public int updateData(UpdateOwnerInfoDto updateOwnerInfoDto) {
        //2.根据设备imei号修改设备拥有者信息
        DeviceOwnerInfoApp deviceOwnerInfoApp=new DeviceOwnerInfoApp();
        deviceOwnerInfoApp.setImei(updateOwnerInfoDto.getImei());
        //需要判断上传的头像是否为空
        if (null!=updateOwnerInfoDto.getImage()) {
            deviceOwnerInfoApp.setImage(updateOwnerInfoDto.getImage());
        }
        deviceOwnerInfoApp.setSex(updateOwnerInfoDto.getSex());
        deviceOwnerInfoApp.setPhone(updateOwnerInfoDto.getPhone());
        deviceOwnerInfoApp.setBirthday(updateOwnerInfoDto.getBirthday());
        deviceOwnerInfoApp.setHeight(updateOwnerInfoDto.getHeight());
        deviceOwnerInfoApp.setWeight(updateOwnerInfoDto.getWeight());
        deviceOwnerInfoApp.setName(updateOwnerInfoDto.getName());

        LambdaQueryWrapper<DeviceOwnerInfoApp> lambdaQueryWrapper1=new LambdaQueryWrapper<>();
        lambdaQueryWrapper1.eq(DeviceOwnerInfoApp::getImei,updateOwnerInfoDto.getImei());
        deviceOwnerInfoAppMapper.update(deviceOwnerInfoApp,lambdaQueryWrapper1);
        //2.根据userId查询用户与设备绑定信息
        LambdaQueryWrapper<UserDeviceBindApp> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(UserDeviceBindApp::getUserId,SecurityContextHolder.getStringUserId());
        lambdaQueryWrapper.eq(UserDeviceBindApp::getImei,updateOwnerInfoDto.getImei());
        UserDeviceBindApp userDeviceBindApp = new UserDeviceBindApp();
        userDeviceBindApp.setRelation(updateOwnerInfoDto.getRelation());
        userDeviceBindApp.setRelationImageId(updateOwnerInfoDto.getRelationImageId());

        LambdaQueryWrapper<AddressBookApp> addressBookAppLambdaQueryWrapper=new LambdaQueryWrapper<>();
        addressBookAppLambdaQueryWrapper.eq(AddressBookApp::getImei,updateOwnerInfoDto.getImei());
        addressBookAppLambdaQueryWrapper.eq(AddressBookApp::getUserId, SecurityContextHolder.getStringUserId());
        AddressBookApp addressBookApp = addressBookAppMapper.selectOne(addressBookAppLambdaQueryWrapper);
        if (null!=addressBookApp && !addressBookApp.getRelation().equals(updateOwnerInfoDto.getRelation())){
            addressBookApp.setRelation(updateOwnerInfoDto.getRelation());
            addressBookApp.setName(updateOwnerInfoDto.getRelation());
            addressBookApp.setImage(updateOwnerInfoDto.getRelationImageId());
            addressBookAppMapper.updateById(addressBookApp);
            //开始指令下发
            addressBookAppService.sendToKafka(addressBookApp.getImei());
        }


        return userDeviceBindAppMapper.update(userDeviceBindApp,lambdaQueryWrapper);
    }

    @Override
    public void sendToKafka(DeviceOwnerInfoApp deviceOwnerInfoApp) {
        RejectStrangeCallStatus rejectCalls=new RejectStrangeCallStatus();
        rejectCalls.setType("rejectStrangeCallStatus");
        rejectCalls.setIdent("66666");
        rejectCalls.setVender("20000");
        rejectCalls.setOpenid(SecurityContextHolder.getStringUserId());
        rejectCalls.setImei(deviceOwnerInfoApp.getImei());
        rejectCalls.setTime(System.currentTimeMillis());
        rejectCalls.setStatus(Integer.valueOf(deviceOwnerInfoApp.getRejectStrange()));
        String object = JSONObject.toJSONString(rejectCalls);
        log.info("转换的JSON数据为:{}",object);
        //3.调用方法缓存入redis
        issueInstructions.emitDeviceDemand("rejectStrangeCallStatus",deviceOwnerInfoApp.getImei(),rejectCalls);
        //MyKafkaProducer.producerKafka(topic,object);
    }

    @Override
    public DeviceOwnerVo queryOwnerInfo(String imei) {
        return deviceOwnerInfoAppMapper.queryOwnerInfo(imei);
    }

    @Override
    public void shutDownToKafka(String imei) {
        DistributeParent distributeParent=new DistributeParent();
        distributeParent.setType("PowerOff");
        distributeParent.setIdent("66666");
        distributeParent.setVender("20000");
        distributeParent.setOpenid(SecurityContextHolder.getStringUserId());
        distributeParent.setImei(imei);
        distributeParent.setTime(System.currentTimeMillis());
        //3.调用方法缓存入redis
        issueInstructions.emitDeviceDemand("PowerOff",imei,distributeParent);
    }

    @Override
    public void sendDialSwitchToKafka(DialSwitchDto deviceOwnerInfoApp) {
        DialSwitch dialSwitch=new DialSwitch();
        dialSwitch.setType("dialSwitch");
        dialSwitch.setIdent("12345678");
        dialSwitch.setVender("20000");
        dialSwitch.setStatus(deviceOwnerInfoApp.getDialSwitch());
        dialSwitch.setOpenid(SecurityContextHolder.getStringUserId());
        dialSwitch.setImei(deviceOwnerInfoApp.getImei());
        dialSwitch.setTime(System.currentTimeMillis());
        //3.调用方法缓存入redis
        issueInstructions.emitDeviceDemand("dialSwitch",deviceOwnerInfoApp.getImei(),dialSwitch);
    }

    @Override
    public QueryOwnerInfoDto queryDeviceInfo(String imei) {

        //1.根据imei和账号获取用户与设备绑定信息
        LambdaQueryWrapper<UserDeviceBindApp> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(UserDeviceBindApp::getImei,imei);
        lambdaQueryWrapper.eq(UserDeviceBindApp::getUserId,SecurityContextHolder.getStringUserId());
        UserDeviceBindApp userDeviceBindApp = userDeviceBindAppMapper.selectOne(lambdaQueryWrapper);
        if (null==userDeviceBindApp){
            return new QueryOwnerInfoDto();
        }
        //2.根据iMei获取设备拥有者信息
        LambdaQueryWrapper<DeviceOwnerInfoApp> ownerInfoAppLambdaQueryWrapper=new LambdaQueryWrapper<>();
        ownerInfoAppLambdaQueryWrapper.eq(DeviceOwnerInfoApp::getImei,imei);
        DeviceOwnerInfoApp deviceOwnerInfoApp = deviceOwnerInfoAppMapper.selectOne(ownerInfoAppLambdaQueryWrapper);
        deviceOwnerInfoApp.setRelation(userDeviceBindApp.getRelation());
        //封装数据返回
        QueryOwnerInfoDto queryOwnerInfoDto=new QueryOwnerInfoDto();
        queryOwnerInfoDto.setBirthday(deviceOwnerInfoApp.getBirthday());
        queryOwnerInfoDto.setHeight(deviceOwnerInfoApp.getHeight());
        queryOwnerInfoDto.setImage(deviceOwnerInfoApp.getImage());
        queryOwnerInfoDto.setName(deviceOwnerInfoApp.getName());
        queryOwnerInfoDto.setPhone(deviceOwnerInfoApp.getPhone());
        queryOwnerInfoDto.setRelation(deviceOwnerInfoApp.getRelation());
        queryOwnerInfoDto.setSex(deviceOwnerInfoApp.getSex());
        queryOwnerInfoDto.setWeight(deviceOwnerInfoApp.getWeight());
        queryOwnerInfoDto.setIdentity(userDeviceBindApp.getIdentity());
        queryOwnerInfoDto.setRelationImageId(userDeviceBindApp.getRelationImageId());
        return queryOwnerInfoDto;
    }
}
