package com.ecell.internationalize.app.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ecell.internationalize.app.entity.DeviceDisableApp;
import com.ecell.internationalize.app.entity.DeviceVideoApp;
import com.ecell.internationalize.app.entity.DeviceVideoRecordApp;
import com.ecell.internationalize.app.entity.VideoModelDto;
import com.ecell.internationalize.app.entity.dto.DeviceVideoConfigDto;
import com.ecell.internationalize.app.entity.dto.DeviceVideoDto;
import com.ecell.internationalize.app.entity.po.VideoModelInfoPO;
import com.ecell.internationalize.app.mapper.*;
import com.ecell.internationalize.app.service.DeviceVideoAppService;
import com.ecell.internationalize.app.service.UserDeviceBindService;
import com.ecell.internationalize.app.util.TimeUtil;
import com.ecell.internationalize.app.util.VideoUtil;
import com.ecell.internationalize.common.core.context.SecurityContextHolder;
import com.ecell.internationalize.common.core.utils.StringUtils;
import com.ecell.internationalize.common.core.utils.bean.BeanUtils;
import com.ecell.internationalize.common.core.utils.uuid.UUID;
import com.ecell.internationalize.common.issue.component.IssueInstructions;
import com.ecell.internationalize.common.issue.constant.CommonConstant;
import com.ecell.internationalize.common.issue.entity.issuedto.BaiRuiVideo;
import com.ecell.internationalize.common.issue.entity.issuedto.DeleteVideoCall;
import com.ecell.internationalize.common.issue.entity.issuedto.DownVideoCallWithWatch;
import com.ecell.internationalize.common.issue.util.ApplicationUtil;
import com.ecell.internationalize.common.issue.util.RandomUtil;
import com.ecell.internationalize.common.redis.service.RedisService;
import com.ecell.internationalize.common.system.entity.DeviceOwnerInfoApp;
import com.ecell.internationalize.common.system.entity.UserDeviceBindApp;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.security.SecureRandom;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * <p>
 * 视频通话表 服务实现类
 * </p>
 *
 * @author ${author}
 * @since 2022-10-17
 */
@Service
public class DeviceVideoAppServiceImpl extends ServiceImpl<DeviceVideoAppMapper, DeviceVideoApp> implements DeviceVideoAppService {
    private static final Logger logger = LoggerFactory.getLogger(DeviceVideoAppServiceImpl.class);
    @Autowired
    private DeviceVideoAppMapper deviceVideoAppMapper;
    @Autowired
    private VideoCallAppMapper videoCallAppMapper;
    @Autowired
    private DeviceDisableAppMapper deviceDisableAppMapper;
    @Autowired
    private RedisService redisService;
    @Autowired
    private DeviceOwnerInfoAppMapper deviceOwnerInfoAppMapper;
    @Autowired
    private UserDeviceBindAppMapper userDeviceBindAppMapper;
    @Autowired
    private DeviceVideoRecordAppMapper deviceVideoRecordAppMapper;
    @Autowired
    private IssueInstructions issueInstructions;
    /**
     * @Title: DeviceVideoAppService
     * @Author: liy
     * @Date: 2022/10/18 10:47
     * @Description: 可视频通话列表查询
     * @Version:1.0
     */
    @Override
    public List<DeviceVideoDto> findVideoList(String imei) {
        DeviceVideoDto deviceVideoDto;
        List<DeviceVideoDto> deviceVideoDtoList=new ArrayList<>();
        //根据imei 查询 可视频列表
        LambdaQueryWrapper<DeviceVideoApp> deviceVideoAppLambdaQueryWrapper=new LambdaQueryWrapper<>();
        deviceVideoAppLambdaQueryWrapper.eq(DeviceVideoApp::getImei,imei);
        List<DeviceVideoApp> deviceVideoAppList=deviceVideoAppMapper.selectList(deviceVideoAppLambdaQueryWrapper);
        if (!CollectionUtils.isEmpty(deviceVideoAppList)){
            for (DeviceVideoApp deviceVideoApp:deviceVideoAppList){
                String key=imei+"__"+deviceVideoApp.getUserId();
                Map<String, UserDeviceBindApp> map=queryUserBindApp();
                if (null!=map.get(key)) {
                    String relationShip = map.get(key).getRelation();
                    Integer relationShipImageId = Integer.valueOf(map.get(key).getRelationImageId());
                    String phone = map.get(key).getPhone();
                    //内部list数据封装
                    deviceVideoDto = new DeviceVideoDto();
                    deviceVideoDto.setPhone(phone);
                    deviceVideoDto.setRelationShip(relationShip);
                    deviceVideoDto.setRelationShipImageId(relationShipImageId);
                    deviceVideoDtoList.add(deviceVideoDto);
                }
            }
        }
        return deviceVideoDtoList;
    }
    /**
     * @Title: DeviceVideoAppService
     * @Author: liy
     * @Date: 2022/10/18 10:47
     * @Description: 与设备进行通话
     * @Version:1.0
     */
    @Override
    public int videoCall(String imei, String callType,String type,String userId,String callDevice,String deviceVideoType,String uidd) {
        QueryWrapper<UserDeviceBindApp> wrapper =new QueryWrapper<>();
        wrapper.eq("imei",imei);
        wrapper.eq("user_id",userId);
        wrapper.eq("bind_status", CommonConstant.AUDIT_STATUS_ONE);
        //根据userId查询绑定表
        UserDeviceBindApp userDeviceBindApp = userDeviceBindAppMapper.selectOne(wrapper);

        DeviceVideoApp deviceVideoApp=null;
        logger.info("imei:{},发起视频通话的类型:{}(0:视频，1：语音)",imei,callType);
        String time=String.valueOf(System.currentTimeMillis());
        //1.获取视频配置信息
        Map<String, Object> videoConfig = videoConfig(imei,callType);
        String userBaiRuiId="";
        Integer baiRuiId;
        //获取设备登陆视频的id
        LambdaQueryWrapper<DeviceOwnerInfoApp> ownerInfoAppLambdaQueryWrapper=new LambdaQueryWrapper<>();
        ownerInfoAppLambdaQueryWrapper.eq(DeviceOwnerInfoApp::getImei,imei);
        DeviceOwnerInfoApp ownerInfoApp = deviceOwnerInfoAppMapper.selectOne(ownerInfoAppLambdaQueryWrapper);
        if (null!=ownerInfoApp.getBaiRuiId()){
            baiRuiId=ownerInfoApp.getBaiRuiId();
        }else {
            String s=time.substring(8,13);
            SecureRandom secureRandom=new SecureRandom();
            int i = secureRandom.nextInt(10000);
            String total=i+s;
            baiRuiId=Integer.parseInt(total);
            ownerInfoApp.setBaiRuiId(baiRuiId);
            deviceOwnerInfoAppMapper.updateById(ownerInfoApp);

        }
        //APP<==>手表
        if (null!=userId) {
            //2.根据imei和userId查询当前视频通话类型
            LambdaQueryWrapper<DeviceVideoApp> videoAppLambdaQueryWrapper = new LambdaQueryWrapper<>();
            videoAppLambdaQueryWrapper.eq(DeviceVideoApp::getUserId, userId);
            videoAppLambdaQueryWrapper.eq(DeviceVideoApp::getImei, imei);
             deviceVideoApp = deviceVideoAppMapper.selectOne(videoAppLambdaQueryWrapper);
            //判断当前设备的用户是否在圈子里面
            if (null == deviceVideoApp) {
                return -1;
            }
            //判断视频类型是否支持视频
            if (CommonConstant.VIDEO_TYPE_ZERO.equals(deviceVideoApp.getVideoType())) {
                return -2;
            }
            //转义视频类型
            deviceVideoType=CommonConstant.VIDEO_TYPE_MAP.get(deviceVideoApp.getVideoType());
            userBaiRuiId=String.valueOf(deviceVideoApp.getVideoId());

        }
        if (CommonConstant.DOWN_VIDEO_CALL_WITH_WATCH.equals(type)) {
            //判断设备是否在上课禁用时间段
           if (null!=userId) {
               if (disBanDevice(imei)) {
                   return -3;
               }
            //判断设备是否在线
               String onlineStatus = null != redisService.getCacheMapValue(imei, "onlineStatus")?redisService.getCacheMapValue(imei, "onlineStatus").toString():"0";
            if (CommonConstant.AUDIT_STATUS_ZERO.equals(onlineStatus)) {
                return 0;
            }
           }
        }
        //手表<==>手表
        if (null != callDevice) {
            //判断设备是否在上课禁用时间段
            if (disBanDevice(callDevice)) {
                return -3;
            }
            baiRuiId = Integer.parseInt(callDevice);
        }
        if (CommonConstant.AUDIT_STATUS_ZERO.equals(callType)){
            int nowTime=Integer.parseInt(time.substring(0,10));
            int callInterval=Integer.parseInt(videoConfig.get("callInterval").toString());
            /** 视频的话,进行限制操作,*/
//            DeviceData redisData = redisService.getCacheObject("imei:"+imei);
//            if (null!=redisData){
//                //上一次通话 结束的 时间点
//                int lastEndVideoTime = null!=redisData.getLastEndVideoTime()?redisData.getLastEndVideoTime():0;
//                 //是否限制设备视频通话
//                int videoLimitStatus =null!=redisData.getVideoLimitStatus()?redisData.getVideoLimitStatus():0;
//                if(nowTime - lastEndVideoTime >= callInterval) {
//                    logger.info("imei:{},与上次视频时间 间隔过大 发起通话前清除缓存限制,正常拨打 nowTime:{}, 上一次通话结束的时间lastEndVideoTime:{}, 配置的限制时间:{}",imei, nowTime, lastEndVideoTime, callInterval);
//                    DeviceData deviceData=new DeviceData();
//
//                    deviceData.setRVideoTime(0);
//                    deviceData.setRIntervalVideoTime(0);
//                    deviceData.setVideoLimitStatus(0);
//                    redisService.setCacheObject("imei:"+imei,deviceData);
//                    videoLimitStatus = 0;
//                }
//                if(1 == videoLimitStatus) {
//                    logger.warn("imei:{} 要等待一会再打,还需要多久:{} 秒" ,imei, callInterval - nowTime - lastEndVideoTime);
//                    return -4;
//                }
//            }
        }
        //新的视频/语音通话ID (视频或语音通话 共用一个)
        String newVideoId=imei+"-"+time.substring(0,10);
        //如果是 APP<==>手表 记录视频通话Id 到device_owner_info表,保存最新的一次视频通话的id
        if (null!=userId) {
            DeviceOwnerInfoApp deviceOwnerInfoApp = new DeviceOwnerInfoApp();
            deviceOwnerInfoApp.setImei(imei);
            deviceOwnerInfoApp.setNewVideoId(newVideoId);
            LambdaQueryWrapper<DeviceOwnerInfoApp> lambdaQueryWrapper = new LambdaQueryWrapper<>();
            lambdaQueryWrapper.eq(DeviceOwnerInfoApp::getImei, imei);
            deviceOwnerInfoAppMapper.update(deviceOwnerInfoApp,lambdaQueryWrapper);
        }
        String password=imei.substring(9);
        String userName="imei-"+imei;
        DownVideoCallWithWatch downVideoCallWithWatch = new DownVideoCallWithWatch();
        BaiRuiVideo baiRuiVideo = new BaiRuiVideo();
        //视频类型为菊风
        if (CommonConstant.VIDEO_TYPE_ONE.equals(deviceVideoType)){
            logger.info("APP获取菊风手表发起的视频通话的配置：{}",deviceVideoType);
            //菊风下发数据封装
            downVideoCallWithWatch=getDownVideoCallWatch(imei,"JF",CommonConstant.JF_VIDEO_KEY,userName,password,4,Integer.parseInt(videoConfig.get("width").toString()),
                    Integer.parseInt(videoConfig.get("height").toString()),newVideoId,Integer.parseInt(videoConfig.get("oneCallTime").toString()),userId,callDevice);
        }
        //视频类型为声网
        if (CommonConstant.VIDEO_TYPE_TWO.equals(deviceVideoType)){
            logger.info("APP获取声网手表发起的视频通话的配置：{}",deviceVideoType);
            //接收的手表端
            int uid = 1122;
            if (null!=uidd){
                uid =Integer.valueOf(uidd).intValue();
            }else {
                uid= RandomUtil.generateVerCodeInt(String.valueOf(System.currentTimeMillis()));
            }
           // int uid= RandomUtil.generateVerCodeInt(String.valueOf(System.currentTimeMillis()));
            //int uid =;
            //app 的 uid
           // int appUid = 2222;
            int appUid =deviceVideoApp.getVideoId();
            //发起端的uid (手表)
            int startUid = 3333;
            //获取token 所需的失效时间 0 是代表都有效
            Integer expireTimestamp = 0;
           String  contact = "mom";
            if (StringUtils.isNotNull(userDeviceBindApp)){
                contact=userDeviceBindApp.getRelation();
            }

            downVideoCallWithWatch.setImei(imei);
            downVideoCallWithWatch.setIdent("12345678");
            downVideoCallWithWatch.setOpenid(userId);
            downVideoCallWithWatch.setVender("20000");
            downVideoCallWithWatch.setTime(System.currentTimeMillis());
            downVideoCallWithWatch.setType("downVideoCallWithWatch");
            downVideoCallWithWatch.setDeviceVideoType("SW");
            downVideoCallWithWatch.setAppid(CommonConstant.SW_VIDEO_KEY);
            downVideoCallWithWatch.setUid(uid);
            downVideoCallWithWatch.setContact(contact);
            downVideoCallWithWatch.setChannelID(imei);
            downVideoCallWithWatch.setVideo_id(newVideoId);
            downVideoCallWithWatch.setDistinguishability_heiger(Integer.parseInt(videoConfig.get("height").toString()));
            downVideoCallWithWatch.setDistinguishability_width(Integer.parseInt(videoConfig.get("width").toString()));
            //保留参数
            downVideoCallWithWatch.setToken("");
            downVideoCallWithWatch.setLimit_time(Integer.parseInt(videoConfig.get("oneCallTime").toString()));
            downVideoCallWithWatch.setWait_time(CommonConstant.WAIT_TIME);
            /** 查询APP是否 和手表正在视频通话 (多个APP 打一个手表的情况) (多手表打一个用户的情况的暂不考虑),需要从声网获取用户状态*/
            if (null!=userId) {
                boolean status = VideoUtil.queryAgoraStatus(imei, appUid);
                if (status){
                    return -5;
                }
                downVideoCallWithWatch.setApp_uid(appUid);
            }
            if (null!=callDevice){
                boolean status = VideoUtil.queryAgoraStatus(imei, startUid);
                if (status){
                    return -5;
                }
                downVideoCallWithWatch.setStart_uid(startUid);
            }
        }
        //视频类型为佰锐
        if (CommonConstant.VIDEO_TYPE_THREE.equals(deviceVideoType)){
            logger.info("APP获取佰锐手表发起的视频通话的配置：{}",deviceVideoType);
            password="123456";
            //佰锐下发数据封装
             baiRuiVideo=getBaiRuiVideo(userBaiRuiId,imei,"BR",CommonConstant.BR_VIDEO_KEY,baiRuiId,password,5,Integer.parseInt(videoConfig.get("width").toString()),
                    Integer.parseInt(videoConfig.get("height").toString()),newVideoId,Integer.parseInt(videoConfig.get("oneCallTime").toString()), userId,callDevice);
        }
        //视频类型为鹈鹕
        if (CommonConstant.VIDEO_TYPE_FOUR.equals(deviceVideoType)){
            logger.info("APP获取鹈鹕手表发起的视频通话的配置：{}",deviceVideoType);
            //鹈鹕下发数据封装
            downVideoCallWithWatch=getDownVideoCallWatch(imei,"TH",CommonConstant.TH_VIDEO_KEY,userName,password,6,Integer.parseInt(videoConfig.get("width").toString()),
                    Integer.parseInt(videoConfig.get("height").toString()),newVideoId,Integer.parseInt(videoConfig.get("oneCallTime").toString()),userId,callDevice);
        }
        //App/手表<==>手表视频通话
        //发起视频成功后,存入视频通话记录表
        DeviceVideoRecordApp deviceVideoRecordApp = new DeviceVideoRecordApp();
        deviceVideoRecordApp.setId(com.ecell.internationalize.common.core.utils.uuid.UUID.fastUUID().toString(true));
        deviceVideoRecordApp.setVideoId(newVideoId);
        deviceVideoRecordApp.setStatus("1");
        deviceVideoRecordApp.setImei(imei);
        deviceVideoRecordApp.setStartTime(time.substring(0,10));
        deviceVideoRecordApp.setCallType(callType);
        deviceVideoRecordApp.setCreateTime(new Date());
        if (CommonConstant.DOWN_VIDEO_CALL_WITH_WATCH.equals(type)) {
            //APP<==>手表
            if (null!=userId) {
                //记录最近一次通话的用户,用于 手表拒绝视频通话时的
                redisService.setCacheMapValue(imei, "video_openid", userId);
                deviceVideoRecordApp.setUserId(userId);
            }
            //手表<==>手表
            else {
                deviceVideoRecordApp.setUserId(callDevice);
            }
            deviceVideoRecordApp.setRequestType(CommonConstant.DOWN_VIDEO_CALL_WITH_WATCH);
            deviceVideoRecordAppMapper.insert(deviceVideoRecordApp);
            logger.info("用户userId:{},发起通话请求成功,通话类型:{}", userId, callType);
                //开始下发指令
            if (CommonConstant.VIDEO_TYPE_THREE.equals(deviceVideoType)) {
                issueInstructions.emitDeviceDemand("downVideoCallWithWatch", imei, baiRuiVideo);
            } else {
                issueInstructions.emitDeviceDemand("downVideoCallWithWatch", imei, downVideoCallWithWatch);
            }
        }
        //手表<==>App/手表 手表打给APP
        if (CommonConstant.UP_VIDEO_CALL_WITH_APP.equals(type)){
            deviceVideoRecordApp.setRequestType(CommonConstant.UP_VIDEO_CALL_WITH_APP);
            //手表<==>APP
            if (null!=userId){
                //记录的是被拨打方的id
                deviceVideoRecordApp.setUserId(userId);
                deviceVideoRecordAppMapper.insert(deviceVideoRecordApp);
            }
            //手表<==>手表
            if (null!=callDevice){
                deviceVideoRecordApp.setUserId(callDevice);
                deviceVideoRecordAppMapper.insert(deviceVideoRecordApp);
                //这个是手表对手表的upVideoCallWithAPP 还得下发一个downVideoCallWithWatch给 被拨打的手表
                //开始下发被拨打的手表的指令
                if (CommonConstant.VIDEO_TYPE_THREE.equals(deviceVideoType)) {
                    issueInstructions.emitDeviceDemand("downVideoCallWithWatch", callDevice, baiRuiVideo);
                } else {
                    downVideoCallWithWatch.setUsername("imei-"+callDevice);
                    downVideoCallWithWatch.setPassword(callDevice.substring(9));
                    issueInstructions.emitDeviceDemand("downVideoCallWithWatch", callDevice, downVideoCallWithWatch);
                }
            }
             //回复原状
            downVideoCallWithWatch.setUsername(userName);
            downVideoCallWithWatch.setPassword(password);
            //这个是回复发起的手表的视频配置
            //回复手表 需要换一个 类型 downVideoCallWithAPPInfo 手表发起和APP的视频通话 所需要的视频信息配置
            logger.info("视频记录插入成功");
            if (CommonConstant.VIDEO_TYPE_THREE.equals(deviceVideoType)) {
                baiRuiVideo.setType("downVideoCallWithAPPInfo");
                issueInstructions.emitDeviceDemand("downVideoCallWithAPPInfo", imei, baiRuiVideo);
            }else {
                downVideoCallWithWatch.setType("downVideoCallWithAPPInfo");
                issueInstructions.emitDeviceDemand("downVideoCallWithAPPInfo", imei, downVideoCallWithWatch);
            }
            logger.info("手表发起===>视频通话给APP用户:{},手表:{},成功",userId,callDevice);
        }
        return 1;
    }



    @Override
    public int videoCallDevice(String imei, String callType, String type, String userId, String callDevice, String deviceVideoType, String uidd) {
        DeviceVideoApp deviceVideoApp=null;
        logger.info("imei:{},发起视频通话的类型:{}(0:视频，1：语音)",imei,callType);
        String time=String.valueOf(System.currentTimeMillis());
        //1.获取视频配置信息
        Map<String, Object> videoConfig = videoConfig(imei,callType);
        logger.info("获取到的配置是:{}",videoConfig);
//        if ("UPVCALLDEV".equals(type)) {
//            //判断设备是否在上课禁用时间段
//            if (null!=userId) {
//                if (disBanDevice(userId)) {
//                    return -3;
//                }
//                //判断设备是否在线
//                String onlineStatus = null != redisService.getCacheMapValue(imei, "onlineStatus")?redisService.getCacheMapValue(userId, "onlineStatus").toString():"0";
//                if (CommonConstant.AUDIT_STATUS_ZERO.equals(onlineStatus)) {
//                    return 0;
//                }
//            }
//        }
//        //手表<==>手表
//        if (null != callDevice) {
//            //判断设备是否在上课禁用时间段
//            if (disBanDevice(callDevice)) {
//                return -3;
//            }
//        }

        //2.根据imei和userId查询当前视频通话类型
        LambdaQueryWrapper<DeviceVideoApp> videoAppLambdaQueryWrapper = new LambdaQueryWrapper<>();
        videoAppLambdaQueryWrapper.eq(DeviceVideoApp::getUserId, userId);
        videoAppLambdaQueryWrapper.eq(DeviceVideoApp::getImei, imei);
        deviceVideoApp = deviceVideoAppMapper.selectOne(videoAppLambdaQueryWrapper);


        //视频通话
        if (CommonConstant.AUDIT_STATUS_ZERO.equals(callType)){
            int nowTime=Integer.parseInt(time.substring(0,10));
            int callInterval=Integer.parseInt(videoConfig.get("callInterval").toString());
        }
        //新的视频/语音通话ID (视频或语音通话 共用一个)
        String newVideoId=imei+"-"+time.substring(0,10);
        //如果是 APP<==>手表 记录视频通话Id 到device_owner_info表,保存最新的一次视频通话的id
        if (null!=userId) {
            DeviceOwnerInfoApp deviceOwnerInfoApp = new DeviceOwnerInfoApp();
            deviceOwnerInfoApp.setImei(imei);
            deviceOwnerInfoApp.setNewVideoId(newVideoId);
            LambdaQueryWrapper<DeviceOwnerInfoApp> lambdaQueryWrapper = new LambdaQueryWrapper<>();
            lambdaQueryWrapper.eq(DeviceOwnerInfoApp::getImei, imei);
            deviceOwnerInfoAppMapper.update(deviceOwnerInfoApp,lambdaQueryWrapper);
        }
        String password=imei.substring(9);
        String userName="imei-"+imei;
        DownVideoCallWithWatch downVideoCallWithWatch = new DownVideoCallWithWatch();
        DownVideoCallWithWatch downVideoCallWithWatchCall = new DownVideoCallWithWatch();
        //视频类型为声网
            logger.info("手表获取声网手表发起的视频通话的配置：{}",deviceVideoType);
            //接收的手表端
            int uid = 3344;
            if (null!=uidd){
                uid =Integer.valueOf(uidd).intValue();
            }else {
                uid= RandomUtil.generateVerCodeInt(String.valueOf(System.currentTimeMillis()));
            }
        logger.info("开始插入========{}",uid);
//            int appUid =deviceVideoApp.getVideoId();
//
//            //发起端的uid (手表)
//            int startUid = 4444;
//            //获取token 所需的失效时间 0 是代表都有效
//            Integer expireTimestamp = 0;
            String  contact = "mom";
            downVideoCallWithWatch.setImei(imei);
            downVideoCallWithWatch.setIdent("12345678");
            downVideoCallWithWatch.setOpenid(userId);
            downVideoCallWithWatch.setVender("20000");
            downVideoCallWithWatch.setTime(System.currentTimeMillis());
            downVideoCallWithWatch.setType("UPVCALLDEV");
            downVideoCallWithWatch.setDeviceVideoType("SW");
            downVideoCallWithWatch.setAppid(CommonConstant.SW_VIDEO_KEY);
            downVideoCallWithWatch.setUid(uid);
            downVideoCallWithWatch.setContact(contact);
            downVideoCallWithWatch.setChannelID(imei);
            downVideoCallWithWatch.setVideo_id(newVideoId);
        logger.info("开始获取111========{}");
            downVideoCallWithWatch.setDistinguishability_heiger(Integer.parseInt(videoConfig.get("height").toString()));
            downVideoCallWithWatch.setDistinguishability_width(Integer.parseInt(videoConfig.get("width").toString()));
        logger.info("开始获取22========{}");
            //保留参数
            downVideoCallWithWatch.setToken("");
            downVideoCallWithWatch.setLimit_time(Integer.parseInt(videoConfig.get("oneCallTime").toString()));
            downVideoCallWithWatch.setWait_time(CommonConstant.WAIT_TIME);
            /** 查询APP是否 和手表正在视频通话 (多个APP 打一个手表的情况) (多手表打一个用户的情况的暂不考虑),需要从声网获取用户状态*/
        logger.info("记录下发========{}",downVideoCallWithWatch);
        //发起视频成功后,存入视频通话记录表
        DeviceVideoRecordApp deviceVideoRecordApp = new DeviceVideoRecordApp();
        deviceVideoRecordApp.setId(UUID.fastUUID().toString(true));
        deviceVideoRecordApp.setVideoId(newVideoId);
        deviceVideoRecordApp.setStatus("1");
        deviceVideoRecordApp.setImei(imei);
        deviceVideoRecordApp.setStartTime(time.substring(0,10));
        deviceVideoRecordApp.setCallType(callType);
        deviceVideoRecordApp.setCreateTime(new Date());
        //打给对方的imei
        deviceVideoRecordApp.setUserId(callDevice);
        deviceVideoRecordApp.setRequestType("UPVCALLDEV");

        logger.info("开始插入========{}",deviceVideoRecordApp);
            try {
                logger.info("开始插入========");
                deviceVideoRecordAppMapper.insert(deviceVideoRecordApp);
                logger.info("发起方imei:{},接收方:{}通话类型:{}", imei, userId, callType);
                //开始下发指令 这里下发
                BeanUtils.copyBeanProp(downVideoCallWithWatchCall,downVideoCallWithWatch);
                downVideoCallWithWatchCall.setImei(userId);
                issueInstructions.emitDeviceDemand("UPVCALLDEV", imei, downVideoCallWithWatch);
                issueInstructions.emitDeviceDemand("UPVCALLDEV", userId, downVideoCallWithWatchCall);
                logger.info("设备imei:{},发起通话请求成功,通话类型:{}", imei, callType);
            }catch (Exception e){
                logger.info("手表跟手表出现异常"+e.getMessage());
            }

        return 1;
    }

    /**
     * @Title: DeviceVideoAppService
     * @Author: liy
     * @Date: 2022/10/18 10:47
     * @Description: APP发起视频/语音通话后主动挂断(在手表还未接到响铃的情况下)
     * @Version:1.0
     */
    @Override
    public void deleteCall(String imei, String videoId) {
        DeleteVideoCall deleteVideoCall=new DeleteVideoCall();
        deleteVideoCall.setIdent("12345678");
        deleteVideoCall.setImei(imei);
        deleteVideoCall.setOpenid(SecurityContextHolder.getStringUserId());
        deleteVideoCall.setVender("20000");
        deleteVideoCall.setType("downAPPHangUp");
        deleteVideoCall.setTime(System.currentTimeMillis());
        deleteVideoCall.setVideo_id(videoId);
        //3.下发指令之前先存入redis,开始下发指令
        issueInstructions.emitDeviceDemand("downAPPHangUp",imei,deleteVideoCall);
        LambdaQueryWrapper<DeviceOwnerInfoApp> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(DeviceOwnerInfoApp::getImei,imei);
        DeviceOwnerInfoApp deviceOwnerInfoApp = deviceOwnerInfoAppMapper.selectOne(lambdaQueryWrapper);
        if (null!=deviceOwnerInfoApp && null!=deviceOwnerInfoApp.getNewVideoId()){
            //清除最新通话newVideoId
            deviceOwnerInfoApp.setNewVideoId("");
            deviceOwnerInfoAppMapper.updateById(deviceOwnerInfoApp);
        }
    }
    /**
     * @Title: DeviceVideoAppService
     * @Author: liy
     * @Date: 2022/10/18 10:47
     * @Description: 返回视频id，参数配置(分辨率、通话时长限制等)信息
     * @Version:1.0
     */
    @Override
    public DeviceVideoConfigDto userQueryVideo(String imei, String callType,String type) {
        DeviceVideoConfigDto deviceVideoConfigDto = new DeviceVideoConfigDto();
        //1.获取视频配置信息
        Map<String, Object> videoConfig = videoConfig(imei, callType);
        deviceVideoConfigDto.setAllCallTime(Integer.parseInt(videoConfig.get("allCallTime").toString()));
        deviceVideoConfigDto.setOneCallTime(Integer.parseInt(videoConfig.get("oneCallTime").toString()));
        deviceVideoConfigDto.setCallInterval(Integer.parseInt(videoConfig.get("callInterval").toString()));
        deviceVideoConfigDto.setDisplayResolution(videoConfig.get("displayResolution").toString());
        deviceVideoConfigDto.setHeight(Integer.parseInt(videoConfig.get("height").toString()));
        deviceVideoConfigDto.setWidth(Integer.parseInt(videoConfig.get("width").toString()));

        if ("APP".equals(type)) {
            Integer baiRuiId = null;
            LambdaQueryWrapper<DeviceVideoApp> lambdaQueryWrapper = new LambdaQueryWrapper<>();
            lambdaQueryWrapper.eq(DeviceVideoApp::getImei, imei);
            lambdaQueryWrapper.eq(DeviceVideoApp::getUserId, SecurityContextHolder.getStringUserId());
            DeviceVideoApp deviceVideoApp = deviceVideoAppMapper.selectOne(lambdaQueryWrapper);
            if (null == deviceVideoApp) {
                return null;
            }
            LambdaQueryWrapper<DeviceOwnerInfoApp> ownerInfoAppLambdaQueryWrapper = new LambdaQueryWrapper<>();
            ownerInfoAppLambdaQueryWrapper.eq(DeviceOwnerInfoApp::getImei, imei);
            DeviceOwnerInfoApp deviceOwnerInfoApp = deviceOwnerInfoAppMapper.selectOne(ownerInfoAppLambdaQueryWrapper);
            if (null != deviceOwnerInfoApp.getBaiRuiId()) {
                baiRuiId = deviceOwnerInfoApp.getBaiRuiId();
            } else {
                String time = String.valueOf(System.currentTimeMillis());
                String s = time.substring(8, 13);
                SecureRandom secureRandom = new SecureRandom();
                int i = secureRandom.nextInt(10000);
                String total = i + s;
                baiRuiId = Integer.parseInt(total);
                deviceOwnerInfoApp.setBaiRuiId(baiRuiId);
                deviceOwnerInfoAppMapper.updateById(deviceOwnerInfoApp);

            }
            deviceVideoConfigDto.setVideoId(deviceVideoApp.getVideoId());
            deviceVideoConfigDto.setImei(deviceVideoApp.getImei());
            deviceVideoConfigDto.setUserId(deviceVideoApp.getUserId());
            deviceVideoConfigDto.setVideoType(deviceVideoApp.getVideoType());
            if (CommonConstant.AUDIT_STATUS_ONE.equals(deviceVideoApp.getVideoType())) {
                deviceVideoConfigDto.setAppKey(CommonConstant.JF_VIDEO_KEY);
                deviceVideoConfigDto.setDeviceUserName("imei-" + imei);
            }
            if (CommonConstant.AUDIT_STATUS_TWO.equals(deviceVideoApp.getVideoType())) {
                deviceVideoConfigDto.setAppKey(CommonConstant.SW_VIDEO_KEY);

            }
            if (CommonConstant.AUDIT_STATUS_THREE.equals(deviceVideoApp.getVideoType())) {
                deviceVideoConfigDto.setAppKey(CommonConstant.BR_VIDEO_KEY);
                deviceVideoConfigDto.setDeviceUserName(String.valueOf(baiRuiId));
            }
            if (CommonConstant.AUDIT_STATUS_FOUR.equals(deviceVideoApp.getVideoType())) {
                deviceVideoConfigDto.setAppKey(CommonConstant.TH_VIDEO_KEY);
                deviceVideoConfigDto.setDeviceUserName("imei-" + imei);
            }
        }
        //手表发起通话后获取配置信息
        else {
            String deviceRecordId="";
            Integer deviceBaiRuiId = null;
            LambdaQueryWrapper<DeviceOwnerInfoApp> ownerInfoAppLambdaQueryWrapper = new LambdaQueryWrapper<>();
            ownerInfoAppLambdaQueryWrapper.eq(DeviceOwnerInfoApp::getImei, imei);
            DeviceOwnerInfoApp deviceOwnerInfoApp = deviceOwnerInfoAppMapper.selectOne(ownerInfoAppLambdaQueryWrapper);
            deviceVideoConfigDto.setImei(imei);
            if (null != deviceOwnerInfoApp) {
                deviceRecordId = deviceOwnerInfoApp.getNewVideoId();
                deviceBaiRuiId=deviceOwnerInfoApp.getBaiRuiId();
            }
            deviceVideoConfigDto.setDeviceUserName(String.valueOf(deviceBaiRuiId));
            deviceVideoConfigDto.setDeviceVideoId(deviceRecordId);
            //查询设备视频类型
            VideoModelInfoPO videoByImei = videoCallAppMapper.getVideoByImei(imei);
            if (null == videoByImei){
                return null;
            }
            logger.info("手表查询的设备类型为：{}",videoByImei.getVideoModelType());
            if (CommonConstant.AUDIT_STATUS_ONE.equals(videoByImei.getVideoModelType())) {
                deviceVideoConfigDto.setDeviceUserName("imei-" + imei);
                // 2022-11-25 我加的
//                deviceVideoConfigDto.setImei(imei);
            }
            if (CommonConstant.AUDIT_STATUS_TWO.equals(videoByImei.getVideoModelType())) {
            }
            if (CommonConstant.AUDIT_STATUS_THREE.equals(videoByImei.getVideoModelType())) {
//                deviceVideoConfigDto.setDeviceUserName(String.valueOf(deviceBaiRuiId));
//                deviceVideoConfigDto.setDeviceVideoId(deviceRecordId);
            }
            if (CommonConstant.AUDIT_STATUS_FOUR.equals(videoByImei.getVideoModelType())) {
                deviceVideoConfigDto.setDeviceUserName("imei-" + imei);
            }
        }
        return deviceVideoConfigDto;
    }

    @Override
    public void hangUpCall(String imei, String videoId) {
        DeleteVideoCall deleteVideoCall=new DeleteVideoCall();
        deleteVideoCall.setIdent("12345678");
        deleteVideoCall.setImei(imei);
        deleteVideoCall.setOpenid(SecurityContextHolder.getStringUserId());
        deleteVideoCall.setVender("20000");
        deleteVideoCall.setType("downAPPHangUp");
        deleteVideoCall.setTime(System.currentTimeMillis());
        deleteVideoCall.setVideo_id(videoId);

        LambdaQueryWrapper<DeviceOwnerInfoApp> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(DeviceOwnerInfoApp::getImei,imei);
        DeviceOwnerInfoApp deviceOwnerInfoApp = deviceOwnerInfoAppMapper.selectOne(lambdaQueryWrapper);
        if (null!=deviceOwnerInfoApp && null!=deviceOwnerInfoApp.getNewVideoId()){
            deleteVideoCall.setVideo_id(deviceOwnerInfoApp.getNewVideoId());
            //清除最新通话newVideoId
            deviceOwnerInfoApp.setNewVideoId("");
            deviceOwnerInfoAppMapper.updateById(deviceOwnerInfoApp);
        }
        //3.下发指令之前先存入redis,开始下发指令
        issueInstructions.emitDeviceDemand("downAPPHangUp",imei,deleteVideoCall);
    }


    /**
     * 查询绑定表的数据用于可视频用户下发
     * */
    public Map<String, UserDeviceBindApp> queryUserBindApp(){
        Map<String,UserDeviceBindApp> map  =new HashMap<>(32);
        List<UserDeviceBindApp> userDeviceBindAppList = ApplicationUtil.getApplicationContext().getBean(UserDeviceBindService.class).getBaseMapper().selectList(null);
        if (CollectionUtils.isEmpty(userDeviceBindAppList)){
            return new HashMap<>(32);
        }
        for (UserDeviceBindApp userDeviceBindApp:userDeviceBindAppList){
            map.put(userDeviceBindApp.getImei()+"__"+userDeviceBindApp.getUserId(),userDeviceBindApp);
        }
        return map;
    }
    /**
     * 视频配置信息
     * */
    public Map<String,Object> videoConfig(String imei,String callType){
        //分辨率 宽*高
        String displayResolution=CommonConstant.DISPLAY_RESOLUTION;
        //单次通话时长 / 秒
        Integer oneCallTime = CommonConstant.ONE_CALL_TIME;
        //通话间隔 / 秒
        Integer callInterval = CommonConstant.CALL_INTERVAL;
        //限制的总视频时长 (在累计间隔时间内达到限制总视频时长的就限制)
        Integer allCallTime = CommonConstant.ALL_CALL_TIME;
        String videoType="";
        //1.获取视频配置信息,为空则使用默认配置
        VideoModelDto videoByImei = videoCallAppMapper.findVideoByImei(imei);
        if (null != videoByImei){
            displayResolution=videoByImei.getResolvingPower();
            oneCallTime=videoByImei.getSingleCall();
            callInterval=videoByImei.getCallInterval();
            allCallTime=videoByImei.getTotalVideo();
            videoType=videoByImei.getVideoModelType();
        }
        if (CommonConstant.AUDIT_STATUS_ONE.equals(callType)){
            // 语音 的话 单次通话时长不限制 (10小时)
            oneCallTime=10 * 60 * 60;
        }
        int width =Integer.parseInt(displayResolution.split("\\*")[0]);
        int height = Integer.parseInt(displayResolution.split("\\*")[1]);
        Map<String,Object> map=new HashMap<>(8);
        map.put("displayResolution",displayResolution);
        map.put("oneCallTime",oneCallTime);
        map.put("callInterval",callInterval);
        map.put("allCallTime",allCallTime);
        map.put("width",width);
        map.put("height",height);
        map.put("videoType",videoType);
        return map;
    }
    /**
     * 判断设备是否在上课禁用时间段
     * */
    public Boolean disBanDevice(String imei){
        LambdaQueryWrapper<DeviceDisableApp> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(DeviceDisableApp::getImei,imei);
        List<DeviceDisableApp> deviceDisableAppList = deviceDisableAppMapper.selectList(lambdaQueryWrapper);
        //存在上课禁用设置
        if (!CollectionUtils.isEmpty(deviceDisableAppList)){
            for (DeviceDisableApp deviceDisableApp:deviceDisableAppList){
                //获取今天是星期几
                String day = TimeUtil.time();
                //1.当前星期在设置的相应区间内
                if (TimeUtil.getDays(deviceDisableApp.getAlarmWeek(),day)){
                    //获取当前几点几分
                    Date date=new Date();
                    SimpleDateFormat simpleDateFormat=new SimpleDateFormat("HH:mm");
                    String time=simpleDateFormat.format(date);
                    //起始时间
                    String[] startTime=deviceDisableApp.getAlarmStartTime().split(":");
                    //当前时间
                    String[] nowTime=time.split(":");
                    //结束时间
                    String[] endTime=deviceDisableApp.getAlarmEndTime().split(":");
                    //起始时间小时数
                    String startHour=startTime[0];
                    //起始时间分钟数
                    String startMinute=startTime[1];
                    //结束时间小时数
                    String endHour=endTime[0];
                    //结束时间分钟数
                    String endMinute=endTime[1];
                    //当前时间小时数
                    String nowHour=nowTime[0];
                    //起始时间分钟数
                    String nowMinute=nowTime[1];
                    int startTotal=Integer.parseInt(startHour)*3600+Integer.parseInt(startMinute)*60;
                    int endTotal=Integer.parseInt(endHour)*3600+Integer.parseInt(endMinute)*60;
                    int nowTotal=Integer.parseInt(nowHour)*3600+Integer.parseInt(nowMinute)*60;
                    //2.当前时间在设置的时间点内
                    if (nowTotal>=startTotal && nowTotal<=endTotal){
                        if ("1".equals(deviceDisableApp.getAlarmStatus())) {
                            logger.info("imei:{},在上课禁用时间段内,不能进行视频通话", imei);
                            return true;
                        }
                    }
                }
            }
        }
        return false;
    }
    /**
     * APP发起视频通话下发数据封装
     * @param deviceVideoType 视频类型
     * @param appKey 视频的key
     * @param  username 设备登录的用户名
     * @param password 设备登录的密码
     * @param videoType 视频数字类型
     * @param width 分辨率的宽
     * @param height 分辨率的高
     * @param newVideoId 最新通话的视频id
     * @param limitTime 限制视频通话时长（s）
     */
    public DownVideoCallWithWatch getDownVideoCallWatch(String imei,String deviceVideoType,String appKey,String username,String password,Integer videoType,Integer width,Integer height,String newVideoId,Integer limitTime,String userId,String callDevice){
        //APP<==>手表
        String appUsername = "";
        String relationship = "";
        String relationshipImageId = "";
        DownVideoCallWithWatch downVideoCallWithWatch=new DownVideoCallWithWatch();
        //APP<==>手表
        if (null!=userId) {
            //根据imei号和userId查询绑定表的关系和头像
            LambdaQueryWrapper<UserDeviceBindApp> lambdaQueryWrapper = new LambdaQueryWrapper<>();
            lambdaQueryWrapper.eq(UserDeviceBindApp::getUserId, userId);
            lambdaQueryWrapper.eq(UserDeviceBindApp::getImei, imei);
            UserDeviceBindApp userDeviceBindApp = userDeviceBindAppMapper.selectOne(lambdaQueryWrapper);
            appUsername=userId;
            if (null != userDeviceBindApp) {
                relationship = null!=userDeviceBindApp.getRelation()?userDeviceBindApp.getRelation():"";
                relationshipImageId = null!=userDeviceBindApp.getRelationImageId()?userDeviceBindApp.getRelationImageId():"9";
            }
        }
        //手表<==>手表
        if (null!=callDevice){
            downVideoCallWithWatch.setDev_username("imei-"+callDevice);
            //根据callDevice(imei)查找设备拥有者信息
            LambdaQueryWrapper<DeviceOwnerInfoApp> ownerInfoAppLambdaQueryWrapper = new LambdaQueryWrapper<>();
            ownerInfoAppLambdaQueryWrapper.eq(DeviceOwnerInfoApp::getImei, callDevice);
            DeviceOwnerInfoApp deviceOwnerInfoApp = deviceOwnerInfoAppMapper.selectOne(ownerInfoAppLambdaQueryWrapper);
            if (null!=deviceOwnerInfoApp) {
                relationship = deviceOwnerInfoApp.getName();
                relationshipImageId="-1";
            }
            downVideoCallWithWatch.setDev_username("imei-"+callDevice);
        }

        downVideoCallWithWatch.setImei(imei);
        downVideoCallWithWatch.setIdent("12345678");
        downVideoCallWithWatch.setOpenid(userId);
        downVideoCallWithWatch.setVender("20000");
        downVideoCallWithWatch.setTime(System.currentTimeMillis());
        downVideoCallWithWatch.setDeviceVideoType(deviceVideoType);
        downVideoCallWithWatch.setAppkey(appKey);
        downVideoCallWithWatch.setUsername(username);
        downVideoCallWithWatch.setPassword(password);
        downVideoCallWithWatch.setType("downVideoCallWithWatch");
        downVideoCallWithWatch.setVideo_type(videoType);
        downVideoCallWithWatch.setDistinguishability_width(width);
        downVideoCallWithWatch.setDistinguishability_heiger(height);
        downVideoCallWithWatch.setVideo_id(newVideoId);
        downVideoCallWithWatch.setLimit_time(limitTime);
        downVideoCallWithWatch.setWait_time(CommonConstant.WAIT_TIME);
        downVideoCallWithWatch.setApp_username(appUsername);
        downVideoCallWithWatch.setRelationship(relationship);
        downVideoCallWithWatch.setRelationship_image_id(relationshipImageId);
       return downVideoCallWithWatch;

    }
    /**
     * 佰锐视频下发数据封装
     * @param deviceVideoType 视频类型
     * @param appKey 视频的key
     * @param  username 设备登录的用户名
     * @param password 设备登录的密码
     * @param videoType 视频数字类型
     * @param width 分辨率的宽
     * @param height 分辨率的高
     * @param newVideoId 最新通话的视频id
     * @param limitTime 限制视频通话时长（s）
     */
    public BaiRuiVideo getBaiRuiVideo(String userBaiRuiId,String imei,String deviceVideoType,String appKey,Integer username,String password,Integer videoType,Integer width,Integer height,String newVideoId,Integer limitTime,String userId,String callDevice){
        String relationship = "";
        String relationshipImageId = "";
        BaiRuiVideo baiRuiVideo=new BaiRuiVideo();
        if (null!=userId) {
            //根据imei号和userId查询绑定表的关系和头像
            LambdaQueryWrapper<UserDeviceBindApp> lambdaQueryWrapper = new LambdaQueryWrapper<>();
            lambdaQueryWrapper.eq(UserDeviceBindApp::getUserId, userId);
            lambdaQueryWrapper.eq(UserDeviceBindApp::getImei, imei);
            UserDeviceBindApp userDeviceBindApp = userDeviceBindAppMapper.selectOne(lambdaQueryWrapper);
            if (null != userDeviceBindApp) {
                relationship = userDeviceBindApp.getRelation();
                relationshipImageId = userDeviceBindApp.getRelationImageId();
            }
        }
        //手表<==>手表
        if (null!=callDevice){
            //根据callDevice(imei)查找设备拥有者信息
            LambdaQueryWrapper<DeviceOwnerInfoApp> ownerInfoAppLambdaQueryWrapper = new LambdaQueryWrapper<>();
            ownerInfoAppLambdaQueryWrapper.eq(DeviceOwnerInfoApp::getImei, callDevice);
            DeviceOwnerInfoApp deviceOwnerInfoApp = deviceOwnerInfoAppMapper.selectOne(ownerInfoAppLambdaQueryWrapper);
            if (null!=deviceOwnerInfoApp) {
                relationship = deviceOwnerInfoApp.getName();
                relationshipImageId="-1";
            }
            baiRuiVideo.setDev_username(username);
        }
        baiRuiVideo.setImei(imei);
        baiRuiVideo.setIdent("12345678");
        baiRuiVideo.setOpenid(userId);
        baiRuiVideo.setVender("20000");
        baiRuiVideo.setTime(System.currentTimeMillis());
        baiRuiVideo.setDeviceVideoType(deviceVideoType);
        baiRuiVideo.setAppkey(appKey);
        baiRuiVideo.setUsername(username);
        baiRuiVideo.setPassword(password);
        baiRuiVideo.setType("downVideoCallWithWatch");
        baiRuiVideo.setVideo_type(videoType);
        baiRuiVideo.setDistinguishability_heiger(height);
        baiRuiVideo.setDistinguishability_width(width);
        baiRuiVideo.setVideo_id(newVideoId);
        baiRuiVideo.setLimit_time(limitTime);
        baiRuiVideo.setWait_time(CommonConstant.WAIT_TIME);
        baiRuiVideo.setApp_username(userBaiRuiId);
        baiRuiVideo.setRelationship(relationship);
        baiRuiVideo.setRelationship_image_id(relationshipImageId);
        return baiRuiVideo;
    }
}
