package com.ecell.internationalize.app.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ecell.internationalize.app.entity.DeviceVideoRecordApp;
import com.ecell.internationalize.app.entity.dto.DeviceVideoRecordDto;
import com.ecell.internationalize.app.entity.dto.DeviceVideoRecordQuery;
import com.ecell.internationalize.app.mapper.DeviceInfoMapper;
import com.ecell.internationalize.app.mapper.DeviceOwnerInfoAppMapper;
import com.ecell.internationalize.app.mapper.DeviceVideoRecordAppMapper;
import com.ecell.internationalize.app.service.DeviceVideoRecordAppService;
import com.ecell.internationalize.common.core.context.SecurityContextHolder;
import com.ecell.internationalize.common.issue.constant.CommonConstant;
import com.ecell.internationalize.common.issue.entity.issuedto.DeviceData;
import com.ecell.internationalize.common.redis.service.RedisService;
import com.ecell.internationalize.common.system.entity.DeviceOwnerInfoApp;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Map;

/**
 * <p>
 * 视频通话记录 服务实现类
 * </p>
 *
 * @author ${author}
 * @since 2022-10-17
 */
@Slf4j
@Service
public class DeviceVideoRecordAppServiceImpl extends ServiceImpl<DeviceVideoRecordAppMapper, DeviceVideoRecordApp> implements DeviceVideoRecordAppService {
    private static final Logger logger = LoggerFactory.getLogger(DeviceVideoRecordAppServiceImpl.class);
    @Autowired
    private DeviceVideoRecordAppMapper deviceVideoRecordAppMapper;
    @Autowired
    private DeviceOwnerInfoAppMapper deviceOwnerInfoAppMapper;
    @Autowired
    private DeviceInfoMapper deviceInfoMapper;
    @Autowired
    private DeviceVideoAppServiceImpl deviceVideoAppService;
    @Autowired
    private RedisService redisService;
    /**
     * @Title: DeviceVideoRecordAppService
     * @Author: liy
     * @Date: 2022/10/18 16:24
     * @Description: 视频通话记录上传
     * @Version:1.0
     */
    @Override
    public void handVideoDuration(String imei, String videoId, int duration, int waitDuration, String type) {
        if (duration>0){
            //根据videoId查询视频通话记录表
            LambdaQueryWrapper<DeviceVideoRecordApp> lambdaQueryWrapper=new LambdaQueryWrapper<>();
            lambdaQueryWrapper.eq(DeviceVideoRecordApp::getVideoId,videoId);
            DeviceVideoRecordApp deviceVideoRecordApp = deviceVideoRecordAppMapper.selectOne(lambdaQueryWrapper);
            //查询设备拥有者信息获取最新的一次通话的id
            LambdaQueryWrapper<DeviceOwnerInfoApp> ownerInfoAppLambdaQueryWrapper=new LambdaQueryWrapper<>();
            ownerInfoAppLambdaQueryWrapper.eq(DeviceOwnerInfoApp::getImei,imei);
            DeviceOwnerInfoApp deviceOwnerInfoApp = deviceOwnerInfoAppMapper.selectOne(ownerInfoAppLambdaQueryWrapper);
            if (null!=deviceVideoRecordApp){
                if ("1".equals(deviceVideoRecordApp.getStatus())){
                   //结束视频时间 - 发起视频时间 = 真正的视频时间  duration(不包含拨打时等待的时间)
                    int startTime = Integer.parseInt(deviceVideoRecordApp.getStartTime());
                    String time=String.valueOf(System.currentTimeMillis()).substring(0,10);
                    int endTime = Integer.parseInt(time);
                    int trueDurationTime = endTime - startTime;
                    //和device_owner_info表记录的 最后一个视频记录不一样
                    if (null!=deviceOwnerInfoApp){
                        String ownerVideoId=null!=deviceVideoRecordApp.getVideoId()?deviceVideoRecordApp.getVideoId():"";
                        if (ownerVideoId.equals(videoId)){
                            log.info("和最后记录的视频id不一样，开始记录视频上传");
                            //视频
                            if (CommonConstant.AUDIT_STATUS_ZERO.equals(deviceVideoRecordApp.getCallType())){
                                log.info("视频的时长为:{}",trueDurationTime);
                                //视频的执行限制规则
                                jfLimit553(imei,trueDurationTime,deviceVideoRecordApp.getCallType());
                                //当前手表视频通话总秒数
                                int videoTotalCallTime = null!=deviceOwnerInfoApp.getVideoTotalcalltime()?deviceOwnerInfoApp.getVideoTotalcalltime():0;
                                videoTotalCallTime += duration;
                                deviceOwnerInfoApp.setVideoTotalcalltime(videoTotalCallTime);
                                logger.info("imei:{},通话时间为:{}秒，更新视频通话总秒数成功:{} 秒",imei, duration, videoTotalCallTime);
                            }
                            //语音通话
                            else {
                                int voiceTotalCallTime =  null!=deviceOwnerInfoApp.getVoiceTotalcalltime()?deviceOwnerInfoApp.getVoiceTotalcalltime():0;
                                voiceTotalCallTime +=duration;
                                deviceOwnerInfoApp.setVoiceTotalcalltime(voiceTotalCallTime);
                                logger.info("imei:{},通话时间为:{}秒，更新语音通话总秒数成功:{} 秒",imei, duration, voiceTotalCallTime);
                            }
                            //记录最后一次通话的 ID
                            deviceOwnerInfoApp.setVideoId(videoId);

                            if (null!=deviceOwnerInfoApp.getNewVideoId()){
                                //清除最新通话newVideoId
                                deviceOwnerInfoApp.setNewVideoId("");
                            }
                            deviceOwnerInfoAppMapper.updateById(deviceOwnerInfoApp);

                            deviceVideoRecordApp.setEndTime(time);
                            deviceVideoRecordApp.setWaitDuration(String.valueOf(waitDuration));
                            deviceVideoRecordApp.setDuration(String.valueOf(duration));
                            deviceVideoRecordApp.setType(type);
                            deviceVideoRecordApp.setStatus("1");
                            deviceVideoRecordAppMapper.updateById(deviceVideoRecordApp);
                        }
                    }
                }
            }
        }
    }

    @Override
    public void appVideoRecord(DeviceVideoRecordDto deviceVideoRecordDto) {
        handVideoDuration(deviceVideoRecordDto.getImei(),deviceVideoRecordDto.getVideoId(),deviceVideoRecordDto.getDuration(),deviceVideoRecordDto.getWaitDuration(),"APP");
    }

    /**
     * 分页查询通话记录
     * @param current 当前页
     * @param pageSize 每页多少条记录
     * @return list
     */
    @Override
    public IPage<DeviceVideoRecordQuery> queryRecord(Integer current, Integer pageSize) {
        String userId= SecurityContextHolder.getStringUserId();
        Page<DeviceVideoRecordQuery> page=new Page<>(current,pageSize);
        return deviceVideoRecordAppMapper.queryRecord(page,userId);
    }

    /**
     * 计算 结束 的时候 否是 达成限制的问题
     * @param trueDurationTime 真正视频的时间
     */
    public void jfLimit553(String imei,int trueDurationTime,String callType){
        //查询视频类型
        String videoType = deviceInfoMapper.getVideoType(imei);
        //转义视频类型
        String deviceVideoType=CommonConstant.VIDEO_TYPE_MAP.get(videoType);
       //菊风
        if (CommonConstant.VIDEO_TYPE_ONE.equals(deviceVideoType)){
            DeviceData deviceData = pushRedis(imei, callType, trueDurationTime);
            log.info("imei:{}, 菊风的表走限制方法 redis 缓存 :  redisData: {}" ,imei, deviceData);
        }
        //TH
        if (CommonConstant.VIDEO_TYPE_FOUR.equals(deviceVideoType)){
            DeviceData deviceData = pushRedis(imei, callType, trueDurationTime);
            log.info("imei:{}, 鹈鹕的表走限制方法 redis 缓存 :  redisData: {}" ,imei, deviceData);
        }
        //BR
        if (CommonConstant.VIDEO_TYPE_THREE.equals(deviceVideoType)){
            DeviceData deviceData = pushRedis(imei, callType, trueDurationTime);
            log.info("imei:{}, 佰锐的表走限制方法 redis 缓存 :  redisData: {}" ,imei, deviceData);
        }

    }
    public DeviceData pushRedis(String imei,String callType,int trueDurationTime){
        String time=String.valueOf(System.currentTimeMillis()).substring(0,10);
        int nowTime=Integer.parseInt(time);
        Map<String, Object> videoConfig = deviceVideoAppService.videoConfig(imei, callType);
        DeviceData redisData = redisService.getCacheObject("imei:"+imei);
        int allCallTime=Integer.parseInt(videoConfig.get("allCallTime").toString());
        if(trueDurationTime > allCallTime){
            //直接通话时长超 总时长限制,直接限制,写入redis, 接着如发起视频需等待休息时间 的 秒数
            DeviceData deviceData=new DeviceData();
            deviceData.setLastEndVideoTime(nowTime);
            deviceData.setRVideoTime(0);
            deviceData.setRIntervalVideoTime(0);
            deviceData.setRVideocount(1);
            deviceData.setVideoLimitStatus(1);
            redisService.setCacheObject("imei:"+imei,deviceData);
        }
        else if (null!=redisData){
            DeviceData deviceData=new DeviceData();
            //上一次通话 结束的 时间点
            //int lastEndVideoTime =redisData.getLastEndVideoTime();
            //是否限制设备视频通话
            //int videoLimitStatus = redisData.getVideoLimitStatus();
            //最近视频通话累计时间 /秒 (只要达到限制就会被清空)
            int rVideoTime = redisData.getRVideoTime();
            //最近视频通话累计间隔时间  /秒 (只要达到限制就会被清空)
            //int rIntervalVideoTime = redisData.getRIntervalVideoTime();
            //最近视频通话累计次数 /次 (一直存在)
            int rVideoCount = null!=redisData.getRVideocount()?redisData.getRVideocount():0;

            if(rVideoTime + trueDurationTime > allCallTime) {
                //超过总时间, 达成限制
                deviceData.setRVideocount(1+rVideoCount);
                deviceData.setRIntervalVideoTime(0);
                deviceData.setRVideoTime(0);
                deviceData.setVideoLimitStatus(1);
            }
            else {
                //未超过总时间 累加
                deviceData.setRVideoTime(rVideoTime+trueDurationTime);
                deviceData.setRVideocount(1+rVideoCount);
            }
            //更新 上一次通话 结束的 时间点
            deviceData.setLastEndVideoTime(nowTime);
            redisService.setCacheObject("imei:"+imei,deviceData);
        }
        else {
            //缓存没有,首次写入
            DeviceData deviceData=new DeviceData();
            deviceData.setLastEndVideoTime(nowTime);
            deviceData.setRVideocount(0);
            deviceData.setRIntervalVideoTime(0);
            deviceData.setVideoLimitStatus(0);
            deviceData.setRVideoTime(trueDurationTime);
            redisService.setCacheObject("imei:"+imei,deviceData);
            logger.info("imei:{}, Redis中没有视频的数据缓存,首次写入 redis的数据为: {}",imei, deviceData);
        }
        return redisData;
    }
}
