package com.ecell.internationalize.app.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ecell.internationalize.app.entity.dto.HeartTemperatureConfigDto;
import com.ecell.internationalize.app.entity.dto.HeartTemperatureDto;
import com.ecell.internationalize.app.mapper.HeartTemperatureConfigMapper;
import com.ecell.internationalize.app.mapper.HeartTemperatureRecordMapper;
import com.ecell.internationalize.app.service.HeartTemperatureConfigService;
import com.ecell.internationalize.common.core.context.SecurityContextHolder;
import com.ecell.internationalize.common.core.utils.uuid.UUID;
import com.ecell.internationalize.common.issue.component.IssueInstructions;
import com.ecell.internationalize.common.issue.constant.CommonConstant;
import com.ecell.internationalize.common.issue.entity.issuedto.HeartRate;
import com.ecell.internationalize.common.system.entity.HeartTemperatureConfig;
import com.ecell.internationalize.common.system.entity.HeartTemperatureRecord;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * <p>
 * 心率/体温配置表 服务实现类
 * </p>
 *
 * @author ${author}
 * @since 2022-10-28
 */
@Service
public class HeartTemperatureConfigServiceImpl extends ServiceImpl<HeartTemperatureConfigMapper, HeartTemperatureConfig> implements HeartTemperatureConfigService {
    @Autowired
    private HeartTemperatureConfigMapper heartTemperatureConfigMapper;
    @Autowired
    private HeartTemperatureRecordMapper heartTemperatureRecordMapper;
    @Autowired
    private IssueInstructions issueInstructions;
    /**
     * 查询心率/体温配置信息
     * @param imei imei
     * @param type type为：1 表示查询心率配置,type为 2 表示查询体温配置
     * @param date 日期
     * @return
     */
    @Override
    public HeartTemperatureConfigDto queryHeartOrTemperature(String imei, String type, String date) throws ParseException {
        //格式化日期
        SimpleDateFormat simpleDateFormat=new SimpleDateFormat("yyyy-MM-dd");
        //1.先查询表中是否存在配置信息记录,没有给默认记录入库
        LambdaQueryWrapper<HeartTemperatureConfig> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(HeartTemperatureConfig::getImei,imei);
        lambdaQueryWrapper.eq(HeartTemperatureConfig::getType,type);
        HeartTemperatureConfig heartTemperatureConfig = heartTemperatureConfigMapper.selectOne(lambdaQueryWrapper);
        //start == 获取这一天的 上下限时间 开始时间时间戳 结束时间时间戳，通过日期找出当天所有心率/体温记录
        Date time=simpleDateFormat.parse(date);
        String subTime = String.valueOf(time.getTime()).substring(0, 10);
        int startTime=Integer.parseInt(subTime);
        int endTime=startTime+86400;
        LambdaQueryWrapper<HeartTemperatureRecord> recordLambdaQueryWrapper=new LambdaQueryWrapper<>();
        recordLambdaQueryWrapper.eq(HeartTemperatureRecord::getImei,imei);
        recordLambdaQueryWrapper.eq(HeartTemperatureRecord::getType,type);
        recordLambdaQueryWrapper.ge(HeartTemperatureRecord::getTimeStamp,startTime);
        recordLambdaQueryWrapper.le(HeartTemperatureRecord::getTimeStamp,endTime);
        List<HeartTemperatureRecord> heartTemperatureRecordList = heartTemperatureRecordMapper.selectList(recordLambdaQueryWrapper);
        //查询为空则新增默认值
        if (null == heartTemperatureConfig){
            createConfig(imei, type);
        }
        //end ==查询结束
        //start == 数据封装返回
        List<HeartTemperatureDto> list=new ArrayList<>();
        HeartTemperatureDto heartTemperatureDto;
        if (!CollectionUtils.isEmpty(heartTemperatureRecordList)){
            for (HeartTemperatureRecord heartTemperatureRecord:heartTemperatureRecordList){
                heartTemperatureDto=new HeartTemperatureDto();
                heartTemperatureDto.setTimestamp(heartTemperatureRecord.getTimeStamp());
                heartTemperatureDto.setData(heartTemperatureRecord.getHeartTemperatureRate());
                list.add(heartTemperatureDto);
            }
        }
        HeartTemperatureConfig mustHaveConfig = heartTemperatureConfigMapper.selectOne(lambdaQueryWrapper);
        HeartTemperatureConfigDto heartTemperatureConfigDto=new HeartTemperatureConfigDto();
        heartTemperatureConfigDto.setFrequency(mustHaveConfig.getHeartTemperatureFreq());
        heartTemperatureConfigDto.setLastData(mustHaveConfig.getLastHeartTemperature());
        heartTemperatureConfigDto.setLastDataTime(mustHaveConfig.getLastHeartTemperatureTime());
        heartTemperatureConfigDto.setLower(mustHaveConfig.getLower());
        heartTemperatureConfigDto.setUpper(mustHaveConfig.getUpper());
        heartTemperatureConfigDto.setSwitchOn(mustHaveConfig.getSwitchOn());
        heartTemperatureConfigDto.setType(type);
        Map<String,List<HeartTemperatureDto>> map=new HashMap<>(32);
        map.put(date,list);
        heartTemperatureConfigDto.setDataListDict(map);
        //end  ==数据封装结束
        return heartTemperatureConfigDto;
    }

    @Override
    public List<HeartTemperatureRecord> queryHeartOrBodyOxygen(String imei) {
        QueryWrapper wrapper =new QueryWrapper();
        wrapper.eq("imei",imei);
        wrapper.eq("type","3");
        List<HeartTemperatureRecord> list = heartTemperatureRecordMapper.selectList(wrapper);

        return list;
    }

    /**
     * 开始测量体温/心率
     * @param imei imei
     * @param type type为：1 表示查询心率配置,type为 2 表示查询体温配置
     * @param cmd 为1:表示开启开关,cmd为2:上传频率
     * @param obj cmd为1上传开关状态,cmd为2上传频率
     * @return
     */
    @Override
    public void checkHeartOrTemperature(String imei, String type, String cmd, String obj) {
        //1.先查询表中是否存在配置信息记录,没有给默认记录入库
        LambdaQueryWrapper<HeartTemperatureConfig> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(HeartTemperatureConfig::getImei,imei);
        lambdaQueryWrapper.eq(HeartTemperatureConfig::getType,type);
        HeartTemperatureConfig heartTemperatureConfig = heartTemperatureConfigMapper.selectOne(lambdaQueryWrapper);
        //查询为空则新增默认值
        if (null == heartTemperatureConfig){
            createConfig(imei, type);
        }
        //内部协议设置心率/体温频率 的指令
        String cmdType="";
        //心率
        if (CommonConstant.AUDIT_STATUS_ONE.equals(type)){
            cmdType="setHeartRate";
        }else {
            cmdType="setBodyTemperature";
        }
        HeartTemperatureConfig mustHaveConfig = heartTemperatureConfigMapper.selectOne(lambdaQueryWrapper);
        //指令下发数据封装
        HeartRate heartRate=new HeartRate();
        heartRate.setIdent("12345678");
        heartRate.setImei(imei);
        heartRate.setOpenid(SecurityContextHolder.getStringUserId());
        heartRate.setVender("20000");
        heartRate.setType(cmdType);
        heartRate.setTime(System.currentTimeMillis());
        //如果为获取开关上传
        if (CommonConstant.AUDIT_STATUS_ONE.equals(cmd)){
            //开关为打开状态
            if (CommonConstant.AUDIT_STATUS_ONE.equals(obj)) {
                //心率/体温上传频率
                Integer devFrequency=mustHaveConfig.getHeartTemperatureFreq();
                heartRate.setFrequency(devFrequency * 60);
                //3.下发指令之前先存入redis,开始下发指令
                issueInstructions.emitDeviceDemand(cmdType, imei, heartRate);
            }else {
                heartRate.setFrequency(0);
                issueInstructions.emitDeviceDemand(cmdType,imei,heartRate);
            }
            //更新配置表开关状态
            mustHaveConfig.setSwitchOn(obj);
            heartTemperatureConfigMapper.updateById(mustHaveConfig);

        }
        //只上传频率
        else if (CommonConstant.AUDIT_STATUS_TWO.equals(cmd)){
            String switchStatus=mustHaveConfig.getSwitchOn();
            //频率为-1
            if (CommonConstant.VIDEO_RESULT_PRE_ONE == Integer.parseInt(obj)){
                //开关打开
                if (CommonConstant.AUDIT_STATUS_ONE.equals(switchStatus)){
                    heartRate.setFrequency(2);
                    //3.下发指令之前先存入redis,开始下发指令
                    issueInstructions.emitDeviceDemand(cmdType,imei,heartRate);
                }else {
                    heartRate.setFrequency(1);
                    //3.下发指令之前先存入redis,开始下发指令
                    issueInstructions.emitDeviceDemand(cmdType,imei,heartRate);
                }
            }
            else {
                //开关为 1 的情况下才下发, 其余不下发
                if (CommonConstant.AUDIT_STATUS_ONE.equals(switchStatus)){
                    heartRate.setFrequency(Integer.parseInt(obj) * 60);
                    //3.下发指令之前先存入redis,开始下发指令
                    issueInstructions.emitDeviceDemand(cmdType,imei,heartRate);
                }
                mustHaveConfig.setHeartTemperatureFreq(Integer.parseInt(obj));
                heartTemperatureConfigMapper.updateById(mustHaveConfig);
            }

        }

    }

    /**
     * 新增体温/心率配置信息
     * @param imei
     * @param type
     */
    public void createConfig(String imei,String type){
        HeartTemperatureConfig heartTemperatureConfig=new HeartTemperatureConfig();
        String id= UUID.fastUUID().toString(true);
        heartTemperatureConfig.setId(id);
        heartTemperatureConfig.setImei(imei);
        heartTemperatureConfig.setHeartTemperatureFreq(30);
        heartTemperatureConfig.setLastHeartTemperature("0");
        heartTemperatureConfig.setSwitchOn("0");
        heartTemperatureConfig.setCreateUser(SecurityContextHolder.getStringUserId());
        heartTemperatureConfig.setCreateTime(new Date());
        heartTemperatureConfig.setType(type);
        heartTemperatureConfig.setLastHeartTemperatureTime(0);
        if (CommonConstant.AUDIT_STATUS_ONE.equals(type)){
            //心率上限
            heartTemperatureConfig.setUpper("100");
            //心率下限
            heartTemperatureConfig.setLower("60");
        }else if (CommonConstant.AUDIT_STATUS_TWO.equals(type)){
            //体温上限
            heartTemperatureConfig.setUpper("37.5");
            //体温下限
            heartTemperatureConfig.setLower("35");
        }else if (CommonConstant.AUDIT_STATUS_THREE.equals(type)){
            //体温上限
            heartTemperatureConfig.setUpper("100");
            //体温下限
            heartTemperatureConfig.setLower("80");
        }
        heartTemperatureConfigMapper.insert(heartTemperatureConfig);
    }
}
