package com.ecell.internationalize.app.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ecell.internationalize.app.entity.dto.ImeiSosInfoDto;
import com.ecell.internationalize.app.mapper.ImeiSosInfoMapper;
import com.ecell.internationalize.app.service.ImeiSosInfoService;
import com.ecell.internationalize.app.util.LocationUtil;
import com.ecell.internationalize.common.core.context.SecurityContextHolder;
import com.ecell.internationalize.common.core.exception.ServiceException;
import com.ecell.internationalize.common.core.utils.StringUtils;
import com.ecell.internationalize.common.core.utils.bean.BeanUtils;
import com.ecell.internationalize.common.core.utils.uuid.UUID;
import com.ecell.internationalize.common.issue.component.IssueInstructions;
import com.ecell.internationalize.common.issue.entity.issuedto.SOS;
import com.ecell.internationalize.common.issue.util.RandomUtil;
import com.ecell.internationalize.common.system.entity.ImeiSosInfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author ${author}
 * @since 2022-09-17
 */
@Service
public class ImeiSosInfoServiceImpl extends ServiceImpl<ImeiSosInfoMapper, ImeiSosInfo> implements ImeiSosInfoService {
    @Autowired
    private IssueInstructions issueInstructions;
    /**
     * 新增sos
     * @param dto
     * @return
     */
    @Override
    public boolean addSOSPhone(ImeiSosInfoDto dto) {
        Boolean flag=false;
        if (!StringUtils.isNotEmpty(dto.getImei())){
            throw new ServiceException("imei不能为空");
        }
        ImeiSosInfo info = new ImeiSosInfo();
        BeanUtils.copyBeanProp(info,dto);
        //编辑
        if (StringUtils.isNotEmpty(dto.getImeiSosId())){
            flag = updateById(info);
        }else {
            info.setImeiSosId(UUID.fastUUID().toString(true));
            flag = save(info);
        }

        if (flag){
            //下发指令
            sendSOSPhone(info);
        }
        return flag;
    }
    /**
     * 根据imei查询sos
     * @param imei
     * @return
     */
    @Override
    public ImeiSosInfo querySOSPhone(String imei) {
        QueryWrapper wrapper =new QueryWrapper();
        wrapper.eq("imei",imei);
        wrapper.last("LIMIT 1");
       return baseMapper.selectOne(wrapper);
    }


    private ImeiSosInfo querySOSByImei(String imei){
        QueryWrapper wrapper =new QueryWrapper();
        wrapper.eq("imei",imei);
        wrapper.last("LIMIT 1");
        return baseMapper.selectOne(wrapper);

    }
    //组装参数 下发指令
    private void  sendSOSPhone(ImeiSosInfo info){
        List<String> list =new ArrayList<>();
        SOS sos =new SOS();
        sos.setVender(LocationUtil.getVender(info.getImei(),"agent"));
        sos.setIdent(RandomUtil.generateVerCode(6));
        sos.setImei(info.getImei());
        sos.setOpenid(SecurityContextHolder.getStringUserId());
        sos.setType("sos");
        sos.setTime(System.currentTimeMillis());
       if (StringUtils.isNotEmpty(info.getSosPhoneOne())){
           list.add(info.getSosPhoneOne());
       }
        if (StringUtils.isNotEmpty(info.getSosPhoneTwo())){
            list.add(info.getSosPhoneTwo());
        }
        if (StringUtils.isNotEmpty(info.getSosPhoneThree())){
            list.add(info.getSosPhoneThree());
        }
        sos.setSos(list);
        issueInstructions.emitDeviceDemand("sos",info.getImei(),sos);
        //下发指令
       // issueInstructions.sendKafka(sos);
    }
}
