package com.ecell.internationalize.app.service.impl;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ecell.internationalize.app.entity.LatestLocationApp;
import com.ecell.internationalize.app.mapper.LatestLocationAppMapper;
import com.ecell.internationalize.app.service.LatestLocationAppService;
import com.ecell.internationalize.app.util.LocationUtil;
import com.ecell.internationalize.common.core.context.SecurityContextHolder;
import com.ecell.internationalize.common.issue.constant.CommonConstant;
import com.ecell.internationalize.common.issue.mqtt.dto.IOSMessage;
import com.ecell.internationalize.common.issue.mqtt.dto.LocationMessages;
import com.ecell.internationalize.common.issue.mqtt.dto.SendIOSParamete;
import com.ecell.internationalize.common.issue.mqtt.push.IOSPush;
import com.ecell.internationalize.common.issue.mqtt.push.MqttPush;
import com.ecell.internationalize.common.redis.service.RedisService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;

/**
 * <p>
 * 位置最新记录 服务实现类
 * </p>
 *
 * @author ${author}
 * @since 2022-09-27
 */
@Service
public class LatestLocationAppServiceImpl extends ServiceImpl<LatestLocationAppMapper, LatestLocationApp> implements LatestLocationAppService {
    private static final Logger log = LoggerFactory.getLogger(DeviceLocationAppServiceImpl.class);
    @Autowired
    private DeviceLocationAppServiceImpl deviceLocationAppServiceImpl;
    @Autowired
    @Lazy
    private BindingDeviceImpl bindingDevice;
    @Autowired
    private MqttPush mqttPush;
    @Autowired
    private RedisService redisService;

    @Autowired
    private IOSPush iosPush;

    /**
     * 根据imei获取最新位置信息
     *
     * @param imei
     * @return
     */
    @Override
    public LatestLocationApp getLastLocation(String imei) {
        LatestLocationApp latestLocation = getLatestLocation(imei);
        log.info("获取到的最新位置是======={}",latestLocation);
        return latestLocation;
    }

    /**
     * 下发指令
     * @param imei
     * @return
     */
    @Override
    public Map<String,String> getLastLocationInstructions(String imei,String s) {
        Map<String,String> map =new HashMap<>(3);
         //设置定位时间间隔 防止不停的 定位;
        //conf.r.hset(imei, , str(dataDict["time"]))
       // redisService.setCacheMapValue(imei,"lnglattime",System.currentTimeMillis());
        //设备是否在线
       // String s = redisService.getCacheMapValue(imei, "onlineStatus").toString();
       // String s = redisService.getCacheMapValue(imei, "onlineStatus");
        if (CommonConstant.AUDIT_STATUS_ONE.equals(s)){
            String userId = SecurityContextHolder.getStringUserId();
            String redisKey=String.format("requestLocation:%s:%s",imei,userId);
            //上传位置的时候要用到
            // redisService.setCacheObject(redisKey,userId,60L, TimeUnit.SECONDS);
            redisService.setCacheObject(redisKey,userId);
            //下发指令
            LocationUtil.sendLocation(imei, SecurityContextHolder.getStringUserId());
            map.put("onlineStatus",CommonConstant.AUDIT_STATUS_ONE);

        }else {

            map.put("onlineStatus",CommonConstant.AUDIT_STATUS_ZERO);

        }

       return map;
    }


    /**
     * 根据imei查询最新位置信息
     *
     * @param imei
     * @return
     */
    public LatestLocationApp getLatestLocation(String imei) {
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("imei", imei);
        wrapper.last("LIMIT 1");
        return baseMapper.selectOne(wrapper);

    }

    public void sendLastLocationApp(String imei, String userId){
        LatestLocationApp app = getLatestLocation(imei);
        //根据imei 查询用户绑定表
      //  List<UserDeviceBindApp> userIdList = bindingDevice.getUserIdList(imei);
     //   if (StringUtils.isNotEmpty(userIdList)){
//            List<String> userIds = userIdList.stream().map(UserDeviceBindApp::getUserId).collect(Collectors.toList());
//
            //根据userId查询用户手机型号
            String phoneType =redisService.getCacheMapValue(userId, CommonConstant.USER_AGENT);

            //发送位置信息
            LocationMessages messages =new LocationMessages();
            messages.setType("deviceNowLocation");
            messages.setLoc_type(app.getLocationType());
            messages.setAddress("");
            messages.setLat(app.getLatitude());
            messages.setLon(app.getLongitude());
            messages.setRadius(app.getRadius());
            messages.setImei(imei);
            messages.setTimestamp(System.currentTimeMillis());
            String s = JSON.toJSONString(messages);

            //安卓
            if (CommonConstant.ANDROID.equals(phoneType)){
                log.info("Android主动请求下发位置:{}",messages);
                mqttPush.sendMqtt(CommonConstant.AUDIT_STATUS_THREE,userId,CommonConstant.AUDIT_STATUS_ONE,"deviceNowLocation","deviceNowLocation",s);

                //IOS
            }else if (CommonConstant.IOS.equals(phoneType)){
                log.info("IOS主动请求下发位置:{}",messages);
                sendLastLocationIOS(userId,messages);

            }

        }

   // }

    //IOS推送 定位不弹窗
    private void sendLastLocationIOS(String userId,LocationMessages messages){
        SendIOSParamete paramete =new SendIOSParamete();
        IOSMessage message =new IOSMessage();
        paramete.setOpenid(userId);
        paramete.setType("deviceNowLocation");
        message.setData(messages);
        message.setSound(CommonConstant.SOUND);
        message.setContent_available(CommonConstant.NUMBER_ONE);
        paramete.setApns_param(message);
        iosPush.sendIOS(paramete);
    }
}
