package com.ecell.internationalize.app.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ecell.internationalize.app.entity.TrackInfoApp;
import com.ecell.internationalize.app.entity.dto.TrackInfoAppDto;
import com.ecell.internationalize.app.entity.dto.TrackInfoDto;
import com.ecell.internationalize.app.entity.po.TrackInfoAppPO;
import com.ecell.internationalize.app.mapper.TrackInfoAppMapper;
import com.ecell.internationalize.app.service.TrackInfoAppService;
import com.ecell.internationalize.app.util.VideoUtil;
import com.ecell.internationalize.common.core.utils.DateUtils;
import com.ecell.internationalize.common.core.utils.StringUtils;
import com.ecell.internationalize.common.core.utils.bean.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 轨迹表 服务实现类
 * </p>
 *
 * @author ${author}
 * @since 2022-10-17
 */
@Service
public class TrackInfoAppServiceImpl extends ServiceImpl<TrackInfoAppMapper, TrackInfoApp> implements TrackInfoAppService {

    @Override
    public  List<TrackInfoAppDto> queryTrackInfoByImei(TrackInfoDto dto) {
        if (!StringUtils.isNotEmpty(dto.getStartTime())){
            dto.setStartTime(DateUtils.getDate());
        }
        List<TrackInfoAppDto> list =new ArrayList<>();
        List<TrackInfoApp> trackInfoApps = baseMapper.queryTrackInfoByImei(dto);
          if(StringUtils.isNotEmpty(trackInfoApps) && trackInfoApps.size()>0){
              list = trackInfoApps.stream().map(s -> {
                  TrackInfoAppDto dto1 = new TrackInfoAppDto();
                  BeanUtils.copyBeanProp(dto1, s);
                  dto1.setLon(Double.parseDouble(s.getLongitude()));
                  dto1.setLat(Double.parseDouble(s.getLatitude()));
                  return dto1;

              }).collect(Collectors.toList());
          }


        return list;
//        Page<TrackInfoApp> page =new Page<>(dto.getCurrent(),dto.getPageSize());
//        page.setRecords(trackInfoApps);

    }

    /**
     * 查询当天轨迹数据
     * @param imei
     * @return
     */
    @Override
    public List<TrackInfoAppPO> queryListByImei(String imei){
        Date endDate = new Date();
        Date startDate = DateUtils.addDays(endDate, -30);
       // String startTime = DateUtils.dateTime(startDate);
        List<TrackInfoAppPO> trackInfoApps = baseMapper.queryListByImei(imei,startDate,endDate);
        if (StringUtils.isNotEmpty(trackInfoApps) && trackInfoApps.size()>0){
            trackInfoApps = trackInfoApps.stream().map(s -> {
                s.setId(DateUtils.dateTime(s.getDate()));
                return s;
            }).collect(Collectors.toList());
        }
        return  trackInfoApps;

    }

    @Override
    public boolean queryInfo() {
       return VideoUtil.queryAgoraStatus("867188040000098",1111);
    }

    /**
     * 根据日期删除轨迹信息
     * @param date
     * @return
     */
    @Override
    public boolean deleteTrackInfoByDate(String imei,String date) {
        return baseMapper.deleteTrackInfoByDate(imei,date)>0 ? true :false;
    }

    /**
     * 保存OR更新轨迹信息表
     * @param old 旧的轨迹
     * @param fresh 新的轨迹
     */
    @Transactional(rollbackFor = Exception.class)
    public void saveOrUpdate(TrackInfoApp fresh,TrackInfoApp old){
        if (StringUtils.isNotNull(fresh)){
            //添加新的轨迹
            save(fresh);
        }
       if (StringUtils.isNotNull(old)){
           //更新旧的轨迹
           updateById(old);
       }


    }

    /**
     * 内部使用的接口
     * @param imei
     * @return
     */
    public List<TrackInfoApp> queryTrackListByImei(String imei){
        List<TrackInfoApp> trackInfoApps = baseMapper.queryTrackListByImei(imei);
        return  trackInfoApps;

    }

    /**
     * 内部使用的接口
     * @param imei
     * @return
     */
    public List<TrackInfoApp> queryTrackListByImeiDes(String imei){
        QueryWrapper<TrackInfoApp> wrapper =new QueryWrapper<>();
        wrapper.eq("imei",imei);
        wrapper.orderByDesc("create_time");
        return  baseMapper.selectList(wrapper);

    }

}
