package com.ecell.internationalize.app.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ecell.internationalize.app.entity.AddressBookApp;
import com.ecell.internationalize.app.entity.UserVo;
import com.ecell.internationalize.app.entity.dto.BindingDeviceDto;
import com.ecell.internationalize.app.entity.dto.UpdateUserDto;
import com.ecell.internationalize.app.entity.dto.UserDto;
import com.ecell.internationalize.app.mapper.AddressBookAppMapper;
import com.ecell.internationalize.app.mapper.UserDeviceBindAppMapper;
import com.ecell.internationalize.app.mapper.UserMapper;
import com.ecell.internationalize.app.service.UserDeviceBindService;
import com.ecell.internationalize.app.service.UserService;
import com.ecell.internationalize.common.core.constant.SecurityConstants;
import com.ecell.internationalize.common.core.context.SecurityContextHolder;
import com.ecell.internationalize.common.core.exception.ServiceException;
import com.ecell.internationalize.common.core.utils.JwtUtils;
import com.ecell.internationalize.common.core.utils.SpringUtils;
import com.ecell.internationalize.common.core.utils.StringUtils;
import com.ecell.internationalize.common.core.utils.uuid.IdUtils;
import com.ecell.internationalize.common.core.utils.uuid.UUID;
import com.ecell.internationalize.common.issue.constant.CommonConstant;
import com.ecell.internationalize.common.redis.service.RedisService;
import com.ecell.internationalize.common.security.utils.SecurityUtils;
import com.ecell.internationalize.common.system.entity.User;
import com.ecell.internationalize.common.system.entity.UserDeviceBindApp;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * <p>
 * 用户表 服务实现类
 * </p>
 *
 * @author liy
 * @since 2022-08-16
 */
@Service
public class UserServiceImpl extends ServiceImpl<UserMapper, User> implements UserService {
    private static final Logger logger = LoggerFactory.getLogger(UserServiceImpl.class);
    @Autowired
    private UserMapper userMapper;
    @Autowired
    private UserDeviceBindAppMapper userDeviceBindAppMapper;
    @Autowired
    private AddressBookAppMapper addressBookAppMapper;
    @Autowired
    private RedisService redisService;
    @Autowired
    private BindingDeviceImpl bindingDevice;
    private static final long EXPIRATION = 30;
    public final static String APP_TOKEN_KEY = "app_tokens:";

    @Override
    @Transactional(rollbackFor = Exception.class)
    public int insertUser(UserDto userDto) {
        //对密码进行加密
        String encryptPassword = SecurityUtils.encryptPassword(userDto.getPassword());
        User user = new User();
        String userId = UUID.randomUUID().toString(true);
        user.setUserId(userId);
        //默认姓名为邮箱
        user.setName(userDto.getEmail());
        user.setPassword(encryptPassword);
        user.setUserEmail(userDto.getEmail());
        user.setUserAgent(userDto.getUserAgent());
        // UserDeviceBindApp userDeviceBindApp = insertData(user);
        //  userDeviceBindAppMapper.insert(userDeviceBindApp);
        return userMapper.insert(user);
    }

    /**
     * token生成
     *
     * @param user
     * @return int
     * @Title: UserService
     * @Author: liy
     * @Date: 2022/8/17 15:03
     */
    @Override
    public Map<String, Object> createToken(User user, String osName, String deviceToken, String packages, String version) {
        //修改登录机型
        user.setUserAgent(osName);
        userMapper.updateById(user);
        String token = IdUtils.fastUUID();
        // Jwt存储信息
        Map<String, Object> claimsMap = new HashMap<>(32);
        claimsMap.put(SecurityConstants.USER_KEY, token);
        claimsMap.put(SecurityConstants.DETAILS_USER_ID, user.getUserId());
        claimsMap.put(SecurityConstants.DETAILS_USERNAME, user.getUserEmail());
        user.setToken(JwtUtils.createToken(claimsMap));
        //key缓存
        refreshToken(user);
        // 接口返回信息
        Map<String, Object> rspMap = new HashMap<>(16);
        rspMap.put("Authorization", user.getToken());
        rspMap.put("expires_in", EXPIRATION);
        rspMap.put("openid", user.getUserId());
        System.out.println("成功返回登录信息:" + rspMap);
        //IOS deviceToken存入redis
        if ("iOS".equals(osName)) {
            logger.info("iOS机型,获取IOS的token并存入redis:{}", deviceToken);
            if (null != deviceToken && !"".equals(deviceToken)) {
                logger.info("iOS机型,获取IOS的token并存入redis:{}", deviceToken);
                redisService.setCacheMapValue(user.getUserId(), CommonConstant.DEVICE_TOKEN, deviceToken);
            } else {
                logger.info("苹果手机拒绝接收通知,token存入空");
                redisService.setCacheMapValue(user.getUserId(), CommonConstant.DEVICE_TOKEN, null);
            }
            redisService.setCacheMapValue(user.getUserId(), CommonConstant.IOS_PACKAGE, packages);
            redisService.setCacheMapValue(user.getUserId(), CommonConstant.IOS_VERSION, version);
        }
        return rspMap;
    }

//    @Override 20241227 废弃
//    public UserVo queryUser(String userId, String imei) {
//        String phone="";
//        //查询到的用户信息
//        UserVo userVo = userMapper.queryUser(userId);
//        //查询用户与设备绑定信息,获取手机号
//        LambdaQueryWrapper<UserDeviceBindApp> lambdaQueryWrapper=new LambdaQueryWrapper<>();
//        lambdaQueryWrapper.eq(UserDeviceBindApp::getImei,imei);
//        lambdaQueryWrapper.eq(UserDeviceBindApp::getUserId,userId);
//        UserDeviceBindApp userDeviceBindApp = userDeviceBindAppMapper.selectOne(lambdaQueryWrapper);
//        if (null != userDeviceBindApp){
//            phone = userDeviceBindApp.getPhone();
//        }
//        userVo.setPhone(phone);
//        userVo.setImei(imei);
//        if (null == userVo.getName() || "".equals(userVo.getName())){
//            userVo.setName(SecurityContextHolder.getUserName());
//        }
//        return userVo;
//    }

//    @Override 20241227 废弃
//    public UserVo queryUser(String userId, String imei) {
//        String phone = "";
//        //查询到的用户信息
//        UserVo userVo = userMapper.queryUser(userId);
//        //查询用户与设备绑定信息,获取手机号
//        LambdaQueryWrapper<UserDeviceBindApp> lambdaQueryWrapper = new LambdaQueryWrapper<>();
//        lambdaQueryWrapper.eq(UserDeviceBindApp::getImei, imei);
//        lambdaQueryWrapper.eq(UserDeviceBindApp::getUserId, userId);
//        UserDeviceBindApp userDeviceBindApp = userDeviceBindAppMapper.selectOne(lambdaQueryWrapper);
//        if (null != userDeviceBindApp) {
//            phone = userDeviceBindApp.getPhone();
//        }
//        userVo.setPhone(phone);
//        userVo.setImei(imei);
//        if (null == userVo.getName() || "".equals(userVo.getName())) {
//            userVo.setName(SecurityContextHolder.getUserName());
//        }
//        return userVo;
//    }

    @Override
    public UserVo queryUser(String userId, String imei) {
        User user = userMapper.selectById(userId);
        UserVo vo = new UserVo();
        vo.setImage(user.getImage());
        if (StringUtils.isNotEmpty(user.getName())){
            vo.setName(user.getName());
        }else {
            vo.setName(SecurityContextHolder.getUserName());
        }
        vo.setPhone(user.getPhone());

        return vo;
    }


//    @Override 20241227 废弃
//    @Transactional(rollbackFor = Exception.class)
//    public int updateData(UpdateUserDto user) {
//            //1.修改用户设备绑定表的手机号
//            UserDeviceBindApp userDeviceBindApp = new UserDeviceBindApp();
//            userDeviceBindApp.setPhone(user.getPhone());
//            LambdaQueryWrapper<UserDeviceBindApp> lambdaQueryWrapper = new LambdaQueryWrapper<>();
//            lambdaQueryWrapper.eq(UserDeviceBindApp::getImei, user.getImei());
//            lambdaQueryWrapper.eq(UserDeviceBindApp::getUserId, SecurityContextHolder.getStringUserId());
//            userDeviceBindAppMapper.update(userDeviceBindApp, lambdaQueryWrapper);
//            //2.修改通讯录的手机号
//            AddressBookApp addressBookApp = new AddressBookApp();
//            addressBookApp.setPhone(user.getPhone());
//            LambdaQueryWrapper<AddressBookApp> lambdaQueryWrapper1 = new LambdaQueryWrapper<>();
//            lambdaQueryWrapper1.eq(AddressBookApp::getImei, user.getImei());
//            lambdaQueryWrapper1.eq(AddressBookApp::getUserId, SecurityContextHolder.getStringUserId());
//            addressBookAppMapper.update(addressBookApp, lambdaQueryWrapper1);
//            //3.修改用户表的信息
//            User user1 = new User();
//            user1.setUserId(SecurityContextHolder.getStringUserId());
//            user1.setPhone(user.getPhone());
//            user1.setName(user.getName());
//            if (null != user.getImage()) {
//                user1.setImage(user.getImage());
//            }
//            userMapper.updateById(user1);
//            //若为管理员，中心号码下发
//            UserDeviceBindApp deviceBindApp = userDeviceBindAppMapper.selectOne(lambdaQueryWrapper);
//            if (null != deviceBindApp && "2".equals(deviceBindApp.getIdentity())) {
//                BindingDeviceDto bindingDeviceDto = new BindingDeviceDto();
//                bindingDeviceDto.setMy_phone(user.getPhone());
//                bindingDeviceDto.setImei(user.getImei());
//                bindingDevice.assembCenterPhone(bindingDeviceDto, deviceBindApp.getUserId());
//            }
//
//
//        return 1;
//    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public User updateData(UpdateUserDto user) {
        String userId = SecurityContextHolder.getStringUserId();
        BindingDeviceImpl bindingDevice = SpringUtils.getBean(BindingDeviceImpl.class);
        UserDeviceBindServiceImpl bean = SpringUtils.getBean(UserDeviceBindServiceImpl.class);

        if (StringUtils.isNotEmpty(user.getPhone())) {
            LambdaQueryWrapper<UserDeviceBindApp> Wrapper = new LambdaQueryWrapper<>();
            Wrapper.eq(UserDeviceBindApp::getUserId, SecurityContextHolder.getStringUserId());
            Integer i = bean.getBaseMapper().selectCount(Wrapper);
            if (i > 0) {
                // 查询用户是否使用其余绑定该设备用户的手机号码进行绑定设备
                UserDeviceBindApp userDeviceBindApp = bindingDevice.queryByPhone(userId, user.getPhone());
                if (StringUtils.isNotNull(userDeviceBindApp)) {
                    throw new ServiceException("messagesIsPhoneBound");
                }

                LambdaQueryWrapper<UserDeviceBindApp> QueryWrapper = new LambdaQueryWrapper<>();
                QueryWrapper.eq(UserDeviceBindApp::getUserId, SecurityContextHolder.getStringUserId());
                QueryWrapper.eq(UserDeviceBindApp::getBindStatus, CommonConstant.AUDIT_STATUS_ONE);
                List<UserDeviceBindApp> userDeviceBindApps = userDeviceBindAppMapper.selectList(QueryWrapper);
                if (StringUtils.isNotNull(userDeviceBindApps) && userDeviceBindApps.size() > 0) {
                    List<UserDeviceBindApp> userDeviceBindApps1  = new ArrayList<>();
                    for (UserDeviceBindApp userDeviceBindApp1 : userDeviceBindApps) {
                        if (StringUtils.isNotEmpty(user.getPhone()) && !userDeviceBindApp1.getPhone().equals(user.getPhone())) {
                            //修改设备表中的手机号
                            userDeviceBindApp1.setPhone( user.getPhone());
                            userDeviceBindApps1.add(userDeviceBindApp1);
                        }
                    }

                    logger.info("获取到的绑定表的数据是{}",userDeviceBindApps1.size());
                   // List<UserDeviceBindApp> list = userDeviceBindApps.stream().map(s -> s.setPhone(user.getPhone())).collect(Collectors.toList());
                if (userDeviceBindApps1.size() > 0){
                    //修改绑定表的手机号
                    bean.updateBatchById(userDeviceBindApps1);
                }


                    //2.修改通讯录的手机号
//                    AddressBookApp addressBookApp = new AddressBookApp();
//                    addressBookApp.setPhone(user.getPhone());

                    LambdaQueryWrapper<AddressBookApp> lambdaQueryWrapper1 = new LambdaQueryWrapper<>();
                    lambdaQueryWrapper1.eq(AddressBookApp::getUserId, SecurityContextHolder.getStringUserId());
                    List<AddressBookApp> addressBookApps = addressBookAppMapper.selectList(lambdaQueryWrapper1);
                    if (StringUtils.isNotNull(addressBookApps) && addressBookApps.size() > 0) {
                        List<AddressBookApp> list1=new ArrayList<>();
                        for (AddressBookApp bookApp : addressBookApps) {
                            if (StringUtils.isNotEmpty(user.getPhone()) && !user.getPhone().equals(bookApp.getPhone())){
                                bookApp.setPhone(user.getPhone());
                                list1.add(bookApp);
                            }
                        }
//                        List<AddressBookApp> list1 = addressBookApps.stream().map(s -> s.setPhone(user.getPhone())).collect(Collectors.toList());
                        logger.info("获取到的通讯录的数据是{}",list1.size());
                        if (StringUtils.isNotNull(list1) && list1.size() > 0){
                            AddressBookAppServiceImpl bean1 = SpringUtils.getBean(AddressBookAppServiceImpl.class);
                            bean1.updateBatchById(list1);
                        }


                    }


                    //若为管理员，中心号码下发
                    if (StringUtils.isNotNull(userDeviceBindApps1) && userDeviceBindApps1.size() > 0) {
                        userDeviceBindApps1.forEach(s -> {
                            if (null != s && "2".equals(s.getIdentity())) {
                                BindingDeviceDto bindingDeviceDto = new BindingDeviceDto();
                                bindingDeviceDto.setMy_phone(s.getPhone());
                                bindingDeviceDto.setImei(s.getImei());
                                bindingDevice.assembCenterPhone(bindingDeviceDto, s.getUserId());

                            }
                        });

                    }


                }

            }


        }
        //3.修改用户表的信息
        User user1 = new User();
        user1.setUserId(SecurityContextHolder.getStringUserId());
        if (StringUtils.isNotEmpty(user.getPhone())) {
            user1.setPhone(user.getPhone());
        }
        if (StringUtils.isNotEmpty(user.getName())) {
            user1.setName(user.getName());
        }

        if (null != user.getImage()) {
            user1.setImage(user.getImage());
        }
        userMapper.updateById(user1);
        return user1;
    }

    @Override
    public void getSystemToken(String type, String token) {
        String userId = SecurityContextHolder.getStringUserId();
        if ("HUAWEI".equals(type)) {
            redisService.setCacheObject("HW:" + userId, token);
            redisService.setCacheObject(CommonConstant.ANDROID_AGENT + userId, "HW");
        } else if ("HONOR".equals(type)) {
            redisService.setCacheObject("HW:" + userId, token);
            redisService.setCacheObject(CommonConstant.ANDROID_AGENT + userId, "HW");
        } else if ("Xiaomi".equals(type)) {
            redisService.setCacheObject("XM:" + userId, token);
            redisService.setCacheObject(CommonConstant.ANDROID_AGENT + userId, "XM");
        } else if ("VIVO".equals(type)) {
            redisService.setCacheObject("VI:" + userId, token);
            redisService.setCacheObject(CommonConstant.ANDROID_AGENT + userId, "VI");
        } else if ("realme".equals(type)) {
            redisService.setCacheObject("OP:" + userId, token);
            redisService.setCacheObject(CommonConstant.ANDROID_AGENT + userId, "OP");
        } else if ("OPPO".equals(type)) {
            redisService.setCacheObject("OP:" + userId, token);
            redisService.setCacheObject(CommonConstant.ANDROID_AGENT + userId, "OP");
        } else {
            redisService.setCacheObject("OT:" + userId, token);
            redisService.setCacheObject(CommonConstant.ANDROID_AGENT + userId, "OT");
        }
    }

    /**
     * App 刷新令牌有效期
     *
     * @param user 登录信息
     */
    public void refreshToken(User user) {
        // 根据uuid将loginUser缓存
        String userKey = getTokenKey(user.getToken());
        redisService.setCacheObject(userKey, user, EXPIRATION, TimeUnit.DAYS);
        //终端只允许一台设备登录
        String redisUserKey = "user_keys:" + user.getUserEmail();
        redisService.setCacheObject(redisUserKey, user.getToken(), EXPIRATION, TimeUnit.DAYS);
    }

    private String getTokenKey(String token) {
        return APP_TOKEN_KEY + token;
    }

    private UserDeviceBindApp insertData(User user) {
        UserDeviceBindApp userDeviceBindApp = new UserDeviceBindApp();
        userDeviceBindApp.setId(UUID.randomUUID().toString(true));
        userDeviceBindApp.setUserId(user.getUserId());
        userDeviceBindApp.setUserEmail(user.getUserEmail());
        return userDeviceBindApp;
    }
}
