package com.ecell.internationalize.app.task;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.aliyun.oss.OSS;
import com.aliyun.oss.OSSClientBuilder;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.ecell.internationalize.app.entity.*;
import com.ecell.internationalize.app.entity.dto.DeviceInfoDTO;
import com.ecell.internationalize.app.entity.dto.HeartTemperatureAbnormal;
import com.ecell.internationalize.app.mapper.DeviceOwnerInfoAppMapper;
import com.ecell.internationalize.app.mapper.DeviceVideoAppMapper;
import com.ecell.internationalize.app.mapper.UserMapper;
import com.ecell.internationalize.app.service.*;
import com.ecell.internationalize.app.service.impl.*;
import com.ecell.internationalize.app.util.I18nUtil;
import com.ecell.internationalize.app.util.LocationUtil;
import com.ecell.internationalize.app.util.Md5Utils;
import com.ecell.internationalize.app.util.TimeUtil;
import com.ecell.internationalize.common.core.utils.DateUtils;
import com.ecell.internationalize.common.core.utils.StringUtils;
import com.ecell.internationalize.common.core.utils.bean.BeanUtils;
import com.ecell.internationalize.common.core.utils.uuid.UUID;
import com.ecell.internationalize.common.issue.component.IssueInstructions;
import com.ecell.internationalize.common.issue.constant.CommonConstant;
import com.ecell.internationalize.common.issue.entity.issuedto.*;
import com.ecell.internationalize.common.issue.entity.tcp.Lives;
import com.ecell.internationalize.common.issue.entity.tcp.PositionInfo;
import com.ecell.internationalize.common.issue.entity.tcp.WeatherResult;
import com.ecell.internationalize.common.issue.mqtt.dto.*;
import com.ecell.internationalize.common.issue.mqtt.push.MqttPush;
import com.ecell.internationalize.common.issue.util.ApplicationUtil;
import com.ecell.internationalize.common.issue.util.RandomUtil;
import com.ecell.internationalize.common.redis.service.RedisService;
import com.ecell.internationalize.common.system.constant.OssClientConstants;
import com.ecell.internationalize.common.system.entity.*;
import com.xiaomi.xmpush.server.Constants;
import com.xiaomi.xmpush.server.Message;
import com.xiaomi.xmpush.server.Result;
import com.xiaomi.xmpush.server.Sender;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.methods.RequestEntity;
import org.apache.commons.httpclient.methods.StringRequestEntity;

import org.apache.commons.httpclient.params.HttpMethodParams;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URL;
import java.security.NoSuchAlgorithmException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.Callable;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.ReentrantLock;
import java.util.stream.Collectors;

@Component
public class AppKafkaTask implements Callable {
    private static final Logger log = LoggerFactory.getLogger(AppKafkaTask.class);
    private static final double DISTANCE = 150.0;
    private Object object;
    private static final Object SYS_OBJECT = new Object();
    private static final Object LK_SYS_OBJECT = new Object();
    private static final ReentrantLock LOCK = new ReentrantLock();

    public AppKafkaTask(Object o) {
        this.object = o;
    }

    public AppKafkaTask() {
    }

    /**
     * 写具体逻辑
     *
     * @return
     * @throws Exception
     */
    @Override
    public Object call() throws Exception {
        log.info("线程Name:{}", Thread.currentThread().getName());
        appCallTask(object);
        return null;
    }

    private void appCallTask(Object o) throws ParseException, IOException, NoSuchAlgorithmException {
        if (StringUtils.isNotNull(o)) {
            String metadata = (String) o;
            if (StringUtils.isNotEmpty(metadata)) {
                JSONObject jsonObject = JSONObject.parseObject(metadata);
                String type = jsonObject.get("type").toString();
                String imei = jsonObject.get("imei").toString();
                //获取位置信息
                if ("location".equals(type)) {
                    log.info("TCP上传的位置信息:{}", jsonObject);
                    //经度
                    String lon = "";
                    //维度
                    String lat = "";
                    PositionInfo positionInfo = JSONObject.parseObject(jsonObject.toString(), PositionInfo.class);
                    log.info("转换后的数据:{}", positionInfo);
                    RedisService redisService = ApplicationUtil.getApplicationContext().getBean(RedisService.class);
                    if (StringUtils.isNotNull(positionInfo)) {
                        DeviceLocationApp app = LocationUtil.getLocation(positionInfo);
                        //  app.setImei(imei)
                        // DeviceLocationApp app=LocationUtil.getLocationGoogle(positionInfo);
                        log.info("获取到的位置信息:{}", app);
                        if (StringUtils.isNotNull(app)) {
                            String s = String.format("longitude:%s,latitude%s", app.getLongitude(), app.getLatitude());
                            //设置缓存最近一次定位的经纬度
                            redisService.setCacheMapValue(imei, "lnglat", s);
                            //设置定位时间
                            redisService.setCacheMapValue(imei, "lnglattime", String.valueOf(System.currentTimeMillis()));
                            //设置解析后的位置信息
                            log.warn("设置解析后的位置信息app.getAddr()={}", app.getAddr());
                            if (app.getAddr() != null)
                                redisService.setCacheMapValue(imei, "lnglataddress", app.getAddr());

                            lon = app.getLongitude();
                            lat = app.getLatitude();
                            synchronized (SYS_OBJECT) {
                                //1.把原始的定位信息也保存到数据库中
                                OriginalLocationInfoApp app1 = new OriginalLocationInfoApp();
                                app1.setId(com.ecell.internationalize.common.core.utils.uuid.UUID.fastUUID().toString(true));
                                app1.setOldInfo(metadata);
                                app1.setNewId(app.getId());
                                ApplicationUtil.getApplicationContext().getBean(OriginalLocationInfoAppService.class).save(app1);
                                if (StringUtils.isNotEmpty(app.getImei())) {
                                    //添加OR更新一条最新的位置信息到最新位置表中
                                    getLastLocationInfo(app, positionInfo);
                                }
                                //主动位置请求推送到APP
                                sendLocaltionApp(imei);
                                //安全围栏处理
                                pushSafeRails(imei, lon, lat);
                                //轨迹生成
                                log.info("位置信息的locationType是:{}", positionInfo.getLocationType());
                                if (StringUtils.isNotEmpty(positionInfo.getLocationType())) {
                                    //try {
                                    TrackInfoAppServiceImpl impl = ApplicationUtil.getApplicationContext().getBean(TrackInfoAppServiceImpl.class);

                                    //根据当前时间查询轨迹表
                                    List<TrackInfoApp> trackInfoApps = impl.queryTrackListByImei(imei);
                                    //查询所有的轨迹安创建时间倒序
                                    List<TrackInfoApp> trackInfoApps1 = impl.queryTrackListByImeiDes(imei);
                                    //  long time = LocationUtil.getZero(new Date()).getTime();
                                    if (StringUtils.isNotEmpty(trackInfoApps) && trackInfoApps.size() > 0) {
                                        //获取最后添加的一条
                                        TrackInfoApp appTwo = trackInfoApps.get(0);
                                        log.info("坐标点:精度1{}，维度1{}，精度2{}，维度2{}", appTwo.getLongitude(), appTwo.getLatitude(), app.getLongitude(), app.getLatitude());
                                        //获取两个坐标的距离
                                        double distance = LocationUtil.getDistance(Double.parseDouble(appTwo.getLongitude()), Double.parseDouble(appTwo.getLatitude()), Double.parseDouble(app.getLongitude()), Double.parseDouble(app.getLatitude()));
                                        //两坐标之间大于150米就新增轨迹点
                                        if (Math.abs(distance) > DISTANCE) {
                                            log.info("当前位置信息距离是否大于:{},距离:{}", distance > DISTANCE, distance);
                                            TrackInfoApp appThree = new TrackInfoApp();
                                            appThree.setAdcode(app.getAdcode());
                                            appThree.setImei(imei);
                                            appThree.setId(com.ecell.internationalize.common.core.utils.uuid.UUID.fastUUID().toString(true));
                                            appThree.setLatitude(app.getLatitude());
                                            appThree.setLongitude(app.getLongitude());
                                            appThree.setAddr(app.getAddr());
                                            appThree.setLocationType(app.getLocationType());
                                            appThree.setRadius(app.getRadius());
                                            appThree.setResidenceTime(CommonConstant.NUMBER_ZERO);
                                            appThree.setCreateTime(new Date());
                                            appThree.setUpdateTime(new Date());
                                            //计算停留时间
                                            //  int abs = (int) Math.abs(appTwo.getUpdateTime().getTime() - appTwo.getCreateTime().getTime())/1000;
                                            // int abs = (int) (Math.abs(System.currentTimeMillis() - appTwo.getUpdateTime().getTime()) / 1000);
                                            int abs = (int) (Math.abs(System.currentTimeMillis() - appTwo.getCreateTime().getTime()) / 1000);
                                            appThree.setResidenceTime(0);
                                            appTwo.setResidenceTime(abs);

                                            //新增OR更新
                                            impl.saveOrUpdate(appThree, appTwo);


                                        } else {
                                            log.info("当前位置信息距离:不大于{}", distance);
                                            //不大于就更新当前的轨迹点
                                            //   appTwo.setResidenceTime((int) Math.abs(date.getTime() - appTwo.getCreateTime().getTime())/1000);
                                            //  appTwo.setResidenceTime((int) Math.abs(date.getTime() - time) / 1000);
                                            Date date = new Date();
                                            appTwo.setUpdateTime(date);
                                            Long l = date.getTime() - appTwo.getCreateTime().getTime();
                                            appTwo.setResidenceTime((int) Math.abs(l) / 1000);
                                            appTwo.setUpdateTime(date);
                                            //更新
                                            impl.updateById(appTwo);

                                        }


                                    } else {
                                        //取最后一条
                                        TrackInfoApp trackInfoApp = null;
                                        if (StringUtils.isNotEmpty(trackInfoApps1) && trackInfoApps1.size() > 0) {
                                            long time = 0;
                                            //更新前一条的停留时间
                                            trackInfoApp = trackInfoApps1.get(0);
                                            //   time = trackInfoApp.getUpdateTime().getTime();
                                            time = trackInfoApp.getCreateTime().getTime();
                                            Long l = System.currentTimeMillis() - time;
                                            trackInfoApp.setResidenceTime((int) Math.abs(l) / 1000);
                                        }
                                        //说明当天还没有轨迹记录初始化轨迹记录信息
                                        log.info("开始初始化轨迹信息");
                                        TrackInfoApp appFour = new TrackInfoApp();
                                        appFour.setAdcode(app.getAdcode());
                                        appFour.setImei(imei);
                                        appFour.setId(com.ecell.internationalize.common.core.utils.uuid.UUID.fastUUID().toString(true));
                                        appFour.setLatitude(app.getLatitude());
                                        appFour.setLongitude(app.getLongitude());
                                        appFour.setAddr(app.getAddr());
                                        appFour.setLocationType(app.getLocationType());
                                        appFour.setRadius(app.getRadius());
                                        appFour.setResidenceTime(0);
                                        //计算当前停留时间
                                        // appFour.setResidenceTime((int) (System.currentTimeMillis() - time) / 1000);
                                        impl.saveOrUpdate(appFour, trackInfoApp);


                                    }
                                    //保存电量
                                    saveOrUpdate(positionInfo.getImei(), Integer.valueOf(positionInfo.getBattery()));
                                    //获取步数上传到数据库
                                    insertOrUpdateStep(positionInfo.getImei(), positionInfo.getStepNumber());

                                    //翻滚次数，根据imei查询设备表
                                    saveOrUpdateRollNumber(positionInfo.getImei(), Integer.valueOf(positionInfo.getRollNumber()));

                                }


                                //  } catch (Exception e) {
                                //   log.info("出现异常:{}", e.getMessage());

                                //  } finally {
                                // LOCK.unlock();
                                // }
//                                //低电量推送
//                                if ("lowPowerAlarm".equals(positionInfo.getLocationType())) {
//                                    log.info("低电推送开始:{}",positionInfo.getBattery());
//                                    sendBattery(positionInfo.getImei(), "lowBattery", positionInfo.getBattery());
//                                }
                                //sos报警信息
                                if ("sos".equals(positionInfo.getLocationType())) {
                                    log.info("收到SOS位置信息,开始发送信息");
                                    sendSOS(imei, "sos", app);


                                }

                            }


                            //主动位置请求推送到App
////                        RedisService redisService = ApplicationUtil.getApplicationContext().getBean(RedisService.class);
//                        //根据imei查询绑定表
//                        BindingDeviceImpl bindingDevice = ApplicationUtil.getApplicationContext().getBean(BindingDeviceImpl.class);
//                        List<UserDeviceBindApp> userIdList = bindingDevice.getUserIdList(imei);
                        }
                    }

                } else if ("LK".equals(type)) {
                    log.info("LK指令:{}", jsonObject);
                    //链路保持 step 步数 battery 电量百分比 翻滚次数 rollNum,
                    //获取Imei 此处获取电量
                    // String imei = jsonObject.get("imei").toString();
                    //电量
                    String batter = jsonObject.get("battery").toString();
                    //翻滚次数
                    String rollNum = jsonObject.get("rollNum").toString();
                    //步数
                    String step = jsonObject.get("step").toString();
                    if (StringUtils.isNotEmpty(imei)) {
                        synchronized (SYS_OBJECT) {
                            DeviceOwnerInfoAppServiceImpl deviceOwnerInfoAppServiceImpl = ApplicationUtil.getApplicationContext().getBean(DeviceOwnerInfoAppServiceImpl.class);
                            DeviceOwnerInfoApp app = deviceOwnerInfoAppServiceImpl.queryOwnerInfoByimei(imei);
                            if (StringUtils.isNotNull(app)) {
                                app.setBattery(Integer.valueOf(batter));
                                app.setRollNum(Integer.valueOf(rollNum));
                                app.setStep(Integer.valueOf(step));
                                //更新设备表
                                deviceOwnerInfoAppServiceImpl.saveOrUpdate(app);
                            }
                            //获取步数上传到数据库
                            insertOrUpdateStep(imei, step);
                            //LK指令上来的低电量也要推
                            //  BatteryInfoApp batteryInfo = getBatteryInfo(ApplicationUtil.getApplicationContext().getBean(BatteryInfoAppServiceImpl.class), imei);
                            //保存电量
                            Integer battery = Integer.valueOf(batter);
                            saveOrUpdate(imei, battery);
                            //LK指令上来的正常电量也要推送
                            sendLKBattery(imei, "battery", batter);
//                            if (StringUtils.isNotNull(batteryInfo)) {
//                                if (batteryInfo.getBattery() != Integer.parseInt(batter) && battery <= 10 && CommonConstant.AUDIT_STATUS_ONE.equals(batteryInfo.getLowBatteryAlarm())&& batteryInfo.getBattery()>=10) {
//                                    log.info("LK指令低电开始推送:{}", battery);
//                                    sendBattery(imei, "lowBattery", battery.toString());
//                                }
//                            } else {
//                                if (battery <= 10) {
//                                    log.info("LK指令低电开始推送:{}", battery);
//                                    sendBattery(imei, "lowBattery", battery.toString());
//                                }
//                            }

                            //更新最新位置信息表
                            LatestLocationApp latestLocationApp = queryLocationByImei(imei);
                            if (StringUtils.isNotNull(latestLocationApp)) {//存在这张表就更新，不存在就忽略
                                updateLocationByImei(latestLocationApp, batter, step, null, null, null);
                            }
                        }
                        //更新设备首次登录时间
                        VideoCallAppServiceImpl videoCall = ApplicationUtil.getApplicationContext().getBean(VideoCallAppServiceImpl.class);
                        DeviceInfoDTO deviceInfoDTO = videoCall.queryDeviceDTOByImei(imei);
                        RedisService redisService = ApplicationUtil.getApplicationContext().getBean(RedisService.class);
                        Object firstLogin = redisService.getCacheMapValue(imei, "firstLogin");
                        Object lastLogin = redisService.getCacheMapValue(imei, "lastlogin");
                        log.info("获取到的首次登录时间是:{},最后登录时间{}", firstLogin, lastLogin);
                        if (StringUtils.isNotNull(firstLogin) && StringUtils.isNotNull(deviceInfoDTO)) {
                            Long l = Long.parseLong(firstLogin.toString()) * 1000;
                            if (StringUtils.isNull(deviceInfoDTO.getFirstLoginTime())) {
                                Date firstLoginTime = new Date(l);
                                log.info("获取到的首次登录时间是:{}", firstLoginTime);
                                videoCall.UpdateDeviceInfoByImei(imei, firstLoginTime, null);

                            } else {
                                //设备解绑会重置首次登录时间
                                if (deviceInfoDTO.getFirstLoginTime().getTime() != l) {
                                    Date firstLoginTime = new Date(l);
                                    log.info("获取到的首次登录时间是:{}", firstLoginTime);
                                    videoCall.UpdateDeviceInfoByImei(imei, firstLoginTime, null);
                                }
                            }

                        }
                        if (StringUtils.isNotNull(lastLogin) && StringUtils.isNotNull(deviceInfoDTO)) {
                            Date lastLoginTime = new Date(Long.parseLong(lastLogin.toString()) * 1000);
                            log.info("获取到的最后登录时间是:{}", lastLoginTime);
                            videoCall.UpdateDeviceInfoByImei(imei, null, lastLoginTime);
                        }


                    }

                }
                //设备语音微聊上传
                else if ("upChatVoice".equals(jsonObject.get("type").toString())) {
                    try {
                        if (StringUtils.isNotEmpty(imei)) {
                            //根据imei获取groupId
                            LambdaQueryWrapper<ChantGroupApp> chantGroupAppLambdaQueryWrapper = new LambdaQueryWrapper<>();
                            chantGroupAppLambdaQueryWrapper.eq(ChantGroupApp::getImei, imei);
                            ChantGroupApp chantGroupApp = ApplicationUtil.getApplicationContext().getBean(ChantGroupAppService.class).getBaseMapper().selectOne(chantGroupAppLambdaQueryWrapper);
                            if (null != chantGroupApp) {
                                //1.根据groupId查询群组下面所有的群成员
                                LambdaQueryWrapper<ChatGroupMemberApp> lambdaQueryWrapper = new LambdaQueryWrapper<>();
                                lambdaQueryWrapper.eq(ChatGroupMemberApp::getGroupId, chantGroupApp.getId());

                                List<ChatGroupMemberApp> chatGroupMessageAppList = ApplicationUtil.getApplicationContext().getBean(ChatGroupMemberAppService.class).getBaseMapper().selectList(lambdaQueryWrapper);
                                ChatMessage chatMessage;
                                //2.userId和groupId一起批量入库
                                List<ChatGroupMessageApp> list = new ArrayList<>();
                                //语音文件大小
                                String size = jsonObject.get("size").toString();
                                //语音文件路径
                                String path = jsonObject.get("voicePath").toString();
                                //创建oss实例 OSSClientBuilder
                                OSS ossClient = new OSSClientBuilder().build(OssClientConstants.ENDPOINT, OssClientConstants.ACCESS_KEY_ID, OssClientConstants.ACCESS_KEY_SECRET);
                                //OSSObject object = ossClient.getObject(OssClientConstants.BUCKET_NAME, path);
                                // 设置URL过期时间为100年，默认这里是int型，转换为long型即可
                                Date expiration = new Date(System.currentTimeMillis() + 3600L * 1000 * 24 * 365 * 100);
                                // 生成URL
                                if (path.contains("http") || path.contains("https")){

                                }else {
                                    URL url = ossClient.generatePresignedUrl(OssClientConstants.BUCKET_NAME, path, expiration);
                                    path=url.toString();
                                }

                                //语音文件时长
                                String duration = jsonObject.get("duration").toString();
                                //群组下其他用户存储设备上来的语音
                                if (!CollectionUtils.isEmpty(chatGroupMessageAppList)) {
                                    for (ChatGroupMemberApp chatGroupMemberApp : chatGroupMessageAppList) {
                                        //封装数据
                                        ChatGroupMessageApp chatGroupMessageApp = new ChatGroupMessageApp();
                                        chatGroupMessageApp.setUserId(chatGroupMemberApp.getUserId());
                                        chatGroupMessageApp.setId(com.ecell.internationalize.common.core.utils.uuid.UUID.fastUUID().toString(true));
                                        chatGroupMessageApp.setSize(Integer.parseInt(size));
                                        chatGroupMessageApp.setDuration(Integer.parseInt(duration));
                                        //用户类型：1.用户 2.设备
                                        chatGroupMessageApp.setSendType("2");
                                        //消息类型：1:文字消息, 2:图片消息, 3:语音消息',
                                        chatGroupMessageApp.setMyType("3");
                                        chatGroupMessageApp.setCreateTime(new Date());
                                        //群组id
                                        chatGroupMessageApp.setGroupId(chantGroupApp.getId());
                                        //语音内容
                                    //    chatGroupMessageApp.setContent(url.toString());
                                        chatGroupMessageApp.setContent(path);
                                        chatGroupMessageApp.setSendUser(imei);
                                        chatGroupMessageApp.setReadStatus("0");
                                        list.add(chatGroupMessageApp);
                                        //发送到mqtt里面去
                                        chatMessage = new ChatMessage();
                                        chatMessage.setType("chatGroupMessage");
                                        chatMessage.setBadge(1);
                                        chatMessage.setGroupid(chatGroupMessageApp.getGroupId());
                                        ChatMessageChild chatMessageChild = new ChatMessageChild();
                                        chatMessageChild.setId(chatGroupMessageApp.getId());
                                        chatMessageChild.setContent(chatGroupMessageApp.getContent());
                                        chatMessageChild.setDuration(chatGroupMessageApp.getDuration());
                                        chatMessageChild.setSendUser(chatGroupMessageApp.getSendUser());
                                        chatMessageChild.setSendType(chatGroupMessageApp.getSendType());

                                        chatMessageChild.setSenderType(chatGroupMessageApp.getSendType());
                                        chatMessageChild.setType(chatGroupMessageApp.getMyType());
                                        chatMessageChild.setSenderID(imei);
                                        chatMessageChild.setTimestamp(System.currentTimeMillis());
                                        chatMessageChild.setImei(imei);
                                        chatMessageChild.setMyType(chatGroupMessageApp.getMyType());
                                        chatMessageChild.setReadStatus("0");
                                        chatMessage.setMessage(chatMessageChild);
                                        String sendPushToChatGroup = JSONObject.toJSONString(chatMessage);
                                        //5.发送到mqtt
                                        String userId = chatGroupMessageApp.getUserId();
                                        String lang = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheMapValue(userId, CommonConstant.IOS_LANGUAGE);
                                        String agent = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheMapValue(userId, CommonConstant.USER_AGENT);
                                        String alert = I18nUtil.getMessage(CommonConstant.ACCEPT_CHANT_VIDEO, lang);
                                        //判断是发IOS还是安卓
                                        if (CommonConstant.ANDROID.equals(agent)) {
                                            ApplicationUtil.getApplicationContext().getBean(MqttPush.class).sendMqtt("1", userId, "1", "chatGroupMessage", "", sendPushToChatGroup);
                                        } else {
                                            ApplicationUtil.getApplicationContext().getBean(BindingDeviceImpl.class).sendIOS(userId, chatMessage, "chatGroupMessage", "", alert);
                                        }
                                        //安卓系统推送
                                        if ("Android".equals(agent)) {
                                            //手机厂商机型：HW、XM等
                                            String AndroidType = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheObject(CommonConstant.ANDROID_AGENT + userId);
                                            if (StringUtils.isNotEmpty(AndroidType)) {
                                                if ("HW".equals(AndroidType)) {
                                                    //获取对应的token
                                                    String deviceToken = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheObject("HW:" + userId);
                                                    //服务端获取华为的token
                                                    String huaweiAccessToken = "";
                                                    if (null != ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheObject("huaweiAccessToken")) {
                                                        huaweiAccessToken = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheObject("huaweiAccessToken");
                                                    } else {
                                                        huaweiAccessToken = getHuaweiAccessToken();
                                                        ApplicationUtil.getApplicationContext().getBean(RedisService.class).setCacheObject("huaweiAccessToken", huaweiAccessToken, 2L, TimeUnit.HOURS);
                                                    }
                                                    JSONObject first = new JSONObject();
                                                    first.put("validate_only", false);
                                                    JSONObject titleObject = new JSONObject();
                                                    titleObject.put("title", I18nUtil.getMessage(CommonConstant.CHANT_VIDEO, lang));
                                                    titleObject.put("body", alert);
                                                    JSONObject notificationObject = new JSONObject();
                                                    notificationObject.put("notification", titleObject);
                                                    JSONObject clickActionObject = new JSONObject();
                                                    clickActionObject.put("type", 1);
                                                    clickActionObject.put("action", "com.lepeiban.chat.main");
                                                    JSONObject notificationClick = new JSONObject();
                                                    notificationClick.put("notification", clickActionObject);
                                                    notificationObject.put("android", notificationClick);
                                                    notificationObject.put("token", deviceToken);
                                                    notificationObject.put("data", chatMessage);
                                                    first.put("message", notificationObject);
                                                    String huaweiMsgData = JSONObject.toJSONString(first);
                                                    pushMessage(userId, huaweiMsgData, huaweiAccessToken);
                                                } else if ("XM".equals(AndroidType)) {
                                                    //获取对应的token
                                                    String deviceToken = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheObject("XM:" + userId);
                                                    String intentUri = "intent:#Intent;component=com.ecellsz.watch.guard/com.lk.chatlibrary.activities.chat.ChatActivity;end";
                                                    pushMiMessage(I18nUtil.getMessage(CommonConstant.CHANT_VIDEO, lang), alert, sendPushToChatGroup, deviceToken, intentUri);
                                                } else if ("VI".equals(AndroidType)) {
                                                    //获取对应的token
                                                    String deviceToken = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheObject("VI:" + userId);
                                                    //服务端获取VIVO的token
                                                    String viVoAccessToken = "";
                                                    if (null != ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheObject("viVoAccessToken")) {
                                                        viVoAccessToken = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheObject("viVoAccessToken");
                                                    } else {
                                                        viVoAccessToken = getViVoAccessToken();
                                                        ApplicationUtil.getApplicationContext().getBean(RedisService.class).setCacheObject("viVoAccessToken", viVoAccessToken, 12L, TimeUnit.HOURS);
                                                    }
                                                    JSONObject msgJson = new JSONObject();
                                                    msgJson.put("regId", deviceToken);
                                                    msgJson.put("notifyType", 4);
                                                    msgJson.put("title", I18nUtil.getMessage(CommonConstant.CHANT_VIDEO, lang));
                                                    msgJson.put("content", alert);
                                                    msgJson.put("skipType", 4);
                                                    msgJson.put("skipContent", "intent://com.ecellsz.push.chat/detail#Intent;scheme=vpushscheme;component=com.ecellsz.watch.guard/com.lk.chatlibrary.activities.chat.ChatActivity;end");
                                                    msgJson.put("requestId", com.ecell.internationalize.common.core.utils.uuid.UUID.fastUUID().toString(true));
                                                    msgJson.put("clientCustomMap", chatMessage);
                                                    msgJson.put("classification", 1);
                                                    sendViVoMessage(viVoAccessToken, msgJson, userId);
                                                } else if ("OP".equals(AndroidType)) {
                                                    //获取对应的token
                                                    String deviceToken = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheObject("OP:" + userId);
                                                    //服务端获取OPPO的token
                                                    String opPoAccessToken = "";
                                                    if (null != ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheObject("opPoAccessToken")) {
                                                        opPoAccessToken = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheObject("opPoAccessToken");
                                                    } else {
                                                        opPoAccessToken = getOpPoAccessToken();
                                                        ApplicationUtil.getApplicationContext().getBean(RedisService.class).setCacheObject("opPoAccessToken", opPoAccessToken, 12L, TimeUnit.HOURS);
                                                    }
                                                    JSONObject postData = new JSONObject();
                                                    postData.put("target_type", 2);
                                                    postData.put("target_value", deviceToken);
                                                    JSONObject postData1 = new JSONObject();
                                                    postData1.put("title", I18nUtil.getMessage(CommonConstant.CHANT_VIDEO, lang));
                                                    postData1.put("content", alert);
                                                    postData1.put("click_action_type", 1);
                                                    postData1.put("click_action_activity", "com.lepeiban.chat.main");
                                                    postData1.put("action_parameters", chatMessage);
                                                    postData.put("notification", postData1);
                                                    String str = JSONObject.toJSONString(postData);
                                                    sendOpPoMessage(opPoAccessToken, str, userId);
                                                }
                                            }
                                        }
                                    }
                                    ApplicationUtil.getApplicationContext().getBean(ChatGroupMessageAppService.class).saveBatch(list);
                                }

                                // ApplicationUtil.getApplicationContext().getBean(ChatGroupMessageAppService.class).getBaseMapper().insert(chatGroupMessageApp);
                            }
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                    //获取Imei
                    //  String imei = jsonObject.get("imei").toString();

                    //心率
                } else if ("upHeartRate".equals(type)) {
                    String data = jsonObject.get("data").toString();
                    log.info("上传心率是:{}", data);
                    LatestLocationApp latestLocationApp = queryLocationByImei(imei);
                    if (StringUtils.isNotNull(latestLocationApp)) {
                        latestLocationApp.setRateTime(new Date());
                        updateLocationByImei(latestLocationApp, null, null, data, null, null);
                    }
                    //处理心率上传
                    handlerHeartRate(imei, "1", data, jsonObject.get("time").toString());
                    //获取设备首次登录时间以及最后的登录时间
                }
                //血氧
                else if ("upBodyOxygen".equals(type)) {
                    String data = jsonObject.get("data").toString();
                    log.info("上传血氧是:{}", data);
                    LatestLocationApp latestLocationApp = queryLocationByImei(imei);
                    if (StringUtils.isNotNull(latestLocationApp)) {
                        latestLocationApp.setBodyOxygenTime(new Date());
                        updateLocationByImei(latestLocationApp, null, null, null, null, data);
                    }

                    String id = com.ecell.internationalize.common.core.utils.uuid.UUID.fastUUID().toString(true);
                    HeartTemperatureRecord heartTemperatureRecord = new HeartTemperatureRecord();
                    heartTemperatureRecord.setId(id);
                    heartTemperatureRecord.setImei(imei);
                    heartTemperatureRecord.setHeartTemperatureRate(data);
                    heartTemperatureRecord.setCreateTime(new Date());
                    heartTemperatureRecord.setType("3");
                    heartTemperatureRecord.setTimeStamp(Integer.parseInt(jsonObject.get("time").toString()));
                    ApplicationUtil.getApplicationContext().getBean(HeartTemperatureRecordService.class).getBaseMapper().insert(heartTemperatureRecord);
                    //处理血氧上传
                    handlerHeartRate(imei, "3", data, jsonObject.get("time").toString());

                    // handlerBodyOxygen(imei, "1", data, jsonObject.get("time").toString());


                } else if ("loginTime".equals(type)) {
                    float firstLoginTime = (float) jsonObject.get("firstLoginTime");
                    float lastLoginTime = (float) jsonObject.get("lastLoginTime");
                    log.info("设备首次登录时间:{}，最后登录时间:{}", firstLoginTime, lastLoginTime);
                    VideoCallAppServiceImpl videoCallInfo = ApplicationUtil.getApplicationContext().getBean(VideoCallAppServiceImpl.class);
                    DeviceInfoDTO deviceInfoDTO = videoCallInfo.queryDeviceDTOByImei(imei);
                    if (StringUtils.isNotNull(deviceInfoDTO)) {
                        //登录时间不为null
                        if (StringUtils.isNotNull(deviceInfoDTO.getFirstLoginTime())) {
                            if (0.0 != lastLoginTime) {
                                long l = Float.valueOf(lastLoginTime * 1000).longValue();
                                deviceInfoDTO.setLastLoginTime(new Date(l));
                            }

                        } else {
                            if (0.0 != firstLoginTime && StringUtils.isNull(deviceInfoDTO.getFirstLoginTime())) {
                                long l = Float.valueOf(firstLoginTime * 1000).longValue();
                                deviceInfoDTO.setFirstLoginTime(new Date(l));
                            }

                        }
                        //更新
                        videoCallInfo.UpdateDeviceInfoByImei(imei, deviceInfoDTO.getFirstLoginTime(), deviceInfoDTO.getLastLoginTime());


                    }
                }
                //体温
                else if ("upBodyTemperature".equals(type)) {
                    String data = jsonObject.get("data").toString();
                    LatestLocationApp latestLocationApp = queryLocationByImei(imei);
                    if (StringUtils.isNotNull(latestLocationApp)) {
                        latestLocationApp.setTemperatureTime(new Date());
                        updateLocationByImei(latestLocationApp, null, null, null, data, null);
                    }
                    //处理体温上传
                    handlerHeartRate(imei, "2", data, jsonObject.get("time").toString());
                }
                /**可视频通话用户列表下发*/
                else if ("upGetVideoUserList".equals(jsonObject.get("type"))) {
                    DownGetVideoUserList downGetVideoUserList = new DownGetVideoUserList();
                    downGetVideoUserList.setImei(imei);
                    downGetVideoUserList.setIdent("12345678");
                    downGetVideoUserList.setType("downGetVideoUserList");
                    downGetVideoUserList.setVender("20000");
                    downGetVideoUserList.setTime(System.currentTimeMillis());
                    List<VideoUser> videoUserList = new ArrayList<>();
                    VideoUser videoUser;
                    //根据imei 查询 可视频列表
                    LambdaQueryWrapper<DeviceVideoApp> deviceVideoAppLambdaQueryWrapper = new LambdaQueryWrapper<>();
                    deviceVideoAppLambdaQueryWrapper.eq(DeviceVideoApp::getImei, imei);
                    List<DeviceVideoApp> deviceVideoAppList = ApplicationUtil.getApplicationContext().getBean(DeviceVideoAppService.class).getBaseMapper().selectList(deviceVideoAppLambdaQueryWrapper);
                    if (CollectionUtils.isEmpty(deviceVideoAppList)) {
                        log.info("可视频列表为空,不进行下发");
                        return;
                    }
                    for (DeviceVideoApp deviceVideoApp : deviceVideoAppList) {
                        String key = imei + "__" + deviceVideoApp.getUserId();

                        //查询用户绑定的信息
                        Map<String, UserDeviceBindApp> map = ApplicationUtil.getApplicationContext().getBean(DeviceVideoAppServiceImpl.class).queryUserBindApp();
                        if (null != map.get(key)) {
                            String relationShip = map.get(key).getRelation();
                            Integer relationShipImageId = Integer.valueOf(map.get(key).getRelationImageId());
                            String phone = map.get(key).getPhone();
                            //内部list数据封装
                            videoUser = new VideoUser();
                            videoUser.setCanVideo(CommonConstant.NUMBER_ONE);
                            videoUser.setPhone(phone);
                            videoUser.setRelationship(relationShip);
                            videoUser.setRelationship_image_id(relationShipImageId);
                            videoUser.setUser_openid(deviceVideoApp.getUserId());

                            // videoUser.setUser_openid(deviceVideoApp.getVideoId());
                            //  videoUser.setVideoType(deviceVideoApp.getVideoType());
                            //声网
                            if (CommonConstant.AUDIT_STATUS_TWO.equals(deviceVideoApp.getVideoType())) {
                                videoUser.setBairui_id(String.valueOf(deviceVideoApp.getVideoId()));
                                videoUser.setVideoType("V0SW");
                            }
                            //如果是百瑞视频就要使用bairui_id，其余的视频就使用user_openid
                            if (CommonConstant.AUDIT_STATUS_THREE.equals(deviceVideoApp.getVideoType())) {
                                //视频类型
                                videoUser.setVideoType("V0BR");
                                videoUser.setBairui_id(String.valueOf(deviceVideoApp.getVideoId()));
                            }
                            videoUserList.add(videoUser);
                        }
                    }
                    downGetVideoUserList.setUser_list(videoUserList);


//                    //根据imei 查询 可视频列表
//                    VideoCallAppServiceImpl videoCallAppServiceImpl = ApplicationUtil.getApplicationContext().getBean(VideoCallAppServiceImpl.class);
//                    BindingDeviceImpl bindingDeviceImpl = ApplicationUtil.getApplicationContext().getBean(BindingDeviceImpl.class);
//                    List<VideoCallApp> videoCallApps = videoCallAppServiceImpl.queryVideoList(imei);
//                    List<VideoUser> videoUsers = new ArrayList<>();
//                    if (StringUtils.isNotEmpty(imei)) {
//                        videoUsers = videoCallApps.stream().map(s -> {
//                            VideoUser videoUser = new VideoUser();
//                            videoUser.setCanVideo(CommonConstant.NUMBER_ONE);
//                            //根据用户id,查询绑定表，查询手机号码及与手表的关系
//                            UserDeviceBindApp userDeviceBindApp = bindingDeviceImpl.getBingInfoByUserId(s.getUserId(), imei);
//                            videoUser.setPhone(userDeviceBindApp.getPhone());
//                            videoUser.setRelationship(userDeviceBindApp.getRelation());
//                            //如果是百瑞视频就要使用bairui_id，其余的视频就使用user_openid
//                            if (CommonConstant.AUDIT_STATUS_ONE.equals(s.getVideoType())) {
//                                videoUser.setVideoType("V0BR"); //视频类型
//                                videoUser.setBairui_id(s.getId().toString());
//                            } else {
//                                videoUser.setUser_openid(s.getUserId());
//                            }
//                            videoUser.setRelationship_image_id(1);
//                            videoUser.setCanVideo(CommonConstant.NUMBER_ONE);
//
//                            return videoUser;
//                        }).collect(Collectors.toList());
//                    }
//
//                    downGetVideoUserList.setUser_list(videoUsers);

                    IssueInstructions issueInstructions = ApplicationUtil.getApplicationContext().getBean(IssueInstructions.class);
                    //下发
                    issueInstructions.emitDeviceDemand("downGetVideoUserList", imei, downGetVideoUserList);
                    log.info("可视频用户列表下发成功");
                }


                /** 手表发起和APP/手表视频通话*/
                else if ("upVideoCallWithAPP".equals(type)) {
                    log.info("解析手表发起 ===>APP/手表视频通话指令");
                    log.info("获取到的视频通话的参数：{}", jsonObject);
                    //视频类型
                    String deviceVideoType = jsonObject.get("deviceVideoType").toString();

                    int uid = RandomUtil.generateVerCodeInt(String.valueOf(System.currentTimeMillis()));
                    //视频通话类型
                    String callType = jsonObject.containsKey("call_type") ? jsonObject.get("call_type").toString() : "0";
                    //指令类型
                    String downType = jsonObject.get("type").toString();

                    String baiRuiId = "";
                    //失败原因进行指令下发 downVideoCallFailInfo
                    DownVideoCallFailInfo downVideoCallFailInfo = new DownVideoCallFailInfo();
                    downVideoCallFailInfo.setIdent("12345678");
                    downVideoCallFailInfo.setVender("20000");
                    downVideoCallFailInfo.setTime(System.currentTimeMillis());
                    downVideoCallFailInfo.setImei(imei);
                    downVideoCallFailInfo.setType("downVideoCallFailInfo");
                    //发起和APP的通话
                    if (jsonObject.containsKey("user_openid")) {
                        String userId = jsonObject.get("user_openid").toString();
                        log.info("手表发起和APP的通话用户：{}", userId);
//                        if (CommonConstant.VIDEO_TYPE_THREE.equals(deviceVideoType)){
//                            deviceVideoType=CommonConstant.VIDEO_TYPE_TWO;
//                        }
                        //判断是否是佰锐视频
                        if (CommonConstant.VIDEO_TYPE_THREE.equals(deviceVideoType) || CommonConstant.VIDEO_TYPE_TWO.equals(deviceVideoType)) {
                            baiRuiId = jsonObject.get("user_openid").toString();
                            LambdaQueryWrapper<DeviceVideoApp> lambdaQueryWrapper = new LambdaQueryWrapper<>();
                            lambdaQueryWrapper.eq(DeviceVideoApp::getVideoId, baiRuiId);
                            lambdaQueryWrapper.eq(DeviceVideoApp::getImei, imei);
                            DeviceVideoApp deviceVideoApp = ApplicationUtil.getApplicationContext().getBean(DeviceVideoAppService.class).getBaseMapper().selectOne(lambdaQueryWrapper);
                            if (null != deviceVideoApp) {
                                userId = deviceVideoApp.getUserId();
                            } else {
                                log.info("佰锐视频用户:{},已被移除圈子,程序终止", userId);
                                return;
                            }
                        }
                        int data = ApplicationUtil.getApplicationContext().getBean(DeviceVideoAppService.class).videoCall(imei, callType, downType, userId, null, deviceVideoType, String.valueOf(uid));
                        //邀请视频通话成功 并成功创建记录
                        if (CommonConstant.NUMBER_ONE == data) {
                            //1.获取视频配置信息
                            Map<String, Object> videoConfig = ApplicationUtil.getApplicationContext().getBean(DeviceVideoAppServiceImpl.class).videoConfig(imei, callType);
                            LambdaQueryWrapper<DeviceOwnerInfoApp> lambdaQueryWrapper = new LambdaQueryWrapper<>();
                            lambdaQueryWrapper.eq(DeviceOwnerInfoApp::getImei, imei);
                            DeviceOwnerInfoApp deviceOwnerInfoApp = ApplicationUtil.getApplicationContext().getBean(DeviceOwnerInfoAppService.class).getBaseMapper().selectOne(lambdaQueryWrapper);
                            DownVideoCallWithWatch downVideoCallWithWatch = new DownVideoCallWithWatch();
                            downVideoCallWithWatch.setUsername(userId);
                            downVideoCallWithWatch.setPassword(imei.substring(9));
                            downVideoCallWithWatch.setTimestamp(System.currentTimeMillis());
                            downVideoCallWithWatch.setImei(imei);
                            downVideoCallWithWatch.setDevice_username("imei-" + imei);
                            downVideoCallWithWatch.setVideo_id(null != deviceOwnerInfoApp ? deviceOwnerInfoApp.getNewVideoId() : "查询空值");
                            downVideoCallWithWatch.setLimit_time(Integer.parseInt(videoConfig.get("oneCallTime").toString()));
                            downVideoCallWithWatch.setWait_time(CommonConstant.WAIT_TIME);
                            downVideoCallWithWatch.setDistinguishability_width(Integer.parseInt(videoConfig.get("width").toString()));
                            downVideoCallWithWatch.setDistinguishability_heiger(Integer.parseInt(videoConfig.get("height").toString()));
                            downVideoCallWithWatch.setOpenid(userId);
                            downVideoCallWithWatch.setType("video");
                            if (CommonConstant.VIDEO_TYPE_ONE.equals(deviceVideoType)) {
                                downVideoCallWithWatch.setDeviceVideoType(CommonConstant.VIDEO_TYPE_ONE);
                                downVideoCallWithWatch.setAppkey(CommonConstant.JF_VIDEO_KEY);
                                downVideoCallWithWatch.setVideo_type(4);
                            }
                            if (CommonConstant.VIDEO_TYPE_TWO.equals(deviceVideoType)) {
                                //获取设备登陆视频的id
                                LambdaQueryWrapper<DeviceVideoApp> queryWrapper = new LambdaQueryWrapper<>();
                                queryWrapper.eq(DeviceVideoApp::getImei, imei);
                                queryWrapper.eq(DeviceVideoApp::getUserId, userId);

                                DeviceVideoApp deviceVideoApp = ApplicationUtil.getApplicationContext().getBean(DeviceVideoAppMapper.class).selectOne(queryWrapper);

                                downVideoCallWithWatch.setDeviceVideoType(CommonConstant.VIDEO_TYPE_TWO);
                                downVideoCallWithWatch.setVideo_type(1);
                                downVideoCallWithWatch.setChannelID(imei);
//                                downVideoCallWithWatch.setDevice_uid(1111);
//                                downVideoCallWithWatch.setApp_uid(2222);
                                downVideoCallWithWatch.setDevice_uid(uid);
                                // String baiRuiIdSW = jsonObject.get("user_openid").toString();
                                log.info("获取到的声网id是:{}", deviceVideoApp.getVideoId());
                                downVideoCallWithWatch.setApp_uid(deviceVideoApp.getVideoId());

                                downVideoCallWithWatch.setAppid(CommonConstant.SW_VIDEO_KEY);
                            }
                            if (CommonConstant.VIDEO_TYPE_THREE.equals(deviceVideoType)) {
                                downVideoCallWithWatch.setDeviceVideoType(CommonConstant.VIDEO_TYPE_THREE);
                                downVideoCallWithWatch.setAppkey(CommonConstant.BR_VIDEO_KEY);
                                downVideoCallWithWatch.setPassword("123456");
                                String deviceBaiRuiId = null != deviceOwnerInfoApp ? String.valueOf(deviceOwnerInfoApp.getBaiRuiId()) : "";
                                downVideoCallWithWatch.setDevice_username(deviceBaiRuiId);
                                downVideoCallWithWatch.setUsername(baiRuiId);
                                downVideoCallWithWatch.setVideo_type(5);
                            }
                            if (CommonConstant.VIDEO_TYPE_FOUR.equals(deviceVideoType)) {
                                downVideoCallWithWatch.setDeviceVideoType(CommonConstant.VIDEO_TYPE_FOUR);
                                downVideoCallWithWatch.setAppkey(CommonConstant.TH_VIDEO_KEY);
                                downVideoCallWithWatch.setVideo_type(6);
                            }
                            String name = null != deviceOwnerInfoApp ? deviceOwnerInfoApp.getName() : "宝贝";
                            String msgTitle = name + "邀请您通话";
                            //封装数据下发mqtt
                            VideoCallMessage videoCallMessage = new VideoCallMessage();
                            videoCallMessage.setType("accountMessage");
                            videoCallMessage.setMsgTitle(msgTitle);
                            videoCallMessage.setMessage(downVideoCallWithWatch);
                            String pushMsgData = JSONObject.toJSONString(videoCallMessage);
                            String lang = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheMapValue(userId, CommonConstant.IOS_LANGUAGE);
                            String agent = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheMapValue(userId, CommonConstant.USER_AGENT);
                            log.info("agent----:{}", agent);
                            String alert = name + I18nUtil.getMessage(CommonConstant.INVITE_CALL, lang);
                            log.info("alert----:{}", alert);
                            log.info("手表邀请APP通话推送MQTT=========={}", agent);
                            //判断是发IOS还是安卓
                            if (CommonConstant.ANDROID.equals(agent)) {

                                ApplicationUtil.getApplicationContext().getBean(MqttPush.class).sendMqtt("3", userId, "1", "accountMessage", "video", pushMsgData);
                            } else {
                                ApplicationUtil.getApplicationContext().getBean(BindingDeviceImpl.class).sendIOS(userId, videoCallMessage, "accountMessage", "video", alert);
                            }
                            log.info("手表成功推送通话邀请推送到APP===》用户openid:{} ,推送内容:{}", userId, videoCallMessage);
                            //安卓系统推送
                            if ("Android".equals(agent)) {
                                //手机厂商机型：HW、XM等
                                String AndroidType = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheObject(CommonConstant.ANDROID_AGENT + userId);
                                log.info("AndroidType----:{}", AndroidType);
                                if (StringUtils.isNotEmpty(AndroidType)) {
                                    if ("HW".equals(AndroidType)) {
                                        //获取对应的token
                                        String deviceToken = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheObject("HW:" + userId);
                                        //服务端获取华为的token
                                        String huaweiAccessToken = "";
                                        if (null != ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheObject("huaweiAccessToken")) {
                                            huaweiAccessToken = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheObject("huaweiAccessToken");
                                        } else {
                                            huaweiAccessToken = getHuaweiAccessToken();
                                            ApplicationUtil.getApplicationContext().getBean(RedisService.class).setCacheObject("huaweiAccessToken", huaweiAccessToken, 2L, TimeUnit.HOURS);
                                        }
                                        int sendType = 1;
                                        String action = "";
                                        if (CommonConstant.VIDEO_TYPE_THREE.equals(deviceVideoType)) {
                                            action = "android.intent.action.BAIRUI";
                                        } else if (CommonConstant.VIDEO_TYPE_ONE.equals(deviceVideoType)) {
                                            action = "android.intent.action.JUPHOON";
                                        } else {
                                            sendType = 3;
                                        }
                                        String msgData = JSONObject.toJSONString(downVideoCallWithWatch);
                                        HuaweiMsg huaweiMsg = new HuaweiMsg();
                                        huaweiMsg.setValidate_only(false);
                                        HuaweiMsgChild huaweiMsgChild = new HuaweiMsgChild();
                                        huaweiMsgChild.setData(msgData);
                                        HuaweiMsgNotification huaweiMsgNotification = new HuaweiMsgNotification();
                                        huaweiMsgNotification.setTitle(I18nUtil.getMessage(CommonConstant.VIDEO_CALL, lang));
                                        huaweiMsgNotification.setBody(alert);
                                        huaweiMsgChild.setNotification(huaweiMsgNotification);
                                        HuaweiMsgThird huaweiMsgThird = new HuaweiMsgThird();
                                        HuaweiMsgClickAction huaweiMsgClickAction = new HuaweiMsgClickAction();
                                        huaweiMsgClickAction.setAction(action);
                                        huaweiMsgClickAction.setType(sendType);
                                        huaweiMsgThird.setNotification(huaweiMsgClickAction);
                                        huaweiMsgChild.setAndroid(huaweiMsgThird);
                                        huaweiMsgChild.setToken(deviceToken);
                                        huaweiMsg.setMessage(huaweiMsgChild);
                                        String huaweiMsgData = JSONObject.toJSONString(huaweiMsg);
                                        pushMessage(userId, huaweiMsgData, huaweiAccessToken);
                                    } else if ("XM".equals(AndroidType)) {
                                        //获取对应的token
                                        String deviceToken = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheObject("XM:" + userId);
                                        String title = I18nUtil.getMessage(CommonConstant.VIDEO_CALL, lang);
                                        String intentUri = "";
                                        if (CommonConstant.VIDEO_TYPE_THREE.equals(deviceVideoType)) {
                                            intentUri = "intent:#Intent;component=com.ecellsz.watch.guard/com.lepeiban.deviceinfo.activity.video_call.bairui.BairuiVideoActivity;end";
                                        } else if (CommonConstant.VIDEO_TYPE_ONE.equals(deviceVideoType)) {
                                            intentUri = "intent:#Intent;component=com.ecellsz.watch.guard/com.lepeiban.deviceinfo.activity.video_call.juphoon.JuPhoonVideoActivity;end";
                                        }
                                        pushMiMessage(title, alert, "", deviceToken, intentUri);
                                    } else if ("VI".equals(AndroidType)) {
                                        //获取对应的token
                                        String deviceToken = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheObject("VI:" + userId);
                                        //服务端获取VIVO的token
                                        String viVoAccessToken = "";
                                        if (null != ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheObject("viVoAccessToken")) {
                                            viVoAccessToken = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheObject("viVoAccessToken");
                                        } else {
                                            viVoAccessToken = getViVoAccessToken();
                                            ApplicationUtil.getApplicationContext().getBean(RedisService.class).setCacheObject("viVoAccessToken", viVoAccessToken, 12L, TimeUnit.HOURS);
                                        }
                                        int skipType = 4;
                                        String skitContent = "";
                                        if (CommonConstant.VIDEO_TYPE_THREE.equals(deviceVideoType)) {
                                            skitContent = "intent://com.ecellsz.push.videocall/bairui#Intent;scheme=vpushscheme;component=com.ecellsz.watch.guard/com.lepeiban.deviceinfo.activity.video_call.bairui.BairuiVideoActivity;end";
                                        } else if (CommonConstant.VIDEO_TYPE_ONE.equals(deviceVideoType)) {
                                            skitContent = "intent://com.ecellsz.push.videocall/juphoon#Intent;scheme=vpushscheme;component=com.ecellsz.watch.guard/com.lepeiban.deviceinfo.activity.video_call.juphoon.JuPhoonVideoActivity;end";
                                        } else {
                                            skipType = 1;
                                        }
                                        JSONObject msgJson = new JSONObject();
                                        msgJson.put("regId", deviceToken);
                                        msgJson.put("notifyType", 4);
                                        msgJson.put("title", I18nUtil.getMessage(CommonConstant.VIDEO_CALL, lang));
                                        msgJson.put("content", alert);
                                        msgJson.put("skipType", skipType);
                                        msgJson.put("skipContent", skitContent);
                                        msgJson.put("requestId", com.ecell.internationalize.common.core.utils.uuid.UUID.fastUUID().toString(true));
                                        msgJson.put("clientCustomMap", downVideoCallWithWatch);
                                        msgJson.put("classification", 1);
                                        sendViVoMessage(viVoAccessToken, msgJson, userId);
                                    } else if ("OP".equals(AndroidType)) {
                                        //获取对应的token
                                        String deviceToken = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheObject("OP:" + userId);
                                        //服务端获取OPPO的token
                                        String opPoAccessToken = "";
                                        if (null != ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheObject("opPoAccessToken")) {
                                            opPoAccessToken = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheObject("opPoAccessToken");
                                        } else {
                                            opPoAccessToken = getOpPoAccessToken();
                                            ApplicationUtil.getApplicationContext().getBean(RedisService.class).setCacheObject("opPoAccessToken", opPoAccessToken, 12L, TimeUnit.HOURS);
                                        }
                                        int clickActionType = 1;
                                        String action = "";
                                        if (CommonConstant.VIDEO_TYPE_THREE.equals(deviceVideoType)) {
                                            action = "android.intent.action.BAIRUI";
                                        } else if (CommonConstant.VIDEO_TYPE_ONE.equals(deviceVideoType)) {
                                            action = "android.intent.action.JUPHOON";
                                        } else {
                                            clickActionType = 0;
                                        }
                                        JSONObject postData = new JSONObject();
                                        postData.put("target_type", 2);
                                        postData.put("target_value", deviceToken);
                                        JSONObject postData1 = new JSONObject();
                                        postData1.put("title", I18nUtil.getMessage(CommonConstant.VIDEO_CALL, lang));
                                        postData1.put("content", alert);
                                        postData1.put("click_action_type", clickActionType);
                                        postData1.put("click_action_activity", action);
                                        postData1.put("action_parameters", downVideoCallWithWatch);
                                        postData.put("notification", postData1);
                                        String str = JSONObject.toJSONString(postData);
                                        sendOpPoMessage(opPoAccessToken, str, userId);
                                    }
                                }
                            }
                        } else {
                            String message = "未知错误,请稍后重试";
                            if (CommonConstant.VIDEO_RESULT_PRE_ONE == data) {
                                message = "用户不可视频";
                            }
                            if (CommonConstant.VIDEO_RESULT_PRE_TWO == data) {
                                message = "不支持视频通话";
                            }
                            if (CommonConstant.PRE_NUMBER_FOUR == data) {
                                log.info("拨打视频电话前限制 .............新JF 553 机制限制提示 ............");
                                message = "视频时间过长，请稍后再拨";
                            }
                            if (CommonConstant.PRE_NUMBER_FIVE == data) {
                                message = "手表正在视频通话，请稍后再试";
                            } else {
                                log.error("手表imei:{} ,发起视频通话失败,未知错误,请稍后重试, data:{} ", imei, data);
                            }
                            downVideoCallFailInfo.setMessage(message);
                            ApplicationUtil.getApplicationContext().getBean(IssueInstructions.class).emitDeviceDemand("downVideoCallFailInfo", imei, downVideoCallFailInfo);
                        }
                    }
                    //手表<==>手表
                    else if (jsonObject.containsKey("call_device")) {
                        String callDeviceIMEI = jsonObject.get("call_device").toString();
                        int data = ApplicationUtil.getApplicationContext().getBean(DeviceVideoAppService.class).videoCall(imei, callType, downType, null, callDeviceIMEI, deviceVideoType, String.valueOf(uid));
                        if (CommonConstant.NUMBER_ONE == data) {
                            //邀请成功 已经在 videoCall方法 里面下发给了 对应的 callDevImei 手表了
                            log.info("手表打给手表成功");
                        } else {
                            String message = "未知错误,请稍后重试";
                            if (CommonConstant.VIDEO_RESULT_PRE_THREE == data) {
                                message = "设备正处于上课禁用状态，无法视频通话";
                            }
                            if (CommonConstant.PRE_NUMBER_FOUR == data) {
                                log.info("拨打视频电话前限制 .............新JF 553 机制限制提示 ............");
                                message = "视频时间过长，请稍后再拨";
                            }
                            if (CommonConstant.PRE_NUMBER_FIVE == data) {
                                message = "手表正在视频通话，请稍后再试";
                            } else {
                                log.error("手表imei:{} ,发起视频通话失败,未知错误,请稍后重试, data:{} ", imei, data);
                            }
                            downVideoCallFailInfo.setMessage(message);
                            ApplicationUtil.getApplicationContext().getBean(IssueInstructions.class).emitDeviceDemand("downVideoCallFailInfo", imei, downVideoCallFailInfo);
                        }
                    } else {
                        String message = "找不到要视频的用户";
                        downVideoCallFailInfo.setMessage(message);
                        ApplicationUtil.getApplicationContext().getBean(IssueInstructions.class).emitDeviceDemand("downVideoCallFailInfo", imei, downVideoCallFailInfo);
                    }
                }
                /** 手表结束通话后上传视频通话时长*/
                else if ("upVideoCallTime".equals(type)) {
                    log.info("手表结束通话后上传视频时长:{}", imei);
                    String videoId = jsonObject.get("video_id").toString();
                    int duration = Integer.parseInt(jsonObject.get("duration").toString());
                    int waitDuration = 0;
                    if (jsonObject.containsKey("wait_duration")) {
                        waitDuration = Integer.parseInt(jsonObject.get("wait_duration").toString());
                    }

                    ApplicationUtil.getApplicationContext().getBean(DeviceVideoRecordAppService.class).handVideoDuration(imei, videoId, duration, waitDuration, "device");
                }
                /**手表 主动挂断 推送 推送给APP */
                else if ("upWatchHangUp".equals(type)) {
                    String videoId = jsonObject.get("video_id").toString();
                    log.info("video_id:{}", videoId);
                    String openId = "";
                    LambdaQueryWrapper<DeviceVideoRecordApp> lambdaQueryWrapper = new LambdaQueryWrapper<>();
                    DeviceVideoRecordApp deviceVideoRecordApp = null;
                    DeviceVideoRecordAppService deviceVideoRecordAppService = ApplicationUtil.getApplicationContext().getBean(DeviceVideoRecordAppService.class);
                    if (StringUtils.isNotEmpty(videoId)) {
                        lambdaQueryWrapper.eq(DeviceVideoRecordApp::getVideoId, videoId);
                        deviceVideoRecordApp = deviceVideoRecordAppService.getBaseMapper().selectOne(lambdaQueryWrapper);
                    } else {
                        lambdaQueryWrapper.eq(DeviceVideoRecordApp::getImei, imei);
                        lambdaQueryWrapper.eq(DeviceVideoRecordApp::getVideoId, null);
                        lambdaQueryWrapper.orderByDesc(DeviceVideoRecordApp::getStartTime);

                        List<DeviceVideoRecordApp> deviceVideoRecordApps = deviceVideoRecordAppService.getBaseMapper().selectList(lambdaQueryWrapper);
                        log.info("获取到视频通话的记录是=======：{}", deviceVideoRecordApps);
                        if (StringUtils.isNotEmpty(deviceVideoRecordApps) && deviceVideoRecordApps.size() > 0) {
                            deviceVideoRecordApp = deviceVideoRecordApps.get(0);
                            log.info("获取最后一条记录是:{}==========", deviceVideoRecordApp);

                        }

                        if (StringUtils.isNull(deviceVideoRecordApp.getVideoId())) {
                            //删除那条记录
                            deviceVideoRecordAppService.getBaseMapper().deleteById(deviceVideoRecordApp.getId());
                        }

                    }


                    if (null != deviceVideoRecordApp) {
                        openId = deviceVideoRecordApp.getUserId();
                    }
                    LambdaQueryWrapper<DeviceOwnerInfoApp> ownerInfoAppLambdaQueryWrapper = new LambdaQueryWrapper<>();
                    ownerInfoAppLambdaQueryWrapper.eq(DeviceOwnerInfoApp::getImei, imei);
                    DeviceOwnerInfoApp ownerInfoApp = ApplicationUtil.getApplicationContext().getBean(DeviceOwnerInfoAppService.class).getBaseMapper().selectOne(ownerInfoAppLambdaQueryWrapper);
                    String lang = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheMapValue(openId, CommonConstant.IOS_LANGUAGE).toString();
                    String msgTitle = I18nUtil.getMessage(CommonConstant.USER_UNBIND_DEVICE, lang);
                    if (null != ownerInfoApp) {
                        ownerInfoApp.setNewVideoId("");
                        int i = ApplicationUtil.getApplicationContext().getBean(DeviceOwnerInfoAppService.class).getBaseMapper().updateById(ownerInfoApp);
                        log.info("清除最新的视频通话id：{}", i);
                        msgTitle = ownerInfoApp.getName() + I18nUtil.getMessage(CommonConstant.DOWN_VIDEO_CALL, lang);
                    }
                    if (StringUtils.isNotEmpty(openId)) {
                        //封装数据下发mqtt
                        DeviceUpHandMessage deviceUpHandMessage = new DeviceUpHandMessage();
                        deviceUpHandMessage.setType("accountMessage");
                        deviceUpHandMessage.setMsgTitle(msgTitle);
                        DeviceUpHandMessageChild deviceUpHandMessageChild = new DeviceUpHandMessageChild();
                        deviceUpHandMessageChild.setType("reject");
                        deviceUpHandMessageChild.setImei(imei);
                        deviceUpHandMessageChild.setTimestamp(Long.parseLong(jsonObject.get("time").toString()));
                        deviceUpHandMessageChild.setName(null != ownerInfoApp && null != ownerInfoApp.getName() ? ownerInfoApp.getName() : "宝贝");
                        deviceUpHandMessageChild.setOpenid(openId);
                        deviceUpHandMessage.setMessage(deviceUpHandMessageChild);
                        String sendPushToAccount = JSONObject.toJSONString(deviceUpHandMessage);
                        String agent = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheMapValue(openId, CommonConstant.USER_AGENT);
                        //判断是发IOS还是安卓
                        if (CommonConstant.ANDROID.equals(agent)) {
                            ApplicationUtil.getApplicationContext().getBean(MqttPush.class).sendMqtt("3", openId, "1", "accountMessage", "reject", sendPushToAccount);
                        } else {
                            ApplicationUtil.getApplicationContext().getBean(BindingDeviceImpl.class).sendIOS(openId, deviceUpHandMessage, "accountMessage", "reject", msgTitle);
                        }
                        log.info("手表 主动挂断后成功推送给APP的信息为:{}", deviceUpHandMessage);
                    }

                }
                //手表打给手表  对方设备IMEI,IMEI,视频类型  upVideoCallWithAPP

                else if ("UPVCALLDEV".equals(type)) {
                    log.info("解析手表发起 ===>手表视频通话指令");
                    log.info("获取到的视频通话的参数：{}", jsonObject);
                    //视频类型
                    String deviceVideoType = jsonObject.get("deviceVideoType").toString();

                    int uid = RandomUtil.generateVerCodeInt(String.valueOf(System.currentTimeMillis()));
                    //视频通话类型
                    String callType = jsonObject.containsKey("call_type") ? jsonObject.get("call_type").toString() : "0";
                    //指令类型
                    String downType = jsonObject.get("type").toString();

                    //失败原因进行指令下发 downVideoCallFailInfo
                    DownVideoCallFailInfo downVideoCallFailInfo = new DownVideoCallFailInfo();
                    downVideoCallFailInfo.setIdent("12345678");
                    downVideoCallFailInfo.setVender("20000");
                    downVideoCallFailInfo.setTime(System.currentTimeMillis());
                    downVideoCallFailInfo.setImei(jsonObject.get("user_openid").toString());
                    downVideoCallFailInfo.setType("downVideoCallFailInfo");
                    //发起和手表的通话 手表==>手表
                    if (jsonObject.containsKey("user_openid")) {
                        String userId = jsonObject.get("user_openid").toString();
                        log.info("手表发起和手表通话：{}", userId);
                        int data = ApplicationUtil.getApplicationContext().getBean(DeviceVideoAppService.class).videoCallDevice(imei, callType, downType, userId, userId, deviceVideoType, String.valueOf(uid));
                        //邀请视频通话成功 并成功创建记录
                        if (CommonConstant.NUMBER_ONE == data) {
                            log.info("手表打给手表成功");
                        } else {
                            String message = "未知错误,请稍后重试";
                            if (CommonConstant.VIDEO_RESULT_PRE_ONE == data) {
                                message = "用户不可视频";
                            }
                            if (CommonConstant.VIDEO_RESULT_PRE_TWO == data) {
                                message = "不支持视频通话";
                            }
                            if (CommonConstant.PRE_NUMBER_FOUR == data) {
                                log.info("拨打视频电话前限制 .............新JF 553 机制限制提示 ............");
                                message = "视频时间过长，请稍后再拨";
                            }
                            if (CommonConstant.PRE_NUMBER_FIVE == data) {
                                message = "手表正在视频通话，请稍后再试";
                            } else {
                                log.error("手表imei:{} ,发起视频通话失败,未知错误,请稍后重试, data:{} ", imei, data);
                            }
                            downVideoCallFailInfo.setMessage(message);
                            ApplicationUtil.getApplicationContext().getBean(IssueInstructions.class).emitDeviceDemand("downVideoCallFailInfo", imei, downVideoCallFailInfo);
                        }
                    }
                } else if ("upWeather".equals(type)) {
                    log.info("天气上传数据:{}", jsonObject);
                    PositionInfo positionInfo = JSONObject.parseObject(jsonObject.toString(), PositionInfo.class);
                    if ((StringUtils.isNotNull(positionInfo.getBaseStation()) && positionInfo.getBaseStation().size() > 0)
                            || (StringUtils.isNotNull(positionInfo.getWifi())) && positionInfo.getWifi().size() > 2) {
                        log.info("解析上传天气位置开始======================");
                        DeviceLocationApp app = LocationUtil.getWeatherLocation(positionInfo);
                        if (StringUtils.isNotNull(app) && StringUtils.isNotEmpty(app.getAdcode())) {
                            String adcode = app.getAdcode();
                            //放入缓存中
                            RedisService redis = ApplicationUtil.getApplicationContext().getBean(RedisService.class);
                            String date = DateUtils.getDate();
                            String weatherDate = redis.getCacheMapValue(adcode, "weatherDate");
                            WeatherLive weatherLive = new WeatherLive();
                            weatherLive.setIdent(RandomUtil.generateVerCode(6));
                            weatherLive.setVender("20000");
                            weatherLive.setImei(imei);
                            weatherLive.setTime(System.currentTimeMillis());
                            weatherLive.setType("downWeather");
                            weatherLive.setIsCDMA(positionInfo.getBaseStationType());
                            weatherLive.setAdcode(adcode);

                            if (StringUtils.isNotEmpty(weatherDate) && date.equals(weatherDate)) {
                                //获取缓存
                                redis.getCacheMapValue(adcode, "adcode");
                                redis.getCacheMapValue(adcode, "weatherDate");
                                weatherLive.setWeather(redis.getCacheMapValue(adcode, "weather"));
                                weatherLive.setTemperature(redis.getCacheMapValue(adcode, "temperature"));
                                weatherLive.setWinddirection(redis.getCacheMapValue(adcode, "winddirection"));
                                weatherLive.setWindpower(redis.getCacheMapValue(adcode, "windpower"));
                                weatherLive.setHumidity(redis.getCacheMapValue(adcode, "humidity"));
                                weatherLive.setReporttime(redis.getCacheMapValue(adcode, "reporttime"));
                                weatherLive.setProvince(redis.getCacheMapValue(adcode, "province"));
                                weatherLive.setCity(redis.getCacheMapValue(adcode, "city"));

                            } else {
                                redis.deleteObject(adcode);
                                log.info("请求天气接口开始======================");
                                WeatherResult weather = LocationUtil.getWeather(app.getAdcode());
                                if (StringUtils.isNotNull(weather) && StringUtils.isNotNull(weather.getLives())) {
                                    Lives lives = weather.getLives().get(0);
                                    log.info("获取到的天气是:{}", lives);
                                    //设置缓存
                                    redis.setCacheMapValue(adcode, "adcode", adcode);
                                    redis.setCacheMapValue(adcode, "weatherDate", date);
                                    redis.setCacheMapValue(adcode, "weather", lives.getWeather());
                                    redis.setCacheMapValue(adcode, "temperature", lives.getTemperature());
                                    redis.setCacheMapValue(adcode, "winddirection", lives.getWinddirection());
                                    redis.setCacheMapValue(adcode, "windpower", lives.getWindpower());
                                    redis.setCacheMapValue(adcode, "humidity", lives.getHumidity());
                                    redis.setCacheMapValue(adcode, "province", lives.getProvince());
                                    redis.setCacheMapValue(adcode, "reporttime", lives.getReporttime());
                                    redis.setCacheMapValue(adcode, "city", lives.getCity());
                                    BeanUtils.copyBeanProp(weatherLive, lives);
                                }
                                //下发天气
                                log.info("下发天气数据:{}", weatherLive);
                                IssueInstructions instructions = ApplicationUtil.getApplicationContext().getBean(IssueInstructions.class);
                                instructions.sendKafka(weatherLive);

                            }

                        }


                    }


                }
            }
        }

    }

    private void pushMiMessage(String title, String body, String messagePayload, String regId, String intentUri) throws IOException {
        String APP_SECRET_KEY_ANDROID = "0alT0qgCYfksM5twTzGXiA==";
        String MY_PACKAGE_NAME = "com.ecellsz.watch.guard";
        Constants.useOfficial();
        Message message;
        Map<String, String> map = new HashMap<>(16);
        map.put("notify_foreground", "1");
        if (null != intentUri) {
            map.put("intent_uri", intentUri);
        }

        message = new Message.Builder()
                .payload(messagePayload)
                .restrictedPackageName(MY_PACKAGE_NAME)
                // 使用默认提示音提示
                .notifyType(1)
                // 消息使用透传方式
                .passThrough(0)
                .extra(map)
                .title(title)
                .description(body)
                .build();
        //安卓推送
        // Sender sender = new Sender(APP_SECRET_KEY, Region.Other);
        Sender senderAndroid = new Sender(APP_SECRET_KEY_ANDROID);
        Result result = senderAndroid.send(message, regId, 3);
        System.out.println("小米推送消息的regId为:" + regId + "推送结果为：" + result);
    }

    //根据邮箱查询用户id
    private User getUser(String email) {

        UserServiceImpl impl = ApplicationUtil.getApplicationContext().getBean(UserServiceImpl.class);
        UserMapper baseMapper = impl.getBaseMapper();
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("user_email", email);
        wrapper.last("LIMIT 1");
        return baseMapper.selectOne(wrapper);
    }

    //计算今日卡路里和腕表今日行走总距离
    public Map<String, Double> getDistanceAndCalorie(Integer step, Double weight) {
        double k = 0.8214;
        double distance = step * 0.3 / 1000;
        double calorie = weight * distance * k;
        Map<String, Double> map = new HashMap<>(16);
        map.put("distance", distance);
        map.put("calorie", calorie);
        return map;

    }

    public void insertOrUpdateStep(String imei, String step) {
        log.info("{}新增或者更新步数{}", imei, step);
        synchronized (SYS_OBJECT) {
            DeviceStepDailyService bean = ApplicationUtil.getApplicationContext().getBean(DeviceStepDailyService.class);

            //2.监听步数上传到数据库
            LambdaQueryWrapper<DeviceStepDaily> lambdaQueryWrapper = new LambdaQueryWrapper<>();
            lambdaQueryWrapper.eq(DeviceStepDaily::getImei, imei);
            //日期格式化
            SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");
            Date date1 = new Date();
            System.out.println("获取到步数的时间是：=======" + date1);
            String date = simpleDateFormat.format(date1);
            System.out.println("获取到步数的GMT时间是：=======" + date);
            lambdaQueryWrapper.eq(DeviceStepDaily::getStepTime, date);
            try {
                //2.1.首先根据imei号和当前时间查询是否存在，存在则更新
                DeviceStepDaily deviceStepDaily = bean.getBaseMapper().selectOne(lambdaQueryWrapper);
                //2.2.获取设备用户体重
                LambdaQueryWrapper<DeviceOwnerInfoApp> lambdaQueryWrapper1 = new LambdaQueryWrapper<>();
                lambdaQueryWrapper1.eq(DeviceOwnerInfoApp::getImei, imei);
                DeviceOwnerInfoApp deviceOwnerInfoApp = ApplicationUtil.getApplicationContext().getBean(DeviceOwnerInfoAppService.class).getBaseMapper().selectOne(lambdaQueryWrapper1);
                //计算行驶距离和卡路里
                double weight = null == deviceOwnerInfoApp ? 0.00 : Double.parseDouble(deviceOwnerInfoApp.getWeight());
                Map<String, Double> distanceAndCalorie = getDistanceAndCalorie(Integer.valueOf(step), weight);
                log.info("查询到的步数==============：{}", deviceStepDaily);
                //存在则更新
                if (null != deviceStepDaily) {
                    log.info("更新步数=============：{}", deviceStepDaily);
                    deviceStepDaily.setStepTime(date);
                    deviceStepDaily.setWalksNum(Integer.valueOf(step));
                    deviceStepDaily.setWalksDistance(distanceAndCalorie.get("distance").intValue());
                    deviceStepDaily.setWalksCalorie(distanceAndCalorie.get("calorie").intValue());
                    deviceStepDaily.setCreateTime(simpleDateFormat.parse(date));
                    deviceStepDaily.setUpdateTime(new Date());
                    bean.getBaseMapper().updateById(deviceStepDaily);

                } else {
                    //新增
                    Date sqlDate = simpleDateFormat.parse(date);
                    DeviceStepDaily deviceStepDaily1 = new DeviceStepDaily();
                    deviceStepDaily1.setId(com.ecell.internationalize.common.core.utils.uuid.UUID.fastUUID().toString(true));
                    deviceStepDaily1.setCreateTime(sqlDate);
                    deviceStepDaily1.setImei(imei);
                    deviceStepDaily1.setWalksNum(Integer.valueOf(step));
                    deviceStepDaily1.setWalksDistance(distanceAndCalorie.get("distance").intValue());
                    deviceStepDaily1.setWalksCalorie(distanceAndCalorie.get("calorie").intValue());
                    deviceStepDaily1.setStepTime(date);
                    deviceStepDaily1.setUpdateTime(new Date());
                    log.info("新增步数=============：{}", deviceStepDaily1);
                    bean.getBaseMapper().insert(deviceStepDaily1);
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }


    /**
     * 保存或更新电量
     *
     * @param imei
     * @param battery 上传电量
     */
    public synchronized void saveOrUpdate(String imei, int battery) {
        //根据imei查询电量表
        BatteryInfoAppServiceImpl impl = ApplicationUtil.getApplicationContext().getBean(BatteryInfoAppServiceImpl.class);
        BatteryInfoApp batteryInfo = getBatteryInfo(impl, imei);
        if (StringUtils.isNotNull(batteryInfo)) {//更新
            log.info("获取到的电量为:{},查询到的电量为：{}", battery, batteryInfo.getBattery());
            if (batteryInfo.getBattery() != battery) {
                if (battery <= 10) {
                    batteryInfo.setLowBatteryAlarm(CommonConstant.AUDIT_STATUS_ONE);
                } else {
                    batteryInfo.setLowBatteryAlarm(CommonConstant.AUDIT_STATUS_ZERO);
                }
                batteryInfo.setBattery(battery);
                batteryInfo.setLastBattery(battery);

                // batteryInfo.getBattery()>=10 低电量只推送一次。
                if (battery <= 10 && CommonConstant.AUDIT_STATUS_ONE.equals(batteryInfo.getLowBatteryAlarm()) && batteryInfo.getBattery() >= 10) {
                    sendBattery(imei, "lowBattery", String.valueOf(battery));
                }
                //更新
                impl.updateById(batteryInfo);

            }

        } else {//保存
            BatteryInfoApp app = new BatteryInfoApp();
            if (battery <= 10) {//低电量
                app.setLowBatteryAlarm(CommonConstant.AUDIT_STATUS_ONE); //低电量
            } else {
                app.setLowBatteryAlarm(CommonConstant.AUDIT_STATUS_ZERO);//高电量
            }
            app.setId(com.ecell.internationalize.common.core.utils.uuid.UUID.fastUUID().toString(true));
            app.setImei(imei);
            app.setBattery(battery);
            app.setLastBattery(battery);
            impl.save(app);
            if (battery <= 10) {
                sendBattery(imei, "lowBattery", String.valueOf(battery));
            }
        }


    }

    /**
     * 查询电量
     *
     * @param impl
     * @param imei
     * @return
     */
    private BatteryInfoApp getBatteryInfo(BatteryInfoAppServiceImpl impl, String imei) {
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("imei", imei);
        wrapper.last("LIMIT 1");
        return impl.getBaseMapper().selectOne(wrapper);

    }

    /**
     * 位置信息上传上来的电量--》低电量推送
     */
    public synchronized void sendBattery(String imei, String type, String battery) {
        BindingDeviceImpl bindingDeviceImpl = ApplicationUtil.getApplicationContext().getBean(BindingDeviceImpl.class);
        ReminderMessageAppServiceImpl impl = ApplicationUtil.getApplicationContext().getBean(ReminderMessageAppServiceImpl.class);
        MqttPush mqttPush = ApplicationUtil.getApplicationContext().getBean(MqttPush.class);
        //查询设备名称
        DeviceOwnerInfoApp deviceOwnerInfoApp = bindingDeviceImpl.getDeviceOwnerInfoApp(imei);
        //查询绑定表
        List<UserDeviceBindApp> userIdList = bindingDeviceImpl.getUserIdList(imei);
        if (StringUtils.isNotEmpty(userIdList) && StringUtils.isNotNull(deviceOwnerInfoApp)) {
            BatteryMessages approvalMessages = new BatteryMessages();
            BatteryMqttMessages baseMqttMessages = new BatteryMqttMessages();
            List<String> userIds = userIdList.stream().map(UserDeviceBindApp::getUserId).collect(Collectors.toList());
            for (String id : userIds) {
                ReminderMessageApp app = new ReminderMessageApp();
                app.setName(Optional.ofNullable(deviceOwnerInfoApp.getName()).orElse("宝贝"));
                app.setId(com.ecell.internationalize.common.core.utils.uuid.UUID.fastUUID().toString(true));
                app.setUserId(id);
                app.setType(type);
                //未读
                app.setMessageRead(CommonConstant.AUDIT_STATUS_ZERO);
                app.setImei(imei);
                app.setBattery(battery);
                //低电量报警类型
                app.setRemindType(CommonConstant.AUDIT_STATUS_TWO);
                String s = JSON.toJSONString(app);
                app.setContext(s);
                impl.save(app);
                //发送推送信息
                approvalMessages.setType("accountMessage");
                baseMqttMessages.setType("lowBattery");
                baseMqttMessages.setName(Optional.ofNullable(deviceOwnerInfoApp.getName()).orElse("宝贝"));
                baseMqttMessages.setTimestamp(System.currentTimeMillis());
                baseMqttMessages.setBattery(battery);
                baseMqttMessages.setImei(imei);
                approvalMessages.setMessage(baseMqttMessages);
                String s1 = JSON.toJSONString(approvalMessages);
                //现在只推送给安卓
                log.info("低电量推送:{}", s1);
                //根据管理员ID查询手机型号
                RedisService redisService = ApplicationUtil.getApplicationContext().getBean(RedisService.class);
                String phoneType = redisService.getCacheMapValue(id, CommonConstant.USER_AGENT);
                if (CommonConstant.ANDROID.equals(phoneType)) {
                    //发送消息
                    mqttPush.sendMqtt(CommonConstant.AUDIT_STATUS_THREE, id, CommonConstant.AUDIT_STATUS_ONE, "accountMessage", "lowBattery", s1);

                } else if (CommonConstant.IOS.equals(phoneType)) {
//                    //根据管理员Id获取语言信息
//                    String language = redisService.getCacheMapValue(id, CommonConstant.IOS_LANGUAGE);
//                    String alert;
//                    //中文
//                    if (CommonConstant.CN.contains(language)){
//
//                        alert = String.format("%s手表当前电量低，请充电！", Optional.ofNullable(deviceOwnerInfoApp.getName()).orElse("宝贝"));
//                        //英文
//                    }else {
//                        alert = String.format("The battery of %s watch is low, please charge it!",  Optional.ofNullable(deviceOwnerInfoApp.getName()).orElse("宝贝"));
//                    }
                    String lage = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheMapValue(id, CommonConstant.IOS_LANGUAGE);
                    String alert = String.format(I18nUtil.getMessage("messages.ios.lowBattery", lage), Optional.ofNullable(deviceOwnerInfoApp.getName()).orElse("宝贝"));
                    bindingDeviceImpl.sendIOS(id, approvalMessages, "accountMessage", "lowBattery", alert);

                }
                // mqttPush.sendMqtt(CommonConstant.AUDIT_STATUS_THREE, id, CommonConstant.AUDIT_STATUS_ONE, "accountMessage", "lowBattery", s1);

            }

        }


    }

    /**
     * 位置信息上传上来的SOS--》SOS推送
     */
    private void sendSOS(String imei, String type, DeviceLocationApp app) throws IOException, NoSuchAlgorithmException {
        BindingDeviceImpl bindingDeviceImpl = ApplicationUtil.getApplicationContext().getBean(BindingDeviceImpl.class);
        ReminderMessageAppServiceImpl impl = ApplicationUtil.getApplicationContext().getBean(ReminderMessageAppServiceImpl.class);
        MqttPush mqttPush = ApplicationUtil.getApplicationContext().getBean(MqttPush.class);
        //查询设备名称
        DeviceOwnerInfoApp deviceOwnerInfoApp = bindingDeviceImpl.getDeviceOwnerInfoApp(imei);
        //查询绑定表
        List<UserDeviceBindApp> userIdList = bindingDeviceImpl.getUserIdList(imei);
        if (StringUtils.isNotEmpty(userIdList) && StringUtils.isNotNull(deviceOwnerInfoApp)) {
            SOSMessages approvalMessages = new SOSMessages();
            SOSMqttMessages baseMqttMessages = new SOSMqttMessages();
            List<String> userIds = userIdList.stream().map(UserDeviceBindApp::getUserId).collect(Collectors.toList());
            for (String id : userIds) {
                ReminderMessageApp reminderMessageApp = new ReminderMessageApp();
                reminderMessageApp.setId(com.ecell.internationalize.common.core.utils.uuid.UUID.fastUUID().toString(true));
                reminderMessageApp.setUserId(id);
                reminderMessageApp.setType(type);
                //未读
                reminderMessageApp.setMessageRead(CommonConstant.AUDIT_STATUS_ZERO);
                reminderMessageApp.setImei(imei);
                reminderMessageApp.setName(Optional.ofNullable(deviceOwnerInfoApp.getName()).orElse("宝贝"));
                String msg = String.format("%s触发,位置信息%s", Optional.ofNullable(deviceOwnerInfoApp.getName()).orElse("宝贝"), app.getAddr());
                reminderMessageApp.setMsgContent(msg);
                //sos报警类型
                reminderMessageApp.setRemindType(CommonConstant.AUDIT_STATUS_THREE);
                String lon = app.getLongitude();
                String lat = app.getLatitude();
                String address = app.getAddr();
                String loc_type = app.getLocationType();

                reminderMessageApp.setAddress(address);
                reminderMessageApp.setLon(lon);
                reminderMessageApp.setLat(lat);
                reminderMessageApp.setLoc_type(loc_type);
                reminderMessageApp.setTimestamp(System.currentTimeMillis());
                String s = JSON.toJSONString(reminderMessageApp);
                reminderMessageApp.setContext(s);
                impl.save(reminderMessageApp);
                //发送推送信息
                approvalMessages.setType("accountMessage");
                baseMqttMessages.setType("sos");
                baseMqttMessages.setName(Optional.ofNullable(deviceOwnerInfoApp.getName()).orElse("宝贝"));
                baseMqttMessages.setTimestamp(System.currentTimeMillis());
                baseMqttMessages.setMsgContent(msg);
                baseMqttMessages.setMsgTitle(msg);
                baseMqttMessages.setImei(imei);
                baseMqttMessages.setLat(lat);
                baseMqttMessages.setLon(lon);
                baseMqttMessages.setAddress(address);
                baseMqttMessages.setLoc_type(loc_type);
                approvalMessages.setMessage(baseMqttMessages);
                String s1 = JSON.toJSONString(approvalMessages);
                //现在只推送给安卓
                log.info("SOS报警推送:{}", s1);

                //根据管理员ID查询手机型号
                RedisService redisService = ApplicationUtil.getApplicationContext().getBean(RedisService.class);
                String phoneType = redisService.getCacheMapValue(id, CommonConstant.USER_AGENT);
                if (CommonConstant.ANDROID.equals(phoneType)) {
                    //发送消息
                    mqttPush.sendMqtt(CommonConstant.AUDIT_STATUS_THREE, id, CommonConstant.AUDIT_STATUS_ONE, "accountMessage", "sos", s1);

                } else if (CommonConstant.IOS.equals(phoneType)) {
//                    //根据管理员Id获取语言信息
//                    String language = redisService.getCacheMapValue(id, CommonConstant.IOS_LANGUAGE);
//                    String alert;
//                    //中文
//                    if (CommonConstant.CN.contains(language)) {
//
//                        alert = String.format("%s 触发SOS", Optional.ofNullable(deviceOwnerInfoApp.getName()).orElse("宝贝"));
//                        //英文
//                    } else {
//                        alert = String.format("%s triggers SOS", Optional.ofNullable(deviceOwnerInfoApp.getName()).orElse("宝贝"));
//                    }
                    String lage = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheMapValue(id, CommonConstant.IOS_LANGUAGE);
                    String alert = String.format(I18nUtil.getMessage("messages.ios.sos", lage), Optional.ofNullable(deviceOwnerInfoApp.getName()).orElse("宝贝"));
                    bindingDeviceImpl.sendIOS(id, approvalMessages, "accountMessage", "sos", alert);


                    // mqttPush.sendMqtt(CommonConstant.AUDIT_STATUS_THREE, id, CommonConstant.AUDIT_STATUS_ONE, "accountMessage", "sos", s1);

                }
                //mqttPush.sendMqtt(CommonConstant.AUDIT_STATUS_THREE, id, CommonConstant.AUDIT_STATUS_ONE, "accountMessage", "sos", s1);
                //安卓系统推送
                String lang = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheMapValue(id, CommonConstant.IOS_LANGUAGE);
                String agent = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheMapValue(id, CommonConstant.USER_AGENT);
                log.info("安卓IOS系统推送开始:{}", agent);
                if ("Android".equals(agent)) {

                    String content = deviceOwnerInfoApp.getName() + I18nUtil.getMessage(CommonConstant.TRIGGER_SOS, lang);
                    //手机厂商机型：HW、XM等
                    String AndroidType = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheObject(CommonConstant.ANDROID_AGENT + id);
                    if (StringUtils.isNotEmpty(AndroidType)) {
                        if ("HW".equals(AndroidType)) {
                            //获取对应的token
                            String deviceToken = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheObject("HW:" + id);
                            //服务端获取华为的token
                            String huaweiAccessToken = "";
                            if (null != ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheObject("huaweiAccessToken")) {
                                huaweiAccessToken = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheObject("huaweiAccessToken");
                            } else {
                                huaweiAccessToken = getHuaweiAccessToken();
                                ApplicationUtil.getApplicationContext().getBean(RedisService.class).setCacheObject("huaweiAccessToken", huaweiAccessToken, 2L, TimeUnit.HOURS);
                            }
                            JSONObject first = new JSONObject();
                            first.put("validate_only", false);
                            JSONObject titleObject = new JSONObject();
                            titleObject.put("title", I18nUtil.getMessage(CommonConstant.ALARM_SOS, lang));
                            titleObject.put("body", content);
                            JSONObject notificationObject = new JSONObject();
                            notificationObject.put("notification", titleObject);
                            JSONObject clickActionObject = new JSONObject();
                            clickActionObject.put("type", 1);
                            clickActionObject.put("action", "com.ecell.intent.action.MESSAGE");
                            JSONObject notificationClick = new JSONObject();
                            notificationClick.put("notification", clickActionObject);
                            notificationObject.put("android", notificationClick);
                            notificationObject.put("token", deviceToken);
                            first.put("message", notificationObject);
                            String huaweiMsgData = JSONObject.toJSONString(first);
                            log.info("华为系统推送开始token为:{}", huaweiAccessToken);
                            pushMessage(id, huaweiMsgData, huaweiAccessToken);
                        } else if ("XM".equals(AndroidType)) {
                            //获取对应的token
                            String deviceToken = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheObject("XM:" + id);
                            String title = I18nUtil.getMessage(CommonConstant.ALARM_SOS, lang);
                            String intentUri = "intent:#Intent;component=com.ecellsz.watch.guard/com.cyrus.mine.function.msg.MsgActivity;end";
                            log.info("小米系统推送开始token为:{}", deviceToken);
                            pushMiMessage(title, content, "", deviceToken, intentUri);
                        } else if ("VI".equals(AndroidType)) {
                            //获取对应的token
                            String deviceToken = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheObject("VI:" + id);
                            //服务端获取VIVO的token
                            String viVoAccessToken = "";
                            if (null != ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheObject("viVoAccessToken")) {
                                viVoAccessToken = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheObject("viVoAccessToken");
                            } else {
                                viVoAccessToken = getViVoAccessToken();
                                ApplicationUtil.getApplicationContext().getBean(RedisService.class).setCacheObject("viVoAccessToken", viVoAccessToken, 12L, TimeUnit.HOURS);
                            }
                            JSONObject msgJson = new JSONObject();
                            msgJson.put("regId", deviceToken);
                            msgJson.put("notifyType", 4);
                            msgJson.put("title", I18nUtil.getMessage(CommonConstant.ALARM_SOS, lang));
                            msgJson.put("content", content);
                            msgJson.put("skipType", 4);
                            msgJson.put("skipContent", "intent://com.ecellsz.push.message/detail#Intent;scheme=vpushscheme;component=com.ecellsz.watch.guard/com.cyrus.mine.function.msg.MsgActivity;end");
                            msgJson.put("requestId", com.ecell.internationalize.common.core.utils.uuid.UUID.fastUUID().toString(true));
                            //msgJson.put("clientCustomMap",downVideoCallWithWatch);
                            msgJson.put("classification", 1);
                            log.info("VIVO系统推送开始token为:{}", viVoAccessToken);
                            sendViVoMessage(viVoAccessToken, msgJson, id);
                        } else if ("OP".equals(AndroidType)) {
                            //获取对应的token
                            String deviceToken = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheObject("OP:" + id);
                            //服务端获取OPPO的token
                            String opPoAccessToken = "";
                            if (null != ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheObject("opPoAccessToken")) {
                                opPoAccessToken = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheObject("opPoAccessToken");
                            } else {
                                opPoAccessToken = getOpPoAccessToken();
                                ApplicationUtil.getApplicationContext().getBean(RedisService.class).setCacheObject("opPoAccessToken", opPoAccessToken, 12L, TimeUnit.HOURS);
                            }
                            JSONObject postData = new JSONObject();
                            postData.put("target_type", 2);
                            postData.put("target_value", deviceToken);
                            JSONObject postData1 = new JSONObject();
                            postData1.put("title", I18nUtil.getMessage(CommonConstant.ALARM_SOS, lang));
                            postData1.put("content", content);
                            postData1.put("click_action_type", 1);
                            postData1.put("click_action_activity", "com.ecell.intent.action.MESSAGE");
                            //postData1.put("action_parameters",downVideoCallWithWatch);
                            postData.put("notification", postData1);
                            String str = JSONObject.toJSONString(postData);
                            log.info("OP系统推送开始token为:{}", opPoAccessToken);
                            sendOpPoMessage(opPoAccessToken, str, id);
                        }
                    }
                }
            }

        }
    }


    /**
     * LK指令上传上来的电量--》正常电量推送
     */
    private void sendLKBattery(String imei, String type, String battery) {
        BindingDeviceImpl bindingDeviceImpl = ApplicationUtil.getApplicationContext().getBean(BindingDeviceImpl.class);
        ReminderMessageAppServiceImpl impl = ApplicationUtil.getApplicationContext().getBean(ReminderMessageAppServiceImpl.class);
        MqttPush mqttPush = ApplicationUtil.getApplicationContext().getBean(MqttPush.class);
        //查询设备名称
        DeviceOwnerInfoApp deviceOwnerInfoApp = bindingDeviceImpl.getDeviceOwnerInfoApp(imei);
        //查询绑定表
        List<UserDeviceBindApp> userIdList = bindingDeviceImpl.getUserIdList(imei);
        if (StringUtils.isNotEmpty(userIdList) && StringUtils.isNotNull(deviceOwnerInfoApp)) {
            BatteryMessages approvalMessages = new BatteryMessages();
            BatteryMqttMessages baseMqttMessages = new BatteryMqttMessages();
            List<String> userIds = userIdList.stream().map(UserDeviceBindApp::getUserId).collect(Collectors.toList());
            for (String id : userIds) {
                //发送推送信息
                approvalMessages.setType("accountMessage");
                baseMqttMessages.setType("battery");
                baseMqttMessages.setName(Optional.ofNullable(deviceOwnerInfoApp.getName()).orElse("宝贝"));
                baseMqttMessages.setTimestamp(System.currentTimeMillis());
                baseMqttMessages.setBattery(battery);
                baseMqttMessages.setImei(imei);
                approvalMessages.setMessage(baseMqttMessages);
                String s1 = JSON.toJSONString(approvalMessages);
                //现在只推送给安卓
                log.info("正常电量推送:{}", s1);
                //根据管理员ID查询手机型号
                RedisService redisService = ApplicationUtil.getApplicationContext().getBean(RedisService.class);
                String phoneType = redisService.getCacheMapValue(id, CommonConstant.USER_AGENT);
                if (CommonConstant.ANDROID.equals(phoneType)) {
                    //发送消息
                    mqttPush.sendMqtt(CommonConstant.AUDIT_STATUS_THREE, id, CommonConstant.AUDIT_STATUS_ONE, "accountMessage", "battery", s1);

                } else if (CommonConstant.IOS.equals(phoneType)) {
//                    //根据管理员Id获取语言信息
//                    String language = redisService.getCacheMapValue(id, CommonConstant.IOS_LANGUAGE);
//                    String alert;
//                    //中文
//                    if (CommonConstant.CN.contains(language)) {
//
//                        alert = String.format("%s 触发SOS", Optional.ofNullable(deviceOwnerInfoApp.getName()).orElse("宝贝"));
//                        //英文
//                    } else {
//                        alert = String.format("%s triggers SOS", Optional.ofNullable(deviceOwnerInfoApp.getName()).orElse("宝贝"));
//                    }
                    bindingDeviceImpl.sendIOS(id, approvalMessages, "accountMessage", "battery", null);
                    // mqttPush.sendMqtt(CommonConstant.AUDIT_STATUS_THREE, id, CommonConstant.AUDIT_STATUS_ONE, "accountMessage", "battery", s1);

                }

            }
        }

    }


    /**
     * 根据imei更新设备表的翻滚次数
     *
     * @param imei
     */
    public void saveOrUpdateRollNumber(String imei, int rollNumber) {
        DeviceOwnerInfoAppServiceImpl deviceOwnerInfoAppServiceImpl = ApplicationUtil.getApplicationContext().getBean(DeviceOwnerInfoAppServiceImpl.class);
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("imei", imei);
        wrapper.last("LIMIT 1");
        DeviceOwnerInfoAppMapper baseMapper = deviceOwnerInfoAppServiceImpl.getBaseMapper();
        DeviceOwnerInfoApp app = baseMapper.selectOne(wrapper);
        app.setRollNum(rollNumber);
        deviceOwnerInfoAppServiceImpl.updateById(app);

    }

    /**
     * 主动请求位置下发
     *
     * @param imei
     */
    public void sendLocaltionApp(String imei) {
        RedisService redisService = ApplicationUtil.getApplicationContext().getBean(RedisService.class);
        LatestLocationAppServiceImpl LatestLocationAppServiceImpl = ApplicationUtil.getApplicationContext().getBean(LatestLocationAppServiceImpl.class);
        //根据imei查询绑定表
        BindingDeviceImpl bindingDevice = ApplicationUtil.getApplicationContext().getBean(BindingDeviceImpl.class);

        List<UserDeviceBindApp> userIdList = bindingDevice.getUserIdList(imei);
        if (StringUtils.isNotEmpty(userIdList)) {
            for (UserDeviceBindApp userDeviceBindApp : userIdList) {
                String redisKey = String.format("requestLocation:%s:%s", imei, userDeviceBindApp.getUserId());
                if (redisService.hasKey(redisKey)) {//redis中存在就发送到APP
                    //就推送
                    LatestLocationAppServiceImpl.sendLastLocationApp(imei, userDeviceBindApp.getUserId());
                    //推送成功就删除这个缓存
                    redisService.deleteObject(redisKey);
                }
            }

        }

    }

    /**
     * 根据imei 添加OR更新最新位置，步数，电量等信息
     *
     * @param app
     * @param positionInfo
     */
    public synchronized void getLastLocationInfo(DeviceLocationApp app, PositionInfo positionInfo) {
        log.info("最新位置解析:{}", app);
        LatestLocationApp latestLocationApp = new LatestLocationApp();
        BeanUtils.copyBeanProp(latestLocationApp, app);
        LatestLocationAppServiceImpl impl = ApplicationUtil.getApplicationContext().getBean(LatestLocationAppServiceImpl.class);
        LatestLocationApp app2 = impl.getLatestLocation(app.getImei());
        //不等于null就更新
        if (StringUtils.isNotNull(app2)) {
//            latestLocationApp.setCountry(app.getCountry());
//            latestLocationApp.setProvince(app.getProvince());
            latestLocationApp.setId(app2.getId());
            latestLocationApp.setBattery(Integer.valueOf(positionInfo.getBattery())); //电量
            latestLocationApp.setStepNumber(Integer.valueOf(positionInfo.getStepNumber()));//步数
            impl.updateById(latestLocationApp);
            //就新增
        } else {
            latestLocationApp.setId(com.ecell.internationalize.common.core.utils.uuid.UUID.fastUUID().toString(true));
//            latestLocationApp.setCountry(app.getCountry());
//            latestLocationApp.setProvince(app.getProvince());
            latestLocationApp.setBattery(Integer.valueOf(positionInfo.getBattery())); //电量
            latestLocationApp.setStepNumber(Integer.valueOf(positionInfo.getStepNumber())); //步数
            latestLocationApp.setCreateTime(new Date());
            latestLocationApp.setUpdateTime(new Date());
            impl.save(latestLocationApp);


        }
    }

    /**
     * 根据imei查询最新位置信息表
     *
     * @param imei
     * @return
     */
    private LatestLocationApp queryLocationByImei(String imei) {
        LatestLocationAppServiceImpl impl = ApplicationUtil.getApplicationContext().getBean(LatestLocationAppServiceImpl.class);
        return impl.getLatestLocation(imei);
    }

    /**
     * 更新最新位置步数，电量,心率，体温信息表
     *
     * @param app         最新位置信息
     * @param battery     电量
     * @param stepNumber  步数
     * @param rate        心率
     * @param temperature 体温
     */
    private void updateLocationByImei(LatestLocationApp app, String battery, String stepNumber, String rate, String temperature, String oxygen) {
        LatestLocationAppServiceImpl impl = ApplicationUtil.getApplicationContext().getBean(LatestLocationAppServiceImpl.class);
        if (StringUtils.isNotEmpty(battery)) {
            app.setBattery(Integer.valueOf(battery));
        }
        if (StringUtils.isNotEmpty(stepNumber)) {
            app.setStepNumber(Integer.valueOf(stepNumber));
            app.setStepNumberTime(new Date());
        }
        if (StringUtils.isNotEmpty(rate)) {
            app.setRate(rate);
        }
        if (StringUtils.isNotEmpty(temperature)) {
            app.setTemperature(temperature);
        }
        if (StringUtils.isNotEmpty(oxygen)) {
            app.setBodyOxygen(oxygen);
            app.setBodyOxygenTime(new Date());
        }
        impl.updateById(app);
    }

    /**
     * 安全围栏判断
     *
     * @param imei   imei号
     * @param newLon 经度
     * @param newLat 纬度
     */
    private void pushSafeRails(String imei, String newLon, String newLat) {
        try {
            //1.根据imei号查询是否有设置电子围栏信息
            LambdaQueryWrapper<SafetyRailApp> lambdaQueryWrapper = new LambdaQueryWrapper<>();
            lambdaQueryWrapper.eq(SafetyRailApp::getImei, imei);
            List<SafetyRailApp> safetyRailAppList = ApplicationUtil.getApplicationContext().getBean(SafetyRailAppService.class).getBaseMapper().selectList(lambdaQueryWrapper);
            if (CollectionUtils.isEmpty(safetyRailAppList)) {
                log.info("设备:{},不存在电子围栏信息,不进行解析", imei);
                return;
            }
            //2.存在电子围栏信息，解析每个电子围栏数据
            for (SafetyRailApp safetyRailApp : safetyRailAppList) {
                //获取今天是星期几
                String day = TimeUtil.time();
                //获取当前几点几分
                Date date = new Date();
                SimpleDateFormat simpleDateFormat = new SimpleDateFormat("HH:mm");
                String time = simpleDateFormat.format(date);
                //起始时间
                String[] startTime = safetyRailApp.getStartTime().split(":");
                //当前时间
                String[] nowTime = time.split(":");
                //结束时间
                String[] endTime = safetyRailApp.getEndTime().split(":");
                //起始时间小时数
                String startHour = startTime[0];
                //起始时间分钟数
                String startMinute = startTime[1];
                //结束时间小时数
                String endHour = endTime[0];
                //结束时间分钟数
                String endMinute = endTime[1];
                //当前时间小时数
                String nowHour = nowTime[0];
                //起始时间分钟数
                String nowMinute = nowTime[1];
                int startTotal = Integer.parseInt(startHour) * 3600 + Integer.parseInt(startMinute) * 60;
                int endTotal = Integer.parseInt(endHour) * 3600 + Integer.parseInt(endMinute) * 60;
                int nowTotal = Integer.parseInt(nowHour) * 3600 + Integer.parseInt(nowMinute) * 60;
                if ("0".equals(safetyRailApp.getStatus())) {
                    log.info("此电子围栏信息未开启,不进行解析操作");
                    continue;
                }
                //当前日期不在设置的日期时间段内
                if (!TimeUtil.getDays(safetyRailApp.getWeek(), day)) {
                    log.info("此电子围栏信息不在设置的星期范围时间内,不进行解析操作");
                    continue;
                }
                //当前时间小于设置的起始时间
                if (nowTotal < startTotal) {
                    log.info("此电子围栏信息当前时间小于设置的起始时间,不进行解析操作");
                    continue;
                }
                //当前时间大于设置的结束时间
                if (nowTotal > endTotal) {
                    log.info("此电子围栏信息当前时间大于设置的结束时间,不进行解析操作");
                    continue;
                }
                //计算走的距离
                double distance = getDistance(Double.parseDouble(safetyRailApp.getLon()), Double.parseDouble(safetyRailApp.getLat()), Double.parseDouble(newLon), Double.parseDouble(newLat));
                log.info("计算出来的距离:{}", distance);
                int radius = safetyRailApp.getRadius();
                // 第一次的不用 推送 和 写入数据
                if (null == safetyRailApp.getAction() || "".equals(safetyRailApp.getAction())) {
                    if (distance > radius) {
                        safetyRailApp.setAction("leave");
                    } else {
                        safetyRailApp.setAction("enter");
                    }
                    //更新数据
                    updateSafeRails(safetyRailApp);
                }
                //在这个 区域内 需要判断 是否 离开了 这个区域
                else if ("enter".equals(safetyRailApp.getAction())) {
                    if (distance > radius) {
                        safetyRailApp.setAction("leave");
                        //更新安全围栏数据
                        updateSafeRails(safetyRailApp);
                        //新建一个轨迹点写入库

                        // 保存到提示信息表
                        insertRemind(safetyRailApp, imei);
                        //推送到mqtt
                        pushToMqtt(safetyRailApp, imei);
                    }
                }
                //不 在这个 区域内 需要判断 是否 进入了 这个区域
                else if ("leave".equals(safetyRailApp.getAction())) {
                    if (distance < radius) {
                        safetyRailApp.setAction("enter");
                        //更新安全围栏数据
                        updateSafeRails(safetyRailApp);
                        //新建一个轨迹点写入库

                        // 保存到提示信息表
                        insertRemind(safetyRailApp, imei);
                        //推送到mqtt
                        pushToMqtt(safetyRailApp, imei);
                    }
                }
            }
        } catch (Exception e) {
            log.info("解析电子围栏信息异常,异常原因:{}", e.getMessage());
        }
    }

    /**
     * 根据经纬度计算距离
     *
     * @param lastLon 设置的经度
     * @param lastLat 设置的维度
     * @param newLon  最新的经度
     * @param newLat  最新的纬度
     */
    public double getDistance(double lastLon, double lastLat, double newLon, double newLat) {
        double x = Math.PI / 180.0;
        if (lastLon == newLon && lastLat == newLat) {
            return 0.00;
        }
        lastLon = lastLon * x;
        lastLat = lastLat * x;
        newLon = newLon * x;
        newLat = newLat * x;
        double con = Math.sin(lastLat) * Math.sin(newLat);
        con += Math.cos(lastLat) * Math.cos(newLat) * Math.cos(lastLon - newLon);
        return Math.acos(con) * 6371137.0;
    }

    /**
     * 更新安全围栏信息
     */
    public void updateSafeRails(SafetyRailApp safetyRailApp) {
        ApplicationUtil.getApplicationContext().getBean(SafetyRailAppService.class).updateById(safetyRailApp);
    }

    /**
     * 新增电子围栏信息到提醒消息记录表
     */
    public void insertRemind(SafetyRailApp safetyRailApp, String imei) {
        try {
            List<ReminderMessageApp> list = new ArrayList<>();
            //封装JSON数据
            SafeRailDto safeRailDto = new SafeRailDto();
            safeRailDto.setType("rail");
            safeRailDto.setAction(safetyRailApp.getAction());
            safeRailDto.setRailId(safetyRailApp.getId());
            safeRailDto.setTime(System.currentTimeMillis());
            safeRailDto.setRailName(safetyRailApp.getName());
            String messageDic = JSONObject.toJSONString(safeRailDto);
            //根据imei号查找绑定的userId
            ReminderMessageApp reminderMessageApp;
            LambdaQueryWrapper<UserDeviceBindApp> lambdaQueryWrapper = new LambdaQueryWrapper<>();
            lambdaQueryWrapper.eq(UserDeviceBindApp::getImei, imei);
            List<UserDeviceBindApp> deviceBindAppList = ApplicationUtil.getApplicationContext().getBean(UserDeviceBindService.class).getBaseMapper().selectList(lambdaQueryWrapper);
            if (!CollectionUtils.isEmpty(deviceBindAppList)) {
                for (UserDeviceBindApp userDeviceBindApp : deviceBindAppList) {
                    reminderMessageApp = new ReminderMessageApp();
                    reminderMessageApp.setId(com.ecell.internationalize.common.core.utils.uuid.UUID.fastUUID().toString(true));
                    reminderMessageApp.setImei(imei);
                    reminderMessageApp.setType("rail");
                    reminderMessageApp.setMessageRead("0");
                    reminderMessageApp.setContext(messageDic);
                    reminderMessageApp.setUserId(userDeviceBindApp.getUserId());
                    reminderMessageApp.setCreateTime(new Date());
                    reminderMessageApp.setRemindType("1");
                    list.add(reminderMessageApp);
                }
                ApplicationUtil.getApplicationContext().getBean(ReminderMessageAppService.class).saveBatch(list);
                log.info("保存电子围栏信息到提醒消息表成功");
            }
        } catch (Exception e) {
            log.info("保存电子围栏信息到提醒消息表异常,异常原因:{}", e.getMessage());
        }
    }

    /**
     * 安全围栏推送到mqtt
     */
    public void pushToMqtt(SafetyRailApp safetyRailApp, String imei) {
        try {
            //根据imei查找设备名称
            LambdaQueryWrapper<DeviceOwnerInfoApp> deviceOwnerInfoAppLambdaQueryWrapper = new LambdaQueryWrapper<>();
            deviceOwnerInfoAppLambdaQueryWrapper.eq(DeviceOwnerInfoApp::getImei, imei);
            DeviceOwnerInfoApp deviceOwnerInfoApp = ApplicationUtil.getApplicationContext().getBean(DeviceOwnerInfoAppService.class).getBaseMapper().selectOne(deviceOwnerInfoAppLambdaQueryWrapper);
            String name = null == deviceOwnerInfoApp ? "" : deviceOwnerInfoApp.getName();
            //根据imei号查找绑定的userId
            // ReminderMessageApp reminderMessageApp;
            LambdaQueryWrapper<UserDeviceBindApp> lambdaQueryWrapper = new LambdaQueryWrapper<>();
            lambdaQueryWrapper.eq(UserDeviceBindApp::getImei, imei);
            List<UserDeviceBindApp> deviceBindAppList = ApplicationUtil.getApplicationContext().getBean(UserDeviceBindService.class).getBaseMapper().selectList(lambdaQueryWrapper);
            if (!CollectionUtils.isEmpty(deviceBindAppList)) {
                for (UserDeviceBindApp userDeviceBindApp : deviceBindAppList) {
                    SafeRailMessage safeRailMessage = new SafeRailMessage();
                    safeRailMessage.setType("accountMessage");
                    SafeRailMessageChild safeRailMessageChild = new SafeRailMessageChild();
                    safeRailMessageChild.setType("rail");
                    safeRailMessageChild.setImei(imei);
                    safeRailMessageChild.setName(name);
                    safeRailMessageChild.setRailID(safetyRailApp.getId());
                    safeRailMessageChild.setRailName(safetyRailApp.getName());
                    safeRailMessageChild.setAction(safetyRailApp.getAction());
                    safeRailMessageChild.setTimestamp(System.currentTimeMillis());
                    safeRailMessage.setMessage(safeRailMessageChild);
                    String sendPushToAccount = JSONObject.toJSONString(safeRailMessage);
                    String alert = "";
                    String lang = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheMapValue(userDeviceBindApp.getUserId(), CommonConstant.IOS_LANGUAGE);
                    String agent = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheMapValue(userDeviceBindApp.getUserId(), CommonConstant.USER_AGENT);
                    if ("enter".equals(safetyRailApp.getAction())) {
                        alert = name + I18nUtil.getMessage(CommonConstant.ENTER_RAIL, lang);
                    } else {
                        alert = name + I18nUtil.getMessage(CommonConstant.LEAVE_RAIL, lang);
                    }
                    //判断是发IOS还是安卓
                    if (CommonConstant.ANDROID.equals(agent)) {
                        ApplicationUtil.getApplicationContext().getBean(MqttPush.class).sendMqtt("3", userDeviceBindApp.getUserId(), "1", "accountMessage", "rail", sendPushToAccount);
                    } else {
                        ApplicationUtil.getApplicationContext().getBean(BindingDeviceImpl.class).sendIOS(userDeviceBindApp.getUserId(), safeRailMessage, "accountMessage", "rail", alert);
                    }
                }
                log.info("电子围栏信息推送成功");
            }
        } catch (Exception e) {
            log.info("电子围栏信息推送mqtt异常,异常原因:{}", e.getMessage());
        }
    }

    /**
     * 处理心率/体温数据上传
     *
     * @param imei   imei号
     * @param type：1 表示查询心率配置,type为 2 表示查询体温配置
     */
    public void handlerHeartRate(String imei, String type, String data, String time) {
        log.info("开始处理心率/体温数据上传:{}", type);
        log.info("获取到的时间为:{}", time);
        // start ---更新最近一次的心率/体温上传
        LambdaQueryWrapper<HeartTemperatureConfig> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(HeartTemperatureConfig::getImei, imei);
        lambdaQueryWrapper.eq(HeartTemperatureConfig::getType, type);
        HeartTemperatureConfig heartTemperatureConfig = ApplicationUtil.getApplicationContext().getBean(HeartTemperatureConfigService.class).getBaseMapper().selectOne(lambdaQueryWrapper);
        //查询为空则新增默认值
        if (null == heartTemperatureConfig) {
            ApplicationUtil.getApplicationContext().getBean(HeartTemperatureConfigServiceImpl.class).createConfig(imei, type);
        }
        HeartTemperatureConfig mustHaveConfig = ApplicationUtil.getApplicationContext().getBean(HeartTemperatureConfigService.class).getBaseMapper().selectOne(lambdaQueryWrapper);
        mustHaveConfig.setLastHeartTemperature(data);
        mustHaveConfig.setLastHeartTemperatureTime(Integer.parseInt(time));
        mustHaveConfig.setCreateTime(new Date());
        ApplicationUtil.getApplicationContext().getBean(HeartTemperatureConfigService.class).updateById(mustHaveConfig);
        //end ---更新最近一次的心率/体温上传
        LambdaQueryWrapper<DeviceOwnerInfoApp> ownerInfoAppLambdaQueryWrapper = new LambdaQueryWrapper<>();
        ownerInfoAppLambdaQueryWrapper.eq(DeviceOwnerInfoApp::getImei, imei);
        DeviceOwnerInfoApp deviceOwnerInfoApp = ApplicationUtil.getApplicationContext().getBean(DeviceOwnerInfoAppService.class).getBaseMapper().selectOne(ownerInfoAppLambdaQueryWrapper);
        //判断是否心率/体温异常,upper上限，lower下限
        String upper = mustHaveConfig.getUpper();
        String lower = mustHaveConfig.getLower();
        boolean isAbnormal = false;
        if (Float.parseFloat(data) > Float.parseFloat(upper)) {
            isAbnormal = true;
        }
        if (Float.parseFloat(data) < Float.parseFloat(lower)) {
            isAbnormal = true;
        }
        //血氧暂时不推送
        if (!"3".equals(type)) {
            //心率/体温异常推送mqtt
            if (isAbnormal) {
                log.info("心率/体温异常开始推送mqtt信息到Android");
                pushHeartRateAnomaliesAlarm(deviceOwnerInfoApp, imei, data, type);
            }
        }
        String redisKey = "";
        String normalType = "";
        if (CommonConstant.AUDIT_STATUS_ONE.equals(type)) {
            redisKey = "upHeartRate:%s:%s";
            normalType = "upHeartRate";
        } else if (CommonConstant.AUDIT_STATUS_TWO.equals(type)) {
            redisKey = "upBodyTemperature:%s:%s";
            normalType = "upBodyTemperature";
        } else if (CommonConstant.AUDIT_STATUS_THREE.equals(type)) {
            redisKey = "upBodyOxygen:%s:%s";
            normalType = "upBodyOxygen";
        }
        //记录心率/体温上传记录
        String id = com.ecell.internationalize.common.core.utils.uuid.UUID.fastUUID().toString(true);
        //String date = String.valueOf(System.currentTimeMillis()).substring(0, 10);
        HeartTemperatureRecord heartTemperatureRecord = new HeartTemperatureRecord();
        heartTemperatureRecord.setId(id);
        heartTemperatureRecord.setImei(imei);
        heartTemperatureRecord.setHeartTemperatureRate(data);
        heartTemperatureRecord.setCreateTime(new Date());
        heartTemperatureRecord.setType(type);
        heartTemperatureRecord.setTimeStamp(Integer.parseInt(time));
        ApplicationUtil.getApplicationContext().getBean(HeartTemperatureRecordService.class).getBaseMapper().insert(heartTemperatureRecord);
        log.info("记录心率/体温上传成功:{}", type);
        //血氧暂时不推送
        if (!"3".equals(type)) {
            LambdaQueryWrapper<UserDeviceBindApp> bindAppLambdaQueryWrapper = new LambdaQueryWrapper<>();
            bindAppLambdaQueryWrapper.eq(UserDeviceBindApp::getImei, imei);
            List<UserDeviceBindApp> userDeviceBindAppList = ApplicationUtil.getApplicationContext().getBean(UserDeviceBindService.class).getBaseMapper().selectList(bindAppLambdaQueryWrapper);
            HeartNormalPush heartNormalPush;
            HeartNormalPushChild heartNormalPushChild;
            if (!CollectionUtils.isEmpty(userDeviceBindAppList)) {
                for (UserDeviceBindApp userDeviceBindApp : userDeviceBindAppList) {
                    String format = String.format(redisKey, imei, userDeviceBindApp.getUserId());
                    String redisData = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheObject(format);
                    if (null != redisData) {
                        heartNormalPush = new HeartNormalPush();
                        heartNormalPushChild = new HeartNormalPushChild();
                        heartNormalPush.setType("accountMessage");
                        heartNormalPushChild.setImei(imei);
                        heartNormalPushChild.setTimestamp(System.currentTimeMillis());
                        heartNormalPushChild.setType(normalType);
                        heartNormalPushChild.setId(id);
                        heartNormalPush.setMessage(heartNormalPushChild);
                        String sendPushToAccount = JSONObject.toJSONString(heartNormalPush);
                        String agent = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheMapValue(userDeviceBindApp.getUserId(), CommonConstant.USER_AGENT);

                        if (CommonConstant.ANDROID.equals(agent)) {
                            ApplicationUtil.getApplicationContext().getBean(MqttPush.class).sendMqtt("3", userDeviceBindApp.getUserId(), "1", "accountMessage", normalType, sendPushToAccount);
                            log.info("安卓系统心率/体温上报 发送推送mqtt成功:{},心率/体温为:{}", userDeviceBindApp.getUserId(), data);
                        } else {
                            ApplicationUtil.getApplicationContext().getBean(BindingDeviceImpl.class).sendIOS(userDeviceBindApp.getUserId(), heartNormalPush, "accountMessage", normalType, null);
                            log.info("IOS系统心率/体温上报 发送IOS推送成功:{},心率/体温为:{}", userDeviceBindApp.getUserId(), data);
                        }
                    }
                }
            }
        }
    }


    /**
     * 心率/体温异常报警推送mqtt
     */
    public void pushHeartRateAnomaliesAlarm(DeviceOwnerInfoApp deviceOwnerInfoApp, String imei, String data, String type) {
        try {
            String name = null == deviceOwnerInfoApp ? "宝贝" : deviceOwnerInfoApp.getName();
            //查询该设备下所有绑定用户
            LambdaQueryWrapper<UserDeviceBindApp> lambdaQueryWrapper = new LambdaQueryWrapper<>();
            lambdaQueryWrapper.eq(UserDeviceBindApp::getImei, imei);
            List<UserDeviceBindApp> userDeviceBindAppList = ApplicationUtil.getApplicationContext().getBean(UserDeviceBindService.class).getBaseMapper().selectList(lambdaQueryWrapper);
            //推送标题和内容
            String msg = "";
            String msgContent = "";
            String abNormalType = "";
            String reminderType = "";
            if (CommonConstant.AUDIT_STATUS_ONE.equals(type)) {
                msg = name + "心率异常，当前监测心率为" + data + "次/分";
                msgContent = name + "手表当前监测心率为" + data + "次/分";
                abNormalType = "abnormalHeartRate";
                reminderType = "4";
            } else {
                msg = name + "体温异常，当前监测体温为" + data + "℃";
                msgContent = name + "手表当前监测体温为" + data + "℃";
                abNormalType = "abnormalBodyTemperature";
                reminderType = "5";
            }
            HeartPush heartPush;
            HeartPushChild heartPushChild;
            if (!CollectionUtils.isEmpty(userDeviceBindAppList)) {
                //新增心率、体温异常信息到提醒消息记录表
                insertHeartOrTemperature(userDeviceBindAppList, name, msgContent, abNormalType, data, imei, reminderType);
                for (UserDeviceBindApp userDeviceBindApp : userDeviceBindAppList) {
                    heartPush = new HeartPush();
                    heartPush.setType("accountMessage");
                    heartPushChild = new HeartPushChild();
                    heartPushChild.setImei(imei);
                    heartPushChild.setType(abNormalType);
                    heartPushChild.setName(name);
                    heartPushChild.setData(data);
                    heartPushChild.setMsgTitle(msg);
                    heartPushChild.setMsgContent(msgContent);
                    heartPushChild.setTimestamp(System.currentTimeMillis());
                    heartPush.setMessage(heartPushChild);
                    String sendPushToAccount = JSONObject.toJSONString(heartPush);
                    String lang = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheMapValue(userDeviceBindApp.getUserId(), CommonConstant.IOS_LANGUAGE);
                    String agent = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheMapValue(userDeviceBindApp.getUserId(), CommonConstant.USER_AGENT);
                    String alert = I18nUtil.getMessage(CommonConstant.ACCEPT_CHANT_VIDEO, lang);
                    //判断是发IOS还是安卓
                    if (CommonConstant.ANDROID.equals(agent)) {
                        ApplicationUtil.getApplicationContext().getBean(MqttPush.class).sendMqtt("3", userDeviceBindApp.getUserId(), "1", "accountMessage", abNormalType, sendPushToAccount);
                    } else {
                        ApplicationUtil.getApplicationContext().getBean(BindingDeviceImpl.class).sendIOS(userDeviceBindApp.getUserId(), heartPush, "accountMessage", abNormalType, null);
                    }

                    log.info("心率/体温告警信息下发mqtt成功:{},心率为:{}，类型为:{}", userDeviceBindApp.getUserId(), data, type);
                    //安卓系统推送
                    if ("Android".equals(agent)) {
                        //手机厂商机型：HW、XM等
                        String AndroidType = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheObject(CommonConstant.ANDROID_AGENT + userDeviceBindApp.getUserId());
                        if (StringUtils.isNotEmpty(AndroidType)) {
                            String content = "";
                            String abNormalMsg = "";
                            if (CommonConstant.AUDIT_STATUS_ONE.equals(type)) {
                                content = name + I18nUtil.getMessage(CommonConstant.HEART_RATE_ABNORMAL, lang) + data + I18nUtil.getMessage(CommonConstant.PRE_MIN, lang);
                                abNormalMsg = I18nUtil.getMessage(CommonConstant.HEART_SIMPLE_ABNORMAL, lang);
                            } else {
                                content = name + I18nUtil.getMessage(CommonConstant.TEMP_ABNORMAL, lang) + data + "℃";
                                abNormalMsg = I18nUtil.getMessage(CommonConstant.TEMP_SIMPLE, lang);
                            }
                            if ("HW".equals(AndroidType)) {
                                //获取对应的token
                                String deviceToken = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheObject("HW:" + userDeviceBindApp.getUserId());
                                //服务端获取华为的token
                                String huaweiAccessToken = "";
                                if (null != ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheObject("huaweiAccessToken")) {
                                    huaweiAccessToken = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheObject("huaweiAccessToken");
                                } else {
                                    huaweiAccessToken = getHuaweiAccessToken();
                                    ApplicationUtil.getApplicationContext().getBean(RedisService.class).setCacheObject("huaweiAccessToken", huaweiAccessToken, 2L, TimeUnit.HOURS);
                                }
                                JSONObject first = new JSONObject();
                                first.put("validate_only", false);
                                JSONObject titleObject = new JSONObject();
                                titleObject.put("title", abNormalMsg);
                                titleObject.put("body", content);
                                JSONObject notificationObject = new JSONObject();
                                notificationObject.put("notification", titleObject);
                                JSONObject clickActionObject = new JSONObject();
                                clickActionObject.put("type", 1);
                                clickActionObject.put("action", "com.ecell.intent.action.MESSAGE");
                                JSONObject notificationClick = new JSONObject();
                                notificationClick.put("notification", clickActionObject);
                                notificationObject.put("android", notificationClick);
                                notificationObject.put("token", deviceToken);
                                first.put("message", notificationObject);
                                String huaweiMsgData = JSONObject.toJSONString(first);
                                pushMessage(userDeviceBindApp.getUserId(), huaweiMsgData, huaweiAccessToken);
                            } else if ("XM".equals(AndroidType)) {
                                //获取对应的token
                                String deviceToken = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheObject("XM:" + userDeviceBindApp.getUserId());
                                String intentUri = "intent:#Intent;component=com.ecellsz.watch.guard/com.cyrus.mine.function.msg.MsgActivity;end";
                                pushMiMessage(abNormalMsg, content, "", deviceToken, intentUri);
                            } else if ("VI".equals(AndroidType)) {
                                //获取对应的token
                                String deviceToken = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheObject("VI:" + userDeviceBindApp.getUserId());
                                //服务端获取VIVO的token
                                String viVoAccessToken = "";
                                if (null != ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheObject("viVoAccessToken")) {
                                    viVoAccessToken = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheObject("viVoAccessToken");
                                } else {
                                    viVoAccessToken = getViVoAccessToken();
                                    ApplicationUtil.getApplicationContext().getBean(RedisService.class).setCacheObject("viVoAccessToken", viVoAccessToken, 12L, TimeUnit.HOURS);
                                }
                                JSONObject msgJson = new JSONObject();
                                msgJson.put("regId", deviceToken);
                                msgJson.put("notifyType", 4);
                                msgJson.put("title", abNormalMsg);
                                msgJson.put("content", content);
                                msgJson.put("skipType", 4);
                                msgJson.put("skipContent", "intent://com.ecellsz.push.message/detail#Intent;scheme=vpushscheme;component=com.ecellsz.watch.guard/com.cyrus.mine.function.msg.MsgActivity;end");
                                msgJson.put("requestId", com.ecell.internationalize.common.core.utils.uuid.UUID.fastUUID().toString(true));
                                //msgJson.put("clientCustomMap",downVideoCallWithWatch);
                                msgJson.put("classification", 1);
                                sendViVoMessage(viVoAccessToken, msgJson, userDeviceBindApp.getUserId());
                            } else if ("OP".equals(AndroidType)) {
                                //获取对应的token
                                String deviceToken = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheObject("OP:" + userDeviceBindApp.getUserId());
                                //服务端获取OPPO的token
                                String opPoAccessToken = "";
                                if (null != ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheObject("opPoAccessToken")) {
                                    opPoAccessToken = ApplicationUtil.getApplicationContext().getBean(RedisService.class).getCacheObject("opPoAccessToken");
                                } else {
                                    opPoAccessToken = getOpPoAccessToken();
                                    ApplicationUtil.getApplicationContext().getBean(RedisService.class).setCacheObject("opPoAccessToken", opPoAccessToken, 12L, TimeUnit.HOURS);
                                }
                                JSONObject postData = new JSONObject();
                                postData.put("target_type", 2);
                                postData.put("target_value", deviceToken);
                                JSONObject postData1 = new JSONObject();
                                postData1.put("title", abNormalMsg);
                                postData1.put("content", content);
                                postData1.put("click_action_type", 1);
                                postData1.put("click_action_activity", "com.ecell.intent.action.MESSAGE");
                                //postData1.put("action_parameters",downVideoCallWithWatch);
                                postData.put("notification", postData1);
                                String str = JSONObject.toJSONString(postData);
                                sendOpPoMessage(opPoAccessToken, str, userDeviceBindApp.getUserId());
                            }
                        }
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.info("心率/体温告警信息下发mqtt失败原因:{}", e.getMessage());
        }

    }

    /**
     * 新增心率、体温异常信息到提醒消息记录表
     */
    public void insertHeartOrTemperature(List<UserDeviceBindApp> userDeviceBindAppList, String name, String msgContent, String type, String data, String imei, String reminderType) {
        try {
            List<ReminderMessageApp> list = new ArrayList<>();
            //封装JSON数据
            HeartTemperatureAbnormal heartTemperatureAbnormal = new HeartTemperatureAbnormal();
            heartTemperatureAbnormal.setType(type);
            heartTemperatureAbnormal.setTime(System.currentTimeMillis());
            heartTemperatureAbnormal.setName(name);
            heartTemperatureAbnormal.setData(data);
            heartTemperatureAbnormal.setImei(imei);
            heartTemperatureAbnormal.setMsgContent(msgContent);
            String messageDic = JSONObject.toJSONString(heartTemperatureAbnormal);
            ReminderMessageApp reminderMessageApp;
            for (UserDeviceBindApp userDeviceBindApp : userDeviceBindAppList) {
                reminderMessageApp = new ReminderMessageApp();
                reminderMessageApp.setId(UUID.fastUUID().toString(true));
                reminderMessageApp.setImei(imei);
                reminderMessageApp.setType(type);
                reminderMessageApp.setMessageRead("0");
                reminderMessageApp.setContext(messageDic);
                reminderMessageApp.setUserId(userDeviceBindApp.getUserId());
                reminderMessageApp.setCreateTime(new Date());
                reminderMessageApp.setRemindType(reminderType);
                list.add(reminderMessageApp);
            }
            ApplicationUtil.getApplicationContext().getBean(ReminderMessageAppService.class).saveBatch(list);
            log.info("心率/体温告警信息保存到提醒消息表成功");

        } catch (Exception e) {
            log.info("保存心率/体温告警信息保存到提醒消息表异常,异常原因:{}", e.getMessage());
        }
    }

    private String getHuaweiAccessToken() {
        HttpClient httpClient = new HttpClient();
        PostMethod postMethod = new PostMethod("https://oauth-login.cloud.huawei.com/oauth2/v2/token");
        //设置Content-Type，此处根据实际情况确定
        postMethod.addRequestHeader("Content-Type", "application/x-www-form-urlencoded");
        //请求内容

        postMethod.addParameter("grant_type", "client_credentials");
        postMethod.addParameter("client_id", "102624281");
        postMethod.addParameter("client_secret", "8446cc8bb2d4ef4c6e1c9a360d2e13737ac3b8f92aca4724926d3d4263b2fd68");
        JSONObject result = new JSONObject();
        try {
            int code = httpClient.executeMethod(postMethod);
            log.info("获取华为token执行返回的code为:{}", code);
            if (code == 200) {
                result = JSON.parseObject(postMethod.getResponseBodyAsString());
                log.info("返回的华为的token:{}", result.get("access_token").toString());
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        return null == result.get("access_token") ? null : result.get("access_token").toString();
    }

    private void pushMessage(String userId, String huaweiMsgData, String huaweiAccessToken) throws UnsupportedEncodingException {
        HttpClient httpClient = new HttpClient();
        PostMethod postMethod = new PostMethod("https://push-api.cloud.huawei.com/v1/102624281/messages:send");
        //设置Content-Type，此处根据实际情况确定
        postMethod.addRequestHeader("Content-Type", "application/json");
        postMethod.addRequestHeader("Authorization", "Bearer" + huaweiAccessToken);

        RequestEntity se = new StringRequestEntity(huaweiMsgData, "application/json", "UTF-8");
        postMethod.setRequestEntity(se);
        try {
            int code = httpClient.executeMethod(postMethod);
            log.info("华为系统推送消息的code:{}", code);
            JSONObject result = JSON.parseObject(postMethod.getResponseBodyAsString());
            log.info("返回的华为系统推送消息结果:{}", result);
            String resultCode = result.get("code").toString();
            if ("80200003".equals(resultCode)) {
                String accessToken = getHuaweiAccessToken();
                ApplicationUtil.getApplicationContext().getBean(RedisService.class).setCacheObject("huaweiAccessToken", accessToken, 1L, TimeUnit.HOURS);
                //pushMessage(userId, huaweiMsgData, accessToken);
            }
            if ("80000000".equals(resultCode)) {
                log.info("推送给华为用户:{},信息成功:{}", userId, result);
            } else {
                log.info("推送给华为用户:{},信息失败:{}", userId, result);
            }
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            httpClient.getHttpConnectionManager().closeIdleConnections(0);
            log.info("线程:{}推送华为用户,释放连接完成", Thread.currentThread().getName());
        }
    }

    private String getViVoAccessToken() throws NoSuchAlgorithmException, UnsupportedEncodingException {
        HttpClient httpClient = new HttpClient();
        PostMethod postMethod = new PostMethod("https://api-push.vivo.com.cn/message/auth");
        String timestamp = String.valueOf(System.currentTimeMillis());
        String sign = Md5Utils.signUtil(timestamp);
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("appId", 104088249);
        jsonObject.put("appKey", "33a6df6da5a10d0e81c6737a192213fc");
        jsonObject.put("timestamp", timestamp);
        jsonObject.put("sign", sign);
        System.out.println("sign:" + sign);
        String str = JSONObject.toJSONString(jsonObject);
        postMethod.addRequestHeader("Content-Type", "application/json");
        RequestEntity se = new StringRequestEntity(str, "application/json", "UTF-8");
        postMethod.setRequestEntity(se);
        String authToken = null;
        try {
            httpClient.executeMethod(postMethod);
            JSONObject result = JSON.parseObject(postMethod.getResponseBodyAsString());
            System.out.println("返回的小米系统推送消息结果:{}" + result);
            if (Integer.parseInt(result.get("result").toString()) == 0) {
                authToken = result.get("authToken").toString();
                log.info("获取VIVO token成功:{}", authToken);
            } else {
                log.info("获取VIVO token失败:{}", result);
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        return authToken;
    }

    private void sendViVoMessage(String viVoAccessToken, JSONObject msgJson, String userId) throws UnsupportedEncodingException {
        String str = JSONObject.toJSONString(msgJson);
        HttpClient httpClient = new HttpClient();
        PostMethod postMethod = new PostMethod("https://api-push.vivo.com.cn/message/send");
        postMethod.addRequestHeader("Content-Type", "application/json");
        postMethod.addRequestHeader("authToken", viVoAccessToken);
        RequestEntity se = new StringRequestEntity(str, "application/json", "UTF-8");
        postMethod.setRequestEntity(se);
        try {
            int code = httpClient.executeMethod(postMethod);
            log.info("VIVO系统推送消息的code:{}", code);
            JSONObject result = JSON.parseObject(postMethod.getResponseBodyAsString());
            log.info("返回的VIVO系统推送消息结果:{}", result);
            int resultCode = Integer.parseInt(result.get("result").toString());
            if (resultCode == 0) {
                log.info("推送给VIVO用户:{},信息成功:{}", userId, result);
            } else {
                log.info("推送给VIVO用户:{},信息失败:{}", userId, result);
            }
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            httpClient.getHttpConnectionManager().closeIdleConnections(0);
            log.info("线程:{}推送VIVO用户,释放连接完成", Thread.currentThread().getName());
        }
    }

    private String getOpPoAccessToken() {
        HttpClient httpClient = new HttpClient();
        PostMethod postMethod = new PostMethod("https://api.push.oppomobile.com/server/v1/auth");
        postMethod.addRequestHeader("Content-Type", "application/x-www-form-urlencoded");
        String timestamp = String.valueOf(System.currentTimeMillis());
        String signData = "2a005bdf43cd450e8ab1622cd4a3d087" + timestamp + "e69449a5e366430ba7cf532745b396f2";
        String sign = Md5Utils.getSha256Str(signData);
        System.out.println(sign);
        postMethod.addParameter("app_key", "2a005bdf43cd450e8ab1622cd4a3d087");
        postMethod.addParameter("timestamp", timestamp);
        postMethod.addParameter("sign", sign);
        String authToken = null;
        try {
            int code = httpClient.executeMethod(postMethod);
            log.info("OPPO系统推送消息的code:{}", code);
            JSONObject result = JSON.parseObject(postMethod.getResponseBodyAsString());
            log.info("返回的OPPO系统推送消息结果:{}", result);
            int resultCode = Integer.parseInt(result.get("code").toString());
            if (resultCode == 0) {
                log.info("推送给VIVO用户,信息成功:{}", result);
                JSONObject data = JSON.parseObject(result.get("data").toString());
                authToken = data.get("auth_token").toString();
            } else {
                log.info("推送给VIVO用户:信息失败:{}", result);
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        return authToken;
    }

    private void sendOpPoMessage(String opPoAccessToken, String str, String userId) {
        HttpClient httpClient = new HttpClient();
        PostMethod postMethod = new PostMethod("https://api.push.oppomobile.com/server/v1/message/notification/unicast");
        postMethod.getParams().setParameter(HttpMethodParams.HTTP_CONTENT_CHARSET, "utf-8");
        postMethod.addRequestHeader("Content-Type", "application/x-www-form-urlencoded");
        postMethod.addRequestHeader("auth_token", opPoAccessToken);
        postMethod.addParameter("message", str);
        try {
            int code = httpClient.executeMethod(postMethod);
            log.info("OPPO系统推送消息的code:{}", code);
            JSONObject result = JSON.parseObject(postMethod.getResponseBodyAsString());
            log.info("返回的OPPO系统推送消息结果:{}", result);
            int resultCode = Integer.parseInt(result.get("code").toString());
            if (resultCode == 0) {
                log.info("推送给OPPO用户{},信息成功:{}", userId, result);
            } else {
                log.info("推送给OPPO用户{}:信息失败:{}", userId, result);
            }
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            httpClient.getHttpConnectionManager().closeIdleConnections(0);
            log.info("线程:{}推送OPPO用户,释放连接完成", Thread.currentThread().getName());
        }
    }
}
