package com.ecell.internationalize.app.util;

import com.ecell.internationalize.app.config.MyLocaleResolverConfig;
import com.ecell.internationalize.common.core.utils.StringUtils;
import lombok.var;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.support.ResourceBundleMessageSource;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.nio.charset.StandardCharsets;
import java.util.Locale;

@Component
public class I18nUtil {
    @Value("${spring.messages.basename}")
    private String basename;

    private final MyLocaleResolverConfig resolver;

    private static MyLocaleResolverConfig customLocaleResolver;

    private static String path;

    public I18nUtil(MyLocaleResolverConfig resolver) {
        this.resolver = resolver;
    }
    @PostConstruct
    public void init() {
        setBasename(basename);
        setCustomLocaleResolver(resolver);
    }
    /**
     * 获取 国际化后内容信息
     * @param code 国际化key
     * @return 国际化后内容信息
     */
    public static String getMessage(String code) {
        Locale locale = customLocaleResolver.getLocal();
        return getMessage(code, null, code, locale);
    }
    /**
     * 获取指定语言中的国际化信息，如果没有则走英文
     * @param code 国际化 key(properties文件中的key)
     * @param lang 语言参数,前端请求头带lang
     * @return 国际化后内容信息
     */
    public static String getMessage(String code, String lang) {
        Locale locale;
        if (StringUtils.isEmpty(lang)) {
            locale = Locale.US;
        } else {
            try {
                var split = lang.split("_");
                locale = new Locale(split[0], split[1]);
            } catch (Exception e) {
                locale = Locale.US;
            }
        }
        return getMessage(code, null, code, locale);
    }
    /**
     * code：信息的键，properties中的key
     * args：系统运行时参数，可为空
     * defaultMessage：默认信息，可为空
     * locale：区域信息，我们可以通过java.util.Locale类下面的静态常量找到对应的值。如：简体中文就是zh_CN；英文就是en_US，详见java.util.Locale中的常量值。
     */
    public static String getMessage(String code, Object[] args, String defaultMessage, Locale locale) {
        ResourceBundleMessageSource messageSource = new ResourceBundleMessageSource();
        messageSource.setDefaultEncoding(StandardCharsets.UTF_8.toString());
        messageSource.setBasename(path);
        String content;
        try {
            content = messageSource.getMessage(code, args, locale);
        } catch (Exception e) {
            e.printStackTrace();
            System.out.println("国际化参数获取失败===>"+e.getMessage());
            content = defaultMessage;
        }
        return content;

    }

    public static void setBasename(String basename) {
        I18nUtil.path = basename;
    }

    public static void setCustomLocaleResolver(MyLocaleResolverConfig resolver) {
        I18nUtil.customLocaleResolver = resolver;
    }
}
