package com.ecell.internationalize.app.util;

import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.ecell.internationalize.app.entity.DeviceLocationApp;
import com.ecell.internationalize.app.service.impl.DeviceLocationAppServiceImpl;
import com.ecell.internationalize.common.core.exception.ServiceException;
import com.ecell.internationalize.common.core.utils.StringUtils;
import com.ecell.internationalize.common.issue.component.IssueInstructions;
import com.ecell.internationalize.common.issue.constant.CommonConstant;
import com.ecell.internationalize.common.issue.entity.tcp.*;
import com.ecell.internationalize.common.issue.util.RandomUtil;
import com.ecell.internationalize.common.redis.service.RedisService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.cloud.context.config.annotation.RefreshScope;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import javax.annotation.PostConstruct;
import java.util.*;

@Component
@RefreshScope
public class LocationUtil {
    private static final Logger log = LoggerFactory.getLogger(LocationUtil.class);
    /**
     * 地球半径,单位 m
     */
    private static final double EARTH_RADIUS = 6371137.0;
    @Autowired
    private DeviceLocationAppServiceImpl deviceLocationAppService;
    @Autowired
    private RedisService redis;
    @Autowired
    private RestTemplate restTemplateOne;
    @Autowired
    private IssueInstructions issueInstructions;
    @Value("${gaudy.url}")
    private String path;
    @Value("${gaudy.key}")
    private String key;
    @Value("${gaudy.gpskey}")
    private String gpsKey;
    @Value("${gaudy.gpsurl}")
    private String gpsPath;

    private static DeviceLocationAppServiceImpl appService;
    private static RedisService redisService;
    private static RestTemplate restTemplate;
    private static String url;
    private static String gaudyKey;
    private static String GPS_Key;
    private static String gpsUrl;
    private static IssueInstructions iss;

    @PostConstruct
    public void init() {
        LocationUtil.restTemplate = restTemplateOne;
        LocationUtil.url = path;
        LocationUtil.redisService = redis;
        LocationUtil.gaudyKey = key;
        LocationUtil.GPS_Key = gpsKey;
        LocationUtil.appService = deviceLocationAppService;
        LocationUtil.gpsUrl = gpsPath;
        LocationUtil.iss = issueInstructions;
    }


    /**
     * 获取定位信息，返回定位信息的ID
     *
     * @param positionInfoap
     * @return
     */
    public static DeviceLocationApp getLocation(PositionInfo positionInfoap) {
        log.info("开始后位置解析=======");
        PositionResult positionResult = null;
        DeviceLocationApp app = null;
        GPSResult gpsResult = null;
        String locationType = "LBS";
        int errCount = 0;
        int gpsErrCount = 0;
        if (StringUtils.isNull(positionInfoap.getGps())) {
            log.info("WiFi 基站解析=======");
            StringBuffer bts_buffer = new StringBuffer();
            StringBuffer nearbts_buffer = new StringBuffer();
            String bts = "";
            String nearbts = "";
            String cdma = positionInfoap.getBaseStationType();
            String accesstype = "0";
            String mmac = "";
            String macs = "";
            String wifiStr = "";
            List<BaseStation> baseStations = positionInfoap.getBaseStation();
            List<WIFI> wifis = positionInfoap.getWifi();
            if (null != baseStations && baseStations.size() > 0) {
                //先请求基站定位(locationType 定位类型 WIFI 还是 LBS)
                if ("0".equals(positionInfoap.getBaseStationType())) {//是否 CDMA(0:否, 1:是)
                    //cdma = positionInfoap.getBaseStationType();
                    // bts_buffer.append("&accesstype=").append(positionInfoap.getBaseStationType());
                    //基站信息
//                //获取第一个基站信息
//                BaseStation baseStation = baseStations.get(0);
//                bts_buffer.append("&mcc=").append(baseStation.getMcc()).append("&mnc=").append(baseStation.getMnc()).append("&lac=").append(baseStation.getLac())
//                        .append("&ci=").append(baseStation.getCi());
                    // bts_buffer.append("&signal=").append(signal);
                    int signal = getSignal(baseStations.get(0).getRxlev());

                    //在遍历这个基站信息
                    for (int i = 0; i < baseStations.size(); i++) {
                        int signal1 = getSignal(baseStations.get(i).getRxlev());
                        if (signal1 < signal) {
                            bts = "";
                            bts = bts_buffer.append(baseStations.get(i).getMcc()).append(",").append(baseStations.get(i).getMnc()).append(",").append(baseStations.get(i).getLac()).append(",")
                                    .append(baseStations.get(i).getCi()).append(",").append(signal1).toString();
                        } else {
                            bts = bts_buffer.append(baseStations.get(0).getMcc()).append(",").append(baseStations.get(0).getMnc()).append(",").append(baseStations.get(0).getLac()).append(",")
                                    .append(baseStations.get(0).getCi()).append(",").append(signal).toString();
                        }

                        if (i != baseStations.size() - 1) {
                            nearbts += nearbts_buffer.append(baseStations.get(i).getMcc()).append(",").append(baseStations.get(i).getMnc()).append(",").append(baseStations.get(i).getLac()).append(",")
                                    .append(baseStations.get(i).getCi()).append(signal1).append("|").toString();

                        } else {
                            nearbts += nearbts_buffer.append(baseStations.get(i).getMcc()).append(baseStations.get(i).getMnc()).append(baseStations.get(i).getLac())
                                    .append(baseStations.get(i).getCi()).append(signal1).toString();

                        }
                    }


                } else { //基站信息

                    String lat = "";
                    String lon = "";
                    String rxlev = "0";
                    cdma = positionInfoap.getBaseStationType();
                    if (baseStations.size() == 1) {
                        bts = bts_buffer.append(baseStations.get(0).getSid()).append(",").append(baseStations.get(0).getNid()).append(",").append(baseStations.get(0).getBid())
                                .append(",").append(lon).append(",").append(lat).append("&").toString();
                    } else {
                        for (int i = 0; i < baseStations.size(); i++) {
                            if (i != baseStations.size() - 1) {
                                bts += bts_buffer.append(baseStations.get(i).getSid()).append(",").append(baseStations.get(i).getNid()).append(",").append(baseStations.get(i).getBid())
                                        .append(",").append(lon).append(lat).append("|").toString();
                            } else {
                                bts += bts += bts_buffer.append(baseStations.get(i).getSid()).append(",").append(baseStations.get(i).getNid()).append(",").append(baseStations.get(i).getBid())
                                        .append(",").append(lon).append(",").append(lat).append("&").toString();
                            }

                        }
                    }


                }
            }
            log.info("电信的LBS 定位 bts 参数 bts,nearbts,locationType,accesstype,{}{}{}{}", bts, nearbts, locationType, accesstype);
            if (null != wifis && wifis.size() > 0) {
                int signal = wifis.get(0).getSignal();
                accesstype = "1";

                if (wifis.size() == 1) {
                    mmac = String.format(mmac + "%s,%s,%s", wifis.get(0).getMac(), signal, getOneWifiSSID(wifis.get(0).getSsid()));

                } else {
                    StringBuffer buffer = new StringBuffer();
                    for (int i = 0; i < wifis.size(); i++) {
                        int signal1 = wifis.get(i).getSignal();
                        if (signal1 > 0) {
                            signal1 = 2 * signal1 - 113;
                            if (signal1 > 0) {
                                log.info("wifi 定位强度不能为正数{}", signal1);
                            }
                        }
                        if (signal1 < signal) {
                            signal = Math.abs(signal1);
                            // mmac = String.format(mmac + "%s,%s,%s", wifis.get(i).getMac(), signal1, getOneWifiSSID(wifis.get(i).getSsid()));
                            buffer.setLength(0);
                            mmac = buffer.append(wifis.get(i).getMac()).append(",").append(signal1).append(",").append(getOneWifiSSID(wifis.get(i).getSsid())).delete(buffer.length() - 1, buffer.length()).toString();

                        }
                        if (i != wifis.size() - 1) {
                            wifiStr += String.format(wifiStr + "%s,%s,%s%s", wifis.get(i).getMac(), signal1, getOneWifiSSID(wifis.get(i).getSsid()), "|");
                        } else {
                            wifiStr += String.format(wifiStr + "%s,%s,%s", wifis.get(i).getMac(), signal1, getOneWifiSSID(wifis.get(i).getSsid()));
                        }
                    }
                    macs = wifiStr;
                    locationType = "WIFI";

                }
            }
            StringBuffer buffer = new StringBuffer();
            buffer.append("accesstype=").append(accesstype).append("&imei=").append(positionInfoap.getImei());
            //查询缓存 imsi
            Object imsi = redisService.getCacheMapValue(positionInfoap.getImei(), "imsi");
            if (StringUtils.isNotNull(imsi)) {
                buffer.append("&imsi=").append(imsi);
            }
            String s = buffer.append("&network=").append("GMS").append("&cdma=").append(cdma).append("&bts=").append(bts).append("&nearbts=").append(nearbts)
                    .append("&mmac=").append(mmac).append("&macs=").append(macs).append("&output=json").append("&key=").append(gaudyKey).toString();
            log.info("locationType:{},参数:{}", locationType, s);
            String s1 = String.format(url + "%s%s", "?", s);
            try {
                positionResult = LocationUtil.getLocationResult(s1);
                log.info("LBS WIFI请求第三接口结果集数据:{}", positionResult);
//                if ("10000".equals(positionResult.getInfocode()) && null ==positionResult.getResult() && !StringUtils.isNotEmpty(positionResult.getResult().getLocation())){
//                    count++;
//                    if (count<6){ //重试6次
//                        positionResult= LocationUtil.getLocationResult(s1);
//                    }
//                }
            } catch (Exception e) {
                errCount++;
                if (errCount < 6) { //重试6次
                    try {
                        Thread.sleep(1000);
                    } catch (InterruptedException interruptedException) {
                        log.info("LBS线程休眠失败:{}", interruptedException.getMessage());
                    }
                    positionResult = LocationUtil.getLocationResult(s1);
                }
                log.info("请求第三接口失败:{},重试次数{}", e.getMessage(), errCount);
                log.info("LBS WIFI error 请求第三方接口数据:{}", positionResult);
            }
            log.info("LBS WIFI 请求第三方接口数据:{}", positionResult);
        } else { //GPS定位
            log.info("GPS解析=======");
            GPSInfo gps = positionInfoap.getGps();
            locationType = "GPS";
            //TODO 这里先注释 以后可以优化
//            try {
//                gpsResult = getGps(gps.getLon(), gps.getLat());
//
//
//            } catch (Exception e) {
//                gpsErrCount++;
//                try {
//                    Thread.sleep(1000);
//                } catch (InterruptedException interruptedException) {
//                    log.info("GPS线程休眠失败:{}", interruptedException.getMessage());
//                }
//                if (gpsErrCount < 6) {
//                    gpsResult = LocationUtil.getGps(gps.getLon(), gps.getLat());
//                }
//                log.info("请求第三接口失败:{},重试次数{}", e.getMessage(), errCount);
//                log.info("GPS error 请求第三方接口数据:{}", gpsResult);
//            }
//            log.info("GPS 请求第三方接口结果集数据:{}", gpsResult);
            //从这里如果GPS接口请求失败，那就只存坐标
            gpsResult = getGps(gps.getLon(), gps.getLat());
            log.info("获取到啊的gpsResult{}", gpsResult);
            if (StringUtils.isNotNull(gpsResult)) {
                return addGPSDataApp(gps.getLon().toString(), gps.getLat().toString(), locationType, positionInfoap.getImei());
            }

        }
        // if ("location".equals(positionInfoap.getType())){
        //添加数据库处理
        if (StringUtils.isNotNull(positionResult) && StringUtils.isNotNull(positionResult.getResult())) {
            app = addLBSData(positionInfoap, positionResult, locationType);
        }

        if (StringUtils.isNotNull(gpsResult) && StringUtils.isNotNull(gpsResult.getRegeoCode().getAddressComponent()) && StringUtils.isNotNull(gpsResult.getRegeoCode().getAddressComponent().getStreetNumber())) {
            app = addGPSData(positionInfoap, gpsResult, locationType);

        }
        // }
        log.info("转换后得到的DeviceLocationApp{}", app);
        if (StringUtils.isNotEmpty(app.getLatitude()) && StringUtils.isNotEmpty(app.getLongitude())) {
            appService.save(app);
            return app;
        } else {
            return null;
        }

    }


    /**
     * 获取定位信息，请求Google接口
     *
     * @param positionInfoap
     * @return
     */
    public static DeviceLocationApp getLocationGoogle(PositionInfo positionInfoap) {
        DeviceLocationApp app = null;
        String locationType = "LBS";
        //不是GPS就是基站定位+wiff
        if (StringUtils.isNull(positionInfoap.getGps())) {
            List<BaseStation> baseStations = positionInfoap.getBaseStation();
            List<WIFI> wifis = positionInfoap.getWifi();
            if (null != baseStations && baseStations.size() > 0) {
                List<CellTowerInfo> cellTowerInfoList = new ArrayList<>();
                List<WifiAccessPoints> wifiAccessPointsList = new ArrayList<>();
                GoogleBaseStationInfo info = new GoogleBaseStationInfo();
                info.setConsiderIp(false);
                if ("0".equals(positionInfoap.getBaseStationType())) {//是否 CDMA(0:否, 1:是)
                    info.setRadioType("gsm");
                    info.setHomeMobileCountryCode(baseStations.get(0).getMcc());
                    info.setHomeMobileNetworkCode(baseStations.get(0).getMnc());
                    for (BaseStation baseStation : baseStations) {
                        CellTowerInfo cellTowerInfo = new CellTowerInfo();
                        cellTowerInfo.setCellId(baseStation.getCi());
                        cellTowerInfo.setLocationAreaCode(baseStation.getLac());
                        cellTowerInfo.setMobileCountryCode(baseStation.getMcc());
                        cellTowerInfo.setMobileNetworkCode(baseStation.getMnc());
                        cellTowerInfoList.add(cellTowerInfo);

                    }


                } else {
                    info.setRadioType("cdma");
                    info.setHomeMobileNetworkCode(baseStations.get(0).getSid());
                    for (BaseStation baseStation : baseStations) {
                        CellTowerInfo cellTowerInfo = new CellTowerInfo();
                        cellTowerInfo.setCellId(baseStation.getBid());
                        cellTowerInfo.setLocationAreaCode(baseStation.getNid());
                        cellTowerInfo.setMobileNetworkCode(baseStation.getSid());
                        cellTowerInfoList.add(cellTowerInfo);
                    }

                }

                if (StringUtils.isNotEmpty(wifis) && wifis.size() >= 2) {

                    for (WIFI wifi : wifis) {
                        WifiAccessPoints wifiAccessPoints = new WifiAccessPoints();
                        wifiAccessPoints.setMacAddress(wifi.getMac());
                        wifiAccessPoints.setSignalStrength(wifi.getSignal());
                        wifiAccessPointsList.add(wifiAccessPoints);

                    }

                    locationType = "WIFI";

                }
                info.setCellTowers(cellTowerInfoList);
                info.setWifiAccessPoints(wifiAccessPointsList);
                //请求Google接口
                Map<String, String> lbsWifi = getLBSWifi(info);

                if (StringUtils.isNotNull(lbsWifi)) {
                    app = newAddLBSData(positionInfoap, lbsWifi, locationType);


                }

            }


        } else {//GPS
            GPSInfo gps = positionInfoap.getGps();
            locationType = "GPS";
            Map<String, String> map = new HashMap<>();
            try {
                map = getGpsGoogle(gps.getLon(), gps.getLat());
                log.info("GPS 请求谷歌接口结果集数据:{}", map);
            } catch (Exception e) {
                log.info("GPS 请求谷歌接口出错:{}", e);
                return nweAddGPSDataTwo(locationType, gps.getLon(), gps.getLat());

            }


            app = nweAddGPSData(positionInfoap, map, locationType, gps.getLon(), gps.getLat());


        }
        if (StringUtils.isNotEmpty(app.getLatitude()) && StringUtils.isNotEmpty(app.getLongitude())) {
            appService.save(app);
            return app;
        } else {
            return null;
        }

    }


    /**
     * //GPS添加数据库
     *
     * @param positionInfoap
     * @param gpsResult
     * @param locationType
     */
    private static DeviceLocationApp addGPSData(PositionInfo positionInfoap, GPSResult gpsResult, String locationType) {
        String lon = null, lat = null; //经度，纬度
        AddressComponent addressComponent = gpsResult.getRegeoCode().getAddressComponent();
        if (StringUtils.isNotEmpty(addressComponent.getStreetNumber().getLocation())) {
            String location = addressComponent.getStreetNumber().getLocation();
            String[] split = location.split(",");
            if (split.length > 1) {
                lon = split[0]; //经度
                lat = split[1]; //纬度
            }
        }
        DeviceLocationApp app = new DeviceLocationApp();
        app.setId(com.ecell.internationalize.common.core.utils.uuid.UUID.fastUUID().toString(true));
        app.setLocationType(locationType); //定位类型
        app.setAdcode(addressComponent.getAdcode()); //区域编码
        app.setAddr(gpsResult.getRegeoCode().getFormattedAddress().trim().replace(" ", ""));  //详细地址
        //app.setRadius();GPS没有返回
        app.setLongitude(lon); //经度
        app.setLatitude(lat);  //纬度
        app.setCity(addressComponent.getCity()); //城市
        app.setImei(positionInfoap.getImei());
        app.setCountry(addressComponent.getCountry());
        app.setProvince(addressComponent.getProvince());
        return app;
    }

    /**
     * //GPS添加数据库,就算请求地址解析接口失败也添加数据库
     *
     * @param locationType
     */
    private static DeviceLocationApp addGPSDataApp(String lon, String lat, String locationType, String imei) {

        DeviceLocationApp app = new DeviceLocationApp();
        app.setId(com.ecell.internationalize.common.core.utils.uuid.UUID.fastUUID().toString(true));
        app.setLocationType(locationType); //定位类型
        app.setLongitude(lon); //经度
        app.setLatitude(lat);  //纬度
        app.setImei(imei);
        return app;
    }

    /**
     * 新GPS添加数据库 Google
     *
     * @param positionInfoap
     * @param map
     * @param locationType
     * @param lon
     * @param lat
     * @return
     */
    private static DeviceLocationApp nweAddGPSData(PositionInfo positionInfoap, Map<String, String> map, String locationType, Float lon, Float lat) {
        DeviceLocationApp app = new DeviceLocationApp();
        app.setId(com.ecell.internationalize.common.core.utils.uuid.UUID.fastUUID().toString(true));
        app.setLocationType(locationType); //定位类型
        app.setAdcode(map.get("postalCode")); //邮政编码
        app.setAddr(map.get("addr").trim().replace(" ", ""));  //详细地址
        //app.setRadius();GPS没有返回
        app.setLongitude(lon.toString()); //经度
        app.setLatitude(lat.toString());  //纬度
        app.setCity(map.get("city")); //城市
        app.setImei(positionInfoap.getImei());
        app.setCountry(map.get("country"));
        app.setProvince(map.get("province"));
        return app;
    }

    private static DeviceLocationApp nweAddGPSDataTwo(String locationType, Float lon, Float lat) {
        DeviceLocationApp app = new DeviceLocationApp();
        app.setId(com.ecell.internationalize.common.core.utils.uuid.UUID.fastUUID().toString(true));
        app.setLocationType(locationType); //定位类型
        app.setLongitude(lon.toString()); //经度
        app.setLatitude(lat.toString());  //纬度
        return app;
    }


    /**
     * //LBS WIFF 添加数据库
     *
     * @param positionInfoap
     * @param positionResult
     * @param locationType
     */
    private static DeviceLocationApp addLBSData(PositionInfo positionInfoap, PositionResult positionResult, String locationType) {
        String lon = null, lat = null; //经度，纬度
        Result result = positionResult.getResult();
        if (StringUtils.isNotEmpty(result.getLocation())) {
            String[] split = result.getLocation().split(",");
            if (split.length > 1) {
                lon = split[0]; //经度
                lat = split[1]; //纬度
            }

        }

        DeviceLocationApp app = new DeviceLocationApp();
        app.setId(com.ecell.internationalize.common.core.utils.uuid.UUID.fastUUID().toString(true));
        app.setLocationType(locationType); //定位类型
        app.setAdcode(result.getAdcode()); //区域编码
        app.setAddr(result.getDesc().trim().replace(" ", ""));  //详细地址
        app.setRadius(result.getRadius());
        app.setLongitude(lon); //经度
        app.setLatitude(lat);  //纬度
        app.setCity(result.getCity()); //城市
        app.setImei(positionInfoap.getImei());
        app.setCountry(result.getCountry());
        app.setProvince(result.getProvince());
        return app;
    }


    private static DeviceLocationApp newAddLBSData(PositionInfo positionInfoap, Map<String, String> map, String locationType) {

        DeviceLocationApp app = new DeviceLocationApp();
        app.setId(com.ecell.internationalize.common.core.utils.uuid.UUID.fastUUID().toString(true));
        app.setLocationType(locationType); //定位类型
        app.setAdcode(map.get("postalCode")); //邮政编码
        app.setAddr(map.get("addr").trim().replace(" ", ""));  //详细地址
        app.setRadius(map.get("radius"));
        app.setLongitude(map.get("lng")); //经度
        app.setLatitude(map.get("lat"));  //纬度
        app.setCity(map.get("city")); //城市
        app.setImei(positionInfoap.getImei());
        app.setCountry(map.get("country"));
        app.setProvince(map.get("province"));
        return app;
    }


    private static int getSignal(int rxlev) {
        //  必须为负数
        if (rxlev > 113) {
            return -48;
        } else {
            return 2 * rxlev - 113;
        }
    }

    private static String getOneWifiSSID(String s) {
        return s.replace(" ", "");
    }

    /**
     * @param lon 经度
     * @param lat 纬度
     * @return
     */
    public static GPSResult getGps(Float lon, Float lat) {
        log.info("GPS解析获取到的经纬度是lon:{},lat:{}", lon, lat);
        StringBuffer buffer = new StringBuffer();
        String s = buffer.append("key=").append(GPS_Key).append("&location=").append(lon).append(",").append(lat).toString();
        String format = String.format(gpsUrl + "%s%s", "?", s);
        log.info("GPS请求第三方接口地址：{}", format);
        try {
            GPSResult forObject = restTemplate.getForObject(format, GPSResult.class);
            if ("10000".equals(forObject.getInfocode()) && "1".equals(forObject.getStatus())) {
                return forObject;
            } else {
                log.info("GPS 请求第三方接返回数据：{}", forObject);
                // throw new ServiceException("GPS 请求第三方接口失败，请重试");
                return null;
            }
        } catch (Exception e) {
            log.error("GPS请求第三方接口异常：{}", e);
             return null;

        }


    }

    /**
     * 请求谷歌API
     *
     * @param lon
     * @param lat
     * @return
     */
    public static Map<String, String> getGpsGoogle(Float lon, Float lat) {
        Map<String, String> map = new HashMap<>(12);
        String uri = String.format("https://maps.googleapis.com/maps/api/geocode/json?latlng=%s,%s&language=zh&key=AIzaSyBhelN5j0OM_AUgtUxVeuW2eeh5HbuRQ9s", lat, lon).trim();
        JSONObject jsonObject = restTemplate.getForObject(uri, JSONObject.class);
        if (StringUtils.isNotNull(jsonObject)) {
            log.info("获取到的结果是:{}", jsonObject);
            try {
                JSONObject jsonObject2 = JSONObject.parseObject(jsonObject.toString());
                JSONArray jsonArray = jsonObject2.getJSONArray("results");
                if (StringUtils.isNotNull(jsonArray)) {
                    JSONObject jsonObject1 = jsonArray.getJSONObject(0);

                    if (StringUtils.isNotNull(jsonObject1)) {

                        map.put("addr", jsonObject1.getString("formatted_address"));
                        JSONArray jsonArray1 = jsonObject1.getJSONArray("address_components");
                        if (StringUtils.isNotNull(jsonArray1)) {
                            log.info("获取到的数组长度是:{}", jsonArray1.size());
                            for (int i = 0; i < jsonArray1.size(); i++) {
                                JSONObject jsonObject3 = jsonArray1.getJSONObject(i);
                                JSONArray array = jsonObject3.getJSONArray("types");
                                if (StringUtils.isNotNull(array) && array.size() >= 1) {
                                    if (array.getString(0).equals("country") && array.getString(1).equals("political")) {
                                        map.put("country", jsonObject3.getString("long_name"));
                                        log.info("获取到的国家是:=====>{}", jsonObject3.getString("long_name"));
                                    }
                                    if (array.getString(0).equals("administrative_area_level_1") && array.getString(1).equals("political")) {
                                        map.put("province", jsonObject3.getString("long_name"));
                                        log.info("获取到的省份是:=====>{}", jsonObject3.getString("long_name"));

                                    }


                                    if (array.getString(0).equals("locality") && array.getString(1).equals("political")) {
                                        map.put("city", jsonObject3.getString("long_name"));
                                        log.info("获取到的城市是:=====>{}", jsonObject3.getString("long_name"));

                                    }

                                }


                            }


                        }

                    }
                    JSONObject jsonObject3 = jsonArray.getJSONObject(1);
                    if (StringUtils.isNotNull(jsonObject3)) {
                        map.put("addrInfo", "formatted_address");
                        JSONArray addressComponents = jsonObject3.getJSONArray("address_components");
                        if (StringUtils.isNotNull(addressComponents)) {
                            for (int i = 0; i < addressComponents.size(); i++) {
                                JSONObject jsonObject4 = addressComponents.getJSONObject(i);
                                JSONArray types = jsonObject4.getJSONArray("types");
                                if (StringUtils.isNotNull(types) && types.size() >= 1) {
                                    if (types.getString(0).equals("postal_code")) {
                                        map.put("postalCode", jsonObject4.getString("long_name"));
                                    }

                                }


                            }


                        }


                    }


                }
            } catch (Exception e) {
                e.printStackTrace();
                throw new ServiceException("解析出错");
            }


        } else {
            log.info("GPS 请求第三方接返回数据：{}", jsonObject);
            throw new ServiceException("GPS 请求第三方接口失败，请重试");
        }

        return map;

    }

    /**
     * 基站 wifi 请求google API
     *
     * @param baseStationInfo
     * @return
     */
    private static Map<String, String> getLBSWifi(GoogleBaseStationInfo baseStationInfo) {
        try {

            String uri = "https://www.googleapis.com/geolocation/v1/geolocate?key=AIzaSyBhelN5j0OM_AUgtUxVeuW2eeh5HbuRQ9s".trim();
            JSONObject jsonObject = restTemplate.postForObject(uri, baseStationInfo, JSONObject.class);
            log.info("基站信息请求API响应结果为：" + jsonObject);

            if (StringUtils.isNotNull(jsonObject)) {
                JSONObject jsonObject1 = JSONObject.parseObject(jsonObject.toString());
                Integer accuracy = jsonObject1.getInteger("accuracy");
                Object o = jsonObject1.get("location");
                if (StringUtils.isNotNull(o)) {
                    GoogleBaseStationResult baseStationResult = JSONObject.parseObject(o.toString(), GoogleBaseStationResult.class);
                    Map<String, String> gpsGoogle = getGpsGoogle(baseStationResult.getLng(), baseStationResult.getLat());
                    gpsGoogle.put("lng", baseStationResult.getLng().toString());
                    gpsGoogle.put("lat", baseStationResult.getLat().toString());
                    gpsGoogle.put("radius", accuracy.toString());
                    return gpsGoogle;

                }
            }

        } catch (Exception e) {
            e.printStackTrace();
            throw new ServiceException("基站wifi请求谷歌出错");
        }


        return null;

    }


    private static PositionResult getLocationResult(String s) {
        PositionResult forObject = restTemplate.getForObject(s, PositionResult.class);
        if ("10000".equals(forObject.getInfocode()) && "1".equals(forObject.getStatus())) {
            return forObject;
        } else {
            log.info("LBS wifi 请求第三方接返回数据：{}", forObject);
            throw new ServiceException("LBS wifi 请求第三方接口失败，请重试");
        }

    }

    /**
     * 获取天气上传位置信息
     *
     * @param positionInfoap
     * @return
     */
    public static DeviceLocationApp getWeatherLocation(PositionInfo positionInfoap) {

        PositionResult positionResult = null;
        DeviceLocationApp app = null;
        GPSResult gpsResult = null;
        String locationType = "LBS";
        int errCount = 0;
        int gpsErrCount = 0;
        if (StringUtils.isNull(positionInfoap.getGps())) {
            StringBuffer bts_buffer = new StringBuffer();
            StringBuffer nearbts_buffer = new StringBuffer();
            String bts = "";
            String nearbts = "";
            String cdma = positionInfoap.getBaseStationType();
            String accesstype = "0";
            String mmac = "";
            String macs = "";
            String wifiStr = "";
            List<BaseStation> baseStations = positionInfoap.getBaseStation();
            List<WIFI> wifis = positionInfoap.getWifi();
            if (null != baseStations && baseStations.size() > 0) {
                //先请求基站定位(locationType 定位类型 WIFI 还是 LBS)
                if ("0".equals(positionInfoap.getBaseStationType())) {//是否 CDMA(0:否, 1:是)
                    //cdma = positionInfoap.getBaseStationType();
                    // bts_buffer.append("&accesstype=").append(positionInfoap.getBaseStationType());
                    //基站信息
//                //获取第一个基站信息
//                BaseStation baseStation = baseStations.get(0);
//                bts_buffer.append("&mcc=").append(baseStation.getMcc()).append("&mnc=").append(baseStation.getMnc()).append("&lac=").append(baseStation.getLac())
//                        .append("&ci=").append(baseStation.getCi());
                    // bts_buffer.append("&signal=").append(signal);
                    int signal = getSignal(baseStations.get(0).getRxlev());

                    //在遍历这个基站信息
                    for (int i = 0; i < baseStations.size(); i++) {
                        int signal1 = getSignal(baseStations.get(i).getRxlev());
                        if (signal1 < signal) {
                            bts = "";
                            bts = bts_buffer.append(baseStations.get(i).getMcc()).append(",").append(baseStations.get(i).getMnc()).append(",").append(baseStations.get(i).getLac()).append(",")
                                    .append(baseStations.get(i).getCi()).append(",").append(signal1).toString();
                        } else {
                            bts = bts_buffer.append(baseStations.get(0).getMcc()).append(",").append(baseStations.get(0).getMnc()).append(",").append(baseStations.get(0).getLac()).append(",")
                                    .append(baseStations.get(0).getCi()).append(",").append(signal).toString();
                        }

                        if (i != baseStations.size() - 1) {
                            nearbts += nearbts_buffer.append(baseStations.get(i).getMcc()).append(",").append(baseStations.get(i).getMnc()).append(",").append(baseStations.get(i).getLac()).append(",")
                                    .append(baseStations.get(i).getCi()).append(signal1).append("|").toString();

                        } else {
                            nearbts += nearbts_buffer.append(baseStations.get(i).getMcc()).append(baseStations.get(i).getMnc()).append(baseStations.get(i).getLac())
                                    .append(baseStations.get(i).getCi()).append(signal1).toString();

                        }
                    }


                } else { //基站信息

                    String lat = "";
                    String lon = "";
                    String rxlev = "0";
                    cdma = positionInfoap.getBaseStationType();
                    if (baseStations.size() == 1) {
                        bts = bts_buffer.append(baseStations.get(0).getSid()).append(",").append(baseStations.get(0).getNid()).append(",").append(baseStations.get(0).getBid())
                                .append(",").append(lon).append(",").append(lat).append("&").toString();
                    } else {
                        for (int i = 0; i < baseStations.size(); i++) {
                            if (i != baseStations.size() - 1) {
                                bts += bts_buffer.append(baseStations.get(i).getSid()).append(",").append(baseStations.get(i).getNid()).append(",").append(baseStations.get(i).getBid())
                                        .append(",").append(lon).append(lat).append("|").toString();
                            } else {
                                bts += bts += bts_buffer.append(baseStations.get(i).getSid()).append(",").append(baseStations.get(i).getNid()).append(",").append(baseStations.get(i).getBid())
                                        .append(",").append(lon).append(",").append(lat).append("&").toString();
                            }

                        }
                    }


                }
                log.info("电信的LBS 定位 bts 参数 bts,nearbts,locationType,accesstype,{}{}{}{}", bts, nearbts, locationType, accesstype);
            } else {
                if (null != wifis && wifis.size() > 0) {
                    int signal = wifis.get(0).getSignal();
                    accesstype = "1";

                    if (wifis.size() == 1) {
                        mmac = String.format(mmac + "%s,%s,%s", wifis.get(0).getMac(), signal, getOneWifiSSID(wifis.get(0).getSsid()));

                    } else {
                        StringBuffer buffer = new StringBuffer();
                        for (int i = 0; i < wifis.size(); i++) {
                            int signal1 = wifis.get(i).getSignal();
                            if (signal1 > 0) {
                                signal1 = 2 * signal1 - 113;
                                if (signal1 > 0) {
                                    log.info("wifi 定位强度不能为正数{}", signal1);
                                }
                            }
                            if (signal1 < signal) {
                                signal = Math.abs(signal1);
                                // mmac = String.format(mmac + "%s,%s,%s", wifis.get(i).getMac(), signal1, getOneWifiSSID(wifis.get(i).getSsid()));
                                buffer.setLength(0);
                                mmac = buffer.append(wifis.get(i).getMac()).append(",").append(signal1).append(",").append(getOneWifiSSID(wifis.get(i).getSsid())).delete(buffer.length() - 1, buffer.length()).toString();

                            }
                            if (i != wifis.size() - 1) {
                                wifiStr += String.format(wifiStr + "%s,%s,%s%s", wifis.get(i).getMac(), signal1, getOneWifiSSID(wifis.get(i).getSsid()), "|");
                            } else {
                                wifiStr += String.format(wifiStr + "%s,%s,%s", wifis.get(i).getMac(), signal1, getOneWifiSSID(wifis.get(i).getSsid()));
                            }
                        }
                        macs = wifiStr;
                        locationType = "WIFI";

                    }
                }
            }


            StringBuffer buffer = new StringBuffer();
            buffer.append("accesstype=").append(accesstype).append("&imei=").append(positionInfoap.getImei());
            //查询缓存 imsi
            Object imsi = redisService.getCacheMapValue(positionInfoap.getImei(), "imsi");
            if (StringUtils.isNotNull(imsi)) {
                buffer.append("&imsi=").append(imsi);
            }
            String s = buffer.append("&network=").append("GMS").append("&cdma=").append(cdma).append("&bts=").append(bts).append("&nearbts=").append(nearbts)
                    .append("&mmac=").append(mmac).append("&macs=").append(macs).append("&output=json").append("&key=").append(gaudyKey).toString();
            log.info("locationType:{},参数:{}", locationType, s);
            String s1 = String.format(url + "%s%s", "?", s);
            try {
                positionResult = LocationUtil.getLocationResult(s1);
                log.info("LBS WIFI请求第三接口结果集数据:{}", positionResult);
//                if ("10000".equals(positionResult.getInfocode()) && null ==positionResult.getResult() && !StringUtils.isNotEmpty(positionResult.getResult().getLocation())){
//                    count++;
//                    if (count<6){ //重试6次
//                        positionResult= LocationUtil.getLocationResult(s1);
//                    }
//                }
            } catch (Exception e) {
                errCount++;
                if (errCount < 6) { //重试6次
                    try {
                        Thread.sleep(1000);
                    } catch (InterruptedException interruptedException) {
                        log.info("LBS线程休眠失败:{}", interruptedException.getMessage());
                    }
                    positionResult = LocationUtil.getLocationResult(s1);
                }
                log.info("请求第三接口失败:{},重试次数{}", e.getMessage(), errCount);
                log.info("LBS WIFI error 请求第三方接口数据:{}", positionResult);
            }
            log.info("LBS WIFI 请求第三方接口数据:{}", positionResult);
        } else { //GPS定位
            GPSInfo gps = positionInfoap.getGps();
            locationType = "GPS";
            try {
                gpsResult = getGps(gps.getLon(), gps.getLat());


            } catch (Exception e) {
                gpsErrCount++;
                try {
                    Thread.sleep(1000);
                } catch (InterruptedException interruptedException) {
                    log.info("GPS线程休眠失败:{}", interruptedException.getMessage());
                }
                if (gpsErrCount < 6) {
                    gpsResult = LocationUtil.getGps(gps.getLon(), gps.getLat());
                }
                log.info("请求第三接口失败:{},重试次数{}", e.getMessage(), errCount);
                log.info("GPS error 请求第三方接口数据:{}", gpsResult);
            }
            log.info("GPS 请求第三方接口结果集数据:{}", gpsResult);

        }
        //添加数据库处理
        if (StringUtils.isNotNull(positionResult) && StringUtils.isNotNull(positionResult.getResult())) {
            app = addLBSData(positionInfoap, positionResult, locationType);
        }

        if (StringUtils.isNotNull(gpsResult) && StringUtils.isNotNull(gpsResult.getRegeoCode().getAddressComponent()) && StringUtils.isNotNull(gpsResult.getRegeoCode().getAddressComponent().getStreetNumber())) {
            app = addGPSData(positionInfoap, gpsResult, locationType);

        }

        log.info("转换后得到的DeviceLocationApp{}", app);
        return app;


    }


    /**
     * 请求天气接口
     *
     * @Title: LocationUtil
     * @Author: liy
     * @Date: 2022/11/30 17:41
     * @Description:
     * @Version:1.0
     */
    public static WeatherResult getWeather(String adcode) {
        String key = "2d9daac715814e2f8edee152cdb2f95d";
        //String key="0c06c89436ba70598d07a26520fad43e";
        String url = "https://restapi.amap.com/v3/weather/weatherInfo?city=";
        String uri = String.format("%s%s&key=%s", url, adcode, key);
        log.info("请求天气解析的URL:{}", uri);
        WeatherResult result = null;
        try {
            result = restTemplate.getForObject(uri, WeatherResult.class);
            log.info("请求天气返回结果:{}=================", result);
            if (StringUtils.isNotNull(result) && CommonConstant.AUDIT_STATUS_ONE.equals(result.getStatus())) {
                return result;
            } else {
                log.info("请求天气返回结果失败:{}", result);
                return result;
            }

        } catch (Exception e) {
            log.info("解析天气异常:{}", e.getMessage());
        }
        return result;

    }

    /**
     * 获取缓存厂商标识
     *
     * @param
     */
    public static String getVender(String imei, String key) {
        String vender = "20000";
        Object cacheMapValue = redisService.getCacheMapValue(imei, key);
        if (StringUtils.isNotNull(cacheMapValue)) {
            return cacheMapValue.toString();
        }
        return vender;
    }

    /**
     * 定位下发
     *
     * @param imei
     * @param userId
     */
    public static void sendLocation(String imei, String userId) {
        DistributeParent distributeParent = new DistributeParent();
        distributeParent.setImei(imei);
        distributeParent.setOpenid(userId); //用户ID
        distributeParent.setType("requestLocation"); //请求位置
        distributeParent.setIdent(RandomUtil.generateVerCode(6));
        distributeParent.setVender(getVender(imei, "agent"));
        distributeParent.setTime(System.currentTimeMillis());
        iss.emitDeviceDemand("requestLocation", imei, distributeParent);
        //下发
        // iss.sendKafka(distributeParent);

    }


//    public static void main(String[] args) {
////        PositionInfo positionInfo = JSONObject.parseObject(" {\n" +
////                "            \"wifi\": [\n" +
////                "            {\n" +
////                "                \"ssid\": \"\",\n" +
////                "                    \"signal\": 58,\n" +
////                "                    \"mac\": \"f4:2a:7d:77:4d:b2\"\n" +
////                "            },\n" +
////                "            {\n" +
////                "                \"ssid\": \"\",\n" +
////                "                    \"signal\": 58,\n" +
////                "                    \"mac\": \"54:a7:03:dc:c4:95\"\n" +
////                "            },\n" +
////                "            {\n" +
////                "                \"ssid\": \"\",\n" +
////                "                    \"signal\": 56,\n" +
////                "                    \"mac\": \"90:5d:7c:a7:d2:70\"\n" +
////                "            },\n" +
////                "            {\n" +
////                "                \"ssid\": \"\",\n" +
////                "                    \"signal\": 56,\n" +
////                "                    \"mac\": \"78:aa:82:3a:5e:30\"\n" +
////                "            },\n" +
////                "            {\n" +
////                "                \"ssid\": \"\",\n" +
////                "                    \"signal\": 54,\n" +
////                "                    \"mac\": \"8c:de:f9:e0:37:fe\"\n" +
////                "            }\n" +
////                "  ],\n" +
////                "            \"ident\": 146404,\n" +
////                "                \"locationType\": \"ordinary\",\n" +
////                "                \"vender\": \"10000\",\n" +
////                "                \"type\": \"location\",\n" +
////                "                \"battery\": \"85\",\n" +
////                "                \"baseStationType\": \"0\",\n" +
////                "                \"rollNumber\": \"0\",\n" +
////                "                \"imei\": \"867188040000098\",\n" +
////                "                \"baseStation\": [\n" +
////                "            {\n" +
////                "                \"mnc\": 11,\n" +
////                "                    \"ci\": 125090567,\n" +
////                "                    \"rxlev\": -17,\n" +
////                "                    \"mcc\": 460,\n" +
////                "                    \"lac\": 30497\n" +
////                "            }\n" +
////                "  ],\n" +
////                "            \"time\": 1662425435,\n" +
////                "                \"stepNumber\": \"0\",\n" +
////                "                \"location_time\": 1662454233\n" +
////                "        }\n" +
////                "        \n" +
////                "    }", PositionInfo.class);
//        //String location = getLocation(positionInfo);
//        //System.out.println(location);
//
//    }


    /**
     * 建议使用这种经度
     * 根据经纬度，计算两点间的距离
     *
     * @param longitude1 第一个点的经度
     * @param latitude1  第一个点的纬度
     * @param longitude2 第二个点的经度
     * @param latitude2  第二个点的纬度
     * @return 返回距离 单位米
     */
    public static double getDistance(double longitude1, double latitude1, double longitude2, double latitude2) {
        if (longitude1 == longitude2 && latitude1 == latitude2) {
            return 0;
        }

        // 纬度
        double lat1 = Math.toRadians(latitude1);
        double lat2 = Math.toRadians(latitude2);
        // 经度
        double lng1 = Math.toRadians(longitude1);
        double lng2 = Math.toRadians(longitude2);
        // 纬度之差
        double a = lat1 - lat2;
        // 经度之差
        double b = lng1 - lng2;
        // 计算两点距离的公式
        double s = 2 * Math.asin(Math.sqrt(Math.pow(Math.sin(a / 2), 2) +
                Math.cos(lat1) * Math.cos(lat2) * Math.pow(Math.sin(b / 2), 2)));
        // 弧长乘地球半径, 返回单位: 米
        s = s * EARTH_RADIUS;

        return Math.round(s * 10000) / 10000;
    }

    /**
     * 根据经纬度计算距离
     *
     * @param lastLon 设置的经度
     * @param lastLat 设置的维度
     * @param newLon  最新的经度
     * @param newLat  最新的纬度
     *                单位米
     */
    public static double getDistance2(double lastLon, double lastLat, double newLon, double newLat) {
        double x = Math.PI / 180.0;
        if (lastLon == newLon && lastLat == newLat) {
            return 0.00;
        }
        lastLon = lastLon * x;
        lastLat = lastLat * x;
        newLon = newLon * x;
        newLat = newLat * x;
        double con = Math.sin(lastLat) * Math.sin(newLat);
        con += Math.cos(lastLat) * Math.cos(newLat) * Math.cos(lastLon - newLon);
        return Math.round(Math.acos(con) * EARTH_RADIUS * 10000) / 10000;
    }


    /**
     * 获取当天0点 2022-10-18 00:00:00
     *
     * @return
     */
    public static Date getZero(Date date) {

        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        return calendar.getTime();

    }

    /**
     * 获取当天最后的日期 2022-10-18 23:59:59
     *
     * @return
     */
    public static Date getEnd(Date date) {

        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.set(Calendar.HOUR_OF_DAY, 23);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        return calendar.getTime();

    }

//    /**
//     * 根据经纬度，计算两点间的距离
//     * @param longitude1 第一个点的经度
//     * @param latitude1  第一个点的纬度
//     * @param longitude2 第二个点的经度
//     * @param latitude2  第二个点的纬度
//     * @return 返回距离 单位米
//     */
//    public static double getDistance2(double longitude1, double latitude1,
//                                      double longitude2, double latitude2) {
//
//        double latDistance = Math.toRadians(longitude1 - longitude2);
//        double lngDistance = Math.toRadians(latitude1 - latitude2);
//
//        double a = Math.sin(latDistance / 2) * Math.sin(latDistance / 2)
//                + Math.cos(Math.toRadians(longitude1)) * Math.cos(Math.toRadians(longitude2))
//                * Math.sin(lngDistance / 2) * Math.sin(lngDistance / 2);
//
//        double c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1 - a));
//        return c * EARTH_RADIUS;
//    }


//    public static void main(String[] args) {
//        double distance = getDistance(112.8807037, 28.231719, 112.8807658, 28.2236627);
//        System.out.println(distance);
//        double distance2 = getDistance2(112.8807037, 28.231719, 112.8807658, 28.2236627);
//        System.out.println(distance2);
//        double distance3 = getDistance3(112.8807037, 28.231719, 112.8807658, 28.2236627);
//        System.out.println(distance3);
//    }


}
