package com.ecell.internationalize.auth.controller;

import com.ecell.internationalize.auth.entity.UserType;
import com.ecell.internationalize.auth.feign.SysUserFeignClient;
import com.ecell.internationalize.auth.service.SysUserLoginService;
import com.ecell.internationalize.auth.utils.AuthUtil;
import com.ecell.internationalize.common.core.domain.UserLogin;
import com.ecell.internationalize.common.core.utils.StringUtils;
import com.ecell.internationalize.common.core.utils.locale.LocaleUtil;
import com.ecell.internationalize.common.core.web.domain.AjaxResult;
import com.ecell.internationalize.common.core.web.domain.R;
import com.ecell.internationalize.common.security.service.TokenService;
import com.ecell.internationalize.common.security.utils.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;

/**
 * @author borui
 */
@RestController
@RequestMapping("/auth")
public class AuthController {
    private final Integer FAIL_CODE = 500;
    @Autowired
    private TokenService tokenServiceImpl;

    @Autowired
    private SysUserLoginService sysUserLoginService;
    @Autowired
    private SysUserFeignClient sysUserFeignClient;

    @PostMapping("login")
    public AjaxResult login(@RequestBody UserType userType) {
        // 用户登录校验
        R<UserLogin> userLogin = sysUserLoginService.login(userType.getUsername(), userType.getPassword());
        if (FAIL_CODE == userLogin.getCode()) {
            return AjaxResult.error(userLogin.getMsg());
        }
        // 获取登录token
        return AjaxResult.success(LocaleUtil.getMessage("messages.success"), tokenServiceImpl.createToken(userLogin.getData()));
    }

    @DeleteMapping("logout")
    public AjaxResult logout(HttpServletRequest request) {
        String token = SecurityUtils.getToken(request);
        if (StringUtils.isNotEmpty(token)) {
            // 删除用户缓存记录
            AuthUtil.logoutByToken(token);
        }
        return AjaxResult.success(LocaleUtil.getMessage("messages.success"));
    }

    @PostMapping("refresh")
    public AjaxResult refresh(HttpServletRequest request) {
        UserLogin userLogin = tokenServiceImpl.getUserLogin(request);
        if (StringUtils.isNotNull(userLogin)) {
            // 刷新令牌有效期
            tokenServiceImpl.refreshToken(userLogin);
            return AjaxResult.success(LocaleUtil.getMessage("messages.success"));
        }
        return AjaxResult.success(LocaleUtil.getMessage("messages.success"));
    }

    @GetMapping("test")
    public UserLogin test(String userName) {
        return sysUserFeignClient.queryByUser(userName);
    }

}
