package com.ecell.internationalize.auth.service.impl;

import com.ecell.internationalize.auth.feign.SysUserFeignClient;
import com.ecell.internationalize.auth.service.SysUserLoginService;
import com.ecell.internationalize.common.core.domain.UserLogin;
import com.ecell.internationalize.common.core.enums.UserStatus;
import com.ecell.internationalize.common.core.utils.ServletUtils;
import com.ecell.internationalize.common.core.utils.StringUtils;
import com.ecell.internationalize.common.core.utils.ip.IpUtils;
import com.ecell.internationalize.common.core.utils.locale.LocaleUtil;
import com.ecell.internationalize.common.core.web.domain.R;
import com.ecell.internationalize.common.security.utils.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * @author borui
 */
@Service
public class SysUserLoginServiceImpl implements SysUserLoginService {
    private final String DELETED="0";
    @Autowired
    private SysUserFeignClient sysUserFeignClient;

    @Override
    public R<UserLogin> login(String username, String password) {

        // 用户名或密码为空
        if (StringUtils.isAnyBlank(username, password)) {
            return R.fail(LocaleUtil.getMessage("messages.login.empty"));
        }
        UserLogin userInfo = sysUserFeignClient.queryByUser(username);
        //查询错误
        if (StringUtils.isNull(userInfo)){
            return R.fail(LocaleUtil.getMessage("messages.login.error"));
        }
        if (!SecurityUtils.matchesPassword(password, userInfo.getPassword())){
            return R.fail(LocaleUtil.getMessage("messages.error.password"));
        }
        //账号被删除
        if (DELETED.equals(userInfo.getDelFlag())) {
            return R.fail(LocaleUtil.getMessage("messages.account.delete"));
        }
        //账号被停用
        if (UserStatus.DISABLE.getCode().equals(userInfo.getStatus())) {
            return R.fail(LocaleUtil.getMessage("messages.account.stop"));
        }
        System.out.println("userInfo:"+userInfo.getStatus());
        System.out.println("校验通过");
        //修改登录时间和IP
        UserLogin userLogin=new UserLogin();
        userLogin.setUserId(userInfo.getUserId());
        userLogin.setLoginIp(IpUtils.getIpAddr(ServletUtils.getRequest()));
        SimpleDateFormat sdf =new SimpleDateFormat("yyyy-MM-dd HH:mm:ss" );
        Date date= new Date();
        String str = sdf.format(date);
        userLogin.setLoginDate(str);
        sysUserFeignClient.update(userLogin);
        return R.ok(userInfo);
    }
}
