package com.ecell.internationalize.common.system.utlis;

import com.aliyun.oss.ClientException;
import com.aliyun.oss.OSS;
import com.aliyun.oss.OSSClientBuilder;
import com.aliyun.oss.OSSException;
import com.aliyun.oss.model.PutObjectRequest;
import com.ecell.internationalize.common.core.exception.ServiceException;
import com.ecell.internationalize.common.core.utils.uuid.UUID;
import com.ecell.internationalize.common.system.constant.OssClientConstants;
import com.ecell.internationalize.common.system.entity.ChatOssDto;
import com.ecell.internationalize.common.system.entity.CheckVersionUpload;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.web.multipart.MultipartFile;

import java.io.*;
import java.net.URL;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.Date;

/**
 * @author borui
 */
public class UploadUtil {
    public static String upload(MultipartFile file) throws IOException {
        // 创建OSSClient实例
        OSS ossClient = new OSSClientBuilder().build(OssClientConstants.ENDPOINT, OssClientConstants.ACCESS_KEY_ID, OssClientConstants.ACCESS_KEY_SECRET);
        // 上传文件流。
        InputStream inputStream = null;
        // 获取文件后缀名
        String suffix =getSuffix(file);
        //唯一key
        String uuid= UUID.fastUUID().toString();
        File tempFile = File.createTempFile(uuid, suffix);
        //MultipartFile  转换为 File
        file.transferTo(tempFile);
        try {
            //文件地址
            inputStream = new FileInputStream(tempFile);
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
        //上传到OSS,第一个参数为存储空间名,第二个为对象名，唯一值
        ossClient.putObject(OssClientConstants.BUCKET_NAME, uuid+"."+suffix, inputStream);
        // 设置URL过期时间为100年，默认这里是int型，转换为long型即可
        Date expiration = new Date(System.currentTimeMillis() + 3600L * 1000 * 24 * 365 * 100);
        // 生成URL
        URL url = ossClient.generatePresignedUrl(OssClientConstants.BUCKET_NAME, uuid+"."+suffix, expiration);
        System.out.println("url:"+url);
        return url.toString();
        // 关闭OSSClient。
        // ossClient.shutdown();
    }

    public static ChatOssDto uploadChat(MultipartFile file) throws IOException {
        ChatOssDto chatOssDto=new ChatOssDto();
        // 创建OSSClient实例
        OSS ossClient = new OSSClientBuilder().build(OssClientConstants.ENDPOINT, OssClientConstants.ACCESS_KEY_ID, OssClientConstants.ACCESS_KEY_SECRET);
        // 上传文件流。
        InputStream inputStream = null;
        // 获取文件后缀名
//        String suffix =getSuffix(file);
        String suffix ="amr";
        System.out.println("语音文件名后缀:"+suffix);
        String fileName=file.getOriginalFilename()+"."+suffix;
        System.out.println("fileName:"+fileName);
        //唯一key
        String uuid= UUID.fastUUID().toString();
        File tempFile = File.createTempFile(uuid, suffix);
        //MultipartFile  转换为 File
        file.transferTo(tempFile);
        try {
            //文件地址
            inputStream = new FileInputStream(tempFile);
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
        //String fileName="static/message/"+uuid+"."+suffix;
        //上传到OSS,第一个参数为存储空间名,第二个为对象名，唯一值
        ossClient.putObject(OssClientConstants.BUCKET_NAME, fileName, inputStream);
        // 设置URL过期时间为100年，默认这里是int型，转换为long型即可
        Date expiration = new Date(System.currentTimeMillis() + 3600L * 1000 * 24 * 365 * 100);
        // 生成URL
        URL url = ossClient.generatePresignedUrl(OssClientConstants.BUCKET_NAME, fileName, expiration);
        chatOssDto.setUrl(url.toString());
        chatOssDto.setFileName(fileName);
        System.out.println("chatOssDto:"+chatOssDto);
        return chatOssDto;
        // 关闭OSSClient。
        // ossClient.shutdown();
    }

    public static CheckVersionUpload versionUpload(MultipartFile file) throws IOException {
        System.out.println("开始文件上传");
        CheckVersionUpload checkVersionUpload=new CheckVersionUpload();
        String fileName=file.getOriginalFilename();
        // 创建OSSClient实例
        OSS ossClient = new OSSClientBuilder().build(OssClientConstants.ENDPOINT, OssClientConstants.ACCESS_KEY_ID, OssClientConstants.ACCESS_KEY_SECRET);
        // 上传文件流。
        InputStream inputStream = null;
        // 获取文件后缀名
        String suffix =getSuffix(file);
        //唯一key
        String uuid= UUID.fastUUID().toString();
        File tempFile = File.createTempFile(uuid, suffix);
        //MultipartFile  转换为 File
        file.transferTo(tempFile);
        try {
            //文件地址
            inputStream = new FileInputStream(tempFile);
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
        //String fileName="static/message/"+uuid+"."+suffix;
        //上传到OSS,第一个参数为存储空间名,第二个为对象名，唯一值
        ossClient.putObject(OssClientConstants.BUCKET_NAME, fileName, inputStream);
        // 设置URL过期时间为100年，默认这里是int型，转换为long型即可
        Date expiration = new Date(System.currentTimeMillis() + 3600L * 1000 * 24 * 365 * 100);
        // 生成URL
        URL url = ossClient.generatePresignedUrl(OssClientConstants.BUCKET_NAME, fileName, expiration);
        checkVersionUpload.setUrl(url.toString());
        checkVersionUpload.setFileName(fileName);
        checkVersionUpload.setGetSuffix(suffix);
        return checkVersionUpload;
        // 关闭OSSClient。
        // ossClient.shutdown();
    }

    public static String uploadTwo(MultipartFile file,String inputBatch) throws IOException {
        // 创建OSSClient实例
        OSS ossClient = new OSSClientBuilder().build(OssClientConstants.ENDPOINT, OssClientConstants.ACCESS_KEY_ID, OssClientConstants.ACCESS_KEY_SECRET);
        // 上传文件流。
        InputStream inputStream = null;
        // 获取文件后缀名
        String suffix =getSuffix(file);
        //唯一key
        //String uuid= UUID.fastUUID().toString();
        File tempFile = File.createTempFile(inputBatch, suffix);
        //MultipartFile  转换为 File
        file.transferTo(tempFile);
        URL url =null;
        try {
            //文件地址
            inputStream = new FileInputStream(tempFile);
            String fileName="device/"+inputBatch+"."+suffix;
            PutObjectRequest putObjectRequest = new PutObjectRequest(OssClientConstants.BUCKET_NAME, fileName, inputStream);
            //上传到OSS,第一个参数为存储空间名,第二个为对象名，唯一值
            ossClient.putObject(putObjectRequest);
            // 设置URL过期时间为100年，默认这里是int型，转换为long型即可
            Date expiration = new Date(System.currentTimeMillis() + 3600L * 1000 * 24 * 365 * 100);
            // 生成URL
            url= ossClient.generatePresignedUrl(OssClientConstants.BUCKET_NAME, fileName, expiration);
            System.out.println("url:"+url);
        } catch (FileNotFoundException e) {
            throw  new ServiceException(e.getMessage());
        }finally {
            ossClient.shutdown();
        }

        return url.toString();
        // 关闭OSSClient。
        // ossClient.shutdown();
    }

    /**
     * 上传excel
     *
     * @param workbook
     * @param excelName 文件名
     * @return
     */
    public String uploadThree(Workbook workbook, String excelName) {
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        try {
            workbook.write(bos);
        } catch (IOException e) {
        } finally {
            try {
                bos.close();
            } catch (IOException e) {
            }
        }
        byte[] brray = bos.toByteArray();
        InputStream is = new ByteArrayInputStream(brray);
        OSS ossClient = new OSSClientBuilder().build(OssClientConstants.ENDPOINT, OssClientConstants.ACCESS_KEY_ID, OssClientConstants.ACCESS_KEY_SECRET);
        try {
            // 创建PutObjectRequest对象。
            PutObjectRequest putObjectRequest = new PutObjectRequest(OssClientConstants.BUCKET_NAME,  excelName, is);
            // 如果需要上传时设置存储类型和访问权限，请参考以下示例代码。
            // ObjectMetadata metadata = new ObjectMetadata();
            // metadata.setHeader(OSSHeaders.OSS_STORAGE_CLASS, StorageClass.Standard.toString());
            // metadata.setObjectAcl(CannedAccessControlList.Private);
            // putObjectRequest.setMetadata(metadata);
            // 上传文件
            ossClient.putObject(putObjectRequest);
            //设置过期时间
            LocalDateTime time = LocalDateTime.now().plusDays(36500);
            Date expiration = Date.from(time.atZone(ZoneId.systemDefault()).toInstant());
            return ossClient.generatePresignedUrl(OssClientConstants.BUCKET_NAME, "device" + "/" + excelName, expiration).toString();
        } catch (OSSException oe) {
            throw  new ServiceException(oe.getMessage());
        } catch (ClientException ce) {
            throw  new ServiceException(ce.getMessage());
        } finally {
            if (ossClient != null) {
                ossClient.shutdown();
            }
        }
    }



    /**
     * 获取文件名后缀
     * @Title: UploadFileUtil
     * @Author: liy
     * @Date: 2022/7/13 17:35
     * @Description:
     * @Version:1.0
     */
    public static String getSuffix(MultipartFile file){
        String originalFilename = file.getOriginalFilename();
        assert originalFilename != null;
        return originalFilename.substring(originalFilename.lastIndexOf(".") + 1);
    }


    /**
     * 获取文件名称
     * @param file
     * @param inputBatch
     * @return
     */
    public static String getMultipartFileName(MultipartFile file,String inputBatch){

        String originalFilename = file.getOriginalFilename();
        assert originalFilename != null;
        String s= originalFilename.substring(originalFilename.lastIndexOf(".") + 1);
        String fileName="device"+inputBatch+"."+s;
        return fileName;
    }

}
