package com.ecell.internationalize.common.issue.mqtt;

import com.ecell.internationalize.common.core.utils.StringUtils;
import com.ecell.internationalize.common.issue.callback.CustomMessageCallback;
import com.ecell.internationalize.common.issue.util.RandomUtil;
import org.eclipse.paho.client.mqttv3.*;
import org.eclipse.paho.client.mqttv3.persist.MemoryPersistence;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @author borui
 */
public class AppSingleMqttClient {
    private static final Logger logger = LoggerFactory.getLogger(AppSingleMqttClient.class);
    private static MqttClient mqttClient=null;
    //防止指令重排
    private volatile static AppSingleMqttClient appSingleMqttClient;

    //初始化链接
    private AppSingleMqttClient()  {
        MqttConnectOptions mqttConnectOptions = getMqttConnectOptions();
        //设置持久化方式
        MemoryPersistence memoryPersistence = new MemoryPersistence();
        //集群就随机
        String clientId ="ecell-internationalize"+ RandomUtil.generateVerCode(6)+System.currentTimeMillis();
        //单实例写死
        //String clientId ="yisai@@@777";
        if (null != clientId) {
            try {
                //tcp://120.77.209.176:1883 //1.13.186.145
                mqttClient = new MqttClient("tcp://8.217.217.70:1883", clientId, memoryPersistence);
            } catch (MqttException e) {
                logger.info("创建链接失败:{}",e.getMessage());
                e.printStackTrace();
            }
        }
        //设置连接和回调
        if (null != mqttClient) {
            if (!mqttClient.isConnected()) {
                //创建回调函数对象
                //  MQTTReceiveCallback MQTTReceiveCallback = new MQTTReceiveCallback();
                //客户端添加回调函数(默认打印)
                mqttClient.setCallback(new CustomMessageCallback());
                //创建连接
                try {
                    logger.info("开始建立链接");
                    mqttClient.connect(mqttConnectOptions);
                } catch (MqttException e) {
                    // destroy();
//                    try {
//                        Thread.sleep(100);
//                        reconnection();
//                    } catch (InterruptedException interruptedException) {
//                        interruptedException.printStackTrace();
//                    }
//                    logger.info("创建链接失败:",e.getMessage());
//                    e.printStackTrace();
                }

            }
        }

    }

    /**
     * 设置参数
     */
    private static MqttConnectOptions getMqttConnectOptions(){
        //初始化连接设置对象
        MqttConnectOptions  mqttConnectOptions = new MqttConnectOptions();
        //设置是否清空session,这里如果设置为false表示服务器会保留客户端的连接记录，
        //这里设置为true表示每次连接到服务器都以新的身份连接
        mqttConnectOptions.setCleanSession(true);
        //设置连接超时时间，单位是秒
        mqttConnectOptions.setConnectionTimeout(10);
        mqttConnectOptions.setUserName("admin");
        mqttConnectOptions.setPassword("public".toCharArray());
        //保留回话
        // mqttConnectOptions.setCleanSession(false);
        // 设置会话心跳时间 单位为秒 服务器会每隔1.5*20秒的时间向客户端发送个消息判断客户端是否在线，但这个方法并没有重连的机制
        // mqttConnectOptions.setKeepAliveInterval(60);
        //自动重连
        // mqttConnectOptions.setAutomaticReconnect(true);
        //建立链接
        //setWill方法，如果项目中需要知道客户端是否掉线可以调用该方法。设置最终端口的通知消息 //遗嘱
        // mqttConnectOptions.setWill(mqttConfig.getSubTopic(), "客户端已经下线".getBytes(), 2, true);
        return  mqttConnectOptions;
    }

    public static AppSingleMqttClient getAppSingleMqttClient(){
        logger.info("开始建立链接:{}",appSingleMqttClient);
        if (null==appSingleMqttClient){
            synchronized (AppSingleMqttClient.class){
                if (null==appSingleMqttClient){
                    appSingleMqttClient= new AppSingleMqttClient();
                }
            }
        }
        return appSingleMqttClient;
    }

    public static void destroy(){
        logger.info("释放链接start:{}",mqttClient);
        if (null!=mqttClient){
            try {
                mqttClient.disconnect();
                mqttClient.close();
            }catch (MqttException e){
                logger.info("释放资源失败:{}",e.getMessage());
            }
        }
    }

    public static void reconnection(){
        logger.info("重新链接方法被调用：{},{}",appSingleMqttClient,mqttClient,mqttClient.isConnected());
        if (null==appSingleMqttClient){
            logger.info("重新链接");
            AppSingleMqttClient.getAppSingleMqttClient();
        }else {
            if (null !=mqttClient){
                try {
                    MqttConnectOptions mqttConnectOptions = getMqttConnectOptions();
                    logger.info("进行重新链接：{}",mqttConnectOptions);
                    mqttClient.connect(mqttConnectOptions);
                } catch (MqttException e) {
                    e.printStackTrace();
                }
            }
        }
    }
    /**自定义发布消息*/
    public static void publishCustomMessage(int qos,String message,String pubTopic) {
        logger.info("开始发送的信息是:{}",message);
        if(StringUtils.isNotNull(mqttClient) && mqttClient.isConnected()) {
            logger.info("发布自定义信息客户端状态:{},id:{},topic:{}",mqttClient.isConnected(),mqttClient.getClientId(),pubTopic);
            MqttMessage mqttMessage = new MqttMessage();
            mqttMessage.setQos(qos);
            mqttMessage.setPayload(message.getBytes());
            MqttTopic topic = mqttClient.getTopic(pubTopic);
            if(null != topic) {
                try {
                    logger.info("发送的信息是:{}",message);
                    MqttDeliveryToken publish = topic.publish(mqttMessage);
                    if(!publish.isComplete()) {
                        System.out.println("消息发布成功");
                    }
                } catch (MqttException exception) {
                    logger.info("发送的信息失败:{}",exception.getMessage());
                }

            }


        }else {
            //  AppSingleMqttClient.getAppSingleMqttClient();
            //这里可以做重新链接
            logger.info("客户端为null{}",mqttClient);
            reconnection();
        }
    }
}
