package com.ecell.internationalize.common.issue.mqtt.push;

import com.alibaba.fastjson2.JSON;
import com.ecell.internationalize.common.core.exception.ServiceException;
import com.ecell.internationalize.common.core.utils.StringUtils;
import com.ecell.internationalize.common.issue.constant.CommonConstant;
import com.ecell.internationalize.common.issue.mqtt.dto.IOSMessage;
import com.ecell.internationalize.common.issue.mqtt.dto.IOSSystemMessage;
import com.ecell.internationalize.common.issue.mqtt.dto.SendIOSParamete;
import com.ecell.internationalize.common.redis.service.RedisService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.cloud.context.config.annotation.RefreshScope;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.stereotype.Component;

/**
 * @author borui
 */

@Component
@RefreshScope
public class IOSPush {
    private static final Logger log = LoggerFactory.getLogger(IOSPush.class);
    @Autowired
    private RedisService redisService;

    @Autowired
    private KafkaTemplate<String,Object> kafkaTemplate;
    @Value("${ios.evn:dis}")
    private String iosVersion;

    public void sendIOS(SendIOSParamete parameter){
        log.info("推送苹果系统消息:{}",parameter);

        IOSSystemMessage iosSystemParam = getIOSSystemParam(parameter.getOpenid());
        if ("LoginOffLine".equals(parameter.getChildType()) ||"identityTransfered".equals(parameter.getChildType()) || "identityTransfer".equals(parameter.getChildType())){
            String iosToken = parameter.getIOSToken();
            if (StringUtils.isNotNull(iosToken)){
                iosSystemParam.setToken(iosToken);
            }
            log.info("修改的token:{}",iosSystemParam.getToken());
        }

        /**
         * token有可能为空，为空就不下发
         */
        if (StringUtils.isNotEmpty(iosSystemParam.getToken())){
            log.info("获取苹果系统参数:{}",iosSystemParam);
            IOSMessage message = getIOSSystemMessage(parameter);
            log.info("获取到的推送内容是:{}",message);

            if (StringUtils.isNotNull(message)){
                log.info("IOS开始推送=====获取到的推送内容是:{},获取到IOS系统参数是:{}",message,iosSystemParam);
                pushKafkaIOS(iosSystemParam,message);
            }

        }

    }


    private IOSMessage  getIOSSystemMessage(SendIOSParamete parameter){
        //IOSMessage iosMessage =new IOSMessage();
        IOSMessage iosMessage=null;
        log.info("获取到的推送类型是=========={}",parameter.getType());
        if ("accountMessage".equals(parameter.getType())){
            log.info("获取到的推送子=====类型是=========={}",parameter.getChildType());
            if ("LoginOffLine".equals(parameter.getChildType())){
                iosMessage =parameter.getApns_param();

            }else if ("systemUpdate".equals(parameter.getChildType())){
                iosMessage= parameter.getApns_param();
            }else if ("Recall".equals(parameter.getType())){
                iosMessage =parameter.getApns_param();

            }else if ("rail".equals(parameter.getChildType())){
                iosMessage= parameter.getApns_param();
            }else if ("sos".equals(parameter.getChildType())){
                iosMessage= parameter.getApns_param();
            }else if ("lowBattery".equals(parameter.getChildType())){
                iosMessage= parameter.getApns_param();
            }else if ("applicationRrecord".equals(parameter.getChildType())){
                iosMessage= parameter.getApns_param();
            }else if ("applicationOvertime".equals(parameter.getChildType())){
                iosMessage= parameter.getApns_param();
            }else if ("identityTransfer".equals(parameter.getChildType())){
                iosMessage= parameter.getApns_param();
            }else if ("identityTransfered".equals(parameter.getChildType())){
                iosMessage= parameter.getApns_param();
            }else if ("unbind".equals(parameter.getChildType())){
                iosMessage= parameter.getApns_param();
            }else if ("reject".equals(parameter.getChildType())){
                iosMessage= parameter.getApns_param();
            }else if ("video".equals(parameter.getChildType())){
                iosMessage= parameter.getApns_param();
            }else if ("abnormalBodyTemperature".equals(parameter.getChildType()) || "abnormalHeartRate".equals(parameter.getChildType())){
                iosMessage= parameter.getApns_param();
            }else if ("upHeartRate".equals(parameter.getChildType())){
                iosMessage= parameter.getApns_param();
            }else if ("upBodyTemperature".equals(parameter.getChildType())){
                iosMessage= parameter.getApns_param();
            }

        }else if ("chatGroupMessage".equals(parameter.getType())){
            iosMessage =parameter.getApns_param();
            iosMessage.setBadge(1);
            iosMessage.setContent_available(0);
        }else if ("chatGroupRemove".equals(parameter.getType())){
            iosMessage =parameter.getApns_param();
        }else if("chatGroupDisband".equals(parameter.getType())) {
            iosMessage = parameter.getApns_param();
        }
//        }else {
//            IOSMessage iosMessageTwo=new IOSMessage();
//            iosMessageTwo.setContent_available(CommonConstant.NUMBER_ONE);
//            iosMessageTwo.setSound(CommonConstant.SOUND);
//            iosMessageTwo.setData(parameter.getApns_param().getData());
//            iosMessage=iosMessageTwo;
//        }
        if (StringUtils.isNull(iosMessage)){
            iosMessage=new IOSMessage();
            iosMessage.setContent_available(CommonConstant.NUMBER_ONE);
            iosMessage.setSound(CommonConstant.SOUND);
            iosMessage.setData(parameter.getApns_param().getData());

        }
        log.info("获取到的IOSMessage:{}",iosMessage);

        return iosMessage;
    }



    //获取IOS系统参数
    private IOSSystemMessage getIOSSystemParam(String userId){
        IOSSystemMessage message =new IOSSystemMessage();
        String deviceToken=null;
        try {
            /**
             * token为空的时候，不存redis。
             */
            if (redisService.hasKey(userId)){
             deviceToken = redisService.getCacheMapValue(userId, CommonConstant.DEVICE_TOKEN);
             log.info("从redis获取到的token是:{}",deviceToken);
            }
            String iosPackage = redisService.getCacheMapValue(userId, CommonConstant.IOS_PACKAGE);
            String iosLanguage = redisService.getCacheMapValue(userId, CommonConstant.IOS_LANGUAGE);
            String iosVersion = redisService.getCacheMapValue(userId, CommonConstant.IOS_VERSION);
            if ( StringUtils.isEmpty(iosPackage) || StringUtils.isEmpty(iosLanguage) || StringUtils.isEmpty(iosVersion)){
                throw   new ServiceException("messages.IOS.send.error");
            }
            message.setToken(deviceToken);
            message.setIdent(iosPackage);
            message.setOpenid(userId);
            message.setVersion(iosVersion);
            message.setAcceptLanguage(iosLanguage);
            return message;
        }catch (Exception e){
            log.info("错误是:{}",e.getMessage());
            throw   new ServiceException("messages.IOS.send.error");
        }

    }


    //发送KAFKA
    public void pushKafkaIOS(IOSSystemMessage systemMessage,IOSMessage message){
        String token =systemMessage.getToken();
       // String  bundleID =systemMessage.getIdent();
     //   String pushFun ="dev";
//        if ("produce".equals(systemMessage.getVersion())){
//            pushFun="dis";
//        }else {
//            pushFun="dev";
//        }
        message.setBundleID(systemMessage.getIdent());
        String imei=token +"&"+ iosVersion;
        log.info("下发kafkatopic:{},key:{},内容:{}",CommonConstant.IOS_PUSH_KAFKA,imei,message);
        log.info("IOS下发kafkatopic:{},key:{},字符串内容是:{}",CommonConstant.IOS_PUSH_KAFKA,imei,JSON.toJSONString(message));
        kafkaTemplate.send(CommonConstant.IOS_PUSH_KAFKA,imei, JSON.toJSONString(message)).addCallback(success->{
            log.info("发送到kafka具体信息{},partition 分区 {},offset {}", success.getRecordMetadata().topic(),success.getRecordMetadata().partition(),success.getRecordMetadata().offset());

        },failure -> {
            //失败
            log.info("下发到kafka的数据信息失败：{}",failure.getMessage());
            log.info("发送到kafka具体信息{}",JSON.toJSONString(message));

        });

    }


}
