package com.ecell.internationalize.common.security.utils;

import com.ecell.internationalize.common.core.constant.SecurityConstants;
import com.ecell.internationalize.common.core.constant.TokenConstants;
import com.ecell.internationalize.common.core.context.SecurityContextHolder;
import com.ecell.internationalize.common.core.domain.LoginUser;
import com.ecell.internationalize.common.core.domain.UserLogin;
import com.ecell.internationalize.common.core.exception.ServiceException;
import com.ecell.internationalize.common.core.utils.ServletUtils;
import com.ecell.internationalize.common.core.utils.StringUtils;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;

/**
 * @author borui
 */
public class SecurityUtils {
    /**
     * 获取用户ID
     */
//    public static Long getUserId()
//    {
//        return SecurityContextHolder.getUserId();
//    }
    public static String getUserId()
    {
        return SecurityContextHolder.getUserId();
    }

    /**
     * yisai获取用户ID
     */
    public static String getStringUserId()
    {
        return SecurityContextHolder.getStringUserId();
    }


    /**
     * 获取用户名称
     */
    public static String getUsername()
    {
        return SecurityContextHolder.getUserName();
    }

    /**
     * 获取用户key
     */
    public static String getUserKey()
    {
        return SecurityContextHolder.getUserKey();
    }

    /**
     * 获取登录用户信息
     */
    public static LoginUser getLoginUser()
    {
        return SecurityContextHolder.get(SecurityConstants.LOGIN_USER, LoginUser.class);
    }


    /**
     * yisai 获取登录用户信息
     */
    public static UserLogin getUserLogin()
    {
        return SecurityContextHolder.get(SecurityConstants.LOGIN_USER, UserLogin.class);
    }

    /**
     * 获取请求token
     */
    public static String getToken()
    {
        return getToken(ServletUtils.getRequest());
    }

    /**
     * 根据request获取请求token
     */
    public static String getToken(HttpServletRequest request)
    {
        // 从header获取token标识
        String token = request.getHeader(TokenConstants.AUTHENTICATION);
        return replaceTokenPrefix(token);
    }

    /**
     * 裁剪token前缀
     */
    public static String replaceTokenPrefix(String token)
    {
        // 如果前端设置了令牌前缀，则裁剪掉前缀
        if (StringUtils.isNotEmpty(token) && token.startsWith(TokenConstants.PREFIX))
        {
            token = token.replaceFirst(TokenConstants.PREFIX, "");
        }
        return token;
    }

    /**
     * 是否为管理员
     *
     * @param userId 用户ID
     * @return 结果
     */
    public static boolean isAdmin(Long userId)
    {
        return userId != null && 1L == userId;
    }


    /**
     * yisai是否为管理员
     *
     * @param userId 用户ID
     * @return 结果
     */
    public static boolean isAdminString(String userId)
    {
        return userId != null && "1".equals(userId);
    }

    /**
     * yisai是否为厂商
     *
     * @param firmFlag
     * @return 结果
     */
    public static boolean isFirm(String firmFlag)
    {
        return firmFlag != null && "1".equals(firmFlag);
    }

    /**
     * yisai是否为代理商
     * @param firmFlag
     * @return 结果
     */
    public static boolean isAgent(String firmFlag)
    {
        return firmFlag != null && "2".equals(firmFlag);
    }


    /**
     * 生成BCryptPasswordEncoder密码
     *
     * @param password 密码
     * @return 加密字符串
     */
    public static String encryptPassword(String password)
    {
        BCryptPasswordEncoder passwordEncoder = new BCryptPasswordEncoder();
        return passwordEncoder.encode(password);
    }

    /**
     * 判断密码是否相同
     *
     * @param rawPassword 真实密码
     * @param encodedPassword 加密后字符
     * @return 结果
     */
    public static boolean matchesPassword(String rawPassword, String encodedPassword)
    {
        BCryptPasswordEncoder passwordEncoder = new BCryptPasswordEncoder();
        return passwordEncoder.matches(rawPassword, encodedPassword);
    }




    /**
     * 获取厂商Id
     * @param userLogin
     * @return
     */
    public static String getFirmId(UserLogin userLogin) {

        if (StringUtils.isNull(userLogin)) {
            throw new ServiceException("用户有误，请重新登录");
        }
        if ("1".equals(userLogin.getFirmFlag())) {
            return userLogin.getFirmId();
        } else if ("2".equals(userLogin.getFirmFlag()) && StringUtils.isNotEmpty(userLogin.getSecondFirmId())) {
            return userLogin.getSecondFirmId();
        }
        return null;
    }

    /**
     * 获取厂商Id,代理商ID
     * @param userLogin
     * @return
     */
    public static Map<String,String> getFirmIdOrSecondFirmId(UserLogin userLogin) {
        Map<String,String> map =new HashMap<>(2);
        if (StringUtils.isNull(userLogin)) {
            throw new ServiceException("用户有误，请重新登录");
        }
//        //如果是管理员
//        if (SecurityUtils.isAdminString(SecurityContextHolder.getStringUserId())){
//            return map;
//        }
        //是厂商
        if ("1".equals(userLogin.getFirmFlag())) {
            map.put("firmFlag","1");
            map.put("firmId",userLogin.getFirmId());
            return map;
            //代理商
        } else if ("2".equals(userLogin.getFirmFlag()) && StringUtils.isNotEmpty(userLogin.getSecondFirmId())) {
            map.put("firmFlag","2");
            map.put("firmId",userLogin.getSecondFirmId());
            return map;
            //这里有可能是管理员，还有其余的角色，其余的角色不用做数据隔离，可以看全部
        }else if ("3".equals(userLogin.getFirmFlag())){
            return map;

        }
        return map;
    }
}
