package com.ecell.internationalize.system.security.controller;

import com.ecell.internationalize.common.core.utils.locale.LocaleUtil;
import com.ecell.internationalize.common.core.web.domain.AjaxResult;
import com.ecell.internationalize.common.security.utils.SecurityUtils;
import com.ecell.internationalize.system.security.service.SysMenuService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * @author borui
 */
@Api(value="菜单",tags={"菜单管理接口"})
@RestController
@RequestMapping("/sys_user")
public class SysMenuController {
    @Autowired
    private SysMenuService sysMenuService;
    /**
     * 构建树形菜单结构
     * @return 路由信息
     */
    @ApiOperation("构建树形菜单结构")
    @GetMapping("/menu/getMenuTree")
    public AjaxResult getMenuTree() {
        return AjaxResult.success(LocaleUtil.getMessage("messages.success"),sysMenuService.getMenuTree());
    }
    /**
     * 角色Id查询菜单相关
     * @Author liy
     * @Date 2022/7/25 10:38
     * @Return AjaxResult
     */
    @ApiOperation("根据角色Id查询菜单")
    @GetMapping(value = "role/queryMenuByRoleId")
    public AjaxResult queryMenuByRoleId(@RequestParam @ApiParam(name="Id",value="角色id",required=true) String roleId) {
        return AjaxResult.success(LocaleUtil.getMessage("messages.success"),sysMenuService.queryMenuByRoleId(roleId));
    }
    /**
     * 获取路由信息
     * @return 路由信息
     */
    @ApiOperation("获取路由信息")
    @GetMapping("menu/getRouter")
    public AjaxResult getRouters() {
        String userId =     SecurityUtils.getUserId();
        return AjaxResult.success(sysMenuService.selectMenuTreeByUserId(userId));
    }
}
