package com.ecell.internationalize.system.security.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ecell.internationalize.common.core.utils.StringUtils;
import com.ecell.internationalize.common.core.utils.locale.LocaleUtil;
import com.ecell.internationalize.common.core.web.domain.AjaxResult;
import com.ecell.internationalize.common.security.utils.SecurityUtils;
import com.ecell.internationalize.common.system.annotation.SystemLog;
import com.ecell.internationalize.common.system.constant.FieldConstant;
import com.ecell.internationalize.common.system.entity.SysRole;
import com.ecell.internationalize.common.system.entity.SysUserRole;
import com.ecell.internationalize.system.security.service.SysRoleService;
import com.ecell.internationalize.system.security.service.SysUserRoleService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author borui
 */
@Api(value = "角色管理", tags = {"角色管理接口"})
@RestController
@RequestMapping("/sys_user")
public class SysRoleController {
    @Autowired
    private SysRoleService roleService;

    @Autowired
    private SysUserRoleService sysUserRoleService;

    /**
     * 角色管理条件分页查询
     *
     * @param map 分页条件查询体
     * @Author liy
     * @Date 2022/7/21 10:38
     * @Return AjaxResult
     */
    @ApiOperation("条件分页查询角色信息")
    @PostMapping("role/pageList")
    public AjaxResult queryAllByPage(@RequestBody Map<String, Object> map) {
        Page<SysRole> page = new Page<>(Integer.parseInt(map.get(FieldConstant.CURRENT).toString()), Integer.parseInt(map.get(FieldConstant.SIZE).toString()));
        LambdaQueryWrapper<SysRole> sysRoleLambdaQueryWrapper = new LambdaQueryWrapper<>();
        if (map.containsKey(FieldConstant.ROLE_NAME) && null != map.get(FieldConstant.ROLE_NAME).toString()) {
            sysRoleLambdaQueryWrapper.eq(SysRole::getRoleName, map.get(FieldConstant.ROLE_NAME).toString());
        }
        if (map.containsKey(FieldConstant.ACCOUNT_STATUS) && null != map.get(FieldConstant.ACCOUNT_STATUS).toString()) {
            sysRoleLambdaQueryWrapper.eq(SysRole::getStatus, map.get(FieldConstant.ACCOUNT_STATUS).toString());
        }
        sysRoleLambdaQueryWrapper.eq(SysRole::getDelFlag, FieldConstant.DEL_FLAG);
        sysRoleLambdaQueryWrapper.orderByDesc(SysRole::getCreateTime);
        return AjaxResult.success(LocaleUtil.getMessage("messages.success"), roleService.getBaseMapper().selectPage(page, sysRoleLambdaQueryWrapper));
    }

    /**
     * 新增角色信息
     *
     * @param sysRole 用户
     * @Author liy
     * @Date 2022/7/21 11:38
     * @Return AjaxResult
     */
    @ApiOperation("新增角色信息")
    @SystemLog(msg = FieldConstant.ROLE_MANAGE, operation = FieldConstant.SAVE_OPERATOR)
    @PostMapping("role/save")
    public AjaxResult save(@RequestBody @ApiParam(name = "角色", value = "角色实体", required = true) SysRole sysRole) {
        int count = roleService.count(new QueryWrapper<SysRole>()
                .eq(FieldConstant.ROLE_NAME, sysRole.getRoleName())
                .eq(FieldConstant.USER_DEL_FLAG, FieldConstant.DEL_FLAG)
        );
        if (count > 0) {
            //角色已存在
            return AjaxResult.error(LocaleUtil.getMessage("messages.role.exit"));
        }
        sysRole.setCreateUser(SecurityUtils.getUsername());
        return AjaxResult.success(LocaleUtil.getMessage("messages.success"), roleService.saveRole(sysRole));
    }

    /**
     * 修改角色信息
     *
     * @param sysRole 用户
     * @Author liy
     * @Date 2022/7/21 16:38
     * @Return AjaxResult
     */
    @ApiOperation("修改角色信息")
    @SystemLog(msg = FieldConstant.ROLE_MANAGE, operation = FieldConstant.UPDATE_OPERATOR)
    @PostMapping("role/update")
    public AjaxResult update(@RequestBody @ApiParam(name = "角色", value = "角色实体", required = true) SysRole sysRole) {
        //admin用户禁止修改操作
        if (FieldConstant.ADMIN.equals(sysRole.getRoleId())) {
            return AjaxResult.error(LocaleUtil.getMessage("messages.not.delete"));
        }
        LambdaQueryWrapper<SysRole> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(SysRole::getRoleName, sysRole.getRoleName());
        lambdaQueryWrapper.eq(SysRole::getDelFlag, FieldConstant.DEL_FLAG);
        SysRole role = roleService.getBaseMapper().selectOne(lambdaQueryWrapper);
        //修改名称相同
        if (null != role && !role.getRoleId().equals(sysRole.getRoleId())) {
            //角色已存在
            return AjaxResult.error(LocaleUtil.getMessage("messages.role.exit"));
        }
        sysRole.setUpdateTime(new Date());
        sysRole.setUpdateUser(SecurityUtils.getUsername());
        return AjaxResult.success(LocaleUtil.getMessage("messages.success"), roleService.updateRole(sysRole));
    }

    /**
     * 删除角色信息
     *
     * @param sysRole 用户
     * @Author liy
     * @Date 2022/7/21 16:38
     * @Return AjaxResult
     */
    @ApiOperation("删除角色信息")
    @SystemLog(msg = FieldConstant.ROLE_MANAGE, operation = FieldConstant.DELETE_OPERATOR)
    @PostMapping("role/del")
    public AjaxResult del(@RequestBody @ApiParam(name = "角色", value = "角色实体", required = true) SysRole sysRole) {
        //admin用户禁止修改操作
        if (FieldConstant.ADMIN.equals(sysRole.getRoleId())) {
            return AjaxResult.error(LocaleUtil.getMessage("messages.not.delete"));
        }
        //根据角色id查询用户
        LambdaQueryWrapper<SysUserRole> eq = new LambdaQueryWrapper<SysUserRole>().eq(SysUserRole::getRoleId, sysRole.getRoleId());
        List<SysUserRole> list = sysUserRoleService.list(eq);
        if (StringUtils.isNotNull(list) && list.size() > 0) {
            return AjaxResult.error(LocaleUtil.getMessage("messages.delete.exist"));
        }
        return AjaxResult.success(LocaleUtil.getMessage("messages.success"), roleService.deleteRoleById(sysRole));
    }

    /**
     * 修改角色状态
     *
     * @param sysRole 角色
     * @Author liy
     * @Date 2022/7/21 16:38
     * @Return AjaxResult
     */
    @ApiOperation("修改角色状态")
    @SystemLog(msg = FieldConstant.ROLE_MANAGE, operation = FieldConstant.CHANGE_STATUS_OPERATOR)
    @PostMapping("role/updateStatus")
    public AjaxResult updateStatus(@RequestBody @ApiParam(name = "角色", value = "角色实体", required = true) SysRole sysRole) {
        //admin用户禁止修改操作
        if (FieldConstant.ADMIN.equals(sysRole.getRoleId())) {
            return AjaxResult.error(LocaleUtil.getMessage("messages.not.delete"));
        }
        return AjaxResult.success(LocaleUtil.getMessage("messages.success"), roleService.updateById(sysRole));
    }

    /**
     * 查询角色名称
     *
     * @Author liy
     * @Date 2022/7/25 10:38
     * @Return AjaxResult
     */
    @ApiOperation("查询角色名称")
    @GetMapping(value = "role/queryRole")
    public AjaxResult queryRole() {
        LambdaQueryWrapper<SysRole> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(SysRole::getDelFlag, FieldConstant.DEL_FLAG);
        return AjaxResult.success(LocaleUtil.getMessage("messages.success"), roleService.getBaseMapper().selectList(lambdaQueryWrapper));
    }
}
