package com.ecell.internationalize.system.security.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.ecell.internationalize.common.core.constant.Constants;
import com.ecell.internationalize.common.core.domain.UserLogin;
import com.ecell.internationalize.common.core.utils.ServletUtils;
import com.ecell.internationalize.common.core.utils.StringUtils;
import com.ecell.internationalize.common.core.utils.ip.IpUtils;
import com.ecell.internationalize.common.core.utils.locale.LocaleUtil;
import com.ecell.internationalize.common.core.web.domain.AjaxResult;
import com.ecell.internationalize.common.security.utils.SecurityUtils;
import com.ecell.internationalize.common.system.annotation.SystemLog;
import com.ecell.internationalize.common.system.constant.FieldConstant;
import com.ecell.internationalize.common.system.entity.SysUser;
import com.ecell.internationalize.common.system.entity.SysUserRole;
import com.ecell.internationalize.common.system.utlis.UploadUtil;
import com.ecell.internationalize.system.security.feign.FirmManageFeignClient;
import com.ecell.internationalize.system.security.service.SysPermissionService;
import com.ecell.internationalize.system.security.service.SysUserRoleService;
import com.ecell.internationalize.system.security.service.SysUserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @author borui
 */
@Api(value="用户管理",tags={"用户管理接口"})
@RestController
@RequestMapping("/sys_user")
@Slf4j
public class SysUserController {
    @Autowired
    private SysUserService sysUserService;
    @Autowired
    private SysPermissionService permissionService;
    @Autowired
    private FirmManageFeignClient firmManageFeignClient;


    /**
     * 查询所有用户
     * @Author: liy
     * @Date: 2022/7/18 16:34
     */
    @ApiOperation("查询所有用户(非分页)")
    @GetMapping("user/list")
    public List<SysUser> queryAll() {
        return sysUserService.getBaseMapper().selectList(null);
    }
    /**
     * 根据userId查询用户信息
     * @Author: liy
     * @Date: 2022/7/18 16:34
     */
    @ApiOperation("根据userId查询用户信息")
    @GetMapping("user/queryByUserId")
    public SysUser queryByUserId() {
        LambdaQueryWrapper<SysUser> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(SysUser::getUserId, SecurityUtils.getUserId());
        return sysUserService.getBaseMapper().selectOne(lambdaQueryWrapper);
    }
    /**
     * 根据用户账号查询
     * @Author: liy
     * @Date: 2022/7/18 16:34
     */
    @ApiOperation("根据用户账号查询")
    @GetMapping("user/queryByUserName")
    public UserLogin queryByUserName(@ApiParam(name="userName",value="用户账号",required=true) String userName) {
        LambdaQueryWrapper<SysUser> sysUserLambdaQueryWrapper=new LambdaQueryWrapper<>();
        sysUserLambdaQueryWrapper.eq(SysUser::getAccount,userName);
        sysUserLambdaQueryWrapper.eq(SysUser::getStatus, Constants.LOGIN_SUCCESS_STATUS);
        sysUserLambdaQueryWrapper.eq(SysUser::getDelFlag, Constants.LOGIN_FAIL_STATUS);
        UserLogin sysUserVo = new UserLogin();
        SysUser sysUser = sysUserService.getBaseMapper().selectOne(sysUserLambdaQueryWrapper);
        if (null == sysUser){
            return null;
        }
        // 角色集合
        Set<String> roles = permissionService.getRolePermission(sysUser.getUserId());
        // 权限集合
        Set<String> permissions = permissionService.getMenuPermission(sysUser.getUserId());
        sysUserVo.setUserId(sysUser.getUserId());
        sysUserVo.setAccount(sysUser.getAccount());
        sysUserVo.setPassword(sysUser.getPassword());
        sysUserVo.setDelFlag(sysUser.getDelFlag());
        sysUserVo.setFirmFlag(sysUser.getFirmFlag());
        sysUserVo.setFirmId(sysUser.getFirmId());
        sysUserVo.setStatus(sysUser.getStatus());
        sysUserVo.setSecondFirmId(sysUser.getSecondFirmId());
        sysUserVo.setRoles(roles);
        sysUserVo.setPermissions(permissions);
        return sysUserVo;
    }
    /**
     * 获取用户信息
     * @Author: liy
     * @Date: 2022/7/20 16:34
     */
    @ApiOperation("获取用户信息")
    @GetMapping("user/getInfo")
    public AjaxResult getInfo()
    {
        String userId = String.valueOf(SecurityUtils.getUserId());
        log.info("获取到的userId:{}",userId);
        // 角色集合
        Set<String> roles = permissionService.getRolePermission(userId);
        // 菜单权限集合
        Set<String> permissions = permissionService.getMenuPermission(userId);
        AjaxResult ajax = AjaxResult.success();
        ajax.put("user", sysUserService.getBaseMapper().selectById(userId));
        ajax.put("roles", roles);
        ajax.put("permissions", permissions);
        return ajax;
    }
    /**
     * 用户管理条件分页查询
     * @Author liy
     * @Date 2022/7/20 15:38
     * @param  map 分页条件查询体
     * @Return AjaxResult
     */
    @ApiOperation("条件分页查询用户信息")
  //  @RequiresPermissions("system:userList:list")
    @PostMapping("user/pageList")
    public AjaxResult queryAllByPage(@RequestBody Map<String,Object> map){
        return AjaxResult.success(LocaleUtil.getMessage("messages.success"),sysUserService.findAllByPage(map));
    }
    /**
     * 新增用户信息
     * @Author liy
     * @Date 2022/7/20 16:38
     * @param sysUser 用户
     * @Return AjaxResult
     */
    @ApiOperation("新增用户信息")
//    @RequiresPermissions("system:userList:add")
    @SystemLog(msg = FieldConstant.USER_MANAGE,operation = FieldConstant.SAVE_OPERATOR)
    @PostMapping("user/save")
    public AjaxResult save(@RequestBody @ApiParam(name="用户", value = "用户实体",required=true) SysUser sysUser) {
        sysUser.setHeadImg(FieldConstant.USER_IMAGE);
        int count = sysUserService.count(new QueryWrapper<SysUser>()
                .eq(FieldConstant.ACCOUNT,sysUser.getAccount())
                .eq(FieldConstant.USER_DEL_FLAG,FieldConstant.DEL_FLAG)
        );
        if (count>0){
            //账号已存在
            return AjaxResult.error(LocaleUtil.getMessage("messages.account.exit"));
        }
        return AjaxResult.success(LocaleUtil.getMessage("messages.success"),sysUserService.saveUser(sysUser));
    }
    /**
     * 修改用户信息
     * @Author liy
     * @Date 2022/7/20 16:38
     * @param sysUser 用户
     * @Return AjaxResult
     */
    @ApiOperation("修改用户信息")
    //@RequiresPermissions("system:userList:update")
    @SystemLog(msg =FieldConstant.USER_MANAGE,operation = FieldConstant.UPDATE_OPERATOR)
    @PostMapping("user/update")
    public AjaxResult update(@RequestBody @ApiParam(name="用户", value = "用户实体",required=true) SysUser sysUser){
        //admin用户禁止修改操作
        if(FieldConstant.ADMIN.equals(sysUser.getUserId())){
            return AjaxResult.error(LocaleUtil.getMessage("messages.not.delete"));
        }
        LambdaQueryWrapper<SysUser> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(SysUser::getAccount,sysUser.getAccount());
        lambdaQueryWrapper.eq(SysUser::getDelFlag,FieldConstant.DEL_FLAG);
        SysUser user = sysUserService.getBaseMapper().selectOne(lambdaQueryWrapper);
        //修改名称相同
        if (null!=user&&!user.getUserId().equals(sysUser.getUserId())){
            //用户已存在
            return AjaxResult.error(LocaleUtil.getMessage("messages.account.exit"));
        }
        sysUser.setUpdateTime(new Date());
        sysUser.setUpdateUser(SecurityUtils.getUsername());
        return AjaxResult.success(LocaleUtil.getMessage("messages.success"),sysUserService.updateUser(sysUser));
    }
    /**
     * 删除用户信息
     * @Author liy
     * @Date 2022/7/20 16:38
     * @param sysUser 用户
     * @Return AjaxResult
     */
    @ApiOperation("删除用户信息")
   // @RequiresPermissions("system:userList:delete")
    @SystemLog(msg =FieldConstant.USER_MANAGE,operation = FieldConstant.DELETE_OPERATOR)
    @PostMapping("user/del")
    public AjaxResult del(@RequestBody @ApiParam(name="用户", value = "用户实体",required=true) SysUser sysUser){
        //admin用户禁止删除
        if(FieldConstant.ADMIN.equals(sysUser.getUserId())){
            return AjaxResult.error(LocaleUtil.getMessage("messages.not.delete"));
        }
        return AjaxResult.success(LocaleUtil.getMessage("messages.success"),sysUserService.deleteUserById(sysUser));
    }
    /**
     * 修改用户状态
     * @Author liy
     * @Date 2022/7/20 16:38
     * @param sysUser 用户
     * @Return AjaxResult
     */
    @ApiOperation("修改用户状态")
   // @RequiresPermissions("system:userList:status")
    @SystemLog(msg =FieldConstant.USER_MANAGE,operation = FieldConstant.CHANGE_STATUS_OPERATOR)
    @PostMapping("user/updateStatus")
    public AjaxResult updateStatus(@RequestBody @ApiParam(name="用户", value = "用户实体",required=true) SysUser sysUser){
        //admin用户禁止修改状态
        if(FieldConstant.ADMIN.equals(sysUser.getUserId())){
            return AjaxResult.error(LocaleUtil.getMessage("messages.not.delete"));
        }
        return AjaxResult.success(LocaleUtil.getMessage("messages.success"),sysUserService.updateById(sysUser));
    }
    /**
     * 显示所有厂商名称
     * @Author liy
     * @Date 2022/7/25 14:38
     * @Return AjaxResult
     */
    @ApiOperation("显示所有厂商名称")
    @GetMapping("user/queryFirm")
    public AjaxResult queryFirm(){
        return firmManageFeignClient.queryAll();
    }
    /**
     * 根据Id查询
     * @Author liy
     * @Date 2022/7/25 10:38
     * @param firmId 主键firmId
     * @Return AjaxResult
     */
    @ApiOperation("根据firmId查询厂商信息")
    @GetMapping(value = "user/queryById")
    public AjaxResult queryById(@ApiParam(name="Id", value = "厂商Id",required=true) String firmId) {
        return firmManageFeignClient.findById(firmId);
    }

    /**
     * 认证修改用户登录时间
     * @Author liy
     * @Date 2022/7/20 16:38
     * @param sysUser 用户
     * @Return AjaxResult
     */
    @ApiOperation("认证修改用户登录时间")
    @PostMapping("user/updateLoginStatus")
    public AjaxResult updateLoginStatus(@RequestBody @ApiParam(name="用户", value = "用户实体",required=true) SysUser sysUser){
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),sysUserService.updateById(sysUser));
    }
    /**
     * 重置密码
     */
    @ApiOperation("重置密码")
    @SystemLog(msg =FieldConstant.USER_MANAGE,operation = FieldConstant.RESET_PASSWORD)
    @PostMapping("user/resetPwd")
    public AjaxResult resetPwd(@RequestBody @ApiParam(name="密码", value = "重置密码,旧密码字段为password,新密码字段为：newPassword",required=true) SysUser sysUser) {
        String userId=SecurityUtils.getUserId();
        sysUser.setUserId(userId);
        //若为admin，禁止操作
        if (FieldConstant.ADMIN.equals(userId)) {
            return AjaxResult.error(LocaleUtil.getMessage("messages.admin.prohibit"));
        }
        LambdaQueryWrapper<SysUser> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(SysUser::getUserId,userId);
        SysUser user = sysUserService.getBaseMapper().selectOne(lambdaQueryWrapper);
        //输入的旧密码与实际密码不一致
        if (!SecurityUtils.matchesPassword(sysUser.getPassword(),user.getPassword())){
            return AjaxResult.error(LocaleUtil.getMessage("messages.different.password"));
        }
        sysUser.setPassword(SecurityUtils.encryptPassword(sysUser.getNewPassword()));
        sysUser.setUpdateUser(SecurityUtils.getUsername());
        return AjaxResult.success(LocaleUtil.getMessage("messages.success"),sysUserService.updateById(sysUser));
    }
    /**
     * 修改个人信息
     */
    @ApiOperation("修改个人信息")
    @SystemLog(msg =FieldConstant.USER_MANAGE,operation = FieldConstant.UPDATE_USER_INFO)
    @PostMapping("user/updateInfo")
    public AjaxResult updateInfo(@RequestParam("file") MultipartFile file, @RequestParam("nickName")String nickName) throws IOException {
        String userId=SecurityUtils.getUserId();
        SysUser sysUser=new SysUser();
        sysUser.setNickName(nickName);
        String path = UploadUtil.upload(file);
        sysUser.setUserId(userId);
        sysUser.setHeadImg(path);
        return AjaxResult.success(LocaleUtil.getMessage("messages.success"),sysUserService.updateById(sysUser));
    }
    /**
     * 获取IP地址
     */
    @ApiOperation("获取IP地址")
    @GetMapping("user/getIpHost")
    public AjaxResult getIpHost(){
        String ip = IpUtils.getIpAddr(ServletUtils.getRequest());
        return AjaxResult.success(LocaleUtil.getMessage("messages.success"), ip);
    }
}
