package com.ecell.internationalize.system.security.service.impl;


import com.ecell.internationalize.common.system.constant.FieldConstant;
import com.ecell.internationalize.system.security.service.SysMenuService;
import com.ecell.internationalize.system.security.service.SysPermissionService;
import com.ecell.internationalize.system.security.service.SysRoleService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashSet;
import java.util.Set;

/**
 * @Title: SysPermissionServiceImpl
 * @Author: liy
 * @Date: 2022/7/19 16:04
 * @Description:
 * @Version:1.0
 */
@Service
public class SysPermissionServiceImpl implements SysPermissionService {
    @Autowired
    private SysRoleService roleService;

    @Autowired
    private SysMenuService menuService;
    /**
     * 获取角色数据权限
     *
     * @param userId 用户Id
     * @return 角色权限信息
     */
    @Override
    public Set<String> getRolePermission(String userId) {
        Set<String> roles = new HashSet<String>();
        // 管理员拥有所有权限
        if (FieldConstant.ADMIN.equals(userId)){
            roles.add("admin");
        } else {
            roles.addAll(roleService.selectRolePermissionByUserId(userId));
        }
        return roles;
    }

    /**
     * 获取菜单数据权限
     *
     * @param userId 用户Id
     * @return 菜单权限信息
     */
    @Override
    public Set<String> getMenuPermission(String userId) {
        Set<String> perms = new HashSet<String>();
        // 管理员拥有所有权限
        if (FieldConstant.ADMIN.equals(userId)) {
            perms.add("*:*:*");
        } else {
            perms.addAll(menuService.selectMenuPermsByUserId(userId));
        }
        return perms;
    }
}
