package com.ecell.internationalize.system.security.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ecell.internationalize.common.core.utils.StringUtils;
import com.ecell.internationalize.common.core.utils.uuid.UUID;
import com.ecell.internationalize.common.security.utils.SecurityUtils;
import com.ecell.internationalize.common.system.entity.SysRole;
import com.ecell.internationalize.common.system.entity.SysRoleMenu;
import com.ecell.internationalize.system.security.mapper.SysRoleMapper;
import com.ecell.internationalize.system.security.mapper.SysRoleMenuMapper;
import com.ecell.internationalize.system.security.service.SysRoleService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

/**
 * @Title: SysRoleServiceImpl
 * @Author: liy
 * @Date: 2022/7/18 16:24
 * @Description:
 * @Version:1.0
 */
@Service
public class SysRoleServiceImpl extends ServiceImpl<SysRoleMapper, SysRole> implements SysRoleService {
    @Autowired
    private SysRoleMapper sysRoleMapper;
    @Autowired
    private SysRoleMenuMapper sysRoleMenuMapper;
    /**
     * 根据用户ID查询权限
     *
     * @param userId 用户ID
     * @return 权限列表
     */
    @Override
    public Set<String> selectRolePermissionByUserId(String userId)
    {
        List<SysRole> perms = sysRoleMapper.selectRolePermissionByUserId(userId);
        Set<String> permsSet = new HashSet<>();
        for (SysRole perm : perms) {
            if (StringUtils.isNotNull(perm)) {
                permsSet.addAll(Arrays.asList(perm.getRoleName().trim().split(",")));
            }
        }
        return permsSet;
    }
    /**
     * 新增角色
     * @Author liy
     * @Date 2022/7/21 16:42
     * @param sysRole 实体类
     * @return void
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public int saveRole(SysRole sysRole) {
        sysRole.setCreateUser(SecurityUtils.getUsername());
        String roleId= UUID.randomUUID().toString(true);
        sysRole.setRoleId(roleId);
        sysRole.setCreateTime(new Date());
        //新增角色信息
        int rows = sysRoleMapper.insert(sysRole);
        //新增关联的角色菜单信息
        insertRoleMenu(sysRole);
        return rows;
    }
    /**
     * 修改保存角色信息
     *
     * @param sysRole 角色信息
     * @return 结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int updateRole(SysRole sysRole) {
        // 修改角色信息
        int rows = sysRoleMapper.updateById(sysRole);
        // 删除角色与菜单关联
        sysRoleMenuMapper.deleteRoleMenuByRoleId(sysRole.getRoleId());
         insertRoleMenu(sysRole);
         return rows;
    }
    /**
     * 删除角色
     * @Author liy
     * @Date 2022/7/21 16:42
     * @param sysRole 角色信息
     * @return int
     */
    @Override
    public int deleteRoleById(SysRole sysRole) {
        // 删除角色与菜单关联
        LambdaQueryWrapper<SysRoleMenu> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(SysRoleMenu::getRoleId,sysRole.getRoleId());
        sysRoleMenuMapper.delete(lambdaQueryWrapper);
        return sysRoleMapper.updateById(sysRole);

    }

    /**
     * 新增角色菜单信息
     * @param sysRole 角色对象
     */
    public void insertRoleMenu(SysRole sysRole) {
        // 新增用户与角色管理
        List<SysRoleMenu> list = new ArrayList<>();
        for (String menuId : sysRole.getMenuIds()) {
            SysRoleMenu rm = new SysRoleMenu();
            rm.setRoleId(sysRole.getRoleId());
            rm.setMenuId(menuId);
            list.add(rm);
        }
        if (list.size() > 0) {
            sysRoleMenuMapper.batchRoleMenu(list);
        }
    }
}
