package com.ecell.internationalize.system.security.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ecell.internationalize.common.core.utils.StringUtils;
import com.ecell.internationalize.common.core.utils.uuid.UUID;
import com.ecell.internationalize.common.security.utils.SecurityUtils;
import com.ecell.internationalize.common.system.constant.FieldConstant;
import com.ecell.internationalize.common.system.entity.SysUser;
import com.ecell.internationalize.common.system.entity.SysUserRole;
import com.ecell.internationalize.system.security.entity.FirmManageVO;
import com.ecell.internationalize.system.security.feign.FirmManageFeignClient;
import com.ecell.internationalize.system.security.mapper.SysUserMapper;
import com.ecell.internationalize.system.security.mapper.SysUserRoleMapper;
import com.ecell.internationalize.system.security.service.SysUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.*;

/**
 * @Title: SysUserServiceImpl
 * @Author: liy
 * @Date: 2022/7/18 16:24
 * @Description:
 * @Version:1.0
 */
@Service
public class SysUserServiceImpl extends ServiceImpl<SysUserMapper, SysUser> implements SysUserService {
    @Autowired
    private SysUserMapper sysUserMapper;
    @Autowired
    private SysUserRoleMapper sysUserRoleMapper;
    @Autowired
    private FirmManageFeignClient firmManageFeignClient;
    /**
     * 用户管理条件分页查询
     * @Author liy
     * @Date 2022/7/20 16:42
     * @param map 分页查询体
     * @return IPage
     */
    @Override
    public IPage<SysUser> findAllByPage(Map<String, Object> map) {
        Page<SysUser> page=new Page<>(Integer.parseInt(map.get(FieldConstant.CURRENT).toString()),Integer.parseInt(map.get(FieldConstant.SIZE).toString()));
        Page<SysUser> list = sysUserMapper.getList(page, map);
        Map<String, String> agent = getAgent();
        list.getRecords().forEach(temp ->{
            //为代理商
            if (FieldConstant.SECOND_FLAG.equals(temp.getFirmFlag())){
                temp.setSecondFirmName(agent.get(temp.getSecondFirmId()));
            }
        });
        return list;
    }
    /**
     * 新增用户
     * @Author liy
     * @Date 2022/7/20 16:42
     * @param sysUser 实体类
     * @return void
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public int saveUser(SysUser sysUser) {
        //密码加密
        sysUser.setPassword(SecurityUtils.encryptPassword(sysUser.getPassword()));
        sysUser.setCreateUser(SecurityUtils.getUsername());
        String userId= UUID.randomUUID().toString(true);
        sysUser.setUserId(userId);
        sysUser.setCreateTime(new Date());
        //新增用户信息
        int rows = sysUserMapper.insert(sysUser);
        //新增关联的用户角色
        insertUserRole(sysUser.getUserId(),sysUser.getRoleId());
        return rows;
    }
    /**
     * 删除用户
     * @Author liy
     * @Date 2022/7/20 16:42
     * @param sysUser 用户
     * @return int
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public int deleteUserById(SysUser sysUser) {
        // 删除用户与角色关联
        LambdaQueryWrapper<SysUserRole> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(SysUserRole::getUserId,sysUser.getUserId());
        sysUserRoleMapper.delete(lambdaQueryWrapper);
        return sysUserMapper.updateById(sysUser);
    }
    /**
     * 修改保存用户信息
     * @param sysUser 用户信息
     * @return 结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int updateUser(SysUser sysUser) {
        String userId = sysUser.getUserId();
        // 删除用户与角色关联
        sysUserRoleMapper.deleteUserRoleByUserId(userId);
        // 新增用户与角色管理
        insertUserRole(userId,sysUser.getRoleId());
        return sysUserMapper.updateById(sysUser);
    }
    /**
     * 新增用户角色信息
     * @param userId 用户ID
     * @param roleId 角色
     */
    public void insertUserRole(String userId, String roleId) {
        if (StringUtils.isNotEmpty(roleId)) {
            // 新增用户与角色管理
            SysUserRole ur = new SysUserRole();
            ur.setUserId(userId);
            ur.setRoleId(roleId);
            sysUserRoleMapper.insert(ur);
        }
    }
    /**
     * 封装代理商信息
     */
    public Map<String,String> getAgent() {
        List<FirmManageVO> agent = firmManageFeignClient.getAgent();
        Map<String,String> map=new HashMap<>(16);
        if (!CollectionUtils.isEmpty(agent)){
            for (FirmManageVO firmManageVO:agent){
                map.put(firmManageVO.getFirmId(),firmManageVO.getFirmName());
            }
        }
        return map;
    }
}
