package com.ecell.internationalize.system.security.util;

import com.ecell.internationalize.common.system.entity.SysMenu;
import com.ecell.internationalize.system.security.entity.MenuTree;
import com.ecell.internationalize.system.security.entity.TreeNode;
import lombok.experimental.UtilityClass;

import java.util.ArrayList;
import java.util.List;

/**
 * @author borui
 */
@UtilityClass
public class TreeUtil {
    /**
     * 两层循环实现建树
     * @param treeNodes 传入的树节点列表
     * @return
     */
    public <T extends TreeNode> List<T> build(List<T> treeNodes, Object root) {
        List<T> trees = new ArrayList<>();
        for (T treeNode : treeNodes) {
            if (root.equals(treeNode.getParentId())) {
                trees.add(treeNode);
            }
            for (T it : treeNodes) {
                if (it.getParentId().equals(treeNode.getMenuId())) {
                    if (treeNode.getChildren() == null) {
                        treeNode.setChildren(new ArrayList<>());
                    }
                    treeNode.add(it);
                }
            }
        }
        return trees;
    }

    /**
     * 通过SysMenu创建树形节点
     * @param sysMenus
     * @param root
     * @return
     */
    public List<MenuTree> buildFirmTree(List<SysMenu> sysMenus, String root) {
        List<MenuTree> trees = new ArrayList<>();
        MenuTree node;
        for (SysMenu sysMenu : sysMenus) {
            node = new MenuTree();
            node.setMenuId(sysMenu.getMenuId());
            node.setParentId(sysMenu.getParentId());
            node.setPath(sysMenu.getPath());
            node.setMenuName(sysMenu.getMenuName());
            node.setHasChildren(false);
            trees.add(node);
        }
        return TreeUtil.build(trees, root);
    }
}
