package com.ecell.internationalize.system.aspect;
import com.ecell.internationalize.common.core.domain.UserLogin;
import com.ecell.internationalize.common.core.utils.StringUtils;
import com.ecell.internationalize.common.core.utils.ip.IpUtils;
import com.ecell.internationalize.common.core.utils.poi.ExcelUtil;
import com.ecell.internationalize.common.core.utils.uuid.UUID;
import com.ecell.internationalize.common.security.utils.SecurityUtils;
import com.ecell.internationalize.system.annotation.DelImeiLog;
import com.ecell.internationalize.system.constant.DeviceModelConstants;
import com.ecell.internationalize.system.entity.DelimeiLogInfo;
import com.ecell.internationalize.system.entity.dto.DeviceDTO;
import com.ecell.internationalize.system.service.impl.DelimeiLogInfoServiceImpl;
import com.ecell.internationalize.system.utils.HttpServletUtil;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.Aspect;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartFile;

import java.util.*;

@Aspect
@Component
public class DeleteImeiLog {
    private static final Logger logg = LoggerFactory.getLogger(DeleteImeiLog.class);
    @Autowired
    private DelimeiLogInfoServiceImpl delimeiLogInfoService;

    /**
     * 处理完请求后执行
     *
     * @param joinPoint 切点
     */
    @AfterReturning(pointcut = "@annotation(controllerLog)", returning = "jsonResult")
    public void doAfterReturning(JoinPoint joinPoint, DelImeiLog controllerLog, Object jsonResult)
    {
        handleLog(joinPoint, controllerLog, null, jsonResult);
    }


    protected void handleLog(final JoinPoint joinPoint, DelImeiLog controllerLog, final Exception e, Object jsonResult)
    {
        try
        {
            // *========数据库日志=========*//
            DelimeiLogInfo info =new DelimeiLogInfo();
            info.setImeiDelId(UUID.fastUUID().toString(true));
            info.setCreateTime(new Date());
            // 请求的地址
            String ip = IpUtils.getIpAddr(HttpServletUtil.getRequest());
            info.setIp(ip);
            String username = SecurityUtils.getUsername();
            UserLogin userLogin = SecurityUtils.getUserLogin();
            if (StringUtils.isNotEmpty(userLogin.getRoles()) && userLogin.getRoles().size()>0){
                info.setUserRole(Arrays.toString(userLogin.getRoles().toArray(new String[0])));
            }else {
                info.setUserRole("");
            }
            if (StringUtils.isNotBlank(username))
            {
                info.setLoginAccount(username);
            }
            // 处理设置注解上的参数
            getControllerMethodDescription(joinPoint, controllerLog, info, jsonResult);
            // 保存数据库
            //imeiLogInfoService.save(info);
            delimeiLogInfoService.saveDelImeiLog(info);

        }
        catch (Exception exp)
        {
            // 记录本地异常日志
            logg.error("==通知异常==");
            logg.error("异常信息:{}", exp.getMessage());

            exp.printStackTrace();
        }
    }

    private void getControllerMethodDescription(JoinPoint joinPoint, DelImeiLog controllerLog, DelimeiLogInfo info, Object jsonResult) {
        //根据id批量删除
        if (DeviceModelConstants.ZERO.equals(controllerLog.delType())){
            info.setUnbinding(DeviceModelConstants.DELLETE_MODE_ZERO);
            argsArrayToString(joinPoint.getArgs(),info);
        }else {
            info.setUnbinding(DeviceModelConstants.DELLETE_MODE_ONE);
            //根据导入的imei进行删除
            argsArrayTwoToString(joinPoint.getArgs(),info,jsonResult);

        }


    }

    private void argsArrayToString(Object[] paramsArray, DelimeiLogInfo info) {
         StringBuilder builder =new StringBuilder();
        List<String> list =new ArrayList<>();
        if (paramsArray != null && paramsArray.length > 0){
            if (paramsArray[0] instanceof String ){
                list= Arrays.asList((String) paramsArray[0]);
            }else {
                list =(List<String>) paramsArray[0];
            }

            if (StringUtils.isNotEmpty(list) && list.size()>0){
                builder.append("共有:"+list.size()+"条：");
                for (String s : list) {
                    builder.append("<br/>"+s+"<br/>").append(",");
                }

            }

        }
        info.setOperationDescribe(builder.toString());

    }

    private void argsArrayTwoToString(Object[] paramsArray, DelimeiLogInfo info, Object jsonResult) {
        List<DeviceDTO> userList = Collections.emptyList();
        StringBuilder builder =new StringBuilder();
        if (paramsArray != null && paramsArray.length > 0) {
            try {

                MultipartFile file = (MultipartFile) paramsArray[0];
                if (StringUtils.isNotNull(file)) {
                    ExcelUtil<DeviceDTO> util = new ExcelUtil<DeviceDTO>(DeviceDTO.class);
                    userList = util.importExcel(file.getInputStream());
                   if (StringUtils.isNotEmpty(userList) && userList.size()>0){
                       for (DeviceDTO deviceDTO : userList) {
                           builder.append("<br/>"+deviceDTO.getImei()+"<br/>").append(",");
                       }
                   }

                }
            } catch (Exception e) {
                logg.info("参数异常:{}", e.getMessage());
            }
        }
        info.setOperationDescribe(builder.toString());
    }


}
