package com.ecell.internationalize.system.aspect;

import com.alibaba.fastjson2.JSON;
import com.ecell.internationalize.common.core.domain.UserLogin;
import com.ecell.internationalize.common.core.utils.StringUtils;
import com.ecell.internationalize.common.core.utils.ip.IpUtils;
import com.ecell.internationalize.common.core.utils.poi.ExcelUtil;
import com.ecell.internationalize.common.core.utils.uuid.UUID;
import com.ecell.internationalize.common.core.web.domain.AjaxResult;
import com.ecell.internationalize.common.security.utils.SecurityUtils;
import com.ecell.internationalize.system.annotation.InputLog;
import com.ecell.internationalize.system.constant.DeviceModelConstants;
import com.ecell.internationalize.system.entity.DeviceModelInfo;
import com.ecell.internationalize.system.entity.ImeiLogInfo;
import com.ecell.internationalize.system.entity.dto.DeviceDTO;
import com.ecell.internationalize.system.service.DeviceInfoService;
import com.ecell.internationalize.system.service.DeviceModelInfoService;
import com.ecell.internationalize.system.service.impl.ImeiLogInfoServiceImpl;
import com.ecell.internationalize.system.utils.DeviceModelInfoUtli;
import com.ecell.internationalize.system.utils.HttpServletUtil;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.Aspect;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.validation.BindingResult;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.*;

@Aspect
@Component
public class InputImeiLog {
    private static final Logger logg = LoggerFactory.getLogger(InputImeiLog.class);

    @Autowired
    private ImeiLogInfoServiceImpl imeiLogInfoService;
    @Autowired
    private DeviceModelInfoService deviceModelInfoService;
    @Autowired
    private DeviceInfoService deviceInfoService;


    /**
     * 处理完请求后执行
     *
     * @param joinPoint 切点
     */
    @AfterReturning(pointcut = "@annotation(controllerLog)", returning = "jsonResult")
    public void doAfterReturning(JoinPoint joinPoint, InputLog controllerLog, Object jsonResult)
    {
        handleLog(joinPoint, controllerLog, null, jsonResult);
    }

//    /**
//     * 拦截异常操作
//     *
//     * @param joinPoint 切点
//     * @param e 异常
//     */
//    @AfterThrowing(value = "@annotation(controllerLog)", throwing = "e")
//    public void doAfterThrowing(JoinPoint joinPoint, InputLog controllerLog, Exception e)
//    {
//        handleLog(joinPoint, controllerLog, e, null);
//    }

    protected void handleLog(final JoinPoint joinPoint, InputLog controllerLog, final Exception e, Object jsonResult)
    {
        try
        {
            // *========数据库日志=========*//
            ImeiLogInfo info =new ImeiLogInfo();
            info.setImeiLogId(UUID.fastUUID().toString(true));
            info.setCreateTime(new Date());
            // 请求的地址
            String ip = IpUtils.getIpAddr(HttpServletUtil.getRequest());
            info.setIp(ip);
            String username = SecurityUtils.getUsername();
            UserLogin userLogin = SecurityUtils.getUserLogin();
            if (StringUtils.isNotEmpty(userLogin.getRoles()) && userLogin.getRoles().size()>0){
                info.setUserRole(Arrays.toString(userLogin.getRoles().toArray(new String[0])));
            }else {
                info.setUserRole("");
            }

            if (StringUtils.isNotBlank(username))
            {
                info.setLoginAccount(username);
            }
//            logg.error("错误是:{}",jsonResult);
            // 处理设置注解上的参数
            getControllerMethodDescription(joinPoint, controllerLog, info, jsonResult);
            // 保存数据库
           //imeiLogInfoService.save(info);
            imeiLogInfoService.saveImeiLogInfo(info);

        }
        catch (Exception exp)
        {
            // 记录本地异常日志
            logg.error("==通知异常==");
            logg.error("异常信息:{}", exp.getMessage());

            exp.printStackTrace();
        }
    }

    /**
     * 获取注解中对方法的描述信息 用于Controller层注解
     *
     * @param log 日志
     * @param operLog 操作日志
     * @throws Exception
     */
    public void getControllerMethodDescription(JoinPoint joinPoint, InputLog log, ImeiLogInfo operLog, Object jsonResult) throws Exception
    {
            if (DeviceModelConstants.ZERO.equals(log.inputType())){
                //如果是单个录入
                argsArrayToString(joinPoint.getArgs(),operLog);
            }else {
                //如果是批量录入
                argsArrayTwoToString(joinPoint.getArgs(),operLog,jsonResult);

            }



    }




    /**
     * 获取请求的参数，放到log中
     *
     * @param operLog 操作日志
     * @throws Exception 异常
     */
//    private void setRequestValue(JoinPoint joinPoint, ImeiLogInfo operLog) throws Exception
//    {
//
//
//             argsArrayToString(joinPoint.getArgs(),operLog);
//           // operLog.setOperParam(StringUtils.substring(params, 0, 2000));
//
//    }

    /**
     * 参数拼装
     */
    public void argsArrayToString(Object[] paramsArray,ImeiLogInfo operLog)
    {
        if (paramsArray != null && paramsArray.length > 0)
        {
            for (Object o : paramsArray)
            {
                if (StringUtils.isNotNull(o) && !isFilterObject(o))
                {
                    try
                    {
                        if (o instanceof DeviceDTO){
                            Object jsonObj = JSON.toJSON(o);
                            DeviceDTO deviceDTO = JSON.parseObject(jsonObj.toString(), DeviceDTO.class);
                            //设备型号ID
                           operLog.setDeviceModelId(deviceDTO.getDeviceModelId());
                           //设备类型名称
                            operLog.setDeviceModelName(deviceDTO.getDeviceModelName());
                            //厂商ID
                            operLog.setFirmId(deviceDTO.getFirmId());
                            //根据设备型号ID查询型号功能说明
                            String s = assembleDescription(deviceDTO.getDeviceModelId(),DeviceModelConstants.ZERO,1,null);
                            //根据IMEI 查询批次号
                            String inputBatch = deviceInfoService.queryInputBatchById(deviceDTO.getImei());
                            operLog.setInputBatch(inputBatch);

                            //组装功能描述
                            operLog.setOperationDescribe(s);
                        }


                    }
                    catch (Exception e)
                    {
                        logg.info("参数异常:{}",e.getMessage());
                    }
                }
            }
        }

    }

    /**
     * 批量录入参数拼接
     * @param paramsArray
     * @param operLog
     */
   public void argsArrayTwoToString(Object[] paramsArray, ImeiLogInfo operLog,Object jsonResult) {
       List<DeviceDTO> userList = Collections.emptyList();
       DeviceDTO deviceDTO =null;
        if (paramsArray != null && paramsArray.length > 0)
        {
            for (Object o : paramsArray)
            {
                if (StringUtils.isNotNull(o) && !isFilterObject(o))
                {

                    try
                    {

                        //判断改参数是不是 MultipartFile实现类
                        if (o instanceof MultipartFile){ //StandardMultipartHttpServletRequest
//                            Object jsonObj = JSON.toJSON(o);
//                            MultipartFile file = JSON.parseObject(jsonObj.toString(), MultipartFile.class);
                            MultipartFile file =(MultipartFile) o;
                            ExcelUtil<DeviceDTO> util = new ExcelUtil<DeviceDTO>(DeviceDTO.class);
                            userList = util.importExcel(file.getInputStream());
                        }

                        if (o instanceof DeviceDTO){
//                            Object jsonObj = JSON.toJSON(o);
//                            deviceDTO = JSON.parseObject(jsonObj.toString(), DeviceDTO.class);
                             deviceDTO =(DeviceDTO)o;
                            //设备型号ID
                            operLog.setDeviceModelId(deviceDTO.getDeviceModelId());
                            //设备类型名称
                            operLog.setDeviceModelName(deviceDTO.getDeviceModelName());
                            //厂商ID
                            operLog.setFirmId(deviceDTO.getFirmId());


                        }

                        if (StringUtils.isNotEmpty(userList) && userList.size()>0 && StringUtils.isNotNull(deviceDTO)){
                            //根据IMEI获取批次号，这里取userList第一个IMEI号
                            //根据IMEI 查询批次号
                            String inputBatch = deviceInfoService.queryInputBatchById(userList.get(0).getImei());
                            operLog.setInputBatch(inputBatch);
                            //组装操作说明
                            String s = assembleDescription(deviceDTO.getDeviceModelId(), DeviceModelConstants.ONE, userList.size(),jsonResult);
                            operLog.setOperationDescribe(s);

                        }



                    }
                    catch (Exception e)
                    {
                        logg.info("参数异常:{}",e.getMessage());
                    }
                }
            }
        }


    }

    /**
     * 判断是否需要过滤的对象。
     *
     * @param o 对象信息。
     * @return 如果是需要过滤的对象，则返回true；否则返回false。
     */
    @SuppressWarnings("rawtypes")
    public boolean isFilterObject(final Object o)
    {
        Class<?> clazz = o.getClass();
        if (clazz.isArray())
        {
            return clazz.getComponentType().isAssignableFrom(MultipartFile.class);
        }
        else if (Collection.class.isAssignableFrom(clazz))
        {
            Collection collection = (Collection) o;
            for (Object value : collection)
            {
                return value instanceof MultipartFile;
            }
        }
        else if (Map.class.isAssignableFrom(clazz))
        {
            Map map = (Map) o;
            for (Object value : map.entrySet())
            {
                Map.Entry entry = (Map.Entry) value;
                return entry.getValue() instanceof MultipartFile;
            }
        }
        return   o instanceof HttpServletRequest || o instanceof HttpServletResponse
                || o instanceof BindingResult;
    }

    /**
     * 组装操作描述
     * @param id 设备型号id
     * @param type 0是单个录入 1是批次录入
     * * @return
     */
    public String assembleDescription(String id,String type,int size,Object jsonResult){
        StringBuffer buffer =new StringBuffer();
        DeviceModelInfo info = deviceModelInfoService.queryById(id);
        String s = DeviceModelInfoUtli.assembleFunDescription(info);
        if (DeviceModelConstants.ZERO.equals(type)) {
            buffer.append("成功录入1个").append(",").append(" 选择的功能是:").append(s);
        }else if (DeviceModelConstants.ONE.equals(type)){
            if (jsonResult instanceof AjaxResult){
                AjaxResult ajaxResult =(AjaxResult) jsonResult;
              buffer.append("总共录入的数据条数是:").append(size).append("<br/>").append(ajaxResult.get("data")).append(" 选择的功能是:").append(s);
            }else {
                buffer.append("总共录入的数据条数是:").append(size).append(" 选择的功能是:").append(s);
            }


        }

        return buffer.toString();
    }

}
