package com.ecell.internationalize.system.aspect;
import com.ecell.internationalize.common.core.exception.ServiceException;
import com.ecell.internationalize.common.core.utils.ServletUtils;
import com.ecell.internationalize.common.core.utils.StringUtils;
import com.ecell.internationalize.common.redis.service.RedisService;
import com.ecell.internationalize.system.annotation.ApiRateLimiter;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.Before;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.annotation.AnnotationUtils;
import org.springframework.data.redis.core.script.RedisScript;

import javax.servlet.http.HttpServletRequest;
import java.util.Collections;

//@Component
//@Aspect
public class RateLimitAspect {
    private static final Logger logger= LoggerFactory.getLogger(RateLimitAspect.class);

    @Autowired
    RedisService redisService;
    @Autowired
    RedisScript<Long> redisScript;

    @Pointcut("@annotation(com.ecell.internationalize.system.annotation.ApiRateLimiter)")
    public void pointCut(){}

    @Before("pointCut()")
    public void beforeRateLimit(JoinPoint jp){
        //获取注解上的值
        MethodSignature methodSignature = (MethodSignature) jp.getSignature();
        ApiRateLimiter rateLimiter = AnnotationUtils.findAnnotation(methodSignature.getMethod(), ApiRateLimiter.class);
        int time = rateLimiter.time();
        int count = rateLimiter.count();
        //构建redis中的key值
        String rateKey=getRateLimitKey(rateLimiter,methodSignature);
        logger.info("redis中key值：{}",rateKey);
        try {
            Long current = (Long) redisService.redisTemplate.execute(redisScript, Collections.singletonList(rateKey), time, count);
            if(current==null||current.intValue()>count){
                throw new ServiceException("messages.limit.max.error",408);
            }
        } catch (Exception e) {
            throw new ServiceException("messages.limit.max.error",408);
        }




    }
    private String getRateLimitKey(ApiRateLimiter rateLimiter, MethodSignature methodSignature) {
        StringBuffer key = new StringBuffer(rateLimiter.key());
        //获取厂商id 先写死
        String firmId="1111";
        HttpServletRequest request = ServletUtils.getRequest();
        if (StringUtils.isNotNull(request)){
           //获取路径
            String requestURI = request.getRequestURI();
            key.append(firmId).append(requestURI);
        }
        return key.toString();
    }
}
