package com.ecell.internationalize.system.config;
import com.ecell.internationalize.common.core.exception.ServiceException;
import com.ecell.internationalize.common.core.utils.locale.LocaleUtil;
import com.ecell.internationalize.common.core.web.domain.AjaxResult;
import com.ecell.internationalize.common.system.constant.ExceptionResultConstant;
import com.ecell.internationalize.common.system.constant.FieldConstant;
import com.ecell.internationalize.common.system.constant.ServiceExceptionResultConstant;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;

/**
 * 统一异常处理
 * @Title: ExceptionAdviceConfig
 * @Author: liy
 * @Date: 2022/7/8 13:47
 * @Version:1.0
 */

@RestControllerAdvice
public class ExceptionAdviceConfig {
    private static final Logger logger = LoggerFactory.getLogger(ExceptionAdviceConfig.class);
    /**
     * 接口入参校验异常处理
     * @Author: liy
     * @Date: 2022/7/8 13:53
     */
    @ExceptionHandler(MethodArgumentNotValidException.class)
    public AjaxResult methodArgumentNotValidExceptionHandler(MethodArgumentNotValidException e) {
        // 从异常对象中拿到ObjectError对象
        ObjectError objectError = e.getBindingResult().getAllErrors().get(0);
        logger.error("接口参数校验异常信息:{}",objectError.getDefaultMessage());
        // 校验结果进行转换调用国际化配置资源
        return AjaxResult.error(LocaleUtil.getMessage(ExceptionResultConstant.RESULT_MAP.get(objectError.getDefaultMessage())));
    }
    /**
     * 全局异常处理,配置国际化
     * @Author: liy
     * @Date: 2022/7/8 13:53
     */
    @ExceptionHandler(Exception.class)
    public AjaxResult exceptionHandler(Exception e) {
        e.printStackTrace();
        logger.error("异常错误信息:{}",e.getCause().getMessage());
        return AjaxResult.error( LocaleUtil.getMessage(FieldConstant.MESSAGES_ERROR));
    }


    /**
     * 全局异常处理,配置国际化
     * @Author: liy
     * @Date: 2022/7/8 13:53
     */
    @ExceptionHandler(ServiceException.class)
    public AjaxResult serviceException(ServiceException e) {
        e.printStackTrace();
        logger.error("异常错误信息:{}",e.getMessage());
        return AjaxResult.error(e.getCode(),LocaleUtil.getMessage(ServiceExceptionResultConstant.RESULT_MSG_MAP.get(e.getMessage())));
    }
}
