package com.ecell.internationalize.system.controller;
import com.ecell.internationalize.common.core.utils.locale.LocaleUtil;
import com.ecell.internationalize.common.core.utils.uuid.UUID;
import com.ecell.internationalize.common.core.web.domain.AjaxResult;
import com.ecell.internationalize.common.security.utils.SecurityUtils;
import com.ecell.internationalize.common.system.annotation.SystemLog;
import com.ecell.internationalize.common.system.constant.FieldConstant;
import com.ecell.internationalize.common.system.utlis.UploadUtil;
import com.ecell.internationalize.system.entity.ActivityBanner;
import com.ecell.internationalize.system.service.ActivityBannerService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.Date;
import java.util.Map;

/**
 * <p>
 * 活动banner图 前端控制器
 * </p>
 *
 * @author liy
 * @since 2022-07-13
 */
@Api(value="活动banner图",tags={"活动banner图接口"})
@RestController
@RequestMapping("/activity_banner")
public class ActivityBannerController {
    @Autowired
    private ActivityBannerService activityBannerService;
    /**
     * 活动banner图条件分页查询
     * @Author liy
     * @Date 2022/7/14 16:38
     * @param  map 分页条件查询体
     * @Return AjaxResult
     */
    @ApiOperation("条件分页查询活动banner图信息")
    @PostMapping("banner/list")
    public AjaxResult queryAllByPage(@RequestBody Map<String,Object> map){
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),activityBannerService.findAllByPage(map));
    }
    /**
     * 活动banner图发布
     * @Author liy
     * @Date 2022/7/14 16:38
     * @param  activityBanner 实体类
     * @Return AjaxResult
     */
    @ApiOperation("活动banner图发布/取消发布")
    @SystemLog(msg =FieldConstant.ACTIVITY_BANNER_BANNER,operation = FieldConstant.ACTIVITY_BANNER_PUBLISH)
    @PostMapping("banner/changeStatus")
    public AjaxResult changeStatusOrDelUser(@RequestBody @ApiParam(name="活动",value="传整个对象,修改其中status字段,（0：取消发布，1：发布）",required=true) ActivityBanner activityBanner){
        //1表示发布图片,新增默认0未发布
        if (FieldConstant.MATH_ONE.equals(activityBanner.getStatus())){
            activityBanner.setPublishUser("admin");
            activityBanner.setPublishTime(new Date());
        }else {
            activityBanner.setPublishUser(null);
            activityBanner.setPublishTime(null);
        }
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),activityBannerService.updateById(activityBanner));
    }
    /**
     * 活动banner图删除
     * @Author liy
     * @Date 2022/7/14 16:38
     * @param  activityBanner 实体类
     * @Return AjaxResult
     */
    @ApiOperation("活动banner图删除")
    @SystemLog(msg =FieldConstant.ACTIVITY_BANNER_BANNER,operation = FieldConstant.DELETE_OPERATOR)
    @PostMapping("banner/del")
    public AjaxResult del(@RequestBody @ApiParam(name="活动",value="传整个对象,修改其中delFlag字段,（0：删除，1：正常）",required=true) ActivityBanner activityBanner){
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),activityBannerService.updateById(activityBanner));
    }
    /**
     * 活动banner图上传/新增
     * @Author liy
     * @Date 2022/7/14 16:38
     * @param  file 文件
     * @Return AjaxResult
     */
    @ApiOperation("活动banner图新增上传")
    @SystemLog(msg =FieldConstant.ACTIVITY_BANNER_BANNER,operation = FieldConstant.SAVE_OPERATOR)
    @PostMapping("banner/save")
    public AjaxResult save(@RequestParam("file") MultipartFile file,@RequestParam("title") String title,@RequestParam("bannerOrder")Integer bannerOrder) throws IOException {
        String path = UploadUtil.upload(file);
        String id= UUID.randomUUID().toString(true);
        ActivityBanner activityBanner=new ActivityBanner();
        activityBanner.setId(id);
        activityBanner.setCreateUser(SecurityUtils.getUsername());
        activityBanner.setBannerOrder(bannerOrder);
        activityBanner.setTitle(title);
        activityBanner.setBanner(path);
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),activityBannerService.save(activityBanner));
    }
    /**
     * 活动banner图上传/修改
     * @Author liy
     * @Date 2022/7/14 16:38
     * @param  file 文件
     * @Return AjaxResult
     */
    @ApiOperation("活动banner图修改上传")
    @SystemLog(msg =FieldConstant.ACTIVITY_BANNER_BANNER,operation = FieldConstant.UPDATE_OPERATOR)
    @PostMapping("banner/update")
    public AjaxResult update(@RequestParam(value = "file",required = false) MultipartFile file,@RequestParam("id")String id, @RequestParam("title") String title,@RequestParam("bannerOrder")Integer bannerOrder) throws IOException {
        ActivityBanner activityBanner=new ActivityBanner();
        if (null!=file) {
            String path = UploadUtil.upload(file);
            activityBanner.setBanner(path);
        }
        activityBanner.setId(id);
        activityBanner.setBannerOrder(bannerOrder);
        activityBanner.setTitle(title);
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),activityBannerService.updateById(activityBanner));
    }

}

