package com.ecell.internationalize.system.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.ecell.internationalize.common.core.utils.locale.LocaleUtil;
import com.ecell.internationalize.common.core.utils.uuid.UUID;
import com.ecell.internationalize.common.core.web.domain.AjaxResult;
import com.ecell.internationalize.common.security.utils.SecurityUtils;
import com.ecell.internationalize.common.system.annotation.SystemLog;
import com.ecell.internationalize.common.system.constant.FieldConstant;
import com.ecell.internationalize.common.system.utlis.UploadUtil;
import com.ecell.internationalize.system.entity.Activity;
import com.ecell.internationalize.system.service.ActivityService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.math.BigDecimal;
import java.util.Date;
import java.util.Map;

/**
 * <p>
 * 活动 前端控制器
 * </p>
 *
 * @author liy
 * @since 2022-07-13
 */
@Api(value="活动",tags={"活动/往期活动接口"})
@RestController
@RequestMapping("/activity")
public class ActivityController {
    @Autowired
    private ActivityService activityService;
    /**
     * 活动条件分页查询
     * @Author liy
     * @Date 2022/7/14 16:38
     * @param  map 分页条件查询体
     * @Return AjaxResult
     */
    @ApiOperation("条件分页查询活动信息")
    @PostMapping("current/list")
    public AjaxResult queryAllByPage(@RequestBody Map<String,Object> map){
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),activityService.findAllByPage(map));
    }
    /**
     * 活动发布
     * @Author liy
     * @Date 2022/7/14 16:38
     * @param  activity 实体类
     * @Return AjaxResult
     */
    @ApiOperation("活动发布/取消发布")
    @SystemLog(msg =FieldConstant.ACTIVITY_BANNER,operation = FieldConstant.ACTIVITY_PUBLISH)
    @PostMapping("current/changeStatus")
    public AjaxResult changeStatusOrDelUser(@RequestBody @ApiParam(name="活动",value="传整个对象,修改其中status字段,（0：取消发布，1：发布）",required=true) Activity activity){
        if (FieldConstant.MATH_ONE.equals(activity.getStatus())){
            activity.setPublishUser("admin");
            activity.setPublishTime(new Date());
        }else {
            activity.setPublishUser(null);
            activity.setPublishTime(null);
        }
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),activityService.updateById(activity));
    }
    /**
     * 活动删除
     * @Author liy
     * @Date 2022/7/14 16:38
     * @param  activity 实体类
     * @Return AjaxResult
     */
    @ApiOperation("活动删除")
    @SystemLog(msg =FieldConstant.ACTIVITY_BANNER,operation = FieldConstant.DELETE_OPERATOR)
    @PostMapping("current/del")
    public AjaxResult del(@RequestBody @ApiParam(name="活动",value="传整个对象,修改其中delFlag字段,（0：删除，1：正常）",required=true) Activity activity){
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),activityService.updateById(activity));
    }

    /**
     * 活动上传/新增
     * @Author liy
     * @Date 2022/7/14 16:38
     * @param  file 文件
     * @Return AjaxResult
     */
    @ApiOperation("活动新增上传")
    @SystemLog(msg =FieldConstant.ACTIVITY_BANNER,operation = FieldConstant.SAVE_OPERATOR)
    @PostMapping("current/save")
    public AjaxResult save(@RequestParam("file") MultipartFile file, @RequestParam("activityTitle") String activityTitle,
                           @RequestParam("sellingPrice") BigDecimal sellingPrice, String priceRepresent, @RequestParam("activityCategory")String activityCategory,
                           @RequestParam("activityAge")String activityAge, @RequestParam("activityAreas")String activityAreas, @RequestParam("activityQuota")String activityQuota,
                           Date expiryDate) throws IOException {
        String path = UploadUtil.upload(file);
        String id= UUID.randomUUID().toString(true);
        Activity activity=new Activity();
        activity.setId(id);
        activity.setActivityTitle(activityTitle);
        activity.setSellingPrice(sellingPrice);
        activity.setPriceRepresent(priceRepresent);
        activity.setActivityCategory(activityCategory);
        activity.setActivityAge(activityAge);
        activity.setActivityAreas(activityAreas);
        activity.setActivityQuota(activityQuota);
        activity.setExpiryDate(expiryDate);
        activity.setCreateUser(SecurityUtils.getUsername());
        activity.setBannerImage(path);
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),activityService.save(activity));
    }
    /**
     * 活动上传/修改
     * @Author liy
     * @Date 2022/7/14 16:38
     * @param  file 文件
     * @Return AjaxResult
     */
    @ApiOperation("活动修改上传")
    @SystemLog(msg =FieldConstant.ACTIVITY_BANNER,operation = FieldConstant.UPDATE_OPERATOR)
    @PostMapping("current/update")
    public AjaxResult update(@RequestParam(value = "file",required = false) MultipartFile file,@RequestParam("id")String id, @RequestParam("activityTitle") String activityTitle,
                           @RequestParam("sellingPrice") BigDecimal sellingPrice, String priceRepresent, @RequestParam("activityCategory")String activityCategory,
                           @RequestParam("activityAge")String activityAge, @RequestParam("activityAreas")String activityAreas, @RequestParam("activityQuota")String activityQuota,
                           @RequestParam("expiryDate")Date expiryDate) throws IOException {
        Activity activity=new Activity();
        if (null!=file) {
           String path = UploadUtil.upload(file);
            activity.setBannerImage(path);
       }
        activity.setId(id);
        activity.setActivityTitle(activityTitle);
        activity.setSellingPrice(sellingPrice);
        activity.setPriceRepresent(priceRepresent);
        activity.setActivityCategory(activityCategory);
        activity.setActivityAge(activityAge);
        activity.setActivityAreas(activityAreas);
        activity.setActivityQuota(activityQuota);
        activity.setExpiryDate(expiryDate);
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),activityService.updateById(activity));
    }
    /**
     * 往期活动条件分页查询
     * @Author liy
     * @Date 2022/7/14 16:38
     * @param  map 分页条件查询体
     * @Return AjaxResult
     */
    @ApiOperation("往期活动条件分页查询")
    @PostMapping("last/list")
    public AjaxResult queryLastListByPage(@RequestBody Map<String,Object> map){
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),activityService.queryLastListByPage(map));
    }
    /**
     * 搜索条件查询接口
     * @Author liy
     * @Date 2022/7/14 16:38
     * @param  type 实体类
     * @Return AjaxResult
     */
    @ApiOperation("搜索条件查询")
    @GetMapping("current/query")
    public AjaxResult del(@RequestParam @ApiParam(name="搜索类型",value="传type字段,（1：所属类别，2：年龄阶段，3： 活动区域）",required=true) String type){
        QueryWrapper<Activity> queryWrapper = new QueryWrapper<>();
        String value = FieldConstant.TYPE_MAP.get(type);
        queryWrapper.select("DISTINCT "+value).eq("del_flag", FieldConstant.MATH_ONE);
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),activityService.getBaseMapper().selectList(queryWrapper));
    }
}

