package com.ecell.internationalize.system.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ecell.internationalize.common.core.utils.locale.LocaleUtil;
import com.ecell.internationalize.common.core.utils.uuid.UUID;
import com.ecell.internationalize.common.core.web.domain.AjaxResult;
import com.ecell.internationalize.common.security.utils.SecurityUtils;
import com.ecell.internationalize.common.system.annotation.SystemLog;
import com.ecell.internationalize.common.system.constant.FieldConstant;
import com.ecell.internationalize.system.entity.ActivitySearch;
import com.ecell.internationalize.system.service.ActivitySearchService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;

/**
 * <p>
 * 热门搜索 前端控制器
 * </p>
 *
 * @author liy
 * @since 2022-07-13
 */
@Api(value="热门搜索",tags={"热门搜索接口"})
@RestController
@RequestMapping("/activity_search")
public class ActivitySearchController {
    @Autowired
    private ActivitySearchService activitySearchService;
    /**
     * 热门搜索分页查询
     * @Author liy
     * @Date 2022/7/14 10:38
     * @param  page 分页
     * @Return AjaxResult
     */
    @ApiOperation("条件分页查询热门搜索信息")
    @PostMapping("search/list")
    public AjaxResult queryAll(@RequestBody Page<ActivitySearch> page){
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),activitySearchService.findAllByPage(page));
    }
    /**
     * 新增热门搜索
     * @Author liy
     * @Date 2022/7/14 10:38
     * @param activitySearch 实体
     * @Return AjaxResult
     */
    @ApiOperation("新增热门搜索")
    @SystemLog(msg =FieldConstant.HOT_SEARCH,operation = FieldConstant.SAVE_OPERATOR)
    @PostMapping("search/save")
    public AjaxResult saveSearch(@RequestBody @Valid  ActivitySearch activitySearch){
        String id= UUID.randomUUID().toString(true);
        activitySearch.setId(id);
        activitySearch.setCreateUser(SecurityUtils.getUsername());
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),activitySearchService.save(activitySearch));
    }
    /**
     * 修改热门搜索
     * @Author liy
     * @Date 2022/7/14 10:38
     * @param activitySearch 实体
     * @Return AjaxResult
     */
    @ApiOperation("修改热门搜索")
    @SystemLog(msg =FieldConstant.HOT_SEARCH,operation = FieldConstant.UPDATE_OPERATOR)
    @PostMapping("search/update")
    public AjaxResult updateSearch(@RequestBody @Valid ActivitySearch activitySearch){
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),activitySearchService.updateById(activitySearch));
    }
    /**
     * 删除热门搜索
     * @Author liy
     * @Date 2022/7/14 10:38
     * @param activitySearch 实体
     * @Return AjaxResult
     */
    @ApiOperation("删除热门搜索")
    @SystemLog(msg =FieldConstant.HOT_SEARCH,operation = FieldConstant.DELETE_OPERATOR)
    @PostMapping("search/del")
    public AjaxResult delCategory(@RequestBody @ApiParam(value="传整个对象,修改其中delFlag字段,（0：删除，1：正常）",required=true) ActivitySearch activitySearch){
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),activitySearchService.updateById(activitySearch));
    }
}

