package com.ecell.internationalize.system.controller;
import com.ecell.internationalize.common.core.utils.locale.LocaleUtil;
import com.ecell.internationalize.common.core.utils.uuid.UUID;
import com.ecell.internationalize.common.core.web.domain.AjaxResult;
import com.ecell.internationalize.common.security.utils.SecurityUtils;
import com.ecell.internationalize.common.system.annotation.SystemLog;
import com.ecell.internationalize.common.system.constant.FieldConstant;
import com.ecell.internationalize.system.entity.Advert;
import com.ecell.internationalize.system.service.AdvertService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.Map;

/**
 * <p>
 * 广告 前端控制器
 * </p>
 *
 * @author liy
 * @since 2022-07-14
 */
@Api(value="广告配置",tags={"广告配置接口"})
@RestController
@RequestMapping("/advert")
public class AdvertController {
    @Autowired
    private AdvertService advertService;
    /**
     * 广告配置分页查询
     * @Author liy
     * @Date 2022/7/15 16:38
     * @param  map 分页条件查询体
     * @Return AjaxResult
     */
    @ApiOperation("条件分页查询广告配置信息")
    @PostMapping("config/list")
    public AjaxResult queryAllByPage(@RequestBody Map<String,Object> map){
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),advertService.findAllByPage(map));
    }
    /**
     * 广告配置删除
     * @Author liy
     * @Date 2022/7/15 16:38
     * @param  advert 实体类
     * @Return AjaxResult
     */
    @ApiOperation("广告配置删除")
    @SystemLog(msg =FieldConstant.ADVERT_MANAGE,operation = FieldConstant.DELETE_OPERATOR)
    @PostMapping("config/del")
    public AjaxResult changeStatusOrDelUser(@RequestBody @ApiParam(value="传整个对象,修改其中delFlag字段,（0：删除，1：正常）",required=true) Advert advert){
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),advertService.updateById(advert));
    }
    /**
     * 新增广告配置
     * @Author liy
     * @Date 2022/7/15 17:38
     * @param advert 类别实体
     * @Return AjaxResult
     */
    @ApiOperation("新增广告配置")
    @SystemLog(msg =FieldConstant.ADVERT_MANAGE,operation = FieldConstant.SAVE_OPERATOR)
    @PostMapping("config/save")
    public AjaxResult save(@RequestBody @Valid Advert advert){
        String id= UUID.randomUUID().toString(true);
        advert.setId(id);
        advert.setCreateUser(SecurityUtils.getUsername());
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),advertService.save(advert));
    }
    /**
     * 修改广告配置
     * @Author liy
     * @Date 2022/7/15 17:38
     * @param advert 类别实体
     * @Return AjaxResult
     */
    @ApiOperation("修改广告配置")
    @SystemLog(msg =FieldConstant.ADVERT_MANAGE,operation = FieldConstant.UPDATE_OPERATOR)
    @PostMapping("config/update")
    public AjaxResult update(@RequestBody @Valid Advert advert){
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),advertService.updateById(advert));
    }
}

