package com.ecell.internationalize.system.controller;
import com.ecell.internationalize.common.core.utils.locale.LocaleUtil;
import com.ecell.internationalize.common.core.utils.uuid.UUID;
import com.ecell.internationalize.common.core.web.domain.AjaxResult;
import com.ecell.internationalize.common.security.utils.SecurityUtils;
import com.ecell.internationalize.common.system.annotation.SystemLog;
import com.ecell.internationalize.common.system.constant.FieldConstant;
import com.ecell.internationalize.common.system.utlis.UploadUtil;
import com.ecell.internationalize.system.entity.AdvertRoll;
import com.ecell.internationalize.system.service.AdvertRollService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.Map;

/**
 * <p>
 * 文字滚动广告列表 前端控制器
 * </p>
 *
 * @author liy
 * @since 2022-07-14
 */
@Api(value="文字滚动广告列表",tags={"文字滚动广告列表接口"})
@RestController
@RequestMapping("/advert_roll")
public class AdvertRollController {
    @Autowired
    private AdvertRollService advertRollService;
    /**
     * 文字滚动广告条件分页查询
     * @Author liy
     * @Date 2022/7/15 16:38
     * @param  map 分页条件查询体
     * @Return AjaxResult
     */
    @ApiOperation("条件分页查询文字滚动广告信息")
    @PostMapping("roll/list")
    public AjaxResult queryAllByPage(@RequestBody Map<String,Object> map){
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),advertRollService.findAllByPage(map));
    }
    /**
     * 文字滚动广告删除
     * @Author liy
     * @Date 2022/7/15 16:38
     * @param  advertRoll 实体类
     * @Return AjaxResult
     */
    @ApiOperation("文字滚动广告删除")
    @SystemLog(msg =FieldConstant.WORD_ROLL_ADVERT,operation = FieldConstant.DELETE_OPERATOR)
    @PostMapping("roll/del")
    public AjaxResult changeStatusOrDelUser(@RequestBody @ApiParam(value="传整个对象,修改其中delFlag字段,（0：删除，1：正常）",required=true) AdvertRoll advertRoll){
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),advertRollService.updateById(advertRoll));
    }
    /**
     * 文字滚动广告上传/新增
     * @Author liy
     * @Date 2022/7/15 16:38
     * @param  file 文件
     * @Return AjaxResult
     */
    @ApiOperation("文字滚动广告新增上传")
    @SystemLog(msg =FieldConstant.WORD_ROLL_ADVERT,operation = FieldConstant.SAVE_OPERATOR)
    @PostMapping("roll/save")
    public AjaxResult save(@RequestParam("file") MultipartFile file, @RequestParam("title") String title) throws IOException {
        String path = UploadUtil.upload(file);
        String id= UUID.randomUUID().toString(true);
        AdvertRoll advertRoll=new AdvertRoll();
        advertRoll.setId(id);
        advertRoll.setCreateUser(SecurityUtils.getUsername());
        advertRoll.setTitle(title);
        advertRoll.setIconImage(path);
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),advertRollService.save(advertRoll));
    }
    /**
     * 文字滚动广告上传/修改
     * @Author liy
     * @Date 2022/7/15 16:38
     * @param  file 文件
     * @Return AjaxResult
     */
    @ApiOperation("文字滚动广告修改上传")
    @SystemLog(msg =FieldConstant.WORD_ROLL_ADVERT,operation = FieldConstant.UPDATE_OPERATOR)
    @PostMapping("roll/update")
    public AjaxResult update(@RequestParam(value = "file",required = false) MultipartFile file, @RequestParam("id")String id, @RequestParam("title") String title) throws IOException {
        AdvertRoll advertRoll=new AdvertRoll();
        if (null!=file) {
            String path = UploadUtil.upload(file);
            advertRoll.setIconImage(path);
        }
        advertRoll.setId(id);
        advertRoll.setTitle(title);
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),advertRollService.updateById(advertRoll));
    }
}

