package com.ecell.internationalize.system.controller;
import com.ecell.internationalize.common.core.utils.locale.LocaleUtil;
import com.ecell.internationalize.common.core.utils.uuid.UUID;
import com.ecell.internationalize.common.core.web.domain.AjaxResult;
import com.ecell.internationalize.common.security.utils.SecurityUtils;
import com.ecell.internationalize.common.system.annotation.SystemLog;
import com.ecell.internationalize.common.system.constant.FieldConstant;
import com.ecell.internationalize.system.entity.Channel;
import com.ecell.internationalize.system.service.ChannelService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.Map;

/**
 * <p>
 * 渠道管理 前端控制器
 * </p>
 *
 * @author liy
 * @since 2022-07-14
 */
@Api(value="渠道管理",tags={"渠道管理接口"})
@RestController
@RequestMapping("/channel")
public class ChannelController {
    @Autowired
    private ChannelService channelService;
    /**
     * 渠道管理分页查询
     * @Author liy
     * @Date 2022/7/15 16:38
     * @param  map 分页条件查询体
     * @Return AjaxResult
     */
    @ApiOperation("条件分页查询渠道管理信息")
    @PostMapping("manage/list")
    public AjaxResult queryAllByPage(@RequestBody Map<String,Object> map){
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),channelService.findAllByPage(map));
    }
    /**
     * 渠道管理删除
     * @Author liy
     * @Date 2022/7/15 16:38
     * @param  channel 实体类
     * @Return AjaxResult
     */
    @ApiOperation("渠道管理删除")
    @SystemLog(msg =FieldConstant.CHANNEL_MANAGE,operation = FieldConstant.DELETE_OPERATOR)
    @PostMapping("manage/del")
    public AjaxResult changeStatusOrDelUser(@RequestBody @ApiParam(value="传整个对象,修改其中delFlag字段,（0：删除，1：正常）",required=true) Channel channel){
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),channelService.updateById(channel));
    }
    /**
     * 渠道管理新增
     * @Author liy
     * @Date 2022/7/15 17:38
     * @param channel 类别实体
     * @Return AjaxResult
     */
    @ApiOperation("渠道管理新增")
    @SystemLog(msg =FieldConstant.CHANNEL_MANAGE,operation = FieldConstant.SAVE_OPERATOR)
    @PostMapping("manage/save")
    public AjaxResult save(@RequestBody @Valid Channel channel){
        String id= UUID.randomUUID().toString(true);
        channel.setId(id);
        channel.setCreateUser(SecurityUtils.getUsername());
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),channelService.save(channel));
    }
    /**
     * 渠道管理修改
     * @Author liy
     * @Date 2022/7/15 17:38
     * @param channel 类别实体
     * @Return AjaxResult
     */
    @ApiOperation("渠道管理修改")
    @SystemLog(msg =FieldConstant.CHANNEL_MANAGE,operation = FieldConstant.UPDATE_OPERATOR)
    @PostMapping("manage/update")
    public AjaxResult update(@RequestBody @Valid Channel channel){
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),channelService.updateById(channel));
    }
}

