package com.ecell.internationalize.system.controller;


import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.ecell.internationalize.common.core.entity.Rest;
import com.ecell.internationalize.common.core.utils.locale.LocaleUtil;
import com.ecell.internationalize.common.core.utils.uuid.UUID;
import com.ecell.internationalize.common.core.web.domain.AjaxResult;
import com.ecell.internationalize.common.security.utils.SecurityUtils;
import com.ecell.internationalize.common.system.annotation.SystemLog;
import com.ecell.internationalize.common.system.constant.FieldConstant;
import com.ecell.internationalize.common.system.entity.CheckVersionUpload;
import com.ecell.internationalize.common.system.utlis.UploadUtil;
import com.ecell.internationalize.system.entity.CheckVersion;
import com.ecell.internationalize.system.entity.dto.CheckVersionDto;
import com.ecell.internationalize.system.service.CheckVersionService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 检查版本更新表 前端控制器
 * </p>
 *
 * @author ${author}
 * @since 2022-10-27
 */
@Api(value="检查版本更新",tags={"检查版本更新接口"})
@RestController
@RequestMapping("/check_version")
public class CheckVersionController {
    @Autowired
    private CheckVersionService checkVersionService;
    @ApiOperation("条件分页查询版本更新信息")
    @PostMapping("version/list")
    public Rest<IPage<CheckVersion>> queryAllVersion(@RequestBody CheckVersionDto checkVersionDto){
        return Rest.ok(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),checkVersionService.findAllByPage(checkVersionDto));
    }
    /**
     * 版本更新新增
     * @Author liy
     * @Date 2022/7/14 16:38
     * @Return AjaxResult
     */
    @ApiOperation("版本更新新增")
    @SystemLog(msg =FieldConstant.VERSION_UPDATE,operation = FieldConstant.SAVE_OPERATOR)
    @PostMapping("version/save")
    public Rest versionSave(@RequestBody CheckVersion checkVersion){
        LambdaQueryWrapper<CheckVersion> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(CheckVersion::getStatus,checkVersion.getStatus());
        lambdaQueryWrapper.eq(CheckVersion::getAppPackage,checkVersion.getAppPackage());
        lambdaQueryWrapper.eq(CheckVersion::getVersionNum,checkVersion.getVersionNum());
        CheckVersion version = checkVersionService.getBaseMapper().selectOne(lambdaQueryWrapper);
        if (null!=version){
            return Rest.fail(LocaleUtil.getMessage(FieldConstant.VERSION_HAS_EXIT));
        }
        String id= UUID.randomUUID().toString(true);
        checkVersion.setId(id);
        checkVersion.setCreateUser(SecurityUtils.getUsername());
        checkVersion.setCreateTime(new Date());
        checkVersionService.save(checkVersion);
        return Rest.ok(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS));
    }
    /**
     * 版本更新修改
     * @Author liy
     * @Date 2022/7/14 16:38
     * @Return AjaxResult
     */
    @ApiOperation("版本更新修改")
    @SystemLog(msg =FieldConstant.VERSION_UPDATE,operation = FieldConstant.UPDATE_OPERATOR)
    @PostMapping("version/update")
    public Rest versionUpdate(@RequestBody CheckVersion checkVersion) {

        LambdaQueryWrapper<CheckVersion> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(CheckVersion::getStatus,checkVersion.getStatus());
        lambdaQueryWrapper.eq(CheckVersion::getAppPackage,checkVersion.getAppPackage());
        lambdaQueryWrapper.eq(CheckVersion::getVersionNum,checkVersion.getVersionNum());
        CheckVersion version = checkVersionService.getBaseMapper().selectOne(lambdaQueryWrapper);
        if (null!=version && !version.getId().equals(checkVersion.getId())){
            return Rest.fail(LocaleUtil.getMessage(FieldConstant.VERSION_HAS_EXIT));
        }
        checkVersion.setUpdateTime(new Date());
        checkVersion.setUpdateUser(SecurityUtils.getUsername());
        checkVersionService.updateById(checkVersion);
        return Rest.ok(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS));
    }

    /**
     * 版本更新删除
     * @Author liy
     * @Date 2022/7/15 16:38
     * @Return AjaxResult
     */
    @ApiOperation("版本更新删除")
    @SystemLog(msg =FieldConstant.VERSION_UPDATE,operation = FieldConstant.DELETE_OPERATOR)
    @PostMapping("version/del")
    public Rest versionDel(@RequestBody List<String> ids){
        checkVersionService.getBaseMapper().deleteBatchIds(ids);
        return Rest.ok(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS));
    }

    /**
     * APK文件上传
     * @Author liy
     * @Date 2022/7/14 16:38
     * @Return AjaxResult
     */
    @ApiOperation("APK文件上传返回AppUrl和APK名称")
    @PostMapping("version/upload")
    public AjaxResult versionUpload(@RequestParam(value = "file") MultipartFile file) throws IOException {
        CheckVersionUpload checkVersionUpload = UploadUtil.versionUpload(file);
        if (!"apk".equals(checkVersionUpload.getGetSuffix())){
            return AjaxResult.error(LocaleUtil.getMessage(FieldConstant.VERSION_APK_UPLOAD));
        }
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),checkVersionUpload);
    }
}

