package com.ecell.internationalize.system.controller;
import com.ecell.internationalize.common.core.context.SecurityContextHolder;
import com.ecell.internationalize.common.core.exception.ServiceException;
import com.ecell.internationalize.common.core.utils.StringUtils;
import com.ecell.internationalize.common.core.utils.locale.LocaleUtil;
import com.ecell.internationalize.common.core.utils.poi.ExcelUtil;
import com.ecell.internationalize.common.core.web.domain.AjaxResult;
import com.ecell.internationalize.common.security.annotation.RequiresPermissions;
import com.ecell.internationalize.common.security.utils.SecurityUtils;
import com.ecell.internationalize.common.system.constant.FieldConstant;
import com.ecell.internationalize.system.annotation.DelImeiLog;
import com.ecell.internationalize.system.annotation.FactoryLog;
import com.ecell.internationalize.system.annotation.InputLog;
import com.ecell.internationalize.system.constant.DeviceModelConstants;
import com.ecell.internationalize.system.entity.DeviceInfo;
import com.ecell.internationalize.system.entity.dto.*;
import com.ecell.internationalize.system.service.DeviceInfoService;
import io.swagger.annotations.Api;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;


/**
 * <p>
 * 设备信息表 前端控制器
 * </p>
 *
 * @author ${author}
 * @since 2022-07-10
 */
@Api(value="设备管理",tags={"设备管理接口"})
@RestController
@RequestMapping("/device-info")
public class DeviceInfoController {
    @Autowired
    private DeviceInfoService deviceInfoService;

    /**
     * 设备单个录入
     * @return
     */
    @InputLog(inputType = DeviceModelConstants.ZERO) //如果是单个录入
    @PostMapping("/input/imei")
   public AjaxResult singleEntry(@RequestBody DeviceDTO dto){
        return   deviceInfoService.singleEntry(dto) ? AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS)) : AjaxResult.error(LocaleUtil.getMessage(FieldConstant.MESSAGES_ERROR));
   }

    /**
     * 设备批量录入
     * @return
     */
    @InputLog(inputType = DeviceModelConstants.ONE) //如果是批量录入
    @PostMapping("/import/imei")
    public AjaxResult importImei(MultipartFile file, DeviceDTO dto)  throws Exception{
        if (!StringUtils.isNotEmpty(dto.getDeviceModelId())){
            throw  new ServiceException("messages.parameter.error");
        }
        if (!StringUtils.isNotEmpty(dto.getFirmId())){
            throw  new ServiceException("messages.parameter.error");
        }

        ExcelUtil<DeviceDTO> util = new ExcelUtil<DeviceDTO>(DeviceDTO.class);
        if (StringUtils.isNull(file)){
            throw  new ServiceException("接收到的 MultipartFile 为空，请重试");
        }
        List<DeviceDTO> imeiList = util.importExcel(file.getInputStream());


        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS), deviceInfoService.importImei(imeiList,dto));


    }

    /**
     * 设备批量录入
     * @return
     */
    @InputLog(inputType = DeviceModelConstants.ONE) //如果是批量录入
    @PostMapping("/import/imei/two")
    public AjaxResult importImeiTwo(MultipartFile file, DeviceDTO dto)  throws Exception{
        if (!StringUtils.isNotEmpty(dto.getDeviceModelId())){
            throw  new ServiceException("messages.parameter.error");
        }
        if (!StringUtils.isNotEmpty(dto.getFirmId())){
            throw  new ServiceException("messages.parameter.error");
        }

        ExcelUtil<DeviceDTO> util = new ExcelUtil<DeviceDTO>(DeviceDTO.class);
        if (StringUtils.isNull(file)){
            throw  new ServiceException("接收到的 MultipartFile 为空，请重试");
        }
        List<DeviceDTO> imeiList = util.importExcel(file.getInputStream());


        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS), deviceInfoService.importImeiTwo(file,imeiList,dto));


    }



    /**
     * 设备分页查询
     * @return
     */
    @GetMapping("/page/list")
    public AjaxResult queryByPageList(DevicePageDTO dto) {

        return  AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS), deviceInfoService.queryByPageList(dto));

    }


    /**
     * 设备审核
     * @return
     */
    @RequiresPermissions(value="system:equipmentexamination:list")
    @GetMapping("/update/status/{id}/{status}")
    public AjaxResult updateStatus(@PathVariable String id,@PathVariable String status){

        return deviceInfoService.updateStatus(id,status) ? AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS)) : AjaxResult.error(LocaleUtil.getMessage(FieldConstant.MESSAGES_ERROR));

    }


    /**
     * 设备批次审核
     * @return
     */
    @RequiresPermissions(value="system:equipmentexamination:list")
    @GetMapping("/update/inputBach/status/{inputBach}/{status}")
    public AjaxResult updateInputBachStatus(@PathVariable String inputBach,@PathVariable String status){

        return deviceInfoService.updateInputBachStatus(inputBach,status) ? AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS)) : AjaxResult.error(LocaleUtil.getMessage(FieldConstant.MESSAGES_ERROR));

    }

    /**
     * 根据批次号分页查询
     * @param dto
     * @return
     */
    @GetMapping("/query/inputBach/List")
    public AjaxResult queryInputBatchList(DevicePageDTO dto){

        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),deviceInfoService.queryInputBatchList(dto));

    }


    /**
     * 批次初始化
     * @return
     */
    @GetMapping("/update/initInputBach")
    public AjaxResult initInputBach(){

        return AjaxResult.success(deviceInfoService.initInputBach());

    }




    /**
     * 分页查询设备激活信息
     * @return
     */
    @PostMapping("/query/list")
    public AjaxResult queryActivationInfoPageList(@RequestBody DeviceActivaPageDTO dto) {

        return  AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS), deviceInfoService.queryActivationInfoPageList(dto));


    }



    /**
     * 根据设备Imei批量删除设备
     * @return
     */
    @DelImeiLog(delType = DeviceModelConstants.ZERO)
    @DeleteMapping("/del/list")
    public AjaxResult delDeviceByIds(@RequestBody List<String> imeis) {

            if (!SecurityUtils.isAdminString(SecurityContextHolder.getStringUserId()) && imeis.size()>1){
                // 不是管理员只能删除单个设备
                throw new ServiceException("messages.delete.device.sing.error");
            }
        return deviceInfoService.delDeviceByIds(imeis) ? AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS)) : AjaxResult.error(LocaleUtil.getMessage(FieldConstant.MESSAGES_ERROR));


    }



    /**
     * 根据设备IMEI批量删除设备(导入删除)
     * @return
     */
    @DelImeiLog(delType = DeviceModelConstants.ONE)
    @DeleteMapping("/del/imei")
    public AjaxResult delDeviceByImei(MultipartFile file) throws Exception {
        ExcelUtil<DeviceDTO> util = new ExcelUtil<DeviceDTO>(DeviceDTO.class);
        if (StringUtils.isNull(file)){
            throw  new ServiceException("接收到的 MultipartFile 为空，请重试");
        }
        List<DeviceDTO> imeiList = util.importExcel(file.getInputStream());
        if (StringUtils.isEmpty(imeiList) || imeiList.size()<=0){
            throw  new ServiceException("接收到的 参数 为空，请重试");
        }
        List<String> imeis = imeiList.parallelStream().filter(s->StringUtils.isNotEmpty(s.getImei())).map(DeviceDTO::getImei).collect(Collectors.toList());

        return deviceInfoService.delDeviceByImei(imeis,DeviceModelConstants.ZERO) ? AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS)) : AjaxResult.error(LocaleUtil.getMessage(FieldConstant.MESSAGES_ERROR));

    }


    /**
     * 根据设备批次删除设备
     * @return
     */
    @DelImeiLog(delType = DeviceModelConstants.ZERO)
    @DeleteMapping("/del/inputBach/list/{inputBach}")
    public AjaxResult delDeviceInputBach(@PathVariable String inputBach) {
        //根据批次号查询设备
        List<DeviceInfo> infos = deviceInfoService.queryInputBatchListTwo(inputBach);
        List<String> imeiList=new ArrayList<>();
        if (StringUtils.isNotEmpty(infos) && infos.size()>0){
             imeiList = infos.stream().map(DeviceInfo::getImei).collect(Collectors.toList());
        }

        if (!SecurityUtils.isAdminString(SecurityContextHolder.getStringUserId()) && imeiList.size()>1){
            // 不是管理员只能删除单个设备
            throw new ServiceException("messages.delete.device.sing.error");
        }
        return deviceInfoService.delDeviceByIds(imeiList) ? AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS)) : AjaxResult.error(LocaleUtil.getMessage(FieldConstant.MESSAGES_ERROR));


    }

    /**
     * 单个设备Imei解绑
     * @return
     */
    @FactoryLog
    @GetMapping("/device/unBing/{imei}")
    public AjaxResult deviceUnBingByImei(@PathVariable String imei) {
        return deviceInfoService.deviceUnBingByImei(imei,DeviceModelConstants.ONE) ? AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS)) : AjaxResult.error(LocaleUtil.getMessage(FieldConstant.MESSAGES_ERROR));


    }

    /**
     * 批量设备Imei解绑
     * @return
     */
    @FactoryLog(unBingType = DeviceModelConstants.ONE)
    @PostMapping("/device/unBing/list")
    public AjaxResult deviceUnBingListByImei(MultipartFile file) throws Exception {
        ExcelUtil<DeviceDTO> util = new ExcelUtil<DeviceDTO>(DeviceDTO.class);
        if (StringUtils.isNull(file)){
            throw  new ServiceException("接收到的 MultipartFile 为空，请重试");
        }
        List<DeviceDTO> imeiList = util.importExcel(file.getInputStream());
        if (StringUtils.isEmpty(imeiList) || imeiList.size()<=0){
            throw  new ServiceException("接收到的 参数 为空，请重试");
        }
        List<String> list = imeiList.parallelStream().filter(s->StringUtils.isNotEmpty(s.getImei())).map(DeviceDTO::getImei).collect(Collectors.toList());
        return deviceInfoService.delDeviceByImei(list,DeviceModelConstants.ONE) ? AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS)) : AjaxResult.error(LocaleUtil.getMessage(FieldConstant.MESSAGES_ERROR));


    }





    /**
     * 查询所有审核通过未删除的设备
     * @return
     */
    @GetMapping("/query/device/approve")
    public AjaxResult queryApprovedDevice(){
        return  AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS), deviceInfoService.queryApprovedDevice());

    }

    /**
     * 根据imei 查询设备信息 对APP使用
     * @return
     */
    @GetMapping("/query/deviceInfo/{imei}")
    public DeviceInfo queryDeviceInfoByImei(@PathVariable String imei){
        return  deviceInfoService.queryDeviceInfoByImei(imei);

    }



    /**
     * 根据imei和日期查询轨迹信息
     * @return
     */
    @PostMapping("/query/track")
    public AjaxResult queryTrackInfo(@RequestBody TrackDTO dto){
        return AjaxResult.success(deviceInfoService.queryTrackInfo(dto));

    }


    /**
     * 远程控制
     * @return
     */
    @PostMapping("/query/remote")
    public AjaxResult queryRemoteControl(@RequestBody RemoteControlDTO dto){
        return AjaxResult.success(deviceInfoService.queryRemoteControl(dto));

    }

        /**
         * 远程控制指令下发
         * @return
         */
        @PostMapping("/send/instructions")
        public AjaxResult sendInstructions(@RequestBody sendInstructionDTO dto){
            return AjaxResult.success(deviceInfoService.sendInstructions(dto));

        }

    /**
     * 查询设备功能信息
     * @param dto
     * @return
     */

    @PostMapping("/query/equipmentFunction")
    public AjaxResult queryEquipmentFunction(@RequestBody RemoteControlDTO dto){
        return AjaxResult.success(deviceInfoService.queryEquipmentFunction(dto));

    }

    /**
     * 设备使用状况统计
     * @return
     */
    @GetMapping("/query/deviceNumber")
    public AjaxResult queryDeviceNumberInfo(){
        return AjaxResult.success(deviceInfoService.queryDeviceNumberInfo());

    }




    //    /**
//     * 根据设备imei批量删除设备
//     * @return
//     */
//    @DelImeiLog(delType = DeviceModelConstants.ZERO)
//    @DeleteMapping("/del/list")
//    public AjaxResult delDeviceByImei(@RequestBody List<String> imeis) {
//
//        return deviceInfoService.delDeviceByIds(ids) ? success(I18nUtil.getMessage(FieldConstant.MESSAGES_SUCCESS)) : error(I18nUtil.getMessage(FieldConstant.MESSAGES_ERROR));
//
//
//    }

}

