package com.ecell.internationalize.system.controller;
import com.ecell.internationalize.common.core.constant.UserConstants;
import com.ecell.internationalize.common.core.utils.locale.LocaleUtil;
import com.ecell.internationalize.common.core.utils.uuid.UUID;
import com.ecell.internationalize.common.core.web.domain.AjaxResult;
import com.ecell.internationalize.common.redis.service.RedisService;
import com.ecell.internationalize.common.security.annotation.RequiresPermissions;
import com.ecell.internationalize.common.system.constant.FieldConstant;
import com.ecell.internationalize.system.constant.DeviceModelConstants;
import com.ecell.internationalize.system.entity.DeviceModelInfo;
import com.ecell.internationalize.system.entity.dto.DeviceModelPageDTO;
import com.ecell.internationalize.system.service.DeviceModelInfoService;
import io.swagger.annotations.Api;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;


/**
 * <p>
 * 设备型号管理信息表 前端控制器
 * </p>
 *
 * @author ${author}
 * @since 2022-07-09
 */
@Api(value="设备模型管理",tags={"设备模型管理接口"})
@RestController
@RequestMapping("/device_model_info")
public class DeviceModelInfoController {
    @Autowired
    private DeviceModelInfoService deviceModelInfoService;
    @Autowired
    private RedisService redisService;


    /**
     * 新增设备类型
     */
    @PostMapping("/add")
    public AjaxResult add(@Validated @RequestBody DeviceModelInfo model) {
        //设置ID
        model.setDeviceModelId(UUID.fastUUID().toString(true));
        if (UserConstants.NOT_UNIQUE.equals(deviceModelInfoService.checkModelNameUnique(model))) {
            return AjaxResult.error("新增设备型号'" + model.getDeviceModelName() + "'失败，同一厂商下该名称已存在");
        }
        //定位模式id,默认为1
        model.setPositioningModeId(DeviceModelConstants.ONE);
        return deviceModelInfoService.save(model) ? AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS)) : AjaxResult.error(LocaleUtil.getMessage(FieldConstant.MESSAGES_ERROR));
    }

    /**
     * 分页查询
     * @param
     * @param dto
     * @return
     */
    @RequiresPermissions("system:equipmentmodel:list")
    @GetMapping("device/page/list")
    public AjaxResult queryAll(DeviceModelPageDTO dto) {
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS), deviceModelInfoService.findListByPage(dto));
    }

    /**
     * 更新设备型号信息
     * @param model
     * @return
     */
    @PostMapping("/edit")
    public AjaxResult edit(@RequestBody DeviceModelInfo model) {
        return deviceModelInfoService.updateById(model) ? AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS)) : AjaxResult.error(LocaleUtil.getMessage(FieldConstant.MESSAGES_ERROR));
    }

    /**
     * 逻辑删除
     * @param
     * @param id
     * @return
     */
    @DeleteMapping("/del/{id}")
    public AjaxResult remove(@PathVariable String id) {

        return deviceModelInfoService.deltaById(id) ? AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS)) : AjaxResult.error(LocaleUtil.getMessage(FieldConstant.MESSAGES_ERROR));
    }

    /**
     * 根据厂商id或代理商id查询设备型号名称跟ID
     * @param id
     * @return
     */
    @GetMapping("/query/firmId/{id}/{type}")
    public AjaxResult queryModelNameByFirmId(@PathVariable String id,@PathVariable String type){
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS), deviceModelInfoService.queryModelNameByFirmId(id,type)) ;
    }


    /**
     * 根据设备型号ID and 定位模式ID 更新设备型号表的定位模式ID
     */
    @GetMapping("device/byId/{mId}/{pId}")
    public AjaxResult updatePositionIdByModelId(@PathVariable String mId, @PathVariable String pId){

        return  deviceModelInfoService.updatePositionIdByModelId(mId,pId) ? AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS)) : AjaxResult.error(LocaleUtil.getMessage(FieldConstant.MESSAGES_ERROR));
    }

    /**
     * 根据设备类型ID查询设备模型信息
     */
    @GetMapping("device/edit/{id}")
    public AjaxResult queryByModelId(@PathVariable String id){


       return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS), deviceModelInfoService.queryById(id));
    }



}

