package com.ecell.internationalize.system.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.ecell.internationalize.common.core.entity.Rest;
import com.ecell.internationalize.common.system.entity.User;
import com.ecell.internationalize.common.system.entity.VersionRatioHistory;
import com.ecell.internationalize.system.entity.dto.RegisterDto;
import com.ecell.internationalize.system.entity.dto.TotalTypeDto;
import com.ecell.internationalize.system.service.UserService;
import com.ecell.internationalize.system.utils.DateUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.temporal.ChronoUnit;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @Title: DeviceTotalController
 * @Author: liy
 * @Date: 2022/11/2 15:14
 * @Description:
 * @Version:1.0
 */
@Api(value="实时大屏统计",tags={"实时大屏统计接口"})
@RestController
@RequestMapping("/device_total")
public class DeviceTotalController {
    @Autowired
    private UserService userService;
    @ApiOperation("实时大屏用户信息统计")
    @GetMapping
    public Rest<List<TotalTypeDto>> queryDeviceTotal() throws ParseException {
        List<TotalTypeDto> list=new ArrayList<>();
        //注册总数
        int registerCount = userService.count();
        list.add(addTotal("1", registerCount));
        //今日注册数
        Date startTime = DateUtil.getDayBegin();
        Date endTime = DateUtil.getDayEnd();
        LambdaQueryWrapper<User> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.ge(User::getCreateTime,startTime);
        lambdaQueryWrapper.le(User::getCreateTime,endTime);
        int currentCount = userService.count(lambdaQueryWrapper);
        list.add(addTotal("2", currentCount));
        //今日活跃数
        int activeCount=userService.activeCount(startTime,endTime);
        list.add(addTotal("3", activeCount));
        //前一个月每天的用户活跃数
        SimpleDateFormat simpleDateFormat=new SimpleDateFormat("yyyy-MM-dd");
        String currentDate=simpleDateFormat.format(startTime);
        LocalDateTime localDate=LocalDateTime.now().minus(30, ChronoUnit.DAYS);
        Date preDate=simpleDateFormat.parse(localDate.toString());
        String preMonthDate=simpleDateFormat.format(preDate);
        System.out.println("当前日期的第三十天的日期为:"+preMonthDate);
        List<VersionRatioHistory> versionRatioHistoryList = userService.monthActiveCount(currentDate, preMonthDate);
        TotalTypeDto totalTypeDto=new TotalTypeDto();
        totalTypeDto.setType("4");
        totalTypeDto.setList(versionRatioHistoryList);
        list.add(totalTypeDto);
        //前一个月每天用户的注册数
        List<RegisterDto> monthRegisterCount= userService.monthRegisterCount(currentDate,preMonthDate);
        TotalTypeDto registerTotal=new TotalTypeDto();
        registerTotal.setType("5");
        registerTotal.setRegisterList(monthRegisterCount);
        list.add(registerTotal);
        return Rest.ok(list);
    }
    public TotalTypeDto addTotal(String type,Integer count){
        TotalTypeDto totalTypeDto=new TotalTypeDto();
        totalTypeDto.setType(type);
        totalTypeDto.setTotal(count);
        return totalTypeDto;
    }
}
