package com.ecell.internationalize.system.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.ecell.internationalize.common.core.utils.StringUtils;
import com.ecell.internationalize.common.core.utils.locale.LocaleUtil;
import com.ecell.internationalize.common.core.utils.uuid.UUID;
import com.ecell.internationalize.common.core.web.controller.BaseController;
import com.ecell.internationalize.common.core.web.domain.AjaxResult;
import com.ecell.internationalize.common.security.utils.SecurityUtils;
import com.ecell.internationalize.common.system.annotation.SystemLog;
import com.ecell.internationalize.common.system.constant.FieldConstant;
import com.ecell.internationalize.system.entity.FirmManage;
import com.ecell.internationalize.system.entity.vo.DeviceActivaVO;
import com.ecell.internationalize.system.service.FirmManageService;
import com.ecell.internationalize.system.service.impl.DeviceInfoServiceImpl;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

/**
 * <p>
 *  代理商管理入口
 * </p>
 *
 * @author liy
 * @since 2022-07-06
 */
@Api(value="厂商管理",tags={"厂商管理接口"})
@RestController
@RequestMapping("/firm_manage")
public class FirmManageController extends BaseController {
    @Autowired
    private FirmManageService firmManageService;
    @Autowired
    private DeviceInfoServiceImpl deviceInfoService;
    /**
     * 厂商管理条件分页查询
     * @Author liy
     * @Date 2022/7/6 10:38
     * @param  map 分页条件查询体
     * @Return AjaxResult
     */
    @ApiOperation("条件分页查询厂商管理信息")
    @PostMapping("firm/pageList")
    public AjaxResult queryAllByPage(@RequestBody Map<String,Object> map){
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),firmManageService.findAllByPage(map));
    }
    /**
     * 显示所有父级厂商名称
     * @Author liy
     * @Date 2022/7/12 14:38
     * @Return AjaxResult
     */
    @ApiOperation("查询接口显示树形结构")
    @GetMapping("firm/queryAll")
    public AjaxResult queryAll(){
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),firmManageService.queryAll());
    }
    /**
     * 新增厂商信息
     * @Author liy
     * @Date 2022/7/6 16:38
     * @param firmManage 代理商
     * @Return AjaxResult
     */
    @ApiOperation("新增厂商信息")
    @SystemLog(msg =FieldConstant.FIRM_MANAGE,operation = FieldConstant.SAVE_OPERATOR)
    @PostMapping("firm/save")
    public AjaxResult insertFirm(@RequestBody FirmManage firmManage){
            int count = firmManageService.count(new QueryWrapper<FirmManage>()
                    .eq(FieldConstant.FIRM_NAME_COLUMN,firmManage.getFirmName())
                    .eq(FieldConstant.PARENT_ID,FieldConstant.TREE_ROOT_ID));
            if (count>0){
                //存在相同父厂商名称,直接返回结果
                return AjaxResult.error(LocaleUtil.getMessage(FieldConstant.FIRM_NAME_EXIT));
            }
        String parentId= StringUtils.isEmpty(firmManage.getParentId())?FieldConstant.TREE_ROOT_ID:firmManage.getId();
        firmManage.setParentId(parentId);
        String firmId= UUID.randomUUID().toString(true);
        firmManage.setFirmId(firmId);
        firmManage.setCreateUser(SecurityUtils.getUsername());
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),firmManageService.save(firmManage));
    }
    /**
     * 修改厂商信息
     * @Author liy
     * @Date 2022/7/6 16:38
     * @param firmManage 代理商
     * @Return AjaxResult
     */
    @ApiOperation("修改厂商信息")
    @SystemLog(msg =FieldConstant.FIRM_MANAGE,operation = FieldConstant.UPDATE_OPERATOR)
    @PostMapping("firm/update")
    public AjaxResult updateFirm(@RequestBody  FirmManage firmManage){
        LambdaQueryWrapper<FirmManage> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(FirmManage::getFirmName,firmManage.getFirmName());
        lambdaQueryWrapper.eq(FirmManage::getParentId,FieldConstant.TREE_ROOT_ID);
        FirmManage firmManage1 = firmManageService.getBaseMapper().selectOne(lambdaQueryWrapper);
        //修改名称相同
        if (null!=firmManage1&&!firmManage1.getFirmId().equals(firmManage.getFirmId())){
            //存在相同父厂商名称,直接返回结果
            return AjaxResult.error(LocaleUtil.getMessage(FieldConstant.FIRM_NAME_EXIT));
        }
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),firmManageService.updateById(firmManage));
    }
    /**
     * 构建树
     * @Author liy
     * @Date 2022/7/19 10:38
     * @param lazy 是否是懒加载
     * @param parentId 父节点ID
     * @Return AjaxResult
     */
    @ApiOperation("构建厂商父子节点树")
    @GetMapping(value = "firm/queryTree")
    public AjaxResult getTree(boolean lazy, String parentId) {
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),firmManageService.queryTree(lazy,parentId));
    }
    /**
     * 根据父级Id查询下面的所有代理商
     * @Author liy
     * @Date 2022/7/19 10:38
     * @param firmId 父级Id
     * @Return AjaxResult
     */
    @ApiOperation("根据Id查询厂商下面所有的代理商")
    @GetMapping(value = "firm/queryById")
    public AjaxResult queryById(@ApiParam(value="父级Id",required=true)String firmId) {
        LambdaQueryWrapper<FirmManage> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(FirmManage::getParentId,firmId);
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),firmManageService.getBaseMapper().selectList(lambdaQueryWrapper));
    }
    /**
     * 根据firmId查询代理商
     * @Author liy
     * @Date 2022/7/19 10:38
     * @param id 父级Id
     * @Return AjaxResult
     */
    @ApiOperation("据firmId查询代理商")
    @GetMapping(value = "firm/queryByAgentId")
    public AjaxResult queryByAgentId(@ApiParam(value="firmId",required=true)String id) {
        LambdaQueryWrapper<FirmManage> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(FirmManage::getFirmId,id);
        FirmManage firmManage = firmManageService.getBaseMapper().selectOne(lambdaQueryWrapper);
        DeviceActivaVO deviceActivaVO;
        if (FieldConstant.TREE_ROOT_ID.equals(firmManage.getParentId())){
            deviceActivaVO= deviceInfoService.queryFirmCountDevice(id,null);
        }else {
            deviceActivaVO = deviceInfoService.queryFirmCountDevice(null,id);
        }
        firmManage.setImportNum(deviceActivaVO.getInputDevice());
        firmManage.setActiveNum(deviceActivaVO.getActivateDevice());
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),firmManage);
    }
    /**
     * 显示所有厂商代理商名称
     * @Author liy
     * @Date 2022/7/12 14:38
     * @Return AjaxResult
     */
    @ApiOperation("查询所有厂商/代理商信息")
    @GetMapping("firm/queryAllData")
    public List<FirmManage> queryAllData(){
        return firmManageService.getBaseMapper().selectList(null);
    }
}

