package com.ecell.internationalize.system.controller;
import com.ecell.internationalize.common.core.utils.locale.LocaleUtil;
import com.ecell.internationalize.common.core.utils.uuid.UUID;
import com.ecell.internationalize.common.core.web.domain.AjaxResult;
import com.ecell.internationalize.common.security.utils.SecurityUtils;
import com.ecell.internationalize.common.system.annotation.SystemLog;
import com.ecell.internationalize.common.system.constant.FieldConstant;
import com.ecell.internationalize.common.system.utlis.UploadUtil;
import com.ecell.internationalize.system.entity.HomeBanner;
import com.ecell.internationalize.system.service.HomeBannerService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.Date;
import java.util.Map;

/**
 * <p>
 * 首页banner图 前端控制器
 * </p>
 *
 * @author liy
 * @since 2022-07-14
 */
@Api(value="首页banner图",tags={"首页banner图接口"})
@RestController
@RequestMapping("/home_banner")
public class HomeBannerController {
    @Autowired
    private HomeBannerService homeBannerService;
    /**
     * 首页banner图条件分页查询
     * @Author liy
     * @Date 2022/7/15 16:38
     * @param  map 分页条件查询体
     * @Return AjaxResult
     */
    @ApiOperation("条件分页查询首页banner图信息")
    @PostMapping("home/list")
    public AjaxResult queryAllByPage(@RequestBody Map<String,Object> map){
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),homeBannerService.findAllByPage(map));
    }
    /**
     * 首页banner图发布
     * @Author liy
     * @Date 2022/7/15 16:38
     * @param  homeBanner 实体类
     * @Return AjaxResult
     */
    @ApiOperation("活动banner图发布/取消发布")
    @SystemLog(msg =FieldConstant.HOME_BANNER_MANAGE,operation = FieldConstant.HOME_BANNER_PUBLISH)
    @PostMapping("home/changeStatus")
    public AjaxResult changeStatusOrDelUser(@RequestBody @ApiParam(value="传整个对象,修改其中status字段,（0：取消发布，1：发布）",required=true) HomeBanner homeBanner){
        if (FieldConstant.MATH_ONE.equals(homeBanner.getStatus())){
            homeBanner.setPublishUser("admin");
            homeBanner.setPublishTime(new Date());
        }else {
            homeBanner.setPublishUser(null);
            homeBanner.setPublishTime(null);
        }
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),homeBannerService.updateById(homeBanner));
    }
    /**
     * 首页banner图删除
     * @Author liy
     * @Date 2022/7/15 16:38
     * @param  homeBanner 实体类
     * @Return AjaxResult
     */
    @ApiOperation("首页banner图删除")
    @SystemLog(msg =FieldConstant.HOME_BANNER_MANAGE,operation = FieldConstant.DELETE_OPERATOR)
    @PostMapping("home/del")
    public AjaxResult del(@RequestBody @ApiParam(value="传整个对象,修改其中delFlag字段,（0：删除，1：正常）",required=true) HomeBanner homeBanner){
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),homeBannerService.updateById(homeBanner));
    }
    /**
     * 首页banner图上传/新增
     * @Author liy
     * @Date 2022/7/14 16:38
     * @param  file 文件
     * @Return AjaxResult
     */
    @ApiOperation("首页banner图新增上传")
    @SystemLog(msg =FieldConstant.HOME_BANNER_MANAGE,operation = FieldConstant.SAVE_OPERATOR)
    @PostMapping("home/save")
    public AjaxResult save(@RequestParam("file") MultipartFile file, @RequestParam("title") String title,  @RequestParam("bannerOrder")Integer bannerOrder) throws IOException {
        String path = UploadUtil.upload(file);
        String id= UUID.randomUUID().toString(true);
        HomeBanner homeBanner=new HomeBanner();
        homeBanner.setId(id);
        homeBanner.setHomeOrder(bannerOrder);
        homeBanner.setTitle(title);
        homeBanner.setCreateUser(SecurityUtils.getUsername());
        homeBanner.setBanner(path);
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),homeBannerService.save(homeBanner));
    }
    /**
     * 首页banner图上传/修改
     * @Author liy
     * @Date 2022/7/14 16:38
     * @param  file 文件
     * @Return AjaxResult
     */
    @ApiOperation("首页banner图修改上传")
    @SystemLog(msg =FieldConstant.HOME_BANNER_MANAGE,operation = FieldConstant.UPDATE_OPERATOR)
    @PostMapping("home/update")
    public AjaxResult update(@RequestParam("file") MultipartFile file,@RequestParam("id")String id, @RequestParam("title") String title,@RequestParam("bannerOrder")Integer bannerOrder) throws IOException {
        HomeBanner homeBanner=new HomeBanner();
        if (null!=file) {
            String path = UploadUtil.upload(file);
            homeBanner.setBanner(path);
        }
        homeBanner.setId(id);
        homeBanner.setHomeOrder(bannerOrder);
        homeBanner.setTitle(title);
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),homeBannerService.updateById(homeBanner));
    }
}

