package com.ecell.internationalize.system.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ecell.internationalize.common.core.utils.StringUtils;
import com.ecell.internationalize.common.core.utils.locale.LocaleUtil;
import com.ecell.internationalize.common.core.utils.uuid.UUID;
import com.ecell.internationalize.common.core.web.domain.AjaxResult;
import com.ecell.internationalize.common.redis.service.RedisService;
import com.ecell.internationalize.common.system.constant.FieldConstant;
import com.ecell.internationalize.system.entity.DeviceModelInfo;
import com.ecell.internationalize.system.entity.FirmManage;
import com.ecell.internationalize.system.entity.LocationLimitInfo;
import com.ecell.internationalize.system.entity.dto.LocationLimitInfoPage;
import com.ecell.internationalize.system.service.DeviceModelInfoService;
import com.ecell.internationalize.system.service.FirmManageService;
import com.ecell.internationalize.system.service.LocationLimitInfoService;
import io.swagger.annotations.Api;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;


/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author ${author}
 * @since 2024-07-15
 */
@Api(value="定位次数限制",tags={"定位次数限制管理接口"})
@RestController
@RequestMapping("/location-limit-info")
public class LocationLimitInfoController {
    @Autowired
    private LocationLimitInfoService locationLimitInfoService;
    @Autowired
    private RedisService redisService;
    @Autowired
    private DeviceModelInfoService deviceModelInfoService;
    @Autowired
    private FirmManageService firmManageService;



    /**
     * 新增定位限制次数
     */
    @PostMapping("/add")
    public AjaxResult add(@Validated @RequestBody LocationLimitInfo model) {

        //  return AjaxResult.error("新增设备型号'" + model.getDeviceModelName() + "'失败，同一厂商下该名称已存在");
        QueryWrapper<LocationLimitInfo> wrapper =new QueryWrapper<>();
        wrapper.eq("firm_id",model.getFirmId());
        wrapper.eq("model_id",model.getModelId());
        LocationLimitInfo locationLimitInfo = locationLimitInfoService.getBaseMapper().selectOne(wrapper);
        if (StringUtils.isNotNull(locationLimitInfo)){
            return AjaxResult.error(LocaleUtil.getMessage(FieldConstant.LOCAL_LIMIT_MESSAGES_ERROR));
        }
        //设置ID
        model.setId(UUID.fastUUID().toString(true));
        boolean save = locationLimitInfoService.save(model);
        if (save){
            String hashKey = "local:limitCount"+":"+model.getModelId();
            redisService.setCacheObject(hashKey,model.getLimitCount());
        }

        return save ? AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS)) : AjaxResult.error(LocaleUtil.getMessage(FieldConstant.MESSAGES_ERROR));
    }

    /**
     * 分页查询定位限制次数
     * @param page
     * @return
     */
    @PostMapping("/query/page")
    public AjaxResult queryPage(@RequestBody LocationLimitInfoPage page) {

        //  return AjaxResult.error("新增设备型号'" + model.getDeviceModelName() + "'失败，同一厂商下该名称已存在");
        QueryWrapper<LocationLimitInfo> wrapper =new QueryWrapper<>();
        if (StringUtils.isNotEmpty(page.getFirmId())){
            wrapper.eq("firm_id",page.getFirmId());
        }
        if (StringUtils.isNotEmpty(page.getModelId())){
            wrapper.eq("model_id",page.getModelId());
        }
        IPage<LocationLimitInfo> page1 = locationLimitInfoService.getBaseMapper().selectPage(new Page<>(page.getCurrent(), page.getPageSize()), wrapper);
        if (StringUtils.isNotNull(page1)){
            if (StringUtils.isNotNull(page1.getRecords()) && page1.getRecords().size()>0){
                page1.getRecords().forEach(item->{
                    if (StringUtils.isNotEmpty(item.getModelId())){
                        DeviceModelInfo deviceModelInfo = deviceModelInfoService.getBaseMapper().selectById(item.getModelId());
                        if (StringUtils.isNotNull(deviceModelInfo)){
                            item.setModelName(deviceModelInfo.getDeviceModelName());
                        }
                    }
                    if (StringUtils.isNotEmpty(item.getFirmId())){
                        FirmManage firmManage = firmManageService.getBaseMapper().selectById(item.getFirmId());
                        if (StringUtils.isNotNull(firmManage)){
                            item.setFirmName(firmManage.getFirmName());
                        }
                    }
                });

            }

        }


       return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS), page1);
    }

    /**
     * 编辑定位限制次数
     * @param info
     * @return
     */


    @PostMapping("/edit")
    public AjaxResult edit(@Validated @RequestBody LocationLimitInfo info) {
        LocationLimitInfo locationLimitInfo=null;
      if (StringUtils.isNotEmpty(info.getId())){
            QueryWrapper<LocationLimitInfo>wrapper =new QueryWrapper<>();
            wrapper.eq("id",info.getId());
          locationLimitInfo= locationLimitInfoService.getBaseMapper().selectOne(wrapper);
            if (StringUtils.isNull(locationLimitInfo)){
                return AjaxResult.error(LocaleUtil.getMessage(FieldConstant.LOCATIONINFO_MESSAGES_ERROR));
            }
      }
        boolean b = locationLimitInfoService.updateById(info);
      if (b){
        String hashKey="local:limitCount"+":"+locationLimitInfo.getModelId();
          if (redisService.hasKey(hashKey)){
            redisService.setCacheObject(hashKey,info.getLimitCount());
          }

      }



        return b? AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS)) : AjaxResult.error(LocaleUtil.getMessage(FieldConstant.MESSAGES_ERROR));
    }


    @GetMapping("/delete/{id}")
    public AjaxResult delete(@PathVariable("id") String id) {
        LocationLimitInfo locationLimitInfo=null;
        if (StringUtils.isNotEmpty(id)){
            QueryWrapper<LocationLimitInfo>wrapper =new QueryWrapper<>();
            wrapper.eq("id",id);
            locationLimitInfo= locationLimitInfoService.getBaseMapper().selectOne(wrapper);
            if (StringUtils.isNull(locationLimitInfo)){
                return AjaxResult.error(LocaleUtil.getMessage(FieldConstant.LOCATIONINFO_MESSAGES_ERROR));
            }
        }
        int i = locationLimitInfoService.getBaseMapper().deleteById(id);
        if (i>0){
            String hashKey="local:limitCount"+":"+locationLimitInfo.getModelId();
            if (redisService.hasKey(hashKey)){
                redisService.deleteObject(hashKey);
            }

        }



        return  i>0? AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS)) : AjaxResult.error(LocaleUtil.getMessage(FieldConstant.MESSAGES_ERROR));
    }

//    /**
//     *
//     * @param id
//     * @param type
//     * @return
//     */
//    @GetMapping("/enabled/{id}/{type}")
//    public AjaxResult enabled(@PathVariable("id") String id,@PathVariable("type") String type) {
//        LocationLimitInfo locationLimitInfo=null;
//        if (StringUtils.isNotEmpty(id)){
//            QueryWrapper<LocationLimitInfo>wrapper =new QueryWrapper<>();
//            wrapper.eq("id",id);
//            locationLimitInfo= locationLimitInfoService.getBaseMapper().selectOne(wrapper);
//            if (StringUtils.isNull(locationLimitInfo)){
//                return AjaxResult.error(LocaleUtil.getMessage(FieldConstant.LOCATIONINFO_MESSAGES_ERROR));
//            }
//        }
//        int i = locationLimitInfoService.getBaseMapper().deleteById(id);
//        if (i>0){
//            String hashKey="local:limitCount"+":"+locationLimitInfo.getModelId();
//            if (redisService.hasKey(hashKey)){
//                redisService.deleteObject(hashKey);
//            }
//
//        }
//
//
//
//        return  i>0? AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS)) : AjaxResult.error(LocaleUtil.getMessage(FieldConstant.MESSAGES_ERROR));
//    }
//




}

