package com.ecell.internationalize.system.controller;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.ecell.internationalize.common.core.utils.locale.LocaleUtil;
import com.ecell.internationalize.common.core.utils.uuid.UUID;
import com.ecell.internationalize.common.core.web.domain.AjaxResult;
import com.ecell.internationalize.common.security.utils.SecurityUtils;
import com.ecell.internationalize.common.system.annotation.SystemLog;
import com.ecell.internationalize.common.system.constant.FieldConstant;
import com.ecell.internationalize.system.entity.MessageCategory;
import com.ecell.internationalize.system.service.MessageCategoryService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.Map;

/**
 * <p>
 * 消息类别 前端控制器
 * </p>
 *
 * @author liy
 * @since 2022-07-12
 */
@Api(value="消息类别",tags={"消息类别接口"})
@RestController
@RequestMapping("/message_category")
public class MessageCategoryController {
    @Autowired
    private MessageCategoryService messageCategoryService;
    /**
     * 消息类别分页查询
     * @Author liy
     * @Date 2022/7/13 10:38
     * @param  map 分页
     * @Return AjaxResult
     */
    @ApiOperation("条件分页查询消息类别信息")
    @PostMapping("category/list")
    public AjaxResult queryAll(@RequestBody Map<String,Object> map){
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),messageCategoryService.findAllByPage(map));
    }
    /**
     * 新增类别
     * @Author liy
     * @Date 2022/7/13 14:38
     * @param messageCategory 类别实体
     * @Return AjaxResult
     */
    @ApiOperation("新增类别")
    @SystemLog(msg =FieldConstant.MESSAGE_CATEGORY,operation = FieldConstant.SAVE_OPERATOR)
    @PostMapping("category/save")
    public AjaxResult saveCategory(@RequestBody @Valid MessageCategory messageCategory){
        int count = messageCategoryService.count(new QueryWrapper<MessageCategory>()
                        .eq(FieldConstant.CATEGORY_NAME,messageCategory.getCategoryName()));
        if (count>0){
            //存在类别名称,直接返回结果
            return AjaxResult.error(LocaleUtil.getMessage(FieldConstant.MESSAGES_CATEGORY_NAME));
        }
        String id= UUID.randomUUID().toString(true);
        messageCategory.setId(id);
        messageCategory.setCreateUser(SecurityUtils.getUsername());
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),messageCategoryService.save(messageCategory));
    }
    /**
     * 修改类别
     * @Author liy
     * @Date 2022/7/13 14:38
     * @param messageCategory 类别实体
     * @Return AjaxResult
     */
    @ApiOperation("修改类别")
    @SystemLog(msg =FieldConstant.MESSAGE_CATEGORY,operation = FieldConstant.UPDATE_OPERATOR)
    @PostMapping("category/update")
    public AjaxResult updateCategory(@RequestBody @Valid MessageCategory messageCategory){
        LambdaQueryWrapper<MessageCategory> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(MessageCategory::getCategoryName,messageCategory.getCategoryName());
        lambdaQueryWrapper.eq(MessageCategory::getDelFlag,FieldConstant.DEL_FLAG);
        MessageCategory messageCategory1 = messageCategoryService.getBaseMapper().selectOne(lambdaQueryWrapper);
        //修改名称相同
        if (null!=messageCategory1&&!messageCategory1.getId().equals(messageCategory.getId())){
            //存在类别名称,直接返回结果
            return AjaxResult.error(LocaleUtil.getMessage(FieldConstant.MESSAGES_CATEGORY_NAME));
        }
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),messageCategoryService.updateById(messageCategory));
    }
    /**
     * 删除类别
     * @Author liy
     * @Date 2022/7/13 15:38
     * @param messageCategory 类别实体
     * @Return AjaxResult
     */
    @ApiOperation("删除类别")
    @SystemLog(msg =FieldConstant.MESSAGE_CATEGORY,operation = FieldConstant.DELETE_OPERATOR)
    @PostMapping("category/del")
    public AjaxResult delCategory(@RequestBody @ApiParam(value="传整个对象,修改其中delFlag字段,（0：删除，1：正常）",required=true) MessageCategory messageCategory){
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),messageCategoryService.delCategory(messageCategory));
    }
    /**
     * 类别查询条件
     * @Author liy
     * @Date 2022/7/8 16:38
     * @Return AjaxResult
     */
    @ApiOperation("消息类别查询")
    @GetMapping("category/queryCategory")
    public AjaxResult queryCategory(){
        LambdaQueryWrapper<MessageCategory> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(MessageCategory::getDelFlag,FieldConstant.DEL_FLAG);
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),messageCategoryService.getBaseMapper().selectList(lambdaQueryWrapper));
    }
}

