package com.ecell.internationalize.system.controller;
import com.ecell.internationalize.common.core.utils.locale.LocaleUtil;
import com.ecell.internationalize.common.core.utils.uuid.UUID;
import com.ecell.internationalize.common.core.web.domain.AjaxResult;
import com.ecell.internationalize.common.security.utils.SecurityUtils;
import com.ecell.internationalize.common.system.annotation.SystemLog;
import com.ecell.internationalize.common.system.constant.FieldConstant;
import com.ecell.internationalize.system.entity.MessageContent;
import com.ecell.internationalize.system.service.MessageContentService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.Map;

/**
 * <p>
 * 消息内容 前端控制器
 * </p>
 *
 * @author liy
 * @since 2022-07-12
 */
@Api(value="消息内容",tags={"消息内容接口"})
@RestController
@RequestMapping("/message_content")
public class MessageContentController {
    @Autowired
    private MessageContentService messageContentService;
    /**
     * 消息内容条件分页查询
     * @Author liy
     * @Date 2022/7/13 16:38
     * @param  map 分页条件查询体
     * @Return AjaxResult
     */
    @ApiOperation("条件分页查询消息内容")
    @PostMapping("content/list")
    public AjaxResult queryAll(@RequestBody Map<String,Object> map){
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),messageContentService.findAllByPage(map));
    }
    /**
     * 新增消息内容
     * @Author liy
     * @Date 2022/7/13 17:38
     * @param messageContent 消息内容实体
     * @Return AjaxResult
     */
    @ApiOperation("新增消息内容")
    @SystemLog(msg =FieldConstant.MESSAGE_CONTENT,operation = FieldConstant.SAVE_OPERATOR)
    @PostMapping("content/save")
    public AjaxResult saveCategory(@RequestBody @Valid MessageContent messageContent){
        String id= UUID.randomUUID().toString(true);
        messageContent.setId(id);
        messageContent.setCreateUser(SecurityUtils.getUsername());
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),messageContentService.save(messageContent));
    }
    /**
     * 修改消息内容
     * @Author liy
     * @Date 2022/7/13 17:38
     * @param messageContent 消息内容实体
     * @Return AjaxResult
     */
    @ApiOperation("修改消息内容")
    @SystemLog(msg =FieldConstant.MESSAGE_CONTENT,operation = FieldConstant.UPDATE_OPERATOR)
    @PostMapping("content/update")
    public AjaxResult updateCategory(@RequestBody @Valid MessageContent messageContent){
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),messageContentService.updateById(messageContent));
    }
    /**
     * 删除消息内容
     * @Author liy
     * @Date 2022/7/13 17:38
     * @param messageContent 消息内容实体
     * @Return AjaxResult
     */
    @ApiOperation("删除消息内容")
    @SystemLog(msg =FieldConstant.MESSAGE_CONTENT,operation = FieldConstant.DELETE_OPERATOR)
    @PostMapping("content/del")
    public AjaxResult del(@RequestBody @ApiParam(value="传整个对象,修改其中delFlag字段,（0：删除，1：正常）",required=true) MessageContent messageContent){
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),messageContentService.updateById(messageContent));
    }
    /**
     * 修改是否为热门问题标识
     * @Author liy
     * @Date 2022/7/13 17:38
     * @param messageContent 消息内容实体
     * @Return AjaxResult
     */
    @ApiOperation("修改是否为热门问题标识")
    @SystemLog(msg =FieldConstant.MESSAGE_CONTENT,operation = FieldConstant.IS_HOT_QUESTION)
    @PostMapping("content/updateHot")
    public AjaxResult updateHot(@RequestBody @ApiParam(value="传整个对象,修改其中hotIssues字段,（0：否，1:是）",required=true) MessageContent messageContent){
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),messageContentService.updateById(messageContent));
    }
}

