package com.ecell.internationalize.system.controller;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.ecell.internationalize.common.core.utils.SpringUtils;
import com.ecell.internationalize.common.core.utils.locale.LocaleUtil;
import com.ecell.internationalize.common.core.utils.uuid.UUID;
import com.ecell.internationalize.common.core.web.domain.AjaxResult;
import com.ecell.internationalize.common.system.constant.FieldConstant;
import com.ecell.internationalize.common.system.entity.PositioningModelInfo;
import com.ecell.internationalize.system.entity.DeviceModelInfo;
import com.ecell.internationalize.system.mapper.DeviceModelInfoMapper;
import com.ecell.internationalize.system.service.PositioningModelInfoService;
import io.swagger.annotations.Api;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;


/**
 * <p>
 * 定位模式信息表 前端控制器
 * </p>
 *
 * @author ${author}
 * @since 2022-07-09
 */
@Api(value="定位模式信息管理",tags={"定位模式信息管理接口"})
@RestController
@RequestMapping("/position_model_info")
public class PositioningModelInfoController {
    @Autowired
    private PositioningModelInfoService positioningModelInfoService;


    /**
     * 新增定位模式
     */
    @PostMapping("position/add")
    public AjaxResult add(@Validated @RequestBody PositioningModelInfo info) {
        //设置ID
        info.setPositioningModeId(UUID.fastUUID().toString(true));
        return  positioningModelInfoService.save(info) ?  AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS)) :AjaxResult.error(LocaleUtil.getMessage(FieldConstant.MESSAGES_ERROR));

    }

    /**
     * 修改
     * @param info
     * @return
     */
    @PostMapping("position/update")
    public AjaxResult updatePositionModel(@Validated @RequestBody PositioningModelInfo info){
        //设备定位模式绑定了型号就不能修改
        DeviceModelInfoMapper modelInfoMapper = SpringUtils.getBean(DeviceModelInfoMapper.class);
        LambdaQueryWrapper<DeviceModelInfo> queryWrapper =new LambdaQueryWrapper<>();
        queryWrapper.eq(DeviceModelInfo::getPositioningModeId,info.getPositioningModeId());
        queryWrapper.eq(DeviceModelInfo::getDelFlag,1);
       if ( modelInfoMapper.selectCount(queryWrapper)>0){
           return AjaxResult.error(LocaleUtil.getMessage(FieldConstant.PROHIBIT_ERROR));
       }

        return positioningModelInfoService.updateById(info) ?  AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS)) :AjaxResult.error(LocaleUtil.getMessage(FieldConstant.MESSAGES_ERROR));
    }

    /**
     * 分页查询
     * @param pageSize
     * @param pageNum
     * @return
     */
    @GetMapping("position/page/list")
    public AjaxResult queryPageList(@RequestParam int pageSize,@RequestParam int pageNum){
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),positioningModelInfoService.findListByPage(pageSize,pageNum));
    }

    /**
     * 查询所有的定位模式
     * @return
     */
    @GetMapping("position/query/all")
    public AjaxResult queryAll(){
        return AjaxResult.success(positioningModelInfoService.queryAll());
    }



}

