package com.ecell.internationalize.system.controller;
import com.ecell.internationalize.common.core.utils.locale.LocaleUtil;
import com.ecell.internationalize.common.core.utils.uuid.UUID;
import com.ecell.internationalize.common.core.web.domain.AjaxResult;
import com.ecell.internationalize.common.security.utils.SecurityUtils;
import com.ecell.internationalize.common.system.annotation.SystemLog;
import com.ecell.internationalize.common.system.constant.FieldConstant;
import com.ecell.internationalize.system.entity.SystemMessage;
import com.ecell.internationalize.system.service.SystemMessageService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.Map;

/**
 * <p>
 * 系统消息 前端控制器
 * </p>
 *
 * @author liy
 * @since 2022-07-12
 */
@Api(value="系统消息",tags={"系统消息接口"})
@RestController
@RequestMapping("/system_message")
public class SystemMessageController {
    @Autowired
    private SystemMessageService systemMessageService;
    /**
     * 系统消息分页查询
     * @Author liy
     * @Date 2022/7/13 10:38
     * @param  map 分页-查询条件
     * @Return AjaxResult
     */
    @ApiOperation("条件分页查询系统消息")
    @PostMapping("system/message/list")
    public AjaxResult queryPageList(@RequestBody Map<String,Object> map){
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),systemMessageService.findAllByPage(map));
    }
    /**
     * 删除系统消息
     * @Author liy
     * @Date 2022/7/13 17:38
     * @param systemMessage 系统消息
     * @Return AjaxResult
     */
    @ApiOperation("删除消息内容")
    @PostMapping("system/message/del")
    @SystemLog(msg =FieldConstant.SYSTEM_MESSAGE_CONTENT,operation = FieldConstant.DELETE_OPERATOR)
    public AjaxResult delOrUpdateHot(@RequestBody @ApiParam(value="传整个对象,修改其中delFlag字段,（0：删除，1：正常）",required=true) SystemMessage systemMessage){
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),systemMessageService.updateById(systemMessage));
    }
    /**
     * 新增系统内容
     * @Author liy
     * @Date 2022/7/13 17:38
     * @param systemMessage 系统消息实体
     * @Return AjaxResult
     */
    @ApiOperation("新增系统内容")
    @SystemLog(msg =FieldConstant.SYSTEM_MESSAGE_CONTENT,operation = FieldConstant.SAVE_OPERATOR)
    @PostMapping("system/message/save")
    public AjaxResult saveCategory(@RequestBody @Valid SystemMessage systemMessage){
        String id= UUID.randomUUID().toString(true);
        systemMessage.setId(id);
        systemMessage.setCreateUser(SecurityUtils.getUsername());
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),systemMessageService.save(systemMessage));
    }
}

