package com.ecell.internationalize.system.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.ecell.internationalize.common.core.utils.locale.LocaleUtil;
import com.ecell.internationalize.common.core.web.domain.AjaxResult;
import com.ecell.internationalize.common.system.annotation.SystemLog;
import com.ecell.internationalize.common.system.constant.FieldConstant;
import com.ecell.internationalize.common.system.entity.User;
import com.ecell.internationalize.system.service.UserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.Map;

/**
 * <p>
 * 用户表 前端控制器
 * </p>
 * @author liy
 * @since 2022-07-07
 */
@Api(value="用户信息",tags={"用户信息接口"})
@RestController
@RequestMapping("/user")
public class UserController {
    @Autowired
    private UserService userService;
    /**
     * 用户信息条件分页查询
     * @Author liy
     * @Date 2022/7/7 16:38
     * @param map 分页条件查询体
     * @Return IPage
     */
    @ApiOperation("条件分页查询用户信息")
    @PostMapping("/page/queryList")
    public AjaxResult queryList(@RequestBody Map<String,Object> map){
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),userService.queryList(map));

    }
    /**
     * 用户操作条件分页查询
     * @Author liy
     * @Date 2022/7/8 16:38
     * @param map 分页条件查询体
     * @Return IPage
     */
    @ApiOperation("条件分页查询用户操作")
    @PostMapping("/page/operatorList")
    public AjaxResult queryOperatorList(@RequestBody Map<String,Object> map){
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),userService.queryOperatorList(map));
    }
    /**
     * 用户操作修改密码
     * @Author liy
     * @Date 2022/7/8 16:38
     * @param user 用户
     * @Return AjaxResult
     */
    @ApiOperation("用户操作修改密码")
    @SystemLog(msg =FieldConstant.USER_OPERATOR,operation = FieldConstant.UPDATE_OPERATOR)
    @PostMapping("operator/update")
    public AjaxResult updateUser(@RequestBody @Valid User user){
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),userService.updateById(user));
    }
    /**
     * 用户操作启用/停用账号
     * @Author liy
     * @Date 2022/7/8 16:38
     * @param user 用户
     * @Return AjaxResult
     */
    @ApiOperation("用户操作启用/停用账号")
    @SystemLog(msg =FieldConstant.USER_OPERATOR,operation = FieldConstant.USER_CHANGE_ACCOUNT)
    @PostMapping("operator/changeStatus")
    public AjaxResult changeStatus(@RequestBody @ApiParam(value="传整个对象,修改其中status字段,（0:启用,1：停用）",required=true) User user){
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),userService.updateById(user));
    }
    /**
     * 用户操作删除
     * @Author liy
     * @Date 2022/7/8 16:38
     * @param user 用户
     * @Return AjaxResult
     */
    @ApiOperation("用户操作删除")
    @SystemLog(msg =FieldConstant.USER_OPERATOR,operation = FieldConstant.DELETE_OPERATOR)
    @PostMapping("operator/del")
    public AjaxResult del(@RequestBody User user){
        LambdaQueryWrapper<User> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(User::getUserId,user.getUserId());
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),userService.getBaseMapper().delete(lambdaQueryWrapper));
    }
    /**
     * 渠道查询条件
     * @Author liy
     * @Date 2022/7/8 16:38
     * @Return AjaxResult
     */
    @ApiOperation("渠道查询")
    @GetMapping("operator/queryChannel")
    public AjaxResult queryChannel(){
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),userService.queryChannel());
    }
}

