package com.ecell.internationalize.system.controller;
import com.ecell.internationalize.common.core.utils.locale.LocaleUtil;
import com.ecell.internationalize.common.core.utils.uuid.UUID;
import com.ecell.internationalize.common.core.web.domain.AjaxResult;
import com.ecell.internationalize.common.system.constant.FieldConstant;
import com.ecell.internationalize.system.constant.DeviceModelConstants;
import com.ecell.internationalize.system.entity.VideoModelInfo;
import com.ecell.internationalize.system.service.VideoModelInfoService;
import io.swagger.annotations.Api;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
;

/**
 * <p>
 * 视频型号管理信息表 前端控制器
 * </p>
 *
 *
 *
 */
@Api(value="视频型号管理",tags={"视频型号管理接口"})
@RestController
@RequestMapping("/videoModel")
public class VideoModelInfoController {
    @Autowired
    private VideoModelInfoService videoModelInfoService;
    /**
     * 新增视频类型
     */
    @PostMapping("video/add")
    public AjaxResult add(@Validated @RequestBody VideoModelInfo info) {
        //设置ID
        info.setVideoModelId(UUID.fastUUID().toString(true));
        if (DeviceModelConstants.NOT_UNIQUE.equals(videoModelInfoService.checkModelNameUnique(info)))
        {
            return AjaxResult.error("新增视频型号'" + info.getVideoModelName() + "'失败，该型号名称已存在");
        }
        return  videoModelInfoService.save(info) ? AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS)) :AjaxResult.error(LocaleUtil.getMessage(FieldConstant.MESSAGES_ERROR));

    }

    /**
     * 分页查询
     * @param pageSize
     * @param pageNum
     * @return
     */
    @GetMapping("video/page/list")
    public AjaxResult queryPage(@RequestParam int pageSize,@RequestParam int pageNum){
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),videoModelInfoService.findListByPage(pageSize,pageNum));
    }

    /**
     * 初始化--》设备型号管理新增用
     * @return
     */
    @GetMapping("video/init/{type}")
    public AjaxResult queryAll(@PathVariable String type){
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS),videoModelInfoService.queryAll(type));
    }



    /**
     * 修改
     * @param info
     * @return
     */
    @PostMapping("video/update")
    public AjaxResult updateVideoModel(@Validated @RequestBody VideoModelInfo info){
        return videoModelInfoService.updateById(info) ?  AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS)) :AjaxResult.error(LocaleUtil.getMessage(FieldConstant.MESSAGES_ERROR));
    }

    /**
     * 逻辑删除
     * @param id
     * @return
     */
    @DeleteMapping("video/del/{id}")
    public AjaxResult deleteVideoModel(@PathVariable String id){

        return videoModelInfoService.deltaById(id) ?  AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS)) :AjaxResult.error(LocaleUtil.getMessage(FieldConstant.MESSAGES_ERROR));
    }

}

