package com.ecell.internationalize.system.controller.api;


import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ecell.internationalize.common.core.context.SecurityContextHolder;
import com.ecell.internationalize.common.core.utils.locale.LocaleUtil;
import com.ecell.internationalize.common.core.utils.uuid.UUID;
import com.ecell.internationalize.common.core.web.domain.AjaxResult;
import com.ecell.internationalize.common.core.web.domain.R;
import com.ecell.internationalize.common.redis.service.RedisService;
import com.ecell.internationalize.common.system.annotation.SystemLog;
import com.ecell.internationalize.common.system.constant.FieldConstant;
import com.ecell.internationalize.system.entity.FirmManage;
import com.ecell.internationalize.system.entity.api.ApiOutward;
import com.ecell.internationalize.system.entity.api.dto.ApiOutwardPage;
import com.ecell.internationalize.system.service.FirmManageService;
import com.ecell.internationalize.system.service.api.ApiOutwardService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Date;

/**
 * <p>
 * API对外开放接口
 * </p>
 *
 * @author ${author}
 * @since 2023-03-07
 */
@RestController
@Api(value="API对外开放接口",tags={"API对外开放接口"})
@RequestMapping("/api_outward")
public class ApiOutwardController {
    @Autowired
    private ApiOutwardService apiOutwardService;
    @Autowired
    private RedisService redisService;
    @Autowired
    private FirmManageService firmManageService;
    /**
     * 条件分页查询
     * @Author liy
     * @Date 2022/7/14 16:38
     * @param  apiOutwardPage 分页条件查询体
     * @Return AjaxResult
     */
    @ApiOperation("APPID和APPKEY条件分页查询")
    @PostMapping("api/list")
    public R<IPage<ApiOutward>> queryAllByPage(@RequestBody ApiOutwardPage apiOutwardPage){
        Page<ApiOutward> page=new Page<>(apiOutwardPage.getCurrent(),apiOutwardPage.getPageSize());
        LambdaQueryWrapper<ApiOutward> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.orderByDesc(ApiOutward::getCreateTime);
        IPage<ApiOutward> selectPage = apiOutwardService.getBaseMapper().selectPage(page, lambdaQueryWrapper);

        if (null!=selectPage) {
            for (ApiOutward apiOutward : selectPage.getRecords()) {
                LambdaQueryWrapper<FirmManage> firm=new LambdaQueryWrapper<>();
                //厂商
                if (FieldConstant.MATH_ONE.equals(apiOutward.getFirmFlag())) {
                    firm.eq(FirmManage::getFirmId,apiOutward.getFirmId());
                }
                //代理商
                if (FieldConstant.SECOND_FLAG.equals(apiOutward.getFirmFlag())) {
                    firm.eq(FirmManage::getParentId,apiOutward.getSecondFirmId());
                }
                FirmManage firmManage = firmManageService.getBaseMapper().selectOne(firm);
                apiOutward.setFirmName(firmManage.getFirmName());
            }
        }
        return R.ok(selectPage);
    }

    /**
     * 新增
     * @Author liy
     * @Date 2023/3/7 16:38
     * @param apiOutward API
     * @Return AjaxResult
     */
    @ApiOperation("生成APPid和APPkey")
    @PostMapping("api/save")
    public AjaxResult save(@RequestBody @ApiParam("API类") ApiOutward apiOutward) {
        int appId = Integer.parseInt(String.valueOf(System.currentTimeMillis()).substring(0,10));
        String appKey= UUID.fastUUID().toString(true);
        apiOutward.setAppId(appId);
        apiOutward.setAppKey(appKey);
        apiOutward.setCreateTime(new Date());
        apiOutward.setCreateUser(SecurityContextHolder.getUserName());
        apiOutward.setId(UUID.fastUUID().toString(true));
        apiOutward.setStatus(FieldConstant.OPERATOR_SUCCESS);
        apiOutwardService.getBaseMapper().insert(apiOutward);
        redisService.setCacheObject(appId+"_"+appKey, apiOutward);
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS));
    }
    /**
     * 删除
     * @Author liy
     * @Date 2023/3/7 16:38
     * @param  apiOutward 实体类
     * @Return AjaxResult
     */
    @ApiOperation("删除APPid和APPkey")
    @PostMapping("api/del")
    public AjaxResult del(@RequestBody @ApiParam("API类")  ApiOutward apiOutward){
        System.out.println("删除之前redis中的数据:"+redisService.getCacheObject(apiOutward.getAppId()+"_"+apiOutward.getAppKey()));
        apiOutwardService.getBaseMapper().deleteById(apiOutward);
        redisService.deleteObject(apiOutward.getAppId()+"_"+apiOutward.getAppKey());
        System.out.println("删除之后redis中的数据:"+redisService.getCacheObject(apiOutward.getAppId()+"_"+apiOutward.getAppKey()));
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS));
    }

    /**
     * 修改状态
     * @Author liy
     * @Date 2023/3/8 16:38
     * @param  apiOutward 实体类
     * @Return AjaxResult
     */
    @ApiOperation("修改启用/禁用状态")
    @SystemLog(msg =FieldConstant.ACTIVITY_BANNER_BANNER,operation = FieldConstant.ACTIVITY_BANNER_PUBLISH)
    @PostMapping("api/changeStatus")
    public AjaxResult changeStatusOrDelUser(@RequestBody @ApiParam(name="状态",value="传整个对象,修改其中status字段,（0：禁用，1：=启用）",required=true) ApiOutward apiOutward){
     apiOutwardService.getBaseMapper().updateById(apiOutward);
        redisService.setCacheObject(apiOutward.getAppId()+"_"+apiOutward.getAppKey(), apiOutward);
        return AjaxResult.success(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS));
    }
}

