package com.ecell.internationalize.system.controller.api;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.ecell.internationalize.common.core.context.SecurityContextHolder;
import com.ecell.internationalize.common.core.entity.Rest;
import com.ecell.internationalize.common.core.utils.locale.LocaleUtil;
import com.ecell.internationalize.common.core.utils.uuid.UUID;
import com.ecell.internationalize.common.core.web.domain.R;
import com.ecell.internationalize.common.issue.entity.tcp.DistributeParent;
import com.ecell.internationalize.common.redis.service.RedisService;
import com.ecell.internationalize.common.system.constant.FieldConstant;
import com.ecell.internationalize.common.system.entity.HeartTemperatureRecord;
import com.ecell.internationalize.common.system.entity.UserDeviceBindApp;
import com.ecell.internationalize.system.entity.DeviceStepDaily;
import com.ecell.internationalize.system.entity.api.DeviceDisableApp;
import com.ecell.internationalize.system.entity.api.DeviceTimeSwitchApp;
import com.ecell.internationalize.system.entity.api.dto.*;
import com.ecell.internationalize.system.entity.api.vo.ApiDeviceInfo;
import com.ecell.internationalize.system.entity.api.vo.ApiDeviceStep;
import com.ecell.internationalize.system.entity.api.vo.ApiDeviceTemp;
import com.ecell.internationalize.system.entity.api.vo.DeviceDisableVo;
import com.ecell.internationalize.system.service.DeviceStepDailyService;
import com.ecell.internationalize.system.service.HeartTemperatureRecordService;
import com.ecell.internationalize.system.service.UserDeviceBindAppService;
import com.ecell.internationalize.system.service.api.DeviceAlarmAppService;
import com.ecell.internationalize.system.service.api.DeviceDisableAppService;
import com.ecell.internationalize.system.service.api.DeviceTimeSwitchAppService;
import com.ecell.internationalize.system.service.impl.DeviceInfoServiceImpl;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Title: StepController
 * @Author: liy
 * @Date: 2023/3/8 17:18
 * @Description:
 * @Version:1.0
 */
@RestController
@Api(value="API对外开放接口",tags={"API对外开放接口"})
@RequestMapping("/api")
public class StepController {
    @Autowired
    private DeviceStepDailyService deviceStepDailyService;
    @Autowired
    private HeartTemperatureRecordService heartTemperatureRecordService;
    @Autowired
    private UserDeviceBindAppService userDeviceBindAppService;
    @Autowired
    private RedisService redisService;
    @Autowired
    private DeviceInfoServiceImpl deviceInfoServiceImpl;
    @Autowired
    private DeviceDisableAppService deviceDisableAppService;
    @Autowired
    private DeviceAlarmAppService deviceAlarmAppService;
    @Autowired
    private DeviceTimeSwitchAppService deviceTimeSwitchAppService;

    /**
     * 设备步数条件分页查询
     *
     * @param stepPage 分页条件查询体
     * @Author liy
     * @Date 2022/7/12 10:38
     * @Return R
     */
    @ApiOperation("条件查询设备步数信息")
    @PostMapping("step/list")
    public R<List<ApiDeviceStep>> queryStepAll(@RequestBody StepPage stepPage) {

        LambdaQueryWrapper<DeviceStepDaily> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        if (null == stepPage.getImei() || null == stepPage.getStartDate() || null == stepPage.getEndDate()) {
            return R.fail(LocaleUtil.getMessage(FieldConstant.MESSAGES_ERROR));
        }
        lambdaQueryWrapper.eq(DeviceStepDaily::getImei, stepPage.getImei());
        lambdaQueryWrapper.ge(DeviceStepDaily::getCreateTime, stepPage.getStartDate());
        lambdaQueryWrapper.le(DeviceStepDaily::getCreateTime, stepPage.getEndDate());
        lambdaQueryWrapper.orderByDesc(DeviceStepDaily::getCreateTime);
        List<ApiDeviceStep> list = new ArrayList<>();
        ApiDeviceStep apiDeviceStep;
        List<DeviceStepDaily> dailyList = deviceStepDailyService.getBaseMapper().selectList(lambdaQueryWrapper);
        if (CollectionUtils.isEmpty(dailyList)) {
            return R.ok(list);
        }
        for (DeviceStepDaily deviceStepDaily : dailyList) {
            apiDeviceStep = new ApiDeviceStep();
            apiDeviceStep.setDate(deviceStepDaily.getCreateTime());
            apiDeviceStep.setNum(deviceStepDaily.getWalksNum());
            list.add(apiDeviceStep);
        }
        return R.ok(list, LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS));
    }

    /**
     * 体温查询
     *
     * @param tempPage 分页条件查询体
     * @Author liy
     * @Date 2022/7/12 10:38
     * @Return R
     */
    @ApiOperation("体温查询")
    @PostMapping("temp/list")
    public R<List<ApiDeviceTemp>> queryTempAll(@RequestBody TempPage tempPage) {
        LambdaQueryWrapper<HeartTemperatureRecord> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        if (null == tempPage.getImei() || null == tempPage.getStartDate() || null == tempPage.getEndDate()) {
            return R.fail(LocaleUtil.getMessage(FieldConstant.MESSAGES_ERROR));
        }
        lambdaQueryWrapper.eq(HeartTemperatureRecord::getImei, tempPage.getImei());
        lambdaQueryWrapper.ge(HeartTemperatureRecord::getCreateTime, tempPage.getStartDate());
        lambdaQueryWrapper.le(HeartTemperatureRecord::getCreateTime, tempPage.getEndDate());
        lambdaQueryWrapper.eq(HeartTemperatureRecord::getType, FieldConstant.SECOND_FLAG);
        lambdaQueryWrapper.orderByDesc(HeartTemperatureRecord::getCreateTime);
        List<ApiDeviceTemp> list = new ArrayList<>();
        ApiDeviceTemp apiDeviceTemp;
        List<HeartTemperatureRecord> temperatureRecordList = heartTemperatureRecordService.getBaseMapper().selectList(lambdaQueryWrapper);

        if (CollectionUtils.isEmpty(temperatureRecordList)) {
            return R.ok(list);
        }
        for (HeartTemperatureRecord heartTemperatureRecord : temperatureRecordList) {
            apiDeviceTemp = new ApiDeviceTemp();
            apiDeviceTemp.setDate(heartTemperatureRecord.getCreateTime());
            apiDeviceTemp.setData(heartTemperatureRecord.getHeartTemperatureRate());
            list.add(apiDeviceTemp);
        }
        return R.ok(list, LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS));
    }

    /**
     * 体温查询
     *
     * @param tempPage 分页条件查询体
     * @Author liy
     * @Date 2022/7/12 10:38
     * @Return R
     */
    @ApiOperation("心率查询")
    @PostMapping("rate/list")
    public R<List<ApiDeviceTemp>> queryRateAll(@RequestBody TempPage tempPage) {
        LambdaQueryWrapper<HeartTemperatureRecord> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        if (null == tempPage.getImei() || null == tempPage.getStartDate() || null == tempPage.getEndDate()) {
            return R.fail(LocaleUtil.getMessage(FieldConstant.MESSAGES_ERROR));
        }
        lambdaQueryWrapper.eq(HeartTemperatureRecord::getImei, tempPage.getImei());
        lambdaQueryWrapper.ge(HeartTemperatureRecord::getCreateTime, tempPage.getStartDate());
        lambdaQueryWrapper.le(HeartTemperatureRecord::getCreateTime, tempPage.getEndDate());
        lambdaQueryWrapper.eq(HeartTemperatureRecord::getType, FieldConstant.DEL_FLAG);
        lambdaQueryWrapper.orderByDesc(HeartTemperatureRecord::getCreateTime);
        List<ApiDeviceTemp> list = new ArrayList<>();
        ApiDeviceTemp apiDeviceTemp;
        List<HeartTemperatureRecord> temperatureRecordList = heartTemperatureRecordService.getBaseMapper().selectList(lambdaQueryWrapper);

        if (CollectionUtils.isEmpty(temperatureRecordList)) {
            return R.ok(list);
        }
        for (HeartTemperatureRecord heartTemperatureRecord : temperatureRecordList) {
            apiDeviceTemp = new ApiDeviceTemp();
            apiDeviceTemp.setDate(heartTemperatureRecord.getCreateTime());
            apiDeviceTemp.setData(heartTemperatureRecord.getHeartTemperatureRate());
            list.add(apiDeviceTemp);
        }
        return R.ok(list, LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS));
    }

    @ApiOperation(value = "获取设备联系人")
    @GetMapping("/query/deviceInfo")
    public Rest<List<ApiDeviceInfo>> queryDeviceInfo(@ApiParam(value = "设备imei号", required = true) String imei) {
        List<ApiDeviceInfo> apiDeviceInfoList = new ArrayList<>();
        ApiDeviceInfo apiDeviceInfo;
        LambdaQueryWrapper<UserDeviceBindApp> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(UserDeviceBindApp::getImei, imei);
        List<UserDeviceBindApp> userDeviceBindAppList = userDeviceBindAppService.getBaseMapper().selectList(lambdaQueryWrapper);
        if (!CollectionUtils.isEmpty(userDeviceBindAppList)) {
            for (UserDeviceBindApp userDeviceBindApp : userDeviceBindAppList) {
                apiDeviceInfo = new ApiDeviceInfo();
                apiDeviceInfo.setDate(userDeviceBindApp.getCreateTime());
                apiDeviceInfo.setIdentity(userDeviceBindApp.getIdentity());
                apiDeviceInfo.setName(userDeviceBindApp.getRelation());
                apiDeviceInfo.setPhone(userDeviceBindApp.getPhone());
                apiDeviceInfoList.add(apiDeviceInfo);
            }
        }
        return Rest.ok(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS), apiDeviceInfoList);
    }

    @ApiOperation(value = "下发查找手表")
    @GetMapping("/find/queryDevice")
    public Rest queryDevice(@ApiParam(value = "设备imei号") String imei) {
        //1.开始下发查找手表指令
        //查询缓存设备状态
        String onlineStatus = redisService.getCacheMapValue(imei, "onlineStatus");
        //设备在线才下发指令
        if (FieldConstant.MATH_ONE.equals(onlineStatus)) {
            System.out.println("设备在线,开始下发");
            Date date = new Date(System.currentTimeMillis());
            DistributeParent deviceDisabledAppDto = new DistributeParent();
            deviceDisabledAppDto.setImei(imei);
            deviceDisabledAppDto.setOpenid(SecurityContextHolder.getStringUserId());
            deviceDisabledAppDto.setIdent("66666");
            deviceDisabledAppDto.setTime(date.getTime());
            deviceDisabledAppDto.setType("find");
            deviceDisabledAppDto.setVender("20000");
            deviceInfoServiceImpl.sendKafka(deviceDisabledAppDto);
            return Rest.ok(1);
        }

        return Rest.ok(2);
    }

    @ApiOperation(value = "查询上课禁用")
    @GetMapping("/query/disabled")
    public Rest<List<DeviceDisableVo>> queryDisabled(@ApiParam(value = "设备imei号", required = true) String imei) {
        LambdaQueryWrapper<DeviceDisableApp> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(DeviceDisableApp::getImei, imei);
        List<DeviceDisableApp> disableAppList = deviceDisableAppService.getBaseMapper().selectList(lambdaQueryWrapper);
        List<DeviceDisableVo> list =
                disableAppList.stream().map(
                        dis -> {
                            DeviceDisableVo disableVo = new DeviceDisableVo();
                            disableVo.setAlarmEndTime(dis.getAlarmEndTime());
                            disableVo.setAlarmStartTime(dis.getAlarmStartTime());
                            disableVo.setAlarmStatus(dis.getAlarmStatus());
                            disableVo.setAlarmWeek(dis.getAlarmWeek());
                            return disableVo;
                        }
                ).collect(Collectors.toList());

        return Rest.ok(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS), list);
    }

    @ApiOperation(value = "新增上课禁用闹钟")
    @PostMapping("/insert/disabled")
    public Rest addDisabled(@RequestBody SaveDeviceDisable saveDeviceDisable) {
        QueryWrapper<DeviceDisableApp> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("imei", saveDeviceDisable.getImei());
        //1.查询设置的闹钟个数
        int count = deviceDisableAppService.getBaseMapper().selectCount(queryWrapper);
        if (count >= 3) {
            return Rest.fail("4");
        }
        DeviceDisableApp deviceDisableApp = new DeviceDisableApp();
        String id = UUID.fastUUID().toString(true);
        deviceDisableApp.setId(id);
        deviceDisableApp.setCreateTime(new Date());
        deviceDisableApp.setAlarmStartTime(saveDeviceDisable.getAlarmStartTime());
        deviceDisableApp.setAlarmEndTime(saveDeviceDisable.getAlarmEndTime());
        deviceDisableApp.setAlarmWeek(saveDeviceDisable.getAlarmWeek());
        deviceDisableApp.setImei(saveDeviceDisable.getImei());
        //2.新增闹钟
        deviceDisableAppService.getBaseMapper().insert(deviceDisableApp);
        //4.开始指令下发
        deviceDisableAppService.sendToKafka(deviceDisableApp);
        return Rest.ok(1);
    }

    @ApiOperation(value = "修改上课禁用闹钟")
    @PostMapping("/update/disabled")
    public Rest updateDisabled(@RequestBody UpdateDeviceDisable updateDeviceDisable) {
        DeviceDisableApp deviceDisableApp = new DeviceDisableApp();
        deviceDisableApp.setId(updateDeviceDisable.getId());
        deviceDisableApp.setUpdateTime(new Date());
        deviceDisableApp.setAlarmStartTime(updateDeviceDisable.getAlarmStartTime());
        deviceDisableApp.setAlarmEndTime(updateDeviceDisable.getAlarmEndTime());
        deviceDisableApp.setAlarmWeek(updateDeviceDisable.getAlarmWeek());
        deviceDisableApp.setImei(updateDeviceDisable.getImei());
        deviceDisableAppService.updateById(deviceDisableApp);
        //2.开始指令下发
        deviceDisableAppService.sendToKafka(deviceDisableApp);
        return Rest.ok(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS));
    }

    @ApiOperation(value = "删除上课禁用")
    @PostMapping("/delete/disabled")
    public Rest deleteDisabled(@RequestBody DelDeviceDisable delDeviceDisable) {
        DeviceDisableApp deviceDisableApp = new DeviceDisableApp();
        deviceDisableApp.setImei(delDeviceDisable.getImei());
        deviceDisableApp.setId(delDeviceDisable.getId());
        deviceDisableAppService.getBaseMapper().deleteById(deviceDisableApp);
        //2.开始指令下发
        deviceDisableAppService.sendToKafka(deviceDisableApp);
        return Rest.ok(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS));
    }

    @ApiOperation(value = "查询闹钟")
    @GetMapping("/query/queryAlarm")
    public Rest<List<DeviceAlarmApp>> queryAlarm(@ApiParam(value = "设备imei号") String imei) {
        LambdaQueryWrapper<DeviceAlarmApp> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(DeviceAlarmApp::getImei, imei);
        return Rest.ok(deviceAlarmAppService.getBaseMapper().selectList(lambdaQueryWrapper));
    }

    @ApiOperation(value = "新增闹钟")
    @PostMapping("/insert/addAlarm")
    public Rest addAlarm( @RequestBody SaveDeviceAlarmDto saveDeviceAlarmDto) {
        QueryWrapper<DeviceAlarmApp> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("imei", saveDeviceAlarmDto.getImei());
        //1.查询设置的闹钟个数
        int count = deviceAlarmAppService.getBaseMapper().selectCount(queryWrapper);
        if (count >= 3) {
            return Rest.ok(3);
        }
        String id = com.ecell.internationalize.common.core.utils.uuid.UUID.fastUUID().toString(true);
        DeviceAlarmApp deviceAlarmApp = new DeviceAlarmApp();
        deviceAlarmApp.setId(id);
        deviceAlarmApp.setImei(saveDeviceAlarmDto.getImei());
        deviceAlarmApp.setAlarmTime(saveDeviceAlarmDto.getAlarmTime());
        deviceAlarmApp.setTag(saveDeviceAlarmDto.getTag());
        deviceAlarmApp.setAlarmWeek(saveDeviceAlarmDto.getAlarmWeek());
        deviceAlarmApp.setCreateTime(new Date());
        //2.新增闹钟
        deviceAlarmAppService.getBaseMapper().insert(deviceAlarmApp);
        //3.开始指令下发
        deviceAlarmAppService.sendToKafka(deviceAlarmApp);
        return Rest.ok(1);
    }

    @ApiOperation(value = "修改闹钟")
    @PostMapping("/update/updateAlarm")
    public Rest updateAlarm(@RequestBody UpdateDeviceAlarmDto updateDeviceAlarmDto) {
        DeviceAlarmApp deviceAlarmApp = new DeviceAlarmApp();
        deviceAlarmApp.setId(updateDeviceAlarmDto.getId());
        deviceAlarmApp.setImei(updateDeviceAlarmDto.getImei());
        deviceAlarmApp.setAlarmTime(updateDeviceAlarmDto.getAlarmTime());
        deviceAlarmApp.setTag(updateDeviceAlarmDto.getTag());
        deviceAlarmApp.setAlarmWeek(updateDeviceAlarmDto.getAlarmWeek());
        deviceAlarmApp.setUpdateTime(new Date());
        //1.修改闹钟信息
        deviceAlarmAppService.updateById(deviceAlarmApp);
        //2.开始指令下发
        deviceAlarmAppService.sendToKafka(deviceAlarmApp);
        return Rest.ok(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS));
    }

    @ApiOperation(value = "删除闹钟")
    @PostMapping("/delete/deleteAlarm")
    public Rest deleteAlarm(@RequestBody DelDeviceAlarmDto delDeviceAlarmDto) {
        DeviceAlarmApp deviceAlarmApp = new DeviceAlarmApp();
        deviceAlarmApp.setId(delDeviceAlarmDto.getId());
        deviceAlarmApp.setImei(delDeviceAlarmDto.getImei());
        //1.删除闹钟信息
        deviceAlarmAppService.getBaseMapper().deleteById(deviceAlarmApp);
        //2.开始指令下发
        deviceAlarmAppService.sendToKafka(deviceAlarmApp);
        return Rest.ok(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS));
    }
    @ApiOperation(value ="保存定时开关机设置")
    @PostMapping("/switch/addSetting")
    public Rest updateAlarm(@RequestBody DeviceTimeSwitchApp deviceTimeSwitchApp) {
        LambdaQueryWrapper<DeviceTimeSwitchApp> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(DeviceTimeSwitchApp::getImei,deviceTimeSwitchApp.getImei());
        DeviceTimeSwitchApp switchApp = deviceTimeSwitchAppService.getBaseMapper().selectOne(lambdaQueryWrapper);
        if (null!=switchApp){
            deviceTimeSwitchApp.setId(switchApp.getId());
            deviceTimeSwitchAppService.getBaseMapper().updateById(deviceTimeSwitchApp);
            //已存在,进行更新操作
        }else {
            //1.新增
            deviceTimeSwitchApp.setId(com.ecell.internationalize.common.core.utils.uuid.UUID.fastUUID().toString(true));
            deviceTimeSwitchAppService.getBaseMapper().insert(deviceTimeSwitchApp);
        }
        //2.开始指令下发
        deviceTimeSwitchAppService.sendToKafka(deviceTimeSwitchApp);
        return Rest.ok(LocaleUtil.getMessage(FieldConstant.MESSAGES_SUCCESS));
    }
    @ApiOperation(value ="查询定时开关机")
    @GetMapping("/switch/querySetting")
    public Rest<DeviceTimeSwitchApp> querySetting(HttpServletRequest request, @ApiParam("设备imei号")String imei) {
        LambdaQueryWrapper<DeviceTimeSwitchApp> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(DeviceTimeSwitchApp::getImei,imei);
        return Rest.ok(deviceTimeSwitchAppService.getBaseMapper().selectOne(lambdaQueryWrapper));
    }
}