package com.ecell.internationalize.system.mapper;


import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ecell.internationalize.system.entity.DeviceInfo;
import com.ecell.internationalize.system.entity.LatestLocationApp;
import com.ecell.internationalize.system.entity.dto.*;
import com.ecell.internationalize.system.entity.vo.*;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * <p>
 * 设备信息表 Mapper 接口
 * </p>
 *
 * @author ${author}
 * @since 2022-07-10
 */
public interface DeviceInfoMapper extends BaseMapper<DeviceInfo> {

    Page<DeviceActivaVO> queryActivationInfoPageList(@Param("page") Page page, @Param("dto") DeviceActivaPageDTO dto);

    /**
     * 根据批次分组查询审批通过的总数
     * @param page
     * @param dto
     * @return
     */
    Page<DeviceActivaVO> queryActivationInfoList(@Param("page") Page page, @Param("dto") DeviceActivaPageDTO dto);

    /**
     * 根据imeis查询最新位置信息表统计激活数据
     * @param list
     * @return
     */
    int queryActivateDeviceCount(@Param("list") List<String> list);
    /**
     * 根据imeis查询绑定表统计绑定数据
     * @param list
     * @return
     */
    int queryBindDeviceCount(@Param("list") List<String> list);

    void updateDeviceOwnerInfo(@Param("pId")String positioningModelId, @Param("mId")String deviceModelId);
    /**
     * 查询管理员id，设备拥有者名称，imei
     */
    List<DeviceAdminOwnerInfo> queryBingOrOwnerInfo(@Param("imei") List<String>imei);
    /**
     * 根据群组id 管理id 查询群组成员信息表
     */
    List<String> queryChatMemberById(@Param("groupId")String groupId,@Param("adminId")String adminId);

    /**
     * 查询已绑定的imei
     * @param list
     * @return
     */
    List<String> queryBindImeiByList(@Param("list") List<String>list);
    /**
     * 查询绑定表管理员id （暂时不用）
     */
    List<DeviceUserBingInfo> queryBindAdminInfoByImei(@Param("list") List<String>list);

    /**
     * 删除设备拥有者信息表deleteDeviceOwnerInfo
     * @param imei
     */
    void deleteDeviceOwnerInfo(@Param("imei") List<String>imei);

    /**
     * 删除绑定数据
     */
    void  deleteUserBindInfo(@Param("imei") List<String>imei);
    /**
     * 删除群组表
     */
    void  deleteChatGroup(@Param("imei") List<String>imei);

    /**
     * 根据imei查询聊天群组表ID
     * @param imei
     * @return
     */
    List<String> queryChatGroupIdByImei(@Param("imei") List<String>imei);
    /**
     * 删除聊天成员表
     */
    void deleteGroupMember(@Param("ids") List<String>ids);
    /**
     * 删除聊天信息表
     */
    void deleteMessage(@Param("ids") List<String>ids);
    /**
     * 删除通讯录表
     */
    void deleteAddressBook(@Param("imei") List<String>imei);
    /**
     * 删除申请关注记录表
     */
    void deleteDeviceAttention(@Param("imei") List<String>imei);
    /**
     * 删除最新位置表
     */
    void deleteLatestLocation(@Param("imei") List<String>imei);
    /**
     * 删除解析后的位置表
     */
    void deleteDeviceLocation(@Param("imei") List<String>imei);
    /**
     * 删除闹钟信息
     */
    void deleteAlarm(@Param("imei") List<String>imei);
    /**
     * 删除上课免打扰信息表
     */
    void deleteDeviceDisable(@Param("imei") List<String>imei);
    /**
     * 删除定时开关机信息表
     */
    void deleteDeviceTimeSwitch(@Param("imei") List<String>imei);
    /**
     * 删除安全信息围栏表
     */
    void deleteSafetyRail(@Param("imei") List<String>imei);
    /**
     * 删除SOS信息表
     */
    void deleteSOSInfo(@Param("imei") List<String>imei);
    /**
     * 删除消息提醒信息表
     */
    void deleteReminderMessage(@Param("imei") List<String>imei);
    /**
     * 删除可是视频信息表
     */
    void deleteVideoInfo(@Param("imei") List<String>imei);
    /**
     * 删除轨迹信息
     */
    void deleteTrackInfo(@Param("imei") List<String>imei);

    /**
     * 根据imei 日期查询轨迹信息
     * @param dto
     */
    List<TrackVO> queryTrackInfo(@Param("dto") TrackDTO dto);

    /**
     * 查询远程控制
     * @param page
     * @param dto
     * @return
     */
    Page<RemoteControlVO> queryRemoteControl(@Param("page")Page page, @Param("dto") RemoteControlDTO dto);

    /**
     * 查询设备功能信息
     * @param page
     * @param dto
     * @return
     */
    Page<EquipmentFunctionVO> queryEquipmentFunction(@Param("page") IPage page, @Param("dto")RemoteControlDTO dto);

    /**
     * 根据imei查询电话本信息
     * @param imei
     * @return
     */
    List<TelephoneBookVO> queryBookByImei(@Param("imei")String imei);

    /**
     * //根据imei查询闹钟信息
     * @param imei
     * @return
     */
    List<AlarmVO>queryAlarmByImei( @Param("imei")String imei);

    /**
     * //根据imei查询上课禁用信息
     * @param imei
     * @return
     */
    List<DisableVO>queryDisableByImei( @Param("imei")String imei);

    /**
     * //根据imei查询安全围栏信息
     * @param imei
     * @return
     */
    List<SafetyRailVO>querySafetyRaliByImei( @Param("imei")String imei);

    List<LatestLocationApp> queryByiMei(@Param("imei") String imei, @Param("iMeiList") List<String> iMeiList);

    List<LocationDto> locationTotal(@Param("iMeiList") List<String> iMeiList);

    List<LocationChildDto> provinceTotal(@Param("country") String country,@Param("iMeiList") List<String> iMeiList);
    /**
     * 查询设备激活数
     * @return
     */
    int queryDeviceActivate(@Param("dto")RemoteControlDTO dto);
    /**
     * 查询设备今日新增数量
     */
    int queryDeviceTodayAdd(@Param("dto")RemoteControlDTO dto);

    /**
     * 厂商列表录入统计专用
     * @param firmId
     * @param agentId
     * @return
     */
    List<String> queryFirmInputCountDevice(@Param("firmId") String firmId, @Param("agentId")String agentId);
    /**
     * 厂商列表录激活计专用
     * @param firmId
     * @param agentId
     * @return
     */
    int queryFirmActivateCountDevice(@Param("firmId") String firmId, @Param("agentId")String agentId);

    List<LocationFindDto> queryByCountryOrProvince(@Param("countryOrProvince") String countryOrProvince,@Param("type") String type,@Param("iMeiList") List<String> iMeiList);
    List<DeviceInfo> initInputBach();
}
