package com.ecell.internationalize.system.service.api.impl;
import com.ecell.internationalize.common.core.exception.ServiceException;
import com.ecell.internationalize.common.core.utils.StringUtils;
import com.ecell.internationalize.system.constant.DeviceModelConstants;
import com.ecell.internationalize.system.entity.api.*;
import com.ecell.internationalize.system.entity.dto.sendInstructionDTO;
import com.ecell.internationalize.system.mapper.api.ApiFirmMapper;
import com.ecell.internationalize.system.service.api.ApiFirmService;
import com.ecell.internationalize.system.service.impl.DeviceInfoServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;


/**
 * @author borui
 */
@Service
public class ApiFirmServiceImpl implements ApiFirmService {
    @Resource
   private ApiFirmMapper apiFirmMapper;

    @Autowired
    private DeviceInfoServiceImpl deviceInfoService;

    @Override
    public FirmDeviceInfo queryDeviceInfoByImei(String imei) {
        FirmDeviceInfo firmDeviceInfo = queryDeviceOwnerByImei(imei);
        if (StringUtils.isNotNull(firmDeviceInfo)){
            ApiLastLocationInfo apiLastLocationInfo = queryLastLocationInfoByImei(imei);
            if (StringUtils.isNotNull(apiLastLocationInfo)){
                firmDeviceInfo.setApiLastLocationInfo(apiLastLocationInfo);
            }

        }

        return firmDeviceInfo;
    }

    /**
     * 查询轨迹数据
     * @param po
     * @return
     */
    @Override
    public Object queryDeviceLocaltionList(LocationPO po) {
        if (!StringUtils.isNotEmpty(po.getImei())||!StringUtils.isNotEmpty(po.getStartTime())||!StringUtils.isNotEmpty(po.getEndTime())){
            throw  new ServiceException("messages.parameter.error");
        }
        ApiLocaltionDTO<ApiTrackInfo> apiLocaltionDTO =new ApiLocaltionDTO<>();

        if (po.getPage()<=0){
            po.setPage(1);
        }
        if (po.getNum()<=0){
            po.setNum(20);
        }
        int count = queryDeviceLocaltionCount(po);
        if (count>0){
            int pageCount = getPageCount(count, po.getNum());
            int startTotal = (po.getPage()-1)*(po.getNum());
            po.setStartTotal(startTotal);
            //查询轨迹信息

            apiLocaltionDTO.setSize(po.getNum());
            apiLocaltionDTO.setPageTotal(pageCount);
            apiLocaltionDTO.setTotal(count);
            List<ApiTrackInfo> apiTrackInfos = queryDeviceLocaltionInfo(po);
            apiLocaltionDTO.setList(apiTrackInfos);

        }

        return apiLocaltionDTO;
    }

    /**
     * 下发指令获取位置信息
     * @param imei
     * @return
     */
    @Override
    public  Map<String, String> getDeviceLocaltion(String imei) {
        if (!StringUtils.isNotEmpty(imei)){
            throw new ServiceException("messages.parameter.error");
        }
        sendInstructionDTO dto =new sendInstructionDTO();
        dto.setImei(imei);
        dto.setType(DeviceModelConstants.ONE);
        Map<String, String> map = deviceInfoService.sendInstructions(dto);

        return map;
    }

    /**
     * 根据imei查询设备拥有者信息
     */
    private FirmDeviceInfo queryDeviceOwnerByImei(String imei){
      return   apiFirmMapper.queryDeviceOwnerByImei(imei);

    }
    /**
     * 根据imei查询位置，体温心率最新信息
     */
    private ApiLastLocationInfo queryLastLocationInfoByImei(String imei){
        return   apiFirmMapper.queryLastLocationInfoByImei(imei);
    }

    /**
     * 查询轨迹总数
     * @param po
     * @return
     */
    private int queryDeviceLocaltionCount(LocationPO po){
      return   apiFirmMapper.queryDeviceLocaltionCount(po);
    }

    /**
     * 查询轨迹信息
     * @param po
     * @return
     */
    private List<ApiTrackInfo> queryDeviceLocaltionInfo(LocationPO po){
        return   apiFirmMapper.queryDeviceLocaltionInfo(po);
    }
    private int getPageCount(int total,int pageSize){
        return  total % pageSize== 0 ? (total/pageSize):(total/pageSize)+1;
    }
}
