package com.ecell.internationalize.system.service.api.impl;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ecell.internationalize.common.core.context.SecurityContextHolder;
import com.ecell.internationalize.common.core.utils.SpringUtils;
import com.ecell.internationalize.common.issue.component.IssueInstructions;
import com.ecell.internationalize.common.issue.entity.issuedto.DeviceDisabledAppDto;
import com.ecell.internationalize.common.issue.entity.issuedto.DeviceDisabledInnerAppDto;
import com.ecell.internationalize.system.entity.api.DeviceDisableApp;
import com.ecell.internationalize.system.mapper.api.DeviceDisableAppMapper;
import com.ecell.internationalize.system.service.api.DeviceDisableAppService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;


/**
 * <p>
 * 上课禁用(免打扰设置)表 服务实现类
 * </p>
 *
 * @author ${author}
 * @since 2022-09-13
 */
@Service
@Slf4j
public class DeviceDisableAppServiceImpl extends ServiceImpl<DeviceDisableAppMapper, DeviceDisableApp> implements DeviceDisableAppService {
    @Autowired
    private DeviceDisableAppMapper deviceDisableAppMapper;

    /**
     *  上课禁用封装数据下发指令
     * @Author: liy
     * @Date: 2022/9/15 10:26
     * @Description:
     * @Version:1.0
     */
    @Override
    public void sendToKafka(DeviceDisableApp deviceDisableApp) {

        Date date = new Date(System.currentTimeMillis());

        DeviceDisabledAppDto deviceDisabledAppDto=new DeviceDisabledAppDto();
        deviceDisabledAppDto.setType("disturb");
        deviceDisabledAppDto.setIdent(66666);
        deviceDisabledAppDto.setVender("20000");
        deviceDisabledAppDto.setOpenid(SecurityContextHolder.getStringUserId());
        deviceDisabledAppDto.setImei(deviceDisableApp.getImei());
        deviceDisabledAppDto.setTime(date.getTime());

        //根据imei查询所有闹钟信息，进行全量下发
        LambdaQueryWrapper<DeviceDisableApp> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(DeviceDisableApp::getImei,deviceDisableApp.getImei());
        List<DeviceDisableApp> deviceDisableAppList = deviceDisableAppMapper.selectList(lambdaQueryWrapper);
        //下发指令集合
        List<DeviceDisabledInnerAppDto> list=new ArrayList<>();
        DeviceDisabledInnerAppDto deviceDisabledInnerAppDto;
        if (!CollectionUtils.isEmpty(deviceDisableAppList)){
            for (DeviceDisableApp deviceDisableApp1:deviceDisableAppList){
                deviceDisabledInnerAppDto=new DeviceDisabledInnerAppDto();
                deviceDisabledInnerAppDto.setStatus(Integer.valueOf(deviceDisableApp1.getAlarmStatus()));
                deviceDisabledInnerAppDto.setStart(deviceDisableApp1.getAlarmStartTime());
                deviceDisabledInnerAppDto.setEnd(deviceDisableApp1.getAlarmEndTime());
                deviceDisabledInnerAppDto.setWeek(deviceDisableApp1.getAlarmWeek());
                list.add(deviceDisabledInnerAppDto);
            }
        }
        deviceDisabledAppDto.setDisturb(list);
        String object = JSONObject.toJSONString(deviceDisabledAppDto);
        log.info("转换的JSON数据为:{}",object);
        IssueInstructions issueInstructions = SpringUtils.getBean(IssueInstructions.class);
        //3.调用方法缓存入redis，开始指令下发
        issueInstructions.emitDeviceDemand("disturb",deviceDisableApp.getImei(),deviceDisabledAppDto);

    }

}
