package com.ecell.internationalize.system.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ecell.internationalize.common.system.constant.FieldConstant;
import com.ecell.internationalize.system.entity.Activity;
import com.ecell.internationalize.system.mapper.ActivityMapper;
import com.ecell.internationalize.system.service.ActivityService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.Map;

/**
 * <p>
 * 活动 服务实现类
 * </p>
 *
 * @author liy
 * @since 2022-07-13
 */
@Service
public class ActivityServiceImpl extends ServiceImpl<ActivityMapper, Activity> implements ActivityService {
    @Autowired
    private ActivityMapper activityMapper;
    /**
     * 活动分页查询
     * @Author liy
     * @Date 2022/7/14 16:42
     * @param map 分页查询体
     * @return IPage
     */
    @Override
    public IPage<Activity> findAllByPage(Map<String, Object> map) {
        Page<Activity> page=new Page<>(Integer.parseInt(map.get(FieldConstant.CURRENT).toString()),Integer.parseInt(map.get(FieldConstant.SIZE).toString()));
        LambdaQueryWrapper<Activity> queryWrapper = conditions(map);
        return activityMapper.selectPage(page,queryWrapper);
    }
    /**
     * 往期活动分页查询
     * @Author liy
     * @Date 2022/7/14 16:42
     * @param map 分页查询体
     * @return IPage
     */
    @Override
    public IPage<Activity> queryLastListByPage(Map<String, Object> map) {
        Page<Activity> page=new Page<>(Integer.parseInt(map.get(FieldConstant.CURRENT).toString()),Integer.parseInt(map.get(FieldConstant.SIZE).toString()));
        LambdaQueryWrapper<Activity> queryWrapper = conditions(map);
        queryWrapper.lt(Activity::getExpiryDate,new Date());
        return activityMapper.selectPage(page,queryWrapper);
    }
    private LambdaQueryWrapper<Activity> conditions(Map<String, Object> map){
        LambdaQueryWrapper<Activity> queryWrapper = new LambdaQueryWrapper<>();
        if (map.containsKey(FieldConstant.ACTIVITY_TITLE)&&null!=map.get(FieldConstant.ACTIVITY_TITLE).toString()){
            queryWrapper.eq(Activity::getActivityTitle,map.get(FieldConstant.ACTIVITY_TITLE).toString());
        }
        if (map.containsKey(FieldConstant.STATUS)&&null!=map.get(FieldConstant.STATUS).toString()){
            queryWrapper.eq(Activity::getStatus,map.get(FieldConstant.STATUS).toString());
        }
        if (map.containsKey(FieldConstant.ACTIVITY_CATEGORY)&&null!=map.get(FieldConstant.ACTIVITY_CATEGORY).toString()){
            queryWrapper.eq(Activity::getActivityCategory,map.get(FieldConstant.ACTIVITY_CATEGORY).toString());
        }
        if (map.containsKey(FieldConstant.ACTIVITY_AGE)&&null!=map.get(FieldConstant.ACTIVITY_AGE).toString()){
            queryWrapper.eq(Activity::getActivityAge,map.get(FieldConstant.ACTIVITY_AGE).toString());
        }
        if (map.containsKey(FieldConstant.ACTIVITY_AREAS)&&null!=map.get(FieldConstant.ACTIVITY_AREAS).toString()){
            queryWrapper.eq(Activity::getActivityAreas,map.get(FieldConstant.ACTIVITY_AREAS).toString());
        }
        queryWrapper.eq(Activity::getDelFlag, FieldConstant.MATH_ONE);
        return queryWrapper;
    }
}
