package com.ecell.internationalize.system.service.impl;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ecell.internationalize.common.core.context.SecurityContextHolder;
import com.ecell.internationalize.common.core.exception.ServiceException;
import com.ecell.internationalize.common.core.utils.DateUtils;
import com.ecell.internationalize.common.core.utils.StringUtils;
import com.ecell.internationalize.common.core.utils.locale.LocaleUtil;
import com.ecell.internationalize.common.core.utils.uuid.UUID;
import com.ecell.internationalize.common.issue.entity.issuedto.ChatGroupDisbandMessageInfo;
import com.ecell.internationalize.common.issue.entity.tcp.DistributeParent;
import com.ecell.internationalize.common.issue.mqtt.AppSingleMqttClient;
import com.ecell.internationalize.common.redis.service.RedisService;
import com.ecell.internationalize.common.security.utils.SecurityUtils;
import com.ecell.internationalize.common.system.constant.FieldConstant;
import com.ecell.internationalize.common.system.entity.SysUser;
import com.ecell.internationalize.common.system.utlis.UploadUtil;
import com.ecell.internationalize.system.api.UserFirmApi;
import com.ecell.internationalize.system.constant.DeviceModelConstants;
import com.ecell.internationalize.system.entity.DeviceInfo;
import com.ecell.internationalize.system.entity.DeviceModelInfo;
import com.ecell.internationalize.system.entity.FirmManage;
import com.ecell.internationalize.system.entity.LatestLocationApp;
import com.ecell.internationalize.system.entity.dto.*;
import com.ecell.internationalize.system.entity.vo.*;
import com.ecell.internationalize.system.mapper.DeviceInfoMapper;
import com.ecell.internationalize.system.mapper.DeviceModelInfoMapper;
import com.ecell.internationalize.system.mapper.FirmManageMapper;
import com.ecell.internationalize.system.service.DeviceInfoService;
import com.ecell.internationalize.system.utils.DeviceModelInfoUtli;
import com.ecell.internationalize.system.utils.RandomUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.text.NumberFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 设备信息表 服务实现类
 * </p>
 *
 * @author ${author}
 * @since 2022-07-10
 */
@Service
public class DeviceInfoServiceImpl extends ServiceImpl<DeviceInfoMapper, DeviceInfo> implements DeviceInfoService {
    private static final Logger log = LoggerFactory.getLogger(DeviceInfoServiceImpl.class);
    private static final NumberFormat nf = NumberFormat.getNumberInstance();
    private static final String USER_MQTT_PREDIX = "iot08/user/";
    @Autowired
    private DeviceInfoMapper mapper;
    @Autowired
    private UserFirmApi api;
    @Autowired
    private FirmManageMapper firmManageMapper;
    @Autowired
    private DeviceModelInfoMapper deviceModelInfoMapper;
    @Autowired
    private RedisService redisService;
    @Autowired
    @Lazy
    private DeviceInfoServiceImpl deviceInfoServiceImpl;
    @Autowired
    private KafkaTemplate<String, Object> kafkaTemplate;

    @Override
    public boolean singleEntry(DeviceDTO dto) {
        //判断IMEI的长度
        if (dto.getImei().trim().length() != 15) {
            throw new ServiceException("IMEI长度有误");
        }
        //判断IMEI的长度
        if (!StringUtils.isNumeric(dto.getImei())) {
            throw new ServiceException("messagesParameterError");
        }
        //根据imei查询该imei有没有绑定过设备模型
        List<DeviceInfo> infos = checkAuditStatus(dto.getImei());
        if (StringUtils.isNotEmpty(infos)) {
            //"该设备正在审批中或者已审批通过不能重复添加"
            throw new ServiceException("messageDeviceAuditError");
        }
        DeviceInfo info = new DeviceInfo();
        info.setDeviceModelId(dto.getDeviceModelId());
        info.setFirmId(dto.getFirmId());
        info.setFirmName(dto.getFirmName());
        info.setDeviceModelName(dto.getDeviceModelName());
        info.setImei(dto.getImei());
        info.setAgentId(dto.getAgentId());
        info.setAgentName(dto.getAgentName());
        //设置id
        info.setDeviceId(com.ecell.internationalize.common.core.utils.uuid.UUID.fastUUID().toString(true));
        //设置录入标志
        info.setInputFlag(DeviceModelConstants.ZERO);
        //设置审核设备
        info.setAuditDevice(info.getImei());
        //设置导入数量
        info.setTotalImport(1);
        String inputBatch= DateUtils.dateTimeNow()+ RandomUtils.generateVerCode(6);
        //设置批次号
        info.setInputBatch(inputBatch);
        DeviceInfo info1 = queryInputBatch(inputBatch);
        if (StringUtils.isNotNull(info1)){
            return false;
        }
        return save(info);
    }



    /**
     * 批量导入
     *
     * @param userList
     * @param dto
     * @return
     */
    @Override
    public String importImei(List<DeviceDTO> userList, DeviceDTO dto) {
        List<DeviceInfo> infos = new ArrayList<>(userList.size());
        if (StringUtils.isNotEmpty(userList) && userList.size() > 0) {
            //设置批次号
            String inputBatch = DateUtils.dateTimeNow()+ RandomUtils.generateVerCode(6);
            DeviceInfo info1 = queryInputBatch(inputBatch);
            if (StringUtils.isNotNull(info1)){
                return "Batch import failed";
            }
            infos = userList.stream().map(DeviceDTO::getImei).filter(StringUtils::isNotEmpty).map(
                    s -> convertStringToClass(s, dto, userList.size(), inputBatch)).collect(Collectors.toList());
        }


        return checkImei(infos);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Object importImeiTwo(MultipartFile file, List<DeviceDTO> imeiList, DeviceDTO dto){

        List<DeviceInfo> infos = new ArrayList<>(imeiList.size());
        String inputBatchOne=null;
        if (StringUtils.isNotEmpty(imeiList) && imeiList.size() > 0) {
            //设置批次号
            String inputBatchTwo = DateUtils.dateTimeNow()+ RandomUtils.generateVerCode(6);
            inputBatchOne=inputBatchTwo;
            infos = imeiList.stream().map(DeviceDTO::getImei).filter(StringUtils::isNotEmpty).map(
                    s -> convertStringToClass(s, dto, imeiList.size(), inputBatchTwo)).collect(Collectors.toList());
        }

        return checkImeiTwo(file,infos,inputBatchOne);
    }

    /**
     * 分页查询
     *
     * @param dto
     * @return
     */
    @Override
    public IPage queryByPageList(DevicePageDTO dto) {
        IPage page = new Page(dto.getCurrent(), dto.getPageSize());
        Map<String, String> map = SecurityUtils.getFirmIdOrSecondFirmId(SecurityUtils.getUserLogin());
        QueryWrapper wrapper = new QueryWrapper<>();
        if (StringUtils.isNotEmpty(map)) {
            //是厂商
            if (StringUtils.isNotEmpty(map.get("firmFlag")) && DeviceModelConstants.ONE.equals(map.get("firmFlag"))) {
                DeviceModelInfoUtli.dataPermissions(wrapper, "firm_id", map.get("firmId"));
            } else {
                //代理商
                DeviceModelInfoUtli.dataPermissions(wrapper, "agent_id", map.get("firmId"));
            }
        }
        if (StringUtils.isNotEmpty(dto.getStatus())) {
            wrapper.eq("audit_status", dto.getStatus());
        }

        if (StringUtils.isNotEmpty(dto.getImei())) {
            wrapper.eq("imei", dto.getImei());
        }
        wrapper.eq("del_flag", DeviceModelConstants.ONE);
        wrapper.groupBy("input_batch");
//        if (DeviceModelConstants.TWO.equals(dto.getStatus())){
//            wrapper.orderByDesc("audit_time");
//        }else {
//            wrapper.orderByDesc("create_time");
//        }
        wrapper.orderByDesc("create_time");

        IPage page1 = mapper.selectPage(page, wrapper);

        this.convertIPage(page1);


        return page1;

    }

    /**
     * 根据批次号查询设备信息
     * @param dto
     * @return
     */
    @Override
    public IPage<DeviceInfo> queryInputBatchList(DevicePageDTO dto){
        IPage page = new Page(dto.getCurrent(), dto.getPageSize());
        QueryWrapper<DeviceInfo> wrapper =new QueryWrapper<>();
//        wrapper.eq("audit_status",dto.getStatus());
        wrapper.eq("del_flag",DeviceModelConstants.ONE);
        wrapper.eq("input_batch",dto.getInputBatch());
        wrapper.orderByDesc("create_time");
        return   baseMapper.selectPage(page,wrapper);
    }


    /**
     * 根据IMEI查询批次号（imei是唯一的）
     *
     * @param imei
     * @return
     */
    @Override
    public String queryInputBatchById(String imei) {
        QueryWrapper wrapper = new QueryWrapper<>();
        wrapper.eq("imei", imei);
        wrapper.eq("del_flag", DeviceModelConstants.ONE);
        //加上limit 不加如果有多条记录会抛异常
        wrapper.last("LIMIT 1");
        DeviceInfo info = mapper.selectOne(wrapper);
        if (StringUtils.isNotNull(info)) {
            return info.getInputBatch();
        }
        return "";
    }

    /**
     * 审批
     *
     * @param id
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean updateStatus(String id, String status) {
        DeviceInfo info = mapper.selectById(id);
        if (StringUtils.isNull(info)) {
            throw new ServiceException("此设备ID不存在");
        }
        //修改审批状态
        info.setAuditStatus(status);

        info.setReviewer(SecurityUtils.getUsername());
        //审核时间
        info.setAuditTime(new Date());
        //审核通过要存入缓存 App上传指令要用到
        if (DeviceModelConstants.TWO.equals(status)) {
            //根据厂商id,查询厂商编号
          //  FirmManage firmManage = firmManageMapper.selectById(info.getFirmId());
            //写死
            redisService.setCacheMapValue(info.getImei(), DeviceModelConstants.AGENT, DeviceModelConstants.TWO_ZERO);
        }
        return super.updateById(info);
    }

    @Override
    public boolean updateInputBachStatus(String inputBath,String status){
        //根据批次号查询设备信息
        QueryWrapper<DeviceInfo> wrapper =new QueryWrapper();
        wrapper.eq("input_batch",inputBath);
        wrapper.eq("audit_status",DeviceModelConstants.ONE);
        wrapper.eq("del_flag",DeviceModelConstants.ONE);
        List<DeviceInfo> list = baseMapper.selectList(wrapper);
        if (StringUtils.isNotEmpty(list) && list.size()>0){
            List<DeviceInfo> collect = list.stream().map(s -> {
                s.setAuditStatus(status);
                return s;
            }).collect(Collectors.toList());
           return super.updateBatchById(collect);

        }else {
            return false;
        }
    }
    @Override
    public List<DeviceInfo> initInputBach(){
        List<DeviceInfo> infos = baseMapper.initInputBach();
        if (StringUtils.isNotEmpty(infos) && infos.size()>0){
                 infos = infos.stream().map(s -> {
                if (StringUtils.isNotEmpty(s.getFirmId()) && StringUtils.isNotEmpty(s.getAgentId())) {
                    //查询代理商名称
                    FirmManage firmManage = firmManageMapper.selectById(s.getAgentId());
                    if (StringUtils.isNotNull(firmManage)) {
                        s.setAgentName(firmManage.getFirmName());
                    }
                } else {
                    //查询代理商名称
                    FirmManage firmManage = firmManageMapper.selectById(s.getFirmId());
                    if (StringUtils.isNotNull(firmManage)) {
                        s.setAgentName(firmManage.getFirmName());
                    }
                }

                return s;

            }).collect(Collectors.toList());
        }
        return infos;
    }

    /**
     * 根据imei 查询设备信息
     *
     * @param imei
     * @return
     */
    @Override
    public DeviceInfo queryDeviceByImei(String imei) {
        QueryWrapper<DeviceInfo> wrapper = new QueryWrapper<>();
        wrapper.eq("imei", imei);
        wrapper.eq("del_flag", DeviceModelConstants.ONE);
        wrapper.lambda().and(s -> s.eq(DeviceInfo::getAuditStatus, DeviceModelConstants.ONE).or().eq(DeviceInfo::getAuditStatus, DeviceModelConstants.TWO));
        //加上limit 不加如果有多条记录会抛异常
        wrapper.last("LIMIT 1");
        return mapper.selectOne(wrapper);
    }

    /**
     * 查询设备激活信息
     *
     * @param dto
     * @return
     */

    @Override
    public IPage queryActivationInfoPageList(DeviceActivaPageDTO dto) {
        //  Page<DeviceActivaVO> deviceActivation = mapper.queryActivationInfoPageList(page, dto);
        Page page = new Page(dto.getCurrent(), dto.getPageSize());
        Page<DeviceActivaVO> deviceActivation = mapper.queryActivationInfoList(page,dto);
        if (StringUtils.isNotNull(deviceActivation) && StringUtils.isNotEmpty(deviceActivation.getRecords()) && deviceActivation.getRecords().size() > 0) {
            List<DeviceActivaVO> collect = deviceActivation.getRecords().stream().filter(s-> null != s.getInputBatch()).map(this::calculation).collect(Collectors.toList());
            deviceActivation.setRecords(collect);

        }

        return deviceActivation;
    }

    /**
     * 单个解绑
     * @param imei
     * @param one
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean deviceUnBingByImei(String imei, String one) {
        //获取当前用户
        Map<String, String> map = SecurityUtils.getFirmIdOrSecondFirmId(SecurityUtils.getUserLogin());
        QueryWrapper<DeviceInfo> wrapper = new QueryWrapper<>();
        wrapper.eq("audit_status", DeviceModelConstants.TWO);
        wrapper.eq("imei", imei);
        wrapper.eq("del_flag", DeviceModelConstants.ONE);
        if (StringUtils.isNotEmpty(map)) {
            if (StringUtils.isNotEmpty(map.get("firmFlag")) && DeviceModelConstants.ONE.equals(map.get("firmFlag"))) {//是厂商
                DeviceModelInfoUtli.dataPermissions(wrapper, "firm_id", map.get("firmId"));
            } else {
                //代理商
                DeviceModelInfoUtli.dataPermissions(wrapper, "agent_id", map.get("firmId"));
            }
        }
        DeviceInfo info = baseMapper.selectOne(wrapper);
        if (!StringUtils.isNotNull(info)){
            // 该设备不可用,请检查该设备是否已经审批通过或已经进行了绑定
            throw new  ServiceException("messages.device.unBing.error");
        }
        List<String> list = Collections.singletonList(imei);
        log.info("开始解绑------->imei:{}",imei);
      return   deviceInfoServiceImpl.delDeviceByImei(list,DeviceModelConstants.ONE);

    }

    /**
     * 根据imei查询轨迹信息
     * @param dto
     * @return
     */
    @Override
    public List<TrackVO> queryTrackInfo(TrackDTO dto) {
        if(!StringUtils.isNotEmpty(dto.getImei()) || dto.getImei().length() !=15){
            throw  new ServiceException("IMEI长度有误");
        }
        //获取当前用户
        Map<String, String> map = SecurityUtils.getFirmIdOrSecondFirmId(SecurityUtils.getUserLogin());
        if (StringUtils.isNotEmpty(map)) {
            //是厂商
            if (StringUtils.isNotEmpty(map.get("firmFlag")) && DeviceModelConstants.ONE.equals(map.get("firmFlag"))) {
                dto.setFirmId(map.get("firmId"));
            } else {
                //代理商
                dto.setAgentId(map.get("firmId"));
            }
        }
        //根据imei查询设备信息
        QueryWrapper<DeviceInfo> wrapper =new QueryWrapper();
        wrapper.eq("imei",dto.getImei());
        wrapper.eq("audit_status",DeviceModelConstants.TWO);
        wrapper.eq("del_flag",DeviceModelConstants.ONE);
        if (StringUtils.isNotEmpty(dto.getFirmId())){
            wrapper.eq("firm_id",dto.getFirmId());
        }
        if (StringUtils.isNotEmpty(dto.getAgentId())){
            wrapper.eq("agent_id",dto.getAgentId());
        }
        wrapper.last("LIMIT 1");
        DeviceInfo info = baseMapper.selectOne(wrapper);
        if (StringUtils.isNull(info)){
            //此设备,不在您的设备中,或此设备未审批或此设备已经删除，请联系管理员
            throw new ServiceException("messages.imei.track.error");
        }


        return baseMapper.queryTrackInfo(dto);
    }

    /**
     * 设备远程控制查询
     * @param dto
     * @return
     */
    @Override
    public IPage queryRemoteControl(RemoteControlDTO dto) {
        Page page =new Page(dto.getCurrent(),dto.getPageSize());
        return baseMapper.queryRemoteControl(page, dto);
    }

    /**
     * 远程指令下发
     * @param dto
     * @return
     */
    @Override
    public Map<String, String> sendInstructions(sendInstructionDTO dto) {
        Map<String,String> map =new HashMap<>(2);
        try {
            Object onlineStatus = redisService.getCacheMapValue(dto.getImei(), "onlineStatus");
            map.put("onlineStatus",onlineStatus.toString());
        }catch (Exception e){
             map.put("onlineStatus",DeviceModelConstants.ZERO);
        }
        if (StringUtils.isNotEmpty(map.get("onlineStatus")) && DeviceModelConstants.ZERO.equals(map.get("onlineStatus"))){
            return map;
        }
        DistributeParent parent =new DistributeParent();
        parent.setIdent(RandomUtils.generateVerCode(6));
        parent.setImei(dto.getImei());
        parent.setOpenid(SecurityContextHolder.getStringUserId());
        parent.setVender(DeviceModelConstants.TWO_ZERO);
        parent.setTime(System.currentTimeMillis());
        //定位指令下发
        if (DeviceModelConstants.ONE.equals(dto.getType())){

            parent.setType("requestLocation");


          //远程关机指令下发
        }else if (DeviceModelConstants.TWO.equals(dto.getType())){
            parent.setType("PowerOff");

        }
        //下发指令
        sendKafka(parent);
        map.put("onlineStatus",DeviceModelConstants.ONE);
        return map;
    }
    /**
     * 查询设备功能信息
     * @param dto
     * @return
     */
    @Override
    public IPage queryEquipmentFunction(RemoteControlDTO dto) {
        IPage page = new Page(dto.getCurrent(), dto.getPageSize());
        Map<String, String> map = SecurityUtils.getFirmIdOrSecondFirmId(SecurityUtils.getUserLogin());
        if (StringUtils.isNotEmpty(map)) {
            //是厂商
            if (StringUtils.isNotEmpty(map.get("firmFlag")) && DeviceModelConstants.ONE.equals(map.get("firmFlag"))) {
                dto.setFirmId(dto.getFirmId());
            } else {
                //代理商
                dto.setAgentId(dto.getAgentId());
            }
        }
        Page<EquipmentFunctionVO> equipmentFunctionVOPage = baseMapper.queryEquipmentFunction(page, dto);
        if (StringUtils.isNotNull(equipmentFunctionVOPage) && StringUtils.isNotEmpty(equipmentFunctionVOPage.getRecords())&& equipmentFunctionVOPage.getRecords().size()>0){

            List<EquipmentFunctionVO> equipmentFunctionVOS = fillProperties(equipmentFunctionVOPage.getRecords());
            equipmentFunctionVOPage.setRecords(equipmentFunctionVOS);
        }

        return equipmentFunctionVOPage;
    }

    @Override
    public List<LatestLocationApp> queryByiMei(String imei) {
        String userId= SecurityContextHolder.getStringUserId();
        SysUser sysUser = firmManageMapper.queryFirmId(userId);
        if (FieldConstant.ADMIN.equals(userId) || "3".equals(sysUser.getFirmFlag())){
            //管理员或者超级厂商可查询所有设备位置信息
            return baseMapper.queryByiMei(imei,null);
        }
        LambdaQueryWrapper<DeviceInfo> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(DeviceInfo::getDelFlag,FieldConstant.DEL_FLAG);
        //厂商
        if (FieldConstant.MATH_ONE.equals(sysUser.getFirmFlag())){
            //如果该用户选择的是厂商，显示厂商以及厂商下面的所有代理商设备数据
            lambdaQueryWrapper.eq(DeviceInfo::getFirmId,sysUser.getFirmId());
        }
        else {
            lambdaQueryWrapper.eq(DeviceInfo::getAgentId,sysUser.getSecondFirmId());
        }
        List<DeviceInfo> deviceInfoList = mapper.selectList(lambdaQueryWrapper);
        if (CollectionUtils.isEmpty(deviceInfoList)){
            return null;
        }
        List<String> iMeiList= deviceInfoList.stream().map(DeviceInfo::getImei).collect(Collectors.toList());
        return baseMapper.queryByiMei(imei,iMeiList);
    }

    @Override
    public List<LocationDto> locationTotal() {
        String userId= SecurityContextHolder.getStringUserId();
        SysUser sysUser = firmManageMapper.queryFirmId(userId);
        if (FieldConstant.ADMIN.equals(userId) || "3".equals(sysUser.getFirmFlag())){
            //管理员或者超级厂商可查询所有设备位置信息
            List<LocationDto> locationDtoList = baseMapper.locationTotal(null);
            if (!CollectionUtils.isEmpty(locationDtoList)){
                for (LocationDto locationDto:locationDtoList){
                    List<LocationChildDto> childDtoList = baseMapper.provinceTotal(locationDto.getCountry(),null);
                    locationDto.setChildList(childDtoList);
                }
            }
            return locationDtoList;
        }
        LambdaQueryWrapper<DeviceInfo> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(DeviceInfo::getDelFlag,FieldConstant.DEL_FLAG);
        //厂商
        if (FieldConstant.MATH_ONE.equals(sysUser.getFirmFlag())){
            //如果该用户选择的是厂商，显示厂商以及厂商下面的所有代理商设备数据
            lambdaQueryWrapper.eq(DeviceInfo::getFirmId,sysUser.getFirmId());
        }
        else {
            lambdaQueryWrapper.eq(DeviceInfo::getAgentId,sysUser.getSecondFirmId());
        }
        List<DeviceInfo> deviceInfoList = mapper.selectList(lambdaQueryWrapper);
        if (CollectionUtils.isEmpty(deviceInfoList)){
            return null;
        }
        List<String> iMeiList= deviceInfoList.stream().map(DeviceInfo::getImei).collect(Collectors.toList());
        List<LocationDto> locationDtoList = baseMapper.locationTotal(iMeiList);
        if (!CollectionUtils.isEmpty(locationDtoList)){
            for (LocationDto locationDto:locationDtoList){
                List<LocationChildDto> childDtoList = baseMapper.provinceTotal(locationDto.getCountry(),iMeiList);
                locationDto.setChildList(childDtoList);
            }
        }
        return locationDtoList;

    }

    @Override
    public List<LocationFindDto> queryByCountryOrProvince(String countryOrProvince, String type) {
        String userId= SecurityContextHolder.getStringUserId();
        SysUser sysUser = firmManageMapper.queryFirmId(userId);
        if (FieldConstant.ADMIN.equals(userId) || "3".equals(sysUser.getFirmFlag())){
            //管理员或者超级厂商可查询所有设备位置信息
            List<LocationFindDto> locationFindDtoList = baseMapper.queryByCountryOrProvince(countryOrProvince, type,null);
            if (!CollectionUtils.isEmpty(locationFindDtoList)){
                for (LocationFindDto locationFindDto:locationFindDtoList){
                    String onlineStatus = null != redisService.getCacheMapValue(locationFindDto.getImei(), "onlineStatus")? redisService.getCacheMapValue(locationFindDto.getImei(), "onlineStatus").toString():"0";
                    locationFindDto.setOnLineStatus(onlineStatus);
                }
            }
            return locationFindDtoList;
        }
        LambdaQueryWrapper<DeviceInfo> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(DeviceInfo::getDelFlag,FieldConstant.DEL_FLAG);
        //厂商
        if (FieldConstant.MATH_ONE.equals(sysUser.getFirmFlag())){
            //如果该用户选择的是厂商，显示厂商以及厂商下面的所有代理商设备数据
            lambdaQueryWrapper.eq(DeviceInfo::getFirmId,sysUser.getFirmId());
        }
        else {
            lambdaQueryWrapper.eq(DeviceInfo::getAgentId,sysUser.getSecondFirmId());
        }
        List<DeviceInfo> deviceInfoList = mapper.selectList(lambdaQueryWrapper);
        if (CollectionUtils.isEmpty(deviceInfoList)){
            return null;
        }
        List<String> iMeiList= deviceInfoList.stream().map(DeviceInfo::getImei).collect(Collectors.toList());
        //管理员或者超级厂商可查询所有设备位置信息
        List<LocationFindDto> locationFindDtoList = baseMapper.queryByCountryOrProvince(countryOrProvince, type,iMeiList);
        if (!CollectionUtils.isEmpty(locationFindDtoList)){
            for (LocationFindDto locationFindDto:locationFindDtoList){
                String onlineStatus = null != redisService.getCacheMapValue(locationFindDto.getImei(), "onlineStatus")? redisService.getCacheMapValue(locationFindDto.getImei(), "onlineStatus").toString():"0";
                locationFindDto.setOnLineStatus(onlineStatus);
            }
        }

        return locationFindDtoList;
    }

    /**
     * 设备统计信息
     * @return
     */
    @Override
    public DeviceCountInfo queryDeviceNumberInfo() {
        RemoteControlDTO dto =new RemoteControlDTO();
        //获取当前用户
        Map<String, String> map = SecurityUtils.getFirmIdOrSecondFirmId(SecurityUtils.getUserLogin());
        QueryWrapper<DeviceInfo> wrapper =new QueryWrapper();
        if (StringUtils.isNotEmpty(map)) {
            //是厂商
            if (StringUtils.isNotEmpty(map.get("firmFlag")) && DeviceModelConstants.ONE.equals(map.get("firmFlag"))) {
                wrapper.eq("firm_id",map.get("firmId"));
                dto.setFirmId(map.get("firmId"));
            } else {
                //代理商
                wrapper.eq("agent_id",map.get("firmId"));
                dto.setAgentId(map.get("firmId"));
            }
        }
        wrapper.eq("audit_status",DeviceModelConstants.TWO);
        wrapper.eq("del_flag",DeviceModelConstants.ONE);
        List<DeviceInfo> infos = baseMapper.selectList(wrapper);
        DeviceCountInfo info =new DeviceCountInfo();
        if (StringUtils.isNotEmpty(infos) && infos.size()>0){
            List<String> collect = infos.parallelStream().map(DeviceInfo::getImei).collect(Collectors.toList());
            //查询今日新增数
            info.setDeviceTodayAddNumber(baseMapper.queryDeviceTodayAdd(dto));
           //查询激活设备数
           // int activateNumber = baseMapper.queryDeviceActivate(dto);
            int activateNumber=queryDeviceActivateCount(collect);
            info.setDeviceActivateNumber(activateNumber);
            //设备总数
            int size = infos.size();
            info.setDeviceTotal(size);
            //查询设备在线数
            int OnLineNumber= queryOneLineNumber(infos);
            info.setDeviceOnlineNumber(OnLineNumber);
            //查询设备离线数,
            info.setDeviceOfflineNumber(activateNumber-OnLineNumber);
            //设备未激活数,
            info.setDeviceNotActivateNumber(size-activateNumber);

        }else {
            info.setDeviceActivateNumber(0);
            info.setDeviceNotActivateNumber(0);
            info.setDeviceTotal(0);
            info.setDeviceOnlineNumber(0);
            info.setDeviceTodayAddNumber(0);
            info.setDeviceOfflineNumber(0);
        }

        return info;
    }
    //查询设备激活数
    private int queryDeviceActivateCount(List<String> list){
        int count=0;
        for (String s : list) {
            if (redisService.hasKey(s)){
                //在线
                if (StringUtils.isNotNull(redisService.getCacheMapValue(s,"firstLogin"))){
                    count++;
                }
            }
        }
        return count;

    }
    //查询设备在线数
    private int queryOneLineNumber(List<DeviceInfo> list){
        List<String> collect = list.parallelStream().map(DeviceInfo::getImei).collect(Collectors.toList());
        int count=0;
        for (String s : collect) {
            if (redisService.hasKey(s)){
                //在线
                if (DeviceModelConstants.ONE.equals(Optional.ofNullable(redisService.getCacheMapValue(s,"onlineStatus")).orElse(DeviceModelConstants.ZERO))){
                    count++;
                }
            }
        }
        return count;

    }


    /**
     * 填充属性
     * @param list
     */
    private List<EquipmentFunctionVO> fillProperties( List<EquipmentFunctionVO> list){
        return list.stream().map(this::fillEquipmentFunctionVOProperties).collect(Collectors.toList());

    }

    private EquipmentFunctionVO fillEquipmentFunctionVOProperties( EquipmentFunctionVO vo){
        String imei = vo.getImei();
        if (vo.getPositioningInterval()==0){
            vo.setPositioningInterval(10);
        }
        //根据imei查询电话本
        List<TelephoneBookVO> telephoneBookVOS = baseMapper.queryBookByImei(imei);
        //根据imei查询闹钟信息
        List<AlarmVO> alarmVOS = baseMapper.queryAlarmByImei(imei);
        //根据imei查询上课禁用信息
        List<DisableVO> disableVOS = baseMapper.queryDisableByImei(imei);
        //根据imei查询安全围栏信息
        List<SafetyRailVO> safetyRailVOS = baseMapper.querySafetyRaliByImei(imei);
        vo.setBooks(telephoneBookVOS);
        vo.setAlarms(alarmVOS);
        vo.setDisables(disableVOS);
        vo.setSafetyRail(safetyRailVOS);
        return vo;
    }

    /**
     * 根据id
     * 批量删除设备
     *
     * @param imeis
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean delDeviceByIds(List<String> imeis) {
        if (!StringUtils.isNotEmpty(imeis)) {
            throw new ServiceException("删除失败,数据不能为空");
        }
        return deviceInfoServiceImpl.delDeviceByImei(imeis, DeviceModelConstants.ZERO);


    }

    /**
     * 导入imei批量删除
     *
     * @param imeis
     * @param sign(0.就是删除设备，1就是解绑设备)
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean delDeviceByImei(List<String> imeis, String sign) {

        if (StringUtils.isNotEmpty(imeis) && imeis.size() > 0) {
            if (imeis.size() > 10000) {
                // 已超过10000条，请分批导入
                throw new ServiceException("messages.import.money.thousand.error");
            }
            //查询绑定表,该imei已经绑定，这里自己写SQL
            List<String> list = baseMapper.queryBindImeiByList(imeis);
            if (StringUtils.isNotEmpty(list) && list.size() > 200) {
                //  "绑定设备已超过200条，请分批导入"
                throw new ServiceException("messages.import.two.hundred.error");
            }
            //如果删除设备
            if (DeviceModelConstants.ZERO.equals(sign)) {
                QueryWrapper wrapper = new QueryWrapper();
                wrapper.eq("audit_status", DeviceModelConstants.TWO);
                wrapper.eq("del_flag", DeviceModelConstants.ONE);
                wrapper.in("imei", imeis);
                List<DeviceInfo> infos = mapper.selectList(wrapper);
                if (!StringUtils.isNotEmpty(infos) || infos.size() <= 0) {
                    throw new ServiceException("删除失败，请联系管理员");
                }
                //这里做的目的是减少字段的更新操作
                List<DeviceInfo> collect = infos.parallelStream().map(s -> {
                    DeviceInfo info = new DeviceInfo();
                    info.setDelFlag(DeviceModelConstants.ZERO);
                    info.setDeviceId(s.getDeviceId());
                    return info;
                }).collect(Collectors.toList());
                log.info("要删除的条数:{},实际删除的条数：{}，相差的条数：{}", imeis.size(), infos.size(), imeis.size() - infos.size());
                updateBatchById(collect);
            }
            log.info("查询绑定表的数据是:{}",list);
            if (StringUtils.isNotEmpty(list) && list.size() > 0) {
                //查询绑定表管理员ID
                List<DeviceAdminOwnerInfo> adminIds = baseMapper.queryBingOrOwnerInfo(imeis);

//                //查询群组表成员
//                if (StringUtils.isNotEmpty(adminIds) && adminIds.size() > DeviceModelConstants.NUMBER_ZERO) {
//                    for (DeviceAdminOwnerInfo info : adminIds) {
//                        String groupId = info.getGroupId();
//                        String adminId = info.getUserId();
//                        List<String> list1 =null;
//                        //根据群组ID and 管理员ID 查询群组成员信息表
//                        if (StringUtils.isNotEmpty(groupId)){
//                         list1 = baseMapper.queryChatMemberById(groupId, adminId);
//                        }
//                        if (StringUtils.isNotNull(list1)){
//                            info.setIds(list1);
//                        }
//
//                    }
//                    //删除设备绑定信息相关表,及redis缓存
//                    deviceInfoServiceImpl.deletaBingImeiInfo(list, adminIds);
//                }

                if (StringUtils.isNotEmpty(adminIds) && adminIds.size()>0){
                    List<DeviceAdminOwnerInfo> collect = adminIds.stream().map(s -> {
                        if (StringUtils.isNull(s)) {
                            String groupId = s.getGroupId();
                            String adminId = s.getUserId();
                            List<String> list1 = null;
                            //根据群组ID and 管理员ID 查询群组成员信息表
                            if (StringUtils.isNotEmpty(groupId)) {
                                list1 = baseMapper.queryChatMemberById(groupId, adminId);
                            }
                            if (StringUtils.isNotNull(list1)) {
                                s.setIds(list1);
                            }

                        }
                        return s;

                    }).collect(Collectors.toList());
                    //删除设备绑定信息相关表,及redis缓存
                   deviceInfoServiceImpl.deletaBingImeiInfo(list, collect);
                }

            }

        }


        return true;
    }

    /**
     * 删除设备绑定信息相关表,及redis缓存
     *
     * @param imei
     */
    public void deletaBingImeiInfo(List<String> imei, List<DeviceAdminOwnerInfo> adminIds) {

        //删除设备拥有者信息表deleteDeviceOwnerInfo
        baseMapper.deleteDeviceOwnerInfo(imei);
        // 删除绑定数据
        baseMapper.deleteUserBindInfo(imei);
        //查询群组id
        List<String> ids = baseMapper.queryChatGroupIdByImei(imei);
        baseMapper.deleteChatGroup(imei);
        baseMapper.deleteGroupMember(ids);
        baseMapper.deleteMessage(ids);
        baseMapper.deleteAddressBook(imei);
        baseMapper.deleteDeviceAttention(imei);
        baseMapper.deleteLatestLocation(imei);
        baseMapper.deleteDeviceLocation(imei);
        baseMapper.deleteAlarm(imei);
        baseMapper.deleteDeviceDisable(imei);
        baseMapper.deleteDeviceTimeSwitch(imei);
        baseMapper.deleteSafetyRail(imei);
        baseMapper.deleteSOSInfo(imei);
        baseMapper.deleteReminderMessage(imei);
        baseMapper.deleteVideoInfo(imei);
        baseMapper.deleteTrackInfo(imei);
        log.info("删除设备成功，开始下发指令到设备");
        //异步下发指令
        if (StringUtils.isNotEmpty(adminIds) && adminIds.size() > 0) {
            deviceInfoServiceImpl.sendIssueInstruction(adminIds);
        }
        // 异步返送信息给APP先不做
           // sendUnbindInfoToApp(adminIds);
    }

    /**
     * 下发指令
     */
    @Async
    public void sendIssueInstruction(List<DeviceAdminOwnerInfo> adminIds) {
        for (DeviceAdminOwnerInfo bingInfo : adminIds) {
            String imei = bingInfo.getImei();
            //获取设备状态是否在线
            String onlineStatus = redisService.getCacheMapValue(imei, "onlineStatus");
            //组装参数
            DistributeParent distributeParent = new DistributeParent();
            distributeParent.setTime(System.currentTimeMillis());
            distributeParent.setOpenid(bingInfo.getUserId());
            distributeParent.setType("reset");
            distributeParent.setVender("20000");
            distributeParent.setIdent(RandomUtils.generateVerCode(6));
            distributeParent.setImei(imei);
            restRedis(imei);
            //在线才下发
            if (DeviceModelConstants.ONE.equals(onlineStatus)) {
                sendKafka(distributeParent);
            }
        }


    }

    /**
     * 发送信息到APP
     */
    @Async
    public void sendUnbindInfoToApp(List<DeviceAdminOwnerInfo> infos) {
        log.info("异步发送信息,异步线程名称:{}", Thread.currentThread().getName());
        if (StringUtils.isNotEmpty(infos) && infos.size() > DeviceModelConstants.NUMBER_ZERO) {
            for (DeviceAdminOwnerInfo info : infos) {
                String groupId = info.getGroupId();
                String adminId = info.getUserId();
                String imei = info.getImei();
                String name = info.getName();
                //根据群组ID and 管理员ID 查询群组成员信息表
                List<String> list = info.getIds();
                if (StringUtils.isNotEmpty(list) && list.size() > 0) {
                    for (String s : list) {
                        ChatGroupDisbandMessageInfo message = new ChatGroupDisbandMessageInfo();
                        message.setType("chatGroupDisband");
                        message.setDisbandTime(new Date());
                        message.setName(name);
                        message.setImei(imei);
                        message.setOperator(adminId);
                        message.setGroupId(groupId);
                        String s1 = JSON.toJSONString(message);
                        String topic = USER_MQTT_PREDIX + s;
                        //发送到APP
                        AppSingleMqttClient.publishCustomMessage(2, s1, topic);
                    }
                }

            }

        }

    }

    /**
     * 恢复出厂设置
     *
     * @param imei
     */
    public void restRedis(String imei) {
        List<String> list = Arrays.asList("messageVoice", "messageVoiceSync", "centerPhone", "weather", "weather_time", "weather_adcode", "locMode_rate", "locMode_rateSync", "reject", "rejectSync", "dialSwitch",
                "dialSwitchSync", "volume", "volumeSync", "downSchoolTimeTable", "downSchoolTimeTableSync", "setHeartRate", "setHeartRateSync", "setBodyTemperature", "setBodyTemperatureSync",
                "setBootOff", "setBootOffSync", "setSetpSwitch", "setSetpSwitchSync", "ppmessage", "ppmessageSync", "sos", "sosSync", "familyPhoneList", "familyPhoneListSync", "whitePhoneList",
                "whitePhoneListSync", "studentInfo", "studentInfoSync");
        log.info("设备恢复出厂设置:{}", imei);
        //  delRedisKey(imei,list);
        if (redisService.deleteObject(imei)) {
            redisService.setCacheMapValue(imei, "contacts", "{\"last\":0,\"ident\":1616,\"contact\":[],\"index\":0,\"type\":'contact'}");
            redisService.setCacheMapValue(imei, "contactsSync", "1");
            redisService.setCacheMapValue(imei, "alarms", "{\"ident\":3030,\"alarm\":[],\"type\":'alarm'}");
            redisService.setCacheMapValue(imei, "alarmsSync", "1");
            redisService.setCacheMapValue(imei, "disBan", "{\"disturb\": [], \"ident\": 2828, \"type\": 'disturb'}");
            redisService.setCacheMapValue(imei, "disBanSync", "1");
            redisService.setCacheMapValue(imei, "onlineStatus", "0");
            redisService.setCacheMapValue(imei, "agent", "20000");
            redisService.setCacheMapValue(imei, "deviceHead", "YS");
        }

    }

    // 下发KAFKA
    public void sendKafka(Object object) {
        log.info("下发到kafka的数据信息:{}", object);
        kafkaTemplate.send(DeviceModelConstants.WATCHPUSH_KAFKA, JSON.toJSONString(object)).addCallback(success -> {
            log.info("发送到kafka具体信息{},partition 分区 {},offset {}", success.getRecordMetadata().topic(), success.getRecordMetadata().partition(), success.getRecordMetadata().offset());

        }, failure -> {
            //失败
            log.info("下发到kafka的数据信息失败：{}", failure.getMessage());
            log.info("发送到kafka具体信息{}", JSON.toJSONString(object));

        });


    }


    /**
     * 查询所有审核通过未删除的设备
     *
     * @return
     */
    @Override
    public List<DeviceInfo> queryApprovedDevice() {
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("audit_status", DeviceModelConstants.TWO);
        wrapper.eq("del_flag", DeviceModelConstants.ONE);

        return deviceModelInfoMapper.selectList(wrapper);
    }

    /**
     * 根据 imei 查询设备信息 对APP使用
     *
     * @return
     */
    @Override
    public DeviceInfo queryDeviceInfoByImei(String imei) {
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("imei", imei);
        wrapper.eq("del_flag", DeviceModelConstants.ONE);
        //加上limit 不加如果有多条记录会抛异常
        wrapper.last("LIMIT 1");
        return mapper.selectOne(wrapper);
    }



    private DeviceInfo convertStringToClass(String imei, DeviceDTO dto, int TotalImport, String inputBatch) {
        DeviceInfo info = new DeviceInfo();
        info.setDeviceModelId(dto.getDeviceModelId());
        info.setFirmId(dto.getFirmId());
        info.setFirmName(dto.getFirmName());
        info.setDeviceModelName(dto.getDeviceModelName());
        info.setImei(imei);
        info.setAgentId(dto.getAgentId());
        info.setAgentName(dto.getAgentName());
        //设置id
        info.setDeviceId(UUID.fastUUID().toString(true));
        //设置录入标志
        info.setInputFlag(DeviceModelConstants.ONE);
        //info.setAuditDevice(imei);
        //设置导入数量
        info.setTotalImport(TotalImport);
        //设置批次号
        info.setInputBatch(inputBatch);
        return info;

    }


    private String checkImei(List<DeviceInfo> imeiList) {
        int successNum = 0;
        int failureNum = 0;
        StringBuilder successMsg = new StringBuilder();
        StringBuilder failureMsg = new StringBuilder();
        if (StringUtils.isNull(imeiList) && imeiList.size() == 0) {
            throw new ServiceException("导入IMEI数据不能为空");
        }
        for (DeviceInfo dto : imeiList) {
            try {
                if (dto.getImei().trim().length() != 15) { // I18nUtil.getMessage(ServiceExceptionResultConstant.RESULT_MSG_MAP.get("messages.length.error"))
                    failureNum++;
                    failureMsg.append("<br/>" + failureNum + "、IMEI:" + dto.getImei() + LocaleUtil.getMessage("messages.length.error") + dto.getImei().length());
                    continue;
                }else if (!org.apache.commons.lang3.StringUtils.isNumeric((dto.getImei()))){
                    failureNum++;
                    failureMsg.append("<br/>" + failureNum + "、IMEI:" + dto.getImei() + LocaleUtil.getMessage("messages.incorrect.content.error"));
                    continue;
                } else {
                    if (StringUtils.isNotNull(queryDeviceByImei(dto.getImei()))) {
                        failureNum++;
                        failureMsg.append("<br/>" + failureNum + "、IMEI " + dto.getImei() + LocaleUtil.getMessage("messages.already.exists"));
                        continue;
                    } else {
                        this.save(dto);
                        successNum++;
                        successMsg.append("<br/>" + successNum + "、IMEI " +  dto.getImei() +LocaleUtil.getMessage("messages.import.success"));

                    }
                }


            } catch (Exception e) {
                failureNum++;
                String msg = "<br/>" + failureNum + "、IMEI " + dto.getImei() + LocaleUtil.getMessage("messages.import.failed");
                failureMsg.append(msg + e.getMessage());
                log.error(msg, e);

            }


        }

//        if (failureNum > 0)
//        {
//            failureMsg.insert(0, "很抱歉，导入失败！共 " + failureNum + " 条数据格式不正确，错误如下：");
//          //  throw new ServiceException(failureMsg.toString());
//            return  failureMsg.toString();
//        }
//        else
//        {
//            successMsg.insert(0, "恭喜您，数据已全部导入成功！共 " + successNum + " 条，数据如下：");
//            return successMsg.toString();
//        }

        if (failureNum > 0 && successNum > 0) {

            failureMsg.insert(0, LocaleUtil.getMessage("messages.sorry.import.success") + failureNum + LocaleUtil.getMessage("messages.data.import.failed") + successNum + LocaleUtil.getMessage("messages.data.import.success") + successNum + LocaleUtil.getMessage("messages.data.import.error"));
            return failureMsg.toString();
        } else if (failureNum > 0) {

            failureMsg.insert(0, LocaleUtil.getMessage("messages.data.import.common.failed") + failureNum + LocaleUtil.getMessage("messages.import.format.error"));
            return failureMsg.toString();

        } else {
            successMsg.insert(0, LocaleUtil.getMessage("messages.import.congratulations.success") + successNum + LocaleUtil.getMessage("messages.data.are"));
            return successMsg.toString();
        }

    }

    @Transactional(rollbackFor = Exception.class)
    public String checkImeiTwo(MultipartFile file,List<DeviceInfo> imeiList,String inputBatch)  {
        int successNum = 0;
        int failureNum = 0;
        StringBuilder successMsg = new StringBuilder();
        StringBuilder failureMsg = new StringBuilder();
        List list =new ArrayList();
        if (StringUtils.isNull(imeiList) && imeiList.size() == 0) {
            throw new ServiceException("导入IMEI数据不能为空");
        }
//        for (DeviceInfo dto : imeiList) {
//            try {
//                if (dto.getImei().trim().length() != 15) { // I18nUtil.getMessage(ServiceExceptionResultConstant.RESULT_MSG_MAP.get("messages.length.error"))
//                    failureNum++;
//                    failureMsg.append("<br/>" + failureNum + "、IMEI:" + dto.getImei() + I18nUtil.getMessage("messages.length.error") + dto.getImei().length());
//                } else {
//                    if (StringUtils.isNotNull(queryDeviceByImei(dto.getImei()))) {
//                        failureNum++;
//                        failureMsg.append("<br/>" + failureNum + "、IMEI " + dto.getImei() + I18nUtil.getMessage("messages.length.error"));
//                    } else {
//                        this.save(dto);
//                        successNum++;
//                        successMsg.append("<br/>" + successNum + "、IMEI " + I18nUtil.getMessage("messages.import.success"));
//
//                    }
//                }
//
//
//            } catch (Exception e) {
//                failureNum++;
//                String msg = "<br/>" + failureNum + "、IMEI " + dto.getImei() + I18nUtil.getMessage("messages.import.failed");
//                failureMsg.append(msg + e.getMessage());
//                log.error(msg, e);
//
//            }
//
//
//        }
        Iterator<DeviceInfo> iterator = imeiList.iterator();
        while (iterator.hasNext()){
            DeviceInfo dto = iterator.next();

            try {
                if (dto.getImei().trim().length() != 15) { // I18nUtil.getMessage(ServiceExceptionResultConstant.RESULT_MSG_MAP.get("messages.length.error"))
                    failureNum++;
                    failureMsg.append("<br/>" + failureNum + "、IMEI:" + dto.getImei() + LocaleUtil.getMessage("messages.length.error") + dto.getImei().length());
                    iterator.remove();
                    continue;
                }else if (!org.apache.commons.lang3.StringUtils.isNumeric((dto.getImei()))){
                    failureNum++;
                    failureMsg.append("<br/>" + failureNum + "、IMEI:" + dto.getImei() + LocaleUtil.getMessage("messages.incorrect.content.error"));
                    iterator.remove();
                    continue;
                } else {
                    if (StringUtils.isNotNull(queryDeviceByImei(dto.getImei()))) {
                        failureNum++;
                        failureMsg.append("<br/>" + failureNum + "、IMEI " + dto.getImei() + LocaleUtil.getMessage("messages.length.error"));
                        iterator.remove();
                        continue;
                    } else {
                        this.save(dto);
                        successNum++;
                        successMsg.append("<br/>" + successNum + "、IMEI " + LocaleUtil.getMessage("messages.import.success"));
                        list.add(dto);
                    }
                }



            } catch (Exception e) {
                failureNum++;
                iterator.remove();
                String msg = "<br/>" + failureNum + "、IMEI " + dto.getImei() + LocaleUtil.getMessage("messages.import.failed");
                failureMsg.append(msg + e.getMessage());
                log.error(msg, e);

            }

        }
        log.info("录入的设备数:{},实际录入的设备数:{}",imeiList.size(),list.size());
        if (list.size()==imeiList.size()){
            try {
                UploadUtil.uploadTwo(file,inputBatch);
            } catch (IOException e) {
                throw new ServiceException(e.getMessage());
            }
        }else {
            String originalFilename = file.getOriginalFilename();
            String substring = originalFilename.substring(originalFilename.lastIndexOf(".") + 1);



        }

        if (failureNum > 0 && successNum > 0) {

            failureMsg.insert(0, LocaleUtil.getMessage("messages.sorry.import.success") + failureNum + LocaleUtil.getMessage("messages.data.import.failed") + successNum + LocaleUtil.getMessage("messages.data.import.success") + successNum + LocaleUtil.getMessage("messages.data.import.error"));
            return failureMsg.toString();
        } else if (failureNum > 0) {

            failureMsg.insert(0, LocaleUtil.getMessage("messages.data.import.common.failed") + failureNum + LocaleUtil.getMessage("messages.import.format.error"));
            return failureMsg.toString();

        } else {
            successMsg.insert(0, LocaleUtil.getMessage("messages.import.congratulations.success") + successNum + LocaleUtil.getMessage("messages.data.are"));
            return successMsg.toString();
        }

    }


    private DeviceActivaVO calculation(DeviceActivaVO vo) {
        //根据批次查询出imei
        List<DeviceInfo> list = queryImeiListByInputBatch(vo.getInputBatch());
        if (StringUtils.isNotEmpty(list)){
            List<String> collect = list.stream().map(DeviceInfo::getImei).collect(Collectors.toList());
            //根据imei查询设备激活的数量（这里查询位置信息最新的表）
            // int i = baseMapper.queryActivateDeviceCount(collect);
            //激活数量
            int i= queryDeviceActivateCount(collect);
            vo.setActivateDevice(i);
            if (i==0){
                //激活数量为0，激活比例也为0
                vo.setActivationRatio("0");
            }else {
                double v = vo.getInputDevice().doubleValue();
                nf.setMaximumFractionDigits(2);
                vo.setActivationRatio(nf.format(i / v));
            }
            // 根据imei查询绑定设备数量（这里查询位置信息最新的表，设备绑定表）
            int bindCount=  baseMapper.queryBindDeviceCount(collect);
            vo.setWhetherBinding(bindCount);

        }
        return vo;
    }


    private List<DeviceInfo> queryImeiListByInputBatch(String inputBatch){
        QueryWrapper<DeviceInfo> wrapper =new QueryWrapper<>();
        wrapper.eq("input_batch",inputBatch);
        wrapper.eq("del_flag",DeviceModelConstants.ONE);
        wrapper.eq("audit_status",DeviceModelConstants.TWO);
       return baseMapper.selectList(wrapper);

    }

    /**
     * 厂商列表统计专用
     * @return
     */
    public DeviceActivaVO queryFirmCountDevice(String firmId,String agentId){
        DeviceActivaVO vo =new DeviceActivaVO();
        //根据厂商ID或代理商id查询设备统计信息
        List<String> list = baseMapper.queryFirmInputCountDevice(firmId, agentId);
        if (StringUtils.isNotEmpty(list) && list.size()>0){
            //设备录入数
            vo.setInputDevice(list.size());
            //设备激活数
            int i = queryDeviceActivateCount(list);
            vo.setActivateDevice(i);
        }
        return vo;
    }



    /**
     * 根据用户名称 查询 厂商，代理商ID
     *
     * @param username
     * @return
     */
    private List<String> queryFrimId(String username) {
        List<String> list = new ArrayList<>();
        SysUser sysUser = api.queryByUserName(username);
        if (StringUtils.isNotNull(sysUser)) {
            //是厂商
            if (DeviceModelConstants.ONE.equals(sysUser.getFirmFlag())) {
                //TODO 根据厂商查询下面的代理商
                QueryWrapper<FirmManage> wrapper = new QueryWrapper();
                wrapper.eq("parent_id", sysUser.getFirmId());
                List<FirmManage> firmManages = firmManageMapper.selectList(wrapper);
                if (StringUtils.isNotEmpty(firmManages) && firmManages.size() > 0) {
                    list = firmManages.stream().map(FirmManage::getFirmId).collect(Collectors.toList());
                }
                list.add(sysUser.getFirmId());

            } else {//是代理商
                //TODO 代理商直接看代理商的
                // list.add(sysUser.getFirmId());
                list.add(sysUser.getSecondFirmId());
            }
        }

        return list;


    }

    /***
     * 转换
     * @return
     */
    private void convertIPage(IPage page) {
        if (StringUtils.isNotNull(page) && page.getRecords().size() > 0) {

            //设备型号id
            List<DeviceInfo> records = page.getRecords();
            List<DeviceInfo> collect = records.stream().map(s -> {
                DeviceModelInfo info = deviceModelInfoMapper.selectById(s.getDeviceModelId());
                if (StringUtils.isNotNull(info)){
                    String s1 = DeviceModelInfoUtli.assembleFunDescription(info);
                    s.setFunDescription(s1);
                }
                //根据批次号查询录入总数
                List<DeviceInfo> infos = queryInputBatchListTwo(s.getInputBatch());
                if (StringUtils.isNotEmpty(infos) && infos.size()>0){
                    s.setInputSuccessTotal(infos.size());
                }

                return s;
            }).collect(Collectors.toList());

            page.setRecords(collect);

        }

    }

    /**
     * 根据imei查询该设备有没有正在审批或者审批通过
     *
     * @param imei
     */
    private List<DeviceInfo> checkAuditStatus(String imei) {
        QueryWrapper<DeviceInfo> wrapper = new QueryWrapper();
        wrapper.eq("imei", imei);
        wrapper.eq("del_flag", DeviceModelConstants.ONE);
        //查询待审核或者已审批的设备
        wrapper.lambda().and(s -> s.eq(DeviceInfo::getAuditStatus, DeviceModelConstants.ONE).or().eq(DeviceInfo::getAuditStatus, DeviceModelConstants.TWO));
        List<DeviceInfo> infos = baseMapper.selectList(wrapper);

        return infos;
    }

    /**
     * 根据批次号查询设备信息
     * @param inputBatch
     * @return
     */
    private DeviceInfo queryInputBatch(String inputBatch){
        QueryWrapper<DeviceInfo> wrapper =new QueryWrapper<>();
        wrapper.eq("input_batch",inputBatch);
        wrapper.eq("audit_status",DeviceModelConstants.ONE);
        wrapper.eq("del_flag",DeviceModelConstants.ONE);
        wrapper.last("LIMIT 1");
        return baseMapper.selectOne(wrapper);
    }

    /**
     * 根据批次号查询设备信息
     * @param inputBatch
     * @return
     */
    @Override
    public List<DeviceInfo> queryInputBatchListTwo(String inputBatch){
        QueryWrapper<DeviceInfo> wrapper =new QueryWrapper<>();
        wrapper.eq("input_batch",inputBatch);
        wrapper.eq("del_flag",DeviceModelConstants.ONE);
        return baseMapper.selectList(wrapper);
    }
}
