package com.ecell.internationalize.system.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ecell.internationalize.common.core.exception.ServiceException;
import com.ecell.internationalize.common.core.utils.StringUtils;
import com.ecell.internationalize.common.security.utils.SecurityUtils;
import com.ecell.internationalize.common.system.entity.PositioningModelInfo;
import com.ecell.internationalize.system.constant.DeviceModelConstants;
import com.ecell.internationalize.system.constant.DeviceModelEnum;
import com.ecell.internationalize.system.entity.DeviceInfo;
import com.ecell.internationalize.system.entity.DeviceModelInfo;
import com.ecell.internationalize.system.entity.FirmManage;
import com.ecell.internationalize.system.entity.dto.DeviceModelPageDTO;
import com.ecell.internationalize.system.entity.vo.DeviceModelVO;
import com.ecell.internationalize.system.mapper.DeviceInfoMapper;
import com.ecell.internationalize.system.mapper.DeviceModelInfoMapper;
import com.ecell.internationalize.system.mapper.FirmManageMapper;
import com.ecell.internationalize.system.mapper.PositioningModelInfoMapper;
import com.ecell.internationalize.system.service.DeviceModelInfoService;
import com.ecell.internationalize.system.utils.DeviceModelInfoUtli;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 设备型号管理信息表 服务实现类
 * </p>
 *
 * @author ${author}
 * @since 2022-07-09
 */
@Service
public class DeviceModelInfoServiceImpl extends ServiceImpl<DeviceModelInfoMapper, DeviceModelInfo> implements DeviceModelInfoService {
    @Autowired
    private DeviceModelInfoMapper mapper;
    @Autowired
    private FirmManageMapper firmManageMapper;
    @Autowired
    DeviceInfoMapper deviceInfoMapper;
    @Autowired
    PositioningModelInfoServiceImpl positioningModelInfoService;
    @Autowired
    PositioningModelInfoMapper positioningModelInfoMapper;

    /**
     * 设备型号名称唯一验证
     * @param model
     * @return
     */
    @Override
    public String checkModelNameUnique(DeviceModelInfo model) {
        QueryWrapper<DeviceModelInfo> wrapper =new QueryWrapper<>();
        wrapper.eq("device_model_name",model.getDeviceModelName());
        if (StringUtils.isNotNull(model) && StringUtils.isNotEmpty(model.getAgentId())){
            wrapper.eq("agent_id",model.getAgentId());
        }

        if (StringUtils.isNotNull(model) && StringUtils.isNotEmpty(model.getFirmId())){
            wrapper.eq("firm_id",model.getFirmId());
        }
        wrapper.eq("del_flag", DeviceModelConstants.ONE);
        //加上limit 不加如果有多条记录会抛异常
        wrapper.last("LIMIT 1");
        DeviceModelInfo modelInfo = mapper.selectOne(wrapper);
        if (StringUtils.isNotNull(modelInfo) && ! (modelInfo .getDeviceModelId().equals(model.getDeviceModelId())))
        {
            return DeviceModelConstants.NOT_UNIQUE;
        }
        return DeviceModelConstants.UNIQUE;
    }

    /**
     * 分页查询
     * @param dto
     * @return
     */
    @Override
    public IPage<DeviceModelVO> findListByPage(DeviceModelPageDTO dto) {
        QueryWrapper<DeviceModelInfo> wrapper =new QueryWrapper<>();
        Map<String,String> map = SecurityUtils.getFirmIdOrSecondFirmId(SecurityUtils.getUserLogin());
        if (StringUtils.isNotEmpty(map)){
            //是厂商
            if (StringUtils.isNotEmpty(map.get("firmFlag")) && DeviceModelConstants.ONE.equals(map.get("firmFlag"))){
                DeviceModelInfoUtli.dataPermissions(wrapper,"firm_id",map.get("firmId"));
            }else {
                //代理商
                DeviceModelInfoUtli.dataPermissions(wrapper,"agent_id",map.get("firmId"));
            }
        }
        //不为空
        if (StringUtils.isNotEmpty(dto.getFirmName())){
            if (DeviceModelConstants.ONE.equals(dto.getFirmFlag())){
                //如果是管理员
                if (SecurityUtils.isAdminString(SecurityUtils.getUserId())){
                    wrapper.eq("firm_id",dto.getFirmId());
                }
            }else {//代理商
                //如果是管理员
                if (SecurityUtils.isAdminString(SecurityUtils.getUserId())){
                    wrapper.eq("agent_id",dto.getFirmId());
                }

            }

        }
        if (StringUtils.isNotEmpty(dto.getDeviceModelName())){
            wrapper.eq("device_model_name",dto.getDeviceModelName());
        }
        wrapper.eq("del_flag",DeviceModelConstants.ONE);
        wrapper.orderByDesc("create_time");
        Page<DeviceModelInfo> page =new Page<DeviceModelInfo>(dto.getCurrent(),dto.getPageSize());
        IPage<DeviceModelInfo> oldPage = mapper.selectPage(page, wrapper);
        IPage<DeviceModelVO> newPage =new Page<>();

        return  this.transform(oldPage,newPage);
    }

    /**
     * 删除
     * @param id
     * @return
     */
    @Override
    public boolean deltaById(String id) {
        DeviceModelInfo info =new DeviceModelInfo();
        info.setDeviceModelId(id);
        info.setDelFlag(DeviceModelConstants.ZERO);
        //根据ID去查询设备表中有没有未审批，跟审批通过的,有的话不能删除
        int i = this.bindingDeviceCount(id);
        if (i>0){
            throw new ServiceException("此设备型号已经关联设备这些设备已经在审批中或已经审批通过不能删除");
        }

        return super.updateById(info);
    }

    /**
     * 根据厂商ID查询设备型号名称跟ID
     * @param id
     * @return
     */
    @Override
    public List<Map<String, String>> queryModelNameByFirmId(String id ,String type) {
        QueryWrapper wrapper =new QueryWrapper();
        if (StringUtils.isNotEmpty(id) && StringUtils.isNotEmpty(type)){
            if (DeviceModelConstants.ONE.equals(type)){
                //厂商
                wrapper.eq("firm_id",id);
            }else {
                //代理商
                wrapper.eq("agent_id",id);
            }
        }else {
            throw new ServiceException("参数错误");
        }

         wrapper.eq("del_flag",DeviceModelConstants.ONE);
        List<DeviceModelInfo> list = mapper.selectList(wrapper);
        if (StringUtils.isNotEmpty(list) && list.size()>0){
          return   convertClassToMap(list);
        }
        return Collections.emptyList();
    }

    /**
     * 根据设备型号MID and 定位模式pID 更新设备型号表的定位模式ID
     * @param mId
     * @param pId
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean updatePositionIdByModelId(String mId, String pId) {
       //先查询在更新
        DeviceModelInfo info = mapper.selectById(mId);
        if (StringUtils.isNull(info)){
            throw  new ServiceException("该设备型号ID有误");
        }
        info.setPositioningModeId(pId);
        //这里可能要去更新设备拥有者信息表的定位模型ID，先暂时不做
        deviceInfoMapper.updateDeviceOwnerInfo(pId,mId);
        return super.updateById(info);
    }

    /**
     * 根据设备模型id查询设备模型信息
     * @param id
     * @return
     */
    @Override
    public DeviceModelInfo queryById(String id) {

        return mapper.selectById(id);
    }


    private  List<Map<String,String>> convertClassToMap( List<DeviceModelInfo> list) {
        List<Map<String,String>> collect = list.stream().map(this::convertStringToMap).collect(Collectors.toList());
        return collect;
    }


    private Map<String,String> convertStringToMap(DeviceModelInfo info){
        Map<String,String> map =new HashMap<>(5);
        map.put("deviceModelId",info.getDeviceModelId());
        map.put("deviceModelName",info.getDeviceModelName());
        map.put("firmId",info.getFirmId());
        map.put("agentId",info.getAgentId());
        map.put("firmName",info.getFirmName());
        return map;
    }





    public IPage<DeviceModelVO> transform( IPage<DeviceModelInfo> oldPage, IPage<DeviceModelVO> newPage){
        newPage.setPages(oldPage.getPages());
        newPage.setCurrent(oldPage.getCurrent());
        newPage.setSize(oldPage.getSize());
        newPage.setTotal(oldPage.getTotal());
        if (StringUtils.isNotNull(oldPage) && StringUtils.isNotEmpty(oldPage.getRecords()) && oldPage.getRecords().size() > 0) {
            List<DeviceModelVO> collectVO = oldPage.getRecords().stream().map(this::getDeviceModelVO).collect(Collectors.toList());
                newPage.setRecords(collectVO);

        } else {
            newPage.setRecords(Collections.emptyList());
        }
        return newPage;

    }

    /**
     * 类装换
     * @param info
     * @return
     */
    public DeviceModelVO getDeviceModelVO(DeviceModelInfo info){
        DeviceModelVO vo = new DeviceModelVO();
        vo.setDeviceModelId(info.getDeviceModelId());
        vo.setDeviceModelName(info.getDeviceModelName());
        if (StringUtils.isNotEmpty(info.getAgentId())){//说明是代理商
            this.getFirmManage(vo,info.getAgentId());
        }else {//厂商
            this.getFirmManage(vo,info.getFirmId());
        }

        //TODO 不知道数据来源先坐为空处理
        vo.setFunModel(this.obtainFunModel(info));
        vo.setDeviceType(info.getDeviceType());
        vo.setOfficialAccountName(info.getOfficialAccountName());
        vo.setCreateTime(info.getCreateTime());
        vo.setCreateUser(info.getCreateUser());
        vo.setUpdateUser(info.getUpdateUser());
        //功能说明
        vo.setFunDescription(this.assembleFunDescription(info));
        vo.setInputTotal(getInputTotal(info.getDeviceModelId()));
        vo.setPositionModel(this.getPositionModel(info.getPositioningModeId()));
        return vo;

    }

    private void getFirmManage(DeviceModelVO vo,String id){
        FirmManage firmManage = firmManageMapper.selectById(id);
        if(StringUtils.isNotNull(firmManage)){
            vo.setFirmName(firmManage.getFirmName());
        }
    }

    /**
     * 组装功能模块代码
     * @return
     */
    //TODO 这里还没有写完，找不到规律
    private String obtainFunModel(DeviceModelInfo info){
        StringBuilder builder =new StringBuilder();
        //定位
        if (StringUtils.isNotEmpty(info.getLocationFun()) && info.getLocationFun().trim().length()==DeviceModelConstants.NUMBER_ONE){
            //定位功能
            String l = String.valueOf(Integer.parseInt(info.getLocationFun())-1);
            builder.append("L").append((l));
        }

        //视频类型名称
        if (StringUtils.isNotEmpty(info.getVideoModelName()) && !DeviceModelConstants.ZERO.equals(info.getVideoModelName()) && info.getVideoModelName().trim().length()==DeviceModelConstants.NUMBER_ONE){
            String v =String.valueOf(Integer.parseInt(info.getVideoModelName())-1);
            builder.append("V").append((v));
        }

        return builder.toString();
    }


    /**
     * 根据设备模型id查询设备总数（只查询审批通过，并且没有删除的设备总数）
     * @param deviceModelId
     * @return
     */
    private int getInputTotal(String deviceModelId){
        QueryWrapper<DeviceInfo> wrapper =new QueryWrapper();
        wrapper.eq("device_model_id",deviceModelId).eq("audit_status",DeviceModelConstants.TWO).eq("del_flag",DeviceModelConstants.ONE);
        List<DeviceInfo> infos = deviceInfoMapper.selectList(wrapper);
        if (StringUtils.isNotEmpty(infos) && infos.size()>0){
            return  infos.size();
        }
        return DeviceModelConstants.NUMBER_ZERO;
    }

    /**
     * 根据定位模型ID查询 定位模型信息
     * @param pid
     * @return
     */
    private String getPositionModel(String pid){
        if (StringUtils.isNotEmpty(pid)){
            //根据id 查询定位模式信息
            PositioningModelInfo positioningModelInfo = positioningModelInfoMapper.selectById(pid);
          return   positioningModelInfoService.classToMap(positioningModelInfo).get("name");
        //为null就使用默认值 默认的数据的id为1
        }else {
            PositioningModelInfo positioningModelInfo = positioningModelInfoMapper.selectById(DeviceModelConstants.ONE);
            return   positioningModelInfoService.classToMap(positioningModelInfo).get("name");
        }
    }

    /**
     * 根据设备模型ID查询设备表中有没有待审批跟审批通过的
     * @param deviceModelId
     * @return
     */
    private int bindingDeviceCount(String deviceModelId){
        QueryWrapper<DeviceInfo> wrapper =new QueryWrapper();
        wrapper.eq("device_model_id",deviceModelId);
        wrapper.eq("del_flag",DeviceModelConstants.ONE);
        wrapper.lambda().and(s->s.eq(DeviceInfo::getAuditStatus,DeviceModelConstants.ONE).or().eq(DeviceInfo::getAuditStatus,DeviceModelConstants.TWO));
        return deviceInfoMapper.selectCount(wrapper);
    }


    /**
     * 组装功能说明
     * @param model
     * @return
     */
    private String assembleFunDescription(DeviceModelInfo model){
        StringBuilder buffer =new StringBuilder();
        if (StringUtils.isNotEmpty(model.getLocationFun())){
            buffer.append(DeviceModelEnum.getLocationFunString(Integer.parseInt(model.getLocationFun()))).append(",");
        }
        //支持视频类型
        if (StringUtils.isNotEmpty(model.getVideoModelName()) && !DeviceModelConstants.ZERO.equals(model.getVideoModelName())){
            if (DeviceModelConstants.ONE.equals(model.getVideoModelName())){
                buffer.append(DeviceModelConstants.JU_FENG).append(DeviceModelConstants.VIDEO_CALL);
            }else if(DeviceModelConstants.TWO.equals(model.getVideoModelName())){
                buffer.append(DeviceModelConstants.SHENG_WANG).append(DeviceModelConstants.VIDEO_CALL);
            }else if(DeviceModelConstants.THREE.equals(model.getVideoModelName())){
                buffer.append(DeviceModelConstants.BAI_RUI).append(DeviceModelConstants.VIDEO_CALL);
            }else if(DeviceModelConstants.FOUR.equals(model.getVideoModelName())){
                buffer.append(DeviceModelConstants.TI_HU).append(DeviceModelConstants.VIDEO_CALL);
            }
            //视频规则
            if (StringUtils.isNotEmpty(model.getVideoRule())){
                //如果视频规则为0那就是默认配置
                if (DeviceModelConstants.ZERO.equals(model.getVideoRule())){
                    buffer.append(DeviceModelConstants.DEFAULT_CONFIG).append(",");
                }else {
                    buffer.append(":").append(model.getVideoRule()).append(",");
                }
            }
        }
        //手表闹钟
        if (StringUtils.isNotEmpty(model.getWatchAlarmClock()) && DeviceModelConstants.ONE.equals(model.getWatchAlarmClock())){
            buffer.append(DeviceModelConstants.WATCH_ALARM_CLOCK).append(",");
        }
        //上课禁用
        if (StringUtils.isNotEmpty(model.getClassDisable()) && DeviceModelConstants.ONE.equals(model.getClassDisable())){
            buffer.append(DeviceModelConstants.CLASS_DISABLE).append(",");
        }
        //SOS号码
        if (StringUtils.isNotEmpty(model.getSosPhone()) && DeviceModelConstants.ONE.equals(model.getSosPhone())){
            buffer.append(DeviceModelConstants.SOS_PHONE).append(",");
        }
        //电子围栏 electronicFence
        if (StringUtils.isNotEmpty(model.getElectronicFence()) && DeviceModelConstants.ONE.equals(model.getElectronicFence())){
            buffer.append(DeviceModelConstants.ELECTRONIC_FENCE).append(",");
        }
        //计步功能 stepRecorder
        if (StringUtils.isNotEmpty(model.getStepRecorder()) && DeviceModelConstants.ONE.equals(model.getStepRecorder())){
            buffer.append(DeviceModelConstants.STEP_RECORDER).append(",");
        }
        //微聊功能 chatFun
        if (StringUtils.isNotEmpty(model.getChatFun()) && DeviceModelConstants.ONE.equals(model.getChatFun())){
            buffer.append(DeviceModelConstants.CHAT_FUN).append(",");
        }

        //查找设备功能 lookupFun
        if (StringUtils.isNotEmpty(model.getLookupFun()) && DeviceModelConstants.ONE.equals(model.getLookupFun())){
            buffer.append(DeviceModelConstants.LOOKUP_FUN).append(",");
        }

        //手表WIFF watchWiff
        if (StringUtils.isNotEmpty(model.getWatchWiff()) && DeviceModelConstants.ONE.equals(model.getWatchWiff())){
            buffer.append(DeviceModelConstants.WATCH_WIFF).append(",");
        }

        // 心率 heartRate;
        if (StringUtils.isNotEmpty(model.getHeartRate()) && DeviceModelConstants.ONE.equals(model.getHeartRate())){
            buffer.append(DeviceModelConstants.HEART_RATE).append(",");
        }
        // 体温 temperature
        if (StringUtils.isNotEmpty(model.getTemperature()) && DeviceModelConstants.ONE.equals(model.getTemperature())){
            buffer.append(DeviceModelConstants.TEMPERATURE).append(",");
        }

        //血压 bloodPressure
        if (StringUtils.isNotEmpty(model.getBloodPressure()) && DeviceModelConstants.ONE.equals(model.getBloodPressure())){
            buffer.append(DeviceModelConstants.BLOOD_PRESSURE).append(",");
        }

        //血氧 bloodOxygen
        if (StringUtils.isNotEmpty(model.getBloodOxygen()) && DeviceModelConstants.ONE.equals(model.getBloodOxygen())){
            buffer.append(DeviceModelConstants.BLOOD_OXYGEN).append(",");
        }

        //白名单 whiteList
        if (StringUtils.isNotEmpty(model.getWhiteList()) && DeviceModelConstants.ONE.equals(model.getWhiteList())){
            buffer.append(DeviceModelConstants.WHITE_LIST).append(",");
        }

        //拨号盘 dial
        if (StringUtils.isNotEmpty(model.getDial()) && DeviceModelConstants.ONE.equals(model.getDial())){
            buffer.append(DeviceModelConstants.DIAL).append(",");
        }

        //定时开关机 timingSwitch
        if (StringUtils.isNotEmpty(model.getTimingSwitch()) && DeviceModelConstants.ONE.equals(model.getTimingSwitch())){
            buffer.append(DeviceModelConstants.TIMING_SWITCH).append(",");
        }

        //电信 telecomVersion
        if (StringUtils.isNotEmpty(model.getTelecomVersion()) && DeviceModelConstants.ONE.equals(model.getTelecomVersion())){
            buffer.append(DeviceModelConstants.TELECOM).append(",");
        }
        String s = buffer.toString();
        if(StringUtils.isNotEmpty(s) && s.length()>=2){
            s.substring(0,s.length()-2);
        }
        return s;
    }
}
