package com.ecell.internationalize.system.service.impl;
import com.ecell.internationalize.common.core.constant.UserConstants;
import com.ecell.internationalize.common.core.utils.StringUtils;
import com.ecell.internationalize.common.core.utils.uuid.UUID;
import com.ecell.internationalize.system.constant.DeviceModelConstants;
import com.ecell.internationalize.system.constant.DeviceModelEnum;
import com.ecell.internationalize.system.entity.DeviceModel;
import com.ecell.internationalize.system.entity.vo.DeviceModelVO;
import com.ecell.internationalize.system.mapper.DeviceModelMapper;
import com.ecell.internationalize.system.service.DeviceModelService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;


/**
 * 设备型号信息
 */
@Service
@Deprecated
public class DeviceModelServiceImpl implements DeviceModelService {
    @Autowired
    private DeviceModelMapper mapper;

    @Override
    public int insertDeviceModel(DeviceModel model) {

        model.setDeviceModelId(UUID.fastUUID().toString(true));
        return mapper.insertDeviceModel(model);
    }

    /**
     * 校验同一厂商，设备类型名称是否唯一
     */
    @Override
    public String checkModelNameUnique(DeviceModel model) {
        DeviceModel deviceModel = mapper.checkModelNameUnique(model.getFirmId(), model.getDeviceModelName());
        if (StringUtils.isNotNull(deviceModel) && !deviceModel.getFirmId().equals(model.getFirmId()))
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 根据条件分页查询设备型号
     * @param model
     * @return
     */
    @Override
    public List<DeviceModelVO> selectModelList(DeviceModel model) {
        List<DeviceModelVO> modelVOS =new ArrayList<>();
        List<DeviceModel> deviceModels = mapper.selectModelLists(model);
        if (StringUtils.isNotEmpty(deviceModels)){
            modelVOS=transform(deviceModels);
        }
        return modelVOS;
    }
    /**
     * 更新设备型号管理信息
     * @param model
     * @return
     */
    @Override
    public int updateDeviceModel(DeviceModel model) {
        return mapper.updateDeviceModel(model);
    }
    /**
     * 删除设备型号
     * @param id
     * @return
     */
    @Override
    public int deleteDeviceModelById(String id) {
        return mapper.deleteDeviceModelById(id);
    }

    private List<DeviceModelVO> transform(List<DeviceModel> models){
        List<DeviceModelVO> collect = models.stream().map(s -> {
            DeviceModelVO vo = new DeviceModelVO();
            vo.setDeviceModelName(s.getDeviceModelName());

            vo.setFirmName(s.getFirmName());

            vo.setFunModel("");
            vo.setDeviceType(s.getDeviceType());
            vo.setOfficialAccountName(s.getOfficialAccountName());
            vo.setCreateTime(s.getCreateTime());
           // vo.setCreateBy(s.getCreateBy());
            //功能说明
            vo.setFunDescription(assembleFunDescription(s));

            vo.setInputTotal(0);

            vo.setPositionModel("");
            return vo;

        }).collect(Collectors.toList());
        return collect ;
    }

    //组装功能说明
    private String assembleFunDescription(DeviceModel model){
        StringBuffer buffer =new StringBuffer();
        if (StringUtils.isNotEmpty(model.getLocationFun())){
            buffer.append(DeviceModelEnum.getLocationFunString(Integer.parseInt(model.getLocationFun()))).append(",");
        }
        //支持视频类型
        if (StringUtils.isNotEmpty(model.getVideoModelName()) && !DeviceModelConstants.ZERO.equals(model.getDeviceModelName())){
            buffer.append(model.getDeviceModelName()).append(DeviceModelConstants.VIDEO_CALL);
            //视频规则
            if (StringUtils.isNotEmpty(model.getVideoRule())){
                //如果视频规则为0那就是默认配置
                if (DeviceModelConstants.ZERO.equals(model.getVideoRule())){
                    buffer.append(DeviceModelConstants.DEFAULT_CONFIG).append(",");
                }else {
                    buffer.append(":").append(model.getVideoRule()).append(",");
                }
            }
        }
        //手表闹钟
        if (StringUtils.isNotEmpty(model.getWatchAlarmClock()) && DeviceModelConstants.ONE.equals(model.getWatchAlarmClock())){
            buffer.append(DeviceModelConstants.WATCH_ALARM_CLOCK).append(",");
        }
        //上课禁用
        if (StringUtils.isNotEmpty(model.getClassDisable()) && DeviceModelConstants.ONE.equals(model.getClassDisable())){
            buffer.append(DeviceModelConstants.CLASS_DISABLE).append(",");
        }
        //SOS号码
        if (StringUtils.isNotEmpty(model.getSosPhone()) && DeviceModelConstants.ONE.equals(model.getSosPhone())){
            buffer.append(DeviceModelConstants.SOS_PHONE).append(",");
        }
        //电子围栏 electronicFence
        if (StringUtils.isNotEmpty(model.getElectronicFence()) && DeviceModelConstants.ONE.equals(model.getElectronicFence())){
            buffer.append(DeviceModelConstants.ELECTRONIC_FENCE).append(",");
        }
        //计步功能 stepRecorder
        if (StringUtils.isNotEmpty(model.getStepRecorder()) && DeviceModelConstants.ONE.equals(model.getStepRecorder())){
            buffer.append(DeviceModelConstants.STEP_RECORDER).append(",");
        }
        //微聊功能 chatFun
        if (StringUtils.isNotEmpty(model.getChatFun()) && DeviceModelConstants.ONE.equals(model.getChatFun())){
            buffer.append(DeviceModelConstants.CHAT_FUN).append(",");
        }

        //查找设备功能 lookupFun
        if (StringUtils.isNotEmpty(model.getLookupFun()) && DeviceModelConstants.ONE.equals(model.getLookupFun())){
            buffer.append(DeviceModelConstants.LOOKUP_FUN).append(",");
        }

        //手表WIFF watchWiff
        if (StringUtils.isNotEmpty(model.getWatchWiff()) && DeviceModelConstants.ONE.equals(model.getWatchWiff())){
            buffer.append(DeviceModelConstants.WATCH_WIFF).append(",");
        }

        // 心率 heartRate;
        if (StringUtils.isNotEmpty(model.getHeartRate()) && DeviceModelConstants.ONE.equals(model.getHeartRate())){
            buffer.append(DeviceModelConstants.HEART_RATE).append(",");
        }
        // 体温 temperature
        if (StringUtils.isNotEmpty(model.getTemperature()) && DeviceModelConstants.ONE.equals(model.getTemperature())){
            buffer.append(DeviceModelConstants.TEMPERATURE).append(",");
        }

        //血压 bloodPressure
        if (StringUtils.isNotEmpty(model.getBloodPressure()) && DeviceModelConstants.ONE.equals(model.getBloodPressure())){
            buffer.append(DeviceModelConstants.BLOOD_PRESSURE).append(",");
        }

        //血氧 bloodOxygen
        if (StringUtils.isNotEmpty(model.getBloodOxygen()) && DeviceModelConstants.ONE.equals(model.getBloodOxygen())){
            buffer.append(DeviceModelConstants.BLOOD_OXYGEN).append(",");
        }

        //白名单 whiteList
        if (StringUtils.isNotEmpty(model.getWhiteList()) && DeviceModelConstants.ONE.equals(model.getWhiteList())){
            buffer.append(DeviceModelConstants.WHITE_LIST).append(",");
        }

        //拨号盘 dial
        if (StringUtils.isNotEmpty(model.getDial()) && DeviceModelConstants.ONE.equals(model.getDial())){
            buffer.append(DeviceModelConstants.DIAL).append(",");
        }

        //定时开关机 timingSwitch
        if (StringUtils.isNotEmpty(model.getTimingSwitch()) && DeviceModelConstants.ONE.equals(model.getTimingSwitch())){
            buffer.append(DeviceModelConstants.TIMING_SWITCH).append(",");
        }

        //电信 telecomVersion
        if (StringUtils.isNotEmpty(model.getTelecomVersion()) && DeviceModelConstants.ONE.equals(model.getTelecomVersion())){
            buffer.append(DeviceModelConstants.TELECOM).append(",");
        }
        return buffer.toString();
    }

}
