package com.ecell.internationalize.system.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ecell.internationalize.common.core.utils.StringUtils;
import com.ecell.internationalize.common.redis.service.RedisService;
import com.ecell.internationalize.common.security.utils.SecurityUtils;
import com.ecell.internationalize.system.constant.DeviceModelConstants;
import com.ecell.internationalize.system.entity.DeviceUserInfo;
import com.ecell.internationalize.system.entity.dto.DeviceUserPageDTO;
import com.ecell.internationalize.system.entity.vo.DeviceUserInfoVO;
import com.ecell.internationalize.system.entity.vo.MemberInfoVO;
import com.ecell.internationalize.system.mapper.DeviceUserInfoMapper;
import com.ecell.internationalize.system.service.DeviceUserInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author ${author}
 * @since 2022-07-15
 */
@Service
public class DeviceUserInfoServiceImpl extends ServiceImpl<DeviceUserInfoMapper, DeviceUserInfo> implements DeviceUserInfoService {
    @Autowired
    RedisService redisService;

    @Override
    public IPage queryPageList(DeviceUserPageDTO dto) {
        Page page =new Page(dto.getCurrent(),dto.getPageSize());
        Page<DeviceUserInfo> deviceUserInfoPage = baseMapper.queryDeviceUserInfo(page, dto);
        convertIPage(deviceUserInfoPage);
        return page;

    }

    /**
     * 设备家庭成员
     * @param dto
     * @return
     */
    @Override
    public IPage queryPageFamilyList(DeviceUserPageDTO dto) {
        //获取当前用户
        Map<String, String> map = SecurityUtils.getFirmIdOrSecondFirmId(SecurityUtils.getUserLogin());
        if (StringUtils.isNotEmpty(map)) {
            //是厂商
            if (StringUtils.isNotEmpty(map.get("firmFlag")) && DeviceModelConstants.ONE.equals(map.get("firmFlag"))) {
              dto.setFirmId(map.get("firmId"));
            } else {
                //代理商
                dto.setAgentId(map.get("firmId"));
            }
        }
        //根据imei

        Page page =new Page(dto.getCurrent(),dto.getPageSize());
        Page<DeviceUserInfoVO> deviceUserInfoVOPage = baseMapper.queryPageFamilyList(page, dto);
        convertIPageFamilyInfo(deviceUserInfoVOPage);
        return deviceUserInfoVOPage;
    }


    /***
     * 转换
     * @return
     */
    private void convertIPage(IPage page){
        if (StringUtils.isNotNull(page) && page.getRecords().size()>0){
            //设备型号id
            List<DeviceUserInfo> records = page.getRecords();
            List<DeviceUserInfo> collect = records.stream().map(s->{
                try {
                    Object onlineStatus = Optional.ofNullable(redisService.getCacheMapValue(s.getImei(), "onlineStatus")).orElse(DeviceModelConstants.ZERO);
                    s.setStatus(onlineStatus.toString());
                }catch (Exception e){
                    s.setStatus(DeviceModelConstants.ZERO);
                }

                return s;
            }).collect(Collectors.toList());

            page.setRecords(collect);

        }

    }

    /***
     * 转换
     * @return
     */
    private void convertIPageFamilyInfo(IPage page){
        if (StringUtils.isNotNull(page) && page.getRecords().size()>0){
            //设备型号id
            List<DeviceUserInfoVO> records = page.getRecords();
            List<DeviceUserInfoVO> collect = records.stream().map(s->{
                //管理员的手机号码不为空就说明此imei绑定了
                if (StringUtils.isNotEmpty(s.getAdminPhone())){
                    //根据imei查询绑定表成员
                    List<MemberInfoVO> memberInfoVOS = baseMapper.queryMemberInfoByImei(s.getImei());
                    s.setMemberInfoVOS(memberInfoVOS);
                }

                return s;
            }).collect(Collectors.toList());

            page.setRecords(collect);

        }

    }
//





//
//    private DeviceUserInfo getDeviceUserInfo(DeviceUserInfo info){
//        if (StringUtils.isNotNull(info)){
//            //根据厂商id 查询厂商名称
//            if(StringUtils.isNotNull(firmManageMapper.selectById(info.getFirmId()))){
//                info.setFirmName(firmManageMapper.selectById(info.getFirmId()).getFirmName());
//            }
//
//        }
//
//        return info;
//    }
}
