package com.ecell.internationalize.system.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ecell.internationalize.common.core.context.SecurityContextHolder;
import com.ecell.internationalize.common.system.constant.FieldConstant;
import com.ecell.internationalize.common.system.entity.SysUser;
import com.ecell.internationalize.system.entity.FirmManage;
import com.ecell.internationalize.system.entity.vo.FirmTree;
import com.ecell.internationalize.system.mapper.FirmManageMapper;
import com.ecell.internationalize.system.service.FirmManageService;
import com.ecell.internationalize.system.utils.TreeUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author liy
 * @since 2022-07-06
 */
@Service
public class FirmManageServiceImpl extends ServiceImpl<FirmManageMapper, FirmManage> implements FirmManageService {
    @Autowired
    private FirmManageMapper firmManageMapper;
    /**
     * 代理商分页查询
     * @Author liy
     * @Date 2022/7/6 10:42
     * @param map 分页查询体
     * @return IPage
     */
    @Override
    public IPage<FirmManage> findAllByPage(Map<String,Object> map) {
        Page<FirmManage> page=new Page<>(Integer.parseInt(map.get(FieldConstant.CURRENT).toString()),Integer.parseInt(map.get(FieldConstant.SIZE).toString()));
        LambdaQueryWrapper<FirmManage> queryWrapper = new LambdaQueryWrapper<>();
        if (map.containsKey(FieldConstant.FIRM_NAME)&&null!=map.get(FieldConstant.FIRM_NAME).toString()){
            queryWrapper.eq(FirmManage::getFirmName,map.get(FieldConstant.FIRM_NAME).toString());
        }
        return firmManageMapper.selectPage(page,queryWrapper);
    }
    /**
     * 显示所有厂商名称
     * @Author liy
     * @Date 2022/7/12 16:42
     * @return list
     */
    @Override
    public List<FirmTree> queryAll() {
        String userId = SecurityContextHolder.getStringUserId();
        LambdaQueryWrapper<FirmManage> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.orderByDesc(FirmManage::getCreateTime);
        SysUser sysUser = firmManageMapper.queryFirmId(userId);
        if (FieldConstant.ADMIN.equals(userId) || "3".equals(sysUser.getFirmFlag())){
            //管理员或者超级厂商显示所有厂商
            return TreeUtil.buildFirmTree(firmManageMapper.selectList(lambdaQueryWrapper),FieldConstant.TREE_ROOT_ID);
        }
        //厂商
        if (FieldConstant.MATH_ONE.equals(sysUser.getFirmFlag())){
            //如果该用户选择的是厂商，显示厂商以及厂商下面的所有代理商
            lambdaQueryWrapper.eq(FirmManage::getFirmId,sysUser.getFirmId());
            lambdaQueryWrapper.or().eq(FirmManage::getParentId,sysUser.getFirmId());
            return TreeUtil.buildFirmTree(firmManageMapper.selectList(lambdaQueryWrapper),FieldConstant.TREE_ROOT_ID);
        }
        //代理商
        else {
            List<FirmTree> list=new ArrayList<>();
            //为代理商
            lambdaQueryWrapper.or().eq(FirmManage::getFirmId,sysUser.getSecondFirmId());
            FirmManage firmManage = firmManageMapper.selectOne(lambdaQueryWrapper);
            FirmTree firmTree=new FirmTree();
            firmTree.setFirmName(firmManage.getFirmName());
            firmTree.setCreateTime(new Date());
            firmTree.setId(firmManage.getFirmId());
            list.add(firmTree);
            return list;
        }
    }

    /**
     * 构建树查询 1. 不是懒加载情况，查询全部 2. 是懒加载，根据parentId 查询 2.1 父节点为空，则查询ID -1
     * @Author  liy
     * @Date   2022/7/19 11:07
     * @param lazy 是否是懒加载
     * @param parentId 父节点ID
     * @return List<FirmTree> 树形集合
     */
    @Override
    public List<FirmTree> queryTree(boolean lazy, String parentId) {
        String userId = SecurityContextHolder.getStringUserId();
        LambdaQueryWrapper<FirmManage> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.orderByDesc(FirmManage::getCreateTime);
        SysUser sysUser = firmManageMapper.queryFirmId(userId);
        if (FieldConstant.ADMIN.equals(userId) || "3".equals(sysUser.getFirmFlag())){
            //管理员或者超级厂商显示所有厂商
            return TreeUtil.buildFirmTree(firmManageMapper.selectList(lambdaQueryWrapper),FieldConstant.TREE_ROOT_ID);
        }
        //厂商
        if (FieldConstant.MATH_ONE.equals(sysUser.getFirmFlag())){
            //如果该用户选择的是厂商，显示厂商以及厂商下面的所有代理商
            lambdaQueryWrapper.eq(FirmManage::getFirmId,sysUser.getFirmId());
            lambdaQueryWrapper.or().eq(FirmManage::getParentId,sysUser.getFirmId());
        }
        //代理商
        else {
            //为代理商
            lambdaQueryWrapper.eq(FirmManage::getFirmId,sysUser.getFirmId());
            lambdaQueryWrapper.or().eq(FirmManage::getFirmId,sysUser.getSecondFirmId());
        }

        return TreeUtil.buildFirmTree(firmManageMapper.selectList(lambdaQueryWrapper),FieldConstant.TREE_ROOT_ID);
    }
}
