package com.ecell.internationalize.system.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ecell.internationalize.common.core.context.SecurityContextHolder;
import com.ecell.internationalize.common.system.constant.FieldConstant;
import com.ecell.internationalize.common.system.entity.HeartTemperatureRecord;
import com.ecell.internationalize.common.system.entity.SysUser;
import com.ecell.internationalize.system.entity.DeviceInfo;
import com.ecell.internationalize.system.mapper.DeviceInfoMapper;
import com.ecell.internationalize.system.mapper.FirmManageMapper;
import com.ecell.internationalize.system.mapper.HeartTemperatureRecordMapper;
import com.ecell.internationalize.system.service.HeartTemperatureRecordService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 心率/体温记录表 服务实现类
 * </p>
 *
 * @author ${author}
 * @since 2022-10-28
 */
@Service
public class HeartTemperatureRecordServiceImpl extends ServiceImpl<HeartTemperatureRecordMapper, HeartTemperatureRecord> implements HeartTemperatureRecordService {

    @Autowired
    private HeartTemperatureRecordMapper heartTemperatureRecordMapper;
    @Autowired
    private DeviceInfoMapper deviceInfoMapper;
    @Autowired
    private FirmManageMapper firmManageMapper;
    /**
     * 心率条件分页查询
     * @Author liy
     * @Date 2022/7/11 11:42
     * @param map 分页查询体
     * @return IPage
     */
    @Override
    public IPage<HeartTemperatureRecord> findAllByPage(Map<String, Object> map) {
        Page<HeartTemperatureRecord> page=new Page<>(Integer.parseInt(map.get(FieldConstant.CURRENT).toString()),Integer.parseInt(map.get(FieldConstant.SIZE).toString()));
        String userId= SecurityContextHolder.getStringUserId();
        SysUser sysUser = firmManageMapper.queryFirmId(userId);
        if (FieldConstant.ADMIN.equals(userId) || "3".equals(sysUser.getFirmFlag())){
            //管理员或者超级厂商显示所有数据
            return heartTemperatureRecordMapper.getList(page,map,null);
        }
        LambdaQueryWrapper<DeviceInfo> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(DeviceInfo::getDelFlag,FieldConstant.DEL_FLAG);
        //厂商
        if (FieldConstant.MATH_ONE.equals(sysUser.getFirmFlag())){
            //如果该用户选择的是厂商，显示厂商以及厂商下面的所有代理商设备数据
            lambdaQueryWrapper.eq(DeviceInfo::getFirmId,sysUser.getFirmId());
        }
        else {
            lambdaQueryWrapper.eq(DeviceInfo::getAgentId,sysUser.getSecondFirmId());
        }
        List<DeviceInfo> deviceInfoList = deviceInfoMapper.selectList(lambdaQueryWrapper);
        if (CollectionUtils.isEmpty(deviceInfoList)){
            return null;
        }
        List<String> iMeiList= deviceInfoList.stream().map(DeviceInfo::getImei).collect(Collectors.toList());
        return heartTemperatureRecordMapper.getList(page,map,iMeiList);
    }
}
