package com.ecell.internationalize.system.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ecell.internationalize.common.core.utils.StringUtils;
import com.ecell.internationalize.system.constant.DeviceModelConstants;
import com.ecell.internationalize.system.entity.ImeiLogInfo;
import com.ecell.internationalize.system.entity.dto.ImeiLogPageDTO;
import com.ecell.internationalize.system.mapper.ImeiLogInfoMapper;
import com.ecell.internationalize.system.service.ImeiLogInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

/**
 * <p>
 * 设备录入日志记录 服务实现类
 * </p>
 *
 * @author ${author}
 * @since 2022-07-12
 */
@Service
public class ImeiLogInfoServiceImpl extends ServiceImpl<ImeiLogInfoMapper, ImeiLogInfo> implements ImeiLogInfoService {
    @Autowired
    private ImeiLogInfoMapper mapper;

    @Override
    public IPage queryByPageList(ImeiLogPageDTO dto) {
       Page page =new Page(dto.getCurrent(),dto.getPageSize());
        QueryWrapper wrapper = getQueryWrapper(dto);

        return  mapper.selectPage(page, wrapper);
    }





    /**
     * 分页查询条件封装
     * @param dto
     * @return
     */
    private QueryWrapper getQueryWrapper(ImeiLogPageDTO dto) {
        QueryWrapper wrapper=new QueryWrapper();
        if (StringUtils.isNotEmpty(dto.getIp())){
            wrapper.like("ip", dto.getIp());
        }
        if (StringUtils.isNotEmpty(dto.getUserRole())){
            wrapper.eq("user_role", dto.getUserRole());
        }
        if (StringUtils.isNotEmpty(dto.getLoginAccount())){
            wrapper.eq("login_account", dto.getLoginAccount());
        }
        wrapper.eq("del_flag", DeviceModelConstants.ONE);
        wrapper.orderByDesc("create_time");
        return wrapper;
    }

    /**
     * 保存到入日志记录
     */
    @Async
    public void saveImeiLogInfo( ImeiLogInfo info){
        this.save(info);
    }
}
