package com.ecell.internationalize.system.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ecell.internationalize.common.core.exception.ServiceException;
import com.ecell.internationalize.common.core.utils.StringUtils;
import com.ecell.internationalize.common.core.utils.uuid.UUID;
import com.ecell.internationalize.common.system.entity.ImeiSosInfo;
import com.ecell.internationalize.system.constant.DeviceModelConstants;
import com.ecell.internationalize.system.mapper.ImeiSosInfoMapper;
import com.ecell.internationalize.system.service.ImeiSosInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author ${author}
 * @since 2022-07-13
 */
@Service
public class ImeiSosInfoServiceImpl extends ServiceImpl<ImeiSosInfoMapper, ImeiSosInfo> implements ImeiSosInfoService {
    @Autowired
    private DeviceInfoServiceImpl deviceInfoService;

    @Override
    public String importImeiSOS(List<ImeiSosInfo> imeiList) {
        if (StringUtils.isNull(imeiList) && imeiList.size() == 0) {
            throw new ServiceException("导入SOS数据不能为空");
        }
       // List<ImeiSosInfo> collect = imeiList.stream().map(this::convertType).collect(Collectors.toList());

        return  checkImei(imeiList);
    }


    private String checkImei(List<ImeiSosInfo> imeiList) {
        int successNum = 0;
        int failureNum = 0;
        StringBuilder successMsg = new StringBuilder();
        StringBuilder failureMsg = new StringBuilder();

        for (ImeiSosInfo dto : imeiList) {
            try {
                if (!(dto.getImei().trim().length() >15 || dto.getImei().trim().length() <14)) {
                    failureNum++;
                    failureMsg.append("<br/>"+failureNum+"、IMEI:"+ dto.getImei() + "长度有误" + dto.getImei().length());
                }else {
                    if (!StringUtils.isNotNull(deviceInfoService.queryDeviceByImei(dto.getImei()))){
                        failureNum++;
                        failureMsg.append("<br/>" + failureNum + "、IMEI " + dto.getImei() + " 不存在");
                    }else {
                        //先查询该设备有没有设置SOS号码
                        ImeiSosInfo imeiSosInfo = querySOSByImei(dto.getImei());
                        if (StringUtils.isNotNull(imeiSosInfo)){//有就更新
                            this.updateById(imeiSosInfo);
                        }else {
                            //设置id
                            dto.setImeiSosId(UUID.fastUUID().toString(true));
                            dto.setSource(DeviceModelConstants.ONE);
                            this.save(dto);
                            successNum++;
                        }

                        successMsg.append("<br/>" + successNum + "、IMEI " +dto.getImei() + " 导入成功");

                    }
                }

            } catch (Exception e) {
                failureNum++;
                String msg = "<br/>" + failureNum + "、IMEI " + dto.getImei() + " 导入失败：";
                failureMsg.append(msg + e.getMessage());
                log.error(msg, e);

            }


        }

        if (failureNum > 0)
        {
            failureMsg.insert(0, "很抱歉，导入失败！共 " + failureNum + " 条数据格式不正确，错误如下：");
            //  throw new ServiceException(failureMsg.toString());
            return  failureMsg.toString();
        }
        else
        {
            successMsg.insert(0, "恭喜您，数据已全部导入成功！共 " + successNum + " 条，数据如下：");
            return successMsg.toString();
        }

    }

    /**
     *
     * 装换类型
     * @return
     */
    private ImeiSosInfo convertType(ImeiSosInfo info){
        info.setImeiSosId(UUID.fastUUID().toString(true));
        return info;
    }

    private ImeiSosInfo querySOSByImei(String imei){
        QueryWrapper wrapper =new QueryWrapper();
        wrapper.eq("imei",imei);
        wrapper.last("LIMIT 1");
        return baseMapper.selectOne(wrapper);

    }

}
