package com.ecell.internationalize.system.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ecell.internationalize.common.system.constant.FieldConstant;
import com.ecell.internationalize.system.entity.MessageCategory;
import com.ecell.internationalize.system.entity.MessageContent;
import com.ecell.internationalize.system.mapper.MessageCategoryMapper;
import com.ecell.internationalize.system.mapper.MessageContentMapper;
import com.ecell.internationalize.system.service.MessageCategoryService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * 消息类别 服务实现类
 * </p>
 *
 * @author liy
 * @since 2022-07-12
 */
@Service
public class MessageCategoryServiceImpl extends ServiceImpl<MessageCategoryMapper, MessageCategory> implements MessageCategoryService {
    @Autowired
    private MessageCategoryMapper messageCategoryMapper;
    @Autowired
    private MessageContentMapper messageContentMapper;
    /**
     * 消息类别分页列表
     * @Author liy
     * @Date 2022/7/13 11:42
     * @param map 分页
     * @return IPage
     */
    @Override
    public IPage<MessageCategory> findAllByPage(Map<String,Object> map) {
        Page<MessageCategory> page=new Page<>(Integer.parseInt(map.get(FieldConstant.CURRENT).toString()),Integer.parseInt(map.get(FieldConstant.SIZE).toString()));
        LambdaQueryWrapper<MessageCategory> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(MessageCategory::getDelFlag, FieldConstant.MATH_ONE);
        queryWrapper.orderByAsc(MessageCategory::getCategoryOrder);
        return messageCategoryMapper.selectPage(page,queryWrapper);
    }
    /**
     * 删除类别
     * @Author liy
     * @Date 2022/7/13 15:42
     * @param messageCategory 类别
     * @return IPage
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int delCategory(MessageCategory messageCategory) {
        //逻辑删除类别信息
        messageCategoryMapper.updateById(messageCategory);
        //查询类别相关消息内容
        LambdaQueryWrapper<MessageContent> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(MessageContent::getCategoryId,messageCategory.getId());
        List<MessageContent> messageContent = messageContentMapper.selectList(queryWrapper);
        //逻辑删除类别下的消息内容
        if (!CollectionUtils.isEmpty(messageContent)) {
            messageContentMapper.updateBatch(messageContent);
        }
        return 0;
    }
}
