package com.ecell.internationalize.system.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ecell.internationalize.common.core.utils.StringUtils;
import com.ecell.internationalize.common.security.utils.SecurityUtils;
import com.ecell.internationalize.common.system.entity.PositioningModelInfo;
import com.ecell.internationalize.system.constant.DeviceModelConstants;
import com.ecell.internationalize.system.mapper.PositioningModelInfoMapper;
import com.ecell.internationalize.system.service.PositioningModelInfoService;
import com.ecell.internationalize.system.utils.DeviceModelInfoUtli;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 定位模式信息表 服务实现类
 * </p>
 *
 * @author ${author}
 * @since 2022-07-09
 */
@Service
public class PositioningModelInfoServiceImpl extends ServiceImpl<PositioningModelInfoMapper, PositioningModelInfo> implements PositioningModelInfoService {
    @Autowired
    private PositioningModelInfoMapper mapper;

    /**
     * 分页查询
     * @param pageSize
     * @param pageNum
     * @return
     */
    @Override
    public IPage<PositioningModelInfo> findListByPage(int pageSize, int pageNum) {
        QueryWrapper<PositioningModelInfo> wrapper =new QueryWrapper<>();
        DeviceModelInfoUtli.dataPermissions(wrapper,"create_user", SecurityUtils.getUsername());
        wrapper.orderByDesc("create_time");

        Page<PositioningModelInfo> page =new Page<PositioningModelInfo>(pageNum,pageSize);
        return mapper.selectPage(page, wrapper);
    }

    /**
     * 查询所有
     * @return
     */
    @Override
    public List queryAll() {
        QueryWrapper wrapper =new QueryWrapper();
        DeviceModelInfoUtli.dataPermissions(wrapper,"create_user",SecurityUtils.getUsername());
        List<PositioningModelInfo> positioningModelInfos = mapper.selectList(wrapper);


        return  convertToString(positioningModelInfos);
    }

    /**
     *根据ID查询
     * @param id
     * @return
     */
    @Override
    public PositioningModelInfo queryById(String id) {
        return mapper.selectById(id);
    }


    public List<Map<String,String>> convertToString( List<PositioningModelInfo> infos){
        if (StringUtils.isNotEmpty(infos) && infos.size()>0){
            List<Map<String,String>> collect = infos.stream().map(this::classToMap).collect(Collectors.toList());

            return collect;
        }else {
            return Collections.emptyList();
        }
    }

    public Map<String,String> classToMap(PositioningModelInfo info){
        Map<String,String> map =new HashMap<>(2);
        if (StringUtils.isNotNull(info)){
            StringBuffer buffer =new StringBuffer();
            buffer.append(info.getModelOneName()).append("(").append(info.getPositioningOneInterva()).append(DeviceModelConstants.MINUTE).append(")").
                    append(info.getModelTwoName()).append("(").append(info.getPositioningTwoInterva()).append(DeviceModelConstants.MINUTE).append(")").
                    append(info.getModelThreeName()).append("(").append(info.getPositioningThreeInterva()).append(DeviceModelConstants.MINUTE).append(")");
            map.put("name",buffer.toString());
            map.put("id",info.getPositioningModeId());

        }
        return map;
    }
}
