package com.ecell.internationalize.system.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ecell.internationalize.common.core.context.SecurityContextHolder;
import com.ecell.internationalize.common.system.constant.FieldConstant;
import com.ecell.internationalize.common.system.entity.SysUser;
import com.ecell.internationalize.system.entity.DeviceInfo;
import com.ecell.internationalize.system.entity.ReminderMessageApp;
import com.ecell.internationalize.system.entity.dto.ReminderMessageAppDto;
import com.ecell.internationalize.system.entity.dto.ReminderMessageQueryDto;
import com.ecell.internationalize.system.mapper.DeviceInfoMapper;
import com.ecell.internationalize.system.mapper.FirmManageMapper;
import com.ecell.internationalize.system.mapper.ReminderMessageAppMapper;
import com.ecell.internationalize.system.service.ReminderMessageAppService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 提醒消息表 服务实现类
 * </p>
 *
 * @author ${author}
 * @since 2022-11-01
 */
@Service
public class ReminderMessageAppServiceImpl extends ServiceImpl<ReminderMessageAppMapper, ReminderMessageApp> implements ReminderMessageAppService {
    @Autowired
    private ReminderMessageAppMapper reminderMessageAppMapper;
    @Autowired
    private FirmManageMapper firmManageMapper;
    @Autowired
    private DeviceInfoMapper deviceInfoMapper;
    /**
     * 设备报警条件分页查询
     * @Author liy
     * @Date 2022/7/12 10:42
     * @return IPage
     */
    @Override
    public IPage<ReminderMessageQueryDto> findAllByPage(ReminderMessageAppDto reminderMessageAppDto) {
        Page<ReminderMessageQueryDto> page=new Page<>(reminderMessageAppDto.getCurrent(),reminderMessageAppDto.getPageSize());
        String userId= SecurityContextHolder.getStringUserId();
        SysUser sysUser = firmManageMapper.queryFirmId(userId);
        if (FieldConstant.ADMIN.equals(userId) || "3".equals(sysUser.getFirmFlag())){
            //管理员或者超级厂商显示所有数据
            return reminderMessageAppMapper.getList(page,reminderMessageAppDto,null);
        }
        LambdaQueryWrapper<DeviceInfo> lambdaQueryWrapper=new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(DeviceInfo::getDelFlag,FieldConstant.DEL_FLAG);
        //厂商
        if (FieldConstant.MATH_ONE.equals(sysUser.getFirmFlag())){
            //如果该用户选择的是厂商，显示厂商以及厂商下面的所有代理商设备数据
            lambdaQueryWrapper.eq(DeviceInfo::getFirmId,sysUser.getFirmId());
        }
        else {
            lambdaQueryWrapper.eq(DeviceInfo::getAgentId,sysUser.getSecondFirmId());
        }
        List<DeviceInfo> deviceInfoList = deviceInfoMapper.selectList(lambdaQueryWrapper);
        if (CollectionUtils.isEmpty(deviceInfoList)){
            return null;
        }
        List<String> iMeiList= deviceInfoList.stream().map(DeviceInfo::getImei).collect(Collectors.toList());
        return reminderMessageAppMapper.getList(page,reminderMessageAppDto,iMeiList);
    }
}
