package com.ecell.internationalize.system.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ecell.internationalize.common.core.exception.ServiceException;
import com.ecell.internationalize.common.core.utils.StringUtils;
import com.ecell.internationalize.common.security.utils.SecurityUtils;
import com.ecell.internationalize.system.constant.DeviceModelConstants;
import com.ecell.internationalize.system.entity.DeviceInfo;
import com.ecell.internationalize.system.entity.DeviceModelInfo;
import com.ecell.internationalize.system.entity.VideoModelInfo;
import com.ecell.internationalize.system.mapper.DeviceInfoMapper;
import com.ecell.internationalize.system.mapper.DeviceModelInfoMapper;
import com.ecell.internationalize.system.mapper.VideoModelInfoMapper;
import com.ecell.internationalize.system.service.VideoModelInfoService;
import com.ecell.internationalize.system.utils.DeviceModelInfoUtli;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 视频型号管理信息表 服务实现类
 * </p>
 *
 * @author ${author}
 * @since 2022-07-08
 */
@Service
public class VideoModelInfoServiceImpl extends ServiceImpl<VideoModelInfoMapper, VideoModelInfo> implements VideoModelInfoService {
    @Autowired
    private VideoModelInfoMapper mapper;
    @Autowired
    private DeviceModelInfoMapper deviceModelInfoMapper;
    @Autowired
    private DeviceInfoMapper deviceInfoMapper;

    /**
     * 验证视频型号名称是否唯一
     * @param info
     * @return
     */
    @Override
    public String checkModelNameUnique(VideoModelInfo info) {
        QueryWrapper<VideoModelInfo> wrapper =new QueryWrapper<>();
        wrapper.eq("video_model_name",info.getVideoModelName());
        wrapper.eq("del_flag", DeviceModelConstants.ONE);
        //加上limit 不加如果有多条记录会抛异常
        wrapper.last("LIMIT 1");
        VideoModelInfo modelInfo = mapper.selectOne(wrapper);
        if (StringUtils.isNotNull(modelInfo) && ! modelInfo .getVideoModelId().equals(info.getVideoModelId()))
        {
            return DeviceModelConstants.NOT_UNIQUE;
        }
        return DeviceModelConstants.UNIQUE;
    }
    /**
     * 分页查询视频型
     * @return
     */
    @Override
    public IPage<VideoModelInfo> findListByPage(int pageSize,int pageNum) {
        QueryWrapper<VideoModelInfo> wrapper =new QueryWrapper<>();
        wrapper.eq("del_flag",DeviceModelConstants.ONE);
        wrapper.orderByDesc("create_time");
        DeviceModelInfoUtli.dataPermissions(wrapper,"create_user", SecurityUtils.getUsername());
//        //不是管理员
//        if (!SecurityUtils.isAdminString(SecurityUtils.getUserId())){
//            wrapper.eq("create_user",SecurityUtils.getUsername());
//        }


        Page<VideoModelInfo> page =new Page<VideoModelInfo>(pageNum,pageSize);
        IPage<VideoModelInfo> videos = mapper.selectPage(page, wrapper);
        if (StringUtils.isNotEmpty(videos.getRecords()) && videos.getRecords().size()>0 ){
                videos.getRecords().stream().map(this::getDeviceTotal).collect(Collectors.toList());
        }

        return videos;
    }

    /**
     *
     * @param id
     * @return
     */
    @Override
    public boolean deltaById(String id) {
        VideoModelInfo info =new VideoModelInfo();
        info.setVideoModelId(id);
        VideoModelInfo deviceTotal = this.getDeviceTotal(info);
        if (deviceTotal.getDeviceTotal()>0){
            throw new ServiceException("此视频类型已绑定设备不能删除");
        }
        info.setDelFlag(DeviceModelConstants.ZERO);
        return super.updateById(info);
    }

    /**
     * 设备模型新增专用
     * @return
     */
    @Override
    public List<VideoModelInfo> queryAll(String type) {
        if (!StringUtils.isNotEmpty(type)){
            throw new ServiceException("视频类型不能为空");
        }
        QueryWrapper<VideoModelInfo> wrapper =new QueryWrapper<>();
        wrapper.eq("del_flag",DeviceModelConstants.ONE);
        wrapper.eq("video_model_type",type);
        return mapper.selectList(wrapper);
    }

    /**
     * 根据视频型号查询设备模型的id，再根据设备模型的id查询设备个数
     * @return
     */
    private VideoModelInfo getDeviceTotal(VideoModelInfo info){
        QueryWrapper<DeviceModelInfo> wrapper =new QueryWrapper<>();
        wrapper.eq("del_flag",DeviceModelConstants.ONE);
        wrapper.eq("video_model_id",info.getVideoModelId());
        List<DeviceModelInfo> deviceModelInfos = deviceModelInfoMapper.selectList(wrapper);
        if (StringUtils.isNotEmpty(deviceModelInfos) && deviceModelInfos.size()>0){
            List<String> collect = deviceModelInfos.stream().map(DeviceModelInfo::getDeviceModelId).collect(Collectors.toList());
            QueryWrapper<DeviceInfo> wrapper1 =new QueryWrapper<>();
            wrapper1.eq("del_flag",DeviceModelConstants.ONE);
            wrapper1.in("device_model_id",collect);
           //根据设备模型id查询设备表的总数
           info.setDeviceTotal(deviceInfoMapper.selectCount(wrapper1));

        }else {
            info.setDeviceTotal(DeviceModelConstants.NUMBER_ZERO);
        }

        return info;
    }


}
