package com.ecell.internationalize.system.utils;

import com.ecell.internationalize.system.entity.FirmManage;
import com.ecell.internationalize.system.entity.vo.FirmTree;
import com.ecell.internationalize.system.entity.vo.TreeNode;
import lombok.experimental.UtilityClass;

import java.util.ArrayList;
import java.util.List;

/**
 * @Title: TreeUtil
 * @Author: liy
 * @Date: 2022/7/19 9:16
 * @Description:
 * @Version:1.0
 */
@UtilityClass
public class TreeUtil {
    /**
     * 两层循环实现建树
     * @param treeNodes 传入的树节点列表
     * @return
     */
    public <T extends TreeNode> List<T> build(List<T> treeNodes, Object root) {

        List<T> trees = new ArrayList<>();

        for (T treeNode : treeNodes) {

            if (root.equals(treeNode.getParentId())) {
                trees.add(treeNode);
            }

            for (T it : treeNodes) {
                if (it.getParentId().equals(treeNode.getId())) {
                    if (treeNode.getChildren() == null) {
                        treeNode.setChildren(new ArrayList<>());
                    }
                    //treeNode.setHasChildren(true);
                    treeNode.add(it);
                }
            }
        }
        return trees;
    }

    /**
     * 通过FirmManage创建树形节点
     * @param firmManages
     * @param root
     * @return
     */
    public List<FirmTree> buildFirmTree(List<FirmManage> firmManages, String root) {
        List<FirmTree> trees = new ArrayList<>();
        FirmTree node;
        for (FirmManage firmManage : firmManages) {
            node = new FirmTree();
            node.setId(firmManage.getFirmId());
            node.setParentId(firmManage.getParentId());
            node.setFirmName(firmManage.getFirmName());
            node.setCreateTime(firmManage.getCreateTime());
            node.setHasChildren(false);
            trees.add(node);
        }
        return TreeUtil.build(trees, root);
    }
}
